///////////////////////////////////////////////////////////////////////////////
//
// File   : ToolsAVP_Message_Editing.cpp
// Author : Chris Wallace
// Date   : 26-Jan-2007
//
// Copyright (C) 2007 Rebellion, All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Include files...
////////////////////////////////////////////////////////////////////////////////

// core
#include "Core/Asura.h"

// header
#include "Entity/ToolsAVP_Message_Editing.h"

// asura
#include "Blueprint/Asura_Blueprint_System.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Gadget.h"
#include "Interface/Asura_Overlay.h"
#include "Rendering/Asura_Project_Skin_Elements.h"

// axon
#include "Core/AxonAVP_Messages.h"

// tools
#include "Entity/Tools_Entity_Data.h"
#include "Entity/Tools_MessageEditor.h"
#include "Misc/Tools_Prompted_Selection.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Movement/Tools_Movement.h"
#include "Objects/Axon/Tools_Object_Axon_ChooseObjects.h"
#include "Objects/Tools_PositionEdit_Object.h"


// avp
#include "Camera/AVP_CameraShake_System.h"
#include "Blueprint/AVP_Blueprint_HashIDs.h"
#include "Entity/AVP_Entity_EventIDs.h"
#include "Entity/AVP_Messages.h"
#include "Entity/AVP_Server_Messages.h"
#include "Entity/Controller/AVP_Controller_Messages.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_Alien.h"
#include "Entity/Controller/AVP_ServerEntity_Squad_Marine.h"
#include "Entity/Controller/AVP_ServerEntity_Team_Alien.h"
#include "Entity/Controller/AVP_ServerEntity_Team_Marine.h"
#include "Entity/ObjectiveSystem/PlayerObjective/AVP_PlayerObjective_Messages.h"
#include "Entity/PhysicalObject/AVP_ServerEntity_PhysicalObject.h"
#include "Entity/PhysicalObject/Actor/AVP_Actor_Messages.h"
#include "Entity/PhysicalObject/Actor/Vehicles/Dropship/AVP_Messages_Dropship.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/AlienQueen/AVP_AlienQueen_Messages.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/FaceHugger/AVP_Messages_FaceHugger.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_Messages_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/AVP_ServerActor_Human.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Predator/AVP_Messages_Predator.h"
#include "Entity/PhysicalObject/Actor/Weapons/Sentrygun/AVP_Sentrygun_Messages.h"
#include "Entity/PhysicalObject/Pickup/ToolsAVP_Entity_Pickup.h"
#include "Entity/PlayerInfo/AVP_PlayerInfo_Messages.h"
#include "Entity/PlayerInfo/AVP_ServerEntity_PlayerInfo.h"
#include "Entity/Spawners/AVP_SpawnerInterface.h"

// avp tools
#include "Entity/Controller/ToolsAVP_Entity_MarineSquad.h"
#include "Entity/GamePlayVolume/ToolsAVP_Entity_GamePlayVolume.h"
#include "Entity/Locations/LookAt/ToolsAVP_Entity_LookAt.h"
#include "Entity/Locations/Objective/Marine/ToolsAVP_Node_Objective_Marine.h"
#include "Entity/Locations/ShootAt/ToolsAVP_Entity_ShootAt.h"
#include "Entity/ObjectiveSystem/PlayerObjective/ToolsAVP_Entity_LocationMarker.h"
#include "Entity/ObjectiveSystem/PlayerObjective/ToolsAVP_Entity_PlayerObjective.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Alien/ToolsAVP_Entity_AlienQueen.h"
#include "Entity/PhysicalObject/Actor/Vertebrates/Human/ToolsAVP_Entity_Human.h"
#include "Entity/Spawners/Human/ToolsAVP_Entity_HumanSpawnPoint.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Objects/ToolsAVP_Object_Type.h"
#include "ToolsAVP_ChooseObjects.h"

////////////////////////////////////////////////////////////////////////////////
// Static Variables...
////////////////////////////////////////////////////////////////////////////////

AVP_Message_ChunkActor*							ToolsAVP_Message_Editing::s_pxChunkActorMessage						= NULL;
AVP_Message_AddObjective*						ToolsAVP_Message_Editing::s_pxAddObjectiveMessage					= NULL;
AVP_Message_AddPlayerObjective*					ToolsAVP_Message_Editing::s_pxAddPlayerObjectiveMessage				= NULL;
AVP_Message_PlayerObjectiveComplete*			ToolsAVP_Message_Editing::s_pxPlayerObjectiveCompleteMessage		= NULL;
AVP_Message_PlayerObjectiveSetMarker*           ToolsAVP_Message_Editing::s_pxSetMarkerLocationMessage              = NULL;
AVP_Message_PlayerObjectiveSetEntity*           ToolsAVP_Message_Editing::s_pxSetTargetEntityMessage                = NULL;
AVP_Message_AddPlayerPickup*                    ToolsAVP_Message_Editing::s_pxSetPlayerPickupMessage                = NULL;
AVP_Message_AddSquadMember*						ToolsAVP_Message_Editing::s_pxAddSquadMemberMessage					= NULL;
AVP_Message_RemoveSquadMember*					ToolsAVP_Message_Editing::s_pxRemoveSquadMemberMessage				= NULL;
AVP_Message_MergeSquads*						ToolsAVP_Message_Editing::s_pxMergeSquadMessage						= NULL;
AVP_Message_SplitSquad*							ToolsAVP_Message_Editing::s_pxSplitSquadMessage						= NULL;
AVP_Message_RequestPatrol*						ToolsAVP_Message_Editing::s_pxRequestPatrolMessage					= NULL;
AVP_Msg_SetAITargetPriorities*					ToolsAVP_Message_Editing::s_pxSetAITargetPrioritiesData				= NULL;
AVP_Message_ChangeMarineSquadSettings*			ToolsAVP_Message_Editing::s_pxChangeHumanoidSquadSettingsMessage	= NULL;
AVP_Message_AttackScriptedTarget*				ToolsAVP_Message_Editing::s_pxAttackScriptedTargetMessage			= NULL;
AVP_Message_DropshipAttackMode*					ToolsAVP_Message_Editing::s_pxDropshipAttackModeMessage				= NULL;
AVP_Message_SetPredatorBehaviour*				ToolsAVP_Message_Editing::s_pxSetPredatorBehaviourMessage			= NULL;
AVP_Message_ChangeHumanoidMovementMode*			ToolsAVP_Message_Editing::s_pxChangeHumanoidMoveModeMessage			= NULL;
AVP_Message_LookAtEntity*						ToolsAVP_Message_Editing::s_pxLookAtEntityMessage					= NULL;
AVP_Message_SetGS2MoveSpeed*					ToolsAVP_Message_Editing::s_pxSetGS2MoveSpeed						= NULL;
AVP_Message_SetHealth*							ToolsAVP_Message_Editing::s_pxSetHealthMessage						= NULL;
AVP_Msg_SetFaceHuggerSquadBehaviour*			ToolsAVP_Message_Editing::s_pxSetFaceHuggerSquadBehaviourMsg		= NULL;
AVP_Message_ChangeHumanoidKnowledgeSettings*	ToolsAVP_Message_Editing::s_pxSetHumanoidKnowledgeSettingsMsg		= NULL;
AVP_Message_SentryGun_Set_SweepSpeed*			ToolsAVP_Message_Editing::s_pxSetSentryGunSweepSpeed				= NULL;
AVP_Message_SentryGun_Set_VisionRange*			ToolsAVP_Message_Editing::s_pxSetSentryGunVisionRange				= NULL;
AVP_Message_SentryGun_Set_HorizontalFOV*		ToolsAVP_Message_Editing::s_pxSetSentryGunHorizontalFOV				= NULL;
AVP_Message_SentryGun_Set_VerticalFOV*			ToolsAVP_Message_Editing::s_pxSetSentryGunVerticalFOV				= NULL;
AVP_Message_SentryGun_Set_FiringArc*			ToolsAVP_Message_Editing::s_pxSetSentryGunFiringArc					= NULL;
AVP_Message_SentryGun_Set_LethalRange*			ToolsAVP_Message_Editing::s_pxSetSentryGunLethalRange				= NULL;
AVP_Message_SentryGun_Toggle_Dry_Fire*			ToolsAVP_Message_Editing::s_pxSetSentryGunDryFireToggle				= NULL;
AVP_Message_Actor_Attachment*					ToolsAVP_Message_Editing::s_pxActorAttachmentMessage				= NULL;
AVP_Msg_Add_Entity_To_Queen_Target_List*		ToolsAVP_Message_Editing::s_pxAddEntityToAlienQueenTargetList		= NULL;
AVP_Msg_Remove_Entity_From_Queen_Target_List*	ToolsAVP_Message_Editing::s_pxRemoveEntityFromAlienQueenTargetList	= NULL;
AVP_Msg_Alien_Queen_Start_Roar_Behaviour*		ToolsAVP_Message_Editing::s_pxForceAlienQueenRoar					= NULL;
AVP_Message_SetAlienQueenBehaviour*				ToolsAVP_Message_Editing::s_pxSetAlienQueenBehaviourMessage			= NULL;
AVP_Msg_CameraShake*							ToolsAVP_Message_Editing::s_pxCameraShakeMessage					= NULL;
AVP_Msg_Dismember*								ToolsAVP_Message_Editing::s_pxDismemberMessage						= NULL;

Asura_Dialog*									ToolsAVP_Message_Editing::s_pxSquadTextDialog						= NULL;
Asura_Dialog*									ToolsAVP_Message_Editing::s_pxHumanoidMemberTextDialog				= NULL;
u_int*											ToolsAVP_Message_Editing::s_puTokenFlags							= NULL;
bool											ToolsAVP_Message_Editing::s_bPatrolTypeChanged						= false;
bool											ToolsAVP_Message_Editing::s_bPatrolFullSquad						= false;
bool											ToolsAVP_Message_Editing::s_bPlayersSquadTypeChanged				= false;
bool											ToolsAVP_Message_Editing::s_bPlayersAILedSquadTypeChanged			= false;
bool											ToolsAVP_Message_Editing::s_bSquadCombatZoneChanged					= false;
bool											ToolsAVP_Message_Editing::s_bIsPlayersSquad							= false;
bool											ToolsAVP_Message_Editing::s_bIsAILedPlayersSquad					= false;
bool											ToolsAVP_Message_Editing::s_bIsRestrictedToCombatZone				= false;
bool											ToolsAVP_Message_Editing::s_bIgnoreAlertState						= false;
bool											ToolsAVP_Message_Editing::s_bIgnoreAlertStateChanged				= false;
bool											ToolsAVP_Message_Editing::s_bShootAtChanged							= false;
bool											ToolsAVP_Message_Editing::s_bSquadCombatStateChanged				= false;
bool											ToolsAVP_Message_Editing::s_bDropshipModeChanged					= false;
bool											ToolsAVP_Message_Editing::s_bObjectiveMoveModeChanged				= false;
bool											ToolsAVP_Message_Editing::s_bLookAtChanged							= false;
bool											ToolsAVP_Message_Editing::s_bLookAtPlayer							= false;
bool											ToolsAVP_Message_Editing::s_bGS2MoveModeChanged						= false;
bool											ToolsAVP_Message_Editing::s_bAlienSquadBehaviourChanged				= false;
bool											ToolsAVP_Message_Editing::s_bHealthSettingTypeChanged				= false;
bool											ToolsAVP_Message_Editing::s_bTakeAsDamageFromEntity					= false;
bool											ToolsAVP_Message_Editing::s_bTreatAsDamage							= false;
bool											ToolsAVP_Message_Editing::s_bLookAtMarkerStringChanged				= false;
Asura_String									ToolsAVP_Message_Editing::s_xLookAtMarkerString("Unset");
Asura_String									ToolsAVP_Message_Editing::s_xObjectNameString("*Unset*");
u_int											ToolsAVP_Message_Editing::s_uSquadMemberIndex						= 0;
int												ToolsAVP_Message_Editing::s_iDamageTypeValue						= 0;
int												ToolsAVP_Message_Editing::s_iHumanoidTypeValue						= 0;
Asura_Guid										ToolsAVP_Message_Editing::s_uSplitSquadSentToGuid					= ASURA_GUID_UNREGISTERED;
Asura_Guid										ToolsAVP_Message_Editing::s_uGuidToIgnoreForSelectionInMessage		= ASURA_GUID_UNREGISTERED;
bool											ToolsAVP_Message_Editing::s_bAddObjectiveChanged					= false;
bool											ToolsAVP_Message_Editing::s_bAddPlayerObjectiveChanged				= false;
bool											ToolsAVP_Message_Editing::s_bPlayerObjectiveCompleteChanged		    = false;
bool											ToolsAVP_Message_Editing::s_bSetMarkerLocationChanged               = false;
bool											ToolsAVP_Message_Editing::s_bSetTargetEntityChanged                 = false;
bool											ToolsAVP_Message_Editing::s_bSetPlayerPickupChanged                 = false;
bool											ToolsAVP_Message_Editing::s_bIntelligentAmountChanged               = false;
bool											ToolsAVP_Message_Editing::s_bHumanoidKnowledgeSettingTypeChanged	= false;
bool											ToolsAVP_Message_Editing::s_bQueenTargetListChanged					= false;
Asura_Guid*										ToolsAVP_Message_Editing::s_puEntityGuid							= NULL;
bool											ToolsAVP_Message_Editing::s_bHuggedEntityChanged					= false;

AVP_Msg_SetAlienSquadBehaviour* ToolsAVP_Message_Editing::s_pxSetAlienSquadBehaviourMsg = NULL;

static EventIDAndDescription s_axProjectEventList[] =
{
	// The text for the 'on health percent' events will be replaced with the correct percentage value in GetEventDescriptionMain.
	AVP_ENTITY_EVENT_ONHEALTHPERCENT0,						"On X% Health 0",									"",
	AVP_ENTITY_EVENT_ONHEALTHPERCENT1,						"On X% Health 1",									"",
	AVP_ENTITY_EVENT_ONHEALTHPERCENT2,						"On X% Health 2",									"",
	AVP_ENTITY_EVENT_ONHEALTHPERCENT3,						"On X% Health 3",									"",
	AVP_ENTITY_EVENT_ONHEALTHPERCENT4,						"On X% Health 4",									"",
	AVP_ENTITY_EVENT_ONHEALTHPERCENT5,						"On X% Health 5",									"",
	AVP_ENTITY_EVENT_ONHEALTHPERCENT6,						"On X% Health 6",									"",
	AVP_ENTITY_EVENT_ONHEALTHPERCENT7,						"On X% Health 7",									"",
	AVP_ENTITY_EVENT_ONHEALTHPERCENT8,						"On X% Health 8",									"",
	AVP_ENTITY_EVENT_ONHEALTHPERCENT9,						"On X% Health 9",									"",
	AVP_ENTITY_EVENT_SQUAD_ELIMINATED,						"Squad Eliminated",									"",
	AVP_ENTITY_EVENT_SQUAD_MEMBER_DIED,						"Squad Member Died",								"",
	AVP_ENTITY_EVENT_SQUAD_ALERTED,							"Squad Alerted",									"",
	AVP_ENTITY_EVENT_SQUAD_ENTERING_COMBAT,					"Squad Entering Combat",							"",
	AVP_ENTITY_EVENT_ONALIENKILLED,							"On Alien Killed",									"",
	AVP_ENTITY_EVENT_ONALIENSGONE,							"No More Aliens",									"",
	AVP_ENTITY_EVENT_CIVILIAN_KILLED,						"Civilian Killed",									"",
    AVP_ENTITY_EVENT_SPAWN_ON_SPAWN,						"On Spawn",											"",
	AVP_ENTITY_EVENT_SPAWN_ON_SPAWN_COUNT_EXHAUSTED,		"On Spawn Count Exhausted",							"",
	AVP_ENTITY_EVENT_SPAWN_ON_ANY_SPAWNEE_DEATH,			"On Spawnee Death",									"",
	AVP_ENTITY_EVENT_SPAWN_ON_ALL_SPAWNEES_DEAD,			"On All Possible Spawnees Dead",					"",
	AVP_ENTITY_EVENT_DESPAWN_ON_SPAWN,						"On Despawn",										"",
	AVP_ENTITY_EVENT_MARINE_AI_OBJECTIVE_COMPLETE,			"On Goal Reached",									"",
	AVP_ENTITY_EVENT_MARINE_AI_OBJECTIVE_ACTIONS_COMPLETE,	"On Goal Actions Complete",							"",
	AVP_ENTITY_EVENT_CAPTURE_POINT_ONCAPTUREDBYTEAM1,		"On Captured By Team1",								"",
	AVP_ENTITY_EVENT_CAPTURE_POINT_ONCAPTUREDBYTEAM2,		"On Captured By Team2",								"",
	AVP_ENTITY_EVENT_CAPTURE_POINT_ONCAPTUREDBYTEAM3,		"On Captured By Team3",								"",
	AVP_ENTITY_EVENT_CAPTURE_POINT_ONCONTESTED,				"On Contested",										"",
	AVP_ENTITY_EVENT_CAPTURE_POINT_ONNEUTRALIZED,			"On Neutralized",									"",
	AVP_ENTITY_EVENT_CIVILIAN_ALERTED,						"On Civilian Alerted",								"",
	AVP_ENTITY_EVENT_PLAYER_LOW_AMMO_CURRENT_WEAPON,		"On Player Ammo Low on Current Weapon", 			"",
	AVP_ENTITY_EVENT_PLAYER_LOW_AMMO_BOTH_WEAPONS,			"On Player Ammo Low on Both Weapons",				"",
	AVP_ENTITY_EVENT_PLAYER_NO_AMMO_PRIMARY_WEAPON,			"On Player No Ammo on Primary Weapon",				"",
	AVP_ENTITY_EVENT_PLAYER_CLOAKED,            			"On Player cloaked",	                			"",
	AVP_ENTITY_EVENT_PLAYER_DECLOAKED,			            "On Player decloaked",	                			"",
	AVP_ENTITY_EVENT_MARINE_INTERACT,						"On Interact Marine (On or Off)",					"",
	AVP_ENTITY_EVENT_ALIEN_INTERACT,						"On Interact Alien (Off)",							"",
	AVP_ENTITY_EVENT_PREDATOR_INTERACT,						"On Interact Predator (Off)",						"",
	AVP_ENTITY_EVENT_ON_PICKED_UP_BY_PLAYER,				"On Picked Up By Player",							"",
	AVP_ENTITY_EVENT_ON_PUT_DOWN_BY_PLAYER,					"On Put Down By Player",							"",
	AVP_ENTITY_EVENT_MARINE_ALERTED,						"On Alert Info ( Recon, Sound, MotionTracker, Alien Distraction)",	"",
	AVP_ENTITY_EVENT_MARINE_THREAT_SPOTTED,					"On Threat Spotted",								"",
	AVP_ENTITY_EVENT_GS2_ACTION_ANIM_FLAG_HIT,				"GS2 Action 1 Anim Flag Hit",						"",
	AVP_ENTITY_EVENT_PLAYER_ON_DEATH,                       "On Player death",                                  "",
	0,														NULL,												NULL,	// Terminator
};

ToolsAVP_Message_Editing::ToolsAVP_Message_Editing(void)
{
	static MessageIDAndDescription axMessageIDAndDescription[] = 
	{
		AVP_MSGID_SET_PREFERRED_TARGET_TYPE,			"Set Preferred Target Type",
		AVP_MSGID_END_OF_LEVEL,							"End Level",
		AVP_MSGID_CHUNK,								"Remove limb - DEPRECATED",
		AVP_MSGID_CHANGE_ALIEN_SPAWN_SETTINGS,			"Change Alien spawn settings",
		AVP_MSGID_ADD_ALIENS_TO_SPAWN_QUEUE,			"DEPRECATED, DON'T USE",
		AVP_MSGID_ADD_AI_OBJECTIVE,						"Add Marine Goal",
		AVP_MSGID_ADD_SQUAD_MEMBER,						"Add Marine To Squad",
		AVP_MSGID_REMOVE_SQUAD_MEMBER,					"Remove Marine From Squad",
		AVP_MSGID_ALIEN_TEAM_SET_BEHAVIOUR,				"Set current alien team behaviour",
		AVP_MSGID_ALIEN_TEAM_ADD_SPAWNTOKEN,			"Add alien spawn token(s)",
		AVP_MSGID_ALIEN_TEAM_REMOVE_SPAWNTOKEN,			"Remove alien spawn token(s)",
		AVP_MSGID_ALIEN_TEAM_PREPARE_ROOM,				"Prepare room for ambush",
		AVP_MSGID_REQUEST_CLOAK_ON,						"Request cloak",
		AVP_MSGID_REQUEST_CLOAK_OFF,					"Request uncloak",
		AVP_MSGID_REQUEST_TORCH_ON,	    				"Request torch on",
		AVP_MSGID_REQUEST_TORCH_OFF,					"Request torch off",
		AVP_MSGID_ALIEN_TEAM_THREAT_BONUS,				"Change Alien threat bonus settings",
		AVP_MSGID_SPAWN_HUMANOIDS,						"Spawn Humanoid(s)",
		AVP_MSGID_CHANGE_HUMANOID_SPAWN_SETTINGS,		"Change Humanoid Spawn Settings",
		AVP_MSGID_MERGE_HUMANOID_SQUADS,				"MERGE SQUAD",
		AVP_MSGID_SPLIT_HUMANOID_SQUAD,					"SPLIT SQUAD",
		AVP_MSGID_LIGHT_CHANGE_EVENT,					"Deprecated Light change message, use at your peril!",
		AVP_MSGID_REQUEST_HUMANOID_PATROL,				"REQUEST SQUAD PATROL",
		AVP_MSGID_ACTIVATE_SELECTED_ENTITIES,			"Activate Selected Entities",
		AVP_MSGID_DEACTIVATE_SELECTED_ENTITIES,			"Deactivate Selected Entities",
		AVP_MSGID_SPAWN,								"Spawn",
		AVP_MSGID_SPAWN_SET_MAX_SPAWN_COUNT,			"Set Max Spawn Count",
		AVP_MSGID_SPAWN_RESET_SPAWN_COUNTER,			"Reset Spawn Counter",
		AVP_MSGID_SPAWN_SET_MAX_IN_PLAY,				"Set Max Spawnees In Play",
		AVP_MSGID_SET_ALIEN_SQUAD_BEHAVIOUR,			"Set Alien Squad Behaviour",
		AVP_MSGID_AI_OBJECTIVE_COMPLETE,				"AI Goal Complete",
		AVP_MSGID_SET_AI_TARGET_PRIORITIES,				"Set AI Target Priorities",
		AVP_MSGID_RESET_AI_TARGET_PRIORITIES,			"Reset AI Target Priorities",
		AVP_MSGID_INTERACTION_OBJECT_ENABLE,			"Enable Interaction Object",
		AVP_MSGID_INTERACTION_OBJECT_DISABLE,			"Disable Interaction Object",
		AVP_MSGID_INTERACTION_OBJECT_INITIALISE_TO_ON, 	"Initialise Interaction Object To ON - Doesn't trigger the 'TurnOn' message block",
		AVP_MSGID_INTERACTION_OBJECT_INITIALISE_TO_OFF,	"Initialise Interaction Object To OFF - Doesn't trigger the 'TurnOff' message block",
		AVP_MSGID_SENTRYGUN_ENABLE,						"Enable Sentry Gun",
		AVP_MSGID_SENTRYGUN_DISABLE,					"Disable Sentry Gun",
		AVP_MSGID_SENTRYGUN_DESTROY,					"Destroy Sentry Gun",
		AVP_MSGID_SET_SQUAD_ALERT_STATUS_TO_PASSIVE,	"Set Squad Alert Status To Passive",
		AVP_MSGID_CHANGE_OWNERSHIP_TO_TEAM1,			"Team1 Controls This Entity",
		AVP_MSGID_CHANGE_OWNERSHIP_TO_TEAM2,			"Team2 Controls This Entity",
		AVP_MSGID_CHANGE_OWNERSHIP_TO_TEAM3,			"Team3 Controls This Entity",
		AVP_MSGID_CAPTUREPOINT_LOCK,					"Lock Capture Point",
		AVP_MSGID_CAPTUREPOINT_UNLOCK,					"Unlock Capture Point",
		AVP_MSGID_CHANGE_MARINE_SQUAD_SETTINGS,			"Change Humanoid Squad Settings",
		AVP_MSGID_PROMOTE_TO_SQUAD_LEADER,				"Promote To Squad Leader",
		AVP_MSGID_ATTACK_SCRIPTED_TARGET,				"Shoot At Target",
		AVP_MSGID_STOP_ATTACKING_SCRIPTED_TARGET,		"Stop Shooting At Target",
		AVP_MSGID_PICKUP_SET_INTELLIGENT_AMOUNT,        "Set Pickup intelligent amount",
		AVP_MSGID_ADD_PLAYER_OBJECTIVE,                 "Add player's objective",
		AVP_MSGID_SET_PLAYER_OBJECTIVE_COMPLETE,        "Set the player's objective Guid as completed",
		AVP_MSGID_SET_DROPSHIP_ATTACK_MODE,				"Set the dropship attack mode",
		AVP_MSGID_TRIGGER_INTERACTION_ON_LOCAL_PLAYER,	"Trigger Interaction With Local Player",
		AVP_MSGID_DISABLE_PICKUP,                       "Pickup cannot be picked.",
		AVP_MSGID_ENABLE_PICKUP,                        "Pickup can be picked.",
		AVP_MSGID_SET_PREDATOR_BEHAVIOUR,				"Set Predator Behaviour", 
		AVP_MSGID_CHANGE_HUMANOID_SQUAD_MOVEMENT_MODE,	"Change Humanoid / Humanoid Squad Movement Mode",
		AVP_MSGID_FUNCTIONALITY_MOTION_TRACKER_ON,      "Turn on the motion tracker functionality",
		AVP_MSGID_FUNCTIONALITY_MOTION_TRACKER_OFF,     "Turn off the motion tracker functionality",
		AVP_MSGID_FUNCTIONALITY_VISION_MODES_ON,        "Turn on the vision modes functionality",
		AVP_MSGID_FUNCTIONALITY_VISION_MODES_OFF,       "Turn off the vision modes functionality",
		AVP_MSGID_FUNCTIONALITY_TORCH_ON,				"Turn on the torch functionality",
		AVP_MSGID_FUNCTIONALITY_TORCH_OFF,				"Turn off the torch functionality",
		AVP_MSGID_REPLENISH_PREDATOR_ENERGY,            "Replenish predator energy",
		AVP_MSGID_LOOK_AT_ENTITY,						"Look At Entity",
		AVP_MSGID_STOP_LOOKING_AT_ENTITY,				"Stop Looking At Entity",
		AVP_MSGID_SET_GS2_MOVE_SPEED,					"Set The GS2 Movement Move Speed",
		AVP_MSGID_SET_HEALTH,							"Set Entity Health",
		AVP_MSGID_SET_FACEHUGGER_SQUAD_BEHAVIOUR,		"Set Squad Behaviour",
		AVP_MSGID_PLAYER_OBJECTIVE_SET_MARKER_LOCATION, "Set objective current marker location",
		AVP_MSGID_PLAYER_OBJECTIVE_SET_TARGET_ENTITY,   "Set objective target entity",
		AVP_MSGID_PLAYER_OBJECTIVE_SET_CURRENT,         "Set objective as current one",
		AVP_MSGID_PLAYER_OBJECTIVE_SET_COMPLETE,        "Set objective as complete",
		AVP_MSGID_PLAYER_OBJECTIVE_SET_USE_OWN_LOCATION,"Set objective to use it's location",
		AVP_MSGID_PLAYER_OBJECTIVE_SET_NO_MARKER,       "set objective to display no marker",
		AVP_MSGID_PREDATOR_LOCK_POSITION_F,             "set predator lock to position 'F'",
		AVP_MSGID_PREDATOR_LOCK_POSITION_T,             "set predator lock to position 'T'",
		AVP_MSGID_PREDATOR_LOCK_POSITION_X,             "set predator lock to position 'X'",
		AVP_MSGID_PREDATOR_LOCK_POSITION_O,             "set predator lock to position 'O'",
		AVP_MSGID_SET_PICKUPABLE,						"Set actor as pick-up-able by player (Tequila)",
		AVP_MSGID_SET_NOT_PICKUPABLE,					"Set actor as not pick-up-able by player (Tequila)",
		AVP_MSGID_SET_DROPPABLE,						"Set actor as drop-able by player (Tequila)",
		AVP_MSGID_SET_NOT_DROPPABLE,					"Set actor as not drop-able by player (Tequila)",
		AVP_MSGID_DEACTIVATE_OCCUPIER,					"Deactivate Occupier",
		AVP_MSGID_DESTROY_OCCUPIER,						"Destroy Occupier",
		AVP_MSGID_ADD_PLAYER_PICKUP,                    "Add pickup to player",
		AVP_MSGID_CHANGE_HUMANOID_KNOWLEDGE_SETTINGS,	"Change Humanoid Knowledge Settings",
		AVP_MSGID_SET_SQUAD_ALERT_STATUS_TO_ACTIVE,		"Set Squad Alert Status To Active",
		AVP_MSGID_SET_SQUAD_ALERT_STATUS_TO_AGGRESSIVE,	"Set Squad Alert Status To Aggressive",
		AVP_MSGID_SET_SENTRYGUN_SWEEP_SPEED,			"Set Sweep Speed of SentryGun (0.05-1.0)",
		AVP_MSGID_SET_SENTRYGUN_VISION_RANGE,			"Set Total Vision Range of SentryGun (0-1000)",
		AVP_MSGID_SET_SENTRYGUN_HOR_FOV,				"Set Horizontal FOV of SentryGun (1-178)",
		AVP_MSGID_SET_SENTRYGUN_VER_FOV,				"Set Vertical FOV of SentryGun (1-90)",
		AVP_MSGID_SET_SENTRYGUN_FIRING_ARC,				"Set Fireing Arc of SentryGun (1-178)",
		AVP_MSGID_SET_SENTRYGUN_LETHAL_RANGE,			"Set Lethal Range of SentryGun (one shot kills) (0-1000)",
		AVP_MSGID_SENTRYGUN_DRYFIRE_TOGGLE,				"Toggle Dry Fireing, behaviour stays the same",
		AVP_MSGID_ATTACH_WEAPON,						"Attach the current weapon to the hand. (and remove from marker point)",
		AVP_MSGID_DETACH_WEAPON,						"Remove the current weapon from the hand (and attach to marker point)",
		AVP_MSGID_CREATE_ATTACHMENT,					"Create an attachment at specified marker.",
		AVP_MSGID_DESTROY_ATTACHMENT,					"Destroy an attachment.",
		AVP_MSGID_DETACH_ATTACHMENT,					"Turn attachment into a physics object.",
		AVP_MSGID_MOUNT_LIFT,							"Mount Lift",
		AVP_MSGID_DISMOUNT_LIFT,						"Dismount Lift",
		AVP_MSGID_ALIENQUEEN_ADD_TO_TARGET_LIST,		"Add Entity To Queen Target List",
		AVP_MSGID_ALIENQUEEN_REMOVE_FROM_TARGET_LIST,	"Remove Entity From Queen Target List",
		AVP_MSGID_ALIENQUEEN_FORCE_ROAR,				"Force Queen To Roar, Target Optional",
		AVP_MSGID_ALIENQUEEN_SET_BEHAVIOUR,				"Set Queen Behaviour",
		AVP_MSGID_START_CAMERA_SHAKE,					"Start a Camera Shake",
		AVP_MSGID_STOP_CAMERA_SHAKE,					"Stop a Camera Shake",
		AVP_MSGID_FACEHUGGER_HUG_ENTITY,				"Force a Facehugger to hug a specific victim.",
		AVP_MSGID_DISMEMBER,							"Dismember Limb",

		AVP_MSGID_FORCE_AIM,							"Force Aim",
		AVP_MSGID_STOP_FORCE_AIM,						"Stop Force Aim",
		
		0,												NULL, // Terminator 2
	};

	m_pxProjectMessageList = axMessageIDAndDescription;

	// Set project-specific message target functions
	SetProjectEditDestinationGuidFunction(AVP_EditDestionationGuid);
	SetProjectGetNameFromGuidFunction(AVP_GetNameFromGuid);
}

void ToolsAVP_Message_Editing::AVP_SetModifiedAndUpdateMessageOverlay()
{
#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::CreateMessageAttributesOverlay(Tools_MessageEditor::GetSelectedStaticMessage());
	Tools_SaveAsura::SetEntityModified(true);
#else
	SetModifiedAndUpdateMessageOverlay( reinterpret_cast< int >( this ) );
#endif
}

void ToolsAVP_Message_Editing::UpdateMain()
{
	PARENT::UpdateMain();

	if ( s_bHumanoidKnowledgeSettingTypeChanged )
	{
		s_bHumanoidKnowledgeSettingTypeChanged = false;
		s_pxSetHumanoidKnowledgeSettingsMsg->m_uHumanoidType = s_iHumanoidTypeValue;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}

	if (s_bPatrolTypeChanged)
	{
		s_pxRequestPatrolMessage->m_bFullSquad = (u_int)s_bPatrolFullSquad;
		s_bPatrolTypeChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}

	if (s_bGS2MoveModeChanged)
	{
		s_bGS2MoveModeChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}

	if ( s_bAlienSquadBehaviourChanged )
	{
		s_bAlienSquadBehaviourChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}


	if ( s_bHealthSettingTypeChanged )
	{
		s_pxSetHealthMessage->m_bDamageComesFromOtherEntity = s_bTakeAsDamageFromEntity;
		s_pxSetHealthMessage->m_bTreatAsDamage = s_bTreatAsDamage;

		if ( s_iDamageTypeValue > 0 )
		{
			Asura_Blueprint* pxBP = Asura_Blueprint_System::GetChildBlueprint( 0xb02e8b6f /*"Damage"*/, 0x9643cb29 /*"DamageType"*/, true, s_iDamageTypeValue - 1 );

			if ( pxBP )
			{
				s_pxSetHealthMessage->m_uDamageType = pxBP->GetHashID();
			}
		}
		else
		{
			s_iDamageTypeValue = 0;
			s_pxSetHealthMessage->m_uDamageType = ASURA_HASH_ID_UNSET;
		}

		s_bHealthSettingTypeChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}

	UpdateChangeHumanoidSquadSettingsMessage();

	UpdateShootAtMessage();

	UpdateLookAtMessage();
	UpdateHuggeEntityMessage();
	
	UpdateDropshipAttackModeMessage();

	UpdateObjectiveMoveModeMessage();

	UpdateAddObjectiveMessage();
	UpdateAddPlayerObjectiveMessage();
	UpdatePlayerObjectiveCompleteMessage();
	UpdateSetMarkerLocationMessage();
	UpdateSetTargetEntityMessage();
	UpdateSetPlayerPickupMessage();
	UpdateSetPlayerIntelligentAmountMessage();

	UpdateQueenMessages();
}

void ToolsAVP_Message_Editing::UpdateSetPlayerIntelligentAmountMessage()
{
	if ( s_bIntelligentAmountChanged )
	{
		s_bIntelligentAmountChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}
}

void ToolsAVP_Message_Editing::UpdateAddObjectiveMessage()
{
	if ( s_bAddObjectiveChanged )
	{
		s_bAddObjectiveChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}
}

void ToolsAVP_Message_Editing::UpdateAddPlayerObjectiveMessage()
{
	if ( s_bAddPlayerObjectiveChanged )
	{
		s_bAddPlayerObjectiveChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}
}
void ToolsAVP_Message_Editing::UpdatePlayerObjectiveCompleteMessage()
{
	if ( s_bPlayerObjectiveCompleteChanged )
	{
		s_bPlayerObjectiveCompleteChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}
}
void ToolsAVP_Message_Editing::UpdateSetMarkerLocationMessage()
{
	if ( s_bSetMarkerLocationChanged )
	{
		s_bSetMarkerLocationChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}
}
void ToolsAVP_Message_Editing::UpdateSetTargetEntityMessage()
{
	if ( s_bSetTargetEntityChanged )
	{
		s_bSetTargetEntityChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}
}
void ToolsAVP_Message_Editing::UpdateSetPlayerPickupMessage()
{
	if ( s_bSetPlayerPickupChanged )
	{
		s_bSetPlayerPickupChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}
}

void ToolsAVP_Message_Editing::UpdateShootAtMessage( void )
{
	if ( s_bShootAtChanged )
	{
		AVP_SetModifiedAndUpdateMessageOverlay();
		s_bShootAtChanged = false;
	}
}

void ToolsAVP_Message_Editing::UpdateQueenMessages( void )
{
	if ( s_bQueenTargetListChanged )
	{
		AVP_SetModifiedAndUpdateMessageOverlay();
		s_bQueenTargetListChanged = false;
	}
}

void ToolsAVP_Message_Editing::UpdateLookAtMessage( void )
{
	if ( s_bLookAtChanged )
	{
		s_pxLookAtEntityMessage->m_bLookAtPlayer = s_bLookAtPlayer;
		AVP_SetModifiedAndUpdateMessageOverlay();
		s_bLookAtChanged = false;
	}
	if ( s_bLookAtMarkerStringChanged )
	{
		s_pxLookAtEntityMessage->m_uLookAtMarker = s_xLookAtMarkerString.GetHash();
		AVP_SetModifiedAndUpdateMessageOverlay();
		s_bLookAtMarkerStringChanged = false;
	}
}

void ToolsAVP_Message_Editing::UpdateHuggeEntityMessage()
{
	if ( s_bHuggedEntityChanged )
	{
		AVP_SetModifiedAndUpdateMessageOverlay();
		s_bHuggedEntityChanged = false;
	}
}

void ToolsAVP_Message_Editing::UpdateChangeHumanoidSquadSettingsMessage( void )
{
	if (s_bPlayersSquadTypeChanged)
	{
		s_pxChangeHumanoidSquadSettingsMessage->m_bIsPlayersSquad = (u_int)s_bIsPlayersSquad;	

		if (s_bIsPlayersSquad)
		{
			s_pxChangeHumanoidSquadSettingsMessage->m_bIsAiLedPlayersSquad = 0;
			s_bIsAILedPlayersSquad = false;
		}

		s_bPlayersSquadTypeChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}

	if (s_bPlayersAILedSquadTypeChanged)
	{
		s_pxChangeHumanoidSquadSettingsMessage->m_bIsAiLedPlayersSquad = (u_int)s_bIsAILedPlayersSquad;

		if (s_bIsAILedPlayersSquad)
		{
			s_pxChangeHumanoidSquadSettingsMessage->m_bIsPlayersSquad = 0;
			s_bIsPlayersSquad = false;
		}

		s_bPlayersAILedSquadTypeChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}


	if (s_bSquadCombatZoneChanged)
	{
		s_pxChangeHumanoidSquadSettingsMessage->m_bIsRestrictedToCombatZone = (u_int)s_bIsRestrictedToCombatZone;
		s_bSquadCombatZoneChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}

	if ( s_bIgnoreAlertStateChanged )
	{
		s_pxChangeHumanoidSquadSettingsMessage->m_bIgnoreAlertState = (u_int)s_bIgnoreAlertState;
		s_bIgnoreAlertStateChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}

	if ( s_bSquadCombatStateChanged )
	{
		s_bSquadCombatStateChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}
}

void ToolsAVP_Message_Editing::UpdateDropshipAttackModeMessage()
{
	if ( s_bDropshipModeChanged )
	{
		s_bDropshipModeChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}
}

void ToolsAVP_Message_Editing::UpdateObjectiveMoveModeMessage()
{
	if ( s_bObjectiveMoveModeChanged )
	{
		s_bObjectiveMoveModeChanged = false;
		AVP_SetModifiedAndUpdateMessageOverlay();
	}
}

bool ToolsAVP_Message_Editing::GetMessageDescriptionMain( int iMessageID, const Asura_StaticMessage* pxMessage, Asura_String& xOutputDesc )
{
	if ( pxMessage )
	{
		switch ( iMessageID )
		{
			case AVP_MSGID_ADD_AI_OBJECTIVE:
			{
				if ( pxMessage )
				{
					const AVP_Message_AddObjective* pxAddObjectiveMessage = static_cast<const AVP_Message_AddObjective*>(pxMessage->m_pData);

					if ( pxAddObjectiveMessage )
					{
						char szMessageBuffer[200];

						AVP_ServerNode_Objective_Marine* pxGoal = AVP_ServerNode_Objective_Marine::FindEntity( pxAddObjectiveMessage->m_uObjectiveGuid );

						if ( pxGoal )
						{
							Asura_PrintfToBfr( szMessageBuffer, "'ADD AI GOAL' Message. %s has Guid %u. Message Was Sent", sizeof(szMessageBuffer), true, pxGoal->ToolsData->Name, pxAddObjectiveMessage->m_uObjectiveGuid );
						}
						else
						{
							Asura_PrintfToBfr( szMessageBuffer, "'ADD AI GOAL' Message. 'AI GOAL' has Guid %u. Message Was Sent", sizeof(szMessageBuffer), true, pxAddObjectiveMessage->m_uObjectiveGuid );
						}

						xOutputDesc = szMessageBuffer;
						return true;
					}
				}

				return PARENT::GetMessageDescriptionMain( iMessageID, pxMessage, xOutputDesc );

			}break;

			default:
			{
				return PARENT::GetMessageDescriptionMain( iMessageID, pxMessage, xOutputDesc );
			}break;
		}
	}
	else
	{
		return PARENT::GetMessageDescriptionMain( iMessageID, pxMessage, xOutputDesc );
	}
}

bool ToolsAVP_Message_Editing::GuidHandlesThisMessage(Asura_Guid uGuid, int iMessageID) const
{
	if (uGuid == ASURA_GUID_CLIENT || uGuid == ASURA_GUID_ALL_CLIENTMACHINES)
	{
		switch (iMessageID)
		{
			case AVP_MSGID_START_CAMERA_SHAKE:
			case AVP_MSGID_STOP_CAMERA_SHAKE:
			{
				return true;
			}
		}
	}
	else if (uGuid == ASURA_GUID_SERVER)
	{
		switch (iMessageID)
		{
			case AVP_MSGID_END_OF_LEVEL:
			{
				return true;
			}

			default: return(PARENT::GuidHandlesThisMessage(uGuid, iMessageID));
		}
	}
	else if ( uGuid == ASURA_GUID_FIRSTPLAYER )
	{
		switch (iMessageID)
		{
			case AVP_MSGID_LOOK_AT_ENTITY:
			case AVP_MSGID_STOP_LOOKING_AT_ENTITY:
			case AVP_MSGID_PICKUP_SET_INTELLIGENT_AMOUNT:
			case AVP_MSGID_ADD_PLAYER_OBJECTIVE:
			case AVP_MSGID_SET_PLAYER_OBJECTIVE_COMPLETE:
			case AVP_MSGID_FUNCTIONALITY_MOTION_TRACKER_ON:
			case AVP_MSGID_FUNCTIONALITY_MOTION_TRACKER_OFF:
			case AVP_MSGID_FUNCTIONALITY_VISION_MODES_ON:
			case AVP_MSGID_FUNCTIONALITY_VISION_MODES_OFF:
			case AVP_MSGID_FUNCTIONALITY_TORCH_ON:
			case AVP_MSGID_FUNCTIONALITY_TORCH_OFF:
			case AVP_MSGID_REQUEST_CLOAK_ON:
			case AVP_MSGID_REQUEST_CLOAK_OFF:
			case AVP_MSGID_REPLENISH_PREDATOR_ENERGY:
			case AVP_MSGID_ADD_PLAYER_PICKUP:
			case AVP_MSGID_ALIENQUEEN_ADD_TO_TARGET_LIST:
			case AVP_MSGID_ALIENQUEEN_REMOVE_FROM_TARGET_LIST:
			case AVP_MSGID_ATTACH_WEAPON:
			case AVP_MSGID_DETACH_WEAPON:
			{
				return true;
			}
		}
	}

	return(PARENT::GuidHandlesThisMessage(uGuid,iMessageID));
}

void ToolsAVP_Message_Editing::InitialiseMessageData(struct Asura_StaticMessage* pxMessage)
{
	Asura_Assert(pxMessage, "Message is NULL");
	pxMessage->DeallocateMessageData();

	switch(pxMessage->m_usMsgID)
	{
		case AVP_MSGID_CHUNK:
		{
			pxMessage->AllocateMessageData(sizeof(AVP_Message_ChunkActor), true);
            break;
		}

		case AVP_MSGID_CHANGE_ALIEN_SPAWN_SETTINGS:
		{
			pxMessage->AllocateMessageData(sizeof(AVP_Message_ChangeAlienSpawnSettings), true);

			AVP_Message_ChangeAlienSpawnSettings* pxMessageData = static_cast<AVP_Message_ChangeAlienSpawnSettings*>(pxMessage->m_pData);
			if( pxMessageData )
			{
				pxMessageData->Initialise();
			}
            break;
		}

		case AVP_MSGID_ADD_ALIENS_TO_SPAWN_QUEUE:
		{
			pxMessage->AllocateMessageData(sizeof(AVP_Message_AddAlienToSpawnQueue), true);
            break;
		}

		case AVP_MSGID_ADD_AI_OBJECTIVE:
		{
			pxMessage->AllocateMessageData(sizeof(AVP_Message_AddObjective), true);

			AVP_Message_AddObjective	*pxAddObjectiveMessage = static_cast<AVP_Message_AddObjective*>(pxMessage->m_pData);

			if ( pxAddObjectiveMessage )
			{
				pxAddObjectiveMessage->Initialise();
			}
			
            break;
		}

		case AVP_MSGID_CHANGE_HUMANOID_SPAWN_SETTINGS:
		{
			pxMessage->AllocateMessageData(sizeof(AVP_Message_ChangeHumanoidSpawnSettings), true);

			AVP_Message_ChangeHumanoidSpawnSettings	*pxSpawnSettingsMessage = static_cast<AVP_Message_ChangeHumanoidSpawnSettings*>(pxMessage->m_pData);

			if (pxSpawnSettingsMessage)
			{
				pxSpawnSettingsMessage->Initialise();
			}
			break;
		}

		case AVP_MSGID_SPAWN_HUMANOIDS:
		{
			pxMessage->AllocateMessageData(sizeof(AVP_Message_SpawnHumanoids), true);

			AVP_Message_SpawnHumanoids	*pxSpawnHumanoidsMessage = static_cast<AVP_Message_SpawnHumanoids*>(pxMessage->m_pData);
			
			if (pxSpawnHumanoidsMessage)
			{
				pxSpawnHumanoidsMessage->Initialise();
			}
			break;
		}

		case AVP_MSGID_MERGE_HUMANOID_SQUADS:
		{
			pxMessage->AllocateMessageData(sizeof(AVP_Message_MergeSquads), true);
			AVP_Message_MergeSquads *pxSquadMessage = static_cast<AVP_Message_MergeSquads*>(pxMessage->m_pData);
			pxSquadMessage->Initialise();
			break;
		}

		case AVP_MSGID_SPLIT_HUMANOID_SQUAD:
		{
			pxMessage->AllocateMessageData(sizeof(AVP_Message_SplitSquad), true);
			AVP_Message_SplitSquad *pxSquadMessage = static_cast<AVP_Message_SplitSquad*>(pxMessage->m_pData);
			pxSquadMessage->Initialise();
			break;
		}

		case AVP_MSGID_ADD_SQUAD_MEMBER:
		{
			pxMessage->AllocateMessageData(sizeof(AVP_Message_AddSquadMember), true);

			AVP_Message_AddSquadMember *pxAddSquadMemberMessage = static_cast<AVP_Message_AddSquadMember*>(pxMessage->m_pData);

			pxAddSquadMemberMessage->m_uVersion = 1;
			pxAddSquadMemberMessage->SetGuidToAdd(ASURA_GUID_UNREGISTERED);
            break;
		}

		case AVP_MSGID_REMOVE_SQUAD_MEMBER:
		{
			pxMessage->AllocateMessageData(sizeof(AVP_Message_RemoveSquadMember), true);

			AVP_Message_RemoveSquadMember *pxRemoveSquadMemberMessage = static_cast<AVP_Message_RemoveSquadMember*>(pxMessage->m_pData);

			pxRemoveSquadMemberMessage->m_uVersion = 1;
			pxRemoveSquadMemberMessage->SetGuidToRemove(ASURA_GUID_UNREGISTERED);
            break;
		}

		case AVP_MSGID_REQUEST_HUMANOID_PATROL:
		{
			pxMessage->AllocateMessageData(sizeof(AVP_Message_RequestPatrol), true);
			AVP_Message_RequestPatrol	*pxRequestPatrol = static_cast<AVP_Message_RequestPatrol*>(pxMessage->m_pData);
			pxRequestPatrol->Initialise();
			break;
		}

		case AVP_MSGID_ALIEN_TEAM_SET_BEHAVIOUR:
		{
			pxMessage->AllocateMessageData(sizeof(AVP_Message_SetAlienTeamBehaviour), true);

			AVP_Message_SetAlienTeamBehaviour *pxMessageData = static_cast<AVP_Message_SetAlienTeamBehaviour*>(pxMessage->m_pData);
			if( pxMessageData )
			{
				pxMessageData->Initialise();
			}
			break;
		}

		case AVP_MSGID_ALIEN_TEAM_ADD_SPAWNTOKEN:
		{
			pxMessage->AllocateMessageData(sizeof(AVP_Message_AddAlienTeamTokens), true);

			AVP_Message_AddAlienTeamTokens *pxMessageData = static_cast<AVP_Message_AddAlienTeamTokens*>(pxMessage->m_pData);
			if( pxMessageData )
			{
				pxMessageData->Initialise();
			}
			break;
		}

		case AVP_MSGID_ALIEN_TEAM_REMOVE_SPAWNTOKEN:
		{
			pxMessage->AllocateMessageData(sizeof(AVP_Message_RemoveAlienTeamTokens), true);

			AVP_Message_RemoveAlienTeamTokens* pxMessageData = static_cast<AVP_Message_RemoveAlienTeamTokens*>(pxMessage->m_pData);
			if( pxMessageData )
			{
				pxMessageData->Initialise();
			}
			break;
		}

		case AVP_MSGID_ALIEN_TEAM_THREAT_BONUS:
		{
			pxMessage->AllocateMessageData(sizeof(AVP_Message_ChangeThreatBonusValues), true);

			AVP_Message_ChangeThreatBonusValues* pxMessageData = static_cast<AVP_Message_ChangeThreatBonusValues*>(pxMessage->m_pData);
			if( pxMessageData )
			{
				pxMessageData->Initialise();
			}
			break;
		}

		case AVP_MSGID_PERFORM_AI_OBJECTIVE_ACTION:
		{
			pxMessage->AllocateMessageData(sizeof(AVP_Message_PerformObjectiveAction), true);

			AVP_Message_PerformObjectiveAction* pxMessageData = static_cast<AVP_Message_PerformObjectiveAction*>(pxMessage->m_pData);
			
			if (pxMessageData)
			{
				pxMessageData->Initialise();
			}

			break;
		}

		case AVP_MSGID_ALIEN_TEAM_PREPARE_ROOM:
        case AVP_MSGID_REQUEST_CLOAK_ON:
        case AVP_MSGID_REQUEST_CLOAK_OFF:
		case AVP_MSGID_REQUEST_TORCH_ON:
		case AVP_MSGID_REQUEST_TORCH_OFF:
		{
			break;
		}

		case AVP_MSGID_SET_PREFERRED_TARGET_TYPE:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_SetPreferredTargetType ), true );

			AVP_Message_SetPreferredTargetType* pxData = static_cast< AVP_Message_SetPreferredTargetType* >( pxMessage->m_pData );
			if ( pxData )
			{
				pxData->m_uVersion			 = AVP_Message_SetPreferredTargetType::CURRENT_VERSION;
				pxData->m_iPreferredTargetType = AVP_Message_SetPreferredTargetType::PTT_NONE;
			}

			break;
		}

		case AVP_MSGID_SPAWN:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Msg_Spawn ), true );

			AVP_Msg_Spawn* pxData = static_cast< AVP_Msg_Spawn* >( pxMessage->m_pData );
			if ( pxData )
			{
				pxData->m_uVersion = AVP_Msg_Spawn::uCURRENT_VERSION;
				pxData->m_uBPTypeHashID = ASURA_HASH_ID_UNSET;
                pxData->m_uBPHashID     = ASURA_HASH_ID_UNSET;
				pxData->m_uSquadGuid	= ASURA_GUID_UNREGISTERED;
			}

			break;
		}

		case AVP_MSGID_SPAWN_SET_MAX_SPAWN_COUNT:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Msg_Spawn_SetMaxSpawnCount ), true );

			AVP_Msg_Spawn_SetMaxSpawnCount* pxData = static_cast< AVP_Msg_Spawn_SetMaxSpawnCount* >( pxMessage->m_pData );
			if ( pxData )
			{
				pxData->m_uVersion		 = AVP_Msg_Spawn_SetMaxSpawnCount::uCURRENT_VERSION;
				pxData->m_iMaxSpawnCount = -1;
			}

			break;
		}

		case AVP_MSGID_SPAWN_RESET_SPAWN_COUNTER:
		{
			break;
		}

		case AVP_MSGID_SPAWN_SET_MAX_IN_PLAY:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Msg_Spawn_SetMaxInPlay ), true );

			AVP_Msg_Spawn_SetMaxInPlay* pxData = static_cast< AVP_Msg_Spawn_SetMaxInPlay* >( pxMessage->m_pData );
			if ( pxData )
			{
				pxData->m_uVersion	 = AVP_Msg_Spawn_SetMaxInPlay::uCURRENT_VERSION;
				pxData->m_iMaxInPlay = 3;
			}

			break;
		}

		case AVP_MSGID_SET_ALIEN_SQUAD_BEHAVIOUR:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Msg_SetAlienSquadBehaviour ), true );

			s_pxSetAlienSquadBehaviourMsg = static_cast< AVP_Msg_SetAlienSquadBehaviour* >( pxMessage->m_pData );
			if ( s_pxSetAlienSquadBehaviourMsg )
			{
				s_pxSetAlienSquadBehaviourMsg->m_uVersion = AVP_Msg_SetAlienSquadBehaviour::uCURRENT_VERSION;

				AVP_AlienBehaviourData xBehaviourData;
				s_pxSetAlienSquadBehaviourMsg->m_iBehaviour				= xBehaviourData.m_iBehaviour;
				s_pxSetAlienSquadBehaviourMsg->m_uProtoBlueprintHashID	= xBehaviourData.m_uProtoBlueprintHashID;
				s_pxSetAlienSquadBehaviourMsg->m_fJumpFraction			= xBehaviourData.m_fJumpFraction;
				s_pxSetAlienSquadBehaviourMsg->m_fStandFraction			= xBehaviourData.m_fStandFraction;
				s_pxSetAlienSquadBehaviourMsg->m_fMinTimeBetweenGrapples= xBehaviourData.m_fMinTimeBetweenGrapples;
				s_pxSetAlienSquadBehaviourMsg->m_fDamageFromBehindMultiplier= xBehaviourData.m_fDamageFromBehindMultiplier;
				s_pxSetAlienSquadBehaviourMsg->m_iMoveInMode			= xBehaviourData.m_iMoveInMode;
				s_pxSetAlienSquadBehaviourMsg->m_fIdealDistanceMin		= xBehaviourData.m_fIdealDistanceMin;
				s_pxSetAlienSquadBehaviourMsg->m_fIdealDistanceMax		= xBehaviourData.m_fIdealDistanceMax;
				s_pxSetAlienSquadBehaviourMsg->m_fAttackDelay			= xBehaviourData.m_fAttackDelay;
				s_pxSetAlienSquadBehaviourMsg->m_iMaxAttackersPerTarget = xBehaviourData.m_iMaxAttackersPerTarget;
				s_pxSetAlienSquadBehaviourMsg->m_iMaxAttackersTotal		= xBehaviourData.m_iMaxAttackersTotal;
				s_pxSetAlienSquadBehaviourMsg->m_iMaxBurstPerTarget		= xBehaviourData.m_iMaxBurstPerTarget;
				s_pxSetAlienSquadBehaviourMsg->m_iPostAttackAction		= xBehaviourData.m_iPostAttackAction;
				s_pxSetAlienSquadBehaviourMsg->m_fRespawnDelay			= xBehaviourData.m_fRespawnDelay;
				s_pxSetAlienSquadBehaviourMsg->m_fTimeBetweenAttacks	= xBehaviourData.m_fTimeBetweenAttacks;
				s_pxSetAlienSquadBehaviourMsg->m_uMaxNumberOfChargers	= xBehaviourData.m_uMaxNumberOfChargers;
				s_pxSetAlienSquadBehaviourMsg->m_bAttackIfInMarineLight		= xBehaviourData.m_bAttackIfInMarineLight;
				s_pxSetAlienSquadBehaviourMsg->m_fTimeInLightBeforeAttack = xBehaviourData.m_fTimeInLightBeforeAttack;
				s_pxSetAlienSquadBehaviourMsg->m_fMinHideDistance		= xBehaviourData.m_fMinHideDistance;
				s_pxSetAlienSquadBehaviourMsg->m_fChaneToRehideIfCloseFraction	= xBehaviourData.m_fChaneToRehideIfCloseFraction;
				s_pxSetAlienSquadBehaviourMsg->m_fChaneToRehideIfInLightFraction = xBehaviourData.m_fChaneToRehideIfInLightFraction;
				s_pxSetAlienSquadBehaviourMsg->m_fChaneToRehideAfterAttackFraction = xBehaviourData.m_fChaneToRehideAfterAttackFraction;
			}

			break;
		}

		case AVP_MSGID_SET_AI_TARGET_PRIORITIES:
		{
			const size_t uDataSize = sizeof( AVP_Msg_SetAITargetPriorities ) + ( 1 * sizeof( AVP_Msg_SetAITargetPriorities::TPData ) );
			pxMessage->AllocateMessageData( uDataSize, true );

			s_pxSetAITargetPrioritiesData = static_cast< AVP_Msg_SetAITargetPriorities* >( pxMessage->m_pData );
			if ( s_pxSetAITargetPrioritiesData )
			{
				s_pxSetAITargetPrioritiesData->Initialise();
				AVP_Msg_SetAITargetPriorities::TPData* pxTPData = s_pxSetAITargetPrioritiesData->GetFirstTPDataPtr();
				pxTPData->m_iPriority = AVP_AI_TargetPriorityData::UNSET;
			}
			break;
		}

		case AVP_MSGID_CHANGE_MARINE_SQUAD_SETTINGS:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_ChangeMarineSquadSettings ), true );

			AVP_Message_ChangeMarineSquadSettings* pxMessageData = static_cast< AVP_Message_ChangeMarineSquadSettings* >( pxMessage->m_pData );
			
			if ( pxMessageData )
			{
				pxMessageData->Initialise();
			}

		}break;

		case AVP_MSGID_ATTACK_SCRIPTED_TARGET:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_AttackScriptedTarget ), true);

			AVP_Message_AttackScriptedTarget* pxMessageData = static_cast< AVP_Message_AttackScriptedTarget*>( pxMessage->m_pData );

			if (pxMessageData)
			{
				pxMessageData->Initialise();
			}

		}break;

		case AVP_MSGID_PICKUP_SET_INTELLIGENT_AMOUNT:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_Pickup_SetIntelligentAmount ), true );

			AVP_Message_Pickup_SetIntelligentAmount* pxData = static_cast< AVP_Message_Pickup_SetIntelligentAmount* >( pxMessage->m_pData );
			if ( pxData )
			{
				pxData->m_uVersion		     = AVP_Message_Pickup_SetIntelligentAmount::uCURRENT_VERSION;
				pxData->m_iIntelligentAmount = AVP_ServerEntity_PlayerInfo::INTELLIGENT_AMOUNT_MEDIUM;
			}

			break;
		}

		case AVP_MSGID_ADD_PLAYER_OBJECTIVE:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_AddPlayerObjective ), true );

			AVP_Message_AddPlayerObjective* pxData = static_cast< AVP_Message_AddPlayerObjective* >( pxMessage->m_pData );
			if ( pxData )
			{
				pxData->m_uVersion		     = AVP_Message_AddPlayerObjective::uCURRENT_VERSION;
				pxData->m_uObjectiveGuid     = ASURA_GUID_UNREGISTERED;
			}

			break;
		}

		case AVP_MSGID_SET_PLAYER_OBJECTIVE_COMPLETE:
	    {
			pxMessage->AllocateMessageData( sizeof( AVP_Message_PlayerObjectiveComplete ), true );

			AVP_Message_PlayerObjectiveComplete* pxData = static_cast< AVP_Message_PlayerObjectiveComplete* >( pxMessage->m_pData );
			if ( pxData )
			{
				pxData->m_uVersion		     = AVP_Message_PlayerObjectiveComplete::uCURRENT_VERSION;
				pxData->m_uObjectiveGuid     = ASURA_GUID_UNREGISTERED;
			}

			break;
	    }

		case AVP_MSGID_PLAYER_OBJECTIVE_SET_MARKER_LOCATION:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_PlayerObjectiveSetMarker ), true );

			AVP_Message_PlayerObjectiveSetMarker* pxData = static_cast< AVP_Message_PlayerObjectiveSetMarker* >( pxMessage->m_pData );
			if ( pxData )
			{
				pxData->m_uVersion		     = AVP_Message_PlayerObjectiveSetMarker::uCURRENT_VERSION;
				pxData->m_uMarkerGuid        = ASURA_GUID_UNREGISTERED;
			}

			break;
		}

		case AVP_MSGID_PLAYER_OBJECTIVE_SET_TARGET_ENTITY:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_PlayerObjectiveSetEntity ), true );

			AVP_Message_PlayerObjectiveSetEntity* pxData = static_cast< AVP_Message_PlayerObjectiveSetEntity* >( pxMessage->m_pData );
			if ( pxData )
			{
				pxData->m_uVersion		     = AVP_Message_PlayerObjectiveSetEntity::uCURRENT_VERSION;
				pxData->m_uEntityGuid        = ASURA_GUID_UNREGISTERED;
			}

			break;
		}
		
		case AVP_MSGID_ADD_PLAYER_PICKUP:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_AddPlayerPickup ), true );

			AVP_Message_AddPlayerPickup* pxData = static_cast< AVP_Message_AddPlayerPickup* >( pxMessage->m_pData );
			if ( pxData )
			{
				pxData->m_uVersion		     = AVP_Message_AddPlayerObjective::uCURRENT_VERSION;
				pxData->m_uPickupGuid        = ASURA_GUID_UNREGISTERED;
				pxData->m_bSetAsCurrent      = true;
			}

			break;
		}

		case AVP_MSGID_SET_DROPSHIP_ATTACK_MODE:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_DropshipAttackMode ), true );

			AVP_Message_DropshipAttackMode* pxData = static_cast< AVP_Message_DropshipAttackMode* >( pxMessage->m_pData );
			if ( pxData )
			{
				pxData->Initialise();
			}

			break;
		}

		case AVP_MSGID_CHANGE_HUMANOID_SQUAD_MOVEMENT_MODE:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_ChangeHumanoidMovementMode ), true );

			AVP_Message_ChangeHumanoidMovementMode* pxData = static_cast< AVP_Message_ChangeHumanoidMovementMode* >( pxMessage->m_pData );

			if ( pxData )
			{
				pxData->Initialise();
			}
			break;
		}

		case AVP_MSGID_SET_PREDATOR_BEHAVIOUR:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_SetPredatorBehaviour ), true );

			AVP_Message_SetPredatorBehaviour* pxData = static_cast< AVP_Message_SetPredatorBehaviour* >( pxMessage->m_pData );
			if ( pxData )
			{
				pxData->m_uVersion		     = AVP_Message_SetPredatorBehaviour::uCURRENT_VERSION;
				pxData->m_iBehaviour		 = AVP_PredatorBehaviourData::STALK;
			}

			break;
		}

		case AVP_MSGID_LOOK_AT_ENTITY:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_LookAtEntity), true );

			AVP_Message_LookAtEntity* pxData = static_cast< AVP_Message_LookAtEntity* >( pxMessage->m_pData );

			if ( pxData )
			{
				pxData->Initialise();
			}

		}break;

		case AVP_MSGID_SET_GS2_MOVE_SPEED:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_SetGS2MoveSpeed ), true );

			AVP_Message_SetGS2MoveSpeed* pxData = static_cast< AVP_Message_SetGS2MoveSpeed* >( pxMessage->m_pData );

			if ( pxData )
			{
				pxData->Initialise();
			}

		}break;

		case AVP_MSGID_SET_HEALTH:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_SetHealth ), true );

			AVP_Message_SetHealth* pxData = static_cast< AVP_Message_SetHealth* >( pxMessage->m_pData );

			if ( pxData )
			{
				pxData->Initialise();
			}

		}break;

		case AVP_MSGID_SET_FACEHUGGER_SQUAD_BEHAVIOUR:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Msg_SetFaceHuggerSquadBehaviour ), true );

			s_pxSetFaceHuggerSquadBehaviourMsg = static_cast< AVP_Msg_SetFaceHuggerSquadBehaviour* >( pxMessage->m_pData );
			if ( s_pxSetFaceHuggerSquadBehaviourMsg )
			{
				s_pxSetFaceHuggerSquadBehaviourMsg->m_uVersion = AVP_Msg_SetFaceHuggerSquadBehaviour::uCURRENT_VERSION;

				AVP_FaceHuggerBehaviourData xBehaviourData;
				s_pxSetFaceHuggerSquadBehaviourMsg->m_iBehaviour					= xBehaviourData.m_iBehaviour;
				s_pxSetFaceHuggerSquadBehaviourMsg->m_fAvoidanceDistance			= xBehaviourData.m_fAvoidanceDistance;
				s_pxSetFaceHuggerSquadBehaviourMsg->m_uMaxTargeters					= xBehaviourData.m_uMaxTargeters;
				s_pxSetFaceHuggerSquadBehaviourMsg->m_fTimeBetweenAttacks			= xBehaviourData.m_fTimeBetweenAttacks;
				s_pxSetFaceHuggerSquadBehaviourMsg->m_fWanderDistance				= xBehaviourData.m_fWanderDistance;
				s_pxSetFaceHuggerSquadBehaviourMsg->m_bAttackIfInMarineLight		= xBehaviourData.m_bAttackIfInMarineLight;
				s_pxSetFaceHuggerSquadBehaviourMsg->m_fTimeInLightBeforeAttack		= xBehaviourData.m_fTimeInLightBeforeAttack;
				s_pxSetFaceHuggerSquadBehaviourMsg->m_fMinHideDistance				= xBehaviourData.m_fMinHideDistance;
				s_pxSetFaceHuggerSquadBehaviourMsg->m_fChaneToRehideIfCloseFraction	= xBehaviourData.m_fChaneToRehideIfCloseFraction;
				s_pxSetFaceHuggerSquadBehaviourMsg->m_fChaneToRehideIfInLightFraction = xBehaviourData.m_fChaneToRehideIfInLightFraction;
				s_pxSetFaceHuggerSquadBehaviourMsg->m_fChaneToRehideAfterAttackFraction = xBehaviourData.m_fChaneToRehideAfterAttackFraction;
			}
		}break;

		case AVP_MSGID_CHANGE_HUMANOID_KNOWLEDGE_SETTINGS:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_ChangeHumanoidKnowledgeSettings ), true );

			s_pxSetHumanoidKnowledgeSettingsMsg = static_cast< AVP_Message_ChangeHumanoidKnowledgeSettings* >( pxMessage->m_pData );

			if ( s_pxSetHumanoidKnowledgeSettingsMsg )
			{
				s_pxSetHumanoidKnowledgeSettingsMsg->Initialise();
			}

		}break;

		case AVP_MSGID_SET_SENTRYGUN_SWEEP_SPEED:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_SentryGun_Set_SweepSpeed ), true );

			AVP_Message_SentryGun_Set_SweepSpeed* pxData = static_cast< AVP_Message_SentryGun_Set_SweepSpeed* >( pxMessage->m_pData );

			if ( pxData )
			{
				pxData->Initialise();
			}
			break;
		}

		case AVP_MSGID_SET_SENTRYGUN_VISION_RANGE:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_SentryGun_Set_VisionRange ), true );

			AVP_Message_SentryGun_Set_VisionRange* pxData = static_cast< AVP_Message_SentryGun_Set_VisionRange* >( pxMessage->m_pData );

			if ( pxData )
			{
				pxData->Initialise();
			}
			break;
		}

		case AVP_MSGID_SET_SENTRYGUN_HOR_FOV:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_SentryGun_Set_HorizontalFOV ), true );

			AVP_Message_SentryGun_Set_HorizontalFOV* pxData = static_cast< AVP_Message_SentryGun_Set_HorizontalFOV* >( pxMessage->m_pData );

			if ( pxData )
			{
				pxData->Initialise();
			}
			break;
		}

		case AVP_MSGID_SET_SENTRYGUN_VER_FOV:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_SentryGun_Set_VerticalFOV ), true );

			AVP_Message_SentryGun_Set_VerticalFOV* pxData = static_cast< AVP_Message_SentryGun_Set_VerticalFOV* >( pxMessage->m_pData );

			if ( pxData )
			{
				pxData->Initialise();
			}
			break;
		}

		case AVP_MSGID_SET_SENTRYGUN_FIRING_ARC:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_SentryGun_Set_FiringArc ), true );

			AVP_Message_SentryGun_Set_FiringArc* pxData = static_cast< AVP_Message_SentryGun_Set_FiringArc* >( pxMessage->m_pData );

			if ( pxData )
			{
				pxData->Initialise();
			}
			break;
		}

		case AVP_MSGID_SET_SENTRYGUN_LETHAL_RANGE:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_SentryGun_Set_LethalRange ), true );

			AVP_Message_SentryGun_Set_LethalRange* pxData = static_cast< AVP_Message_SentryGun_Set_LethalRange* >( pxMessage->m_pData );

			if ( pxData )
			{
				pxData->Initialise();
			}
			break;
		}

		case AVP_MSGID_SENTRYGUN_DRYFIRE_TOGGLE:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_SentryGun_Toggle_Dry_Fire ), true );

			AVP_Message_SentryGun_Toggle_Dry_Fire* pxData = static_cast< AVP_Message_SentryGun_Toggle_Dry_Fire* >( pxMessage->m_pData );

			if ( pxData )
			{
				pxData->Initialise();
			}
			break;
		}

		case AVP_MSGID_DETACH_WEAPON:
		case AVP_MSGID_CREATE_ATTACHMENT:
		case AVP_MSGID_DESTROY_ATTACHMENT:
		case AVP_MSGID_DETACH_ATTACHMENT:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_Actor_Attachment ), true );

			AVP_Message_Actor_Attachment* pxData = static_cast< AVP_Message_Actor_Attachment* >( pxMessage->m_pData );

			if ( pxData )
			{
				pxData->Initialise();
			}
			break;
		}

		case AVP_MSGID_ATTACH_WEAPON:
		{
			break; // no data
		}

		case AVP_MSGID_ALIENQUEEN_ADD_TO_TARGET_LIST:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Msg_Add_Entity_To_Queen_Target_List ), true );

			AVP_Msg_Add_Entity_To_Queen_Target_List* pxData = static_cast< AVP_Msg_Add_Entity_To_Queen_Target_List* >( pxMessage->m_pData );

			if ( pxData )
			{
				pxData->Initialise();
			}
			break;
		}

		case AVP_MSGID_ALIENQUEEN_REMOVE_FROM_TARGET_LIST:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Msg_Remove_Entity_From_Queen_Target_List ), true );

			AVP_Msg_Remove_Entity_From_Queen_Target_List* pxData = static_cast< AVP_Msg_Remove_Entity_From_Queen_Target_List* >( pxMessage->m_pData );

			if ( pxData )
			{
				pxData->Initialise();
			}
			break;
		}

		case AVP_MSGID_ALIENQUEEN_FORCE_ROAR:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Msg_Alien_Queen_Start_Roar_Behaviour ), true );

			AVP_Msg_Alien_Queen_Start_Roar_Behaviour* pxData = static_cast<AVP_Msg_Alien_Queen_Start_Roar_Behaviour*>( pxMessage->m_pData );

			if ( pxData )
			{
				pxData->Initialise();
			}
			break;
		}

		case AVP_MSGID_ALIENQUEEN_SET_BEHAVIOUR:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Message_SetAlienQueenBehaviour ), true );

			AVP_Message_SetAlienQueenBehaviour* pxData = static_cast< AVP_Message_SetAlienQueenBehaviour* >( pxMessage->m_pData );
			if ( pxData )
			{
				pxData->Initialise();
			}
			break;
		}

		case AVP_MSGID_START_CAMERA_SHAKE:
		{
			pxMessage->AllocateMessageData( sizeof(AVP_Msg_CameraShake), true );

			AVP_Msg_CameraShake* pxMsgData = static_cast<AVP_Msg_CameraShake*>(pxMessage->m_pData);
			pxMsgData->m_uVersion = AVP_Msg_CameraShake::CURRENT_VERSION;
			pxMsgData->m_uShake = ASURA_HASH_ID_UNSET;
			pxMsgData->m_uSourceGuid = ASURA_GUID_UNREGISTERED;
			break;
		}

		case AVP_MSGID_STOP_CAMERA_SHAKE:
		{
			pxMessage->AllocateMessageData( sizeof(AVP_Msg_CameraShake), true );

			AVP_Msg_CameraShake* pxMsgData = static_cast<AVP_Msg_CameraShake*>(pxMessage->m_pData);
			pxMsgData->m_uVersion = AVP_Msg_CameraShake::CURRENT_VERSION;
			pxMsgData->m_uSourceGuid = ASURA_GUID_UNREGISTERED;
			break;
		}

		case AVP_MSGID_FACEHUGGER_HUG_ENTITY:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_FaceHugger_Hug_Entity ), true );

			AVP_FaceHugger_Hug_Entity* pxData = static_cast< AVP_FaceHugger_Hug_Entity* >( pxMessage->m_pData );

			if ( pxData )
			{
				pxData->Initialise();
			}
			break;

		}
		case AVP_MSGID_DISMEMBER:
		{
			pxMessage->AllocateMessageData( sizeof( AVP_Msg_Dismember ), true );

			AVP_Msg_Dismember* pxData = static_cast< AVP_Msg_Dismember* >( pxMessage->m_pData );

			if ( pxData )
			{
				pxData->Initialise();
			}

		}break;


		default:
		{
			PARENT::InitialiseMessageData(pxMessage);
            break;
		}
	}
}

void ToolsAVP_Message_Editing::AddOverlayControls(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	// Set the intellignet quantities array of strings
	static const char* s_ppszIntelligentQuantities[10] = {"Scarce", "medium", "plenty"};

	switch(pxMessage->m_usMsgID)
	{
		// Deprecated messages.
		case AVP_MSGID_CHANGE_ALIEN_SPAWN_SETTINGS:
		case AVP_MSGID_ADD_ALIENS_TO_SPAWN_QUEUE:
		case AVP_MSGID_ALIEN_TEAM_SET_BEHAVIOUR:
		case AVP_MSGID_CHANGE_HUMANOID_SPAWN_SETTINGS:
		case AVP_MSGID_SPAWN_HUMANOIDS:
		case AVP_MSGID_ALIEN_TEAM_ADD_SPAWNTOKEN:
		case AVP_MSGID_ALIEN_TEAM_REMOVE_SPAWNTOKEN:
		case AVP_MSGID_ALIEN_TEAM_PREPARE_ROOM:
		{
			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_CHUNK:
		{
			Asura_Dialog* pxNewDialogue;

			s_pxChunkActorMessage = static_cast<AVP_Message_ChunkActor*>(pxMessage->m_pData);

			pxNewDialogue = new Asura_DropDown_List_Dialog(xOffset,Asura_Project_Skin_Elements::GetElementNames(),&s_pxChunkActorMessage->m_iElement, NUM_AVP_SKIN_ELEMENT_TYPES, NUM_AVP_SKIN_ELEMENT_TYPES, NULL, NULL,"");
			pxNewDialogue->SetHelpText( "The limb to be removed" );
			pxOverlay->AddDialog(pxNewDialogue);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxNewDialogue = new Asura_Numerical_Dialog("Damage",xOffset,&s_pxChunkActorMessage->m_fDamage, 0.0f, 10000.0f , 10.0f, "%.1f", NULL);
			pxNewDialogue->SetHelpText( "Damage amount (Health)" );
			pxOverlay->AddDialog(pxNewDialogue);
			xOffset.y += Asura_Dialog::DefaultSpacing();
            break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_ADD_AI_OBJECTIVE:
		{
			s_pxAddObjectiveMessage = static_cast<AVP_Message_AddObjective*>(pxMessage->m_pData);
			AVP_Message_AddObjective* pxMsgData = s_pxAddObjectiveMessage;

			if (pxMsgData->m_uVersion > 2)
			{
				Asura_Assert(false, "AVP_MSGID_ADD_OBJECTIVE - Version has changed. Fix up the data!!!");
			}

			Asura_String xObjectiveString( "AI Goal: " );

			if (pxMsgData->GetObjectiveGuid() == ASURA_GUID_UNREGISTERED)
			{
				xObjectiveString += "None";
			}
			else
			{
				ToolsAVP_Node_Objective_Marine* pxToolsObjective = (ToolsAVP_Node_Objective_Marine*)ToolsAVP_Node_Objective_Marine::FindEntity(pxMsgData->GetObjectiveGuid());

				if (pxToolsObjective)
				{
					if (pxToolsObjective->GetName())
					{
						xObjectiveString += pxToolsObjective->GetName();
					}
					else
					{
						Asura_String xGuidString;
						Asura_StringUtil::ToString( pxToolsObjective->GetGuid(), xGuidString );
						xObjectiveString += xGuidString;
					}
				}
				else
				{
					xObjectiveString += "Error";
				}
			}

			Asura_Dialog* pxDialog;
			pxDialog = new Asura_Function_Dialog(xObjectiveString.GetString(), xOffset, SelectObjective);
			pxDialog->SetHelpText( "The Goal Point" );
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

// 			pxDialog = new Asura_DropDown_List_Dialog(xOffset, AVP_ServerNode_Objective_Marine::GetObjectiveTypeNames(), &pxMsgData->m_iObjectiveType, AVP_ServerNode_Objective_Marine::MAX_OBJECTIVE_TYPES, AVP_ServerNode_Objective_Marine::MAX_OBJECTIVE_TYPES, NULL, NULL,"");
// 			pxDialog->SetHelpText("The type of Goal");
// 			pxOverlay->AddDialog(pxDialog);
// 			xOffset.y += Asura_Dialog::LargeSpacing();
			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_MERGE_HUMANOID_SQUADS:
		{
			s_pxMergeSquadMessage = static_cast<AVP_Message_MergeSquads*>(pxMessage->m_pData);
			AVP_Message_MergeSquads* pxMsgData = s_pxMergeSquadMessage;

			if (pxMsgData->m_uVersion > 1)
			{
				Asura_Assert(false, "AVP_MSGID_MERGE_HUMANOIDS_SQUADS - Version has changed. Fix up the data!!!");
			}

			Asura_Dialog* pxDialog;

			pxDialog = new Asura_StringDisplay_Dialog("Squad Merge Humanoids Into:", xOffset);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_Function_Dialog("", xOffset, SelectMarineSquadToMergeInto);
			pxDialog->SetHelpText("Select Squad To Merge Into");
			s_pxSquadTextDialog = pxDialog;
			UpdateSquadTextDialog(pxMsgData->m_uSquadToMergeInto);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SPLIT_HUMANOID_SQUAD:
		{
			s_uSplitSquadSentToGuid = pxMessage->m_uSendTo;
			s_pxSplitSquadMessage = static_cast<AVP_Message_SplitSquad*>(pxMessage->m_pData);
			AVP_Message_SplitSquad* pxMsgData = s_pxSplitSquadMessage;

			if (pxMsgData->m_uVersion > 1)
			{
				Asura_Assert(false, "AVP_MSGID_MERGE_HUMANOIDS_SQUADS - Version has changed. Fix up the data!!!");
			}

			Asura_Dialog* pxDialog;

			pxDialog = new Asura_StringDisplay_Dialog("Squad To Add Members To:", xOffset);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_Function_Dialog("", xOffset, SelectSquadToAddTo);
			pxDialog->SetHelpText("Select Squad To Add Members Too");
			s_pxSquadTextDialog = pxDialog;
			UpdateSquadTextDialog(pxMsgData->m_uSquadToAddTo);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::LargeSpacing();

			// Members to remove!
			pxDialog = new Asura_StringDisplay_Dialog("Select Members To Move To Other Squad:", xOffset);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			u_int uNumSelected = 0;

			for (u_int u = 0; u < AVP_Message_SplitSquad::MAX_SQUAD_MEMBERS; ++u)
			{
				if (pxMsgData->m_auSplitMembers[u] != ASURA_GUID_UNREGISTERED)
				{
					++uNumSelected;
				}
			}

			for (u_int u = 0; u < AVP_Message_SplitSquad::MAX_SQUAD_MEMBERS; ++u)
			{
				if (pxMsgData->m_auSplitMembers[u] != ASURA_GUID_UNREGISTERED)
				{
					pxDialog = new Asura_Function_Dialog("", xOffset, SelectSquadMemberToMove, u);
					pxDialog->SetHelpText("Select A Squad Member To Move To The Other Squad");
					s_pxHumanoidMemberTextDialog = pxDialog;
					UpdateSquadMemberTextDialog(pxMsgData->m_auSplitMembers[u]);
					pxOverlay->AddDialog(pxDialog);
					xOffset.y += Asura_Dialog::DefaultSpacing();

					pxDialog = new Asura_Function_Dialog("Clear Member", xOffset, ClearSplitSquadMember, u);
					pxDialog->SetHelpText("Clear the selected squad member for splitting");
					pxOverlay->AddDialog(pxDialog);
					xOffset.y += 1.5f;
				}
			}

			if (uNumSelected < AVP_Message_SplitSquad::MAX_SQUAD_MEMBERS)
			{
				pxDialog = new Asura_Function_Dialog("", xOffset, SelectSquadMemberToMove, uNumSelected);
				pxDialog->SetHelpText("Select A Squad Member To Move To The Other Squad");
				s_pxHumanoidMemberTextDialog = pxDialog;
				UpdateSquadMemberTextDialog(pxMsgData->m_auSplitMembers[uNumSelected]);
				pxOverlay->AddDialog(pxDialog);
				xOffset.y += Asura_Dialog::DefaultSpacing();
			}

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_ADD_SQUAD_MEMBER:
		{
			s_pxAddSquadMemberMessage = static_cast<AVP_Message_AddSquadMember*>(pxMessage->m_pData);
			AVP_Message_AddSquadMember* pxMsgData = s_pxAddSquadMemberMessage;

			if (pxMsgData->m_uVersion > 1)
			{
				Asura_Assert(false, "AVP_MSGID_ADD_SQUAD_MEMBER - Version has changed. Fix up the data!!!");
			}

			Asura_Dialog* pxDialog;
			pxDialog = new Asura_Function_Dialog("Select Marine To Add To Squad", xOffset, SelectMarineToAdd);
			pxDialog->SetHelpText( "Select Marine To Add To Squad" );
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
            break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_REMOVE_SQUAD_MEMBER:
		{
			s_pxRemoveSquadMemberMessage = static_cast<AVP_Message_RemoveSquadMember*>(pxMessage->m_pData);
			AVP_Message_RemoveSquadMember* pxMsgData = s_pxRemoveSquadMemberMessage;

			if (pxMsgData->m_uVersion > 1)
			{
				Asura_Assert(false, "AVP_MSGID_REMOVE_SQUAD_MEMBER - Version has changed. Fix up the data!!!");
			}

			Asura_Dialog* pxDialog;
			pxDialog = new Asura_Function_Dialog("Select Marine To Remove From Squad", xOffset, SelectMarineToRemove);
			pxDialog->SetHelpText( "Select Marine To Remove To Squad" );
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
            break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_REQUEST_HUMANOID_PATROL:
		{
			s_pxRequestPatrolMessage = static_cast<AVP_Message_RequestPatrol*>(pxMessage->m_pData);
			AVP_Message_RequestPatrol*	pxMsgData = s_pxRequestPatrolMessage;

			if (pxMsgData->m_bFullSquad == 1)
			{
				s_bPatrolFullSquad = true;
			}
			else
			{
				s_bPatrolFullSquad = false;
			}

			if (pxMsgData->m_uVersion > 1)
			{
				Asura_Assert(false, "AVP_MSGID_REQUEST_HUMANOID_PATROL - Version has changed. Fix up the data!!!");
			}

			Asura_Dialog* pxDialog;

			pxDialog = new Asura_Function_Dialog("Select Guard Zone for Marines / Androids to Patrol", xOffset, SelectPatrolGuardZone);
			pxDialog->SetHelpText("Select Guard Zone for Marines / Androids to Patrol");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_Function_Dialog("Clear Selected Guard Zone", xOffset, ClearPatrolGuardZone);
			pxDialog->SetHelpText("Clear the selected Guard Zone");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::LargeSpacing();

			pxDialog = new Asura_Tickbox_Dialog("Follow Squad Leader Patrol", xOffset, &s_bPatrolFullSquad, &s_bPatrolTypeChanged);
			pxDialog->SetHelpText("Selecting this will make the leader use the guard zone the rest of the squad will use normal behaviours but follow the laedr around.");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::LargeSpacing();

			u_int uMaxMembers = AVP_ServerEntity_Squad_Marine::MAX_SQUAD_MEMBERS;

			pxDialog = new Asura_Numerical_Dialog("Number of Squad Members To Go On Patrol", xOffset, &pxMsgData->m_uNumPatrolMembers, 0, uMaxMembers, 1, "%d", NULL);
			pxDialog->SetHelpText("The Number of Squad Members Who Will Go On Patrol");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::LargeSpacing();

			if (pxMsgData->m_bFullSquad == false)
			{
				pxDialog->SetEnabled(true);
			}
			else
			{
				pxDialog->SetEnabled(false);
			}
			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_ALIEN_TEAM_THREAT_BONUS:
		{
			AVP_Message_ChangeThreatBonusValues* pxMsgData = static_cast<AVP_Message_ChangeThreatBonusValues*>(pxMessage->m_pData);

			Asura_Dialog* pxDialog;

			if( !pxMsgData )
			{
				Asura_Assert(false, "Can't access message data");
				return;
			}

			if (pxMsgData->m_iVersion > AVP_Message_ChangeThreatBonusValues::CURRENT_VERSION)
			{
				Asura_Assert(false, "AVP_MSGID_ALIEN_TEAM_THREAT_BONUS - Version has changed. Fix up the data!!!");
			}

			pxDialog = new Asura_StringDisplay_Dialog("Check appropriate box to edit bonus multiplier\nfor each threat type", xOffset);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing() + Asura_Dialog::DefaultSpacing();

			float fXOffset = xOffset.x;
			static const float fMAX_VAL = 10.0f;
			static const float fVAL_STEP = 0.25f;

			static const int iVARNUM = 13;
			static const int iGAP_IDX = 10;
			static const int iPERATTACKER_IDX = 12;

			// Note: the ordering must always align with the flags in AVP_Message_ChangeThreatBonusValues
			float* apfBonusVariables[iVARNUM] = { &pxMsgData->m_fUnknownBonus, &pxMsgData->m_fCivilianBonus, &pxMsgData->m_fMarineBonus, &pxMsgData->m_fMarineShotgunBonus,
				&pxMsgData->m_fMarineFlameBonus, &pxMsgData->m_fMarineSmartgunBonus, &pxMsgData->m_fMarinePlayerBonus, &pxMsgData->m_fAndroidBonus, &pxMsgData->m_fSentryGunBonus,
				&pxMsgData->m_fPredatorBonus, &pxMsgData->m_fPredatorPlayerBonus, &pxMsgData->m_fSameTargetBonus, &pxMsgData->m_fPerAttackerBonus };

			static const char* apszDescriptions[iVARNUM] =
				{ "Unknown threat", "Civilian", "Marine - Pulse Rifle", "Marine - Shotgun", "Marine - Flamethrower", "Marine - Smartgun",
				  "Marine Player", "Android", "SentryGun", "Predator", "Predator Player", "Same target as last turn", "Per additional attacker" };

			s_puTokenFlags = &pxMsgData->m_uFlags;

			for( int i = 0; i < iVARNUM; i++ )
			{
				pxDialog = new Asura_FlagTickbox_Dialog("", xOffset, s_puTokenFlags, 1<<i);
				pxOverlay->AddDialog(pxDialog);
				xOffset.x += Asura_BaseTickbox_Gadget::WIDTH + Asura_Dialog::SmallSpacing();

				pxDialog = new Asura_Numerical_Dialog(apszDescriptions[i], xOffset, apfBonusVariables[i], 0.0f, ( ( i == iPERATTACKER_IDX ) ? 1.0f : fMAX_VAL ), fVAL_STEP, "%.2f", NULL);
				pxDialog->SetIsEnabledFunction(IsFlagChecked, i);
				pxOverlay->AddDialog(pxDialog);
				xOffset.x = fXOffset;
				xOffset.y += ( i == iGAP_IDX ) ? Asura_Dialog::LargeSpacing() : Asura_Dialog::DefaultSpacing();
			}
			//
			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SET_PREFERRED_TARGET_TYPE:
		{
			AVP_Message_SetPreferredTargetType* pxData = static_cast< AVP_Message_SetPreferredTargetType* >( pxMessage->m_pData );

			Asura_Dialog* pxDialogue = new Asura_DropDown_List_Dialog( xOffset,
				AVP_Message_SetPreferredTargetType::s_aszTargetTypeStrings,
				&pxData->m_iPreferredTargetType,
				AVP_Message_SetPreferredTargetType::PTT_END_ENUM,
				AVP_Message_SetPreferredTargetType::PTT_END_ENUM,
				NULL );
			pxOverlay->AddDialog( pxDialogue );
			xOffset.y += Asura_Dialog::DefaultSpacing();

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SPAWN:
		{
			AVP_Msg_Spawn* pxData = static_cast< AVP_Msg_Spawn* >( pxMessage->m_pData );

			Asura_String xOverlayString( "Enemy To Spawn: " );
			if ( pxData->BlueprintSet() )
			{
				Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( pxData->m_uBPTypeHashID, pxData->m_uBPHashID );
				if ( pxBlueprint )
				{
					xOverlayString += pxBlueprint->GetName();
				}
				else
				{
					xOverlayString += "error";
				}
			}
			else
			{
				xOverlayString += "unset";
			}

			Asura_Dialog* pxDialogue = new Asura_Function_Dialog( xOverlayString.GetString(), xOffset, SetSpawnBlueprint, reinterpret_cast< int >( pxData ) );
			pxOverlay->AddDialog( pxDialogue );
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialogue = new Asura_Function_Dialog( "Reset", xOffset, ResetSpawnBlueprint, reinterpret_cast< int >( pxData ) );
			pxOverlay->AddDialog( pxDialogue );
			xOffset.y += Asura_Dialog::DefaultSpacing();

			xOverlayString  = "Squad To Spawn In: ";

			if ( pxData->m_uSquadGuid == ASURA_GUID_UNREGISTERED )
			{
				xOverlayString += "unset";
			}
			else
			{
				ToolsAVP_Entity_Squad* pxSquad = ToolsAVP_Entity_Squad::FindEntity( pxData->m_uSquadGuid  );
				if ( pxSquad )
				{
					xOverlayString += pxSquad->GetName();
				}
				else
				{
					xOverlayString += "ERROR - CANNOT FIND SQUAD";
				}
			}

			pxDialogue = new Asura_Function_Dialog( xOverlayString.GetString(), xOffset, SelectSpawneeAlienSquad, reinterpret_cast< int >( pxData ) );
			pxDialogue->SetHelpText("Select squad for alien.");
			pxOverlay->AddDialog( pxDialogue );
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialogue = new Asura_Function_Dialog("Reset Squad", xOffset, ClearSpawneeAlienSquad, reinterpret_cast< int >( pxData ) );
			pxDialogue->SetHelpText("Clears selected squad.");
			pxOverlay->AddDialog(pxDialogue);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SPAWN_SET_MAX_SPAWN_COUNT:
		{
			AVP_Msg_Spawn_SetMaxSpawnCount* pxData = static_cast< AVP_Msg_Spawn_SetMaxSpawnCount* >( pxMessage->m_pData );

			Asura_Dialog* pxDialogue = new Asura_Numerical_Dialog( "Max To Spawn (-1 for infinite)", xOffset, &pxData->m_iMaxSpawnCount, -1, 999, 1, "%d" );
			pxOverlay->AddDialog( pxDialogue );
			xOffset.y += Asura_Dialog::DefaultSpacing();

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SPAWN_RESET_SPAWN_COUNTER:
		{
			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SPAWN_SET_MAX_IN_PLAY:
		{
			AVP_Msg_Spawn_SetMaxInPlay* pxData = static_cast< AVP_Msg_Spawn_SetMaxInPlay* >( pxMessage->m_pData );

			Asura_Dialog* pxDialogue = new Asura_Numerical_Dialog( "Max In Play", xOffset, &pxData->m_iMaxInPlay, 1, 32, 1, "%d" );
			pxOverlay->AddDialog( pxDialogue );
			xOffset.y += Asura_Dialog::DefaultSpacing();

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SET_ALIEN_SQUAD_BEHAVIOUR:
		{
			AVP_Msg_SetAlienSquadBehaviour* pxData = static_cast< AVP_Msg_SetAlienSquadBehaviour* >( pxMessage->m_pData );
			s_pxSetAlienSquadBehaviourMsg = pxData;

			Asura_Dialog* pxDialogue = new Asura_Dialog( "Behaviour:", xOffset );
			pxOverlay->AddDialog( pxDialogue );
			xOffset.x += 7.0f;

			Asura_DropDown_List_Dialog* pxDDLDialogue = new Asura_DropDown_List_Dialog(
				xOffset,
				AVP_AlienBehaviourData::s_apszBehaviourStrings,
				&pxData->m_iBehaviour,
				AVP_AlienBehaviourData::NUM_BEHAVIOURS,
				AVP_AlienBehaviourData::NUM_BEHAVIOURS,
				NULL,
				NULL,
				"" );
			pxDDLDialogue->SetOnSelectCallback( SetAlienSquadBehaviour, reinterpret_cast< int >( this ) );
			pxOverlay->AddDialog( pxDDLDialogue );
			xOffset.x -= 7.0f;
			xOffset.y += Asura_Dialog::LargeSpacing();

			const bool bDialoguesEnabled = ( pxData->m_iBehaviour == AVP_AlienBehaviourData::CUSTOM );
			const bool bOldBehaviour = pxData->m_iBehaviour != AVP_AlienBehaviourData::AGGRESIVE && pxData->m_iBehaviour != AVP_AlienBehaviourData::HIDE_IN_SHADOWS &&
									   pxData->m_iBehaviour != AVP_AlienBehaviourData::AMBUSH;
			if (bOldBehaviour)
			{
				pxDialogue = new Asura_Dialog( "Move In Mode:", xOffset );
				pxDialogue->SetEnabled( bDialoguesEnabled );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 9.0f;

				pxDDLDialogue = new Asura_DropDown_List_Dialog(
					xOffset,
					AVP_AlienBehaviourData::s_apszMoveInModeStrings,
					&pxData->m_iMoveInMode,
					AVP_AlienBehaviourData::NUM_MOVE_STYLES,
					AVP_AlienBehaviourData::NUM_MOVE_STYLES,
					NULL,
					NULL,
					"" );
				pxDDLDialogue->SetEnabled( bDialoguesEnabled );
				pxOverlay->AddDialog( pxDDLDialogue );
				xOffset.x -= 9.0f;
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialogue = new Asura_Numerical_Dialog( "Ideal Target Distance", xOffset, &pxData->m_fIdealDistanceMin, 0.0f, 999.0f, 1.0f, "%.1f" );
				pxDialogue->SetEnabled( bDialoguesEnabled );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 22.0f;

				pxDialogue = new Asura_Numerical_Dialog( "", xOffset, &pxData->m_fIdealDistanceMax, 0.0f, 999.0f, 1.0f, "%.1f" );
				pxDialogue->SetEnabled( bDialoguesEnabled );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x -= 22.0f;
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialogue = new Asura_Numerical_Dialog( "Attack Delay", xOffset, &pxData->m_fAttackDelay, 0.0f, 99.0f, 0.5f, "%.2f" );
				pxDialogue->SetHelpText( "The time delay between attacks." );
				pxDialogue->SetEnabled( bDialoguesEnabled );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialogue = new Asura_Numerical_Dialog( "Max Attackers Per Target", xOffset, &pxData->m_iMaxAttackersPerTarget, 1, 99, 1, "%d" );
				pxDialogue->SetEnabled( bDialoguesEnabled );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialogue = new Asura_Numerical_Dialog( "Max Attackers Total", xOffset, &pxData->m_iMaxAttackersTotal, 0, 99, 1, "%d" );
				pxDialogue->SetEnabled( bDialoguesEnabled );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialogue = new Asura_Numerical_Dialog( "Max Burst Per TArget", xOffset, &pxData->m_iMaxBurstPerTarget, 0, 99, 1, "%d" );
				pxDialogue->SetEnabled( bDialoguesEnabled );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.y += Asura_Dialog::DefaultSpacing();


				pxDialogue = new Asura_Dialog( "Post Attack Action:", xOffset );
				pxDialogue->SetEnabled( bDialoguesEnabled );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 11.0f;

				pxDDLDialogue = new Asura_DropDown_List_Dialog(
					xOffset,
					AVP_AlienBehaviourData::s_apszPostAttackActionStrings,
					&pxData->m_iPostAttackAction,
					AVP_AlienBehaviourData::NUM_POST_ATTACK_ACTIONS,
					AVP_AlienBehaviourData::NUM_POST_ATTACK_ACTIONS,
					NULL,
					&s_bAlienSquadBehaviourChanged,
					"" );
				pxDDLDialogue->SetEnabled( bDialoguesEnabled );
				pxOverlay->AddDialog( pxDDLDialogue );
				xOffset.x -= 11.0f;
				xOffset.y += Asura_Dialog::DefaultSpacing();

				if ( pxData->m_iPostAttackAction == AVP_AlienBehaviourData::CYCLE_ROUND_DELAYED )
				{
					pxDialogue = new Asura_Numerical_Dialog( "Respawn Delay", xOffset, &pxData->m_fRespawnDelay, 2.0f, 99.0f, 0.5f, "%.2f" );
					pxDialogue->SetHelpText( "The time delay between Aliens unspawning and respawning again." );
					pxDialogue->SetEnabled( bDialoguesEnabled );
					pxOverlay->AddDialog( pxDialogue );
					xOffset.y += Asura_Dialog::DefaultSpacing();
				}

			}
			
			bool bUseCustom = true;

			if (pxData->m_iBehaviour == AVP_AlienBehaviourData::AGGRESIVE || pxData->m_iBehaviour == AVP_AlienBehaviourData::HIDE_IN_SHADOWS || 
				pxData->m_iBehaviour == AVP_AlienBehaviourData::AMBUSH )
			{
				Asura_Dialog* pxDialog;

				Asura_String xPrototypeString("Prototype: ");

				if ( pxData->m_uProtoBlueprintHashID == ASURA_HASH_ID_UNSET )
				{
					xPrototypeString += "UNSET";
				}
				else
				{
					Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_AI_SETTINGS, pxData->m_uProtoBlueprintHashID);
					if ( pxBlueprint )
					{
						xPrototypeString += pxBlueprint->GetName();
					}
					else
					{
						xPrototypeString += "ERROR";
					}
				}

				pxDialog = new Asura_Function_Dialog( xPrototypeString.GetString(), xOffset, SetAlienBehviourPrototype, reinterpret_cast< int >( this ) );
				pxOverlay->AddDialog( pxDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				bUseCustom = pxData->m_uProtoBlueprintHashID == ASURA_HASH_ID_UNSET;

				pxDialogue = new Asura_Dialog( "Jump: ", xOffset);
				pxDialogue->SetEnabled( bUseCustom );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 7.0;
				pxDialogue = new Asura_Numerical_Dialog( "", xOffset, &pxData->m_fJumpFraction, 0.0f, 1.0f, .05f, "%0.2f");
				pxDialogue->SetEnabled( bUseCustom );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x -= 7.0;
				xOffset.y += Asura_Dialog::SmallSpacing();

				if (pxData->m_fStandFraction > 1.0f - pxData->m_fJumpFraction)
				{
					pxData->m_fStandFraction = 1.0f - pxData->m_fJumpFraction;
				}

				pxDialogue = new Asura_Dialog( "Stand: ", xOffset);
				pxDialogue->SetEnabled( bUseCustom );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 7.0;
				pxDialogue = new Asura_Numerical_Dialog( "", xOffset, &pxData->m_fStandFraction, 0.0f, 1.0f - pxData->m_fJumpFraction, .05f, "%0.2f");
				pxDialogue->SetEnabled( bUseCustom );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x -= 7.0;
				xOffset.y += Asura_Dialog::SmallSpacing();

				float fGrapple = 1.0f - pxData->m_fJumpFraction - pxData->m_fStandFraction;
				static float s_fDummy;
				s_fDummy = fGrapple;

				pxDialogue = new Asura_Dialog( "Grapple: ", xOffset);
				pxDialogue->SetEnabled( bUseCustom );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 7.0;
				pxDialogue = new Asura_Numerical_Dialog( "", xOffset, &s_fDummy, fGrapple, fGrapple, .05f, "%0.2f");
				pxDialogue->SetEnabled( bUseCustom );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x -= 7.0;
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialogue = new Asura_Numerical_Dialog( "Time between Grapples: ", xOffset, &pxData->m_fMinTimeBetweenGrapples, 10.0f, 100.0f, 1.0f, "%0.1f");
				pxDialogue->SetEnabled( bUseCustom );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialogue = new Asura_Numerical_Dialog( "Damage from behind multipiler", xOffset, &pxData->m_fDamageFromBehindMultiplier, 0.0f, 1.0f, 0.05f, "%0.1f");
				pxDialogue->SetEnabled( bUseCustom );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.y += Asura_Dialog::DefaultSpacing();


				pxDialogue = new Asura_Numerical_Dialog( "Time Between Attacks", xOffset, &pxData->m_fTimeBetweenAttacks, 0.0f, 5.0f, 0.05f, "%.2f" );
				pxDialogue->SetEnabled( true );
				pxOverlay->AddDialog( pxDialogue );				
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialogue = new Asura_Numerical_Dialog( "Max Number Of Chargers (0 unlimited)", xOffset, &pxData->m_uMaxNumberOfChargers , 0, 25, 1, "%d" );
				pxDialogue->SetEnabled( true );
				pxOverlay->AddDialog( pxDialogue );				
				xOffset.y += Asura_Dialog::DefaultSpacing();
			}
			
			if (pxData->m_iBehaviour == AVP_AlienBehaviourData::HIDE_IN_SHADOWS)
			{
				pxDialogue = new Asura_Tickbox_Dialog("Attack if in marine light", xOffset, &pxData->m_bAttackIfInMarineLight);
				pxDialogue->SetEnabled( true );
				static_cast<Asura_Tickbox_Dialog *>(pxDialogue)->SetClickedFunction(ToolsAVP_Message_Editing::ConditionedSetModified, reinterpret_cast<int>(this));
				pxOverlay->AddDialog( pxDialogue );				
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialogue = new Asura_Numerical_Dialog( "Time in light before attack", xOffset, &pxData->m_fTimeInLightBeforeAttack , 0.5f, 10.0f, 0.1f, "%.2f" );
				pxDialogue->SetEnabled( pxData->m_bAttackIfInMarineLight );
				pxOverlay->AddDialog( pxDialogue );				
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialogue = new Asura_Numerical_Dialog( "Min distance from target to hide at", xOffset, &pxData->m_fMinHideDistance , 5.0f, 10.0f, 0.5f, "%.2f" );
				pxDialogue->SetEnabled( true );
				pxOverlay->AddDialog( pxDialogue );				
				xOffset.y += Asura_Dialog::DefaultSpacing();

				static float s_fMin = 0.0f, s_fMax = 1.0f;

				pxDialogue = new Asura_Dialog( "Too Close :", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 7.0f;
				pxDialogue = new Asura_Dialog( "Attack", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 5.0f;

				pxDialogue = new Asura_NumberSlider_Dialog( xOffset, &pxData->m_fChaneToRehideIfCloseFraction, &s_fMin, &s_fMax, 13.0f, "%0.2f" );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 15.0f;

				pxDialogue = new Asura_Dialog( "Hide", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x -= 27.0f;
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialogue = new Asura_Dialog( "In Light :", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 7.0f;
				pxDialogue = new Asura_Dialog( "Attack", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 5.0f;

				pxDialogue = new Asura_NumberSlider_Dialog( xOffset, &pxData->m_fChaneToRehideIfInLightFraction, &s_fMin, &s_fMax, 13.0f, "%0.2f" );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 15.0f;

				pxDialogue = new Asura_Dialog( "Hide", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x -= 27.0f;
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialogue = new Asura_Dialog( "After Attack :", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 7.0f;
				pxDialogue = new Asura_Dialog( "Attack", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 5.0f;

				pxDialogue = new Asura_NumberSlider_Dialog( xOffset, &pxData->m_fChaneToRehideAfterAttackFraction, &s_fMin, &s_fMax, 13.0f, "%0.2f" );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 15.0f;

				pxDialogue = new Asura_Dialog( "Hide", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x -= 27.0f;
				xOffset.y += Asura_Dialog::DefaultSpacing();

			}

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SET_AI_TARGET_PRIORITIES:
		{
			s_pxSetAITargetPrioritiesData = static_cast< AVP_Msg_SetAITargetPriorities* >( pxMessage->m_pData );

			Asura_DropDown_List_Dialog* pxDDLDialogue;
			Asura_Function_Dialog* pxFnDialogue;

			AVP_Msg_SetAITargetPriorities::TPData* pxTPData = s_pxSetAITargetPrioritiesData->GetFirstTPDataPtr();
			for ( u_int u = 0; u < s_pxSetAITargetPrioritiesData->m_uNumEntries; ++u )
			{
				pxDDLDialogue = new Asura_DropDown_List_Dialog(
					xOffset,
					AVP_AI_TargetPriorityData::s_aszPriorityStrings,
					&pxTPData[ u ].m_iPriority,
					AVP_AI_TargetPriorityData::NUM_PRIORITIES,
					AVP_AI_TargetPriorityData::NUM_PRIORITIES,
					NULL,
					NULL,
					"" );
				pxDDLDialogue->SetOnSelectCallback( SetAITargetPriority, u );
				pxOverlay->AddDialog( pxDDLDialogue );

				// Add the target type dialogues if we should.
				if ( pxTPData[ u ].m_iPriority != AVP_AI_TargetPriorityData::UNSET )
				{
					xOffset.x += 7.5f;

					pxDDLDialogue = new Asura_DropDown_List_Dialog(
						xOffset,
						AVP_AI_TargetPriorityData::s_aszTargetTypeStrings,
						&pxTPData[ u ].m_iTargetType,
						AVP_AI_TargetPriorityData::NUM_TARGET_TYPES,
						AVP_AI_TargetPriorityData::NUM_TARGET_TYPES,
						NULL,
						NULL,
						"" );
					pxDDLDialogue->SetOnSelectCallback( SetAITargetType, u );
					pxOverlay->AddDialog( pxDDLDialogue );

					// Add the guid choose button if we should.
					if ( pxTPData[ u ].m_iTargetType == AVP_AI_TargetPriorityData::SPECIFIC_ENTITY )
					{
						xOffset.x += 10.0f;

						Asura_String xString;
						if ( pxTPData[ u ].m_uSpecificGuid == ASURA_GUID_UNREGISTERED )
						{
							xString = "entity not set";
						}
						else if ( pxTPData[ u ].m_iSpecificGuidType == AVP_AI_TargetPriorityData::ACTOR )
						{
							const AVP_ServerEntity_Actor* pxActor = AVP_ServerEntity_Actor::FindEntity( pxTPData[ u ].m_uSpecificGuid );
							if ( pxActor )
							{
								xString = pxActor->ToolsData->Name.GetString();
							} 
							else
							{
								xString = "can't find actor";
							}
						} 
						else if ( pxTPData[ u ].m_iSpecificGuidType == AVP_AI_TargetPriorityData::SQUAD )
						{
							const AVP_ServerEntity_Squad* pxSquad = AVP_ServerEntity_Squad::FindEntity( pxTPData[ u ].m_uSpecificGuid );
							if ( pxSquad )
							{
								xString = pxSquad->ToolsData->Name.GetString();
							} 
							else
							{
								xString = "can't find squad";
							}
						}

						pxFnDialogue = new Asura_Function_Dialog( xString.GetString(), xOffset, ChooseAITargetEntity, u );
						pxOverlay->AddDialog( pxFnDialogue );
						xOffset.x -= 10.0f;
					}

					xOffset.x -= 7.5f;
				}

				xOffset.y += Asura_Dialog::DefaultSpacing();
			}

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_CHANGE_MARINE_SQUAD_SETTINGS:
		{
			s_pxChangeHumanoidSquadSettingsMessage = static_cast<AVP_Message_ChangeMarineSquadSettings*>(pxMessage->m_pData);
			AVP_Message_ChangeMarineSquadSettings*	pxMsgData = s_pxChangeHumanoidSquadSettingsMessage;

			if (pxMsgData->m_uVersion > 3)
			{
				Asura_Assert(false, "AVP_MSGID_CHANGE_MARINE_SQUAD_SETTINGS - Version has changed. Fix up the data!!!");
			}

			if ( pxMsgData->m_iSquadCombatType == 2 )
			{
				pxMsgData->m_iSquadCombatType = AVP_ServerEntity_Squad_Marine::SQUAD_COMBAT_STATE_NORMAL;
			}

			s_bIsPlayersSquad			= ( pxMsgData->m_bIsPlayersSquad == 1 ) ? true : false;
			s_bIsAILedPlayersSquad		= ( pxMsgData->m_bIsAiLedPlayersSquad == 1 ) ? true : false;
			s_bIsRestrictedToCombatZone	= ( pxMsgData->m_bIsRestrictedToCombatZone == 1 ) ? true : false;
			s_bIgnoreAlertState			= ( pxMsgData->m_bIgnoreAlertState == 1 ) ? true : false;

			Asura_Dialog* pxDialog;

			// Player's squad tick box
			pxDialog = new Asura_Tickbox_Dialog("Is Player's Squad", xOffset, &s_bIsPlayersSquad, &s_bPlayersSquadTypeChanged);
			pxDialog->SetHelpText("Have this selected to indicate that this is the players squad");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			// Player's squad tick box
			pxDialog = new Asura_Tickbox_Dialog("Is AI Led Player's Squad", xOffset, &s_bIsAILedPlayersSquad, &s_bPlayersAILedSquadTypeChanged);
			pxDialog->SetHelpText("Have this selected to indicate that this is a players squad being led by an AI Actor");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			// Player's squad tick box
			pxDialog = new Asura_Tickbox_Dialog("Ignore Alert State", xOffset, &s_bIgnoreAlertState, &s_bIgnoreAlertStateChanged);
			pxDialog->SetHelpText("Have this selected to indicate that this is a players squad being led by an AI Actor");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			// Combat Zone Restriction Tick Box
			pxDialog = new Asura_Tickbox_Dialog("Restrict to Combat Zone", xOffset, &s_bIsRestrictedToCombatZone, &s_bSquadCombatZoneChanged);
			pxDialog->SetHelpText("Select this to restrict the squad to a combat zone.");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			// Select Combat Zone
			pxDialog = new Asura_Function_Dialog("Select Combat Zone", xOffset, SelectRestrictedCombatZoneGPV);
			pxDialog->SetHelpText("Allows Selection of a game play volume to use as a combat zone.");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			if ( pxMsgData->m_uCombatZoneGPVGuid != ASURA_GUID_UNREGISTERED )
			{
				ToolsAVP_Entity_GamePlayVolume* pxGPV = ToolsAVP_Entity_GamePlayVolume::FindEntity( pxMsgData->m_uCombatZoneGPVGuid );
				if ( pxGPV ) pxDialog->UpdateLabel( pxGPV->GetName() );
			}
			if ( !pxMsgData->m_bIsRestrictedToCombatZone ) pxDialog->SetEnabled( false );

			// Clear combat zone
			pxDialog = new Asura_Function_Dialog("Clear Selected Combat Zone Game Play Volume", xOffset, ClearRestrictedCombatZoneGPV);
			pxDialog->SetHelpText("Clears the selected Combat Zone Game Play Volume");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::LargeSpacing();
			if ( !pxMsgData->m_bIsRestrictedToCombatZone || pxMsgData->m_uCombatZoneGPVGuid == ASURA_GUID_UNREGISTERED ) pxDialog->SetEnabled( false );

			pxDialog = new Asura_StringDisplay_Dialog("Select The Squads Combat Type:", xOffset);
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_DropDown_List_Dialog( xOffset, AVP_ServerEntity_Squad_Marine::GetSquadCombatStateNames(), &pxMsgData->m_iSquadCombatType, AVP_ServerEntity_Squad_Marine::MAX_SQUAD_COMBAT_STATES, AVP_ServerEntity_Squad_Marine::MAX_SQUAD_COMBAT_STATES, NULL, &s_bSquadCombatStateChanged);
			pxDialog->SetHelpText("The Squads Current Combat State");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::LargeSpacing();

		}break;

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_ATTACK_SCRIPTED_TARGET:
		{
			s_pxAttackScriptedTargetMessage = static_cast<AVP_Message_AttackScriptedTarget*>(pxMessage->m_pData);

			Asura_Dialog* pxDialog;

			pxDialog = new Asura_Function_Dialog("Select Shoot At Node", xOffset, SelectShootAt);
			pxDialog->SetHelpText("Allows selection of the shoot At point to be used.");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();
			if ( s_pxAttackScriptedTargetMessage->m_uShootAtGuid != ASURA_GUID_UNREGISTERED )
			{
				ToolsAVP_Entity_ShootAt* pxShootAt = ToolsAVP_Entity_ShootAt::FindEntity(s_pxAttackScriptedTargetMessage->m_uShootAtGuid);
				if (pxShootAt)
				{
					pxDialog->UpdateLabel(pxShootAt->GetName());
				}
			}

			pxDialog = new Asura_Function_Dialog("Clear Shoot At Node", xOffset, ClearShootAt);
			pxDialog->SetHelpText("Clears the selection of the shoot At point being used.");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

		}break;

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_PICKUP_SET_INTELLIGENT_AMOUNT:
		{
			AVP_Message_Pickup_SetIntelligentAmount* pxData = static_cast< AVP_Message_Pickup_SetIntelligentAmount* >( pxMessage->m_pData );

			// Create numerical dialog for Intelligent Quantity
			Asura_Dialog* pxDialog = new Asura_DropDown_List_Dialog(xOffset, s_ppszIntelligentQuantities, &pxData->m_iIntelligentAmount, 3, 3, NULL, &s_bIntelligentAmountChanged, "Set the intelligent amount");	
			pxDialog->SetHelpText("Set intelligent amount of Health and Ammo generated by this pickup  1 Scarce, 2 Medium, 3 Plenty");
			pxOverlay->AddDialog( pxDialog );
			xOffset.y += Asura_Dialog::DefaultSpacing();

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_ADD_PLAYER_PICKUP:
		{
			AVP_Message_AddPlayerPickup* pxData = static_cast<AVP_Message_AddPlayerPickup*>(pxMessage->m_pData);

			s_pxSetPlayerPickupMessage = pxData;

			Asura_String xPickupString( "Pickup: " );

			if (pxData->m_uPickupGuid == ASURA_GUID_UNREGISTERED)
			{
				xPickupString += "None";
			}
			else
			{
				ToolsAVP_Entity_Pickup* pxToolsPickup = (ToolsAVP_Entity_Pickup*)ToolsAVP_Entity_Pickup::FindEntity(pxData->m_uPickupGuid);

				if (pxToolsPickup)
				{
					if (pxToolsPickup->GetName())
					{
						xPickupString += pxToolsPickup->GetName();
					}
					else
					{
						Asura_String xGuidString;
						Asura_StringUtil::ToString( pxToolsPickup->GetGuid(), xGuidString );
						xPickupString += xGuidString;
					}
				}
				else
				{
					xPickupString += "Error";
				}
			}

			Asura_Dialog* pxDialog;
			pxDialog = new Asura_Function_Dialog(xPickupString.GetString(), xOffset, SelectPlayerPickup, &s_bSetPlayerPickupChanged );
			pxDialog->SetHelpText( "The pickup" );
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_Tickbox_Dialog("Set as current?", xOffset, &pxData->m_bSetAsCurrent, &s_bSetPlayerPickupChanged);
			pxDialog->SetHelpText("Select this pickup to be the current one");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_ADD_PLAYER_OBJECTIVE:
		{
			AVP_Message_AddPlayerObjective* pxData = static_cast<AVP_Message_AddPlayerObjective*>(pxMessage->m_pData);

			s_pxAddPlayerObjectiveMessage = pxData;

			Asura_String xObjectiveString( "Objective: " );

			if (pxData->m_uObjectiveGuid == ASURA_GUID_UNREGISTERED)
			{
				xObjectiveString += "None";
			}
			else
			{
				ToolsAVP_Entity_PlayerObjective* pxToolsObjective = (ToolsAVP_Entity_PlayerObjective*)ToolsAVP_Entity_PlayerObjective::FindEntity(pxData->m_uObjectiveGuid);

				if (pxToolsObjective)
				{
					if (pxToolsObjective->GetName())
					{
						xObjectiveString += pxToolsObjective->GetName();
					}
					else
					{
						Asura_String xGuidString;
						Asura_StringUtil::ToString( pxToolsObjective->GetGuid(), xGuidString );
						xObjectiveString += xGuidString;
					}
				}
				else
				{
					xObjectiveString += "Error";
				}
			}

			Asura_Dialog* pxDialog;
			pxDialog = new Asura_Function_Dialog(xObjectiveString.GetString(), xOffset, SelectPlayerObjective, &s_bAddObjectiveChanged);
			pxDialog->SetHelpText( "The Objective Point" );
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SET_PLAYER_OBJECTIVE_COMPLETE:
		{
			AVP_Message_PlayerObjectiveComplete* pxData = static_cast<AVP_Message_PlayerObjectiveComplete*>(pxMessage->m_pData);

			s_pxPlayerObjectiveCompleteMessage = pxData;

			Asura_String xObjectiveString( "Completed Objective: " );

			if (pxData->m_uObjectiveGuid == ASURA_GUID_UNREGISTERED)
			{
				xObjectiveString += "None";
			}
			else
			{
				ToolsAVP_Entity_PlayerObjective* pxToolsObjective = (ToolsAVP_Entity_PlayerObjective*)ToolsAVP_Entity_PlayerObjective::FindEntity(pxData->m_uObjectiveGuid);

				if (pxToolsObjective)
				{
					if (pxToolsObjective->GetName())
					{
						xObjectiveString += pxToolsObjective->GetName();
					}
					else
					{
						Asura_String xGuidString;
						Asura_StringUtil::ToString( pxToolsObjective->GetGuid(), xGuidString );
						xObjectiveString += xGuidString;
					}
				}
				else
				{
					xObjectiveString += "Error";
				}
			}

			Asura_Dialog* pxDialog;
			pxDialog = new Asura_Function_Dialog(xObjectiveString.GetString(), xOffset, SelectCompletedPlayerObjective, &s_bPlayerObjectiveCompleteChanged);
			pxDialog->SetHelpText( "The completed Objective Point" );
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_PLAYER_OBJECTIVE_SET_MARKER_LOCATION:
		{
			AVP_Message_PlayerObjectiveSetMarker* pxData = static_cast<AVP_Message_PlayerObjectiveSetMarker*>(pxMessage->m_pData);

			s_pxSetMarkerLocationMessage = pxData;

			Asura_String xMarkerString( "Marker: " );

			if (pxData->m_uMarkerGuid == ASURA_GUID_UNREGISTERED)
			{
				xMarkerString += "None";
			}
			else
			{
				ToolsAVP_Entity_LocationMarker* pxToolsMarker = ToolsAVP_Entity_LocationMarker::FindEntity(pxData->m_uMarkerGuid);

				if (pxToolsMarker)
				{
					if (pxToolsMarker->GetName())
					{
						xMarkerString += pxToolsMarker->GetName();
					}
					else
					{
						Asura_String xGuidString;
						Asura_StringUtil::ToString( pxToolsMarker->GetGuid(), xGuidString );
						xMarkerString += xGuidString;
					}
				}
				else
				{
					xMarkerString += "Error";
				}
			}

			Asura_Dialog* pxDialog;
			pxDialog = new Asura_Function_Dialog(xMarkerString.GetString(), xOffset, SelectMarkerLocation, &s_bSetMarkerLocationChanged);
			pxDialog->SetHelpText( "The marker location" );
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_PLAYER_OBJECTIVE_SET_TARGET_ENTITY:
		{
			AVP_Message_PlayerObjectiveSetEntity* pxData = static_cast<AVP_Message_PlayerObjectiveSetEntity*>(pxMessage->m_pData);

			s_pxSetTargetEntityMessage = pxData;

			Asura_String xTargetString( "Target entity: " );

			if (pxData->m_uEntityGuid == ASURA_GUID_UNREGISTERED)
			{
				xTargetString += "None";
			}
			else
			{
				ToolsAVP_Entity_PhysicalObject* pxToolsEntity = ToolsAVP_Entity_PhysicalObject::FindEntity(pxData->m_uEntityGuid);

				if (pxToolsEntity)
				{
					if (pxToolsEntity->GetName())
					{
						xTargetString += pxToolsEntity->GetName();
					}
					else
					{
						Asura_String xGuidString;
						Asura_StringUtil::ToString( pxToolsEntity->GetGuid(), xGuidString );
						xTargetString += xGuidString;
					}
				}
				else
				{
					xTargetString += "Error";
				}
			}

			Asura_Dialog* pxDialog;
			pxDialog = new Asura_Function_Dialog(xTargetString.GetString(), xOffset, SelectTargetEntity, &s_bSetTargetEntityChanged);
			pxDialog->SetHelpText( "The target entity" );
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::DefaultSpacing();

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SET_DROPSHIP_ATTACK_MODE:
		{
			AVP_Message_DropshipAttackMode* pxData = static_cast<AVP_Message_DropshipAttackMode*>(pxMessage->m_pData);

			s_pxDropshipAttackModeMessage = pxData;

			Asura_Dialog* pxDialog;
			pxDialog = new Asura_DropDown_List_Dialog( xOffset, 
													   AVP_Message_DropshipAttackMode::GetModeNames(), 
													   reinterpret_cast<int *>(&pxData->m_eMode), 
													   AVP_Message_DropshipAttackMode::DSAM_MODES_COUNT,
													   AVP_Message_DropshipAttackMode::DSAM_MODES_COUNT, 
													   NULL,
													   &s_bDropshipModeChanged);
			pxDialog->SetHelpText("The Dropship Attack Mode");
			pxOverlay->AddDialog(pxDialog);
			xOffset.y += Asura_Dialog::LargeSpacing();

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_CHANGE_HUMANOID_SQUAD_MOVEMENT_MODE:
		{
			AVP_Message_ChangeHumanoidMovementMode* pxData = static_cast< AVP_Message_ChangeHumanoidMovementMode* >( pxMessage->m_pData );

			if ( pxData )
			{
				s_pxChangeHumanoidMoveModeMessage = pxData;

				Asura_Dialog* pxDialog;
				pxDialog = new Asura_DropDown_List_Dialog( xOffset, AVP_ServerNode_Objective_Marine::GetObjectiveMoveModeNames(), &s_pxChangeHumanoidMoveModeMessage->m_iMovementMode, AVP_ServerNode_Objective_Marine::MAX_OBJECTIVE_MOVEMENT_MODES_TYPES, AVP_ServerNode_Objective_Marine::MAX_OBJECTIVE_MOVEMENT_MODES_TYPES, NULL, &s_bObjectiveMoveModeChanged );
				pxDialog->SetHelpText("The Type Of Movement Mode");
				pxOverlay->AddDialog(pxDialog);
				xOffset.y += Asura_Dialog::LargeSpacing();
			}
		}break;

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SET_PREDATOR_BEHAVIOUR:
		{
			AVP_Message_SetPredatorBehaviour* pxData = static_cast< AVP_Message_SetPredatorBehaviour* >( pxMessage->m_pData );
			s_pxSetPredatorBehaviourMessage = pxData;

			Asura_Dialog* pxDialogue = new Asura_Dialog( "Behaviour:", xOffset );
			pxOverlay->AddDialog( pxDialogue );
			xOffset.x += 7.0f;

			Asura_DropDown_List_Dialog* pxDDLDialogue = new Asura_DropDown_List_Dialog(
				xOffset,
				AVP_PredatorBehaviourData::s_apszBehaviourStrings,
				&pxData->m_iBehaviour,
				AVP_PredatorBehaviourData::NUM_BEHAVIOURS,
				AVP_PredatorBehaviourData::NUM_BEHAVIOURS,
				NULL,
				NULL,
				"" );
			pxDDLDialogue->SetOnSelectCallback( SePredatorBehaviour, reinterpret_cast< int >( this ) );
			pxOverlay->AddDialog( pxDDLDialogue );
			xOffset.x -= 7.0f;
			xOffset.y += Asura_Dialog::LargeSpacing();
			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_LOOK_AT_ENTITY:
		{
			s_pxLookAtEntityMessage = static_cast<AVP_Message_LookAtEntity*>(pxMessage->m_pData);
			s_uGuidToIgnoreForSelectionInMessage = pxMessage->m_uSendTo;

			Asura_Dialog* pxDialog;

			if ( s_pxLookAtEntityMessage->m_bLookAtPlayer == 0 )
			{
				s_bLookAtPlayer = false;
			}
			else
			{
				s_bLookAtPlayer = true;
			}

			pxDialog = new Asura_Tickbox_Dialog( "Look At Player", xOffset, &s_bLookAtPlayer, &s_bLookAtChanged );
			pxDialog->SetHelpText( "Look At The Player Rather Than the Selected Entity" );
			pxOverlay->AddDialog( pxDialog );
			xOffset.y += Asura_Dialog::LargeSpacing();

			pxDialog = new Asura_Function_Dialog("Select Look At Entity", xOffset, SelectLookAt);
			pxDialog->SetHelpText("Allows selection of the Look At Entity to be used.");
			pxOverlay->AddDialog(pxDialog);
			if ( s_bLookAtPlayer )
			{
				pxDialog->SetEnabled( false );
			}
			else
			{
				pxDialog->SetEnabled( true );
			}
			xOffset.y += Asura_Dialog::DefaultSpacing();

			if ( s_pxLookAtEntityMessage->m_uLookAtGuid != ASURA_GUID_UNREGISTERED )
			{
				ToolsAVP_Entity_PhysicalObject* pxPhysObj = ToolsAVP_Entity_PhysicalObject::FindEntity( s_pxLookAtEntityMessage->m_uLookAtGuid );
				
				if ( pxPhysObj )
				{
					pxDialog->UpdateLabel( pxPhysObj->GetName() );
				}
				else
				{
					ToolsAVP_Entity_LookAt* pxLookAt = ToolsAVP_Entity_LookAt::FindEntity( s_pxLookAtEntityMessage->m_uLookAtGuid );

					if ( pxLookAt )
					{
						pxDialog->UpdateLabel( pxLookAt->GetName() );
					}
				}
			}

			pxDialog = new Asura_String_Dialog("Select Look At Marker Point", xOffset, &s_xLookAtMarkerString, 16, &s_bLookAtMarkerStringChanged );
			pxDialog->SetHelpText("Allows selection of a marker point on the Look At Entity to be used.");
			pxOverlay->AddDialog(pxDialog);
			if ( s_bLookAtPlayer )
			{
				pxDialog->SetEnabled( false );
			}
			else
			{
				pxDialog->SetEnabled( true );
			}
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_Function_Dialog("Clear Look At Entity", xOffset, ClearLookAt);
			pxDialog->SetHelpText("Clears the selection of the Look At Entity being used.");
			pxOverlay->AddDialog(pxDialog);
			if ( s_bLookAtPlayer )
			{
				pxDialog->SetEnabled( false );
			}
			else
			{
				pxDialog->SetEnabled( true );
			}
			xOffset.y += Asura_Dialog::DefaultSpacing();

		}break;

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SET_GS2_MOVE_SPEED:
		{
			s_pxSetGS2MoveSpeed = static_cast< AVP_Message_SetGS2MoveSpeed* >( pxMessage->m_pData );

			Asura_Dialog* pxDialog;

			pxDialog = new Asura_StringDisplay_Dialog( "Select the speed the actor should move while using the GS2 movement behaviour", xOffset );
			pxOverlay->AddDialog( pxDialog );
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_DropDown_List_Dialog( xOffset, AVP_Navigation::s_aszMoveModeStrings, &s_pxSetGS2MoveSpeed->m_uMoveSpeed, AVP_Navigation::NUM_MOVEMENT_MODES, AVP_Navigation::NUM_MOVEMENT_MODES, NULL, &s_bGS2MoveModeChanged, "" );
			pxDialog->SetHelpText( "The speed to use while moving on GS2 using FREE MOVEMENT" );
			pxOverlay->AddDialog( pxDialog );
			xOffset.y += Asura_Dialog::LargeSpacing();

		}break;

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SET_HEALTH:
		{
			s_pxSetHealthMessage = static_cast< AVP_Message_SetHealth* >( pxMessage->m_pData );

			Asura_Dialog* pxDialog;

			pxDialog = new Asura_StringDisplay_Dialog( "Select the method to set entity health by:", xOffset );
			pxOverlay->AddDialog( pxDialog );
			xOffset.y += Asura_Dialog::DefaultSpacing();

			static const char* s_aszDamageTypeSelectNames[ 2 ] = { "Set Health Directly", "Set Health as a Percentage of Max Health" };
			pxDialog = new Asura_DropDown_List_Dialog( xOffset, s_aszDamageTypeSelectNames, &s_pxSetHealthMessage->m_uSetHealthAsPercentageOfMaxHealth, 2, 2, NULL, &s_bHealthSettingTypeChanged, "" );
			pxDialog->SetHelpText( "The method to set health" );
			pxOverlay->AddDialog( pxDialog );
			xOffset.y += Asura_Dialog::LargeSpacing();

			if ( s_pxSetHealthMessage->m_uSetHealthAsPercentageOfMaxHealth == 0 )
			{
				// Set health Directly
				pxDialog = new Asura_Numerical_Dialog( "Direct Method", xOffset, &s_pxSetHealthMessage->m_fDirectHealthValue, 0.0f, 1000.0f, 1.0f, "%.1f", &s_bHealthSettingTypeChanged );
				pxDialog->SetHelpText( "" );
				pxOverlay->AddDialog( pxDialog );
				xOffset.y += Asura_Dialog::LargeSpacing();
			}
			else
			{
				// Set health as a percentage of max health
				pxDialog = new Asura_Numerical_Dialog( "Percentage Method", xOffset, &s_pxSetHealthMessage->m_fPercentageHealthValue, 0.0f, 1000.0f, 1.0f, "%.1f", &s_bHealthSettingTypeChanged );
				pxDialog->SetHelpText( "" );
				pxOverlay->AddDialog( pxDialog );
				xOffset.y += Asura_Dialog::LargeSpacing();
			}

			if ( s_pxSetHealthMessage->m_bTreatAsDamage == 0 )
			{
				s_bTreatAsDamage = false;
			}
			else
			{
				s_bTreatAsDamage = true;
			}

			pxDialog = new Asura_Tickbox_Dialog( "Treat set health as damage", xOffset, &s_bTreatAsDamage, &s_bHealthSettingTypeChanged );
			pxDialog->SetHelpText( "If selected it will treat the set health as damage, causing the character to do anything it would normally do when taking damage." );
			pxOverlay->AddDialog( pxDialog );
			xOffset.y += Asura_Dialog::LargeSpacing();

			if ( s_pxSetHealthMessage->m_bDamageComesFromOtherEntity == 0 )
			{
				s_bTakeAsDamageFromEntity = false;
			}
			else
			{
				s_bTakeAsDamageFromEntity = true;
			}

			u_int uNumChildren = Asura_Blueprint_System::GetNumChildBlueprints( 0xb02e8b6f /*"Damage"*/, 0x9643cb29 /*"DamageType"*/, true );

			if  ( s_pxSetHealthMessage->m_uDamageType == ASURA_HASH_ID_UNSET )
			{
				s_iDamageTypeValue = 0;
			}
			else
			{
				for ( u_int u = 0; u < uNumChildren; ++u )
				{
					Asura_Blueprint* pxBP = Asura_Blueprint_System::GetChildBlueprint( 0xb02e8b6f /*"Damage"*/, 0x9643cb29 /*"DamageType"*/, true, u );

					if ( pxBP )
					{ 
						if ( pxBP->GetHashID() == s_pxSetHealthMessage->m_uDamageType )
						{
							s_iDamageTypeValue = u + 1;
						}
					}
				}
			}

			Asura_StringValuePair_DropDownList_Dialog* pxDDDDialog = NULL;
			pxDDDDialog = new Asura_StringValuePair_DropDownList_Dialog( xOffset, "", &s_iDamageTypeValue, &s_bHealthSettingTypeChanged );
			pxDDDDialog->AddItem( "NO TYPE", 0 );

			for ( u_int u = 0; u < uNumChildren; ++u )
			{
				Asura_Blueprint* pxBP = Asura_Blueprint_System::GetChildBlueprint( 0xb02e8b6f /*"Damage"*/, 0x9643cb29 /*"DamageType"*/, true, u );
				if ( pxBP ) { pxDDDDialog->AddItem( pxBP->GetName().GetString(), u + 1 ); }
			}

			pxDDDDialog->InitialiseData();
			pxDDDDialog->CreateGadgets( xOffset );
			pxOverlay->AddDialog( pxDDDDialog );
			if ( !s_bTreatAsDamage )
			{
				pxDDDDialog->SetEnabled( false );
			}
			else
			{
				pxDDDDialog->SetEnabled( true );
			}
			xOffset.y += Asura_Dialog::DefaultSpacing();

			pxDialog = new Asura_Tickbox_Dialog( "Take as damage from entity", xOffset, &s_bTakeAsDamageFromEntity, &s_bHealthSettingTypeChanged );
			pxDialog->SetHelpText( "If selected it will treat the set health as damage from a specific entity." );
			pxOverlay->AddDialog( pxDialog );
			if ( !s_bTreatAsDamage )
			{
				pxDialog->SetEnabled( false );
			}
			else
			{
				pxDialog->SetEnabled( true );
			}
			xOffset.y += Asura_Dialog::LargeSpacing();

			pxDialog = new Asura_Function_Dialog("Select damaging entity", xOffset, SelectDamager);
			pxDialog->SetHelpText("Allows selection of the Look At Entity to be used.");
			pxOverlay->AddDialog(pxDialog);
			if ( !s_bTakeAsDamageFromEntity || !s_bTreatAsDamage )
			{
				pxDialog->SetEnabled( false );
			}
			else
			{
				pxDialog->SetEnabled( true );
			}
			xOffset.y += Asura_Dialog::DefaultSpacing();
			
			pxDialog = new Asura_StringDisplay_Dialog( "** NO DAMAGING ENTITY SELECTED **", xOffset );
			if ( s_pxSetHealthMessage->m_uDamageFromGuid != ASURA_GUID_UNREGISTERED )
			{
				ToolsAVP_Entity_PhysicalObject* pxActor = ToolsAVP_Entity_PhysicalObject::FindEntity( s_pxSetHealthMessage->m_uDamageFromGuid );

				if ( pxActor )
				{
					pxDialog->UpdateLabel( pxActor->GetName() );
				}
			}
			pxOverlay->AddDialog( pxDialog );
			xOffset.y += Asura_Dialog::DefaultSpacing();
			if ( !s_bTakeAsDamageFromEntity || !s_bTreatAsDamage )
			{
				pxDialog->SetEnabled( false );
			}
			else
			{
				pxDialog->SetEnabled( true );
			}

			pxDialog = new Asura_Function_Dialog("Clear damaging entity", xOffset, ClearDamager);
			pxDialog->SetHelpText("Clears the selection of the Look At Entity being used.");
			pxOverlay->AddDialog(pxDialog);
			if ( !s_bTakeAsDamageFromEntity || !s_bTreatAsDamage )
			{
				pxDialog->SetEnabled( false );
			}
			else
			{
				pxDialog->SetEnabled( true );
			}
			xOffset.y += Asura_Dialog::LargeSpacing();

			

		}break;

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SET_FACEHUGGER_SQUAD_BEHAVIOUR:
		{

			AVP_Msg_SetFaceHuggerSquadBehaviour* pxData = static_cast< AVP_Msg_SetFaceHuggerSquadBehaviour* >( pxMessage->m_pData );
			s_pxSetFaceHuggerSquadBehaviourMsg = pxData;

			Asura_Dialog* pxDialogue = new Asura_Dialog( "Behaviour:", xOffset );
			pxOverlay->AddDialog( pxDialogue );
			xOffset.x += 7.0f;

			Asura_DropDown_List_Dialog* pxDDLDialogue = new Asura_DropDown_List_Dialog(
				xOffset,
				AVP_FaceHuggerBehaviourData::s_apszBehaviourStrings,
				&pxData->m_iBehaviour,
				AVP_FaceHuggerBehaviourData::NUM_BEHAVIOURS,
				AVP_FaceHuggerBehaviourData::NUM_BEHAVIOURS,
				NULL,
				NULL,
				"" );
			pxDDLDialogue->SetOnSelectCallback( SetFacehuggerSquadBehaviour, reinterpret_cast< int >( this ) );
			pxOverlay->AddDialog( pxDDLDialogue );
			xOffset.x -= 7.0f;
			xOffset.y += Asura_Dialog::LargeSpacing();

			if (pxData->m_iBehaviour == AVP_FaceHuggerBehaviourData::ATTACK || 
				pxData->m_iBehaviour == AVP_FaceHuggerBehaviourData::HIDE_IN_SHADOWS )
			{
				pxDialogue = new Asura_Numerical_Dialog( "Avoidance Distance", xOffset, &pxData->m_fAvoidanceDistance , 5.0f, 20.0f, .5f, "%f" );
				pxDialogue->SetEnabled( true );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialogue = new Asura_Numerical_Dialog( "Max Targeters", xOffset, &pxData->m_uMaxTargeters, 1, 10, 1, "%d" );
				pxDialogue->SetEnabled( true );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialogue = new Asura_Numerical_Dialog( "Time Between Attacks", xOffset, &pxData->m_fTimeBetweenAttacks, 1.0f, 60.0f, .5f, "%f" );
				pxDialogue->SetEnabled( true );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialogue = new Asura_Numerical_Dialog( "Wander Distance", xOffset, &pxData->m_fWanderDistance, 4.0f, 10.0f, .5f, "%f" );
				pxDialogue->SetEnabled( true );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.y += Asura_Dialog::DefaultSpacing();
			}		

			if (pxData->m_iBehaviour == AVP_FaceHuggerBehaviourData::HIDE_IN_SHADOWS )
			{
				pxDialogue = new Asura_Tickbox_Dialog("Attack if in marine light", xOffset, &pxData->m_bAttackIfInMarineLight);
				pxDialogue->SetEnabled( true );
				static_cast<Asura_Tickbox_Dialog *>(pxDialogue)->SetClickedFunction(ToolsAVP_Message_Editing::ConditionedSetModified, reinterpret_cast<int>(this));
				pxOverlay->AddDialog( pxDialogue );				
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialogue = new Asura_Numerical_Dialog( "Time in light before attack", xOffset, &pxData->m_fTimeInLightBeforeAttack , 0.5f, 10.0f, 0.1f, "%.2f" );
				pxDialogue->SetEnabled( pxData->m_bAttackIfInMarineLight );
				pxOverlay->AddDialog( pxDialogue );				
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialogue = new Asura_Numerical_Dialog( "Min distance from target to hide at", xOffset, &pxData->m_fMinHideDistance , 5.0f, 10.0f, 0.5f, "%.2f" );
				pxDialogue->SetEnabled( true );
				pxOverlay->AddDialog( pxDialogue );				
				xOffset.y += Asura_Dialog::DefaultSpacing();

				static float s_fMin = 0.0f, s_fMax = 1.0f;

				pxDialogue = new Asura_Dialog( "Too Close :", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 7.0f;
				pxDialogue = new Asura_Dialog( "Attack", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 5.0f;

				pxDialogue = new Asura_NumberSlider_Dialog( xOffset, &pxData->m_fChaneToRehideIfCloseFraction, &s_fMin, &s_fMax, 13.0f, "%0.2f" );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 15.0f;

				pxDialogue = new Asura_Dialog( "Hide", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x -= 27.0f;
				xOffset.y += Asura_Dialog::LargeSpacing();

				pxDialogue = new Asura_Dialog( "In Light :", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 7.0f;
				pxDialogue = new Asura_Dialog( "Attack", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 5.0f;

				pxDialogue = new Asura_NumberSlider_Dialog( xOffset, &pxData->m_fChaneToRehideIfInLightFraction, &s_fMin, &s_fMax, 13.0f, "%0.2f" );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 15.0f;

				pxDialogue = new Asura_Dialog( "Hide", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x -= 27.0f;
				xOffset.y += Asura_Dialog::LargeSpacing();

				pxDialogue = new Asura_Dialog( "After Attack :", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 7.0f;
				pxDialogue = new Asura_Dialog( "Attack", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 5.0f;

				pxDialogue = new Asura_NumberSlider_Dialog( xOffset, &pxData->m_fChaneToRehideAfterAttackFraction, &s_fMin, &s_fMax, 13.0f, "%0.2f" );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x += 15.0f;

				pxDialogue = new Asura_Dialog( "Hide", xOffset );
				pxOverlay->AddDialog( pxDialogue );
				xOffset.x -= 27.0f;
				xOffset.y += Asura_Dialog::LargeSpacing();

			}
			break;
		}

		//-------------------------------------------------------------------------------------------
		
		case AVP_MSGID_CHANGE_HUMANOID_KNOWLEDGE_SETTINGS:
		{
			AVP_Message_ChangeHumanoidKnowledgeSettings* pxData = static_cast< AVP_Message_ChangeHumanoidKnowledgeSettings* >( pxMessage->m_pData );
			s_pxSetHumanoidKnowledgeSettingsMsg = pxData;

			s_iHumanoidTypeValue = s_pxSetHumanoidKnowledgeSettingsMsg->m_uHumanoidType;

			Asura_StringValuePair_DropDownList_Dialog* pxDDDDialog = NULL;
			pxDDDDialog = new Asura_StringValuePair_DropDownList_Dialog( xOffset, "", &s_iHumanoidTypeValue, &s_bHumanoidKnowledgeSettingTypeChanged );
			pxDDDDialog->AddItem( "ANDROID",	AVP_ENTITYCLASS_ANDROID );
			pxDDDDialog->AddItem( "CIVILIAN",	AVP_ENTITYCLASS_CIVILIAN );
			pxDDDDialog->AddItem( "MARINE",		AVP_ENTITYCLASS_MARINE );
			pxDDDDialog->InitialiseData();
			pxDDDDialog->CreateGadgets( xOffset );
			pxOverlay->AddDialog( pxDDDDialog );

			xOffset.y += Asura_Dialog::DefaultSpacing();

			Asura_Dialog* pxDialog;
			pxDialog = new Asura_Function_Dialog( "Knowledge Blueprint: ", xOffset, ChooseKnowledgeSettingsBlueprint, &s_bHumanoidKnowledgeSettingTypeChanged );
			pxOverlay->AddDialog( pxDialog );
			xOffset.y += Asura_Dialog::DefaultSpacing();
			pxDialog = new Asura_StringDisplay_Dialog( "", xOffset );
			pxOverlay->AddDialog( pxDialog );

			Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( 0x7ad7bd1b /*"AI Settings"*/, s_pxSetHumanoidKnowledgeSettingsMsg->m_uBluepprintHashID );

			if ( pxBlueprint )
			{
				pxDialog->UpdateLabel( pxBlueprint->GetName().GetString() );
			}
			else
			{
				pxDialog->UpdateLabel( "ERROR" );
			}

			xOffset.y += Asura_Dialog::DefaultSpacing();

		}break;

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SET_SENTRYGUN_SWEEP_SPEED:
		{
			AVP_Message_SentryGun_Set_SweepSpeed* pxData = static_cast< AVP_Message_SentryGun_Set_SweepSpeed* >( pxMessage->m_pData );

			if( pxData )
			{
				s_pxSetSentryGunSweepSpeed = pxData;

				Asura_Dialog* pxDialog;
				pxDialog = new Asura_Numerical_Dialog( "Sweep Speed", xOffset, &pxData->m_fSpeed, 0.05f, 1.0f, 0.05f, "%0.2f" );
				pxDialog->SetHelpText( "Sets the speed at which the weapon sweeps from side to side (0.05 to 1.0)" );
				pxOverlay->AddDialog(pxDialog);
				xOffset.y += Asura_Dialog::DefaultSpacing();
			}
		}break;

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SET_SENTRYGUN_VISION_RANGE:
		{
			AVP_Message_SentryGun_Set_VisionRange* pxData = static_cast< AVP_Message_SentryGun_Set_VisionRange* >( pxMessage->m_pData );

			if( pxData )
			{
				s_pxSetSentryGunVisionRange = pxData;

				Asura_Dialog* pxDialog;
				pxDialog = new Asura_Numerical_Dialog( "Vision Range", xOffset, &pxData->m_fRange, 0.0f, 1000.0f, 1.0f, "%0.0f" );
				pxDialog->SetHelpText( "Sets the range for the field of view ( in meters )" );
				pxOverlay->AddDialog( pxDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();
			}
		}break;

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SET_SENTRYGUN_HOR_FOV:
		{
			AVP_Message_SentryGun_Set_HorizontalFOV* pxData = static_cast< AVP_Message_SentryGun_Set_HorizontalFOV* >( pxMessage->m_pData );

			if( pxData )
			{
				s_pxSetSentryGunHorizontalFOV = pxData;

				Asura_Dialog* pxDialog;
				pxDialog = new Asura_Numerical_Dialog( "Horizontal Field of View", xOffset, &pxData->m_fHorFov, 0.0f, 178.0f, 1.0f, "%0.0f" );
				pxDialog->SetHelpText( "Sets the horizontal field of view ( in degrees )" );
				pxOverlay->AddDialog( pxDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();
			}
		}break;

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SET_SENTRYGUN_VER_FOV:
		{
			AVP_Message_SentryGun_Set_VerticalFOV* pxData = static_cast< AVP_Message_SentryGun_Set_VerticalFOV* >( pxMessage->m_pData );

			if( pxData )
			{
				s_pxSetSentryGunVerticalFOV = pxData;

				Asura_Dialog* pxDialog;
				pxDialog = new Asura_Numerical_Dialog( "Vertical Field of View", xOffset, &pxData->m_fVerFov, 0.0f, 90.0f, 1.0f, "%0.0f" );
				pxDialog->SetHelpText( "Sets the vertical field of view ( in degrees )" );
				pxOverlay->AddDialog( pxDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();
			}
		}break;

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SET_SENTRYGUN_FIRING_ARC:
		{
			AVP_Message_SentryGun_Set_FiringArc* pxData = static_cast< AVP_Message_SentryGun_Set_FiringArc* >( pxMessage->m_pData );

			if( pxData )
			{
				s_pxSetSentryGunFiringArc = pxData;

				Asura_Dialog* pxDialog;
				pxDialog = new Asura_Numerical_Dialog( "Firing Arc", xOffset, &pxData->m_fArc, 0.0f, 180.0f, 1.0f, "%0.0f" );
				pxDialog->SetHelpText( "Sets the horizontal angle that the gun will sweep for enemies ( in degrees )" );
				pxOverlay->AddDialog( pxDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();
			}
		}break;

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SET_SENTRYGUN_LETHAL_RANGE:
		{
			AVP_Message_SentryGun_Set_LethalRange* pxData = static_cast< AVP_Message_SentryGun_Set_LethalRange* >( pxMessage->m_pData );

			if( pxData )
			{
				s_pxSetSentryGunLethalRange = pxData;

				Asura_Dialog* pxDialog;
				pxDialog = new Asura_Numerical_Dialog( "Lethal Range (%)", xOffset, &pxData->m_fRange, 0.0f, 100.0f, 0.5f, "%0.1f" );
				pxDialog->SetHelpText( "Sets the % of Vision Range to perform lethal kills from the gun - where enemies will die with a single hit" );
				pxOverlay->AddDialog( pxDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();
			}
		}break;

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_SENTRYGUN_DRYFIRE_TOGGLE:
		{
			AVP_Message_SentryGun_Toggle_Dry_Fire* pxData = static_cast< AVP_Message_SentryGun_Toggle_Dry_Fire* >( pxMessage->m_pData );

			if( pxData )
			{
				s_pxSetSentryGunDryFireToggle = pxData;

				Asura_Dialog* pxDialog;
				pxDialog = new Asura_Tickbox_Dialog("Toggle Dry Fire ( Tick for ON )", xOffset, &pxData->m_bToggleDryFire);
				pxDialog->SetEnabled( true );
				static_cast<Asura_Tickbox_Dialog *>(pxDialog)->SetClickedFunction(ToolsAVP_Message_Editing::ConditionedSetModified, reinterpret_cast<int>(this));
				pxOverlay->AddDialog( pxDialog );				
				xOffset.y += Asura_Dialog::DefaultSpacing();
			}
		}break;

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_FACEHUGGER_HUG_ENTITY:
		{
			AVP_FaceHugger_Hug_Entity* pxData = static_cast< AVP_FaceHugger_Hug_Entity* >( pxMessage->m_pData );

			if( pxData )
			{

				Asura_String xTargetString( "Target entity: " );

				if (pxData->m_uTarget == ASURA_GUID_UNREGISTERED)
				{
					xTargetString += "None";
				}
				else
				{
					ToolsAVP_Entity_PhysicalObject* pxToolsEntity = ToolsAVP_Entity_PhysicalObject::FindEntity(pxData->m_uTarget);

					if (pxToolsEntity)
					{
						if (pxToolsEntity->GetName())
						{
							xTargetString += pxToolsEntity->GetName();
						}
						else
						{
							Asura_String xGuidString;
							Asura_StringUtil::ToString( pxToolsEntity->GetGuid(), xGuidString );
							xTargetString += xGuidString;
						}
					}
					else
					{
						xTargetString += "Error";
					}
				}


				s_puEntityGuid = &pxData->m_uTarget;
				Asura_Dialog* pxDialog = new Asura_Function_Dialog(xTargetString.GetString(), xOffset, SelectHuggedEntity);
				pxDialog->SetHelpText("Allows selection of the Hugged entity.");
				pxOverlay->AddDialog(pxDialog);
				pxDialog->SetEnabled( true );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialog = new Asura_Function_Dialog("Clear Target entity", xOffset, ClearHuggedEntity);
				pxDialog->SetHelpText("Clears the selection of the Hugged entity being used.");
				pxOverlay->AddDialog(pxDialog);
				xOffset.y += Asura_Dialog::DefaultSpacing();

			}
		}
		break;

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_DETACH_WEAPON:
		case AVP_MSGID_CREATE_ATTACHMENT:
		case AVP_MSGID_DESTROY_ATTACHMENT:
		case AVP_MSGID_DETACH_ATTACHMENT:
		{
			AVP_Message_Actor_Attachment* pxData = static_cast< AVP_Message_Actor_Attachment* >( pxMessage->m_pData );

			if( pxData )
			{
				s_pxActorAttachmentMessage = pxData;

				Asura_Dialog* pxDialog;
				pxDialog = new Asura_BlueprintPicker_Dialog( "Attachment Blueprint", xOffset, 20, &pxData->m_uBlueprintHash, NULL, uAVP_BLUEPRINT_TYPE_ATTACHMENTS);
				pxDialog->SetHelpText( "Marker point to attach weapon to / detach weapon from." );
				pxOverlay->AddDialog( pxDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();
			}
			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_ATTACH_WEAPON:
		{
			// No data
			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_ALIENQUEEN_ADD_TO_TARGET_LIST:
		{
			// assign the messge to the static pointer in this class
			s_pxAddEntityToAlienQueenTargetList = static_cast<AVP_Msg_Add_Entity_To_Queen_Target_List*>( pxMessage->m_pData );

			Asura_Dialog *pxDialog;

			// add the option to set the queen to add to, notice the function to select the queen is defined here. 
			pxDialog = new Asura_Function_Dialog( "Select Queen To Add To", xOffset, SelectQueenToAddTo );
			pxDialog->SetHelpText( "Allows Selection of the Queen that holds the Target list to be added to." );
			pxOverlay->AddDialog( pxDialog );
			pxDialog->SetEnabled( true );
		
			xOffset.y += Asura_Dialog::DefaultSpacing();

			if( s_pxAddEntityToAlienQueenTargetList->m_uQueenGuid != ASURA_GUID_UNREGISTERED )
			{
				ToolsAVP_Entity_AlienQueen *pxPhysObj = static_cast<ToolsAVP_Entity_AlienQueen*>( ToolsAVP_Entity_AlienQueen::FindEntity( s_pxAddEntityToAlienQueenTargetList->m_uQueenGuid ) );

				if( pxPhysObj )
				{
					// if we already have a queen selected then update the name in the dialoge. 
					pxDialog->UpdateLabel( pxPhysObj->GetName() );
				}
			}

			// add the option to clear the selected queen. 
			pxDialog = new Asura_Function_Dialog( "Clear Selected Queen", xOffset, ClearSelectedQueen_ADD );
			pxDialog->SetHelpText( "Clears the Currently selected queen to add to" );
			pxOverlay->AddDialog( pxDialog );
			pxDialog->SetEnabled( true );
			
			xOffset.y += Asura_Dialog::DefaultSpacing();

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_ALIENQUEEN_REMOVE_FROM_TARGET_LIST:
		{
			// assign the messge to the static pointer in this class
			s_pxRemoveEntityFromAlienQueenTargetList = static_cast<AVP_Msg_Remove_Entity_From_Queen_Target_List*>( pxMessage->m_pData );

			Asura_Dialog *pxDialog;

			// add the option to set the queen to remove from, notice the function to select the queen is defined here. 
			pxDialog = new Asura_Function_Dialog( "Select Queen To Remove From", xOffset, SelectQueenToRemoveFrom );
			pxDialog->SetHelpText( "Allows Selection of the Queen that holds the Target list to be Removed from." );
			pxOverlay->AddDialog( pxDialog );
			pxDialog->SetEnabled( true );

			xOffset.y += Asura_Dialog::DefaultSpacing();

			if( s_pxRemoveEntityFromAlienQueenTargetList->m_uQueenGuid != ASURA_GUID_UNREGISTERED )
			{
				ToolsAVP_Entity_AlienQueen *pxPhysObj = static_cast<ToolsAVP_Entity_AlienQueen*>( ToolsAVP_Entity_AlienQueen::FindEntity( s_pxRemoveEntityFromAlienQueenTargetList->m_uQueenGuid ) );

				if( pxPhysObj )
				{
					// if we already have a queen selected then update the name in the dialoge. 
					pxDialog->UpdateLabel( pxPhysObj->GetName() );
				}
			}

			// add the option to clear the selected queen. 
			pxDialog = new Asura_Function_Dialog( "Clear Selected Queen", xOffset, ClearSelectedQueen_REMOVE );
			pxDialog->SetHelpText( "Clears the Currently selected queen to Remove From" );
			pxOverlay->AddDialog( pxDialog );
			pxDialog->SetEnabled( true );

			xOffset.y += Asura_Dialog::DefaultSpacing();

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_ALIENQUEEN_FORCE_ROAR:
		{
			// assign the messge to the static pointer in this class
			s_pxForceAlienQueenRoar = static_cast<AVP_Msg_Alien_Queen_Start_Roar_Behaviour*>( pxMessage->m_pData );

			Asura_Dialog *pxDialog;

			// add a tick box to define if its the player that should be roared at. 
			pxDialog = new Asura_Tickbox_Dialog( "Roar At Player", xOffset, &s_pxForceAlienQueenRoar->m_bRoarAtPlayer, &s_bQueenTargetListChanged );
			pxDialog->SetHelpText( "If you want the queen to roar at the player tick this box." );
			pxOverlay->AddDialog( pxDialog );
			xOffset.y += Asura_Dialog::LargeSpacing();

			// add the option to set the target guid of the queens roar
			pxDialog = new Asura_Function_Dialog( "Select Target To Roar At", xOffset, SelectTargetForQueenToRoarAt );
			pxDialog->SetHelpText( "Allows Selection of A Target Entity to roar at, if left unselected the queen will just roar on spot" );
			pxOverlay->AddDialog( pxDialog );
			if( s_pxForceAlienQueenRoar->GetRoarAtPlayer() )
			{
				pxDialog->SetEnabled( false );
			}
			else
			{
				pxDialog->SetEnabled( true );
			}
			xOffset.y += Asura_Dialog::DefaultSpacing();

			if( s_pxForceAlienQueenRoar->m_uTargetGuid != ASURA_GUID_UNREGISTERED )
			{
				Tools_Entity_Object *pxPhysObj = static_cast<Tools_Entity_Object*>( Tools_Entity_Object::FindEntity( s_pxForceAlienQueenRoar->m_uTargetGuid ) );

				if( pxPhysObj )
				{
					// if we already have a queen selected then update the name in the dialoge. 
					pxDialog->UpdateLabel( pxPhysObj->GetName() );
				}
			}

			// add the option to clear the selected queen. 
			pxDialog = new Asura_Function_Dialog( "Clear Selected Target", xOffset, ClearSelectedTargetForQueensRoar );
			pxDialog->SetHelpText( "Clears the Currently selected Target of Queens Roar" );
			pxOverlay->AddDialog( pxDialog );
			if( s_pxForceAlienQueenRoar->GetRoarAtPlayer() )
			{
				pxDialog->SetEnabled( false );
			}
			else
			{
				pxDialog->SetEnabled( true );
			}
			xOffset.y += Asura_Dialog::DefaultSpacing();

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_ALIENQUEEN_SET_BEHAVIOUR:
		{
			AVP_Message_SetAlienQueenBehaviour* pxData = static_cast<AVP_Message_SetAlienQueenBehaviour*>( pxMessage->m_pData );
			s_pxSetAlienQueenBehaviourMessage = pxData;

			// add the title for the drop down selecton box and move the xOffset over enough. 
			Asura_Dialog* pxDialogue = new Asura_Dialog( "Behaviour:", xOffset );
			pxOverlay->AddDialog( pxDialogue );
			xOffset.x += 7.0f;

			// setup and add the drop down seletion box. 
			Asura_DropDown_List_Dialog* pxDDLDialogue = new Asura_DropDown_List_Dialog(
				xOffset,
				AVP_AlienQueenBehaviourData::s_apszBehaviourStrings,
				&pxData->m_iBehaviour,
				AVP_AlienQueenBehaviourData::NUM_BEHAVIOURS,
				AVP_AlienQueenBehaviourData::NUM_BEHAVIOURS,
				NULL,
				NULL,
				"" );
			pxDDLDialogue->SetOnSelectCallback( SePredatorBehaviour, reinterpret_cast< int >( this ) );
			pxOverlay->AddDialog( pxDDLDialogue );

			// return the xOffset back to its original position and lower the y for the next entry. 
			xOffset.x -= 7.0f;
			xOffset.y += Asura_Dialog::LargeSpacing();
			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_START_CAMERA_SHAKE:
		{
			AVP_Msg_CameraShake* pxData = static_cast<AVP_Msg_CameraShake*>( pxMessage->m_pData );

			if(pxData)
			{
				s_pxCameraShakeMessage = pxData;
				
				Asura_Dialog* pxNewDialog = new Asura_Function_Dialog( "Shake Object:", xOffset, ChooseShakeObject );
				pxOverlay->AddDialog( pxNewDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();

	 			SetObjectNameString( s_xObjectNameString, pxData->m_uSourceGuid );
	 			pxNewDialog = new Asura_Dynamic_Dialog( &s_xObjectNameString, xOffset );
	 			pxOverlay->AddDialog( pxNewDialog );
	 			xOffset.y += Asura_Dialog::DefaultSpacing();

				pxNewDialog = new Asura_StringDisplay_Dialog("Leave blank for a global shake", xOffset );
				pxOverlay->AddDialog( pxNewDialog );
				xOffset.y += Asura_Dialog::LargeSpacing();

				// Won't work on 64bit
				pxNewDialog = new Asura_Function_Dialog( "Shake Type:", xOffset, ChooseShakeBlueprint, reinterpret_cast<int>(&(pxData->m_uShake)) );
				pxOverlay->AddDialog( pxNewDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				// Find our shake name
				Asura_Blueprint* pxBlueprint = Asura_Blueprint_System::GetBlueprintByHashID( uAVP_BLUEPRINT_TYPE_PLAYERS, pxData->m_uShake);
				if (pxBlueprint)
				{
					pxNewDialog = new Asura_StringDisplay_Dialog( pxBlueprint->GetName().GetString(), xOffset );	
				}
				else if (pxData->m_uShake == ASURA_HASH_ID_UNSET)
				{
					pxNewDialog = new Asura_StringDisplay_Dialog( "NONE", xOffset );	
				}
				else
				{
					pxNewDialog = new Asura_StringDisplay_Dialog( "NOT FOUND", xOffset );	
				}
				pxOverlay->AddDialog( pxNewDialog );
				xOffset.y += Asura_Dialog::LargeSpacing();	
			}

			break;
		}
	
		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_STOP_CAMERA_SHAKE:
		{
			AVP_Msg_CameraShake* pxData = static_cast<AVP_Msg_CameraShake*>( pxMessage->m_pData );

			if(pxData)
			{
				s_pxCameraShakeMessage = pxData;

				Asura_Dialog* pxNewDialog = new Asura_Function_Dialog( "Shake Object:", xOffset, ChooseShakeObject );
				pxOverlay->AddDialog( pxNewDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();

	 			SetObjectNameString( s_xObjectNameString, pxData->m_uSourceGuid );
	 			pxNewDialog = new Asura_Dynamic_Dialog( &s_xObjectNameString, xOffset );
	 			pxOverlay->AddDialog( pxNewDialog );
			}

			break;
		}

		//-------------------------------------------------------------------------------------------

		case AVP_MSGID_DISMEMBER:
		{
			s_pxDismemberMessage = static_cast< AVP_Msg_Dismember* >( pxMessage->m_pData );

			if( s_pxDismemberMessage )
			{
				static bool blud;

				Asura_Dialog* pxDialog;

				Asura_StringValuePair_DropDownList_Dialog* pxDDDDialog = NULL;
				pxDDDDialog = new Asura_StringValuePair_DropDownList_Dialog( xOffset, "", &s_pxDismemberMessage->m_iElement, &blud );

				const char **aszElementNames = Asura_Project_Skin_Elements::GetElementNames();
				for( u_int u=0; u<NUM_AVP_SKIN_ELEMENT_TYPES; u++ )
				{
					pxDDDDialog->AddItem( aszElementNames[u], u );
				}
				pxDDDDialog->InitialiseData();
				pxDDDDialog->CreateGadgets( xOffset );
				pxOverlay->AddDialog( pxDDDDialog );
				pxDDDDialog->SetEnabled( true );
//				xOffset.y += Asura_Dialog::DefaultSpacing();


				pxDialog = new Asura_Function_Dialog("Set Damage Position", xOffset, ToolsAVP_Message_Editing::MsgDismember_SetDamagePosition );
				pxOverlay->AddDialog(pxDialog);
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialog = new Asura_Function_Dialog("Set Rag-doll Direction", xOffset, ToolsAVP_Message_Editing::MsgDismember_SetRagdollDirection );
				pxOverlay->AddDialog(pxDialog);
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxDialog = new Asura_Function_Dialog("Set Gib Direction", xOffset, ToolsAVP_Message_Editing::MsgDismember_SetGibDirection );
				pxOverlay->AddDialog(pxDialog);
				xOffset.y += Asura_Dialog::DefaultSpacing();


				Asura_Numerical_Dialog *pxNumDialog = NULL;

				pxNumDialog = new Asura_Numerical_Dialog( "Ragdoll Impulse", xOffset, &s_pxDismemberMessage->m_fRagdollImpulse, 0.0f, 100.0f, 0.1f, "%0.1f" );
				pxOverlay->AddDialog( pxNumDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxNumDialog = new Asura_Numerical_Dialog( "Gib Impulse", xOffset, &s_pxDismemberMessage->m_fGibImpulse, 0.0f, 100.0f, 0.1f, "%0.1f" );
				pxOverlay->AddDialog( pxNumDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxNumDialog = new Asura_Numerical_Dialog( "Damage", xOffset, &s_pxDismemberMessage->m_fDamage, 0.0f, 100.0f, 1.0f, "%0.1f" );
				pxOverlay->AddDialog( pxNumDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();


/*
				pxNumDialog = new Asura_Numerical_Dialog( "Damage Position.x", xOffset, &s_pxDismemberMessage->m_xDamagePosition.x, -1000000.0f, 1000000.0f, 0.1f, "%0.1f" );
				pxNumDialog->SetReadOnly(true);
				pxOverlay->AddDialog( pxNumDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxNumDialog = new Asura_Numerical_Dialog( "Damage Position.y", xOffset, &s_pxDismemberMessage->m_xDamagePosition.y, -1000000.0f, 1000000.0f, 0.1f, "%0.1f" );
				pxNumDialog->SetReadOnly(true);
				pxOverlay->AddDialog( pxNumDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxNumDialog = new Asura_Numerical_Dialog( "Damage Position.z", xOffset, &s_pxDismemberMessage->m_xDamagePosition.z, -1000000.0f, 1000000.0f, 0.1f, "%0.1f" );
				pxNumDialog->SetReadOnly(true);
				pxOverlay->AddDialog( pxNumDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxNumDialog = new Asura_Numerical_Dialog( "Ragdoll Direction.x", xOffset, &s_pxDismemberMessage->m_xRagdollDirection.x, -1000000.0f, 1000000.0f, 0.1f, "%0.1f" );
				pxNumDialog->SetReadOnly(true);
				pxOverlay->AddDialog( pxNumDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxNumDialog = new Asura_Numerical_Dialog( "Ragdoll Direction.y", xOffset, &s_pxDismemberMessage->m_xRagdollDirection.y, -1000000.0f, 1000000.0f, 0.1f, "%0.1f" );
				pxNumDialog->SetReadOnly(true);
				pxOverlay->AddDialog( pxNumDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxNumDialog = new Asura_Numerical_Dialog( "Ragdoll Direction.z", xOffset, &s_pxDismemberMessage->m_xRagdollDirection.z, -1000000.0f, 1000000.0f, 0.1f, "%0.1f" );
				pxNumDialog->SetReadOnly(true);
				pxOverlay->AddDialog( pxNumDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxNumDialog = new Asura_Numerical_Dialog( "Gib Direction.x", xOffset, &s_pxDismemberMessage->m_xGibDirection.x, -1000000.0f, 1000000.0f, 0.1f, "%0.1f" );
				pxNumDialog->SetReadOnly(true);
				pxOverlay->AddDialog( pxNumDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxNumDialog = new Asura_Numerical_Dialog( "Gib Direction.y", xOffset, &s_pxDismemberMessage->m_xGibDirection.y, -1000000.0f, 1000000.0f, 0.1f, "%0.1f" );
				pxNumDialog->SetReadOnly(true);
				pxOverlay->AddDialog( pxNumDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();

				pxNumDialog = new Asura_Numerical_Dialog( "Gib Direction.z", xOffset, &s_pxDismemberMessage->m_xGibDirection.z, -1000000.0f, 1000000.0f, 0.1f, "%0.1f" );
				pxNumDialog->SetReadOnly(true);
				pxOverlay->AddDialog( pxNumDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();
*/

				pxNumDialog = new Asura_Numerical_Dialog( "From Guid", xOffset, &s_pxDismemberMessage->m_uFromGuid, 0, 100000000, 1, "%d" );
				pxOverlay->AddDialog( pxNumDialog );
				xOffset.y += Asura_Dialog::DefaultSpacing();

			}
		}break;

		//-------------------------------------------------------------------------------------------


		default:
		{
			PARENT::AddOverlayControls(pxMessage, pxOverlay, xOffset);
			break;
		} 
	}
}

void ToolsAVP_Message_Editing::MsgDismember_SetDamagePosition()
{
	if( s_pxDismemberMessage )
	{
		Tools_PositionEdit_Object* pxPosObject = new Tools_PositionEdit_Object(&s_pxDismemberMessage->m_xDamagePosition, NULL, "Damage Position point", "Edit position of damage point.", NULL );
		if (pxPosObject)
		{
			// We need to remember our currently selected object, because we are going to select PositionEdit object.
			// However, upon destruction of PositionEdit object, we need to return focus to this object.
			Tools_Object* pxPreviousObject = Tools_Object_System::GetMainSelectionForCurrentSelectionMode();

			// Now set new object to have a focus. It is important so we can move this object using F and V keys.
			Tools_Object_System::SelectObject( pxPosObject );

			// Create "Chooser" object. This object is special, because it will set tools to special mode. Where you are
			// selecting new object.
			Tools_ChooseMoveObject* pxChooseObject = new Tools_ChooseMoveObject( pxPosObject, pxPreviousObject );
			Tools_Object_System::ChooseObject("Please move position object. Use 'f' and 'v' keys to do this", pxChooseObject);
		}
	}
}

void ToolsAVP_Message_Editing::MsgDismember_SetRagdollDirection()
{
	static Asura_Vector_3 xDummyPosition;

	if( s_pxDismemberMessage )
	{
		xDummyPosition.Zero();
		Tools_PositionEdit_Object* pxPosObject = new Tools_PositionEdit_Object(&xDummyPosition, NULL, "Ragdoll Direction", "Edit rotation of Ragdoll hit direction.", NULL );
		if (pxPosObject)
		{
			pxPosObject->SetDirectionToEdit(&s_pxDismemberMessage->m_xRagdollDirection);

			// We need to remember our currently selected object, because we are going to select PositionEdit object.
			// However, upon destruction of PositionEdit object, we need to return focus to this object.
			Tools_Object* pxPreviousObject = Tools_Object_System::GetMainSelectionForCurrentSelectionMode();

			// Now set new object to have a focus. It is important so we can move this object using F and V keys.
			Tools_Object_System::SelectObject( pxPosObject );

			// Create "Chooser" object. This object is special, because it will set tools to special mode. Where you are
			// selecting new object.
			Tools_ChooseMoveObject* pxChooseObject = new Tools_ChooseMoveObject( pxPosObject, pxPreviousObject );
			Tools_Object_System::ChooseObject("Please rotate ragdoll direction object. Use 'f' and 'v' keys to do this", pxChooseObject);
		}
	}
}

void ToolsAVP_Message_Editing::MsgDismember_SetGibDirection()
{
	static Asura_Vector_3 xDummyPosition;

	if( s_pxDismemberMessage )
	{
		xDummyPosition.Zero();
		Tools_PositionEdit_Object* pxPosObject = new Tools_PositionEdit_Object(&xDummyPosition, NULL, "Gib Direction point", "Edit rotation of Gib fly direction.", NULL );
		if (pxPosObject)
		{
			pxPosObject->SetDirectionToEdit(&s_pxDismemberMessage->m_xGibDirection);

			// We need to remember our currently selected object, because we are going to select PositionEdit object.
			// However, upon destruction of PositionEdit object, we need to return focus to this object.
			Tools_Object* pxPreviousObject = Tools_Object_System::GetMainSelectionForCurrentSelectionMode();

			// Now set new object to have a focus. It is important so we can move this object using F and V keys.
			Tools_Object_System::SelectObject( pxPosObject );

			// Create "Chooser" object. This object is special, because it will set tools to special mode. Where you are
			// selecting new object.
			Tools_ChooseMoveObject* pxChooseObject = new Tools_ChooseMoveObject( pxPosObject, pxPreviousObject );
			Tools_Object_System::ChooseObject("Please rotate gib direction object. Use 'f' and 'v' keys to do this", pxChooseObject);
		}
	}
}

void ToolsAVP_Message_Editing::ChooseKnowledgeSettingsBlueprint( void )
{
	Asura_Hash_ID uBlueprintTypeHashID = 0x7ad7bd1b; //"AI Settings"
	Asura_Hash_ID uBlueprintHashID     = s_pxSetHumanoidKnowledgeSettingsMsg->m_uBluepprintHashID;

	BlueprintSelect_Overlay xBPOverlay( "Choose A Blueprint", &uBlueprintHashID, &uBlueprintTypeHashID );

	if ( !xBPOverlay.DoModal() ) 
	{
		return;
	}

	s_pxSetHumanoidKnowledgeSettingsMsg->m_uBluepprintHashID = uBlueprintHashID;
	s_bHumanoidKnowledgeSettingTypeChanged = true;
}

void ToolsAVP_Message_Editing::SelectDamager(void)
{
	ToolsAVP_SelectDamager_ChooseObject* pxChooseObject = new ToolsAVP_SelectDamager_ChooseObject( SetSelectedDamager );
	Tools_Object_System::ChooseObject( "Pick the Entity that the damage will come from.", pxChooseObject );
}

void ToolsAVP_Message_Editing::ClearDamager(void)
{
	s_pxSetHealthMessage->m_uDamageFromGuid = ASURA_GUID_UNREGISTERED;
	s_bHealthSettingTypeChanged = true;
}

void ToolsAVP_Message_Editing::SetSelectedDamager( Asura_Guid uGuid )
{
	s_pxSetHealthMessage->m_uDamageFromGuid = uGuid;
	s_bHealthSettingTypeChanged = true;
}


void ToolsAVP_Message_Editing::SelectSpawneeAlienSquad(int iDataPtr)
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType( TOT_Entity_AlienSquad, ToolsAVP_Message_Editing::SetSelectedSpawneeAlienSquad, iDataPtr );
	Tools_Object_System::ChooseObject( "Select A Squad", pxChooseObject );
}

void ToolsAVP_Message_Editing::ClearSpawneeAlienSquad(int iDataPtr)
{
	AVP_Msg_Spawn *pxData = reinterpret_cast<AVP_Msg_Spawn *>(iDataPtr);
	pxData->m_uSquadGuid = ASURA_GUID_UNREGISTERED;

	(static_cast<ToolsAVP_Message_Editing*>(s_pxThis))->AVP_SetModifiedAndUpdateMessageOverlay();
}

void ToolsAVP_Message_Editing::SetSelectedSpawneeAlienSquad( int iDataPtr, Asura_Guid uGuid )
{
	AVP_Msg_Spawn *pxData = reinterpret_cast<AVP_Msg_Spawn *>(iDataPtr);
	pxData->m_uSquadGuid = uGuid;

	(static_cast<ToolsAVP_Message_Editing*>(s_pxThis))->AVP_SetModifiedAndUpdateMessageOverlay();
}

bool ToolsAVP_SelectDamager_ChooseObject::CanBeSelected( Tools_Object* pxObject )
{
	return true;
}

void ToolsAVP_Message_Editing::SelectLookAt( void )
{
	ToolsAVP_LookAtChooseObject* pxChooseObject = new ToolsAVP_LookAtChooseObject( SetSelectedLookAt, s_uGuidToIgnoreForSelectionInMessage );
	Tools_Object_System::ChooseObject( "Pick the LookAt Entity.", pxChooseObject );
}

void ToolsAVP_Message_Editing::SetSelectedLookAt( Asura_Guid uGuid )
{
	if ( s_pxLookAtEntityMessage )
	{
		s_pxLookAtEntityMessage->m_uLookAtGuid = uGuid;
		s_bLookAtChanged = true;
	}
}

void ToolsAVP_Message_Editing::ClearLookAt(void)
{
	if ( s_pxLookAtEntityMessage )
	{
		s_pxLookAtEntityMessage->m_uLookAtGuid = ASURA_GUID_UNREGISTERED;
		s_bLookAtChanged = true;
	}
}

////////////////////////////////////////////////////////////////////////////////
// AVP_MSGID_ALIENQUEEN_ADD_TO_TARGET_LIST functions

void ToolsAVP_Message_Editing::SelectQueenToAddTo( void )
{		
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType( TOT_Entity_AlienQueen, SetSelectedQueen_ADD );
	Tools_Object_System::ChooseObject( "Pick the Queen To Add Entity To.", pxChooseObject );
}

void ToolsAVP_Message_Editing::SetSelectedQueen_ADD( Asura_Guid uGuid )
{
	if ( s_pxAddEntityToAlienQueenTargetList )
	{
		s_pxAddEntityToAlienQueenTargetList->m_uQueenGuid = uGuid;
		s_bQueenTargetListChanged = true;
	}
}

void ToolsAVP_Message_Editing::ClearSelectedQueen_ADD(void)
{
	if ( s_pxAddEntityToAlienQueenTargetList )
	{
		s_pxAddEntityToAlienQueenTargetList->m_uQueenGuid = ASURA_GUID_UNREGISTERED;
		s_bQueenTargetListChanged = true;
	}
}

////////////////////////////////////////////////////////////////////////////////
// AVP_MSGID_ALIENQUEEN_REMOVE_FROM_TARGET_LIST functions

void ToolsAVP_Message_Editing::SelectQueenToRemoveFrom( void )
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType( TOT_Entity_AlienQueen, SetSelectedQueen_REMOVE );
	Tools_Object_System::ChooseObject( "Pick the Queen To Remove Entity From.", pxChooseObject );
}

void ToolsAVP_Message_Editing::SetSelectedQueen_REMOVE( Asura_Guid uGuid )
{
	if ( s_pxRemoveEntityFromAlienQueenTargetList )
	{
		s_pxRemoveEntityFromAlienQueenTargetList->m_uQueenGuid = uGuid;
		s_bQueenTargetListChanged = true;
	}
}

void ToolsAVP_Message_Editing::ClearSelectedQueen_REMOVE(void)
{
	if ( s_pxRemoveEntityFromAlienQueenTargetList )
	{
		s_pxRemoveEntityFromAlienQueenTargetList->m_uQueenGuid = ASURA_GUID_UNREGISTERED;
		s_bQueenTargetListChanged = true;
	}
}

////////////////////////////////////////////////////////////////////////////////
// AVP_MSGID_ALIENQUEEN_FORCE_ROAR functions

void ToolsAVP_Message_Editing::SelectTargetForQueenToRoarAt( void )
{
	// declare the choose opbject class and add all of the types that are allowed. 
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType( TOT_Entity_Predator, SetSelectedTargetOfQueensRoar );
	pxChooseObject->AddType( TOT_Entity_Human );
	pxChooseObject->AddType( TOT_Entity_Android );
	pxChooseObject->AddType( TOT_Entity_LookAt );
	pxChooseObject->AddType( TOT_Entity_Civilian );
	pxChooseObject->AddType( TOT_Entity_Marine );
	pxChooseObject->AddType( TOT_Entity_ShootAt );
	pxChooseObject->AddType( TOT_Entity_AVPGS2Node );
	pxChooseObject->AddType( TOT_Entity_LocationMarker );
	pxChooseObject->AddType( TOT_Entity_Predalien );

	pxChooseObject->SetAllowMultipleSelection( false );

	Tools_Object_System::ChooseObject( "Pick What To Roar At", pxChooseObject );
}

void ToolsAVP_Message_Editing::SetSelectedTargetOfQueensRoar( Asura_Guid uGuid )
{
	if ( s_pxForceAlienQueenRoar )
	{
		s_pxForceAlienQueenRoar->m_uTargetGuid = uGuid;

		// reuse the queen target list variable here, no need to have two when both message types are used in the same way. 
		s_bQueenTargetListChanged = true;
	}
}

void ToolsAVP_Message_Editing::ClearSelectedTargetForQueensRoar(void)
{
	if ( s_pxForceAlienQueenRoar )
	{
		s_pxForceAlienQueenRoar->m_uTargetGuid = ASURA_GUID_UNREGISTERED;

		// reuse the queen target list variable here, no need to have two when both message types are used in the same way. 
		s_bQueenTargetListChanged = true;
	}
}

void ToolsAVP_Message_Editing::SetAlienQueenBehaviour( int iThis )
{
	if ( !s_pxSetAlienQueenBehaviourMessage )
	{
		return;
	}

	// fransfer the data from a message into the actual data type it will be used as and work out any other data needed through its functions. 
	AVP_AlienQueenBehaviourData xBehaviourData;
	xBehaviourData.m_iBehaviour = s_pxSetAlienQueenBehaviourMessage->m_iBehaviour;
	xBehaviourData.SetParamsFromBehaviour();

	// here copy any data that has just been determined back into the message data for transfer. 

	// Becuase we're updating the message overlay we have to mark the message data as modified or it'll get forgotten.
	ToolsAVP_Message_Editing* pxThis = reinterpret_cast< ToolsAVP_Message_Editing* >( iThis );
	pxThis->AVP_SetModifiedAndUpdateMessageOverlay();
}

void ToolsAVP_Message_Editing::SelectShootAt( void )
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType( TOT_Entity_ShootAt, SetSelectedShootAt );
	Tools_Object_System::ChooseObject( "Pick the ShootAt Point.", pxChooseObject );
}

void ToolsAVP_Message_Editing::SetSelectedShootAt( Asura_Guid uGuid )
{
	if ( s_pxAttackScriptedTargetMessage )
	{
		s_pxAttackScriptedTargetMessage->m_uShootAtGuid = uGuid;
		s_bShootAtChanged = true;
	}
}

void ToolsAVP_Message_Editing::ClearShootAt(void)
{
	if ( s_pxAttackScriptedTargetMessage )
	{
		s_pxAttackScriptedTargetMessage->m_uShootAtGuid = ASURA_GUID_UNREGISTERED;
		s_bShootAtChanged = true;
	}
}


void ToolsAVP_Message_Editing::SelectHuggedEntity()
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType( TOT_Entity_Predator, s_puEntityGuid, &s_bHuggedEntityChanged );
	pxChooseObject->AddType(TOT_Entity_Human);
	pxChooseObject->AddType(TOT_Entity_Marine);
	pxChooseObject->AddType(TOT_Entity_Civilian);
	Tools_Object_System::ChooseObject( "Pick the Hugged Target Entity.", pxChooseObject );
}

void ToolsAVP_Message_Editing::ClearHuggedEntity()
{
	*s_puEntityGuid = ASURA_GUID_UNREGISTERED;
}

const EventIDAndDescription* ToolsAVP_Message_Editing::GetProjectEventList()
{
	return s_axProjectEventList;
}

const char* ToolsAVP_Message_Editing::GetEventDescriptionMain(int iEventID)
{
	static Asura_String s_axHealthEventString[ ( AVP_ENTITY_EVENT_ONHEALTHPERCENT9 - AVP_ENTITY_EVENT_ONHEALTHPERCENT0 ) + 1 ];

	// We override this in order to replace the dummy text with the correct text for the 'on health percent' messages.
	if ( AVP_ENTITY_EVENT_ONHEALTHPERCENT0 <= iEventID && iEventID <= AVP_ENTITY_EVENT_ONHEALTHPERCENT9 )
	{
		// Normally use the message edit object but, if it's null (i.e. we're not editing 
		// messages yet - we must be adding a message block or similar), use the currently selected object.
		Tools_Object* pxObject = Tools_Object_System::GetMessageEditObject();
		if ( !pxObject )
		{
			pxObject = Tools_Object_System::SelectedObject;
		}	
		if ( !pxObject )
		{
			Asura_Assert( false, "ToolsAVP_Message_Editing::GetEventDescriptionMain - failed to obtain real event name. Returning default." );
			return Tools_Message_Editing::GetEventDescriptionMain( iEventID );
		}

		float fHealthPercent = 100.0f;		
		AVP_ServerEntity_PhysicalObject* pxPhyObj = AVP_ServerEntity_PhysicalObject::FindEntity( pxObject->GetGuid() );
		if ( !pxPhyObj )
		{
			Asura_Assert( false, "ToolsAVP_Message_Editing::GetEventDescriptionMain - failed to obtain real event name. Returning default." );
			return Tools_Message_Editing::GetEventDescriptionMain( iEventID );
		}
		else
		{
			// Value is stored as a fraction 0.0f -> 1.0f, so convert back into a percentage.
			fHealthPercent = pxPhyObj->GetHealthValueForOnHealthEvent( iEventID ) * 100.0f;
		}	

		Asura_String& xHealthEventDesc = s_axHealthEventString[ iEventID - AVP_ENTITY_EVENT_ONHEALTHPERCENT0 ];
		xHealthEventDesc = "On ";
		Asura_String xValueString;
		Asura_StringUtil::ToString( Asura_Maths::FloatToInt_Near( fHealthPercent ), xValueString );
		xHealthEventDesc += xValueString;
		xHealthEventDesc += "% Health";
		return xHealthEventDesc.GetString();
	}
	else
	{
		return Tools_Message_Editing::GetEventDescriptionMain( iEventID );
	}
}

const char* ToolsAVP_Message_Editing::AVP_GetNameFromGuid(Asura_Guid guid)
{
	// Check alien team
	if(AVP_ServerEntity_Team_Alien::GetInstance())
	{
		if(guid == AVP_ServerEntity_Team_Alien::GetInstance()->GetGuid())
		{
			return AVP_ServerEntity_Team_Alien::GetInstance()->ToolsData->Name.GetString();
		}
	}

	// Check marine team
	if(AVP_ServerEntity_Team_Marine::GetInstance())
	{
		if(guid == AVP_ServerEntity_Team_Marine::GetInstance()->GetGuid())
		{
			return AVP_ServerEntity_Team_Marine::GetInstance()->ToolsData->Name.GetString();
		}
	}
 
	// Check all marine squads
	for(Asura_Hashed_List_It<AVP_ServerEntity_Squad_Marine> xSquadListIt(AVP_ServerEntity_Squad_Marine::GetList()); !xSquadListIt.Done(); xSquadListIt.Next() )
	{
		if(guid == xSquadListIt.GetCurrent()->GetGuid())
		{
			return xSquadListIt.GetCurrent()->ToolsData->Name.GetString();
		}
	}

	return NULL;
}

void ToolsAVP_Message_Editing::AVP_EditDestionationGuid(ListSelection_Overlay& xOverlay)
{
	// Add team names & Guids to overlay
	if (AVP_ServerEntity_Team_Alien::GetInstance())
	{
		xOverlay.AddItem(AVP_ServerEntity_Team_Alien::GetInstance()->ToolsData->Name.GetString(),AVP_ServerEntity_Team_Alien::GetInstance()->GetGuid());
	}
	if (AVP_ServerEntity_Team_Marine::GetInstance())
	{
		xOverlay.AddItem(AVP_ServerEntity_Team_Marine::GetInstance()->ToolsData->Name.GetString(),AVP_ServerEntity_Team_Marine::GetInstance()->GetGuid());
	}
	for(Asura_Hashed_List_It<AVP_ServerEntity_Squad_Marine> xSquadListIt(AVP_ServerEntity_Squad_Marine::GetList()); !xSquadListIt.Done(); xSquadListIt.Next() )
	{
		xOverlay.AddItem(xSquadListIt.GetCurrent()->ToolsData->Name.GetString(),xSquadListIt.GetCurrent()->GetGuid());
	}
}


void ToolsAVP_Message_Editing::SePredatorBehaviour( int iThis )
{
	if ( !s_pxSetPredatorBehaviourMessage)
	{
		return;
	}

	AVP_PredatorBehaviourData xBehaviourData;
	xBehaviourData.m_iBehaviour = s_pxSetPredatorBehaviourMessage->m_iBehaviour;
	xBehaviourData.SetParamsFromBehaviour();

	// Becuase we're updating the message overlay we have to mark the message data as modified or it'll get forgotten.
	ToolsAVP_Message_Editing* pxThis = reinterpret_cast< ToolsAVP_Message_Editing* >( iThis );
	pxThis->AVP_SetModifiedAndUpdateMessageOverlay();
}

void ToolsAVP_Message_Editing::ConditionedSetModified( int iThis )
{
	
	ToolsAVP_Message_Editing* pxThis = reinterpret_cast< ToolsAVP_Message_Editing* >( iThis );
	pxThis->AVP_SetModifiedAndUpdateMessageOverlay();
}

const Asura_Hash_ID g_uAVP_BLUEPRINT_HASH_AGGRESSIVE = 0x445fdc04; //"Aggressive"
const Asura_Hash_ID g_uAVP_BLUEPRINT_HASH_HIDEINSHADOWS = 0x84a4678c; //"HideInShadows"

void ToolsAVP_Message_Editing::SetAlienBehviourPrototype( int iThis )
{
	if ( !s_pxSetAlienSquadBehaviourMsg )
	{
		return;
	}

	AVP_AlienBehaviourData xBehaviourData;
	xBehaviourData.m_iBehaviour = s_pxSetAlienSquadBehaviourMsg->m_iBehaviour;
	xBehaviourData.m_uProtoBlueprintHashID = s_pxSetAlienSquadBehaviourMsg->m_uProtoBlueprintHashID;

	Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_AI_SETTINGS;
	Asura_Hash_ID uParentBlueprintHashID = ASURA_HASH_ID_UNSET;

	switch (xBehaviourData.m_iBehaviour)
	{
	case AVP_AlienBehaviourData::AGGRESIVE:
		uParentBlueprintHashID = g_uAVP_BLUEPRINT_HASH_AGGRESSIVE;
		break;
	case AVP_AlienBehaviourData::HIDE_IN_SHADOWS:
		uParentBlueprintHashID = g_uAVP_BLUEPRINT_HASH_HIDEINSHADOWS;
		break;
	case AVP_AlienBehaviourData::AMBUSH:
		uParentBlueprintHashID = g_uAVP_BLUEPRINT_HASH_AGGRESSIVE; // for now the same as agrgessive
		break;		
	}

	Asura_Hash_ID uBlueprintTypeResultHashID = uAVP_BLUEPRINT_TYPE_AI_SETTINGS;

	BlueprintSelect_Overlay xBPOverlay( "Choose A Blueprint", &xBehaviourData.m_uProtoBlueprintHashID, &uBlueprintTypeResultHashID, 1, &uBlueprintTypeHashID, 1, &uParentBlueprintHashID );
	xBPOverlay.DoModal();

	if (xBehaviourData.m_uProtoBlueprintHashID != ASURA_HASH_ID_UNSET)
	{
		xBehaviourData.SetParamsFromBlueprint();
	}
	s_pxSetAlienSquadBehaviourMsg->m_iMoveInMode			= xBehaviourData.m_iMoveInMode;
	s_pxSetAlienSquadBehaviourMsg->m_uProtoBlueprintHashID	= xBehaviourData.m_uProtoBlueprintHashID;
	s_pxSetAlienSquadBehaviourMsg->m_fJumpFraction			= xBehaviourData.m_fJumpFraction;
	s_pxSetAlienSquadBehaviourMsg->m_fStandFraction			= xBehaviourData.m_fStandFraction;
	s_pxSetAlienSquadBehaviourMsg->m_fMinTimeBetweenGrapples= xBehaviourData.m_fMinTimeBetweenGrapples;
	s_pxSetAlienSquadBehaviourMsg->m_fDamageFromBehindMultiplier= xBehaviourData.m_fDamageFromBehindMultiplier;
	s_pxSetAlienSquadBehaviourMsg->m_fIdealDistanceMin		= xBehaviourData.m_fIdealDistanceMin;
	s_pxSetAlienSquadBehaviourMsg->m_fIdealDistanceMax		= xBehaviourData.m_fIdealDistanceMax;
	s_pxSetAlienSquadBehaviourMsg->m_fAttackDelay			= xBehaviourData.m_fAttackDelay;
	s_pxSetAlienSquadBehaviourMsg->m_iMaxAttackersPerTarget = xBehaviourData.m_iMaxAttackersPerTarget;
	s_pxSetAlienSquadBehaviourMsg->m_iMaxAttackersTotal		= xBehaviourData.m_iMaxAttackersTotal;
	s_pxSetAlienSquadBehaviourMsg->m_iMaxBurstPerTarget		= xBehaviourData.m_iMaxBurstPerTarget;
	s_pxSetAlienSquadBehaviourMsg->m_iPostAttackAction		= xBehaviourData.m_iPostAttackAction;
	s_pxSetAlienSquadBehaviourMsg->m_fRespawnDelay			= xBehaviourData.m_fRespawnDelay;
	s_pxSetAlienSquadBehaviourMsg->m_fTimeBetweenAttacks	= xBehaviourData.m_fTimeBetweenAttacks;
	s_pxSetAlienSquadBehaviourMsg->m_uMaxNumberOfChargers	= xBehaviourData.m_uMaxNumberOfChargers;
	s_pxSetAlienSquadBehaviourMsg->m_bAttackIfInMarineLight		= xBehaviourData.m_bAttackIfInMarineLight;
	s_pxSetAlienSquadBehaviourMsg->m_fTimeInLightBeforeAttack = xBehaviourData.m_fTimeInLightBeforeAttack;
	s_pxSetAlienSquadBehaviourMsg->m_fMinHideDistance		= xBehaviourData.m_fMinHideDistance;
	s_pxSetAlienSquadBehaviourMsg->m_fChaneToRehideIfCloseFraction	= xBehaviourData.m_fChaneToRehideIfCloseFraction;
	s_pxSetAlienSquadBehaviourMsg->m_fChaneToRehideIfInLightFraction = xBehaviourData.m_fChaneToRehideIfInLightFraction;
	s_pxSetAlienSquadBehaviourMsg->m_fChaneToRehideAfterAttackFraction = xBehaviourData.m_fChaneToRehideAfterAttackFraction;

	// Becuase we're updating the message overlay we have to mark the message data as modified or it'll get forgotten.
	ToolsAVP_Message_Editing* pxThis = reinterpret_cast< ToolsAVP_Message_Editing* >( iThis );
	pxThis->AVP_SetModifiedAndUpdateMessageOverlay();
}



//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SelectRestrictedCombatZoneGPV( void )
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType(TOT_Entity_GamePlayVolume, SetSelectedRestrictedCombatZoneGPV);
	Tools_Object_System::ChooseObject("Pick the Objective point.", pxChooseObject);
}
//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::ClearRestrictedCombatZoneGPV( void )
{
	if ( s_pxChangeHumanoidSquadSettingsMessage )
	{
		s_pxChangeHumanoidSquadSettingsMessage->m_uCombatZoneGPVGuid = ASURA_GUID_UNREGISTERED;
		s_bSquadCombatZoneChanged = true;
	}
}
//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SetSelectedRestrictedCombatZoneGPV( Asura_Guid uGuid )
{
	if ( s_pxChangeHumanoidSquadSettingsMessage )
	{
		s_pxChangeHumanoidSquadSettingsMessage->m_uCombatZoneGPVGuid = uGuid;
		s_bSquadCombatZoneChanged = true;
	}
}

void ToolsAVP_Message_Editing::SetModifiedAndUpdateMessageOverlay( int iThis )
{
	ToolsAVP_Message_Editing* pxThis = reinterpret_cast< ToolsAVP_Message_Editing* >( iThis );
	pxThis->AVP_SetModifiedAndUpdateMessageOverlay();
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::UpdateSquadTextDialog(Asura_Guid uSquadGuid)
{
	if (!s_pxSquadTextDialog)
	{
		return;
	}

	if (uSquadGuid != ASURA_GUID_UNREGISTERED)
	{
		ToolsAVP_Entity_MarineSquad* pxSquad = (ToolsAVP_Entity_MarineSquad*)ToolsAVP_Entity_MarineSquad::FindEntity(uSquadGuid);

		if (pxSquad)
		{
			s_pxSquadTextDialog->UpdateLabel(pxSquad->GetName());
		}
		else
		{
			s_pxSquadTextDialog->UpdateLabel("Squad Not Found");
		}
	}
	else
	{
		s_pxSquadTextDialog->UpdateLabel("UNSET");
	}
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::UpdateSquadMemberTextDialog(Asura_Guid uGuid)
{
	if (!s_pxHumanoidMemberTextDialog)
	{
		return;
	}

	if (uGuid != ASURA_GUID_UNREGISTERED)
	{
		ToolsAVP_Entity_Human* pxHuman = (ToolsAVP_Entity_Human*)ToolsAVP_Entity_Human::FindEntity(uGuid);

		if (pxHuman)
		{
			s_pxHumanoidMemberTextDialog->UpdateLabel(pxHuman->GetName());
		}
		else
		{
			s_pxHumanoidMemberTextDialog->UpdateLabel("HUmanoid Not Found");
		}
	}
	else
	{
		s_pxHumanoidMemberTextDialog->UpdateLabel("UNSET");
	}
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SelectObjective(void)
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType(TOT_Entity_AI_Marine_Objective, SetAddObjectiveMessageObjectiveGuid);
	Tools_Object_System::ChooseObject("Pick the Objective point.", pxChooseObject);
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SelectPlayerObjective(void)
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType(TOT_Entity_Player_Objective, SetAddPlayerObjectiveMessageObjectiveGuid);
	Tools_Object_System::ChooseObject("Pick the Objective point.", pxChooseObject);
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SelectCompletedPlayerObjective(void)
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType(TOT_Entity_Player_Objective, SetPlayerObjectiveCompleteMessageObjectiveGuid);
	Tools_Object_System::ChooseObject("Pick the Completed Objective point.", pxChooseObject);
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SelectMarkerLocation(void)
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType(TOT_Entity_LocationMarker, SetPlayerObjectiveMessageMarkerGuid);
	Tools_Object_System::ChooseObject("Pick the location marker.", pxChooseObject);
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SelectTargetEntity(void)
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType(TOT_Entity_Marine, SetPlayerObjectiveMessageTargetEntityGuid);
	pxChooseObject->AddType(TOT_Entity_Alien);
	pxChooseObject->AddType(TOT_Entity_Predator);
	pxChooseObject->AddType(TOT_Entity_AlienQueen);
	pxChooseObject->AddType(TOT_Entity_Android);
	pxChooseObject->AddType(TOT_Entity_Civilian);
	pxChooseObject->AddType(TOT_Entity_Predalien);
	Tools_Object_System::ChooseObject("Pick the location marker.", pxChooseObject);
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SelectPlayerPickup(void)
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType(TOT_Entity_Pickup, SetPlayerPickupMessagePickupGuid);
	Tools_Object_System::ChooseObject("Pick the pickup.", pxChooseObject);
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SetAddObjectiveMessageObjectiveGuid(Asura_Guid uGuid)
{
	s_pxAddObjectiveMessage->m_uObjectiveGuid = uGuid;
	RefreshMessageOverlay();
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SetAddPlayerObjectiveMessageObjectiveGuid(Asura_Guid uGuid)
{
	s_pxAddPlayerObjectiveMessage->m_uObjectiveGuid = uGuid;
	RefreshMessageOverlay();
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SetPlayerObjectiveCompleteMessageObjectiveGuid(Asura_Guid uGuid)
{
	s_pxPlayerObjectiveCompleteMessage->m_uObjectiveGuid = uGuid;
	RefreshMessageOverlay();
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SetPlayerObjectiveMessageMarkerGuid(Asura_Guid uGuid)
{
	s_pxSetMarkerLocationMessage->m_uMarkerGuid = uGuid;
	RefreshMessageOverlay();
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SetPlayerObjectiveMessageTargetEntityGuid(Asura_Guid uGuid)
{
	s_pxSetTargetEntityMessage->m_uEntityGuid = uGuid;
	RefreshMessageOverlay();
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SetPlayerPickupMessagePickupGuid(Asura_Guid uGuid)
{
	s_pxSetPlayerPickupMessage->m_uPickupGuid = uGuid;
	RefreshMessageOverlay();
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SelectMarineToAdd(void)
{
	Tools_ChooseObject_ByType* pxChooseObject = new Tools_ChooseObject_ByType( TOT_Entity_Marine, &s_pxAddSquadMemberMessage->m_uGuidToAdd );
	Tools_Object_System::ChooseObject("Pick the Marine to add to the squad.", pxChooseObject);
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SelectMarineToRemove(void)
{
	Tools_ChooseObject_ByType* pxChooseObject = new Tools_ChooseObject_ByType( TOT_Entity_Marine, &s_pxRemoveSquadMemberMessage->m_uGuidToRemove );
	Tools_Object_System::ChooseObject("Pick the Marine to remove from the squad.", pxChooseObject);
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SelectMarineSquadToMergeInto(void)
{
	Tools_ChooseObject_ByType* pxChooseObject = new Tools_ChooseObject_ByType(TOT_Entity_MarineSquad, SetIntoSquadGuid_MergeSquadMessage);
	Tools_Object_System::ChooseObject("Select The Squad Entity", pxChooseObject);
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SetIntoSquadGuid_MergeSquadMessage(Asura_Guid uGuid)
{
	s_pxMergeSquadMessage->m_uSquadToMergeInto = uGuid;
	RefreshMessageOverlay();
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SelectSquadToAddTo(void)
{
	Tools_ChooseObject_ByType* pxChooseObject = new Tools_ChooseObject_ByType(TOT_Entity_MarineSquad, SetSquadToAddToGuid_SplitSquadMessage);
	Tools_Object_System::ChooseObject("Select The Squad Entity", pxChooseObject);
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SetSquadToAddToGuid_SplitSquadMessage(Asura_Guid uGuid)
{
	s_pxSplitSquadMessage->m_uSquadToAddTo = uGuid;
	RefreshMessageOverlay();
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SelectSquadMemberToMove(int iArrayIndex)
{
	s_uSquadMemberIndex = (u_int)iArrayIndex;

	// need to choose the correct tools type.

	if (s_uSplitSquadSentToGuid != ASURA_GUID_UNREGISTERED)
	{
		u_int uToolsType;

		for (Asura_Hashed_List_It<AVP_ServerActor_Human> xIt(AVP_ServerActor_Human::GetList()); !xIt.Done(); xIt.Next())
		{
			AVP_ServerActor_Human* pxHuman = xIt.GetCurrent();

			if (pxHuman && pxHuman->GetInitialSquadGuid() == s_uSplitSquadSentToGuid)
			{
				if (pxHuman->GetClassification() == AVP_ENTITYCLASS_ANDROID)
				{
					uToolsType = TOT_Entity_Android;
				}
				else
				{
					uToolsType = TOT_Entity_Marine;
				}

				ToolsAVP_SplitSquad_ChooseObject* pxChooseObject = new ToolsAVP_SplitSquad_ChooseObject(uToolsType, SetMemberToMove_SplitSquadMessage, s_uSplitSquadSentToGuid);
				Tools_Object_System::ChooseObject("Select The Humanoid To Move", pxChooseObject);
			}
			else
			{
				Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_ERROR, "ERROR : No Members in Squad To Split the Squad");
			}
		}
	}
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SetMemberToMove_SplitSquadMessage(Asura_Guid uGuid)
{
	bool bExists = false;

	for (u_int u = 0; u < AVP_Message_SplitSquad::MAX_SQUAD_MEMBERS; ++u)
	{
		if (s_pxSplitSquadMessage->m_auSplitMembers[u] == uGuid)
		{
			bExists = true;
			break;
		}
	}

	if (!bExists)
	{
		s_pxSplitSquadMessage->m_auSplitMembers[s_uSquadMemberIndex] = uGuid;
		RefreshMessageOverlay();
	}
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::ClearSplitSquadMember(int iArrayIndex)
{
	if (s_pxSplitSquadMessage)
	{
		s_pxSplitSquadMessage->m_auSplitMembers[iArrayIndex] = ASURA_GUID_UNREGISTERED;
		RefreshMessageOverlay();
	}
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
bool ToolsAVP_SplitSquad_ChooseObject::CanBeSelected(Tools_Object* pxObject)
{
	AVP_Message_SplitSquad* pxSplitMessage = ToolsAVP_Message_Editing::GetSplitSquadMessage();

	if (pxSplitMessage->m_uSquadToAddTo == ASURA_GUID_UNREGISTERED)
	{
		Tools_Timed_Message_System::AddMessage(Tools_Timed_Message_System::COLOUR_ERROR, "ERROR : Please Select a Squad To Send To Before Selecting Marines");
		return false;
	}
	else
	{
		if (ToolsAVP_Message_Editing::GetSplitSquadSentToGuid() != ASURA_GUID_UNREGISTERED)
		{
			Asura_Guid uCanBeSelectedGuid = pxObject->GetGuid();

			// Make sure we haven't already selected the member
			for (u_int u = 0; u < AVP_Message_SplitSquad::MAX_SQUAD_MEMBERS; ++u)
			{
				if (pxSplitMessage->m_auSplitMembers[u] == uCanBeSelectedGuid)
				{
					return false;
				}
			}

			for (Asura_Hashed_List_It<AVP_ServerActor_Human> xIt(AVP_ServerActor_Human::GetList()); !xIt.Done(); xIt.Next())
			{
				AVP_ServerActor_Human* pxHuman = xIt.GetCurrent();

				if (pxHuman && pxHuman->GetInitialSquadGuid() == ToolsAVP_Message_Editing::GetSplitSquadSentToGuid())
				{
					return true;
				}
			}
		}
	}

	return false;
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SelectPatrolGuardZone(void)
{
	ToolsAVP_ChooseObject_ByType* pxChooseObject = new ToolsAVP_ChooseObject_ByType(TOT_Entity_GuardZone, SetGuardZone_RequestPatrolMessage);
	Tools_Object_System::ChooseObject("Select The Guard Zone Object", pxChooseObject);
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::SetGuardZone_RequestPatrolMessage(Asura_Guid uGuid)
{
	if (!s_pxRequestPatrolMessage)
		return;

	s_pxRequestPatrolMessage->m_uGuardZoneGuid = uGuid;

}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
void ToolsAVP_Message_Editing::ClearPatrolGuardZone(void)
{
	if (!s_pxRequestPatrolMessage)	
		return;

	s_pxRequestPatrolMessage->m_uGuardZoneGuid = ASURA_GUID_UNREGISTERED;
}

void ToolsAVP_Message_Editing::SetSpawnBlueprint( int iData )
{
	AVP_Msg_Spawn* pxData = reinterpret_cast< AVP_Msg_Spawn* >( iData );

	Asura_Hash_ID uBlueprintHashID     = Asura_GetHashID( "Actor" );
	Asura_Hash_ID uBlueprintTypeHashID = uAVP_BLUEPRINT_TYPE_ACTORS;

	BlueprintSelect_Overlay xBPOverlay( "Choose A Blueprint", &uBlueprintHashID, &uBlueprintTypeHashID );
	if ( !xBPOverlay.DoModal() ) { return; }

	pxData->m_uBPTypeHashID = uBlueprintTypeHashID;
	pxData->m_uBPHashID		= uBlueprintHashID;

	(static_cast<ToolsAVP_Message_Editing*>(s_pxThis))->AVP_SetModifiedAndUpdateMessageOverlay();
}

void ToolsAVP_Message_Editing::ResetSpawnBlueprint( int iData )
{
	AVP_Msg_Spawn* pxData = reinterpret_cast< AVP_Msg_Spawn* >( iData );

	pxData->m_uBPTypeHashID = ASURA_HASH_ID_UNSET;
	pxData->m_uBPHashID		= ASURA_HASH_ID_UNSET;

	(static_cast<ToolsAVP_Message_Editing*>(s_pxThis))->AVP_SetModifiedAndUpdateMessageOverlay();
}

void ToolsAVP_Message_Editing::SetAlienSquadBehaviour( int iThis )
{
	if ( !s_pxSetAlienSquadBehaviourMsg )
	{
		return;
	}

	AVP_AlienBehaviourData xBehaviourData;
	xBehaviourData.m_iBehaviour = s_pxSetAlienSquadBehaviourMsg->m_iBehaviour;
	xBehaviourData.SetParamsFromBehaviour();
	s_pxSetAlienSquadBehaviourMsg->m_iMoveInMode			= xBehaviourData.m_iMoveInMode;
	s_pxSetAlienSquadBehaviourMsg->m_uProtoBlueprintHashID	= xBehaviourData.m_uProtoBlueprintHashID;
	s_pxSetAlienSquadBehaviourMsg->m_fJumpFraction			= xBehaviourData.m_fJumpFraction;
	s_pxSetAlienSquadBehaviourMsg->m_fStandFraction			= xBehaviourData.m_fStandFraction;
	s_pxSetAlienSquadBehaviourMsg->m_fMinTimeBetweenGrapples= xBehaviourData.m_fMinTimeBetweenGrapples;
	s_pxSetAlienSquadBehaviourMsg->m_fDamageFromBehindMultiplier= xBehaviourData.m_fDamageFromBehindMultiplier;
	s_pxSetAlienSquadBehaviourMsg->m_fIdealDistanceMin		= xBehaviourData.m_fIdealDistanceMin;
	s_pxSetAlienSquadBehaviourMsg->m_fIdealDistanceMax		= xBehaviourData.m_fIdealDistanceMax;
	s_pxSetAlienSquadBehaviourMsg->m_fAttackDelay			= xBehaviourData.m_fAttackDelay;
	s_pxSetAlienSquadBehaviourMsg->m_iMaxAttackersPerTarget = xBehaviourData.m_iMaxAttackersPerTarget;
	s_pxSetAlienSquadBehaviourMsg->m_iMaxAttackersTotal		= xBehaviourData.m_iMaxAttackersTotal;
	s_pxSetAlienSquadBehaviourMsg->m_iMaxBurstPerTarget		= xBehaviourData.m_iMaxBurstPerTarget;
	s_pxSetAlienSquadBehaviourMsg->m_iPostAttackAction		= xBehaviourData.m_iPostAttackAction;
	s_pxSetAlienSquadBehaviourMsg->m_fRespawnDelay			= xBehaviourData.m_fRespawnDelay;
	s_pxSetAlienSquadBehaviourMsg->m_fTimeBetweenAttacks	= xBehaviourData.m_fTimeBetweenAttacks;
	s_pxSetAlienSquadBehaviourMsg->m_uMaxNumberOfChargers	= xBehaviourData.m_uMaxNumberOfChargers;
	s_pxSetAlienSquadBehaviourMsg->m_bAttackIfInMarineLight		= xBehaviourData.m_bAttackIfInMarineLight;
	s_pxSetAlienSquadBehaviourMsg->m_fTimeInLightBeforeAttack = xBehaviourData.m_fTimeInLightBeforeAttack;
	s_pxSetAlienSquadBehaviourMsg->m_fMinHideDistance		= xBehaviourData.m_fMinHideDistance;
	s_pxSetAlienSquadBehaviourMsg->m_fChaneToRehideIfCloseFraction	= xBehaviourData.m_fChaneToRehideIfCloseFraction;
	s_pxSetAlienSquadBehaviourMsg->m_fChaneToRehideIfInLightFraction = xBehaviourData.m_fChaneToRehideIfInLightFraction;
	s_pxSetAlienSquadBehaviourMsg->m_fChaneToRehideAfterAttackFraction = xBehaviourData.m_fChaneToRehideAfterAttackFraction;

	// Becuase we're updating the message overlay we have to mark the message data as modified or it'll get forgotten.
	ToolsAVP_Message_Editing* pxThis = reinterpret_cast< ToolsAVP_Message_Editing* >( iThis );
	pxThis->AVP_SetModifiedAndUpdateMessageOverlay();
}

void ToolsAVP_Message_Editing::SetFacehuggerSquadBehaviour( int iThis )
{
	if ( !s_pxSetFaceHuggerSquadBehaviourMsg )
	{
		return;
	}

	// Becuase we're updating the message overlay we have to mark the message data as modified or it'll get forgotten.
	ToolsAVP_Message_Editing* pxThis = reinterpret_cast< ToolsAVP_Message_Editing* >( iThis );
	pxThis->AVP_SetModifiedAndUpdateMessageOverlay();
}




void ToolsAVP_Message_Editing::SetAITargetPriority( int iData )
{
	if ( !s_pxSetAITargetPrioritiesData )
	{
		return;
	}

	Asura_StaticMessage* pxMessage = Tools_MessageEditor::GetSelectedStaticMessage();
	if ( !pxMessage || pxMessage->m_pData != s_pxSetAITargetPrioritiesData )
	{
		Asura_Assert( false, "ToolsAVP_Message_Editing::SetAITargetPriority - wrong message" );
		return;
	}

	const AVP_Msg_SetAITargetPriorities::TPData* pxOldTPData = s_pxSetAITargetPrioritiesData->GetFirstTPDataPtr();
	const u_int uOldNumEntries = s_pxSetAITargetPrioritiesData->m_uNumEntries;

	AVP_Msg_SetAITargetPriorities::TPData* pxTPDataToEdit = &s_pxSetAITargetPrioritiesData->GetFirstTPDataPtr()[ iData ];
	if ( pxTPDataToEdit->m_iPriority == AVP_AI_TargetPriorityData::UNSET )
	{
		// This has just been cleared - we should now remove it, unless it's the last entry.
		if ( static_cast< u_int >( iData ) < uOldNumEntries - 1 )
		{
			AVP_Msg_SetAITargetPriorities::TPData* pxNewTPData = new AVP_Msg_SetAITargetPriorities::TPData[ uOldNumEntries - 1 ];
			for ( u_int uRead = 0, uWrite = 0; uRead < uOldNumEntries; ++uRead )
			{
				if ( &pxOldTPData[ uRead ] != pxTPDataToEdit )
				{
					pxNewTPData[ uWrite++ ] = pxOldTPData[ uRead ];
				}
			}

			pxMessage->DeallocateMessageData();
			const size_t uDataSize = sizeof( AVP_Msg_SetAITargetPriorities ) + ( uOldNumEntries - 1 ) * sizeof( AVP_Msg_SetAITargetPriorities::TPData );
			pxMessage->AllocateMessageData( uDataSize, true );

			s_pxSetAITargetPrioritiesData = static_cast< AVP_Msg_SetAITargetPriorities* >( pxMessage->m_pData );
			s_pxSetAITargetPrioritiesData->Initialise();
			s_pxSetAITargetPrioritiesData->m_uNumEntries = uOldNumEntries - 1;
			for ( u_int u = 0; u < s_pxSetAITargetPrioritiesData->m_uNumEntries; ++u )
			{
				s_pxSetAITargetPrioritiesData->GetFirstTPDataPtr()[ u ] = pxNewTPData[ u ];
			}
			delete [] pxNewTPData;
		}
	}
	else if ( pxOldTPData[ uOldNumEntries - 1 ].m_iPriority != AVP_AI_TargetPriorityData::UNSET )
	{
		// The last entry has been used up - we should add a new blank TPData at the end of the message.
		AVP_Msg_SetAITargetPriorities::TPData* pxNewTPData = new AVP_Msg_SetAITargetPriorities::TPData[ uOldNumEntries + 1 ];
		for ( u_int u = 0; u < uOldNumEntries; ++u )
		{
			pxNewTPData[ u ] = pxOldTPData[ u ];
		}

		pxMessage->DeallocateMessageData();
		const size_t uDataSize = sizeof( AVP_Msg_SetAITargetPriorities ) + ( uOldNumEntries + 1 ) * sizeof( AVP_Msg_SetAITargetPriorities::TPData );
		pxMessage->AllocateMessageData( uDataSize, true );

		s_pxSetAITargetPrioritiesData = static_cast< AVP_Msg_SetAITargetPriorities* >( pxMessage->m_pData );
		s_pxSetAITargetPrioritiesData->Initialise();
		s_pxSetAITargetPrioritiesData->m_uNumEntries = uOldNumEntries + 1;
		for ( u_int u = 0; u < uOldNumEntries; ++u )
		{
			s_pxSetAITargetPrioritiesData->GetFirstTPDataPtr()[ u ] = pxNewTPData[ u ];
		}
		s_pxSetAITargetPrioritiesData->GetFirstTPDataPtr()[ uOldNumEntries ].m_iPriority = AVP_AI_TargetPriorityData::UNSET;
		delete [] pxNewTPData;
	}

	(static_cast<ToolsAVP_Message_Editing*>(s_pxThis))->AVP_SetModifiedAndUpdateMessageOverlay();
}

void ToolsAVP_Message_Editing::SetAITargetType( int iData )
{
	if ( !s_pxSetAITargetPrioritiesData )
	{
		return;
	}

	AVP_Msg_SetAITargetPriorities::TPData* pxTPDataToEdit = &s_pxSetAITargetPrioritiesData->GetFirstTPDataPtr()[ iData ];
	if ( pxTPDataToEdit->m_iTargetType == AVP_AI_TargetPriorityData::SPECIFIC_ENTITY )
	{
		pxTPDataToEdit->m_uSpecificGuid = ASURA_GUID_UNREGISTERED;
	}

	(static_cast<ToolsAVP_Message_Editing*>(s_pxThis))->AVP_SetModifiedAndUpdateMessageOverlay();
}

void ToolsAVP_Message_Editing::ChooseAITargetEntity( int iData )
{
	if ( !s_pxSetAITargetPrioritiesData )
	{
		return;
	}

	AVP_Msg_SetAITargetPriorities::TPData* pxTPDataToEdit = &s_pxSetAITargetPrioritiesData->GetFirstTPDataPtr()[ iData ];

	ToolsAVP_ChooseActorOrSquad* pxChooseObject = new ToolsAVP_ChooseActorOrSquad( &pxTPDataToEdit->m_uSpecificGuid );
	pxChooseObject->SetCallbackFn( ChosenAITargetEntity, iData );
	Tools_Object_System::ChooseObject( "Select An Actor Or A Squad", pxChooseObject );

	(static_cast<ToolsAVP_Message_Editing*>(s_pxThis))->AVP_SetModifiedAndUpdateMessageOverlay();
}

void ToolsAVP_Message_Editing::ChosenAITargetEntity( int iData )
{
	if ( !s_pxSetAITargetPrioritiesData )
	{
		return;
	}

	AVP_Msg_SetAITargetPriorities::TPData* pxTPDataToEdit = &s_pxSetAITargetPrioritiesData->GetFirstTPDataPtr()[ iData ];
	if ( AVP_ServerEntity_Actor::FindEntity( pxTPDataToEdit->m_uSpecificGuid ) )
	{
		pxTPDataToEdit->m_iSpecificGuidType = AVP_AI_TargetPriorityData::ACTOR;
	} 
	else if ( AVP_ServerEntity_Squad::FindEntity( pxTPDataToEdit->m_uSpecificGuid ) )
	{
		pxTPDataToEdit->m_iSpecificGuidType = AVP_AI_TargetPriorityData::SQUAD;
	}

	(static_cast<ToolsAVP_Message_Editing*>(s_pxThis))->AVP_SetModifiedAndUpdateMessageOverlay();
}

void ToolsAVP_Message_Editing::OnSelectShakeObject(Tools_Object* pxObject)
{
	if(s_pxCameraShakeMessage)
	{		
		s_pxCameraShakeMessage->m_uSourceGuid = pxObject->GetGuid();
		SetObjectNameString( s_xObjectNameString, pxObject->GetGuid() );
	}
}

void ToolsAVP_Message_Editing::ChooseShakeObject(void)
{
	Tools_Prompted_Selection::ChooseObject( "Select Object which is the epicenter of the camera shake", OnSelectShakeObject );
}


void ToolsAVP_Message_Editing::ChooseShakeBlueprint(int pShake)
{
	Asura_Hash_ID uBPHash = 0x7BD0BB01; //"CameraShake"
	Asura_Hash_ID uBPType = uAVP_BLUEPRINT_TYPE_PLAYERS;
	BlueprintSelect_Overlay xBPSelOvrly( "Select a Camera Shake", &uBPHash, &uBPType, 1, &uBPType, 1, &uBPHash );

	if (xBPSelOvrly.DoModal() && uBPType == uAVP_BLUEPRINT_TYPE_PLAYERS)
	{
		int* puShake = reinterpret_cast<int*>(pShake);
		*puShake = uBPHash;

		Tools_Message_Editing::UpdateMessageOverlay();
	}
	else
	{
		// This should be impossible but might as well check
		Asura_Assert(uBPType == uAVP_BLUEPRINT_TYPE_PLAYERS, "You must select a Camera Shake Blueprint");
	}
}

void ToolsAVP_Message_Editing::SetObjectNameString( Asura_String &xNameString, const Asura_Guid uGuid )
{
	if ( uGuid == ASURA_GUID_UNREGISTERED )
	{
		xNameString = "*Unset*";
	}
	else
	{
		Asura_Entity_Server* pxPhyObj = Asura_Entity_Server::FindEntity( uGuid );
		if ( !pxPhyObj )
		{
			// Check if we can get a tools object from it.
			Tools_Object *pxToolsObject = Tools_Object_System::GetObjectFromGuid( uGuid );
			if ( pxToolsObject )
			{
				xNameString = pxToolsObject->GetName();
			}
			else
			{
				xNameString = "*Not Found*";
			}
		}
		else if ( pxPhyObj->ToolsData )
		{
			xNameString = pxPhyObj->ToolsData->Name;
		}
		else
		{
			static char s_szNumberString[ 16 ];
			sprintf( s_szNumberString, "%u", uGuid );
			xNameString = s_szNumberString;
		}
	}
}
