#include "Core/Asura.h"
#include "Cutscene/Tools_Cutscene_Event.h"

#include "Animation/Asura_Hierarchy.h"
#include "Animation/Asura_Animation.h"
#include "Core/Asura_HashedLocalisedText.h"
#include "Core/Asura_TiledTextureManagement.h"
#include "Cutscene/Asura_Cutscene.h"
#include "Cutscene/Asura_Cutscene_Event.h"
#include "Cutscene/Asura_Cutscene_Actor.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "GUIMenu/Asura_GUIMenu_Menu_Page.h"
#include "GUIMenu/Asura_GUIMenu_System.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Graph_Dialog.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Rendering/Asura_Direct3D_HDR.h"
#include "Rendering/Asura_Text_Output.h"
#include "Sound/Asura_Audio_System.h"
#include "Sound/Asura_Sound_Mixer.h"

#include "Tools_Overlays.h"
#include "Tools_System.h"
#include "Cutscene/Tools_Mode_Cutscene.h"
#include "Cutscene/Tools_Cutscene_Attachment.h"
#include "Cutscene/Tools_CutsceneActor_Object.h"
#include "Cutscene/Tools_Cutscene_Timeline.h"
#include "Cutscene/Tools_Cutscene_Object.h"
#include "Cutscene/Tools_Cutscene_Dialog.h"
#include "Cutscene/Tools_Cutscene_Subtitles.h"
#include "Entity/Tools_MessageEditor.h"
#ifdef ASURA_USE_FACIAL_ANIMATION_SYSTEM
#include "FacialAnimation/Tools_FacialAnimation_System.h"
#include "FacialAnimation/Tools_FacialAnimation_FaceAnimation.h"
#include "FacialAnimation/Dialog/Tools_Dialog_FacialAnimation_PickAnimation.h"
#endif
#include "Load&Save/Tools_SaveAsura.h"
#include "Misc/Tools_Flag_Inifile.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_PositionEdit_Object.h"
#include "Sound/Tools_PickSound.h"
#include "Sound/Tools_SoundMixer_Overlay.h"

//////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------**
** Tools_CutsceneEvent_Object **
**----------------------------*/

Tools_CutsceneEvent_Object::Tools_CutsceneEvent_Object(Tools_Cutscene_Event* event)
:Tools_Object(TOT_CutsceneEvent)
{

	Orientation.Identity();

	Asura_String mesh_name = "ToolsShape_CutsceneEvent";
	Mesh = Asura_Mesh_System::GetMesh(mesh_name);

	Event = event;
	Position = Event->Position;

}

Tools_CutsceneEvent_Object::~Tools_CutsceneEvent_Object()
{
	if(Event)
	{
		Event->EventObject = 0;
	}
}

const char* Tools_CutsceneEvent_Object::GetName()
{
	Asura_Assert(Event,"");
	char buffer[250];
	if(Event->Event->m_xName.IsBlank())
	{
		sprintf(buffer,"%.3f %s",Event->Event->m_fTriggerTime,Event->GetTypeDescription());

		Name = buffer;
	}
	else
	{
		Name = Event->Event->m_xName;
	}
	return Name.GetString();
}

void Tools_CutsceneEvent_Object::OnSelection()
{
	Asura_Assert(Event,"");
	Event->CutsceneObject->SelectEvent(Event);
}

void Tools_CutsceneEvent_Object::OnDeselection()
{
	Asura_Assert(Event,"");
	if(Event->CutsceneObject->SelectedEvent == Event)
	{
		Event->CutsceneObject->SelectEvent(0);
	}
}

void Tools_CutsceneEvent_Object::Destroy()
{
	if(Event)
	{
		//destroying the event will cause this object to be deleted as well
		Event->Destroy();
		return;
	}
}

bool Tools_CutsceneEvent_Object::CanBeDestroyed()
{
	if (Event)
	{
		return Event->IsEditable();
	}
	return true;
}

void Tools_CutsceneEvent_Object::UpdatePositionInAsuraData(bool bUpdateLinkedObjects)
{
	if(Position != Event->Position)
	{
		Event->Position = Position;
		Event->Event->m_xPosition = Position;
	}
}


////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------**
** Tools_Cutscene_Event **
**----------------------*/
Asura_Vector_2 Tools_Cutscene_Event::OverlayPosition(10,10);
Tools_Cutscene_Event* Tools_Cutscene_Event::ThisEvent = 0;
int Tools_Cutscene_Event::NextEventID = 0;

int Tools_Cutscene_Event::s_iEventListSize = 0;
char** Tools_Cutscene_Event::s_pszEventListStrings = 0;
int* Tools_Cutscene_Event::s_piEventListConversion = 0;
int Tools_Cutscene_Event::s_iEventListIndex = 0;
bool Tools_Cutscene_Event::s_bEventListForCurrentActorOnly = true;


Tools_Cutscene_Event::Tools_Cutscene_Event(Asura_Cutscene_Event* event,Tools_Cutscene_Object* cutsceneObject,int eventID)
{
	Event = event;
	CutsceneObject = cutsceneObject;
	Index = -1;
	Changed = false;
	m_uTimelineSelectionFlags = 0;
	EventID = (eventID<0) ? NextEventID++ : eventID;
	EventObject = new Tools_CutsceneEvent_Object(this);

	ParentEventID = -1;
	ParentEventIDBackup = -1;
	RelativeTime = 0;

	//add to the parent's list of events
	CutsceneObject->EventList.add_entry(this);
}

Tools_Cutscene_Event::~Tools_Cutscene_Event()
{
	CutsceneObject->EventList.delete_entry(this);

	if(EventObject)
	{
		Tools_Object_System::DeleteObject(EventObject);
		EventObject = 0;
	}
}

void Tools_Cutscene_Event::Destroy()
{
	Tools_Mode_Cutscene::s_pxThis->RefreshPlayback();

	if(CutsceneObject->SelectedEvent==this)
	{
		CutsceneObject->SelectEvent(0);
	}
	
	if(CutsceneUndo) CutsceneUndo->Log_Event_Changed(this);
	if(CutsceneUndo) CutsceneUndo->Log_Event_Destroyed(this);

	
	CutsceneObject->RemoveEventFromCutscene(Event);
	Event->PreDestructionCleanup(CutsceneObject->Cutscene);
	delete Event;
	Event = NULL;


	Tools_Cutscene_Object* cutsceneObject = CutsceneObject;
	delete this;
   	cutsceneObject->UpdateEventIndeces();
	cutsceneObject->UpdateParentEvents();

	if(cutsceneObject->SelectedEvent)
	{
		//update the overlay for the selected event
		cutsceneObject->SelectEvent(cutsceneObject->SelectedEvent);
	}

}

void Tools_Cutscene_Event::OnSelection()
{
	Asura_Assert(!ThisEvent,"");
	ThisEvent = this;
	CreateOverlay();

	Tools_Object_System::SetCommentText(Event->m_xComment.GetString(),this);
}

void Tools_Cutscene_Event::OnDeselection()
{
	Asura_Assert(ThisEvent,"");
	DeleteOverlay();
	ThisEvent = 0;

	if(!Event->m_xComment.IsBlank())
	{
		Tools_Object_System::ClearCommentText(this);
	}
}

void Tools_Cutscene_Event::CreateOverlay()
{

	char heading[100];
	sprintf(heading,"Event : %s",GetTypeDescription());

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay(heading,false,&OverlayPosition);
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	pxDialog = new Asura_Numerical_Dialog("Event To Edit",xOffset,&CutsceneObject->SelectedEventIndex,0,(float)(CutsceneObject->EventList.size()-1),1,"%.0f",&CutsceneObject->SelectedEventIndexChanged);
	pxDialog->SetHelpText("Index of event currently being edited (sorted by time).\nChange value to select a different event.");
	pxDialog->m_iControlID = DialogID_EventSelection;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	UpdateEventListStrings();
	if(s_iEventListSize)
	{
		pxDialog = new Asura_DropDown_List_Dialog(xOffset,s_pszEventListStrings,&s_iEventListIndex,10,s_iEventListSize,EventListSelectionChanged,NULL,NULL);
		pxDialog->SetHelpText("List of events that share the same target as this one.");
		pxDialog->m_iControlID = DialogID_EventList;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::LargeSpacing();
	}


	pxDialog = new Tools_CutsceneTime_Dialog("Time",xOffset,&Event->m_fTriggerTime,0,1000,0.1f,&Changed);
	pxDialog->SetEnabled(Event->m_xAnimationFlagActorName.IsBlank());
	pxDialog->SetHelpText("Time in seconds at which event is triggered.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	char text[200];
	Asura_PrintfToBfr(text,"Name : %s",200,true,Event->m_xName.IsBlank() ? "<Not Set>" : Event->m_xName.GetString());
	
	pxDialog = new Asura_Function_Dialog(text,xOffset,_Rename,(int)this);
	pxDialog->SetHelpText("Change the name for this event.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Disabled",xOffset,&Event->m_uEventFlags,AsuraCutsceneEventFlag_Disabled,&Changed);
	pxDialog->SetHelpText("Disables this event for testing purposes.\nThis will prevent the event from being triggered.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
    

	//Tab control
	static char* s_apszTabs[2] = {"Event Specific","Common"};
	Asura_Tabs_Dialog* pxTabsDialog = new Asura_Tabs_Dialog(xOffset,s_apszTabs,2,25);
	pxOverlay->AddDialog(pxTabsDialog);
	xOffset.y = pxTabsDialog->m_xMaxCoord.y + 1.0f;

	//redirect all subsequent dialogs to the tab dialog
	pxOverlay->SetCurrentParentDialog(pxTabsDialog);	
	

	float fTabsStartY = xOffset.y;
	float fTabsEndY = xOffset.y;


	//Common Tab
	pxTabsDialog->SelectTab(1);
    		

	pxDialog = new Asura_Function_Dialog("Edit Comment",xOffset,_EditComment,(int)this);
	pxDialog->SetHelpText("Edit the tools comment text for this event.");
  	pxOverlay->AddDialog(pxDialog);


	xOffset.x += 12.0f;
	pxDialog = new Asura_Tickbox_Dialog("Show",xOffset,&Tools_Object_System::s_bShowComment);
	pxDialog->SetHelpText("Displays the comment (if any) in the corner of the screen");
  	pxOverlay->AddDialog(pxDialog);
	xOffset.x -= 12.0f;

  	xOffset.y+=Asura_Dialog::DefaultSpacing();


	// (AW - 31.05.2004) - Add dialog for animation flag stuff.

	//add actor selection control
	if (Event->m_xAnimationFlagActorName.GetLength())
		sprintf(text,"Anim Flag Actor : %s",Event->m_xAnimationFlagActorName.GetString());
	else
		strcpy(text,"Trigger Event on Animation Flag");

	pxDialog = new Asura_Function_Dialog(text,xOffset,ChooseAnimFlagActor);
	pxDialog->SetHelpText("Select actor to check for animation flags.");
	pxDialog->m_iControlID = DialogID_AnimFlagActor;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Remove Anim Flag Actor", xOffset, RemoveAnimFlagActor);
	pxDialog->SetEnabled(!Event->m_xAnimationFlagActorName.IsBlank());
	pxDialog->SetHelpText("Remove the actor to check for animation flags.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	

	Tools_Flag_List* pxAnimFlagList = Tools_System::CommonFlagIni->GetFlagList("Keyframe Flags");	
	const char *szFlagName = NULL;
	if (pxAnimFlagList)
	{
		szFlagName = pxAnimFlagList->GetFlagName(Event->m_iAnimationFlag);
	}
	if (szFlagName)
	{
		sprintf(text,"Animation Flag : %s", szFlagName);
	}
	else
	{
		strcpy(text,"Animation Flag : Invalid");
	}

	pxDialog = new Asura_Function_Dialog(text,xOffset,SetAnimFlag);
	pxDialog->SetEnabled(!Event->m_xAnimationFlagActorName.IsBlank());
	pxDialog->SetHelpText("Select animation to look for to trigger this event.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Tools_CutsceneTime_Dialog("Retrigger Delay",xOffset,&Event->m_fTriggerTime,0,1000,0.1f,&Changed);
	pxDialog->SetEnabled(!Event->m_xAnimationFlagActorName.IsBlank());
	pxDialog->SetHelpText("Time in seconds to wait before retriggering the event.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Ignore when advancing time",xOffset,&Event->m_uEventFlags,AsuraCutsceneEventFlag_IgnoreWhenAdvancingToTime,&Changed);
	pxDialog->SetHelpText("Causes this event to be skipped when fastforwarding to a point in the cutscene , or when skipping to the end of the cutscene");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Enabled ID",xOffset,&Event->m_iEnableID,0,32,1,"%d",&Changed);
	pxDialog->SetHelpText("ID of this event for the purposes of enabling/disabling it from outside trigger messages.\nZero is equivalent to having no ID , meaning the event cannot be affected by such messages.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//End of common tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;
	
	//Event specific tab
	pxTabsDialog->SelectTab(0);
	xOffset.y = fTabsStartY;

	//add the event specific controls
	AddOverlayControls(pxOverlay,xOffset);

	//End of event specific tab
	if(xOffset.y > fTabsEndY) fTabsEndY = xOffset.y;

	//End of tabs
	pxOverlay->SetCurrentParentDialog(0);
	static u_int s_uSelectedTab = 0;
	pxTabsDialog->SetSelectedTabPointer(&s_uSelectedTab);
	xOffset.y = fTabsEndY + 1.0f;

	
	pxDialog = new Asura_Function_Dialog("Delete",xOffset,DestroyEvent);
	pxDialog->SetHelpText("Delete this event.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Duplicate",xOffset,_CopyEvent,(int)this);
	pxDialog->SetHelpText("Creates and selects a copy of this event.");
  	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

    // (MH - 17.06.2005) if this event is not editable, disable all the controls, except the event selection.
	if (!IsEditable())
	{
		Asura_Dialog* pxDialogToDisable = pxOverlay->m_pxDialogList;
		while (pxDialogToDisable)
		{
			if (pxDialogToDisable->m_iControlID != DialogID_EventSelection)
			{
				pxDialogToDisable->SetIsEnabledFunction(0);
				pxDialogToDisable->SetEnabled(false);
			}
			pxDialogToDisable = pxDialogToDisable->m_pxNextDialog;
		}
	}

	pxOverlay->m_uID = TOOLS_OVERLAY_CUTSCENE_EVENT;

	pxOverlay->m_xOrigin = OverlayPosition;
 	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);

}

void Tools_Cutscene_Event::ChooseActor()
{
	Asura_Assert(ThisEvent,"");

	ListSelection_Overlay overlay("Select Actor :");
	for(LIF<Tools_CutsceneActor_Object*> alif(&ThisEvent->CutsceneObject->ActorObjects);!alif.done();alif.next())
	{
		//if(!alif()->IsHierarchyActor()) continue;
        if(!ThisEvent->ActorIsValidTarget(alif()->Actor)) continue;
		overlay.AddItem(alif()->Actor->ActorName.GetString(),(int)alif()->Actor);
	}
	if(ThisEvent->AllowNoTarget())
	{
		overlay.AddItem(ThisEvent->GetNoTargetString(),0);
	}


	if(!overlay.GetNumItems())
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"There aren't any actors which can be affected by this event.");
		return;
	}

	if(!overlay.DoModal()) return;

    ThisEvent->SetTargetActor( reinterpret_cast<Asura_Cutscene_Actor*>(overlay.GetSelectionData()) );

    Tools_Mode_Cutscene::s_pxThis->RefreshPlayback();

	Asura_Overlay* event_overlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_CUTSCENE_EVENT);
	if(event_overlay)
	{
		//overlay will need updating
		ThisEvent->DeleteOverlay();
		ThisEvent->CreateOverlay();
	}
}

void Tools_Cutscene_Event::SetTargetActor( Asura_Cutscene_Actor* pxActor )
{
    if( pxActor )
    {
        Asura_Assert( ActorIsValidTarget(pxActor), "" );

	    Event->m_pxTargetActor    = pxActor;
        Event->m_xTargetActorName = pxActor->ActorName;
    }
    else
    {
	    Event->m_pxTargetActor    = NULL;
        Event->m_xTargetActorName = "";
    }

	UpdateEventListStrings();

    if( CutsceneUndo ) { CutsceneUndo->Log_Event_Changed( this ); }
}

void Tools_Cutscene_Event::ChooseAnimFlagActor()
{
	// (AW - 31.05.2004) - This is a copy of the above function. This is bad code, but I'm in a rush.
	Asura_Assert(ThisEvent,"");
	Asura_Cutscene_Event* asuraEvent = (Asura_Cutscene_Event*) ThisEvent->Event;

	ListSelection_Overlay overlay("Select Anim Flag Actor :");
	for(LIF<Tools_CutsceneActor_Object*> alif(&ThisEvent->CutsceneObject->ActorObjects);!alif.done();alif.next())
	{
		if(!alif()->IsHierarchyActor()) continue;
		overlay.AddItem(alif()->Actor->ActorName.GetString(),(int)alif()->Actor);
	}

	if(!overlay.GetNumItems())
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"There aren't any actors which can be checked for anim flags.");
		return;
	}

	if(!overlay.DoModal()) return;

	Tools_Mode_Cutscene::s_pxThis->RefreshPlayback();

	asuraEvent->m_pxAnimationFlagActor = (Asura_Cutscene_Actor*)overlay.GetSelectionData();
	asuraEvent->m_xAnimationFlagActorName = asuraEvent->m_pxAnimationFlagActor->ActorName;
	asuraEvent->m_fTriggerTime = 0.0f;

	if(CutsceneUndo) CutsceneUndo->Log_Event_Changed(ThisEvent);

	ThisEvent->DeleteOverlay();
	ThisEvent->CreateOverlay();
}

void Tools_Cutscene_Event::RemoveAnimFlagActor()
{
	// (AW - 31.05.2004) - This is a copy of the above function. This is bad code, but I'm in a rush.
	Asura_Assert(ThisEvent,"");
	Asura_Cutscene_Event* asuraEvent = (Asura_Cutscene_Event*) ThisEvent->Event;

	Tools_Mode_Cutscene::s_pxThis->RefreshPlayback();

	asuraEvent->m_pxAnimationFlagActor = NULL;
	asuraEvent->m_xAnimationFlagActorName = "";

	if(CutsceneUndo) CutsceneUndo->Log_Event_Changed(ThisEvent);

	ThisEvent->DeleteOverlay();
	ThisEvent->CreateOverlay();
}


void Tools_Cutscene_Event::SetAnimFlag()
{
	Asura_Assert(ThisEvent,"");
	Asura_Cutscene_Event* asuraEvent = (Asura_Cutscene_Event*) ThisEvent->Event;

	Tools_Flag_List* pxAnimFlagList = Tools_System::CommonFlagIni->GetFlagList("Keyframe Flags");	
	
	if(!pxAnimFlagList)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find list of flag names");
		return;
	}

	//create an overlay for selecting the flags
	ListSelection_Overlay overlay("Select a flag");

	for(LIF<NameAndValue> flif(&pxAnimFlagList->ValueList);!flif.done();flif.next())
	{
		overlay.AddItem(flif().Name.GetString(),flif().Value);
	}
	
	if(!overlay.DoModal()) return;

	Tools_Mode_Cutscene::s_pxThis->RefreshPlayback();

	asuraEvent->m_iAnimationFlag = overlay.GetSelectionData();

	if(CutsceneUndo) CutsceneUndo->Log_Event_Changed(ThisEvent);

	ThisEvent->DeleteOverlay();
	ThisEvent->CreateOverlay();
}


void Tools_Cutscene_Event::DeleteOverlay()
{
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_CUTSCENE_EVENT);
}

void Tools_Cutscene_Event::DestroyEvent()
{
	Asura_Assert(ThisEvent,"");
	
	YesNo_Overlay dlg("Delete this event?");
	if(!dlg.DoModal()) return;

	//take a copy of ThisEvent since it may change
	Tools_Cutscene_Event* event = ThisEvent;
	
	//select a different event before destroying this one
	int numEvents = event->CutsceneObject->EventList.size();
	if(numEvents>1)
	{
		int index = event->Index+1;
		if(index>=numEvents) index = numEvents-2;			
		event->CutsceneObject->SelectEvent(event->CutsceneObject->EventList[index]);
	}
	

	event->Destroy();

	Tools_Timed_Message_System::AddMessage(0xffffff,"Event Deleted");
}

void Tools_Cutscene_Event::Update()
{
	if(Changed)
	{
		if(Event->m_fTriggerTime!=TriggerTimeBackup)
		{
			CutsceneObject->UpdateEventsAfterTimeChange(this);
		}

		if(CutsceneUndo) CutsceneUndo->Log_Event_Changed(this);

		CutsceneObject->m_bEventSortRequired = true;
	
		Tools_Mode_Cutscene::s_pxThis->RefreshPlayback();
		Tools_SaveAsura::SetCutscenesModified();
		Changed = false;
	}
}

void Tools_Cutscene_Event::UpdatePosition()
{
	if(Event->m_xPosition.x || Event->m_xPosition.y || Event->m_xPosition.z)
	{
		//use the user set position
		Position = Event->m_xPosition;
	}
	else
	{
	 	//default position is near the cutscene object
		Position = CutsceneObject->Position;
		Position.y-=1.0f;	

		Tools_CutsceneActor_Object* actorObject = GetActorObject(Event->m_xTargetActorName);
		if(actorObject)
		{
			//if this is attached to an actor , get the position for the actor at the time of the event
			actorObject->GetPositionAtTime(Event->m_fTriggerTime,&Position);
			Position.y -= 0.5f;
		}
	}

	//update position in event object
	Asura_Assert(EventObject,"");
	EventObject->Position = Position;

}

void Tools_Cutscene_Event::FillInTimelineData(Tools_Timeline_Data_Event* data)
{
	data->Time = Event->m_fTriggerTime;
}

void Tools_Cutscene_Event::UpdateBackupVariables()
{
	TriggerTimeBackup = Event->m_fTriggerTime;
	TargetActorNameBackup = Event->m_xTargetActorName;
	ParentEventIDBackup = ParentEventID;
	m_uEventFlagsBackup = Event->m_uEventFlags;
	m_iEnableIDBackup = Event->m_iEnableID;
	m_xNameBackup = Event->m_xName;//For overiding the name displayed in tools
	m_xCommentBackup = Event->m_xComment;//Optional comment for use in tools
	m_xPositionBackup = Event->m_xPosition;//For overiding the event object's position
	m_xNameOfSourceTemplateBackup = Event->m_xNameOfSourceTemplate;//if this event belongs to a template, template event's name is here.
	m_xNameOfEventWrapperBackup = Event->m_xNameOfEventWrapper;//if this event belongs to a template, wrapper event name is here.
}

const char* Tools_Cutscene_Event::GetNameOfChangedVariable()
{
	if(Event->m_fTriggerTime != TriggerTimeBackup) return "Trigger Time";
	if((Event->m_xTargetActorName.IsBlank() != TargetActorNameBackup.IsBlank()) ||
		(!TargetActorNameBackup.IsBlank() && Event->m_xTargetActorName.Compare(TargetActorNameBackup)))
	{
		return "Target";
	}
	
	if(Event->m_iEnableID != m_iEnableIDBackup) return "Enabled ID";

	if(Event->m_uEventFlags != m_uEventFlagsBackup)
	{
		u_int uChangedFlags = Event->m_uEventFlags ^ m_uEventFlagsBackup;
		if(uChangedFlags & AsuraCutsceneEventFlag_IgnoreWhenAdvancingToTime) return "Ignore When Advancing Time";
		if(uChangedFlags & AsuraCutsceneEventFlag_Disabled) return "Disabled";
		return "Flags";
	}

	return 0;
}

void Tools_Cutscene_Event::PrintDescription(float x,float y)
{
	Asura_String text = GetDescription();
	if(Event->m_xName.IsBlank())
	{
		Asura_Text_Output::Print(x,y,0xffffffff,0,"%s",text.GetString());
	}
	else
	{
		Asura_Text_Output::Print(x,y,0xffffffff,0,"%s : %s",Event->m_xName.GetString(),text.GetString());
	}
}

void Tools_Cutscene_Event::UpdateEventIDsFromIndeces()
{
	if(Event->m_iParentEventIndex>=0)
	{
		Asura_Assert(Event->m_iParentEventIndex<CutsceneObject->EventList.size(),"");
		Tools_Cutscene_Event* parentEvent = CutsceneObject->EventList[Event->m_iParentEventIndex];
		ParentEventID = parentEvent->EventID;
		ParentEventIDBackup = ParentEventID;

		RelativeTime = Event->m_fTriggerTime - parentEvent->Event->m_fTriggerTime;
	}
	else
	{
		ParentEventID = -1;
		ParentEventIDBackup = -1;
	}
}

void Tools_Cutscene_Event::UpdateEventIndecesFromIDs()
{
	Tools_Cutscene_Event* parentEvent = GetToolsCutsceneEvent(ParentEventID);
	if(parentEvent)
	{
		Event->m_iParentEventIndex = parentEvent->Index;
		RelativeTime = Event->m_fTriggerTime - parentEvent->Event->m_fTriggerTime;
	}
	else
	{
		Event->m_iParentEventIndex = -1;
	}
}

bool Tools_Cutscene_Event::ActorIsValidTarget(Asura_Cutscene_Actor* pxActor)
{
	return pxActor->IsHierarchyActor();
}

bool Tools_Cutscene_Event::IsEditable() const
{
    // (MH - 17.06.2005) currently only events which came from a template event are non-editable
	return (Event->m_xNameOfSourceTemplate.IsBlank());
}

void Tools_Cutscene_Event::Rename()
{
	Tools_Object_System::AskForName(Event->m_xName,"Rename event :",EventObject);
	DeleteOverlay();
	CreateOverlay();
}

void Tools_Cutscene_Event::EditComment()
{
	GetString_Overlay xOverlay("Comment Text :",Event->m_xComment.GetString(),1000);
	if(xOverlay.DoModal())
	{
		Event->m_xComment = xOverlay.GetString();
		Tools_Object_System::SetCommentText(Event->m_xComment.GetString(),this);
		Tools_SaveAsura::SetCutscenesModified();
	}
}

Asura_String Tools_Cutscene_Event::GetNameOrDescription()
{
	if(!Event->m_xName.IsBlank())
	{
		return Event->m_xName;
	}
	return GetDescription();
}

void Tools_Cutscene_Event::CopyEvent()
{
	CutsceneObject->CreateCutsceneEvent(false,-1,-1,Event);
}

void Tools_Cutscene_Event::UpdateEventListStrings()
{
	DeleteEventListStrings();
	if(!ThisEvent) return;

	//determine number of entries needed in list
	if(s_bEventListForCurrentActorOnly)
	{
		s_iEventListSize = 0; 
		for(LIF<Tools_Cutscene_Event*> xEventLIF(&ThisEvent->CutsceneObject->EventList);!xEventLIF.done();xEventLIF.next())
		{
			if(ThisEvent->Event->m_pxTargetActor == xEventLIF()->Event->m_pxTargetActor)
			{
				s_iEventListSize++;
			}
		}
	}
	else
	{
		s_iEventListSize = ThisEvent->CutsceneObject->EventList.size();
	}


	s_pszEventListStrings = new char*[s_iEventListSize];
	s_piEventListConversion = new int[s_iEventListSize];
	s_iEventListIndex = 0;

	//fill in the list of strings
	int iListIndex = 0;
	int iEventIndex = 0;
	for(LIF<Tools_Cutscene_Event*> xEventLIF(&ThisEvent->CutsceneObject->EventList);!xEventLIF.done();xEventLIF.next(),iEventIndex++)
	{
		if(!s_bEventListForCurrentActorOnly || xEventLIF()->Event->m_pxTargetActor == ThisEvent->Event->m_pxTargetActor)
		{
			if(xEventLIF() == ThisEvent)
			{
				//store index of current event
				s_iEventListIndex = iListIndex;
			}

			s_piEventListConversion[iListIndex] = iEventIndex;

			Asura_String xName = xEventLIF()->GetNameOrDescription();
			s_pszEventListStrings[iListIndex] = new char[xName.GetLength()+1];
			strcpy(s_pszEventListStrings[iListIndex],xName.GetString());

			iListIndex++;
			
		}
	}

	//If the overlay already exists , then update the existing drop down list
	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_CUTSCENE_EVENT);
	if(pxOverlay)
	{
		Asura_DropDown_List_Dialog* pxListDialog = (Asura_DropDown_List_Dialog*) pxOverlay->GetDialog(DialogID_EventList);
		if(pxListDialog)
		{
			pxListDialog->ReplaceStrings(s_iEventListSize,s_pszEventListStrings);
		}
	}

}

void Tools_Cutscene_Event::DeleteEventListStrings()
{
	for(int i=0;i<s_iEventListSize;i++)
	{
		delete [] s_pszEventListStrings[i];
	}
	delete [] s_pszEventListStrings;
	delete [] s_piEventListConversion;

	s_iEventListSize = 0;
	s_pszEventListStrings = 0;
	s_piEventListConversion = 0;
}

void Tools_Cutscene_Event::EventListSelectionChanged()
{
	if(!ThisEvent) return;

	//Change to the event selected from the list
	int iEventIndex = s_piEventListConversion[s_iEventListIndex];

	if(iEventIndex != ThisEvent->CutsceneObject->SelectedEventIndex)
	{
		ThisEvent->CutsceneObject->SelectEventByIndex(iEventIndex);
	}
}


////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------------**
** Tools_Cutscene_Event_AnimationChange **
**--------------------------------------*/

Tools_Cutscene_Event_AnimationChange::Tools_Cutscene_Event_AnimationChange(Asura_Cutscene_Event* event,Tools_Cutscene_Object* cutsceneObject,int eventID)
:Tools_Cutscene_Event(event,cutsceneObject,eventID)
{
}


void Tools_Cutscene_Event_AnimationChange::AddOverlayControls(class Asura_Overlay* newOverlay,Asura_Vector_2& offset)
{
	Asura_Cutscene_Event_AnimationChange* animEvent = (Asura_Cutscene_Event_AnimationChange*) Event;

	Asura_Dialog* newDialog;
	char text[200];


	//add actor selection control
	if(animEvent->m_xTargetActorName.GetLength())
		sprintf(text,"Actor : %s",animEvent->m_xTargetActorName.GetString());
	else
		strcpy(text,"Actor : *None*");

	newDialog = new Asura_Function_Dialog(text,offset,ChooseActor);
	newDialog->SetHelpText("Select actor whose animation should be changed.");
	newDialog->m_iControlID = DialogID_Actor;
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();

	//add animation selection control
	if (animEvent->m_iAnimationIndex != -1)
    {
	    Asura_String & AnimationName = CutsceneObject->Cutscene->AnimationNames.GetName(animEvent->m_iAnimationIndex);
		sprintf(text,"Anim : %s",AnimationName.GetString());
    }
    else
    {
        strcpy(text,"Anim : *None*");
    }

	newDialog = new Asura_Function_Dialog(text,offset,ChooseAnimation);
	newDialog->SetHelpText("Select animation to change to.");
	newDialog->m_iControlID = DialogID_Anim;
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();
		
	float x = offset.x;

	
	//add tickboxes for the flags
	newDialog = new Asura_FlagTickbox_Dialog("Loop",offset,&animEvent->m_iAnimationFlags,AsuraAnimationFlag_Loop,&Changed);
	newDialog->SetHelpText("Loop animation.");
	newOverlay->AddDialog(newDialog);
		
	offset.x += newDialog->m_xMaxCoord.x;
 	newDialog = new Asura_FlagTickbox_Dialog("Tween",offset,&animEvent->m_iAnimationFlags,AsuraAnimationFlag_Tween,&Changed);
	newDialog->SetHelpText("Tween animation in.");
	newDialog->SetIsEnabledFunction(IsTargetHierarchy);
	newOverlay->AddDialog(newDialog);
	offset.x = x;
	offset.y+=Asura_Dialog::DefaultSpacing();

	if(Event->m_iType == ACET_SecondaryAnimation)
	{
 		newDialog = new Asura_FlagTickbox_Dialog("Tween Out",offset,&animEvent->m_iAnimationFlags,AsuraAnimationFlag_TweenOut,&Changed);
		newDialog->SetHelpText("Tween animation out when finished.");
		newDialog->SetIsEnabledFunction(IsTargetHierarchy);
		newOverlay->AddDialog(newDialog);
		offset.y+=Asura_Dialog::DefaultSpacing();
	}

	newDialog = new Tools_CutsceneTime_Dialog("Tween Duration", offset, &animEvent->m_fTweenDuration, 0, 1000, 0.1f, &Changed);
	newDialog->SetHelpText("Time taken to tween animation in/out.");
	newDialog->SetIsEnabledFunction(IsTargetHierarchy);
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();
	

	if(animEvent->m_iAnimationFlags & (AsuraAnimationFlag_Tween|AsuraAnimationFlag_TweenOut))
	{
		newDialog = new Asura_FlagTickbox_Dialog("Don't Tween Root",offset,&animEvent->m_iAnimationFlags,AsuraAnimationFlag_DontTweenRoot,&Changed);
		newDialog->SetHelpText("Turns off tweening on the root section of the hierarchy (for animations that move the object from the pivot point).");
		newDialog->SetIsEnabledFunction(IsTargetHierarchy);
		newOverlay->AddDialog(newDialog);
		offset.y+=Asura_Dialog::DefaultSpacing();
	}

	newDialog = new Asura_FlagTickbox_Dialog("To Position",offset,&animEvent->m_iAnimationFlags,AnimationChangeFlag_ToPosition,&Changed);
	newDialog->SetHelpText("Plays animation to a specified point , instead of running to the end.\nAlso used for choosing a fixed number of loops to play.");
	newDialog->SetIsEnabledFunction(IsTargetHierarchy);
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();

	if(animEvent->m_iAnimationFlags & AnimationChangeFlag_ToPosition)
	{
		newDialog = new Asura_Numerical_Dialog("Start", offset, &animEvent->m_fStartTimer, 0, 1, 0.01f, "  %.3f  ", &Changed);
		newDialog->SetHelpText("Where to start the animation from (0 to 1).");
		newDialog->SetIsEnabledFunction(IsTargetHierarchy);
		newOverlay->AddDialog(newDialog);
		offset.y+=Asura_Dialog::DefaultSpacing();
	
		newDialog = new Asura_Numerical_Dialog("End", offset, &animEvent->m_fEndTimer, -1000.0f, 1000.0f, 0.01f, "%.3f", &Changed);
		newDialog->SetHelpText("Where to play the animation to.\nIf less than the Start position then animation will be reversed.\nUnits are complete loops of the animation.");
		newDialog->SetIsEnabledFunction(IsTargetHierarchy);
		newOverlay->AddDialog(newDialog);
		offset.y+=Asura_Dialog::DefaultSpacing();

	}
	else
	{
		newDialog = new Asura_FlagTickbox_Dialog("Reversed",offset,&animEvent->m_iAnimationFlags,AsuraAnimationFlag_Reversed,&Changed);
		newDialog->SetHelpText("Play animation backwards.");
		newDialog->SetIsEnabledFunction(IsTargetHierarchy);
		newOverlay->AddDialog(newDialog);
		offset.y+=Asura_Dialog::DefaultSpacing();
	}


	if(Event->m_iType != ACET_SecondaryAnimation)
	{
		newDialog = new Asura_FlagTickbox_Dialog("Ignore Movement",offset,&animEvent->m_iAnimationFlags,AnimationChangeFlag_IgnoreMovement,&Changed);
		newDialog->SetHelpText("Disables the use of any movement data stored in the animation.\nBy default animation movement gets used if there is no movement from the tracks.");
		newDialog->SetIsEnabledFunction(IsTargetHierarchy);
		newOverlay->AddDialog(newDialog);
		offset.y+=Asura_Dialog::DefaultSpacing();
	}

	newDialog = new Asura_FlagTickbox_Dialog("Mirror",offset,&animEvent->m_iAnimationFlags,AsuraAnimationFlag_Mirror,&Changed);
	newDialog->SetHelpText("Mirrors the animation. This requires project-specific implementation to work correctly.");
	newDialog->SetIsEnabledFunction(IsTargetHierarchy);
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();


	newDialog = new Asura_FlagTickbox_Dialog("Use Camera Target",offset,&animEvent->m_iAnimationFlags,AnimationChangeFlag_UseCameraTarget,&Changed);
	newDialog->SetHelpText("For camera animations.\nThis determines whether the camera target should still be used when calculating the initial orientation that the animation is applied to.");
	newDialog->SetIsEnabledFunction(IsTargetCamera);
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();

	newDialog = new Tools_CutsceneTime_Dialog("Anim Delay",offset,&animEvent->m_fAnimDelay,0,999.0f,0.01f,&Changed);
	newDialog->SetHelpText("Time delay before the animation should start playing.");
	newDialog->SetIsEnabledFunction(IsTargetHierarchy);
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();


}

void Tools_Cutscene_Event_AnimationChange::Update()
{
	Asura_Cutscene_Event_AnimationChange* pxAnimEvent = (Asura_Cutscene_Event_AnimationChange*) Event;

	if(Changed)
	{
		//if the ToPosition or Tween flag was changed we will need to re-create the overlay
		if((pxAnimEvent->m_iAnimationFlags^AnimationFlagsBackup)&(AnimationChangeFlag_ToPosition|AsuraAnimationFlag_Tween|AsuraAnimationFlag_TweenOut))
		{
			DeleteOverlay();
			CreateOverlay();
		}
	}

	Tools_Cutscene_Event::Update();
}





void Tools_Cutscene_Event_AnimationChange::ChooseAnimation()
{
	Asura_Assert(ThisEvent,"");
	Asura_Cutscene_Event_AnimationChange* animEvent = (Asura_Cutscene_Event_AnimationChange*) ThisEvent->Event;

	//make sure we have a valid target first
	if(!animEvent->m_xTargetActorName.GetLength())
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Please choose the actor first");
		return;
	}

	Tools_CutsceneActor_Object* actor = ThisEvent->CutsceneObject->GetActor(animEvent->m_xTargetActorName.GetString());
	if(!actor)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Actor '%s' doesn't exist",animEvent->m_xTargetActorName.GetString());
		return;
	}

    if (!actor->ChooseAnim(animEvent->m_iAnimationIndex)) return;

	if(CutsceneUndo) CutsceneUndo->Log_Event_Changed(ThisEvent);

	//update the overlay
	Asura_Overlay* overlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_CUTSCENE_EVENT);
	if(overlay)
	{
		char text[200];

		if(animEvent->m_iAnimationIndex!=-1)
		{
		    Asura_String & AnimationName = ThisEvent->CutsceneObject->Cutscene->AnimationNames.GetName(animEvent->m_iAnimationIndex);
			sprintf(text,"Anim : %s",AnimationName.GetString());
		}
		else
		{
			strcpy(text,"Anim : *None*");
		}
		
		overlay->UpdateDialogLabel(DialogID_Anim,text);
	}
}



void Tools_Cutscene_Event_AnimationChange::FillInTimelineData(Tools_Timeline_Data_Event* data)
{
	Asura_Cutscene_Event_AnimationChange* animEvent = (Asura_Cutscene_Event_AnimationChange*) Event;
	Tools_Cutscene_Event::FillInTimelineData(data);

	Asura_Cutscene_Actor* pxTargetActor = animEvent->m_pxTargetActor;
	
	if(!pxTargetActor)
	{
		data->IsOkay = false;
	}

	bool bLoop = false;
	bool bTween = false;
	bool bTweenOut = false;
	bool bToPos = false;

	bLoop = (animEvent->m_iAnimationFlags & AsuraAnimationFlag_Loop) ? true : false;

	if(pxTargetActor && pxTargetActor->IsHierarchyActor())
	{
		bTween = (animEvent->m_iAnimationFlags & AsuraAnimationFlag_Tween) ? true : false;
		bTweenOut = (animEvent->m_iAnimationFlags & AsuraAnimationFlag_TweenOut) ? true : false;
		bToPos = (animEvent->m_iAnimationFlags & AnimationChangeFlag_ToPosition) ? true : false;
	}
	
	if((!bLoop || bToPos) && animEvent->m_iAnimationIndex != -1)
	{
		//get the length of the animation
        Asura_String AnimationName = Tools_Mode_Cutscene::CurrentCutscene->Cutscene->AnimationNames.GetName(animEvent->m_iAnimationIndex);
		Asura_Hierarchy_Anim* anim = Asura_Animation_System::GetAnim(AnimationName.GetString());
		if(anim)
		{
			data->Duration = anim->Duration;
			if(bToPos)
			{
				data->Duration *= Asura_Maths::Abs(animEvent->m_fEndTimer-animEvent->m_fStartTimer);
			}
			if(!UsesAnimatedTween())
			{
				if(bTween) data->Duration += animEvent->m_fTweenDuration;
				if(bTweenOut) data->Duration += animEvent->m_fTweenDuration;
			}
			data->Duration += animEvent->m_fAnimDelay;
		}
	}
}

bool Tools_Cutscene_Event_AnimationChange::CanEditDuration() const
{
	Asura_Cutscene_Event_AnimationChange* pxAsuraEvent = (Asura_Cutscene_Event_AnimationChange*) Event;

	Asura_Cutscene_Actor* pxTargetActor = pxAsuraEvent->m_pxTargetActor;
	
	//Can only set the duration if abnimation to a specific position
	bool bToPos = false;
	if(pxTargetActor && pxTargetActor->IsHierarchyActor())
	{
		bToPos = (pxAsuraEvent->m_iAnimationFlags & AnimationChangeFlag_ToPosition) ? true : false;
	}

	return bToPos;
}

void Tools_Cutscene_Event_AnimationChange::SetTotalDuration(float fDuration)
{
	Asura_Cutscene_Event_AnimationChange* pxAsuraEvent = (Asura_Cutscene_Event_AnimationChange*) Event;

	Asura_Cutscene_Actor* pxTargetActor = pxAsuraEvent->m_pxTargetActor;

	bool bTween = false;
	bool bTweenOut = false;
	bool bToPos = false;
	if(pxTargetActor && pxTargetActor->IsHierarchyActor())
	{
		bTween = (pxAsuraEvent->m_iAnimationFlags & AsuraAnimationFlag_Tween) ? true : false;
		bTweenOut = (pxAsuraEvent->m_iAnimationFlags & AsuraAnimationFlag_TweenOut) ? true : false;
		bToPos = (pxAsuraEvent->m_iAnimationFlags & AnimationChangeFlag_ToPosition) ? true : false;
	}

	if(bToPos && pxAsuraEvent->m_iAnimationIndex != -1)
	{
		//get the length of the animation
        Asura_String AnimationName = Tools_Mode_Cutscene::CurrentCutscene->Cutscene->AnimationNames.GetName(pxAsuraEvent->m_iAnimationIndex);
		Asura_Hierarchy_Anim* pxAnim = Asura_Animation_System::GetAnim(AnimationName.GetString());
		if(pxAnim && pxAnim->Duration > 0.0f)
		{
			float fModifiedDuration = fDuration;
			//If using a non animated tween then reduce then remove the tween duration from the anim duration
			if(!UsesAnimatedTween())
			{
				if(bTween) fModifiedDuration -= pxAsuraEvent->m_fTweenDuration;
				if(bTweenOut) fModifiedDuration -= pxAsuraEvent->m_fTweenDuration;
			}
			fModifiedDuration -= pxAsuraEvent->m_fAnimDelay;
			if(fModifiedDuration < 0.0f) fModifiedDuration = 0.0f;
			//Determine how many loops of the animation is required
			float fParametricDuration = fModifiedDuration / pxAnim->Duration; 

			//Update the end position , keeping the same direction as before
			float fSign = pxAsuraEvent->m_fEndTimer >= pxAsuraEvent->m_fStartTimer ? 1.0f : -1.0f;
			pxAsuraEvent->m_fEndTimer = pxAsuraEvent->m_fStartTimer + (fSign*fParametricDuration);
		}
	}

}

Asura_String Tools_Cutscene_Event_AnimationChange::GetDescription()
{
	Asura_Cutscene_Event_AnimationChange* animEvent = (Asura_Cutscene_Event_AnimationChange*) Event;
	char buffer[250];

    if(animEvent->m_iAnimationIndex != -1)
	{
		Asura_String AnimationName = Tools_Mode_Cutscene::CurrentCutscene->Cutscene->AnimationNames.GetName(animEvent->m_iAnimationIndex);
		sprintf(buffer,"Animation Change : %s",AnimationName.GetString());
		
	}
	else
	{
		sprintf(buffer,"Animation Change : *None*");
	}
	return buffer;
}



void Tools_Cutscene_Event_AnimationChange::UpdateBackupVariables()
{
	Tools_Cutscene_Event::UpdateBackupVariables();
	Asura_Cutscene_Event_AnimationChange* asuraAnimEvent = (Asura_Cutscene_Event_AnimationChange*) Event;

	if(asuraAnimEvent->m_iAnimationIndex == -1)
	{
		AnimationNameBackup.MakeNull();
		AnimationFileNameBackup.MakeNull();
	}
	else
	{
		AnimationNameBackup = Tools_Mode_Cutscene::CurrentCutscene->Cutscene->AnimationNames.GetName(asuraAnimEvent->m_iAnimationIndex);
		AnimationFileNameBackup = Tools_Mode_Cutscene::CurrentCutscene->Cutscene->AnimationFileNames.GetName(asuraAnimEvent->m_iAnimationIndex);
	}
	AnimationFlagsBackup = asuraAnimEvent->m_iAnimationFlags;
	TweenDurationBackup = asuraAnimEvent->m_fTweenDuration;
	m_fStartTimerBackup = asuraAnimEvent->m_fStartTimer;
	m_fEndTimerBackup = asuraAnimEvent->m_fEndTimer;
	m_fAnimDelayBackup = asuraAnimEvent->m_fAnimDelay;
}

const char* Tools_Cutscene_Event_AnimationChange::GetNameOfChangedVariable()
{
	Asura_Cutscene_Event_AnimationChange* asuraAnimEvent = (Asura_Cutscene_Event_AnimationChange*) Event;
	
	Asura_String xAnimName;
	if(asuraAnimEvent->m_iAnimationIndex >=0) xAnimName = Tools_Mode_Cutscene::CurrentCutscene->Cutscene->AnimationNames.GetName(asuraAnimEvent->m_iAnimationIndex);
	
	if((xAnimName.IsBlank() != AnimationNameBackup.IsBlank()) ||
		(!xAnimName.IsBlank() && xAnimName.Compare(AnimationNameBackup)))
	{
		return "Animation";
	}
	if(AnimationFlagsBackup != asuraAnimEvent->m_iAnimationFlags)
	{
		int iChangedFlags = AnimationFlagsBackup ^ asuraAnimEvent->m_iAnimationFlags;
		if(iChangedFlags & AsuraAnimationFlag_Loop) return "Looped";
		if(iChangedFlags & AsuraAnimationFlag_Tween) return "Tween";
		if(iChangedFlags & AsuraAnimationFlag_TweenOut) return "Tween Out";
		if(iChangedFlags & AsuraAnimationFlag_Reversed) return "Reversed";
		if(iChangedFlags & AnimationChangeFlag_UseCameraTarget) return "UseCameraTarget";
		if(iChangedFlags & AnimationChangeFlag_IgnoreMovement) return "Ignore Movement";
		if(iChangedFlags & AsuraAnimationFlag_Mirror) return "Mirror";
		return "Animation Flags";
	}
	if(TweenDurationBackup != asuraAnimEvent->m_fTweenDuration) return "Tween Duration";
	if(m_fStartTimerBackup != asuraAnimEvent->m_fStartTimer) return "Start Timer";
	if(m_fEndTimerBackup != asuraAnimEvent->m_fEndTimer) return "End Timer";
	if(m_fAnimDelayBackup != asuraAnimEvent->m_fAnimDelay) return "Anim Delay";
	

	return Tools_Cutscene_Event::GetNameOfChangedVariable();
}

bool Tools_Cutscene_Event_AnimationChange::IsTargetHierarchy()
{
	if(!ThisEvent) return false;
	if(!ThisEvent->Event->m_pxTargetActor) return false;
	
	return ThisEvent->Event->m_pxTargetActor->IsHierarchyActor();
}

bool Tools_Cutscene_Event_AnimationChange::IsTargetCamera()
{
	if(!ThisEvent) return false;
	if(!ThisEvent->Event->m_pxTargetActor) return false;
	
	return ThisEvent->Event->m_pxTargetActor->IsCamera();
}

bool Tools_Cutscene_Event_AnimationChange::ActorIsValidTarget(class Asura_Cutscene_Actor* pxActor)
{
	return pxActor->IsHierarchyActor() || pxActor->Type == ACAT_Camera;
}

void Tools_Cutscene_Event_AnimationChange::AskForRequiredData()
{
	//select target
	if(!Event->m_pxTargetActor)
	{
		ChooseActor();
	}
	if(!Event->m_xTargetActorName.GetLength()) return;
	
	//select the anim
	ChooseAnimation();
}

////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------------------**
** Tools_Cutscene_Event_SecondaryAnimation **
**-----------------------------------------*/

Tools_Cutscene_Event_SecondaryAnimation::Tools_Cutscene_Event_SecondaryAnimation(Asura_Cutscene_Event* event,Tools_Cutscene_Object* cutsceneObject,int eventID)
:Tools_Cutscene_Event_AnimationChange(event,cutsceneObject,eventID)
{
}


void Tools_Cutscene_Event_SecondaryAnimation::AddOverlayControls(class Asura_Overlay* newOverlay,Asura_Vector_2& offset)
{
	Tools_Cutscene_Event_AnimationChange::AddOverlayControls(newOverlay,offset);

	Asura_Dialog* newDialog;
	Asura_Cutscene_Event_SecondaryAnimation* animEvent = (Asura_Cutscene_Event_SecondaryAnimation*) Event;
		

	//Removing the duration control for now , to see if any one complains.
	//Its slightly obsolete with the new start/end position options anyway , and i suspect
	//may be broken for server cutscenes at the moment. RAR 16/08/02

	//Reinstated again , since this functionality is desirable once again. Still broken for
	//server cutscenes , but i'm not sure that matters for current projects. RAR 29/03/04
	
	newDialog = new Tools_CutsceneTime_Dialog("Duration",offset,&animEvent->m_fDuration,0,1000,0.1f,&Changed);
	newDialog->SetHelpText("Duration in seconds that this secondary animation should remain playing for.\nIf set to zero then the animation will play forever or for one loop according to the loop setting.");
	newOverlay->AddDialog(newDialog);
	offset.y+=1.5;

	newDialog = new Asura_Numerical_Dialog("Weight",offset,&animEvent->m_fWeight,0,1,0.1f,"%.3f",&Changed);
	newDialog->SetHelpText("Weighting value for animation (0 to 1).\nTypically set to 1 to allow secondary animation to have full control over the bones it affects.");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();
	

}


void Tools_Cutscene_Event_SecondaryAnimation::FillInTimelineData(Tools_Timeline_Data_Event* data)
{
	Asura_Cutscene_Event_SecondaryAnimation* animEvent = (Asura_Cutscene_Event_SecondaryAnimation*) Event;
	Tools_Cutscene_Event_AnimationChange::FillInTimelineData(data);

	
	if(animEvent->m_fDuration)
	{
		data->Duration = animEvent->m_fDuration;
	}
}

void Tools_Cutscene_Event_SecondaryAnimation::SetTotalDuration(float fDuration)
{
	Asura_Cutscene_Event_SecondaryAnimation* pxAsuraEvent = (Asura_Cutscene_Event_SecondaryAnimation*) Event;

	//The duration can be defined by the duration in the secondary animation event or by 'to position' in the parent.
	//Best to set in both , I guess.
	pxAsuraEvent->m_fDuration = fDuration;
	Tools_Cutscene_Event_AnimationChange::SetTotalDuration(fDuration);

}

Asura_String Tools_Cutscene_Event_SecondaryAnimation::GetDescription()
{
	Asura_Cutscene_Event_SecondaryAnimation* animEvent = (Asura_Cutscene_Event_SecondaryAnimation*) Event;
	char buffer[250];

    if(animEvent->m_iAnimationIndex != -1)
	{
		Asura_String & AnimationName = Tools_Mode_Cutscene::CurrentCutscene->Cutscene->AnimationNames.GetName(animEvent->m_iAnimationIndex);
		sprintf(buffer,"Secondary Animation : %s",AnimationName.GetString());
	}
	else
	{
		sprintf(buffer,"Secondary Animation : *Not Set*");
	}
	return buffer;
}



void Tools_Cutscene_Event_SecondaryAnimation::UpdateBackupVariables()
{
	Tools_Cutscene_Event_AnimationChange::UpdateBackupVariables();
	Asura_Cutscene_Event_SecondaryAnimation* asuraAnimEvent = (Asura_Cutscene_Event_SecondaryAnimation*) Event;

	WeightBackup = asuraAnimEvent->m_fWeight;
	DurationBackup = asuraAnimEvent->m_fDuration;
}

const char* Tools_Cutscene_Event_SecondaryAnimation::GetNameOfChangedVariable()
{
	Asura_Cutscene_Event_SecondaryAnimation* asuraAnimEvent = (Asura_Cutscene_Event_SecondaryAnimation*) Event;
	
	if(WeightBackup != asuraAnimEvent->m_fWeight) return "Weight";
	if(DurationBackup != asuraAnimEvent->m_fDuration) return "Duration";

	return Tools_Cutscene_Event_AnimationChange::GetNameOfChangedVariable();
}

////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------------**
** Tools_Cutscene_Event_TextureAnimation **
**--------------------------------------*/

Tools_Cutscene_Event_TextureAnimation::Tools_Cutscene_Event_TextureAnimation(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID)
:Tools_Cutscene_Event(pxEvent,pxCutsceneObject,iEventID)
{
}


void Tools_Cutscene_Event_TextureAnimation::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Event_TextureAnimation* pxAnimEvent = (Asura_Cutscene_Event_TextureAnimation*) Event;

	Asura_Dialog* pxDialog;
	char text[200];


	//add actor selection control
	if(pxAnimEvent->m_xTargetActorName.GetLength())
		sprintf(text,"Actor : %s",pxAnimEvent->m_xTargetActorName.GetString());
	else
		strcpy(text,"Actor : *None*");

	pxDialog = new Asura_Function_Dialog(text,xOffset,ChooseActor);
	pxDialog->SetHelpText("Select actor whose texture animation should be changed.");
	pxDialog->m_iControlID = DialogID_Actor;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Change Sequence",xOffset,&pxAnimEvent->m_uTextureAnimFlags,ASURA_CUTSCENEEVENT_TEXTUREANIMATION_CHANGESEQUENCE,&Changed);
	pxDialog->SetHelpText("Change the texture animation sequence used.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Sequence",xOffset,&pxAnimEvent->m_iSequence,0,99,1,"%d",&Changed);
	pxDialog->SetHelpText("Change the texture animation sequence used.");
	pxDialog->SetIsEnabledFunction(IsEnabled_Sequence);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Change Speed",xOffset,&pxAnimEvent->m_uTextureAnimFlags,ASURA_CUTSCENEEVENT_TEXTUREANIMATION_CHANGESPEED,&Changed);
	pxDialog->SetHelpText("Change the speed multiplier for texture animation.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Speed",xOffset,&pxAnimEvent->m_fSpeed,-999.0f,999.0f,0.1f,"%.2f",&Changed);
	pxDialog->SetHelpText("Speed multiplier for texture animation.\n1 is normal speed.\n0 is paused.\nNegative values reverse the animation.");
	pxDialog->SetIsEnabledFunction(IsEnabled_Speed);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}



void Tools_Cutscene_Event_TextureAnimation::UpdateBackupVariables()
{
	Tools_Cutscene_Event::UpdateBackupVariables();
	Asura_Cutscene_Event_TextureAnimation* pxAsuraAnimEvent = (Asura_Cutscene_Event_TextureAnimation*) Event;

	m_iSequenceBackup = pxAsuraAnimEvent->m_iSequence;
	m_fSpeedBackup = pxAsuraAnimEvent->m_fSpeed;
	m_uTextureAnimFlagsBackup = pxAsuraAnimEvent->m_uTextureAnimFlags;
}

const char* Tools_Cutscene_Event_TextureAnimation::GetNameOfChangedVariable()
{
	Asura_Cutscene_Event_TextureAnimation* pxAsuraAnimEvent = (Asura_Cutscene_Event_TextureAnimation*) Event;
		
	if(pxAsuraAnimEvent->m_iSequence != m_iSequenceBackup) return "Sequence";
	if(pxAsuraAnimEvent->m_fSpeed != m_fSpeedBackup) return "Speed";
	if(pxAsuraAnimEvent->m_uTextureAnimFlags != m_uTextureAnimFlagsBackup) return "Anim Flags";
	

	return Tools_Cutscene_Event::GetNameOfChangedVariable();
}

bool Tools_Cutscene_Event_TextureAnimation::IsTargetHierarchy()
{
	if(!ThisEvent) return false;
	if(!ThisEvent->Event->m_pxTargetActor) return false;
	
	return ThisEvent->Event->m_pxTargetActor->IsHierarchyActor();
}

bool Tools_Cutscene_Event_TextureAnimation::ActorIsValidTarget(class Asura_Cutscene_Actor* pxActor)
{
	return pxActor->IsHierarchyActor();
}

bool Tools_Cutscene_Event_TextureAnimation::IsEnabled_Sequence()
{
	if(!ThisEvent) return false;
	if(ThisEvent->Event->m_iType != ACET_TextureAnimation) return false;

	Asura_Cutscene_Event_TextureAnimation* pxEvent = (Asura_Cutscene_Event_TextureAnimation*) ThisEvent->Event;
	
	return (pxEvent->m_uTextureAnimFlags & ASURA_MESSAGE_TEXTUREANIMATION_CHANGESEQUENCE) ? true : false;
}

bool Tools_Cutscene_Event_TextureAnimation::IsEnabled_Speed()
{
	if(!ThisEvent) return false;
	if(ThisEvent->Event->m_iType != ACET_TextureAnimation) return false;

	Asura_Cutscene_Event_TextureAnimation* pxEvent = (Asura_Cutscene_Event_TextureAnimation*) ThisEvent->Event;
	
	return (pxEvent->m_uTextureAnimFlags & ASURA_MESSAGE_TEXTUREANIMATION_CHANGESPEED) ? true : false;
}


void Tools_Cutscene_Event_TextureAnimation::AskForRequiredData()
{
	//select target
	ChooseActor();
}

////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------**
** Tools_Cutscene_Event_TimeScale **
**--------------------------------*/

Tools_Cutscene_Event_TimeScale::Tools_Cutscene_Event_TimeScale(Asura_Cutscene_Event* event,Tools_Cutscene_Object* cutsceneObject,int eventID)
:Tools_Cutscene_Event(event,cutsceneObject,eventID)
{
}


void Tools_Cutscene_Event_TimeScale::AddOverlayControls(class Asura_Overlay* newOverlay,Asura_Vector_2& offset)
{
	Asura_Cutscene_Event_TimeScale* scaleEvent = (Asura_Cutscene_Event_TimeScale*) Event;

	Asura_Dialog* newDialog;

	newDialog = new Asura_Numerical_Dialog("Time Scale",offset,&scaleEvent->m_fTimeScale,0.1f,99.0f,0.1f,"%.3f",&Changed);
	newDialog->SetHelpText("Multiplier for speed at which cutscene should be played.");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();

	
}




Asura_String Tools_Cutscene_Event_TimeScale::GetDescription()
{
	Asura_Cutscene_Event_TimeScale* scaleEvent = (Asura_Cutscene_Event_TimeScale*) Event;
	char buffer[250];

	sprintf(buffer,"Change time scale to %.3f",scaleEvent->m_fTimeScale);
	return buffer;
}

void Tools_Cutscene_Event_TimeScale::UpdateBackupVariables()
{
	Tools_Cutscene_Event::UpdateBackupVariables();
	Asura_Cutscene_Event_TimeScale* asuraScaleEvent = (Asura_Cutscene_Event_TimeScale*) Event;
	TimeScaleBackup = asuraScaleEvent->m_fTimeScale;
}

const char* Tools_Cutscene_Event_TimeScale::GetNameOfChangedVariable()
{
	Asura_Cutscene_Event_TimeScale* asuraScaleEvent = (Asura_Cutscene_Event_TimeScale*) Event;

	if(TimeScaleBackup != asuraScaleEvent->m_fTimeScale) return "Time Scale";

	return Tools_Cutscene_Event::GetNameOfChangedVariable();
	
}

////////////////////////////////////////////////////////////////////////////////////////////
/*----- ----------------------------**
** Tools_Cutscene_Event_FieldOfView **
**----------------------------------*/

Tools_Cutscene_Event_FieldOfView::Tools_Cutscene_Event_FieldOfView(Asura_Cutscene_Event* event,Tools_Cutscene_Object* cutsceneObject,int eventID)
:Tools_Cutscene_Event(event,cutsceneObject,eventID)
{
}


void Tools_Cutscene_Event_FieldOfView::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Event_FieldOfView* fovEvent = (Asura_Cutscene_Event_FieldOfView*) Event;

	Asura_Dialog* pxDialog;

	char szText[200];

	//add actor selection control
	if(Event->m_xTargetActorName.GetLength())
		sprintf(szText,"Camera : %s",Event->m_xTargetActorName.GetString());
	else
		strcpy(szText,"Camera : *All*");

    pxDialog = new Asura_Function_Dialog(szText,xOffset,Tools_Cutscene_Event::ChooseActor);
	pxDialog->SetHelpText("Choose camera to target. If none is chosen , then the event will effect all cameras.");
	pxDialog->m_iControlID = DialogID_Actor;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Tools_CutsceneTime_Dialog("Duration",xOffset,&fovEvent->m_fDuration,0.0f,1000.0f,0.1f,&Changed);
	pxDialog->SetHelpText("Time taken to reach the destination field of view.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("FOV",xOffset,&fovEvent->m_fFinalFOV,5.0f,175.0f,0.1f,"%.3f",&Changed);
	pxDialog->SetHelpText("Field of view (in degrees) that should be interpolated towards.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

}


void Tools_Cutscene_Event_FieldOfView::FillInTimelineData(Tools_Timeline_Data_Event* data)
{
	Asura_Cutscene_Event_FieldOfView* fovEvent = (Asura_Cutscene_Event_FieldOfView*) Event;
	Tools_Cutscene_Event::FillInTimelineData(data);

	data->Duration = fovEvent->m_fDuration;
}

void Tools_Cutscene_Event_FieldOfView::SetTotalDuration(float fDuration)
{
	Asura_Cutscene_Event_FieldOfView* pxEvent = (Asura_Cutscene_Event_FieldOfView*) Event;
	pxEvent->m_fDuration = fDuration;
}


Asura_String Tools_Cutscene_Event_FieldOfView::GetDescription()
{
	Asura_Cutscene_Event_FieldOfView* fovEvent = (Asura_Cutscene_Event_FieldOfView*) Event;
	char buffer[250];

	if(Event->m_xTargetActorName.IsBlank())
	{
		sprintf(buffer,"Change field of view to %.3f",fovEvent->m_fFinalFOV);
	}
	else
	{
		sprintf(buffer,"Change '%s' field of view to %.3f",Event->m_xTargetActorName.GetString(),fovEvent->m_fFinalFOV);
	}

	return buffer;

}

void Tools_Cutscene_Event_FieldOfView::UpdateBackupVariables()
{
	Tools_Cutscene_Event::UpdateBackupVariables();
	Asura_Cutscene_Event_FieldOfView* asuraFOVEvent = (Asura_Cutscene_Event_FieldOfView*) Event;
	DurationBackup = asuraFOVEvent->m_fDuration;
	FinalFOVBackup = asuraFOVEvent->m_fFinalFOV;
}

const char* Tools_Cutscene_Event_FieldOfView::GetNameOfChangedVariable()
{
	Asura_Cutscene_Event_FieldOfView* asuraFOVEvent = (Asura_Cutscene_Event_FieldOfView*) Event;

	if(DurationBackup != asuraFOVEvent->m_fDuration) return "Duration";
	if(FinalFOVBackup != asuraFOVEvent->m_fFinalFOV) return "FOV";

	return Tools_Cutscene_Event::GetNameOfChangedVariable();	
}

void Tools_Cutscene_Event_FieldOfView::AskForRequiredData()
{
	//select the camera
	ChooseActor();
}

bool Tools_Cutscene_Event_FieldOfView::ActorIsValidTarget(Asura_Cutscene_Actor* pxActor)
{
	return pxActor->IsCamera();
}

//////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------**
** Tools_Cutscene_Event_Trigger **
**------------------------------*/

Tools_Cutscene_Event_Trigger::Tools_Cutscene_Event_Trigger(Asura_Cutscene_Event* event,Tools_Cutscene_Object* cutsceneObject,int eventID)
:Tools_Cutscene_Event(event,cutsceneObject,eventID)
{
}


void Tools_Cutscene_Event_Trigger::AddOverlayControls(class Asura_Overlay* newOverlay,Asura_Vector_2& offset)
{
	Asura_Cutscene_Event_Trigger* asuraEvent = (Asura_Cutscene_Event_Trigger*) Event;

	Asura_Dialog* newDialog;
	char text[200];
	
	sprintf(text,"Edit Messages (%d)",Asura_StaticMessage_System::GetNumberOfMessagesInBlock(asuraEvent->m_sMessageBlock));
	newDialog = new Asura_Function_Dialog(text,offset,EditMessageBlock);
	newDialog->SetHelpText("Edit messages to be sent when this trigger event is reached.");
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();
}


void Tools_Cutscene_Event_Trigger::EditMessageBlock()
{
	Asura_Assert(ThisEvent,"");
	Asura_Cutscene_Event_Trigger* asuraEvent = (Asura_Cutscene_Event_Trigger*) ThisEvent->Event;

	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_Object_System::SelectObject(Tools_Mode_Cutscene::CurrentCutscene);
	Tools_MessageEditor::StartMessageEditingMode(&asuraEvent->m_sMessageBlock);
	#else
	//Ensure that the event object is selected before the message editing.
	//This will have the side effect of closing the overlay while message editing is done , 
	//and reopening it once finished.
	Tools_Object_System::SelectObject(ThisEvent->EventObject);
	Tools_Object_System::EditMessageBlock("Event Trigger :",&asuraEvent->m_sMessageBlock,ASURA_STATICMESSAGE_CUTSCENESET);
	#endif
}


Asura_String Tools_Cutscene_Event_Trigger::GetDescription()
{
	return "Trigger";
}


//////////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------**
** Tools_Cutscene_Event_LoopBack**
**------------------------------*/

Tools_Cutscene_Event_LoopBack::Tools_Cutscene_Event_LoopBack(Asura_Cutscene_Event* event,Tools_Cutscene_Object* cutsceneObject,int eventID)
:Tools_Cutscene_Event(event,cutsceneObject,eventID)
{
	m_iLoopBackEventID = -1;
	m_iLoopBackEventIDBackup = -1;	
}

void Tools_Cutscene_Event_LoopBack::AddOverlayControls(class Asura_Overlay* newOverlay,Asura_Vector_2& offset)
{
	Asura_Cutscene_Event_LoopBack* asuraEvent = (Asura_Cutscene_Event_LoopBack*) Event;

	Asura_Dialog* newDialog;

	char text[200];

	//add actor selection control
	if(asuraEvent->m_xTargetActorName.GetLength())
		sprintf(text,"Actor : %s",asuraEvent->m_xTargetActorName.GetString());
	else
		strcpy(text,"Actor : *None*");

    newDialog = new Asura_Function_Dialog(text,offset,Tools_Cutscene_Event::ChooseActor);
	newDialog->SetHelpText("Choose actor whose events should be looped.\nIf no actor is chosen then all events will be looped.");
	newDialog->m_iControlID = DialogID_Actor;
	newOverlay->AddDialog(newDialog);
	offset.y+=Asura_Dialog::DefaultSpacing();

	//overlay controls depend on whether an event to loop back to has been set
	if(m_iLoopBackEventID>=0)
	{
		Tools_Cutscene_Event* loopToEvent = GetToolsCutsceneEvent(m_iLoopBackEventID);
		if(loopToEvent)
			sprintf(text,"To Event : (%d) %s",loopToEvent->Index,loopToEvent->GetDescription());
		else
			strcpy(text,"To Event : (Event Not Found)");

   		newDialog = new Asura_Function_Dialog(text,offset,ChooseEvent);
		newDialog->SetHelpText("Choose event to loop back to.");
		newOverlay->AddDialog(newDialog);
		offset.y+=Asura_Dialog::DefaultSpacing();

	}
	else
	{
		strcpy(text,"To Event : (None)");
   		newDialog = new Asura_Function_Dialog(text,offset,ChooseEvent);
		newDialog->SetHelpText("Choose event to loop back to.");
		newOverlay->AddDialog(newDialog);
		offset.y+=Asura_Dialog::DefaultSpacing();
   
		newDialog = new Tools_CutsceneTime_Dialog("Time to loop back", offset, &asuraEvent->m_fNumSecondsToLoopBack,0.0f,1000.0f,0.1f,&Changed);
		newDialog->SetHelpText("Sets amount of time to loop back (instead of going to a particular event).");
		newOverlay->AddDialog(newDialog);
		offset.y+=Asura_Dialog::DefaultSpacing();
	}

}

void Tools_Cutscene_Event_LoopBack::FillInTimelineData(Tools_Timeline_Data_Event* data)
{
	Asura_Cutscene_Event_LoopBack* asuraEvent = (Asura_Cutscene_Event_LoopBack*) Event;
	Tools_Cutscene_Event::FillInTimelineData(data);

	if(!asuraEvent->m_pxTargetActor)
	{
		data->IsOkay = false;
	}

    // start of event is point looped back to and end is trigger time, in timeline display
    data->Time = asuraEvent->m_fTriggerTime - asuraEvent->m_fNumSecondsToLoopBack;
    data->Duration = asuraEvent->m_fNumSecondsToLoopBack;
}

Asura_String Tools_Cutscene_Event_LoopBack::GetDescription()
{
	Asura_Cutscene_Event_LoopBack* asuraEvent = (Asura_Cutscene_Event_LoopBack*) Event;
	char buffer[256];

	if(Tools_Mode_Cutscene::GetTimerMode() == CUTSCENETIMER_SECONDS)
	{
		sprintf(buffer,"Loop back %.3f seconds", asuraEvent->m_fNumSecondsToLoopBack);
	}
	else
	{
		sprintf(buffer,"Loop back %.0f frames", Tools_Mode_Cutscene::ConvertSecondsToDisplayedTime(asuraEvent->m_fNumSecondsToLoopBack));
	}
	return buffer;
}

void Tools_Cutscene_Event_LoopBack::UpdateBackupVariables()
{
	Tools_Cutscene_Event::UpdateBackupVariables();
	Asura_Cutscene_Event_LoopBack* asuraEvent = (Asura_Cutscene_Event_LoopBack*) Event;
	m_fSecondsToLoopBack = asuraEvent->m_fNumSecondsToLoopBack;

	m_iLoopBackEventIDBackup = m_iLoopBackEventID;
}

const char* Tools_Cutscene_Event_LoopBack::GetNameOfChangedVariable()
{
	Asura_Cutscene_Event_LoopBack* asuraEvent = (Asura_Cutscene_Event_LoopBack*) Event;

	if(m_iLoopBackEventIDBackup != m_iLoopBackEventID) return "Loop Back Event ID";
	if(m_fSecondsToLoopBack != asuraEvent->m_fNumSecondsToLoopBack) return "Seconds To Loop";

	return Tools_Cutscene_Event::GetNameOfChangedVariable();

}

void Tools_Cutscene_Event_LoopBack::UpdateEventIDsFromIndeces()
{
	Tools_Cutscene_Event::UpdateEventIDsFromIndeces();

	Asura_Cutscene_Event_LoopBack* asuraEvent = (Asura_Cutscene_Event_LoopBack*) Event;

	if(asuraEvent->m_iEventToLoopBackTo>=0)
	{
		Asura_Assert(asuraEvent->m_iEventToLoopBackTo<CutsceneObject->EventList.size(),"");
		Tools_Cutscene_Event* parentEvent = CutsceneObject->EventList[asuraEvent->m_iEventToLoopBackTo];

		m_iLoopBackEventID = parentEvent->EventID;
		m_iLoopBackEventIDBackup = m_iLoopBackEventID;
	}
	else
	{
		m_iLoopBackEventID = -1;
		m_iLoopBackEventIDBackup = -1;
	}
}

void Tools_Cutscene_Event_LoopBack::UpdateEventIndecesFromIDs()
{
	Tools_Cutscene_Event::UpdateEventIndecesFromIDs();
	Asura_Cutscene_Event_LoopBack* asuraEvent = (Asura_Cutscene_Event_LoopBack*) Event;

	//use the id to get the tools event of the event to be looped to
	Tools_Cutscene_Event* loopToEvent = GetToolsCutsceneEvent(m_iLoopBackEventID);
	if(loopToEvent)
	{
		//update the index in the asura data
		asuraEvent->m_iEventToLoopBackTo = loopToEvent->Index;
	}
	else
	{
		asuraEvent->m_iEventToLoopBackTo = -1;
	}
}

void Tools_Cutscene_Event_LoopBack::ChooseEvent()
{
	Asura_Assert(ThisEvent,"");
	Tools_Cutscene_Event_LoopBack* toolsEvent = (Tools_Cutscene_Event_LoopBack*) ThisEvent;
	Asura_Cutscene_Event_LoopBack* asuraEvent = (Asura_Cutscene_Event_LoopBack*) toolsEvent->Event;

	//ask which event to go back to
	Tools_Cutscene_Event* event;
	if(toolsEvent->CutsceneObject->ChooseEvent("Select event to loop back to",event,ChooseEventFilterFunction,(int)toolsEvent))
	{
		//update the ids and indeces
		toolsEvent->m_iLoopBackEventID = event ? event->EventID : -1;
		asuraEvent->m_iEventToLoopBackTo = event ? event->Index : -1;

		toolsEvent->Changed = true;

		//update the overlay
		toolsEvent->DeleteOverlay();
		toolsEvent->CreateOverlay();

	}

}

bool Tools_Cutscene_Event_LoopBack::ChooseEventFilterFunction(Tools_Cutscene_Event* pxEvent,int iData)
{
	Tools_Cutscene_Event_LoopBack* pxLoopBack = (Tools_Cutscene_Event_LoopBack*) iData;
	Asura_Assert(pxEvent,"");
	Asura_Assert(pxLoopBack,"");

	//Only allow earlier events to  be selected
	if(pxLoopBack->Event->m_fTriggerTime<=pxEvent->Event->m_fTriggerTime) return false;

	//If the loop back event has an actor associated with it , filter out any events that don't use the same actor.
	if(pxLoopBack->Event->m_pxTargetActor && pxLoopBack->Event->m_pxTargetActor!=pxEvent->Event->m_pxTargetActor) return false;

	return true;
}
//////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------------**
** Tools_Cutscene_Event_Attachment **
**---------------------------------*/
Tools_Cutscene_Event_Attachment::Tools_Cutscene_Event_Attachment(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID)
:Tools_Cutscene_Event(pxEvent,pxCutsceneObject,iEventID)
{
	m_uAttachmentIDBackup = 0;
}

void Tools_Cutscene_Event_Attachment::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Event_Attachment* pxAsuraEvent = (Asura_Cutscene_Event_Attachment*) Event;

	Asura_Dialog* pxDialog;

	char szText[200];

	//add actor selection control
	if(pxAsuraEvent->m_xTargetActorName.GetLength())
		sprintf(szText,"Actor : %s",pxAsuraEvent->m_xTargetActorName.GetString());
	else
		strcpy(szText,"Actor : *None*");

    pxDialog = new Asura_Function_Dialog(szText,xOffset,Tools_Cutscene_Event::ChooseActor);
	pxDialog->SetHelpText("Choose actor whose attachment should be activated/deactivated.");
	pxDialog->m_iControlID = DialogID_Actor;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//attachment selection
	if(pxAsuraEvent->m_uAttachmentID)
	{
		Tools_Cutscene_Attachment* pxAttachment = GetToolsAttachment();
		if(pxAttachment)
		{
			sprintf(szText,"Attachment : %s",pxAttachment->GetAttachmentOrMarkerName());
		}
		else
		{
			strcpy(szText,"Attachment : *Not Found*");
		}
	}
	else
	{
		strcpy(szText,"Attachment : *Not Set*");
	}
    pxDialog = new Asura_Function_Dialog(szText,xOffset,_SelectAttachment);
	pxDialog->SetHelpText("Choose attachment to activate/deactivate.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

}

void Tools_Cutscene_Event_Attachment::FillInTimelineData(Tools_Timeline_Data_Event* pxData)
{
	Asura_Cutscene_Event_Attachment* pxAsuraEvent = (Asura_Cutscene_Event_Attachment*) Event;
	Tools_Cutscene_Event::FillInTimelineData(pxData);

	if(!pxAsuraEvent->m_pxTargetActor)
	{
		pxData->IsOkay = false;
	}
}

const char* Tools_Cutscene_Event_Attachment::GetTypeDescription()
{
	if (Event->m_iType == ACET_ActivateAttachment)
	{
		return "Activate Attachment";
	}
	else if (Event->m_iType == ACET_DeactivateAttachment)
	{
		return "Deactivate Attachment";
	}
	else
	{
		Asura_Assert(Event->m_iType == ACET_ReactivateAttachment, "unexpected attachment event type");
		return "Reactivate Attachment";
	}
}

const char* Tools_Cutscene_Event_Attachment::GetAbbreviatedTypeDescription()
{
	if (Event->m_iType == ACET_ActivateAttachment)
	{
		return "ActAtt";
	}
	else if (Event->m_iType == ACET_DeactivateAttachment)
	{
		return "DeaAtt";
	}
	else
	{
		Asura_Assert(Event->m_iType == ACET_ReactivateAttachment, "unexpected attachment event type");
		return "ReaAtt";
	}
}


Asura_String Tools_Cutscene_Event_Attachment::GetDescription()
{
	Asura_Cutscene_Event_Attachment* pxAsuraEvent = (Asura_Cutscene_Event_Attachment*) Event;
	char buffer[256];

	strcpy(buffer,GetTypeDescription());
	strcat(buffer," ");

	if(pxAsuraEvent->m_uAttachmentID)
	{
		Tools_Cutscene_Attachment* pxAttachment = GetToolsAttachment();
		if(pxAttachment)
		{
			strcat(buffer,pxAttachment->GetAttachmentOrMarkerName());
		}
		else
		{
			strcat(buffer,"*Not Found*");
		}
	}
	else
	{
		strcat(buffer,"*Not Set*");
	}

	return buffer;
}

void Tools_Cutscene_Event_Attachment::UpdateBackupVariables()
{
	Tools_Cutscene_Event::UpdateBackupVariables();
	Asura_Cutscene_Event_Attachment* pxAsuraEvent = (Asura_Cutscene_Event_Attachment*) Event;

	m_uAttachmentIDBackup = pxAsuraEvent->m_uAttachmentID;
}

const char* Tools_Cutscene_Event_Attachment::GetNameOfChangedVariable()
{
	Asura_Cutscene_Event_Attachment* pxAsuraEvent = (Asura_Cutscene_Event_Attachment*) Event;

	if(m_uAttachmentIDBackup != pxAsuraEvent->m_uAttachmentID) return "Attachment ID";

	return Tools_Cutscene_Event::GetNameOfChangedVariable();
}

void Tools_Cutscene_Event_Attachment::SelectAttachment()
{
	Asura_Cutscene_Event_Attachment* pxAsuraEvent = (Asura_Cutscene_Event_Attachment*) Event;
	if(pxAsuraEvent->m_xTargetActorName.IsBlank())
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Select a target actor first");
		return;
	}

	//find the tools actor object
	Tools_CutsceneActor_Object* pxToolsActor = GetActorObject(pxAsuraEvent->m_xTargetActorName);
	if(!pxToolsActor)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find actor '%s'",pxAsuraEvent->m_xTargetActorName.GetString());
		return;
	}

	if(!pxToolsActor->m_xAttachmentList.size())
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"'%s' doesen't have any attachments",pxAsuraEvent->m_xTargetActorName.GetString());
		return;
	}

	//add all the attachments to a selection overlay
	ListSelection_Overlay xOverlay("Select Attachment");

	for(LIF<Tools_Cutscene_Attachment*> xAttachmentLIF(&pxToolsActor->m_xAttachmentList);!xAttachmentLIF.done();xAttachmentLIF.next())
	{
		xOverlay.AddItem(xAttachmentLIF()->GetAttachmentOrMarkerName(),(int)xAttachmentLIF()->GetID());
	}

	if(!xOverlay.DoModal()) return;

	pxAsuraEvent->m_uAttachmentID = (u_int)xOverlay.GetSelectionData();
	Changed = true;

	if(ThisEvent == this)
	{
		DeleteOverlay();
		CreateOverlay();
	}
}

Tools_Cutscene_Attachment* Tools_Cutscene_Event_Attachment::GetToolsAttachment()
{
	Asura_Cutscene_Event_Attachment* pxAsuraEvent = (Asura_Cutscene_Event_Attachment*) Event;
	if(!pxAsuraEvent->m_uAttachmentID) return 0;

	//find the tools actor object
	Tools_CutsceneActor_Object* pxToolsActor = GetActorObject(pxAsuraEvent->m_xTargetActorName);
	if(!pxToolsActor)
	{
		return 0;
	}
	//search for attachment with correct ID
	for(LIF<Tools_Cutscene_Attachment*> xAttachmentLIF(&pxToolsActor->m_xAttachmentList);!xAttachmentLIF.done();xAttachmentLIF.next())
	{
		if(xAttachmentLIF()->GetID() == pxAsuraEvent->m_uAttachmentID)
		{
			return xAttachmentLIF();
		}
	}

	return 0;
	
}

void Tools_Cutscene_Event_Attachment::AskForRequiredData()
{
	//select target
	ChooseActor();
	if(!Event->m_xTargetActorName.GetLength()) return;

	//select attachment
	SelectAttachment();
}

//////////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------------------**
** Tools_Cutscene_Event_AttachmentControl **
**----------------------------------------*/
Tools_Cutscene_Event_AttachmentControl::Tools_Cutscene_Event_AttachmentControl(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID)
:Tools_Cutscene_Event(pxEvent,pxCutsceneObject,iEventID)
{
	m_uNumAttachmentsBackup = 0;
	m_pxAttachmentsBackup = 0;
}

Tools_Cutscene_Event_AttachmentControl::~Tools_Cutscene_Event_AttachmentControl()
{
	delete [] m_pxAttachmentsBackup;
}

void Tools_Cutscene_Event_AttachmentControl::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{

	Asura_Cutscene_Event_AttachmentControl* pxAsuraEvent = (Asura_Cutscene_Event_AttachmentControl*) Event;

	Asura_Dialog* pxDialog;

	char szText[200];

	//add actor selection control
	if(pxAsuraEvent->m_xTargetActorName.GetLength())
		sprintf(szText,"Actor : %s",pxAsuraEvent->m_xTargetActorName.GetString());
	else
		strcpy(szText,"Actor : *None*");

    pxDialog = new Asura_Function_Dialog(szText,xOffset,Tools_Cutscene_Event::ChooseActor);
	pxDialog->SetHelpText("Choose actor whose attachments should be activated/deactivated.");
	pxDialog->m_iControlID = DialogID_Actor;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

    pxDialog = new Asura_Function_Dialog("Add Attachment",xOffset,_AddAttachment);
	pxDialog->SetHelpText("Add an attachment to activate/deactivate.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	static const char* s_aszListStrings[] =
	{
		"No change",
		"Activate",
		"Deactivate",
		"Reactivate",
	};
	
	//attachment selection
	for(u_int i=0;i<pxAsuraEvent->m_uNumAttachments;i++)
	{
		const char* szAttachmentName = GetAttachmentNameByIndex(i);
		if(!szAttachmentName) szAttachmentName = "Unknown";
	

		pxDialog = new Asura_DropDown_List_Dialog(xOffset,s_aszListStrings,(int*)&pxAsuraEvent->m_pxAttachments[i].m_uType,4,4,NULL,&Changed,szAttachmentName);
		pxDialog->SetHelpText("The change to be made for this attachment.");
		xOffset.y+=Asura_Dialog::DefaultSpacing();
		pxOverlay->AddDialog(pxDialog);

	}
}

void Tools_Cutscene_Event_AttachmentControl::FillInTimelineData(Tools_Timeline_Data_Event* pxData)
{
	Asura_Cutscene_Event_AttachmentControl* pxAsuraEvent = (Asura_Cutscene_Event_AttachmentControl*) Event;
	Tools_Cutscene_Event::FillInTimelineData(pxData);

	if(!pxAsuraEvent->m_pxTargetActor)
	{
		pxData->IsOkay = false;
	}
}



Asura_String Tools_Cutscene_Event_AttachmentControl::GetDescription()
{
	char szBuffer[256];

	if(!Event->m_xTargetActorName.IsBlank())
	{
		Asura_PrintfToBfr(szBuffer,"Attachment Control : %s",sizeof(szBuffer),true,Event->m_xTargetActorName.GetString());
	}
	else
	{
		strcpy(szBuffer,"Attachment Control");
	}

	return szBuffer;
}

void Tools_Cutscene_Event_AttachmentControl::UpdateBackupVariables()
{
	Tools_Cutscene_Event::UpdateBackupVariables();
	Asura_Cutscene_Event_AttachmentControl* pxAsuraEvent = (Asura_Cutscene_Event_AttachmentControl*) Event;

	if(m_uNumAttachmentsBackup != pxAsuraEvent->m_uNumAttachments)
	{
		delete [] m_pxAttachmentsBackup;
		m_pxAttachmentsBackup = pxAsuraEvent->m_uNumAttachments ? new Asura_Cutscene_Event_AttachmentControl_Attachment[pxAsuraEvent->m_uNumAttachments] : 0;
	}
	m_uNumAttachmentsBackup = pxAsuraEvent->m_uNumAttachments;

	for(u_int i=0;i<m_uNumAttachmentsBackup;i++)
	{
		m_pxAttachmentsBackup[i] = pxAsuraEvent->m_pxAttachments[i];
	}
}

const char* Tools_Cutscene_Event_AttachmentControl::GetNameOfChangedVariable()
{
	Asura_Cutscene_Event_AttachmentControl* pxAsuraEvent = (Asura_Cutscene_Event_AttachmentControl*) Event;

	if(m_uNumAttachmentsBackup != pxAsuraEvent->m_uNumAttachments) return "Num Attachments";
	for(u_int i=0;i<m_uNumAttachmentsBackup;i++)
	{
		if(pxAsuraEvent->m_pxAttachments[i].m_uType != m_pxAttachmentsBackup[i].m_uType)
		{
			return "Attachment control type";
		}
	}

	return Tools_Cutscene_Event::GetNameOfChangedVariable();
}

void Tools_Cutscene_Event_AttachmentControl::AddAttachment()
{

	Asura_Cutscene_Event_AttachmentControl* pxAsuraEvent = (Asura_Cutscene_Event_AttachmentControl*) Event;
	if(pxAsuraEvent->m_xTargetActorName.IsBlank())
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Select a target actor first");
		return;
	}

	//find the tools actor object
	Tools_CutsceneActor_Object* pxToolsActor = GetActorObject(pxAsuraEvent->m_xTargetActorName);
	if(!pxToolsActor)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find actor '%s'",pxAsuraEvent->m_xTargetActorName.GetString());
		return;
	}

	if(!pxToolsActor->m_xAttachmentList.size())
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"'%s' doesn't have any attachments",pxAsuraEvent->m_xTargetActorName.GetString());
		return;
	}

	//add all the attachments to a selection overlay
	ListSelection_Overlay xOverlay("Select attachment to activate/deactivate");

	for(LIF<Tools_Cutscene_Attachment*> xAttachmentLIF(&pxToolsActor->m_xAttachmentList);!xAttachmentLIF.done();xAttachmentLIF.next())
	{
		//ignore attachments that are already controlled by this event
		u_int uID = xAttachmentLIF()->GetID();
		bool bAlreadyAdded = false;
		for(u_int i=0;i<pxAsuraEvent->m_uNumAttachments;i++)
		{
			if(pxAsuraEvent->m_pxAttachments[i].m_uID == uID)
			{
				bAlreadyAdded = true;
				break;
			}
		}
		if(bAlreadyAdded) continue;

		xOverlay.AddItem(xAttachmentLIF()->GetAttachmentOrMarkerName(),(int) uID);
	}
	if(!xOverlay.GetNumItems())
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"All attachments have already been addded");
		return;
	}


	if(!xOverlay.DoModal()) return;

	//allocate enlarged array of attachments
	Asura_Cutscene_Event_AttachmentControl_Attachment* pxNewAttachments = new Asura_Cutscene_Event_AttachmentControl_Attachment[pxAsuraEvent->m_uNumAttachments+1];
	for(u_int i=0;i<pxAsuraEvent->m_uNumAttachments;i++)
	{
		pxNewAttachments[i] = pxAsuraEvent->m_pxAttachments[i];	
	}
	u_int uNewIndex = pxAsuraEvent->m_uNumAttachments;

	pxNewAttachments[uNewIndex].m_uID = (u_int)xOverlay.GetSelectionData();
	pxNewAttachments[uNewIndex].m_uType = Asura_Cutscene_Event_AttachmentControl_Attachment::NONE;

	delete [] pxAsuraEvent->m_pxAttachments;
	pxAsuraEvent->m_pxAttachments = pxNewAttachments;
	pxAsuraEvent->m_uNumAttachments++;

	
	Changed = true;

	if(ThisEvent == this)
	{
		DeleteOverlay();
		CreateOverlay();
	}

}

const char* Tools_Cutscene_Event_AttachmentControl::GetAttachmentNameByIndex(u_int uIndex) const
{

	Asura_Cutscene_Event_AttachmentControl* pxAsuraEvent = (Asura_Cutscene_Event_AttachmentControl*) Event;
	Asura_Assert(uIndex < pxAsuraEvent->m_uNumAttachments,"");

	//find the tools actor object
	Tools_CutsceneActor_Object* pxToolsActor = GetActorObject(pxAsuraEvent->m_xTargetActorName);
	if(!pxToolsActor)
	{
		return 0;
	}
	u_int uID = pxAsuraEvent->m_pxAttachments[uIndex].m_uID;

	//search for attachment with correct ID
	for(LIF<Tools_Cutscene_Attachment*> xAttachmentLIF(&pxToolsActor->m_xAttachmentList);!xAttachmentLIF.done();xAttachmentLIF.next())
	{
		if(xAttachmentLIF()->GetID() == uID)
		{
			return xAttachmentLIF()->GetAttachmentOrMarkerName();
		}
	}

	return 0;
}

void Tools_Cutscene_Event_AttachmentControl::AskForRequiredData()
{
	//select target
	ChooseActor();
	if(!Event->m_xTargetActorName.GetLength()) return;

}


//////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------**
** Tools_Cutscene_Event_Beam **
**---------------------------*/
Tools_Cutscene_Event_Beam::Tools_Cutscene_Event_Beam(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID)
:Tools_Cutscene_Event(pxEvent,pxCutsceneObject,iEventID)
{
	m_uTailPositionEditID = 0;
	m_uHeadPositionEditID = 0;
}

Tools_Cutscene_Event_Beam::~Tools_Cutscene_Event_Beam()
{
	Tools_PositionEdit_Object::Delete(m_uTailPositionEditID);
	Tools_PositionEdit_Object::Delete(m_uHeadPositionEditID);
}

void Tools_Cutscene_Event_Beam::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Event_Beam* pxAsuraEvent = (Asura_Cutscene_Event_Beam*) Event;

	Asura_Dialog* pxDialog;

	pxDialog = new Asura_Function_Dialog( "Beam Texture", xOffset, _EditTexture );
	pxDialog->m_iControlID = Asura_GetHashID("texture");
	pxDialog->SetHelpText("Determines graphic to use for beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Tail Speed", xOffset, &pxAsuraEvent->m_xParam.m_xTemplate.fTailSpeed, 0, 999.0f, 0.1f, "%.3f", &Changed);
	pxDialog->SetHelpText("Speed in metres/second for tail of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Head Speed", xOffset, &pxAsuraEvent->m_xParam.m_xTemplate.fHeadSpeed, 0, 999.0f, 0.1f, "%.3f", &Changed);
	pxDialog->SetHelpText("Speed in metres/second for head of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Initial Radius", xOffset, &pxAsuraEvent->m_xParam.m_xTemplate.fInitialRadius, 0, 1, 0.01f, "%.3f", &Changed);
	pxDialog->SetHelpText("Initial radius of beam in metres.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("End Radius", xOffset, &pxAsuraEvent->m_xParam.m_xTemplate.fEndRadius, 0, 1, 0.01f, "%.3f", &Changed);
	pxDialog->SetHelpText("Final radius of beam in metres.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Tools_CutsceneTime_Dialog("Duration", xOffset, &pxAsuraEvent->m_xParam.m_fDuration, 0, 999.0f, 0.1f, &Changed);
	pxDialog->SetHelpText("How long the beam lasts (in seconds).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	const float fColumnOffset = 15.0f;

	//add initial/end headings for colour
	pxDialog = new Asura_Dialog("      Initial :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x+=fColumnOffset;

	pxDialog = new Asura_Dialog("      End :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x-=fColumnOffset;
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	//and add all the colour controls
	pxDialog = new Asura_Dialog("Tail :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//tail
	pxDialog = new Asura_ColourNumerical_Dialog("Red",xOffset,&pxAsuraEvent->m_xParam.m_xTemplate.xInitialTailColour.m_afChannels[0],"%d",&Changed);
	pxDialog->SetHelpText("Initial colour at tail of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x+=fColumnOffset;

	pxDialog = new Asura_ColourNumerical_Dialog("Red",xOffset,&pxAsuraEvent->m_xParam.m_xTemplate.xEndTailColour.m_afChannels[0],"%d",&Changed);
	pxDialog->SetHelpText("Final colour at tail of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x-=fColumnOffset;
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_ColourNumerical_Dialog("Green",xOffset,&pxAsuraEvent->m_xParam.m_xTemplate.xInitialTailColour.m_afChannels[1],"%d",&Changed);
	pxDialog->SetHelpText("Initial colour at tail of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x+=fColumnOffset;

	pxDialog = new Asura_ColourNumerical_Dialog("Green",xOffset,&pxAsuraEvent->m_xParam.m_xTemplate.xEndTailColour.m_afChannels[1],"%d",&Changed);
	pxDialog->SetHelpText("Final colour at tail of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x-=fColumnOffset;
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_ColourNumerical_Dialog("Blue",xOffset,&pxAsuraEvent->m_xParam.m_xTemplate.xInitialTailColour.m_afChannels[2],"%d",&Changed);
	pxDialog->SetHelpText("Initial colour at tail of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x+=fColumnOffset;

	pxDialog = new Asura_ColourNumerical_Dialog("Blue",xOffset,&pxAsuraEvent->m_xParam.m_xTemplate.xEndTailColour.m_afChannels[2],"%d",&Changed);
	pxDialog->SetHelpText("Final colour at tail of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x-=fColumnOffset;
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_ColourNumerical_Dialog("Alpha",xOffset,&pxAsuraEvent->m_xParam.m_xTemplate.xInitialTailColour.m_afChannels[3],"%d",&Changed);
	pxDialog->SetHelpText("Initial colour at tail of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x+=fColumnOffset;

	pxDialog = new Asura_ColourNumerical_Dialog("Alpha",xOffset,&pxAsuraEvent->m_xParam.m_xTemplate.xEndTailColour.m_afChannels[3],"%d",&Changed);
	pxDialog->SetHelpText("Final colour at tail of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x-=fColumnOffset;
	xOffset.y+=Asura_Dialog::LargeSpacing();


	pxDialog = new Asura_Dialog("Head :",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//and head
	pxDialog = new Asura_ColourNumerical_Dialog("Red",xOffset,&pxAsuraEvent->m_xParam.m_xTemplate.xInitialHeadColour.m_afChannels[0],"%d",&Changed);
	pxDialog->SetHelpText("Initial colour at head of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x+=fColumnOffset;

	pxDialog = new Asura_ColourNumerical_Dialog("Red",xOffset,&pxAsuraEvent->m_xParam.m_xTemplate.xEndHeadColour.m_afChannels[0],"%d",&Changed);
	pxDialog->SetHelpText("Final colour at head of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x-=fColumnOffset;
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_ColourNumerical_Dialog("Green",xOffset,&pxAsuraEvent->m_xParam.m_xTemplate.xInitialHeadColour.m_afChannels[1],"%d",&Changed);
	pxDialog->SetHelpText("Initial colour at head of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x+=fColumnOffset;

	pxDialog = new Asura_ColourNumerical_Dialog("Green",xOffset,&pxAsuraEvent->m_xParam.m_xTemplate.xEndHeadColour.m_afChannels[1],"%d",&Changed);
	pxDialog->SetHelpText("Final colour at head of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x-=fColumnOffset;
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_ColourNumerical_Dialog("Blue",xOffset,&pxAsuraEvent->m_xParam.m_xTemplate.xInitialHeadColour.m_afChannels[2],"%d",&Changed);
	pxDialog->SetHelpText("Initial colour at head of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x+=fColumnOffset;

	pxDialog = new Asura_ColourNumerical_Dialog("Blue",xOffset,&pxAsuraEvent->m_xParam.m_xTemplate.xEndHeadColour.m_afChannels[2],"%d",&Changed);
	pxDialog->SetHelpText("Final colour at head of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x-=fColumnOffset;
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_ColourNumerical_Dialog("Alpha",xOffset,&pxAsuraEvent->m_xParam.m_xTemplate.xInitialHeadColour.m_afChannels[3],"%d",&Changed);
	pxDialog->SetHelpText("Initial colour at head of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x+=fColumnOffset;

	pxDialog = new Asura_ColourNumerical_Dialog("Alpha",xOffset,&pxAsuraEvent->m_xParam.m_xTemplate.xEndHeadColour.m_afChannels[3],"%d",&Changed);
	pxDialog->SetHelpText("Final colour at head of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x-=fColumnOffset;
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Edit Tail Position",xOffset,_EditTailPosition);
	pxDialog->SetHelpText("Creates and selects the object for editing the tail of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Edit Head Position",xOffset,_EditHeadPosition);
	pxDialog->SetHelpText("Creates and selects the object for editing the head of beam.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

}

#include "Core/Asura_TiledTextureManagement.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "SpecialFX/Asura_Beam_System.h"
#include "PC/FileSystem/Tools_FileSystem.h"

void Tools_Cutscene_Event_Beam::EditTexture()
{
	Asura_Cutscene_Event_Beam* pxAsuraEvent = (Asura_Cutscene_Event_Beam*) Event;
	
	// make sure we've loaded the tiled texture file for the beam system
	LoadAsuraFile( ASURA_BEAM_SYSTEM_TILED_TEXTURE_FILENAME );

	Asura_String xBeamTextureFilename = ASURA_BEAM_SYSTEM_TILED_TEXTURE_FILENAME;
	Asura_String xTemp;

	xBeamTextureFilename.SplitFilename( 0, &xTemp, 0 );
	xBeamTextureFilename = xTemp;
	xBeamTextureFilename += ".dds";

	static Asura_String s_xDefaultPath = "Graphics\\SpecialFX\\Beams";

	Tools_FileSystem_DialogInfo	xFileSelector;

	xFileSelector.DefaultExtension		= ".tga";
	xFileSelector.DefaultFilename		= "";
	xFileSelector.DefaultPath			= s_xDefaultPath.GetString();
	xFileSelector.FileFilter			= "Texture Files\0*.tga;*.bmp;*.png;\0";
	xFileSelector.WindowTitle			= "Select Texture";
	xFileSelector.AllowMultipleSelect	= false;

	Asura_String xTextureName;
	xTextureName = Tools_FileSystem::GetLoadFilename( xFileSelector );
	if( xTextureName.IsNull() ) return;
		
	Asura_String xTextureExt;
	xTextureName.SplitFilename( 0, &xTemp, &xTextureExt );
	xTextureName = xTemp;
	xTextureName += ".";
	xTextureName += xTextureExt;	

	Asura_Hash_ID uTextureHash = Asura_GetHashID( xTextureName.GetString() );

	bool bFoundInBeamTexture = false;
	// run through the tiles in the tiled-texture system
	for( int u=0; u< Asura_TiledTextureManagement::MaxIndex(); ++u )
	{
		const Asura_TiledTextureInfo* pxTileInfo = Asura_TiledTextureManagement::GetTileInfoByID(u);
		if( pxTileInfo->m_xNameOfTextureUsed == xBeamTextureFilename )
		{
			// add to list of selectable textures			
			if( pxTileInfo->m_uTileHashID == uTextureHash )
			{
				pxAsuraEvent->m_xParam.m_xTemplate.uTileHashID = uTextureHash;
				Tools_SaveAsura::SetCutscenesModified();

				bFoundInBeamTexture = true;
				break;
			}
		}
	}	


	if( !bFoundInBeamTexture )
	{
		char szTemp[1024];	
		Asura_PrintfToBfr( szTemp, "Requested texture (%s) was not found in the beam tiled texture. It must be added to the tiled texture before it can be used.", 1024, true, xTextureName.GetString() );
		
		Asura_Assert( bFoundInBeamTexture, szTemp );
		Tools_Timed_Message_System::AddMessage( 0xff0000, szTemp );
	}
}

void Tools_Cutscene_Event_Beam::EditTailPosition()
{
	Asura_Cutscene_Event_Beam* pxAsuraEvent = (Asura_Cutscene_Event_Beam*) Event;

	//see if the position edit object already exists
	Tools_PositionEdit_Object* pxPositionObject = Tools_PositionEdit_Object::Find(m_uTailPositionEditID);

	if(!pxPositionObject)
	{
		Asura_Vector_3* pxPosition = & pxAsuraEvent->m_xParam.m_xInitialPosition;

		//set a default value for the position if needed
		if(pxPosition->x==0 && pxPosition->y==0 && pxPosition->z==0)
		{
			*pxPosition = Position;
		}

		pxPositionObject = new Tools_PositionEdit_Object(pxPosition,NULL,"Beam Start","Edit beam tail position",&Changed);
		m_uTailPositionEditID = pxPositionObject->GetID();
	}

	Tools_Object_System::SelectObject(pxPositionObject);
	
}

void Tools_Cutscene_Event_Beam::EditHeadPosition()
{
	Asura_Cutscene_Event_Beam* pxAsuraEvent = (Asura_Cutscene_Event_Beam*) Event;

	//see if the position edit object already exists
	Tools_PositionEdit_Object* pxPositionObject = Tools_PositionEdit_Object::Find(m_uHeadPositionEditID);

	if(!pxPositionObject)
	{
		Asura_Vector_3* pxPosition = & pxAsuraEvent->m_xParam.m_xEndPosition;

		//set a default value for the position if needed
		if(pxPosition->x==0 && pxPosition->y==0 && pxPosition->z==0)
		{
			*pxPosition = Position;
		}

		pxPositionObject = new Tools_PositionEdit_Object(pxPosition,NULL,"Beam End","Edit beam head position",&Changed);
		m_uHeadPositionEditID = pxPositionObject->GetID();
	}

	Tools_Object_System::SelectObject(pxPositionObject);
	
}




void Tools_Cutscene_Event_Beam::UpdateBackupVariables()
{
	Tools_Cutscene_Event::UpdateBackupVariables();
	Asura_Cutscene_Event_Beam* pxAsuraEvent = (Asura_Cutscene_Event_Beam*) Event;

	m_xParamBackup = pxAsuraEvent->m_xParam;
}

void Tools_Cutscene_Event_Beam::FillInTimelineData(Tools_Timeline_Data_Event* pxData)
{
	Asura_Cutscene_Event_Beam* pxAsuraEvent = (Asura_Cutscene_Event_Beam*) Event;
	Tools_Cutscene_Event::FillInTimelineData(pxData);

	pxData->Duration = pxAsuraEvent->m_xParam.m_fDuration;
}

void Tools_Cutscene_Event_Beam::SetTotalDuration(float fDuration)
{
	Asura_Cutscene_Event_Beam* pxAsuraEvent = (Asura_Cutscene_Event_Beam*) Event;
	pxAsuraEvent->m_xParam.m_fDuration = fDuration;
}

////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------**
** Tools_Cutscene_Event_Fade **
**---------------------------*/

Tools_Cutscene_Event_Fade::Tools_Cutscene_Event_Fade(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID)
:Tools_Cutscene_Event(pxEvent,pxCutsceneObject,iEventID)
{
}


void Tools_Cutscene_Event_Fade::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Event_Fade* pxFadeEvent = (Asura_Cutscene_Event_Fade*) Event;

	Asura_Dialog* pxDialog;

	pxDialog = new Asura_FlagTickbox_Dialog("Fade To",xOffset,&pxFadeEvent->m_uFlags,ASURACUTSCENEFADEFLAG_FADETO,&Changed);
	pxDialog->SetHelpText("If ticked then the event will fade towards the selected colour.\nOtherwise the fade will be from the colour.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Cancel After Cutscene",xOffset,&pxFadeEvent->m_uFlags,ASURACUTSCENEFADEFLAG_CANCELAFTERCUTSCENE,&Changed);
	pxDialog->SetHelpText("Causes the fade effect to be cancelled automatically once the the cutscene has finished.\n(Note that setting this flag doesn't so much cancel this fade , as cancel any fade that happens to be in progress).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	pxDialog = new Tools_CutsceneTime_Dialog("Duration",xOffset,&pxFadeEvent->m_fDuration,0.0f,1000.0f,0.1f,&Changed);
	pxDialog->SetHelpText("Time over which to fade in/out.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_ColourNumerical_Dialog("Red",xOffset,&pxFadeEvent->m_fRed,"%d",&Changed);
	pxDialog->SetHelpText("Colour to fade to/from.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_ColourNumerical_Dialog("Green",xOffset,&pxFadeEvent->m_fGreen,"%d",&Changed);
	pxDialog->SetHelpText("Colour to fade to/from.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_ColourNumerical_Dialog("Blue",xOffset,&pxFadeEvent->m_fBlue,"%d",&Changed);
	pxDialog->SetHelpText("Colour to fade to/from.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	
}


void Tools_Cutscene_Event_Fade::FillInTimelineData(Tools_Timeline_Data_Event* pxData)
{
	Asura_Cutscene_Event_Fade* pxFadeEvent = (Asura_Cutscene_Event_Fade*) Event;
	Tools_Cutscene_Event::FillInTimelineData(pxData);

	pxData->Duration = pxFadeEvent->m_fDuration;

}

void Tools_Cutscene_Event_Fade::SetTotalDuration(float fDuration)
{
	Asura_Cutscene_Event_Fade* pxAsuraEvent = (Asura_Cutscene_Event_Fade*) Event;
	pxAsuraEvent->m_fDuration = fDuration;
}


Asura_String Tools_Cutscene_Event_Fade::GetDescription()
{
	Asura_Cutscene_Event_Fade* pxFadeEvent = (Asura_Cutscene_Event_Fade*) Event;
	char szBuffer[250];

	if(Tools_Mode_Cutscene::GetTimerMode() == CUTSCENETIMER_SECONDS)
	{
		sprintf(szBuffer,"Fade %s Colour over %.3f seconds",(pxFadeEvent->m_uFlags & ASURACUTSCENEFADEFLAG_FADETO) ? "To" : "From",pxFadeEvent->m_fDuration);
	}
	else
	{
		sprintf(szBuffer,"Fade %s Colour over %.0f frames",(pxFadeEvent->m_uFlags & ASURACUTSCENEFADEFLAG_FADETO) ? "To" : "From",Tools_Mode_Cutscene::ConvertSecondsToDisplayedTime(pxFadeEvent->m_fDuration));
	}

	return szBuffer;

}

void Tools_Cutscene_Event_Fade::UpdateBackupVariables()
{
	Tools_Cutscene_Event::UpdateBackupVariables();
	Asura_Cutscene_Event_Fade* pxFadeEvent = (Asura_Cutscene_Event_Fade*) Event;
	m_fRedBackup = pxFadeEvent->m_fRed;
	m_fGreenBackup = pxFadeEvent->m_fGreen;
	m_fBlueBackup = pxFadeEvent->m_fBlue;
	m_fDurationBackup = pxFadeEvent->m_fDuration;
	m_uFlagsBackup = pxFadeEvent->m_uFlags;
}

const char* Tools_Cutscene_Event_Fade::GetNameOfChangedVariable()
{
	Asura_Cutscene_Event_Fade* pxFadeEvent = (Asura_Cutscene_Event_Fade*) Event;

	if(m_fRedBackup != pxFadeEvent->m_fRed || 
		m_fGreenBackup != pxFadeEvent->m_fGreen || 
		m_fBlueBackup != pxFadeEvent->m_fBlue) return "Colour";

	if(m_fDurationBackup != pxFadeEvent->m_fDuration) return "Duration";
	if(m_uFlagsBackup != pxFadeEvent->m_uFlags)
	{
		u_int uChangedFlags = m_uFlagsBackup ^ pxFadeEvent->m_uFlags;
		if(uChangedFlags & ASURACUTSCENEFADEFLAG_CANCELAFTERCUTSCENE) return "Cancel After Cutscene";
		return "Fade Flags";

	}
	
	return Tools_Cutscene_Event::GetNameOfChangedVariable();
}

////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------------**
** Tools_Cutscene_Event_TemplateWrapper **
**--------------------------------------*/

Tools_Cutscene_Event_TemplateWrapper::Tools_Cutscene_Event_TemplateWrapper
(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID)
: Tools_Cutscene_Event(pxEvent, pxCutsceneObject, iEventID)
{
	Asura_Cutscene_Event_TemplateWrapper* pxAsuraEvent = reinterpret_cast<Asura_Cutscene_Event_TemplateWrapper*>(pxEvent);
	m_fTimeOffset = pxAsuraEvent->m_fTriggerTime;
}

void Tools_Cutscene_Event_TemplateWrapper::AddOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;

	pxDialog = new Asura_Function_Dialog("Unwrap events",xOffset,UnwrapEvents_Static,(int)this,&Changed);
	pxDialog->SetHelpText("Makes all the events from this template individually editable.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
}

void Tools_Cutscene_Event_TemplateWrapper::Destroy()
{
	// first destroy any events contained in the wrapper.
	List<Tools_Cutscene_Event*> xDestroyList;
	for(LIF<Tools_Cutscene_Event*> elif(&CutsceneObject->EventList);!elif.done();elif.next())
	{
		if (elif()->Event->m_xNameOfEventWrapper == Event->m_xName)
		{
			xDestroyList.add_entry(elif());
		}
	}

	//destroy all the objects in the list
	while(xDestroyList.size())
	{
		//get the next object to destroy
		Tools_Cutscene_Event* pxEvent = xDestroyList.first_entry();
		xDestroyList.delete_first_entry();
		// check it's still in the list (events could delete other events)
		if (!CutsceneObject->EventList.contains(pxEvent))
		{
			continue;
		}
		// destroy the event
		pxEvent->Destroy();
	}

	// call base class function to destroy this event object.
	Tools_Cutscene_Event::Destroy();
}


void Tools_Cutscene_Event_TemplateWrapper::FillInTimelineData(Tools_Timeline_Data_Event* data)
{
	Asura_Assert(!Event->m_xName.IsBlank(), "template wrappers should have a name");
	Tools_Cutscene_Event::FillInTimelineData(data);

	// set the start and duration so the event is as long as all the events contained.
	float fStartTime = Asura_Maths::LargeFloat;
	float fEndTime = 0;
	for(LIF<Tools_Cutscene_Event*> elif(&CutsceneObject->EventList);!elif.done();elif.next())
	{
		Tools_Cutscene_Event* pxToolsEvent = elif();
		if (pxToolsEvent->Event->m_xNameOfEventWrapper == Event->m_xName)
		{
			if (pxToolsEvent->Event->m_iType == ACET_TemplateWrapper)
			{
				Asura_Assert(false, "unexpected to find template wrapper inside another one");
				continue;
			}

			// create dummy timeline data for this event.
			Tools_Timeline_Data_Event xTimelineData(pxToolsEvent);
			pxToolsEvent->FillInTimelineData(&xTimelineData);
			if (xTimelineData.Time < fStartTime)
			{
				fStartTime = xTimelineData.Time;
			}
			if (fEndTime < xTimelineData.Time + xTimelineData.Duration)
			{
				fEndTime = xTimelineData.Time + xTimelineData.Duration;
			}
		}
	}

	Asura_Assert(fStartTime != Asura_Maths::LargeFloat, "invalid event start time");

	data->Time = fStartTime;
    data->Duration = fEndTime - fStartTime;
}

void Tools_Cutscene_Event_TemplateWrapper::SetNewTriggerTime(float fTriggerTime)
{
	Tools_Cutscene_Event::SetNewTriggerTime(fTriggerTime);
	UpdateWrappedEventTriggerTimes();
}

void Tools_Cutscene_Event_TemplateWrapper::UpdateWrappedEventTriggerTimes()
{
	// see if we need to update the trigger times for the wrapped events
	if (m_fTimeOffset != Event->m_fTriggerTime)
	{
		float fTimeToChangeOnEvents = Event->m_fTriggerTime - m_fTimeOffset;

		for(LIF<Tools_Cutscene_Event*> elif(&CutsceneObject->EventList);!elif.done();elif.next())
		{
			Tools_Cutscene_Event* pxToolsEvent = elif();
			if (pxToolsEvent->Event->m_xNameOfEventWrapper == Event->m_xName)
			{
				pxToolsEvent->Event->m_fTriggerTime += fTimeToChangeOnEvents;
			}
		}

		// update the template asura event, now that all the wrapped events are updated.
		m_fTimeOffset = Event->m_fTriggerTime;
	}
}

void Tools_Cutscene_Event_TemplateWrapper::Update()
{
	UpdateWrappedEventTriggerTimes();
	Tools_Cutscene_Event::Update();
}

void Tools_Cutscene_Event_TemplateWrapper::UnwrapEvents()
{
	// confirmation dialog
	YesNo_Overlay dlg("Unwrap the events?");
	if(!dlg.DoModal()) return;

	// clear the name of event wrapper and source template from wrapped events
	for(LIF<Tools_Cutscene_Event*> elif(&CutsceneObject->EventList);!elif.done();elif.next())
	{
		Tools_Cutscene_Event* pxToolsEvent = elif();
		if (pxToolsEvent->Event->m_xNameOfEventWrapper == Event->m_xName)
		{
			pxToolsEvent->Event->m_xNameOfEventWrapper.MakeBlank();
			pxToolsEvent->Event->m_xNameOfSourceTemplate.MakeBlank();
			if(CutsceneUndo) CutsceneUndo->Log_Event_Changed(pxToolsEvent);
		}
	}
	// destroy the wrapper
	Destroy();
}

////////////////////////////////////////////////////////////////////////////////////////////
/*-------------------------------**
** Tools_Cutscene_Event_Subtitle **
**-------------------------------*/

Tools_Cutscene_Event_Subtitle::Tools_Cutscene_Event_Subtitle(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID)
:Tools_Cutscene_Event(pxEvent,pxCutsceneObject,iEventID)
{
}


void Tools_Cutscene_Event_Subtitle::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Event_Subtitle* pxSubtitleEvent = (Asura_Cutscene_Event_Subtitle*) Event;

	Asura_Dialog* pxDialog;

	//get TextId label
	char szText[200];
	Asura_PrintfToBfr(szText,"Text ID: %s",200,true,Tools_Cutscene_Subtitles::GetSubtitleID(pxSubtitleEvent->m_uTextHash));

	//get the full text as a help string
	char szHelpText[500] = "";
	if(pxSubtitleEvent->m_uTextHash)
	{
		const Asura_UnicodeChar* wszText = Asura_HashedLocalisedText::GetString(uASURA_HASHEDTEXTPAGE_CUTSCENE, pxSubtitleEvent->m_uTextHash);
	    
		if(wszText)
		{
			Asura_PrintfToBfr(szHelpText,"%S",500,true,wszText);
		}
	}

	
	pxDialog  = new Asura_Function_Dialog(szText,xOffset,_SelectSubtitle);
	pxDialog->SetHelpText(szHelpText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	pxDialog = new Tools_CutsceneTime_Dialog("Duration",xOffset,&pxSubtitleEvent->m_fDuration,0.0f,1000.0f,0.1f,&Changed);
	pxDialog->SetHelpText("Duration of subtitle.\nIf set to zero then it will last until replaced by another subtitle.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog  = new Asura_Function_Dialog("Set Subtitle End",xOffset,_SetSubtitleEnd);
	pxDialog->SetHelpText("Sets the duration so that the subtitle will end at the current time on the timeline slider.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	
}


void Tools_Cutscene_Event_Subtitle::FillInTimelineData(Tools_Timeline_Data_Event* pxData)
{
	Asura_Cutscene_Event_Subtitle* pxSubtitleEvent = (Asura_Cutscene_Event_Subtitle*) Event;
	Tools_Cutscene_Event::FillInTimelineData(pxData);

	pxData->Duration = pxSubtitleEvent->m_fDuration;

	//get trigger time of next subtitle event
	Asura_Cutscene* pxCutscene = Event->m_pxCutscene;
	bool bFoundThis = false;
	float fTime = pxCutscene->CutsceneEndTime;
	for(int i=0;i<pxCutscene->NumberOfEvents;i++)
	{
		if(!bFoundThis)
		{
			//found this event , so want to get the next subtitle event after it
			if(pxCutscene->Events[i] == Event)
			{
				bFoundThis = true;
			}
		}
		else
		{
			if(pxCutscene->Events[i]->m_iType == ACET_Subtitle)
			{
				fTime = pxCutscene->Events[i]->m_fTriggerTime;
				break;
			}
		}
	}
	if(fTime)
	{
		float fMaxDuration = fTime - Event->m_fTriggerTime;
		if(pxData->Duration == 0 || pxData->Duration > fMaxDuration)
		{
			pxData->Duration = fMaxDuration;
		}
	}

}

void Tools_Cutscene_Event_Subtitle::SetTotalDuration(float fDuration)
{
	Asura_Cutscene_Event_Subtitle* pxSubtitleEvent = (Asura_Cutscene_Event_Subtitle*) Event;
	pxSubtitleEvent->m_fDuration = fDuration;

}


Asura_String Tools_Cutscene_Event_Subtitle::GetDescription()
{
	Asura_Cutscene_Event_Subtitle* pxSubtitleEvent = (Asura_Cutscene_Event_Subtitle*) Event;
	
	char szBuffer[250];
	Asura_PrintfToBfr(szBuffer,"Subtitle : %s",250,true,Tools_Cutscene_Subtitles::GetSubtitleID(pxSubtitleEvent->m_uTextHash));

	return szBuffer;
}

void Tools_Cutscene_Event_Subtitle::UpdateBackupVariables()
{
	Tools_Cutscene_Event::UpdateBackupVariables();
	Asura_Cutscene_Event_Subtitle* pxSubtitleEvent = (Asura_Cutscene_Event_Subtitle*) Event;

	m_fDurationBackup = pxSubtitleEvent->m_fDuration;
	m_uTextHashBackup = pxSubtitleEvent->m_uTextHash;
}

const char* Tools_Cutscene_Event_Subtitle::GetNameOfChangedVariable()
{
	Asura_Cutscene_Event_Subtitle* pxSubtitleEvent = (Asura_Cutscene_Event_Subtitle*) Event;

	if(m_fDurationBackup != pxSubtitleEvent->m_fDuration) return "Duration";
	if(m_uTextHashBackup != pxSubtitleEvent->m_uTextHash) return "Text ID";
	
	return Tools_Cutscene_Event::GetNameOfChangedVariable();
}

void Tools_Cutscene_Event_Subtitle::AskForRequiredData()
{
	//select for the subtitle
	SelectSubtitle();
}

void Tools_Cutscene_Event_Subtitle::SelectSubtitle()
{
	Asura_Cutscene_Event_Subtitle* pxAsuraEvent = (Asura_Cutscene_Event_Subtitle*) Event;

	//do the subtitle selection overlay thing
	Asura_Hash_ID uTextHash = Tools_Cutscene_Subtitles::SelectSubtitle(pxAsuraEvent->m_uTextHash);

	//update if changed
	if(uTextHash != pxAsuraEvent->m_uTextHash)
	{
		pxAsuraEvent->m_uTextHash = uTextHash;
		Changed = true;
		
		if(ThisEvent == this)
		{
			//overlay will need updating
			DeleteOverlay();
			CreateOverlay();
		}	
	}
}

void Tools_Cutscene_Event_Subtitle::SetSubtitleEnd()
{
	Asura_Cutscene_Event_Subtitle* pxAsuraEvent = (Asura_Cutscene_Event_Subtitle*) Event;
	
	float fTimer = Event->m_pxCutscene->Timer;
	if(fTimer > pxAsuraEvent->m_fTriggerTime)
	{
		pxAsuraEvent->m_fDuration = fTimer - pxAsuraEvent->m_fTriggerTime;
		Changed = true;
	}
	else
	{
		ShowMessageOverlay("Can't set the end time to be before the trigger time.");
	}
}


////////////////////////////////////////////////////////////////////////////////////////////
/*----------------------------**
** Tools_Cutscene_Event_Sound **
**----------------------------*/

Tools_Cutscene_Event_Sound::Tools_Cutscene_Event_Sound(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID)
:Tools_Cutscene_Event(pxEvent,pxCutsceneObject,iEventID)
{
}


void Tools_Cutscene_Event_Sound::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Event_Sound* pxSoundEvent = (Asura_Cutscene_Event_Sound*) Event;

	Asura_Dialog* pxDialog;

	char szText[200];

	//add actor selection control
	if(pxSoundEvent->m_xTargetActorName.GetLength())
		sprintf(szText,"Actor : %s",pxSoundEvent->m_xTargetActorName.GetString());
	else
		strcpy(szText,"Actor : *None*");

	pxDialog = new Asura_Function_Dialog(szText,xOffset,ChooseActor);
	pxDialog->SetHelpText("Select actor that sound should come from.");
	pxDialog->m_iControlID = DialogID_Actor;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	//get Sound label
	
	if(pxSoundEvent->m_xSoundName.IsBlank())
	{
		strcpy(szText,"Sound: *None*");
	}
	else
	{
		Asura_PrintfToBfr(szText,"Sound: %s",200,true,Tools_FileSystem::StripPath(pxSoundEvent->m_xSoundName.GetString()));
	}

	pxDialog  = new Asura_Function_Dialog(szText,xOffset,_SelectSound);
	pxDialog->SetHelpText("Select the wav file to use");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	
}


Asura_String Tools_Cutscene_Event_Sound::GetDescription()
{
	Asura_Cutscene_Event_Sound* pxSoundEvent = (Asura_Cutscene_Event_Sound*) Event;
	
	char szBuffer[250];
	Asura_PrintfToBfr(szBuffer,"Sound : %s",250,true,pxSoundEvent->m_xSoundName.GetString());

	return szBuffer;
}

void Tools_Cutscene_Event_Sound::UpdateBackupVariables()
{
	Tools_Cutscene_Event::UpdateBackupVariables();
	Asura_Cutscene_Event_Sound* pxSoundEvent = (Asura_Cutscene_Event_Sound*) Event;

	m_xSoundNameBackup = pxSoundEvent->m_xSoundName;
}

const char* Tools_Cutscene_Event_Sound::GetNameOfChangedVariable()
{
	Asura_Cutscene_Event_Sound* pxSoundEvent = (Asura_Cutscene_Event_Sound*) Event;

	if(!m_xSoundNameBackup.Compare(pxSoundEvent->m_xSoundName)) return "Sound";
	
	return Tools_Cutscene_Event::GetNameOfChangedVariable();
}

void Tools_Cutscene_Event_Sound::AskForRequiredData()
{
	Asura_Cutscene_Event_Sound* pxSoundEvent = (Asura_Cutscene_Event_Sound*) Event;
	pxSoundEvent->m_xSoundName = "";//initialise to empty string instead of null
	
	SelectSound();
}

void Tools_Cutscene_Event_Sound::SelectSound()
{
	Asura_Cutscene_Event_Sound* pxAsuraEvent = (Asura_Cutscene_Event_Sound*) Event;

	#ifdef ASURA_USE_AUDIO_SYSTEM

		//determine current ID (if any)		
		Asura_Hash_ID uWave = ASURA_HASH_ID_UNSET;
		if(!pxAsuraEvent->m_xSoundName.IsBlank())
		{
			uWave = pxAsuraEvent->m_xSoundName.GetHash();
		}

		Asura_Hash_ID uNewWave = Tools_PickSound::PickSoundFromDisk(uWave,NULL,"Pick Wav File");
		if(uNewWave == uWave) return;

		//get the filename for the new file
		if(uNewWave == ASURA_HASH_ID_UNSET)
		{
			pxAsuraEvent->m_xSoundName.MakeBlank();
		}
		else
		{
			pxAsuraEvent->m_xSoundName = Asura_Audio_System::GetWaveFilename(uNewWave);
		}

	#else

		//determine current ID (if any)	
		int iSoundID = ASURA_SID_NO_SOUND;
		if(!pxAsuraEvent->m_xSoundName.IsBlank())
		{
			iSoundID = Tools_PickSound::GetSoundIDForFilename(pxAsuraEvent->m_xSoundName.GetString(),TOOLS_SID_MIN,TOOLS_SID_MAX);
		}

		int iNewSoundID = Tools_PickSound::PickSoundFromDisk((Asura_Sound_ID)iSoundID,TOOLS_SID_MIN, TOOLS_SID_MAX,NULL,"Pick Wav File");
		if(iNewSoundID == iSoundID) return;

		//get the filename for the new file
		if(iNewSoundID == ASURA_SID_NO_SOUND)
		{
			pxAsuraEvent->m_xSoundName.MakeBlank();
		}
		else
		{
			pxAsuraEvent->m_xSoundName = Asura_Sound_System::GetFileNameFromSoundID((Asura_Sound_ID)iNewSoundID);
		}

	#endif

	Changed = true;

	if(ThisEvent == this)
	{
		//overlay will need updating
		DeleteOverlay();
		CreateOverlay();
	}	
}

////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------------**
** Tools_Cutscene_Event_ChangeCamera **
**-----------------------------------*/

Tools_Cutscene_Event_ChangeCamera::Tools_Cutscene_Event_ChangeCamera(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID)
:Tools_Cutscene_Event(pxEvent,pxCutsceneObject,iEventID)
{
}


void Tools_Cutscene_Event_ChangeCamera::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Event_Attachment* pxAsuraEvent = (Asura_Cutscene_Event_Attachment*) Event;

	Asura_Dialog* pxDialog;

	char szText[200];

	//add actor selection control
	if(pxAsuraEvent->m_xTargetActorName.GetLength())
		sprintf(szText,"Actor : %s",pxAsuraEvent->m_xTargetActorName.GetString());
	else
		strcpy(szText,"Actor : *None*");

    pxDialog = new Asura_Function_Dialog(szText,xOffset,Tools_Cutscene_Event::ChooseActor);
	pxDialog->SetHelpText("Choose camera to change to.");
	pxDialog->m_iControlID = DialogID_Actor;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
}


Asura_String Tools_Cutscene_Event_ChangeCamera::GetDescription()
{
	char szBuffer[250];
	Asura_PrintfToBfr(szBuffer,"Change Camera : %s",250,true,Event->m_xTargetActorName.GetString());

	return szBuffer;
}


void Tools_Cutscene_Event_ChangeCamera::AskForRequiredData()
{
	//select the camera
	ChooseActor();
}

bool Tools_Cutscene_Event_ChangeCamera::ActorIsValidTarget(Asura_Cutscene_Actor* pxActor)
{
	return pxActor->IsCamera();
}

////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------------------**
** Tools_Cutscene_Event_ChangeCameraTarget **
**-----------------------------------------*/

Tools_Cutscene_Event_ChangeCameraTarget::Tools_Cutscene_Event_ChangeCameraTarget(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID)
:Tools_Cutscene_Event(pxEvent,pxCutsceneObject,iEventID)
{
}


void Tools_Cutscene_Event_ChangeCameraTarget::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Event_ChangeCameraTarget* pxAsuraEvent = (Asura_Cutscene_Event_ChangeCameraTarget*) Event;


	Asura_Dialog* pxDialog;

	char szText[200];

	//add actor selection control
	if(pxAsuraEvent->m_xTargetActorName.GetLength())
		sprintf(szText,"Actor : %s",pxAsuraEvent->m_xTargetActorName.GetString());
	else
		strcpy(szText,"Actor : *None*");

    pxDialog = new Asura_Function_Dialog(szText,xOffset,Tools_Cutscene_Event::ChooseActor);
	pxDialog->SetHelpText("Choose camera to change to.");
	pxDialog->m_iControlID = DialogID_Actor;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//and camera target selection
	if(pxAsuraEvent->m_xCameraTargetName.GetLength())
		sprintf(szText,"Target : %s",pxAsuraEvent->m_xCameraTargetName.GetString());
	else
		strcpy(szText,"Target : *None*");

    pxDialog = new Asura_Function_Dialog(szText,xOffset,_SelectCameraTarget);
	pxDialog->SetHelpText("Choose the new camera target for the camera");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

}

Asura_String Tools_Cutscene_Event_ChangeCameraTarget::GetDescription()
{
	Asura_Cutscene_Event_ChangeCameraTarget* pxEvent = (Asura_Cutscene_Event_ChangeCameraTarget*) Event;
	
	char szBuffer[250];
	const char* pszTarget = pxEvent->m_xCameraTargetName.IsBlank() ? "None" : pxEvent->m_xCameraTargetName.GetString();
	Asura_PrintfToBfr(szBuffer,"Change camera target : Camera '%s' to target '%s'",250,true,pxEvent->m_xTargetActorName.GetString(),pszTarget);

	return szBuffer;
}

void Tools_Cutscene_Event_ChangeCameraTarget::UpdateBackupVariables()
{
	Tools_Cutscene_Event::UpdateBackupVariables();
	Asura_Cutscene_Event_ChangeCameraTarget* pxChangeCameraTargetEvent = (Asura_Cutscene_Event_ChangeCameraTarget*) Event;

	m_xCameraTargetNameBackup = pxChangeCameraTargetEvent->m_xCameraTargetName;
}

const char* Tools_Cutscene_Event_ChangeCameraTarget::GetNameOfChangedVariable()
{
	Asura_Cutscene_Event_ChangeCameraTarget* pxChangeCameraTargetEvent = (Asura_Cutscene_Event_ChangeCameraTarget*) Event;

	if(m_xCameraTargetNameBackup != pxChangeCameraTargetEvent->m_xCameraTargetName) return "Camera Target";
	
	return Tools_Cutscene_Event::GetNameOfChangedVariable();
}

void Tools_Cutscene_Event_ChangeCameraTarget::AskForRequiredData()
{
	//select the camera
	ChooseActor();
	//and the target
	SelectCameraTarget();
}

void Tools_Cutscene_Event_ChangeCameraTarget::SelectCameraTarget()
{
	Asura_Cutscene_Event_ChangeCameraTarget* pxAsuraEvent = (Asura_Cutscene_Event_ChangeCameraTarget*) Event;

	//build a list overlay of camera targets.
	ListSelection_Overlay xOverlay("Select Camera Target");
	
	for(LIF<Tools_CutsceneActor_Object*> alif(&ThisEvent->CutsceneObject->ActorObjects);!alif.done();alif.next())
	{
		if(!alif()->IsCameraTargetActor()) continue;

		const char* pszName = alif()->Actor->ActorName.GetString();
		bool bSelected = false;
		if(pxAsuraEvent->m_xCameraTargetName.GetString())
		{
			bSelected = !strcmp(pszName,pxAsuraEvent->m_xCameraTargetName.GetString());
		}

		xOverlay.AddItem(pszName,(int)alif()->Actor,bSelected);
	}

	//add no target option
	xOverlay.AddItem("*None*",0,!pxAsuraEvent->m_xCameraTargetName.GetString());

	//show the list overlay
	if(!xOverlay.DoModal()) return;

	//update if changed
	if(xOverlay.GetSelectionData())
	{
		pxAsuraEvent->m_xCameraTargetName = xOverlay.GetSelectionString();
	}
	else
	{
		pxAsuraEvent->m_xCameraTargetName.MakeNull();
	}
	Changed = true;
	
	if(ThisEvent == this)
	{
		//overlay will need updating
		DeleteOverlay();
		CreateOverlay();
	}	
}

bool Tools_Cutscene_Event_ChangeCameraTarget::ActorIsValidTarget(Asura_Cutscene_Actor* pxActor)
{
	return pxActor->IsCamera();
}

////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------------**
** Tools_Cutscene_Event_Hide		 **
**-----------------------------------*/

Tools_Cutscene_Event_Hide::Tools_Cutscene_Event_Hide(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID)
:Tools_Cutscene_Event(pxEvent,pxCutsceneObject,iEventID)
{
}


void Tools_Cutscene_Event_Hide::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Event_Attachment* pxAsuraEvent = (Asura_Cutscene_Event_Attachment*) Event;

	Asura_Dialog* pxDialog;

	char szText[200];

	//add actor selection control
	if(pxAsuraEvent->m_xTargetActorName.GetLength())
	{
		sprintf(szText,"Actor : %s",pxAsuraEvent->m_xTargetActorName.GetString());
	}
	else
	{
		strcpy(szText,"Actor : *None*");
	}

	pxDialog = new Asura_Function_Dialog(szText,xOffset,Tools_Cutscene_Event::ChooseActor);
	pxDialog->SetHelpText("Choose actor.");
	pxDialog->m_iControlID = DialogID_Actor;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
}


Asura_String Tools_Cutscene_Event_Hide::GetDescription()
{
	char szBuffer[250];
	Asura_PrintfToBfr(szBuffer,"Hide : %s",250,true,Event->m_xTargetActorName.GetString());

	return szBuffer;
}


void Tools_Cutscene_Event_Hide::AskForRequiredData()
{
	//select the camera
	ChooseActor();
}

bool Tools_Cutscene_Event_Hide::ActorIsValidTarget(Asura_Cutscene_Actor* pxActor)
{
	return pxActor->IsHierarchyActor();
}

/*-----------------------------------**
** Tools_Cutscene_Event_Unhide		 **
**-----------------------------------*/

Tools_Cutscene_Event_Unhide::Tools_Cutscene_Event_Unhide(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID)
:Tools_Cutscene_Event(pxEvent,pxCutsceneObject,iEventID)
{
}


void Tools_Cutscene_Event_Unhide::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Event_Attachment* pxAsuraEvent = (Asura_Cutscene_Event_Attachment*) Event;

	Asura_Dialog* pxDialog;

	char szText[200];

	//add actor selection control
	if(pxAsuraEvent->m_xTargetActorName.GetLength())
	{
		sprintf(szText,"Actor : %s",pxAsuraEvent->m_xTargetActorName.GetString());
	}
	else
	{
		strcpy(szText,"Actor : *None*");
	}

	pxDialog = new Asura_Function_Dialog(szText,xOffset,Tools_Cutscene_Event::ChooseActor);
	pxDialog->SetHelpText("Choose actor.");
	pxDialog->m_iControlID = DialogID_Actor;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
}


Asura_String Tools_Cutscene_Event_Unhide::GetDescription()
{
	char szBuffer[250];
	Asura_PrintfToBfr(szBuffer,"Hide : %s",250,true,Event->m_xTargetActorName.GetString());

	return szBuffer;
}


void Tools_Cutscene_Event_Unhide::AskForRequiredData()
{
	//select the camera
	ChooseActor();
}

bool Tools_Cutscene_Event_Unhide::ActorIsValidTarget(Asura_Cutscene_Actor* pxActor)
{
	return pxActor->IsHierarchyActor();
}

////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------------------**
** Tools_Cutscene_Event_ChangeCameraShake **
**-----------------------------------------*/

Tools_Cutscene_Event_ChangeCameraShake::Tools_Cutscene_Event_ChangeCameraShake(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID)
:Tools_Cutscene_Event(pxEvent,pxCutsceneObject,iEventID)
{
}


void Tools_Cutscene_Event_ChangeCameraShake::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Event_ChangeCameraShake* pxAsuraEvent = (Asura_Cutscene_Event_ChangeCameraShake*) Event;

	Asura_Dialog* pxDialog = NULL;
	char szText[200];

	//add actor selection control
	if(pxAsuraEvent->m_xTargetActorName.GetLength())
	{
		sprintf(szText,"Camera : %s",pxAsuraEvent->m_xTargetActorName.GetString());
	}
	else
	{
		strcpy(szText,"Camera : *None*");
	}

	pxDialog = new Asura_Function_Dialog(szText,xOffset,Tools_Cutscene_Event::ChooseActor);
	pxDialog->SetHelpText("Choose camera to change shake on");
	pxDialog->m_iControlID = DialogID_Actor;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Change duration", xOffset, &pxAsuraEvent->m_fDuration, 0.f, 999.f, 0.1f, "%3.1f", &Changed);
	pxDialog->SetHelpText("Set time taken for shake to blend from current to target parameters");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FunctionTickbox_Dialog( "Enabled", xOffset, _IsShakeEnabled, _SetShakeEnabled);
	pxDialog->SetHelpText("Whether camera shake is enabled.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog( "Frequency", xOffset, &pxAsuraEvent->m_xShake.m_fFrequency, 0.1f,20.0f,0.1f,"%.2f", &Changed);
	pxDialog->SetHelpText("How quickly the camera jitters around.");
	pxDialog->SetIsEnabledFunction(_IsShakeEnabled);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Pitch",xOffset,&pxAsuraEvent->m_xShake.m_fPitch,0.0f,4.0f,0.05f,"%.2f",&Changed);
	pxDialog->SetHelpText("How much the camera pitches.");
	pxDialog->SetIsEnabledFunction(_IsShakeEnabled);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Yaw",xOffset,&pxAsuraEvent->m_xShake.m_fYaw,0.0f,4.0f,0.05f,"%.2f",&Changed);
	pxDialog->SetHelpText("How much the camera yaws.");
	pxDialog->SetIsEnabledFunction(_IsShakeEnabled);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Roll",xOffset,&pxAsuraEvent->m_xShake.m_fRoll,0.0f,4.0f,0.05f,"%.2f",&Changed);
	pxDialog->SetHelpText("How much the camera rolls.");
	pxDialog->SetIsEnabledFunction(_IsShakeEnabled);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Speed Scale",xOffset,&pxAsuraEvent->m_xShake.m_fSpeedScale,0.0f,50.0f,0.1f,"%.2f",&Changed);
	pxDialog->SetHelpText("How much the camera's movement is affected by the speed of the camera.\nSmaller numbers mean the camera needs to move faster before the camera will reach full wobbliness.");
	pxDialog->SetIsEnabledFunction(_IsShakeEnabled);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Min Jitter",xOffset,&pxAsuraEvent->m_xShake.m_fMinScale,0.0f,1.0f,0.05f,"%.2f",&Changed);
	pxDialog->SetHelpText("The minimum amount of jitter that will be applied to the camera.\nIf 0, the camera will be motionless when stationary.\nWhen 1, the camera will always be at full wobbliness, regardless of how slow it's moving.");
	pxDialog->SetIsEnabledFunction(_IsShakeEnabled);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
}

bool Tools_Cutscene_Event_ChangeCameraShake::_IsShakeEnabled()
{
	return static_cast<Tools_Cutscene_Event_ChangeCameraShake*>(ThisEvent)->IsShakeEnabled();
}

bool Tools_Cutscene_Event_ChangeCameraShake::IsShakeEnabled()
{
	Asura_Cutscene_Event_ChangeCameraShake* pxAsuraEvent = (Asura_Cutscene_Event_ChangeCameraShake*) Event;
	return pxAsuraEvent->m_xShake.IsEnabled();
}

void Tools_Cutscene_Event_ChangeCameraShake::_SetShakeEnabled(bool bShakeEnabled)
{
	static_cast<Tools_Cutscene_Event_ChangeCameraShake*>(ThisEvent)->SetShakeEnabled(bShakeEnabled);
}

void Tools_Cutscene_Event_ChangeCameraShake::SetShakeEnabled(bool bShakeEnabled)
{
	Asura_Cutscene_Event_ChangeCameraShake* pxAsuraEvent = (Asura_Cutscene_Event_ChangeCameraShake*) Event;
	pxAsuraEvent->m_xShake.SetEnabled(bShakeEnabled);
	Changed = true;
}

Asura_String Tools_Cutscene_Event_ChangeCameraShake::GetDescription()
{
	Asura_Cutscene_Event_ChangeCameraShake* pxEvent = (Asura_Cutscene_Event_ChangeCameraShake*) Event;

	char szBuffer[250];
	Asura_PrintfToBfr(szBuffer,"Change camera shake: '%s'",250,true,pxEvent->m_xTargetActorName.GetString());

	return szBuffer;
}

void Tools_Cutscene_Event_ChangeCameraShake::UpdateBackupVariables()
{
	Tools_Cutscene_Event::UpdateBackupVariables();
	Asura_Cutscene_Event_ChangeCameraShake* pxChangeCameraShakeEvent = (Asura_Cutscene_Event_ChangeCameraShake*) Event;

	m_xShakeBackup = pxChangeCameraShakeEvent->m_xShake;
	m_fDurationBackup = pxChangeCameraShakeEvent->m_fDuration;
}

const char* Tools_Cutscene_Event_ChangeCameraShake::GetNameOfChangedVariable()
{
	Asura_Cutscene_Event_ChangeCameraShake* pxChangeCameraShakeEvent = (Asura_Cutscene_Event_ChangeCameraShake*) Event;

	if(m_fDurationBackup != pxChangeCameraShakeEvent->m_fDuration) return "Duration";
	if(m_xShakeBackup.IsEnabled() != pxChangeCameraShakeEvent->m_xShake.IsEnabled()) return "Shake enabled";
	if(m_xShakeBackup.GetFrequency() != pxChangeCameraShakeEvent->m_xShake.GetFrequency()) return "Shake frequency";
	if(m_xShakeBackup.GetPitch() != pxChangeCameraShakeEvent->m_xShake.GetPitch()) return "Shake pitch";
	if(m_xShakeBackup.GetYaw() != pxChangeCameraShakeEvent->m_xShake.GetYaw()) return "Shake yaw";
	if(m_xShakeBackup.GetRoll() != pxChangeCameraShakeEvent->m_xShake.GetRoll()) return "Shake roll";
	if(m_xShakeBackup.GetSpeedScale() != pxChangeCameraShakeEvent->m_xShake.GetSpeedScale()) return "Shake speed scale";
	if(m_xShakeBackup.GetMinScale() != pxChangeCameraShakeEvent->m_xShake.GetMinScale()) return "Shake min jitter";

	return Tools_Cutscene_Event::GetNameOfChangedVariable();
}

void Tools_Cutscene_Event_ChangeCameraShake::AskForRequiredData()
{
	//select the camera
	ChooseActor();
}

bool Tools_Cutscene_Event_ChangeCameraShake::ActorIsValidTarget(Asura_Cutscene_Actor* pxActor)
{
	return pxActor->IsCamera();
}

void Tools_Cutscene_Event_ChangeCameraShake::FillInTimelineData(Tools_Timeline_Data_Event* pxData)
{
	Asura_Cutscene_Event_ChangeCameraShake* pxAsuraEvent = (Asura_Cutscene_Event_ChangeCameraShake*) Event;
	Tools_Cutscene_Event::FillInTimelineData(pxData);

	pxData->Duration = pxAsuraEvent->m_fDuration;
}

void Tools_Cutscene_Event_ChangeCameraShake::SetTotalDuration(float fDuration)
{
	Asura_Cutscene_Event_ChangeCameraShake* pxAsuraEvent = (Asura_Cutscene_Event_ChangeCameraShake*) Event;
	pxAsuraEvent->m_fDuration = fDuration;
}


/*-----------------------------------**
** Tools_Cutscene_Event_SetAlpha	 **
**-----------------------------------*/

Tools_Cutscene_Event_SetAlpha::Tools_Cutscene_Event_SetAlpha(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID)
:Tools_Cutscene_Event(pxEvent,pxCutsceneObject,iEventID)
{
}


void Tools_Cutscene_Event_SetAlpha::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Event_SetAlpha* pxAsuraEvent = (Asura_Cutscene_Event_SetAlpha*) Event;

	Asura_Dialog* pxDialog;

	char szText[200];

	//add actor selection control
	if(pxAsuraEvent->m_xTargetActorName.GetLength())
	{
		sprintf(szText,"Actor : %s",pxAsuraEvent->m_xTargetActorName.GetString());
	}
	else
	{
		strcpy(szText,"Actor : *None*");
	}

	pxDialog = new Asura_Function_Dialog(szText,xOffset,Tools_Cutscene_Event::ChooseActor);
	pxDialog->SetHelpText("Choose actor.");
	pxDialog->m_iControlID = DialogID_Actor;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	// duration
	pxDialog = new Asura_Numerical_Dialog("Duration", xOffset, &pxAsuraEvent->m_fDuration, 0.f, 999.f, 0.1f, "%3.1f", &Changed);
	pxDialog->SetHelpText("Set the duration of the event.");
	pxOverlay->AddDialog(pxDialog); 
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// add graph dialog for alpha 
	Asura_Graph_Dialog<float>* pxGraphDialog = new Asura_Graph_Dialog<float>( "Alpha", xOffset, &pxAsuraEvent->m_xAlpha, 0.0f, 1.0f, &Changed );
	pxGraphDialog->SetHelpText("Alpha over time");
	pxOverlay->AddDialog(pxGraphDialog);
	xOffset.y = pxGraphDialog->m_xMaxCoord.y;
}


Asura_String Tools_Cutscene_Event_SetAlpha::GetDescription()
{
	char szBuffer[250];
	Asura_PrintfToBfr(szBuffer,"SetAlpha : %s",250,true,Event->m_xTargetActorName.GetString());

	return szBuffer;
}


void Tools_Cutscene_Event_SetAlpha::AskForRequiredData()
{
	//select the camera
	ChooseActor();
}

bool Tools_Cutscene_Event_SetAlpha::ActorIsValidTarget(Asura_Cutscene_Actor* pxActor)
{
	return (pxActor->Type == ACAT_Hierarchy) || (pxActor->Type == ACAT_Shape);
}

void Tools_Cutscene_Event_SetAlpha::UpdateBackupVariables()
{
	Tools_Cutscene_Event::UpdateBackupVariables();
	Asura_Cutscene_Event_SetAlpha* pxSetAlphaEvent = (Asura_Cutscene_Event_SetAlpha*) Event;

	m_xAlphaBackup = pxSetAlphaEvent->m_xAlpha;
	m_fDurationBackup = pxSetAlphaEvent->m_fDuration;
}

const char* Tools_Cutscene_Event_SetAlpha::GetNameOfChangedVariable()
{
	Asura_Cutscene_Event_SetAlpha* pxSetAlphaEvent = (Asura_Cutscene_Event_SetAlpha*) Event;

	if(m_fDurationBackup != pxSetAlphaEvent->m_fDuration) return "Duration";
	if(m_xAlphaBackup.GetAreaUnderGraph(1.0f) != pxSetAlphaEvent->m_xAlpha.GetAreaUnderGraph(1.0f)) return "Alpha";

	return Tools_Cutscene_Event::GetNameOfChangedVariable();
}

void Tools_Cutscene_Event_SetAlpha::FillInTimelineData(Tools_Timeline_Data_Event* pxData)
{
	Asura_Cutscene_Event_SetAlpha* pxAsuraEvent = (Asura_Cutscene_Event_SetAlpha*) Event;
	pxData->Duration = pxAsuraEvent->m_fDuration;
}

void Tools_Cutscene_Event_SetAlpha::SetTotalDuration(float fDuration)
{
	Asura_Cutscene_Event_SetAlpha* pxAsuraEvent = (Asura_Cutscene_Event_SetAlpha*) Event;
	pxAsuraEvent->m_fDuration = fDuration;
}

////////////////////////////////////////////////////////////////////////////////////////////
/*------------------------------------------------**
** Tools_Cutscene_Event_ChangePlayerCameraControl **
**------------------------------------------------*/

Tools_Cutscene_Event_ChangePlayerCameraControl::Tools_Cutscene_Event_ChangePlayerCameraControl(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID)
:Tools_Cutscene_Event(pxEvent,pxCutsceneObject,iEventID)
{
}


void Tools_Cutscene_Event_ChangePlayerCameraControl::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Event_ChangePlayerCameraControl* pxAsuraEvent = (Asura_Cutscene_Event_ChangePlayerCameraControl*) Event;

	Asura_Dialog* pxDialog = NULL;
	char szText[200];

	//add actor selection control
	if(pxAsuraEvent->m_xTargetActorName.GetLength())
	{
		sprintf(szText,"Camera : %s",pxAsuraEvent->m_xTargetActorName.GetString());
	}
	else
	{
		strcpy(szText,"Camera : *None*");
	}

	pxDialog = new Asura_Function_Dialog(szText,xOffset,Tools_Cutscene_Event::ChooseActor);
	pxDialog->SetHelpText("Choose camera to change player control on");
	pxDialog->m_iControlID = DialogID_Actor;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Change duration", xOffset, &pxAsuraEvent->m_fDuration, 0.f, 999.f, 0.1f, "%3.1f", &Changed);
	pxDialog->SetHelpText("Set time taken for shake to blend from current to target parameters");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Yaw limit", xOffset, &pxAsuraEvent->m_fYawLimit, 0.f, 180.f, 1.f, "%1.0f", &Changed);
	pxDialog->SetHelpText("The maximum amount by which the player may affect the yaw of the camera");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Pitch limit", xOffset, &pxAsuraEvent->m_fPitchLimit, 0.f, 180.f, 1.f, "%1.0f", &Changed);
	pxDialog->SetHelpText("The maximum amount by which the player may affect the pitch of the camera");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

Asura_String Tools_Cutscene_Event_ChangePlayerCameraControl::GetDescription()
{
	Asura_Cutscene_Event_ChangeCameraShake* pxEvent = (Asura_Cutscene_Event_ChangeCameraShake*) Event;

	char szBuffer[250];
	Asura_PrintfToBfr(szBuffer,"Change player camera control: '%s'",250,true,pxEvent->m_xTargetActorName.GetString());

	return szBuffer;
}

void Tools_Cutscene_Event_ChangePlayerCameraControl::UpdateBackupVariables()
{
	Tools_Cutscene_Event::UpdateBackupVariables();
	Asura_Cutscene_Event_ChangePlayerCameraControl* pxChangePlayerCameraControlEvent = (Asura_Cutscene_Event_ChangePlayerCameraControl*) Event;

	m_fPitchBackup = pxChangePlayerCameraControlEvent->m_fPitchLimit;
	m_fYawBackup = pxChangePlayerCameraControlEvent->m_fYawLimit;
	m_fDurationBackup = pxChangePlayerCameraControlEvent->m_fDuration;
}

const char* Tools_Cutscene_Event_ChangePlayerCameraControl::GetNameOfChangedVariable()
{
	Asura_Cutscene_Event_ChangePlayerCameraControl* pxChangePlayerCameraControlEvent = (Asura_Cutscene_Event_ChangePlayerCameraControl*) Event;

	if(m_fPitchBackup != pxChangePlayerCameraControlEvent->m_fPitchLimit) return "Pitch limit";
	if(m_fYawBackup != pxChangePlayerCameraControlEvent->m_fYawLimit) return "Yaw limit";
	if(m_fDurationBackup != pxChangePlayerCameraControlEvent->m_fDuration) return "Duration";

	return Tools_Cutscene_Event::GetNameOfChangedVariable();
}

void Tools_Cutscene_Event_ChangePlayerCameraControl::AskForRequiredData()
{
	//select the camera
	ChooseActor();
}

bool Tools_Cutscene_Event_ChangePlayerCameraControl::ActorIsValidTarget(Asura_Cutscene_Actor* pxActor)
{
	return pxActor->IsCamera();
}

void Tools_Cutscene_Event_ChangePlayerCameraControl::FillInTimelineData(Tools_Timeline_Data_Event* pxData)
{
	Asura_Cutscene_Event_ChangePlayerCameraControl* pxAsuraEvent = (Asura_Cutscene_Event_ChangePlayerCameraControl*) Event;
	pxData->Duration = pxAsuraEvent->m_fDuration;
}

void Tools_Cutscene_Event_ChangePlayerCameraControl::SetTotalDuration(float fDuration)
{
	Asura_Cutscene_Event_ChangePlayerCameraControl* pxAsuraEvent = (Asura_Cutscene_Event_ChangePlayerCameraControl*) Event;
	pxAsuraEvent->m_fDuration = fDuration;
}


////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------------**
** Tools_Cutscene_Event_FacialAnimation **
**--------------------------------------*/

#ifdef ASURA_USE_FACIAL_ANIMATION_SYSTEM
Tools_Cutscene_Event_FacialAnimation::Tools_Cutscene_Event_FacialAnimation(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID)
:Tools_Cutscene_Event(pxEvent,pxCutsceneObject,iEventID)
{
}


void Tools_Cutscene_Event_FacialAnimation::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Event_FacialAnimation* pxEvent = (Asura_Cutscene_Event_FacialAnimation*) Event;

	
	Asura_Dialog* pxDialog;

	//add actor selection control
	char acText[200];

	if(pxEvent->m_xTargetActorName.GetLength())
		Asura_PrintfToBfr(acText,"Actor : %s",sizeof(acText),true,pxEvent->m_xTargetActorName.GetString());
	else
		strcpy(acText,"Actor : *None*");

	pxDialog = new Asura_Function_Dialog(acText,xOffset,ChooseActor);
	pxDialog->SetHelpText("Select actor whose animation should be changed.");
	pxDialog->m_iControlID = DialogID_Actor;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Tools_Dialog_FacialAnimation_PickAnimation("Anim: ", xOffset, &pxEvent->m_uAnimID, &Changed);
	pxDialog->SetHelpText("The facial animation to use.\nIf set to 'none' then it will cancel the current animation");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Loop", xOffset, &pxEvent->m_uAnimationFlags, ASURA_CUTSCENE_EVENT_FACIALANIMATION_LOOPED, &Changed);
	pxDialog->SetHelpText("Causes the animation to loop until stopped by another facial animation event");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	
}


void Tools_Cutscene_Event_FacialAnimation::FillInTimelineData(Tools_Timeline_Data_Event* pxData)
{
	Asura_Cutscene_Event_FacialAnimation* pxEvent = (Asura_Cutscene_Event_FacialAnimation*) Event;
	Tools_Cutscene_Event::FillInTimelineData(pxData);

	Tools_FacialAnimation_FaceAnimation* pxAnim = Tools_FacialAnimation_System::FindOrLoadAnimation(pxEvent->m_uAnimID);
	
	if(pxAnim)
	{
		pxData->Duration = pxAnim->GetDuration();
	}
}


Asura_String Tools_Cutscene_Event_FacialAnimation::GetDescription()
{
	Asura_Cutscene_Event_FacialAnimation* pxEvent = (Asura_Cutscene_Event_FacialAnimation*) Event;
	
	Tools_FacialAnimation_FaceAnimation* pxAnim = Tools_FacialAnimation_System::FindOrLoadAnimation(pxEvent->m_uAnimID);

	char acBuffer[250];
	if(pxAnim)
	{
		Asura_String xName;
		pxAnim->GetName().SplitFilename(0,&xName,0);
		Asura_PrintfToBfr(acBuffer,"Facial Animation : %s",250,true,xName.GetString());
	}
	else if(pxEvent->m_uAnimID)
	{
		strcpy(acBuffer, "Facial Animation : *Not Found*");
	}
	else
	{
		strcpy(acBuffer, "Facial Animation : *None*");
	}

	return acBuffer;
}

void Tools_Cutscene_Event_FacialAnimation::UpdateBackupVariables()
{
	Tools_Cutscene_Event::UpdateBackupVariables();
	Asura_Cutscene_Event_FacialAnimation* pxFacialAnimationEvent = (Asura_Cutscene_Event_FacialAnimation*) Event;

	m_uAnimIDBackup = pxFacialAnimationEvent->m_uAnimID;
	m_uAnimationFlagsBackup = pxFacialAnimationEvent->m_uAnimationFlags;
}

const char* Tools_Cutscene_Event_FacialAnimation::GetNameOfChangedVariable()
{
	Asura_Cutscene_Event_FacialAnimation* pxFacialAnimationEvent = (Asura_Cutscene_Event_FacialAnimation*) Event;

	if(m_uAnimIDBackup != pxFacialAnimationEvent->m_uAnimID) return "Animation";
	if(m_uAnimationFlagsBackup != pxFacialAnimationEvent->m_uAnimationFlags) return "Animation Flags";
	
	return Tools_Cutscene_Event::GetNameOfChangedVariable();
}

void Tools_Cutscene_Event_FacialAnimation::AskForRequiredData()
{
	//select target
	if(!Event->m_pxTargetActor)
	{
		ChooseActor();
	}
}


#endif //ASURA_USE_FACIAL_ANIMATION_SYSTEM

////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------------------**
** Tools_Cutscene_Event_MixerPreset **
**--------------------------------------*/

Tools_Cutscene_Event_MixerPreset::Tools_Cutscene_Event_MixerPreset(Asura_Cutscene_Event* pxEvent,Tools_Cutscene_Object* pxCutsceneObject,int iEventID)
:Tools_Cutscene_Event(pxEvent,pxCutsceneObject,iEventID)
{
}


void Tools_Cutscene_Event_MixerPreset::AddOverlayControls(class Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	Asura_Cutscene_Event_MixerPreset* pxEvent = (Asura_Cutscene_Event_MixerPreset*) Event;

	
	Asura_Dialog* pxDialog;
	pxDialog = new Tools_SoundMixerPreset_DropDownList_Dialog( "Mixer Preset", xOffset, 10, &pxEvent->m_uMixerPreset, &Changed );
	pxDialog->SetHelpText("Causes the cutscene to start using this mixer preset.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
}


Asura_String Tools_Cutscene_Event_MixerPreset::GetDescription()
{
	Asura_Cutscene_Event_MixerPreset* pxEvent = (Asura_Cutscene_Event_MixerPreset*) Event;
	
	Asura_Sound_Mixer_Preset* pxPreset = Asura_Sound_Mixer_System::GetPresetByHash(pxEvent->m_uMixerPreset);

	char acBuffer[250];
	if(pxPreset)
	{
		Asura_PrintfToBfr(acBuffer,"Mixer Preset : %s",250,true,pxPreset->GetName());
	}
	else if(pxEvent->m_uMixerPreset)
	{
		strcpy(acBuffer, "Mixer Preset : *Not Found*");
	}
	else
	{
		strcpy(acBuffer, "Mixer Preset : *None*");
	}

	return acBuffer;
}

void Tools_Cutscene_Event_MixerPreset::UpdateBackupVariables()
{
	Tools_Cutscene_Event::UpdateBackupVariables();
	Asura_Cutscene_Event_MixerPreset* pxMixerPresetEvent = (Asura_Cutscene_Event_MixerPreset*) Event;

	m_uMixerPresetBackup = pxMixerPresetEvent->m_uMixerPreset;
}

const char* Tools_Cutscene_Event_MixerPreset::GetNameOfChangedVariable()
{
	Asura_Cutscene_Event_MixerPreset* pxMixerPresetEvent = (Asura_Cutscene_Event_MixerPreset*) Event;

	if(m_uMixerPresetBackup != pxMixerPresetEvent->m_uMixerPreset) return "Mixer Preset";
	
	return Tools_Cutscene_Event::GetNameOfChangedVariable();
}


//////////////////////////////////////////////////////////////////////////////////////////////////
/*---------------**
** Cutscene Undo **
**---------------*/

/////////Event_Created
void CutsceneModeUndo::Log_Event_Created(Tools_Cutscene_Event* event)
{
	RootUndoData** undo = AllocateNextUndoData();
	if(!undo) return;
	
	*undo = new CutsceneUndoData_Event_Created(event);
}

CutsceneUndoData_Event_Created::CutsceneUndoData_Event_Created(Tools_Cutscene_Event* event)
{
	EventID = event->EventID;

	SetDescription("Event Created '%s'",event->GetNameOrDescription().GetString());
}

RootUndoData* CutsceneUndoData_Event_Created::ProcessUndoData()
{
	//find the tools event for this
	Tools_Cutscene_Event* event = GetToolsCutsceneEvent(EventID);
	Asura_Assert(event,"");

	//create the reverse undo data
	CutsceneUndoData_Event_Destroyed* undo = new CutsceneUndoData_Event_Destroyed(event);
	undo->SetDescription("%s",GetDescription());

	//do the undo deeds
	event->Destroy();
	
	return undo;
}

int CutsceneUndoData_Event_Created::GetPriority() const
{
	return CutsceneModeUndo::PRIORITY_EVENT_CREATED;
}

/////////Event_Destroyed
void CutsceneModeUndo::Log_Event_Destroyed(Tools_Cutscene_Event* event)
{
	RootUndoData** undo = AllocateNextUndoData();
	if(!undo) return;
	
	*undo = new CutsceneUndoData_Event_Destroyed(event);
}

CutsceneUndoData_Event_Destroyed::CutsceneUndoData_Event_Destroyed(Tools_Cutscene_Event* event)
{
	EventID = event->EventID;
	Type = event->Event->m_iType;
	
	SetDescription("Event Destroyed '%s'",event->GetNameOrDescription().GetString());

}

RootUndoData* CutsceneUndoData_Event_Destroyed::ProcessUndoData()
{
	//create the event
	Asura_Assert(Tools_Mode_Cutscene::CurrentCutscene,"");
	Tools_Cutscene_Event* event = Tools_Mode_Cutscene::CurrentCutscene->CreateCutsceneEvent(false,Type,EventID);

	
	CutsceneUndoData_Event_Created* pxUndo = new CutsceneUndoData_Event_Created(event);
	pxUndo->SetDescription("%s",GetDescription());
	return pxUndo;
}

int CutsceneUndoData_Event_Destroyed::GetPriority() const
{
	return CutsceneModeUndo::PRIORITY_EVENT_DESTROYED;
}


/////////Event_Changed
void CutsceneModeUndo::Log_Event_Changed(Tools_Cutscene_Event* event)
{
	RootUndoData** undo = AllocateNextUndoData();
	if(undo)
	{
		*undo = CreateUndo_Event_Changed(event);
	}
	event->UpdateBackupVariables();
}


RootUndoData* CutsceneModeUndo::CreateUndo_Event_Changed(class Tools_Cutscene_Event* event)
{
	switch(event->Event->m_iType)
	{
		case ACET_AnimationChange :
			return new CutsceneUndoData_Event_Changed_AnimationChange(event);
			break;

		case ACET_SecondaryAnimation :
			return new CutsceneUndoData_Event_Changed_SecondaryAnimation(event);
			break;

		case ACET_TextureAnimation :
			return new CutsceneUndoData_Event_Changed_TextureAnimation(event);
			break;

		case ACET_TimeScale :
			return new CutsceneUndoData_Event_Changed_TimeScale(event);
			break;

		case ACET_FieldOfView :
			return new CutsceneUndoData_Event_Changed_FieldOfView(event);
			break;

		case ACET_Trigger ://trigger specific changes not dealt with yet
			return new CutsceneUndoData_Event_Changed(event);
			break;

        case ACET_LoopBack:
            return new CutsceneUndoData_Event_Changed_LoopBack(event);
            break;
   
        case ACET_ActivateAttachment:
        case ACET_DeactivateAttachment:
		case ACET_ReactivateAttachment:
            return new CutsceneUndoData_Event_Changed_Attachment(event);
            break;

		case ACET_AttachmentControl :
            return new CutsceneUndoData_Event_Changed_AttachmentControl(event);
			break;


        case ACET_Beam:
            return new CutsceneUndoData_Event_Changed_Beam(event);
            break;

        case ACET_Fade:
            return new CutsceneUndoData_Event_Changed_Fade(event);
            break;

		case ACET_TemplateWrapper :
			return new CutsceneUndoData_Event_Changed_TemplateWrapper(event);

        case ACET_Subtitle:
            return new CutsceneUndoData_Event_Changed_Subtitle(event);

	    case ACET_Sound:
            return new CutsceneUndoData_Event_Changed_Sound(event);

		case ACET_ChangeCamera://camera doesn't have any extra data
			return new CutsceneUndoData_Event_Changed(event);
			break;

        case ACET_ChangeCameraTarget:
            return new CutsceneUndoData_Event_Changed_ChangeCameraTarget(event);

		case ACET_ChangeCameraShake:
			return new CutsceneUndoData_Event_Changed_ChangeCameraShake(event);

		case ACET_SetAlpha:
			return new CutsceneUndoData_Event_Changed_SetAlpha(event);

		case ACET_ChangePlayerCameraControl:
			return new CutsceneUndoData_Event_Changed_ChangePlayerCameraControl(event);

#if defined(ASURA_USE_GUI_MENU_SYSTEM)
		case ACET_GUIMenu:
			return new CutsceneUndoData_Event_Changed_GUIMenu(event);

		case ACET_ConsoleVar:
			return new CutsceneUndoData_Event_Changed_ConsoleVar(event);
#endif	//defined(ASURA_USE_GUI_MENU_SYSTEM)

#if defined ASURA_USE_HDR
		case ACET_HDRSettings:
			return new CutsceneUndoData_Event_Changed_HDRSettings(event);
#endif	// ASURA_USE_HDR

#ifdef ASURA_USE_FACIAL_ANIMATION_SYSTEM
        case ACET_FacialAnimation:
            return new CutsceneUndoData_Event_Changed_FacialAnimation(event);
#endif
        case ACET_MixerPreset:
            return new CutsceneUndoData_Event_Changed_MixerPreset(event);

		default :
			Asura_Assert(0,"Unknown event type");
	}

	return 0;
}

CutsceneUndoData_Event_Changed::CutsceneUndoData_Event_Changed(Tools_Cutscene_Event* event)
{
	EventID = event->EventID;
	TriggerTime = event->TriggerTimeBackup;
	TargetActorName = event->TargetActorNameBackup;
	ParentEventID = event->ParentEventIDBackup;
	m_uEventFlags = event->m_uEventFlagsBackup;
	m_iEnableID = event->m_iEnableIDBackup;

	m_xName = event->m_xNameBackup;//For overiding the name displayed in tools
	m_xComment = event->m_xCommentBackup;//Optional comment for use in tools
	m_xPosition = event->m_xPositionBackup;//For overiding the event object's position
	m_xNameOfSourceTemplate = event->m_xNameOfSourceTemplateBackup;//if this event belongs to a template, template event's name is here.
	m_xNameOfEventWrapper = event->m_xNameOfEventWrapperBackup;//if this event belongs to a template, wrapper event name is here.

	//fill in desc of change
	const char* pszVarName = event->GetNameOfChangedVariable();
	if(pszVarName)
		SetDescription("Event Changed (%s) '%s'",pszVarName,event->GetNameOrDescription().GetString());
	else
		SetDescription("Event Changed  '%s'",event->GetNameOrDescription().GetString());
}

int CutsceneUndoData_Event_Changed::GetPriority() const
{
	return CutsceneModeUndo::PRIORITY_EVENT_CHANGED;
}


RootUndoData* CutsceneUndoData_Event_Changed::ProcessUndoData()
{
	//find the event
	Tools_Cutscene_Event* event = GetToolsCutsceneEvent(EventID);
	Asura_Assert(event,"");

		
	event->UpdateBackupVariables();
	SwapWithEvent(event);
	event->UpdateBackupVariables();

	return this;
}

void CutsceneUndoData_Event_Changed::SwapWithEvent(Tools_Cutscene_Event* event)
{
	event->Event->m_fTriggerTime = TriggerTime;
	TriggerTime = event->TriggerTimeBackup;

	event->Event->m_xTargetActorName = TargetActorName;
	TargetActorName = event->TargetActorNameBackup;

	event->ParentEventID = ParentEventID;
	ParentEventID = event->ParentEventIDBackup;

	event->Event->m_uEventFlags = m_uEventFlags;
	m_uEventFlags = event->m_uEventFlagsBackup;

	event->Event->m_iEnableID = m_iEnableID;
	m_iEnableID = event->m_iEnableIDBackup;

	event->Event->m_xName = m_xName;
	m_xName = event->m_xNameBackup;

	event->Event->m_xComment = m_xComment;
	m_xComment = event->m_xCommentBackup;

	event->Event->m_xPosition = m_xPosition;
	m_xPosition = event->m_xPositionBackup;

	event->Event->m_xNameOfSourceTemplate = m_xNameOfSourceTemplate;
	m_xNameOfSourceTemplate = event->m_xNameOfSourceTemplateBackup;

	event->Event->m_xNameOfEventWrapper = m_xNameOfEventWrapper;
	m_xNameOfEventWrapper = event->m_xNameOfEventWrapperBackup;
}

/////////Event_Changed_AnimationChange
CutsceneUndoData_Event_Changed_AnimationChange::CutsceneUndoData_Event_Changed_AnimationChange(Tools_Cutscene_Event* event)
:CutsceneUndoData_Event_Changed(event)
{
	Tools_Cutscene_Event_AnimationChange* toolsAnimEvent = (Tools_Cutscene_Event_AnimationChange*) event;

	AnimationName = toolsAnimEvent->AnimationNameBackup;
	AnimationFileName = toolsAnimEvent->AnimationFileNameBackup;
	AnimationFlags = toolsAnimEvent->AnimationFlagsBackup;
	TweenDuration = toolsAnimEvent->TweenDurationBackup;
	m_fStartTimer = toolsAnimEvent->m_fStartTimerBackup;
	m_fEndTimer = toolsAnimEvent->m_fEndTimerBackup;
	m_fAnimDelay = toolsAnimEvent->m_fAnimDelayBackup;
}

void CutsceneUndoData_Event_Changed_AnimationChange::SwapWithEvent(Tools_Cutscene_Event* event)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(event);
	
	
	Tools_Cutscene_Event_AnimationChange* toolsAnimEvent = (Tools_Cutscene_Event_AnimationChange*) event;
	Asura_Cutscene_Event_AnimationChange* asuraAnimEvent = (Asura_Cutscene_Event_AnimationChange*) event->Event;

    Asura_Cutscene * cur_cutscene = Tools_Mode_Cutscene::CurrentCutscene->Cutscene;
	if(asuraAnimEvent->m_iAnimationIndex != -1)
	{
		cur_cutscene->AnimationNames.RemoveName(asuraAnimEvent->m_iAnimationIndex);
		cur_cutscene->AnimationFileNames.RemoveName(asuraAnimEvent->m_iAnimationIndex);
		asuraAnimEvent->m_iAnimationIndex = -1;
	}

	if(!AnimationName.IsBlank())
	{
		asuraAnimEvent->m_iAnimationIndex = cur_cutscene->AnimationNames.AddName(AnimationName);

		#if defined ASURA_ASSERT
			int filenameIndex =
		#endif
			cur_cutscene->AnimationFileNames.AddName(AnimationFileName);

		Asura_Assert(filenameIndex==asuraAnimEvent->m_iAnimationIndex,"");
	}
	
	AnimationName = toolsAnimEvent->AnimationNameBackup;
	AnimationFileName = toolsAnimEvent->AnimationFileNameBackup;


	asuraAnimEvent->m_iAnimationFlags = AnimationFlags;
	AnimationFlags = toolsAnimEvent->AnimationFlagsBackup;

	asuraAnimEvent->m_fTweenDuration = TweenDuration;
	TweenDuration = toolsAnimEvent->TweenDurationBackup;

	asuraAnimEvent->m_fStartTimer = m_fStartTimer;
	m_fStartTimer = toolsAnimEvent->m_fStartTimerBackup;

	asuraAnimEvent->m_fEndTimer = m_fEndTimer;
	m_fEndTimer = toolsAnimEvent->m_fEndTimerBackup;

	asuraAnimEvent->m_fAnimDelay = m_fAnimDelay;
	m_fAnimDelay = toolsAnimEvent->m_fAnimDelayBackup;
}

/////////Event_Changed_SecondaryAnimation
CutsceneUndoData_Event_Changed_SecondaryAnimation::CutsceneUndoData_Event_Changed_SecondaryAnimation(Tools_Cutscene_Event* event)
:CutsceneUndoData_Event_Changed_AnimationChange(event)
{
	Tools_Cutscene_Event_SecondaryAnimation* toolsAnimEvent = (Tools_Cutscene_Event_SecondaryAnimation*) event;

	Weight = toolsAnimEvent->WeightBackup;
	Duration = toolsAnimEvent->DurationBackup;
}

void CutsceneUndoData_Event_Changed_SecondaryAnimation::SwapWithEvent(Tools_Cutscene_Event* event)
{
	CutsceneUndoData_Event_Changed_AnimationChange::SwapWithEvent(event);

	//get the event
	Asura_Assert(event->Event->m_iType == ACET_SecondaryAnimation,"");


	Tools_Cutscene_Event_SecondaryAnimation* toolsAnimEvent = (Tools_Cutscene_Event_SecondaryAnimation*) event;
	Asura_Cutscene_Event_SecondaryAnimation* asuraAnimEvent = (Asura_Cutscene_Event_SecondaryAnimation*) event->Event;

	//swap the values between the undo data and the event
	asuraAnimEvent->m_fDuration = Duration;
	Duration = toolsAnimEvent->DurationBackup;

	asuraAnimEvent->m_fWeight = Weight;
	Weight = toolsAnimEvent->WeightBackup;

}


/////////Event_Changed_TextureAnimation
CutsceneUndoData_Event_Changed_TextureAnimation::CutsceneUndoData_Event_Changed_TextureAnimation(Tools_Cutscene_Event* pxEvent)
:CutsceneUndoData_Event_Changed(pxEvent)
{
	Tools_Cutscene_Event_TextureAnimation* pxToolsAnimEvent = (Tools_Cutscene_Event_TextureAnimation*) pxEvent;

	m_iSequence = pxToolsAnimEvent->m_iSequenceBackup;
	m_fSpeed = pxToolsAnimEvent->m_fSpeedBackup;
	m_uTextureAnimFlags = pxToolsAnimEvent->m_uTextureAnimFlagsBackup;
}

void CutsceneUndoData_Event_Changed_TextureAnimation::SwapWithEvent(Tools_Cutscene_Event* pxEvent)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(pxEvent);

	//get the event
	Asura_Assert(pxEvent->Event->m_iType == ACET_TextureAnimation,"");


	Tools_Cutscene_Event_TextureAnimation* pxToolsAnimEvent = (Tools_Cutscene_Event_TextureAnimation*) pxEvent;
	Asura_Cutscene_Event_TextureAnimation* pxAsuraAnimEvent = (Asura_Cutscene_Event_TextureAnimation*) pxEvent->Event;

	//swap the values between the undo data and the event
	pxAsuraAnimEvent->m_iSequence = m_iSequence;
	m_iSequence = pxToolsAnimEvent->m_iSequenceBackup;
	
	pxAsuraAnimEvent->m_fSpeed = m_fSpeed;
	m_fSpeed = pxToolsAnimEvent->m_fSpeedBackup;

	pxAsuraAnimEvent->m_uTextureAnimFlags = m_uTextureAnimFlags;
	m_uTextureAnimFlags = pxToolsAnimEvent->m_uTextureAnimFlagsBackup;

}


/////////Event_Changed_TimeScale
CutsceneUndoData_Event_Changed_TimeScale::CutsceneUndoData_Event_Changed_TimeScale(Tools_Cutscene_Event* event)
:CutsceneUndoData_Event_Changed(event)
{
	Tools_Cutscene_Event_TimeScale* toolsScaleEvent = (Tools_Cutscene_Event_TimeScale*) event;

	TimeScale = toolsScaleEvent->TimeScaleBackup;
}

void CutsceneUndoData_Event_Changed_TimeScale::SwapWithEvent(Tools_Cutscene_Event* event)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(event);

	//get the event
	Asura_Assert(event->Event->m_iType == ACET_TimeScale,"");


	Tools_Cutscene_Event_TimeScale* toolsScaleEvent = (Tools_Cutscene_Event_TimeScale*) event;
	Asura_Cutscene_Event_TimeScale* asuraScaleEvent = (Asura_Cutscene_Event_TimeScale*) event->Event;

	//swap the values between the undo data and the event
	asuraScaleEvent->m_fTimeScale = TimeScale;
	TimeScale = toolsScaleEvent->TimeScaleBackup;
}


/////////Event_Changed_FieldOfView
CutsceneUndoData_Event_Changed_FieldOfView::CutsceneUndoData_Event_Changed_FieldOfView(Tools_Cutscene_Event* event)
:CutsceneUndoData_Event_Changed(event)
{
	Tools_Cutscene_Event_FieldOfView* toolsScaleEvent = (Tools_Cutscene_Event_FieldOfView*) event;

	Duration = toolsScaleEvent->DurationBackup;
	FinalFOV = toolsScaleEvent->FinalFOVBackup;
}

void CutsceneUndoData_Event_Changed_FieldOfView::SwapWithEvent(Tools_Cutscene_Event* event)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(event);

	//get the event
	Asura_Assert(event->Event->m_iType == ACET_FieldOfView,"");


	Tools_Cutscene_Event_FieldOfView* toolsFOVEvent = (Tools_Cutscene_Event_FieldOfView*) event;
	Asura_Cutscene_Event_FieldOfView* asuraFOVEvent = (Asura_Cutscene_Event_FieldOfView*) event->Event;

	//swap the values between the undo data and the event
	asuraFOVEvent->m_fDuration = Duration;
	Duration = toolsFOVEvent->DurationBackup;

	asuraFOVEvent->m_fFinalFOV = FinalFOV;
	FinalFOV = toolsFOVEvent->FinalFOVBackup;
}


/////////Event_Changed_LoopBack
CutsceneUndoData_Event_Changed_LoopBack::CutsceneUndoData_Event_Changed_LoopBack(Tools_Cutscene_Event* event)
:CutsceneUndoData_Event_Changed(event)
{
    Tools_Cutscene_Event_LoopBack * toolsEvent = (Tools_Cutscene_Event_LoopBack *) event;

    m_fSecondsToLoopBack = toolsEvent->m_fSecondsToLoopBack;
	m_iLoopBackEventID = toolsEvent->m_iLoopBackEventIDBackup;
}

void CutsceneUndoData_Event_Changed_LoopBack::SwapWithEvent(Tools_Cutscene_Event* event)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(event);

	//get the event
	Asura_Assert(event->Event->m_iType == ACET_LoopBack,"");


	Tools_Cutscene_Event_LoopBack* toolsEvent = (Tools_Cutscene_Event_LoopBack*) event;
	Asura_Cutscene_Event_LoopBack* asuraEvent = (Asura_Cutscene_Event_LoopBack*) event->Event;

	//swap the values between the undo data and the event
	asuraEvent->m_fNumSecondsToLoopBack = m_fSecondsToLoopBack;
	m_fSecondsToLoopBack = toolsEvent->m_fSecondsToLoopBack;

	toolsEvent->m_iLoopBackEventID = m_iLoopBackEventID;
	m_iLoopBackEventID = toolsEvent->m_iLoopBackEventIDBackup;


}

/////////Event_Changed_Attachment
CutsceneUndoData_Event_Changed_Attachment::CutsceneUndoData_Event_Changed_Attachment(Tools_Cutscene_Event* pxEvent)
:CutsceneUndoData_Event_Changed(pxEvent)
{
    Tools_Cutscene_Event_Attachment * pxToolsEvent = (Tools_Cutscene_Event_Attachment *) pxEvent;

	m_uAttachmentID = pxToolsEvent->m_uAttachmentIDBackup;
}

void CutsceneUndoData_Event_Changed_Attachment::SwapWithEvent(Tools_Cutscene_Event* pxEvent)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(pxEvent);

	//get the event
	Asura_Assert(pxEvent->Event->m_iType == ACET_ActivateAttachment || 
				pxEvent->Event->m_iType == ACET_DeactivateAttachment ||
				pxEvent->Event->m_iType == ACET_ReactivateAttachment,"");


	Tools_Cutscene_Event_Attachment* pxToolsEvent = (Tools_Cutscene_Event_Attachment*) pxEvent;
	Asura_Cutscene_Event_Attachment* pxAsuraEvent = (Asura_Cutscene_Event_Attachment*) pxEvent->Event;

	//swap the values between the undo data and the event

	pxAsuraEvent->m_uAttachmentID = m_uAttachmentID;
	m_uAttachmentID = pxToolsEvent->m_uAttachmentIDBackup;


}

/////////Event_Changed_AttachmentContrl
CutsceneUndoData_Event_Changed_AttachmentControl::CutsceneUndoData_Event_Changed_AttachmentControl(Tools_Cutscene_Event* pxEvent)
:CutsceneUndoData_Event_Changed(pxEvent)
{
    Tools_Cutscene_Event_AttachmentControl * pxToolsEvent = (Tools_Cutscene_Event_AttachmentControl *) pxEvent;

	m_uNumAttachments = pxToolsEvent->m_uNumAttachmentsBackup;
	
	m_pxAttachments = m_uNumAttachments ? new Asura_Cutscene_Event_AttachmentControl_Attachment[m_uNumAttachments] : 0;
	for(u_int i=0;i<m_uNumAttachments;i++)
	{
		m_pxAttachments[i] = pxToolsEvent->m_pxAttachmentsBackup[i];
	}
}

CutsceneUndoData_Event_Changed_AttachmentControl::~CutsceneUndoData_Event_Changed_AttachmentControl()
{
	delete [] m_pxAttachments;
}

void CutsceneUndoData_Event_Changed_AttachmentControl::SwapWithEvent(Tools_Cutscene_Event* pxEvent)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(pxEvent);

	//get the event
	Asura_Assert(pxEvent->Event->m_iType == ACET_AttachmentControl ,"");


	Tools_Cutscene_Event_AttachmentControl* pxToolsEvent = (Tools_Cutscene_Event_AttachmentControl*) pxEvent;
	Asura_Cutscene_Event_AttachmentControl* pxAsuraEvent = (Asura_Cutscene_Event_AttachmentControl*) pxEvent->Event;

	//swap the values between the undo data and the event
	if(pxAsuraEvent->m_uNumAttachments != m_uNumAttachments)
	{
		delete [] pxAsuraEvent->m_pxAttachments;
		pxAsuraEvent->m_pxAttachments = m_uNumAttachments ? new Asura_Cutscene_Event_AttachmentControl_Attachment[m_uNumAttachments] : 0;
	}
	pxAsuraEvent->m_uNumAttachments = m_uNumAttachments;
	for(u_int i=0;i<m_uNumAttachments;i++)
	{
		pxAsuraEvent->m_pxAttachments[i] = m_pxAttachments[i];
	}


	if(m_uNumAttachments != pxToolsEvent->m_uNumAttachmentsBackup)
	{
		delete [] m_pxAttachments;
		m_pxAttachments = pxToolsEvent->m_uNumAttachmentsBackup ? new Asura_Cutscene_Event_AttachmentControl_Attachment[pxToolsEvent->m_uNumAttachmentsBackup] : 0;
	}
	m_uNumAttachments = pxToolsEvent->m_uNumAttachmentsBackup;
	for(u_int i=0;i<m_uNumAttachments;i++)
	{
		m_pxAttachments[i] = pxToolsEvent->m_pxAttachmentsBackup[i];
	}



}


/////////Event_Changed_Beam
CutsceneUndoData_Event_Changed_Beam::CutsceneUndoData_Event_Changed_Beam(Tools_Cutscene_Event* pxEvent)
:CutsceneUndoData_Event_Changed(pxEvent)
{
    Tools_Cutscene_Event_Beam * pxToolsEvent = (Tools_Cutscene_Event_Beam *) pxEvent;

	m_xParam = pxToolsEvent->m_xParamBackup;
}

void CutsceneUndoData_Event_Changed_Beam::SwapWithEvent(Tools_Cutscene_Event* pxEvent)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(pxEvent);

	//get the event
	Asura_Assert(pxEvent->Event->m_iType == ACET_Beam,"");

	Tools_Cutscene_Event_Beam* pxToolsEvent = (Tools_Cutscene_Event_Beam*) pxEvent;
	Asura_Cutscene_Event_Beam* pxAsuraEvent = (Asura_Cutscene_Event_Beam*) pxEvent->Event;

	//swap the values between the undo data and the event

	pxAsuraEvent->m_xParam = m_xParam;
	m_xParam = pxToolsEvent->m_xParamBackup;


}

/////////Event_Changed_Fade
CutsceneUndoData_Event_Changed_Fade::CutsceneUndoData_Event_Changed_Fade(Tools_Cutscene_Event* pxEvent)
:CutsceneUndoData_Event_Changed(pxEvent)
{
	Tools_Cutscene_Event_Fade* pxToolsFadeEvent = (Tools_Cutscene_Event_Fade*) pxEvent;

	m_fRed = pxToolsFadeEvent->m_fRedBackup;
	m_fGreen = pxToolsFadeEvent->m_fGreenBackup;
	m_fBlue = pxToolsFadeEvent->m_fBlueBackup;
	m_fDuration = pxToolsFadeEvent->m_fDurationBackup;
	m_uFlags = pxToolsFadeEvent->m_uFlagsBackup;
}

void CutsceneUndoData_Event_Changed_Fade::SwapWithEvent(Tools_Cutscene_Event* pxEvent)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(pxEvent);

	//get the event
	Asura_Assert(pxEvent->Event->m_iType == ACET_Fade,"");


	Tools_Cutscene_Event_Fade* pxToolsFadeEvent = (Tools_Cutscene_Event_Fade*) pxEvent;
	Asura_Cutscene_Event_Fade* pxAsuraFadeEvent = (Asura_Cutscene_Event_Fade*) pxEvent->Event;

	//swap the values between the undo data and the event
	pxAsuraFadeEvent->m_fRed = m_fRed;
	m_fRed = pxToolsFadeEvent->m_fRedBackup;

	pxAsuraFadeEvent->m_fGreen = m_fGreen;
	m_fGreen = pxToolsFadeEvent->m_fGreenBackup;

	pxAsuraFadeEvent->m_fBlue = m_fBlue;
	m_fBlue = pxToolsFadeEvent->m_fBlueBackup;

	pxAsuraFadeEvent->m_fDuration = m_fDuration;
	m_fDuration = pxToolsFadeEvent->m_fDurationBackup;

	pxAsuraFadeEvent->m_uFlags = m_uFlags;
	m_uFlags = pxToolsFadeEvent->m_uFlagsBackup;


}

/////////Event_Changed_TemplateWrapper
CutsceneUndoData_Event_Changed_TemplateWrapper::CutsceneUndoData_Event_Changed_TemplateWrapper(Tools_Cutscene_Event* pxEvent)
:CutsceneUndoData_Event_Changed(pxEvent)
{
}

void CutsceneUndoData_Event_Changed_TemplateWrapper::SwapWithEvent(Tools_Cutscene_Event* pxEvent)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(pxEvent);

	Tools_Cutscene_Event_TemplateWrapper* pxToolsTemplateWrapperEvent = (Tools_Cutscene_Event_TemplateWrapper*) pxEvent;
	pxToolsTemplateWrapperEvent->UpdateWrappedEventTriggerTimes();
}


/////////Event_Changed_Subtitle
CutsceneUndoData_Event_Changed_Subtitle::CutsceneUndoData_Event_Changed_Subtitle(Tools_Cutscene_Event* pxEvent)
:CutsceneUndoData_Event_Changed(pxEvent)
{
	Tools_Cutscene_Event_Subtitle* pxToolsSubtitleEvent = (Tools_Cutscene_Event_Subtitle*) pxEvent;

	m_fDuration = pxToolsSubtitleEvent->m_fDurationBackup;
	m_uTextHash = pxToolsSubtitleEvent->m_uTextHashBackup;
}

void CutsceneUndoData_Event_Changed_Subtitle::SwapWithEvent(Tools_Cutscene_Event* pxEvent)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(pxEvent);

	//get the event
	Asura_Assert(pxEvent->Event->m_iType == ACET_Subtitle,"");


	Tools_Cutscene_Event_Subtitle* pxToolsSubtitleEvent = (Tools_Cutscene_Event_Subtitle*) pxEvent;
	Asura_Cutscene_Event_Subtitle* pxAsuraSubtitleEvent = (Asura_Cutscene_Event_Subtitle*) pxEvent->Event;

	//swap the values between the undo data and the event

	pxAsuraSubtitleEvent->m_fDuration = m_fDuration;
	m_fDuration = pxToolsSubtitleEvent->m_fDurationBackup;

	pxAsuraSubtitleEvent->m_uTextHash = m_uTextHash;
	m_uTextHash = pxToolsSubtitleEvent->m_uTextHashBackup;


}


/////////Event_Changed_Sound
CutsceneUndoData_Event_Changed_Sound::CutsceneUndoData_Event_Changed_Sound(Tools_Cutscene_Event* pxEvent)
:CutsceneUndoData_Event_Changed(pxEvent)
{
	Tools_Cutscene_Event_Sound* pxToolsSoundEvent = (Tools_Cutscene_Event_Sound*) pxEvent;

	m_xSoundName = pxToolsSoundEvent->m_xSoundNameBackup;
}

void CutsceneUndoData_Event_Changed_Sound::SwapWithEvent(Tools_Cutscene_Event* pxEvent)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(pxEvent);

	//get the event
	Asura_Assert(pxEvent->Event->m_iType == ACET_Sound,"");


	Tools_Cutscene_Event_Sound* pxToolsSoundEvent = (Tools_Cutscene_Event_Sound*) pxEvent;
	Asura_Cutscene_Event_Sound* pxAsuraSoundEvent = (Asura_Cutscene_Event_Sound*) pxEvent->Event;

	//swap the values between the undo data and the event

	pxAsuraSoundEvent->m_xSoundName = m_xSoundName;
	m_xSoundName = pxToolsSoundEvent->m_xSoundNameBackup;

}


/////////Event_Changed_ChangeCameraTarget
CutsceneUndoData_Event_Changed_ChangeCameraTarget::CutsceneUndoData_Event_Changed_ChangeCameraTarget(Tools_Cutscene_Event* pxEvent)
:CutsceneUndoData_Event_Changed(pxEvent)
{
	Tools_Cutscene_Event_ChangeCameraTarget* pxToolsChangeCameraTargetEvent = (Tools_Cutscene_Event_ChangeCameraTarget*) pxEvent;

	m_xCameraTargetName = pxToolsChangeCameraTargetEvent->m_xCameraTargetNameBackup;
}

void CutsceneUndoData_Event_Changed_ChangeCameraTarget::SwapWithEvent(Tools_Cutscene_Event* pxEvent)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(pxEvent);

	//get the event
	Asura_Assert(pxEvent->Event->m_iType == ACET_ChangeCameraTarget,"");


	Tools_Cutscene_Event_ChangeCameraTarget* pxToolsChangeCameraTargetEvent = (Tools_Cutscene_Event_ChangeCameraTarget*) pxEvent;
	Asura_Cutscene_Event_ChangeCameraTarget* pxAsuraChangeCameraTargetEvent = (Asura_Cutscene_Event_ChangeCameraTarget*) pxEvent->Event;

	//swap the values between the undo data and the event
	pxAsuraChangeCameraTargetEvent->m_xCameraTargetName = m_xCameraTargetName;
	m_xCameraTargetName = pxToolsChangeCameraTargetEvent->m_xCameraTargetNameBackup;
}

/////////Event_Changed_ChangeCameraShake
CutsceneUndoData_Event_Changed_ChangeCameraShake::CutsceneUndoData_Event_Changed_ChangeCameraShake(Tools_Cutscene_Event* pxEvent)
:CutsceneUndoData_Event_Changed(pxEvent)
{
	Tools_Cutscene_Event_ChangeCameraShake* pxToolsChangeCameraShakeEvent = (Tools_Cutscene_Event_ChangeCameraShake*) pxEvent;

	m_xShake = pxToolsChangeCameraShakeEvent->m_xShakeBackup;
	m_fDuration = pxToolsChangeCameraShakeEvent->m_fDurationBackup;
}

void CutsceneUndoData_Event_Changed_ChangeCameraShake::SwapWithEvent(Tools_Cutscene_Event* pxEvent)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(pxEvent);

	//get the event
	Asura_Assert(pxEvent->Event->m_iType == ACET_ChangeCameraShake,"");

	Tools_Cutscene_Event_ChangeCameraShake* pxToolsChangeCameraShakeEvent = (Tools_Cutscene_Event_ChangeCameraShake*) pxEvent;
	Asura_Cutscene_Event_ChangeCameraShake* pxAsuraChangeCameraShakeEvent = (Asura_Cutscene_Event_ChangeCameraShake*) pxEvent->Event;

	//swap the values between the undo data and the event
	pxAsuraChangeCameraShakeEvent->m_xShake = m_xShake;
	pxAsuraChangeCameraShakeEvent->m_fDuration = m_fDuration;
	m_xShake = pxToolsChangeCameraShakeEvent->m_xShakeBackup;
	m_fDuration = pxToolsChangeCameraShakeEvent->m_fDurationBackup;
}


/////////Event_Changed_SetAlpha
CutsceneUndoData_Event_Changed_SetAlpha::CutsceneUndoData_Event_Changed_SetAlpha(Tools_Cutscene_Event* pxEvent)
:CutsceneUndoData_Event_Changed(pxEvent)
{
	Tools_Cutscene_Event_SetAlpha* pxToolsSetAlphaEvent = (Tools_Cutscene_Event_SetAlpha*) pxEvent;

	m_fDuration = pxToolsSetAlphaEvent->m_fDurationBackup;
	m_xAlpha    = pxToolsSetAlphaEvent->m_xAlphaBackup;
}

void CutsceneUndoData_Event_Changed_SetAlpha::SwapWithEvent(Tools_Cutscene_Event* pxEvent)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(pxEvent);

	//get the event
	Asura_Assert(pxEvent->Event->m_iType == ACET_SetAlpha,"");

	Tools_Cutscene_Event_SetAlpha* pxToolsSetAlphaEvent = (Tools_Cutscene_Event_SetAlpha*) pxEvent;
	Asura_Cutscene_Event_SetAlpha* pxAsuraSetAlphaEvent = (Asura_Cutscene_Event_SetAlpha*) pxEvent->Event;

	//swap the values between the undo data and the event
	pxAsuraSetAlphaEvent->m_fDuration = m_fDuration;
	m_fDuration = pxToolsSetAlphaEvent->m_fDurationBackup;

	pxAsuraSetAlphaEvent->m_xAlpha = m_xAlpha;
	m_xAlpha = pxToolsSetAlphaEvent->m_xAlphaBackup;
}

/////////Event_Changed_ChangePlayerCameraControl
CutsceneUndoData_Event_Changed_ChangePlayerCameraControl::CutsceneUndoData_Event_Changed_ChangePlayerCameraControl(Tools_Cutscene_Event* pxEvent)
:CutsceneUndoData_Event_Changed(pxEvent)
{
	Tools_Cutscene_Event_ChangePlayerCameraControl* pxToolsChangePlayerCameraControlEvent = (Tools_Cutscene_Event_ChangePlayerCameraControl*) pxEvent;

	m_fPitch = pxToolsChangePlayerCameraControlEvent->m_fPitchBackup;
	m_fYaw = pxToolsChangePlayerCameraControlEvent->m_fYawBackup;
	m_fDuration = pxToolsChangePlayerCameraControlEvent->m_fDurationBackup;
}

void CutsceneUndoData_Event_Changed_ChangePlayerCameraControl::SwapWithEvent(Tools_Cutscene_Event* pxEvent)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(pxEvent);

	//get the event
	Asura_Assert(pxEvent->Event->m_iType == ACET_ChangePlayerCameraControl,"");

	Tools_Cutscene_Event_ChangePlayerCameraControl* pxToolsChangePlayerCameraControlEvent = (Tools_Cutscene_Event_ChangePlayerCameraControl*) pxEvent;
	Asura_Cutscene_Event_ChangePlayerCameraControl* pxAsuraChangePlayerCameraControlEvent = (Asura_Cutscene_Event_ChangePlayerCameraControl*) pxEvent->Event;

	//swap the values between the undo data and the event
	pxAsuraChangePlayerCameraControlEvent->m_fPitchLimit = m_fPitch;
	pxAsuraChangePlayerCameraControlEvent->m_fYawLimit = m_fYaw;
	pxAsuraChangePlayerCameraControlEvent->m_fDuration = m_fDuration;
	m_fPitch = pxToolsChangePlayerCameraControlEvent->m_fPitchBackup;
	m_fYaw = pxToolsChangePlayerCameraControlEvent->m_fYawBackup;
	m_fDuration = pxToolsChangePlayerCameraControlEvent->m_fDurationBackup;
}

/////////Event_Changed_FacialAnimation
#ifdef ASURA_USE_FACIAL_ANIMATION_SYSTEM
CutsceneUndoData_Event_Changed_FacialAnimation::CutsceneUndoData_Event_Changed_FacialAnimation(Tools_Cutscene_Event* pxEvent)
:CutsceneUndoData_Event_Changed(pxEvent)
{
	Tools_Cutscene_Event_FacialAnimation* pxToolsFacialAnimationEvent = (Tools_Cutscene_Event_FacialAnimation*) pxEvent;

	m_uAnimID = pxToolsFacialAnimationEvent->m_uAnimIDBackup;
	m_uAnimationFlags = pxToolsFacialAnimationEvent->m_uAnimationFlagsBackup;
}

void CutsceneUndoData_Event_Changed_FacialAnimation::SwapWithEvent(Tools_Cutscene_Event* pxEvent)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(pxEvent);

	//get the event
	Asura_Assert(pxEvent->Event->m_iType == ACET_FacialAnimation,"");


	Tools_Cutscene_Event_FacialAnimation* pxToolsFacialAnimationEvent = (Tools_Cutscene_Event_FacialAnimation*) pxEvent;
	Asura_Cutscene_Event_FacialAnimation* pxAsuraFacialAnimationEvent = (Asura_Cutscene_Event_FacialAnimation*) pxEvent->Event;

	//swap the values between the undo data and the event

	pxAsuraFacialAnimationEvent->m_uAnimID = m_uAnimID;
	m_uAnimID = pxToolsFacialAnimationEvent->m_uAnimIDBackup;

	pxAsuraFacialAnimationEvent->m_uAnimationFlags = m_uAnimationFlags;
	m_uAnimationFlags = pxToolsFacialAnimationEvent->m_uAnimationFlagsBackup;
	
}
#endif// ASURA_USE_FACIAL_ANIMATION_SYSTEM

/////////Event_Changed_MixerPreset
CutsceneUndoData_Event_Changed_MixerPreset::CutsceneUndoData_Event_Changed_MixerPreset(Tools_Cutscene_Event* pxEvent)
:CutsceneUndoData_Event_Changed(pxEvent)
{
	Tools_Cutscene_Event_MixerPreset* pxToolsMixerPresetEvent = (Tools_Cutscene_Event_MixerPreset*) pxEvent;

	m_uMixerPreset = pxToolsMixerPresetEvent->m_uMixerPresetBackup;
}

void CutsceneUndoData_Event_Changed_MixerPreset::SwapWithEvent(Tools_Cutscene_Event* pxEvent)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(pxEvent);

	//get the event
	Asura_Assert(pxEvent->Event->m_iType == ACET_MixerPreset,"");


	Tools_Cutscene_Event_MixerPreset* pxToolsMixerPresetEvent = (Tools_Cutscene_Event_MixerPreset*) pxEvent;
	Asura_Cutscene_Event_MixerPreset* pxAsuraMixerPresetEvent = (Asura_Cutscene_Event_MixerPreset*) pxEvent->Event;

	//swap the values between the undo data and the event

	pxAsuraMixerPresetEvent->m_uMixerPreset = m_uMixerPreset;
	m_uMixerPreset = pxToolsMixerPresetEvent->m_uMixerPresetBackup;
}





#if defined(ASURA_USE_GUI_MENU_SYSTEM)

// Tools_Cutscene_Event_GUIMenu
//
// Created by Ali Jakamy 08/07/2008

Tools_Cutscene_Event_GUIMenu::Tools_Cutscene_Event_GUIMenu(Asura_Cutscene_Event* pxEvent, Tools_Cutscene_Object* pxCutsceneObject, int iEventID)
: PARENT(pxEvent,pxCutsceneObject,iEventID)
, m_uGUIMenuHashBackup(ASURA_HASH_ID_UNSET)
, m_fDurationBackup(0.0f)
{
}

void Tools_Cutscene_Event_GUIMenu::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	// Get GUIMenu cutscene event
	Asura_Cutscene_Event_GUIMenu* pxMenu = static_cast<Asura_Cutscene_Event_GUIMenu*> (Event);

	// Create drop down dialog for vision modes
	Asura_Dialog* pxDialog = new Tools_GUIMenu_Event_DropDownList_Dialog(xOffset, NUM_MENU_PAGES_DISPLAYED, &pxMenu->m_uGUIMenuHash, &Changed, "GUI menu");
	pxDialog->SetHelpText("Set the GUI menu during this cutscene.");
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Tools_GUIMenu_Event_DropDownList_Dialog::DefaultSpacing();

	// Create duration time dialog
	pxDialog = new Tools_CutsceneTime_Dialog("Duration", xOffset,&pxMenu->m_fDuration, 0.0f, 1000.0f, 0.1f, &Changed);
	pxDialog->SetHelpText("Time over which to show the GUI menu.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}


void Tools_Cutscene_Event_GUIMenu::UpdateBackupVariables()
{
	PARENT::UpdateBackupVariables();

	Asura_Cutscene_Event_GUIMenu* pxEvent = static_cast<Asura_Cutscene_Event_GUIMenu*> (Event);

	m_uGUIMenuHashBackup = pxEvent->m_uGUIMenuHash;
	m_fDurationBackup    = pxEvent->m_fDuration;
}

//
// CLASS Tools_GUIMenu_Event_DropDownList_Dialog
//
Tools_GUIMenu_Event_DropDownList_Dialog::Tools_GUIMenu_Event_DropDownList_Dialog( const Asura_Vector_2& xOffset, const int iNumberOfLines, Asura_Hash_ID* puSelected, bool* pbChangedFlag, const char* szName)
: PARENT(xOffset)
{
	Initialise(xOffset, iNumberOfLines, puSelected, pbChangedFlag, szName);
}


u_int Tools_GUIMenu_Event_DropDownList_Dialog::GetNumberOfHashValues() const
{
	return Asura_GUIMenu_System::GetCurrentMenuListCount();
}

const char* Tools_GUIMenu_Event_DropDownList_Dialog::GetNameByIndex(int iIndex) const
{
	Asura_GUIMenu_Menu_Page* pxMenu = Asura_GUIMenu_System::GetMenuPageByIndex(iIndex);

	if(pxMenu)
	{
		return (pxMenu->GetNameAsChar());
	}
	else
	{
		Asura_Assert(false, "Index return no menu page!");
		return NULL;
	}
}


// Tools_Cutscene_Event_Changed_GUIMenu
//
// Created by Ali Jakamy 08/07/2008

CutsceneUndoData_Event_Changed_GUIMenu::CutsceneUndoData_Event_Changed_GUIMenu(Tools_Cutscene_Event* pxEvent)
: CutsceneUndoData_Event_Changed(pxEvent)
{
	Tools_Cutscene_Event_GUIMenu* pxToolsEvent = static_cast<Tools_Cutscene_Event_GUIMenu*> (pxEvent);

	m_uGUIMenuHash = pxToolsEvent->GetGUIMenuHashBackup();
	m_fDuration    = pxToolsEvent->GetDurationBackup();
}

void CutsceneUndoData_Event_Changed_GUIMenu::SwapWithEvent(Tools_Cutscene_Event* pxEvent)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(pxEvent);

	//get the event
	Asura_Assert(pxEvent->Event->m_iType == ACET_GUIMenu,"");

	Tools_Cutscene_Event_GUIMenu* pxToolsEvent = static_cast<Tools_Cutscene_Event_GUIMenu*> (pxEvent);
	Asura_Cutscene_Event_GUIMenu* pxAsuraEvent = static_cast<Asura_Cutscene_Event_GUIMenu*> (pxEvent->Event);

	//swap the values between the undo data and the event
	pxAsuraEvent->SetMenuHash(m_uGUIMenuHash);
    m_uGUIMenuHash = pxToolsEvent->GetGUIMenuHashBackup();

	pxAsuraEvent->SetDuration(m_fDuration);
    m_fDuration = pxToolsEvent->GetDurationBackup();
}


// Tools_Cutscene_Event_ConsoleVar
//
// Created by Ali Jakamy 08/07/2008

Tools_Cutscene_Event_ConsoleVar::Tools_Cutscene_Event_ConsoleVar(Asura_Cutscene_Event* pxEvent, Tools_Cutscene_Object* pxCutsceneObject, int iEventID)
: PARENT(pxEvent,pxCutsceneObject,iEventID)
{
}

void Tools_Cutscene_Event_ConsoleVar::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{
	//// Get AVP cutscene event
	Asura_Cutscene_Event_ConsoleVar* pxDisplayConsole = static_cast<Asura_Cutscene_Event_ConsoleVar*> (Event);

	const char* szHelpText = "Select the console variable to be altered.";
	int iType = Asura_CommandConsole_VarRepository::Var::TYPE_INVALID;
    
	pxDisplayConsole->m_xConsoleVar.AddOverlayControls( pxOverlay, xOffset, "Console variables", szHelpText, iType, &Changed, true);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	int iMaxArgsLen = 12; // arbitrary limit (enough for MAX_INT)
	Asura_Dialog* pxDialog = new Asura_String_Dialog( "Value", xOffset, &pxDisplayConsole->m_xValue, iMaxArgsLen, &Changed );
	pxOverlay->AddDialog( pxDialog );

}

void Tools_Cutscene_Event_ConsoleVar::UpdateBackupVariables()
{
	PARENT::UpdateBackupVariables();

	Asura_Cutscene_Event_ConsoleVar* pxEvent = static_cast<Asura_Cutscene_Event_ConsoleVar*> (Event);

	m_xValueBackup = pxEvent->m_xValue;
	m_xConsoleVarBackup.Copy(&pxEvent->m_xConsoleVar);
}

// Tools_Cutscene_Event_Changed_ConsoleVar.cpp
//
// Created by Ali Jakamy 08/07/2008

CutsceneUndoData_Event_Changed_ConsoleVar::CutsceneUndoData_Event_Changed_ConsoleVar(Tools_Cutscene_Event* pxEvent)
: CutsceneUndoData_Event_Changed(pxEvent)
{
	Tools_Cutscene_Event_ConsoleVar* pxToolsEvent = static_cast<Tools_Cutscene_Event_ConsoleVar*> (pxEvent);

	m_xValue = pxToolsEvent->GetValueBackup();
    m_xConsoleVar.Copy(&pxToolsEvent->GetConsoleVarBackup());
}

void CutsceneUndoData_Event_Changed_ConsoleVar::SwapWithEvent(Tools_Cutscene_Event* pxEvent)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(pxEvent);

	//get the event
	Asura_Assert(pxEvent->Event->m_iType == ACET_ConsoleVar,"");

	Tools_Cutscene_Event_ConsoleVar* pxToolsEvent = static_cast<Tools_Cutscene_Event_ConsoleVar*> (pxEvent);
	Asura_Cutscene_Event_ConsoleVar* pxAsuraEvent = static_cast<Asura_Cutscene_Event_ConsoleVar*> (pxEvent->Event);

	//swap the values between the undo data and the event
	pxAsuraEvent->SetValue(pxToolsEvent->GetValueBackup());
    pxAsuraEvent->SetConsoleVar(pxToolsEvent->GetConsoleVarBackup());
}

#endif	//defined(ASURA_USE_GUI_MENU_SYSTEM)

// Tools_Cutscene_Event_HDRSettings
//
// Created by Ali Jakamy 21/07/2008
#ifdef ASURA_USE_HDR

// Statics
Asura_CameraData::HDRSettings Tools_Cutscene_Event_HDRSettings::s_xHDRSettingsBackup;

Tools_Cutscene_Event_HDRSettings::Tools_Cutscene_Event_HDRSettings(Asura_Cutscene_Event* pxEvent, Tools_Cutscene_Object* pxCutsceneObject, int iEventID)
: PARENT(pxEvent,pxCutsceneObject,iEventID)
, m_fDurationBackup(0.0f)
{
}

void Tools_Cutscene_Event_HDRSettings::AddOverlayControls(Asura_Overlay* pxOverlay,Asura_Vector_2& xOffset)
{

	/*-----------**
	** Constants **
	**-----------*/
	static const float s_fKEY_VALUE_MIN = 0.0f;
	static const float s_fKEY_VALUE_MAX = 1.0f;

	static const float s_fADAPTATION_RATE_MIN = 0.0f;
	static const float s_fADAPTATION_RATE_MAX = 100.0f;

	static const float s_fMIN_LUMINANCE_MIN = 0.0f;
	static const float s_fMIN_LUMINANCE_MAX = 10.0f;

	static const float s_fMAX_LUMINANCE_MIN = 0.0f;
	static const float s_fMAX_LUMINANCE_MAX = 10.0f;

	static const float s_fWHITE_POINT_MIN = 0.0f;
	static const float s_fWHITE_POINT_MAX = 1000.0f;

	static const float s_fBLACK_POINT_MIN = 0.0f;
	static const float s_fBLACK_POINT_MAX = 1000.0f;

	static bool s_bCaptureFixedLuminance = false;



	// Get HDRSettings Asura cutscene event
	Asura_Cutscene_Event_HDRSettings* pxHDRSettings = static_cast<Asura_Cutscene_Event_HDRSettings*> (Event);

	// Scene exposure
	Asura_Dialog* pxDialog = new Asura_Numerical_Dialog( "HDR: Scene Exposure", xOffset, &Asura_Cutscene_Event_HDRSettings::s_xHDRSettings.m_fKeyValue, s_fKEY_VALUE_MIN, s_fKEY_VALUE_MAX, 0.01f, "%.02f", &Changed );
	pxDialog->SetHelpText( "Sets the overall exposure of the scene." );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Numerical_Dialog::DefaultSpacing();

	// White point
	pxDialog = new Asura_Numerical_Dialog( "HDR: White Point", xOffset, &Asura_Cutscene_Event_HDRSettings::s_xHDRSettings.m_fWhitePoint, s_fWHITE_POINT_MIN, s_fWHITE_POINT_MAX, 0.1f, "%.01f", &Changed );
	pxDialog->SetHelpText( "Sets the white point, above which values will white out. Set to a large number to remove the effect." );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Numerical_Dialog::DefaultSpacing();

	// Black point
	pxDialog = new Asura_Numerical_Dialog( "HDR: Black Point", xOffset, &Asura_Cutscene_Event_HDRSettings::s_xHDRSettings.m_fBlackPoint, s_fBLACK_POINT_MIN, s_fBLACK_POINT_MAX, 0.001f, "%.03f", &Changed );
	pxDialog->SetHelpText( "Sets the black point, below which values will come out pure black. Set to 0 to remove the effect." );
	pxDialog->AddDialog( pxDialog );
	xOffset.y += Asura_Numerical_Dialog::LargeSpacing();

	// Use dynamic exposure
	pxDialog = new Asura_Tickbox_Dialog( "HDR: Use Dynamic Exposure", xOffset, &Asura_Cutscene_Event_HDRSettings::s_xHDRSettings.m_bDynamicExposure, &Changed );
	pxDialog->SetHelpText( "Toggles dynamic exposure adjustments." );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Tickbox_Dialog::LargeSpacing();

	// Light adaptation rate
	pxDialog = new Asura_Numerical_Dialog( "HDR: Light Adaptation Rate", xOffset, &Asura_Cutscene_Event_HDRSettings::s_xHDRSettings.m_fAdaptationRate, s_fADAPTATION_RATE_MIN, s_fADAPTATION_RATE_MAX,
		0.1f, "%.01f", &Changed );
	pxDialog->SetHelpText( "Sets the rate at which the camera adapts to light levels. The higher the number, the faster the eye will adjust." );
	pxDialog->SetIsEnabledFunction( Asura_Cutscene_Event_HDRSettings::IsUsingDynamicExposure);
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Numerical_Dialog::DefaultSpacing();

	// Min light adaptation
	pxDialog = new Asura_Numerical_Dialog( "HDR: Min Light Adaptation", xOffset, &Asura_Cutscene_Event_HDRSettings::s_xHDRSettings.m_fMinLuminance, s_fMIN_LUMINANCE_MIN, s_fMIN_LUMINANCE_MAX, 0.1f,
		"%.01f", &Changed );
	pxDialog->SetHelpText( "Sets the minimum luminance the camera is capable of adapting to. If light levels drop below this, the scene will get dark." );
	pxDialog->SetIsEnabledFunction( Asura_Cutscene_Event_HDRSettings::IsUsingDynamicExposure);
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Numerical_Dialog::DefaultSpacing();

	// Max light adaptation
	pxDialog = new Asura_Numerical_Dialog( "HDR: Max Light Adaptation", xOffset, &Asura_Cutscene_Event_HDRSettings::s_xHDRSettings.m_fMaxLuminance, s_fMAX_LUMINANCE_MIN, s_fMAX_LUMINANCE_MAX, 0.1f,
		"%.01f", &Changed );
	pxDialog->SetHelpText( "Sets the maximum luminance the camera is capable of adapting to. If light levels rise above this, the scene will white out." );
	pxDialog->SetIsEnabledFunction( Asura_Cutscene_Event_HDRSettings::IsUsingDynamicExposure);
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Numerical_Dialog::LargeSpacing();

	// Fixed scene luminance
	pxDialog = new Asura_Numerical_Dialog( "HDR: Fixed Scene Luminance", xOffset, &Asura_Cutscene_Event_HDRSettings::s_xHDRSettings.m_fFixedLuminance, 0.0f, 32.0f, 0.01f, "%.02f", &Changed);
	pxDialog->SetHelpText( "Fixes the scene luminance at the given value." );
	pxDialog->SetIsEnabledFunction( Asura_Cutscene_Event_HDRSettings::IsUsingFixedExposure );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Numerical_Dialog::DefaultSpacing();

	// Capture current luminance
	pxDialog = new Asura_Function_Dialog( "HDR: Capture Current Luminance", xOffset, Tools_Cutscene_Event_HDRSettings::CaptureFixedLuminance, &Changed, Asura_Cutscene_Event_HDRSettings::IsUsingFixedExposure, true );
	pxDialog->SetHelpText( "Fills in the value of the above control with the current scene luminance." );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::LargeSpacing();

	// Create duration time dialog
	pxDialog = new Tools_CutsceneTime_Dialog("Duration", xOffset, &pxHDRSettings->m_fDuration, 0.0f, 1000.0f, 0.1f, &Changed);
	pxDialog->SetHelpText("Time over which to alter the HDR Settings.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}


void Tools_Cutscene_Event_HDRSettings::CaptureFixedLuminance()
{
	Asura_Direct3D_HDR::CaptureFixedLuminance( Asura_Cutscene_Event_HDRSettings::s_xHDRSettings.m_fFixedLuminance );
}

void Tools_Cutscene_Event_HDRSettings::UpdateBackupVariables()
{
	PARENT::UpdateBackupVariables();

	Asura_Cutscene_Event_HDRSettings* pxEvent = static_cast<Asura_Cutscene_Event_HDRSettings*> (Event);

	s_xHDRSettingsBackup = pxEvent->s_xHDRSettings;
	m_fDurationBackup    = pxEvent->m_fDuration;
}

// CutsceneUndoData_Event_Changed_HDRSettings
//
// Created by Ali Jakamy 21/07/2008

Asura_CameraData::HDRSettings CutsceneUndoData_Event_Changed_HDRSettings::s_xHDRSettings;

CutsceneUndoData_Event_Changed_HDRSettings::CutsceneUndoData_Event_Changed_HDRSettings(Tools_Cutscene_Event* pxEvent)
: CutsceneUndoData_Event_Changed(pxEvent)
{
	Tools_Cutscene_Event_HDRSettings* pxToolsEvent = static_cast<Tools_Cutscene_Event_HDRSettings*> (pxEvent);

	s_xHDRSettings = pxToolsEvent->GetHDRSettingsBackup();
	m_fDuration    = pxToolsEvent->GetDurationBackup();
}

void CutsceneUndoData_Event_Changed_HDRSettings::SwapWithEvent(Tools_Cutscene_Event* pxEvent)
{
	CutsceneUndoData_Event_Changed::SwapWithEvent(pxEvent);

	//get the event
	Asura_Assert(pxEvent->Event->m_iType == ACET_HDRSettings,"");

	Tools_Cutscene_Event_HDRSettings* pxToolsEvent = static_cast<Tools_Cutscene_Event_HDRSettings*> (pxEvent);
	Asura_Cutscene_Event_HDRSettings* pxAsuraEvent = static_cast<Asura_Cutscene_Event_HDRSettings*> (pxEvent->Event);

	//swap the values between the undo data and the event
	pxAsuraEvent->SetHDRSettings(s_xHDRSettings);
    s_xHDRSettings = pxToolsEvent->GetHDRSettingsBackup();

	pxAsuraEvent->SetDuration(m_fDuration);
    m_fDuration = pxToolsEvent->GetDurationBackup();
}
#endif // ASURA_USE_HDR


/// End of file. ///
