//
// Tools_Message_Editing.cpp
//
// Created by Richard Rice - 13/05/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//
#include "Core/Asura.h"
#include "Entity/Tools_Message_Editing.h"

#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Hierarchy.h"
#include "Core/Asura_SortAndSearch.h"
#include "Core/Asura_String.h"
#include "Core/Asura_UserInput.h"
#include "Debugging/Asura_Assert.h"
#include "Entity/Asura_StaticMessage_System.h"
#include "Entity/Asura_StaticMessageBlock.h"
#include "Entity/Asura_Entity_Classification.h"
#ifdef ASURA_USE_CONTAINER_SYSTEM
#include "Entity/Container/Asura_Container_Messages.h"
#include "Entity/Container/Tools_Entity_Container.h"
#endif //ASURA_USE_CONTAINER_SYSTEM
#include "Entity/Cutscene/Asura_ServerEntity_CutsceneController.h"
#include "Entity/EntityTemplate/Asura_ServerEntity_Template.h"

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
#include "Entity/FMVController/Asura_FMVController_Messages.h"
#endif

#include "Entity/Glass/Asura_Entity_Glass_Messages.h"
#include "Entity/PhysicalObject/Asura_ApplyForce_Messages.h"
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
#include "Entity/Intangible/Asura_ServerEntity_ConsoleVar.h"
#endif //ASURA_SUPPORT_COMMAND_CONSOLE
#include "Entity/Sound/Asura_ServerEntity_DialogueController.h"
#include "Entity/Triggers/Asura_ServerEntity_CountedTrigger.h"
#include "Entity/Triggers/Asura_ServerEntity_SplitterBlock.h"
#include "Entity/Triggers/Asura_ServerEntity_LogicTrigger.h"
#include "Entity/Triggers/Asura_ServerEntity_TimeTrigger.h"
#include "Entity/Triggers/Asura_ServerEntity_TimelineTrigger.h"
#include "Entity/Triggers/Asura_ServerEntity_ProxyTrigger.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_Lift.h"
#include "Entity/PhysicalObject/Asura_ServerEntity_PhysicalObject.h"
#include "Entity/Locations/OrientedNode/LiftNode/Asura_ServerNode_LiftNode.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/Asura_ServerNode_GS2Node.h"
#include "Entity/Locations/OrientedNode/GamesceneNode/Tools_EntityNode_GS2Node.h"
#include "Entity/Locations/Spline/GamesceneSpline/Tools_EntityNode_GS2Spline.h"
#include "Entity/Asura_Entity_Event.h"
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Interface.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Table_Dialog.h"
#include "Rendering/Asura_Text_Output.h"
#ifdef ASURA_USE_DIALOGUE_SYSTEM
#include "Sound/Asura_Dialogue.h"
#include "Sound/Asura_Dialogue_System.h"
#endif
#include "Sound/Asura_MetaMusic_System.h"

#include "Tools_Overlays.h"
#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
#include "Entity/Intangible/Tools_Entity_ConsoleVar.h"
#endif //ASURA_SUPPORT_COMMAND_CONSOLE
#include "Entity/Sound/Tools_Entity_DialogueController.h"
#include "Entity/Sound/Tools_Entity_DialogueTrigger.h"
#include "Entity/Tools_Entity_Data.h"
#include "Entity/Tools_Entity_Object.h"
#include "EntityTemplate/Tools_Entity_Template_Object.h"
#include "Entity/Tools_Entity_LogicTrigger_Object.h"
#include "Entity/Tools_Entity_CutsceneController_Object.h"
#include "Entity/Tools_SecondaryEntityFile_System.h"
#include "Entity/Tools_Entity_TimelineTrigger_Object.h"
#include "Entity/Tools_Entity_Trigger_Object.h"
#include "Entity/Tools_Entity_CountedTrigger_Object.h"
#include "Entity/Tools_Entity_ProxyTrigger_Object.h"
#include "Entity/Tools_Entity_Lift.h"
#include "Entity/Locations/OrientedNode/LiftNode/Tools_EntityNode_LiftNode.h"
#include "Entity/Tools_ReceivedMessages.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Misc/list_tem.hpp"
#include "Misc/Tools_Timed_Messages.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Help.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_Object.h"
#include "Particles/Tools_PFX_Dialog.h"
#ifdef ASURA_USE_METAMUSICZONES
#include "Sound/Tools_MetaMusicZone_Editor.h"
#endif
#include "Sound/Tools_SoundMixer_Overlay.h"
#ifdef ASURA_USE_DIALOGUE_SYSTEM
#include "Sound/Tools_DialogueEvent_Overlay.h"
#include "Sound/Tools_Dialogue_System.h"
#include "Sound/Tools_DialogueSpeaker_Object.h"
#endif
#include "Sound/Asura_Sound.h"
#include "Texturing/Tools_TextureAnimation.h"
#ifdef ASURA_USE_FSFX
#include "FSFX/Tools_Mode_FSFX.h"
#include "FSFX/Asura_FSFX_System.h"
#endif
#if !defined(ASURA_DONT_USE_TELEPORTERS)
#include "Entity/Tools_Entity_Teleporter.h"
#include "Teleport/Asura_Teleport_Messages.h"
#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)
#ifdef ASURA_TARGET_SM3
#include "Entity/DynamicLight/Asura_DynamicLight_Messages.h"
#include "Entity/Tools_Entity_DynamicLight.h"
#endif

#ifdef ASURA_USE_AXON
#include "AnimTemplates/Axon_Gamescene_AnimTemplates.h"
#include "AnimTemplates/Axon_Gamescene_AnimTemplate_System.h"
#include "AnimTemplates/Tools_Mode_AnimTemplates.h"
#include "Behaviour/BehaviourParamsTypes/Axon_BehaviourParams_GS2Node.h"
#include "Core/Axon_Messages.h"
#include "Objects/Axon/Tools_Object_Axon_ChooseObjects.h"
#endif

#include "Entity/Tools_MessageEditor.h"
#include "Entity/Tools_ApplyForce_Messages.h"
#include "Movement/Tools_Movement.h"

#ifdef ASURA_USE_GUI_MENU_SYSTEM
#include "GUIMenu/Tools_GUIMenu_System.h"
#endif

Tools_Message_Editing*		  Tools_Message_Editing::s_pxThis = 0;
Tools_MessageEdit_Overlay*	  Tools_Message_Editing::s_pxTestMessageOverlay = 0;
Asura_MsgData_Lift_Goto_Node* Tools_Message_Editing::s_pxLiftGotoNodeMessage = NULL;

#ifdef ASURA_USE_AXON
Axon_Msg_GS2_Take_Link*							Tools_Message_Editing::s_pxTakeLinkMessage           = NULL;
u_int											Tools_Message_Editing::s_uNumGS2LinkWeightStrings	 = 0;
char**											Tools_Message_Editing::s_ppszGS2LinkWeightStrings	 = NULL;
Asura_Table_Dialog*								Tools_Message_Editing::s_pxGS2LinkWeightTable		 = NULL;
Axon_Msg_GS2_Set_Link_Weight*					Tools_Message_Editing::s_pxGS2SetLinkWeightMsg		 = NULL;
Axon_Msg_GS2_Anim_Change*						Tools_Message_Editing::s_pxGS2AnimChangeMessage		 = NULL;
Axon_Msg_GS2_Teleport_To_Node*					Tools_Message_Editing::s_pxGS2TeleportToNodeMessage	 = NULL;
Axon_Msg_GS2_Pathfind_To_Node*					Tools_Message_Editing::s_pxGS2PathfindToNodeMessage	 = NULL;
#endif

Asura_StaticMessage Tools_Message_Editing::s_xMessageBackup;

ProjectEditDestinationGuidFunction* Tools_Message_Editing::s_pfnProjectEditDestinationGuid = NULL;
ProjectGetNameFromGuidFunction* Tools_Message_Editing::s_pfnProjectGetNameFromGuid = NULL;

bool Tools_Message_Editing::s_bIsInEditMode = false;
bool Tools_Message_Editing::s_bOnSplitterOutputIndexChanged = false;
Asura_StringDisplay_Dialog* Tools_Message_Editing::s_pxSplitterOutputDescDialog = NULL;
Asura_StaticMessage* Tools_Message_Editing::s_pxSplitterBlockMessage = NULL;

static Asura_Vector_2 OverlayPosition = Asura_Vector_2(25.0f,3.0f);
static u_int g_uOverlayMinimised = 0;

static Asura_Vector_2 EditMessageOverlayPosition = Asura_Vector_2(25.0f,13.0f);
static u_int g_uEditMessageOverlayMinimised = 0;

static const int iMAX_MESSAGES_TO_SHOW = 20;


///////////////////////////////////////////////////////////////////////////////////////////////////


//list of messages that can be set in the tools
static MessageIDAndDescription s_axMessageList[] = 
{
	ASURA_MSG_DESTRUCT,		"Destroy",
	ASURA_MSG_ACTIVATE,		"Activate",
	ASURA_MSG_DEACTIVATE,	"Deactivate",
	ASURA_MSG_TRIGGER,		"Trigger",

	ASURA_MSG_SPLITTER_ACTIVATEOUTPUT,		"Activate Output",
	ASURA_MSG_SPLITTER_DEACTIVATEOUTPUT,	"Deactivate Output",
	ASURA_MSG_SPLITTER_TRIGGEROUTPUT,	"Trigger Output",

	ASURA_MSG_CHANGE_ANIMATION, "Change Animation",

	ASURA_MSG_GLASS_SMASH, "Smash Glass",

	ASURA_MSG_UNTRIGGER,	"Untrigger",

	ASURA_MSG_LOCK,			"Lock",
	ASURA_MSG_UNLOCK,		"Unlock",
	ASURA_MSG_LIFTCALL,		"Lift call",

	ASURA_MSG_RESET,		"Reset",

	ASURA_MSG_SWITCHON,		"Switch On",
	ASURA_MSG_SWITCHOFF,	"Switch Off",

	ASURA_MSG_TEMPLATE_TRIGGERINPUT,	"Trigger Input",
	ASURA_MSG_TEMPLATE_TRIGGEROUTPUT,	"Trigger Output",

	ASURA_MSG_ACTIVATECUTSCENEWITHTIME, "Activate cutscene at time", 

	ASURA_MSG_LOGICTRIGGER_INPUTON,	"Input On",
	ASURA_MSG_LOGICTRIGGER_INPUTOFF,"Input Off",

	ASURA_MSG_CUTSCENE_ENABLEEVENT,"Enable Event",
	ASURA_MSG_CUTSCENE_DISABLEEVENT,"Disable Event",
	ASURA_MSG_CUTSCENE_SETCAMERA,"Set Cutscene Camera",

	ASURA_MSG_TIMELINETRIGGER_ENABLEMESSAGE,"Enable Msg",
	ASURA_MSG_TIMELINETRIGGER_DISABLEMESSAGE,"Disable Msg",

	ASURA_MSG_TEXTUREANIMATION,"Texture Animation",

	ASURA_MSG_HIDE,"Hide",
	ASURA_MSG_UNHIDE,"Unhide",

	ASURA_MSG_COLLISIONSON,"Collisions On",
	ASURA_MSG_COLLISIONSOFF,"Collisions Off",

#ifdef ASURA_USE_METAMUSICZONES
	ASURA_MSG_METAMUSIC_ACTIVATEZONE, "Activate Meta Music Zone",
	ASURA_MSG_METAMUSIC_DEACTIVATEZONE,"Deactivate Meta Music Zone",
#endif //ASURA_USE_METAMUSICZONES

#ifdef ASURA_USE_FSFX
	ASURA_MSG_SET_FSFX, "Set Full-screen effect (OLD)",
	ASURA_MSG_SET_FSFX_NEW, "Set Full-screen effect (NEW)",
#endif // ASURA_USE_FSFX

	ASURA_MSG_SET_DESTRUCTIBLE, "Set Destructible",

	ASURA_MSG_SOUND_MIXER_PRESET, "Sound Mixer Preset (OLD)",
	ASURA_MSG_SOUND_MIXER_PRESET_QUEUE, "Queue Sound Mixer Preset (NEW)",
	ASURA_MSG_SOUND_MIXER_PRESET_UNQUEUE, "Unqueue Sound Mixer Preset (NEW)",

	ASURA_MSG_SOUND_SET_REVERB, "Set Reverb",

	ASURA_MSG_SETTIMER, "Set Timer",
	ASURA_MSG_TIMETRIGGER_SETTRIGGERTIME, "Set Trigger Time",
	ASURA_MSG_TIMETRIGGER_ADDTOTRIGGERTIME, "Add To Trigger Time",
	ASURA_MSG_TIMELINETRIGGER_SETTRIGGERTIME, "Set Trigger Time",
	ASURA_MSG_PAUSETIMER, "Pause Timer",
	ASURA_MSG_UNPAUSETIMER, "Unpause Timer",

	ASURA_MSG_COUNTEDTRIGGER_ENABLEMESSAGE, "Enable Msg",
	ASURA_MSG_COUNTEDTRIGGER_DISABLEMESSAGE, "Disable Msg",
	ASURA_MSG_COUNTEDTRIGGER_SETCOUNTER, "Set Counter",
	ASURA_MSG_COUNTEDTRIGGER_ADDCOUNTER, "Add Counter",
	ASURA_MSG_COUNTEDTRIGGER_SETTRIGGERCOUNT, "Set Trigger Count",

	ASURA_MSG_PROXY_ACTIVATE, "Proxy Activate",
	ASURA_MSG_PROXY_DEACTIVATE, "Proxy Deactivate",
	ASURA_MSG_PROXY_ADDTARGET, "Proxy Add Target",
	ASURA_MSG_PROXY_REMOVETARGET, "Proxy Remove Target",
	ASURA_MSG_PROXY_SETTARGET, "Proxy Set Target",

	ASURA_MSG_AXON_GS2_TAKE_LINK, "GS2 - Take Link If Possible",
	ASURA_MSG_AXON_GS2_PATHFIND_TO_NODE, "GS2 - Pathfind To Node",
	ASURA_MSG_AXON_GS2_DONT_STOP_AT_NODE, "GS2 - Don't Stop At Node",
	ASURA_MSG_AXON_GS2_SET_NODE_PARAMS, "GS2 - Set Node Parameters",
	ASURA_MSG_AXON_GS2_LEAVE_NODE, "GS2 - Leave Node",
	ASURA_MSG_AXON_GS2_EXIT_GAMESCENE, "GS2 - Exit Gamescene",
	ASURA_MSG_AXON_GS2_SET_LINK_WEIGHT, "GS2 - Set Link Weight",
	ASURA_MSG_AXON_GS2_ANIM_CHANGE, "GS2 - Change Actor Anim",
	ASURA_MSG_AXON_GS2_TELEPORT_TO_NODE, "GS2 - Teleport To Node",

	ASURA_MSG_LIFT_START, "Start Lift",
	ASURA_MSG_LIFT_STOP, "Stop Lift",
	ASURA_MSG_LIFT_REVERSE, "Reverse Lift",
	ASURA_MSG_LIFT_GOTO_NODE, "Goto Node",
	ASURA_MSG_LIFT_SETFLAGS, "Set Lift Flags",
	ASURA_MSG_LIFT_NODE_SETFLAGS, "Set Lift Node Flags",

#ifdef ASURA_USE_DIALOGUE_SYSTEM
	ASURA_MSG_DIALOGUE_FROMINSTANCE, "Msg from dialogue instance",
	ASURA_MSG_DIALOGUE_PLAYLINE, "Play dialogue line",
	ASURA_MSG_DIALOGUE_PLAYLINEONENTITY, "Play dialogue line",
	ASURA_MSG_DIALOGUE_CANCELSUBTITLE, "Cancel dialogue subtitle",
#endif

#ifdef ASURA_USE_GUI_MENU_SYSTEM
	ASURA_MSG_GUIMENU_ACTIVATE, "Activate GUI Menu Page",
#endif

#if !defined(ASURA_DONT_USE_TELEPORTERS)
	ASURA_MSGID_TELEPORT_ENTITY, "Teleport entity",
#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)

	ASURA_MSG_DYNAMICLIGHT_FADE, "Dynamic Light - Fade",

	ASURA_MSG_TIMETRIGGER_ADDTIME, "Add (or subtract) time",

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
	ASURA_MSG_CHANGE_CONSOLEVAR, "Change console variable",
#endif //ASURA_SUPPORT_COMMAND_CONSOLE

	ASURA_MSG_PLAY_PFX_ON_ENTITY, "Play PFX on Entity",

	ASURA_MSG_OBJECTIVE_READY, "Objective - Ready Objective",
	ASURA_MSG_OBJECTIVE_COMPLETE, "Objective - Complete Objective",
	ASURA_MSG_OBJECTIVE_FAIL, "Objective - Fail Objective",
	ASURA_MSG_OBJECTIVE_SKIP, "Objective - Skip Objective",

	ASURA_MSG_RESTORE_HEALTH, "Restore Health",

#ifdef ASURA_USE_CONTAINER_SYSTEM
	ASURA_MSG_CONTAINER_CHANGE_STATE, "Container - Change State",
	ASURA_MSG_CONTAINER_CHANGE_STATE_EXTENDED, "Container - Change State Extended",
#endif //ASURA_USE_CONTAINER_SYSTEM

	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	ASURA_MSG_DIALOGUE_CONTROLLER_TOGGLE_LINE, "Toggle Line Active",
	ASURA_MSG_DIALOGUE_CONTROLLER_INTERRUPT, "Interrupt",
	ASURA_MSG_DIALOGUE_CONTROLLER_TRIGGER_LINE, "Trigger Line",
	#endif

	ASURA_MSG_APPLY_LINEAR_FORCE, "Apply Linear Force" ,
	ASURA_MSG_APPLY_RADIAL_FORCE, "Apply Radial Force" ,

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
	ASURA_MSG_FMV_CONTROLLER_PLAY, "FMV Play",
	ASURA_MSG_FMV_CONTROLLER_PAUSE, "FMV Pause",
#endif

	0,0 //terminator
};

static EventIDAndDescription g_axEventList[] =
{
	ASURA_ENTITY_EVENT_ONACTIVATION,	"On Activation",		"",
	ASURA_ENTITY_EVENT_ONDEACTIVATION,	"On Deactivation",		"",
	ASURA_ENTITY_EVENT_ONBEINGTRIGGERED,"On Being Triggered",	"",
	ASURA_ENTITY_EVENT_ONDEATH,			"On Death",				"",
	ASURA_ENTITY_EVENT_ONUNTRIGGER,		"On Untrigger",			"",

	ASURA_ENTITY_EVENT_ONSTARTOPENING,	"On Start Opening",		"",
	ASURA_ENTITY_EVENT_ONFULLYOPENED,	"On Fully Opened",		"",
	ASURA_ENTITY_EVENT_ONSTARTCLOSING,	"On Start Closing",		"",
	ASURA_ENTITY_EVENT_ONFULLYCLOSED,	"On Fully Closed",		"",
	ASURA_ENTITY_EVENT_ONLOCKED,		"On Locked",			"",
	ASURA_ENTITY_EVENT_ONUNLOCKED,		"On Unlocked",			"",

	ASURA_ENTITY_EVENT_ONARRIVEDAT,		"On Arrived At",		"",
	ASURA_ENTITY_EVENT_ONFINISHED,		"On Finished",			"",

	ASURA_ENTITY_EVENT_ONCOMPLETED,		"On Completed",			"",
	ASURA_ENTITY_EVENT_ONFAILED,		"On Failed",			"",

	ASURA_ENTITY_EVENT_ONDEPARTEDFROM,	"On Departed From",		"",
	ASURA_ENTITY_EVENT_ONSTARTED,		"On Started",			"",
	ASURA_ENTITY_EVENT_ONSTOPPED,		"On Stopped",			"",
	ASURA_ENTITY_EVENT_ONREVERSED,		"On Reversed",			"",

	ASURA_ENTITY_EVENT_ONDESTRUCTION,	"On Destruction",		"",

	0,0,0 //terminator
};


/// Richard May - 24/01/2003 16:22:31 - list of special guids that can be set in tools
static GuidAndDescription s_axGuidList[] = 
{
	ASURA_GUID_FIRSTPLAYER, "Player 1",
	ASURA_GUID_FIRSTPLAYER+1, "Player 2",
	ASURA_GUID_FIRSTPLAYER+2, "Player 3",
	ASURA_GUID_FIRSTPLAYER+3, "Player 4",
	ASURA_GUID_SERVER, "Game Server",
	ASURA_GUID_ALL_CLIENTMACHINES, "All Clients",

#ifdef ASURA_USE_DIALOGUE_SYSTEM
	ASURA_GUID_DIALOGUE_SPEAKER, "Dialogue Speaker",
#endif
	0,0 // terminator
};


///////////////////////////////////////////////////////////////////////////////////////
/*--------------**
** Help Strings **
**--------------*/
static const char* MessageEditingHelpStrings [] =
{
	"Ctrl+Up :        Select previous message",
	"Ctrl+Down :      Select next message",
	"Page Up :        Go back a 'page' of messages",
	"Page Down :      Go forwards a 'page' of messages",
	"Shift+Up :       Move message up list (earlier)",
	"Shift+Down :     Move message down list (later)",
	"M :              Create new message to selected object",
	"Shift + M :      Create new message to sub-entity of selected object",
	"X :              Edit selected message",
	"G :              Goto target of selected message",
	"Delete :         Delete selected message",
	"",
	"F :              Finish message editing",
	NULL
};


///////////////////////////////////////////////////////////////////////////////////////////////////

//I don't want to delete Tools_Message_Editing when exiting the tools , so the message descriptions
//are available when testing the game. 
//However that leaves no convenient place to deallocate it.
//So I'm adding this to ensure that it gets cleared up when the program stops , hopefully avoiding memory leaks
//RAR 04/05/03
struct Tools_Message_Editing_Deallocator
{
	~Tools_Message_Editing_Deallocator(){delete Tools_Message_Editing::s_pxThis;}
};
static Tools_Message_Editing_Deallocator g_xDeallocatorInstance;

Tools_Message_Editing::Tools_Message_Editing()
{
	s_pxThis = this;
	m_iCurrentMessageIndex = 0;
	m_iTopMessageIndex = 0;

	m_pxAsuraMessageList = s_axMessageList;
	m_pxEventList = 0;
	if(m_pxAsuraMessageList)
	{
		MessageIDAndDescription* message = m_pxAsuraMessageList;
		for(;message->iID;message++)
		{
			Asura_Assert(message->iID < ASURA_MSG_END_OF_STATIC_MESSAGES, "Error, message used in static message system is not in the static section of ASURA_MSG_ID");
		}
	}
	m_pxProjectMessageList = 0;

	m_iNumPasteBufferMessages = 0;
	m_pxPasteBufferMessages = 0;

	m_fFirstMessagePrintX = 0;
	m_fFirstMessagePrintY = 0;

	m_bRequestRefreshMessageOverlay = false;
}

Tools_Message_Editing::~Tools_Message_Editing()
{
	delete [] m_pxEventList;
	s_pxThis = 0;

	ClearPasteBuffer();
}

void Tools_Message_Editing::LoadSettingsFromIniFile()
{
	TransferModeData(TOOLSMODEDATAFILE_LOAD);
}

void Tools_Message_Editing::SaveSettingsToIniFile()
{
	TransferModeData(TOOLSMODEDATAFILE_SAVE);
}

void Tools_Message_Editing::TransferModeData(int iLoadOrSave)
{
	//open the file
	Tools_ModeDataFile xFile("MessageEditing.dat",iLoadOrSave);

	//tranfer the data
	//(none of these should be removed , and new data should go on the end, otherwise
	//the files will get screwed up)
	if( false == xFile.GetError())
	{
		xFile.Transfer(OverlayPosition);
		xFile.Transfer(EditMessageOverlayPosition);
	}
}

void Tools_Message_Editing::StartEdit(const char* pszPrompt,unsigned short* pusMessageBlock,enum Asura_StaticMessageSetIndex eMessageSet,bool bClientMessages,Tools_Object* pxMessageBlockObject)
{
	Asura_Assert(s_pxThis,"");
	Asura_Assert(pusMessageBlock,"");

	s_bIsInEditMode = true;

	s_pxThis->m_xMessageBlock.SetMessageBlock(pusMessageBlock,eMessageSet);

	s_pxThis->m_xPrompt = pszPrompt ? pszPrompt : "";
	s_pxThis->m_bClientMessages = bClientMessages;
	s_pxThis->m_xMessageBlockEntityFile = "";


	if(*pusMessageBlock!=ASURA_STATICMESSAGE_EMPTYBLOCK)
	{
		if(!Asura_StaticMessage_System::IsMessageBlockValid(*pusMessageBlock))
		{
			*pusMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;
			Tools_Timed_Message_System::AddMessage(0xff0000,"This message block doesn't exist anymore, any old messages will have been lost.");
		}
		else if(!Asura_StaticMessage_System::GetNumberOfMessagesInBlock(*pusMessageBlock))
		{
			//message block is valid , but empty , so get rid of it.
			Asura_StaticMessage_System::DeleteMessageBlock(*pusMessageBlock);
			*pusMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;
		}

	}

	//is the object being edited an entity in a secondary file?
	if(pxMessageBlockObject && pxMessageBlockObject->IsEntityObject())
	{
		Tools_Entity_Object* pxEntityObject = (Tools_Entity_Object*) pxMessageBlockObject;
		Tools_Entity_Data* pxToolsData = pxEntityObject->Entity->ToolsData;

		s_pxThis->m_xMessageBlockEntityFile = pxToolsData->m_xSecondaryEntityFilename;		
	}


	//add help
	Tools_Help::AddHelpTopic("Message Block Editing",MessageEditingHelpStrings);

	s_pxThis->CreateOverlay();
	s_pxThis->SelectMessage(0);
	s_pxThis->SortMessages();

}

void Tools_Message_Editing::StartEdit(const char* pszPrompt,unsigned short usMessageBlockID,int iMessageIndex,enum Asura_StaticMessageSetIndex eMessageSet,Tools_Object* pxMessageBlockObject)
{
	Asura_Assert(s_pxThis,"");

	Asura_Assert(usMessageBlockID != ASURA_STATICMESSAGE_EMPTYBLOCK,"");

	s_bIsInEditMode = true;

	s_pxThis->m_xMessageBlock.SetMessageBlock(usMessageBlockID,eMessageSet);

	s_pxThis->m_xPrompt = pszPrompt ? pszPrompt : "";
	s_pxThis->m_bClientMessages = false;
	s_pxThis->m_xMessageBlockEntityFile = "";


	//is the object being edited an entity in a secondary file?
	if(pxMessageBlockObject && pxMessageBlockObject->IsEntityObject())
	{
		Tools_Entity_Object* pxEntityObject = (Tools_Entity_Object*) pxMessageBlockObject;
		Tools_Entity_Data* pxToolsData = pxEntityObject->Entity->ToolsData;

		s_pxThis->m_xMessageBlockEntityFile = pxToolsData->m_xSecondaryEntityFilename;		
	}


	//add help
	Tools_Help::AddHelpTopic("Message Block Editing",MessageEditingHelpStrings);

	s_pxThis->CreateOverlay();
	s_pxThis->SelectMessage(iMessageIndex);
	s_pxThis->SortMessages();
}

void Tools_Message_Editing::StartEdit(const char* szPrompt,Asura_StaticMessageBlock* pxMessageBlock,bool bClientMessages,Tools_Object* pxMessageBlockObject,bool* pbModifiedFlag)
{
	Asura_Assert(s_pxThis,"");
	Asura_Assert(pxMessageBlock,"");

	s_bIsInEditMode = true;

	s_pxThis->m_xMessageBlock.SetMessageBlock(pxMessageBlock,pbModifiedFlag);

	s_pxThis->m_xPrompt = szPrompt ? szPrompt : "";
	s_pxThis->m_bClientMessages = bClientMessages;
	s_pxThis->m_xMessageBlockEntityFile = "";


	//is the object being edited an entity in a secondary file?
	if(pxMessageBlockObject && pxMessageBlockObject->IsEntityObject())
	{
		Tools_Entity_Object* pxEntityObject = (Tools_Entity_Object*) pxMessageBlockObject;
		Tools_Entity_Data* pxToolsData = pxEntityObject->Entity->ToolsData;

		s_pxThis->m_xMessageBlockEntityFile = pxToolsData->m_xSecondaryEntityFilename;		
	}


	//add help
	Tools_Help::AddHelpTopic("Message Block Editing",MessageEditingHelpStrings);

	s_pxThis->CreateOverlay();
	s_pxThis->SelectMessage(0);
	s_pxThis->SortMessages();

}


void Tools_Message_Editing::EndEdit()
{
	s_pxThis->m_xMessageBlock.Reset();
	s_pxThis->m_iCurrentMessageIndex = 0;
	s_pxThis->m_iTopMessageIndex = 0;
	s_xMessageBackup.DeallocateMessageData();

#ifdef ASURA_USE_AXON
	DestroyGS2LinkWeightStrings();
#endif

	//remove help
	Tools_Help::RemoveHelpTopic("Message Block Editing");

	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SELECTEDOBJECT);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_MESSAGEEDIT);

	//after message editing ,the list on the 'recieved messages' overlay may be out of date
	Tools_ReceivedMessages::InvalidateList();

	s_bIsInEditMode = false;
}

void Tools_Message_Editing::CreateOverlay()
{
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Message Editing",false,&OverlayPosition,&g_uOverlayMinimised);

	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	pxDialog = new Asura_Function_Dialog("Add Message [M]",xOffset,AddMessage);
	pxDialog->SetHelpText("Adds a message to be sent to the selected object.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	if(!m_bClientMessages)
	{
		pxDialog = new Asura_Function_Dialog("Add Message - Sub Entity",xOffset,AddMessageSubEntity);
		pxDialog->SetHelpText("Adds a message to be sent to an entity within the selected object.\nThis is for sending messages to actors within cutscenes.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}


	//	pxDialog = new Asura_Function_Dialog("Edit Message [X]",xOffset,EditMessage);
	//	pxOverlay->AddDialog(pxDialog);
	//	xOffset.y+=1.5;

	if(!m_bClientMessages)
	{
		pxDialog = new Asura_Function_Dialog("Goto Target [G]",xOffset,_GotoTarget);
		pxDialog->SetHelpText("Finds the target of the currently selected message.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}

	pxDialog = new Asura_Function_Dialog("Delete Message [DEL]",xOffset,_DeleteMessage);
	pxDialog->SetHelpText("Deletes the currently selected message.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Disable All",xOffset,_DisableAllMessagesInBlock);
	pxDialog->SetHelpText("Disables all messages in this message block.\nEquivalent to using the Disabled tickbox on each message.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Enable All",xOffset,_EnableAllMessagesInBlock);
	pxDialog->SetHelpText("Re-enables all messages in this message block.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();


	pxDialog = new Asura_Function_Dialog("Copy Messages [Ctrl+C]",xOffset,_CopyMessages);
	pxDialog->SetIsEnabledFunction(_CanCopyMessages);
	pxDialog->SetHelpText("Copy one or more messages from this message block to the paste buffer.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Paste Messages [Ctrl+V]",xOffset,_PasteMessages);
	pxDialog->SetIsEnabledFunction(_CanPasteMessages);
	pxDialog->SetHelpText("Paste the messages in the paste buffer into this message block.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();



	pxDialog = new Asura_Function_Dialog("Finish [ESC]",xOffset,Tools_Object_System::EndMessageBlockEditing);
	pxDialog->SetHelpText("Finishes message editing , and reselects the object that was selected before this started..");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxOverlay->m_uID = TOOLS_OVERLAY_SELECTEDOBJECT;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}


void Tools_Message_Editing::UpdateMain()
{
	//Check for any changes to the current message.
	//Being done like this , since it prevents having to update all the message editing code for all the messages
	//(including all the project ones) to get them to set the modified flag when they do anything.
	//RAR 23/05/2007
	if(m_xMessageBlock.IsValid() && m_xMessageBlock.GetNumberOfMessagesInBlock())
	{
		const Asura_StaticMessage* pxMessage = m_xMessageBlock.GetMessagePointer(m_iCurrentMessageIndex);

		if(s_xMessageBackup != *pxMessage)
		{
			SortMessages();

			//The message differs from the backup copy , so flag as modified
			SetModified();
			s_xMessageBackup.Copy(pxMessage);
		}
	}

	if ( m_bRequestRefreshMessageOverlay )
	{
		m_bRequestRefreshMessageOverlay = false;

		#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
		if(Tools_MessageEditor::GetSelectedStaticMessage())
		{
			Tools_MessageEditor::CreateMessageAttributesOverlay(Tools_MessageEditor::GetSelectedStaticMessage());
		}
		else
		{
			//Work around to prevent crashing if the old method has been used. RAR 16/02/2009
			UpdateMessageOverlay();
		}
		#else
		UpdateMessageOverlay();
		#endif
	}

	if(s_bOnSplitterOutputIndexChanged)
	{
		s_bOnSplitterOutputIndexChanged = false;

		Asura_String xMessageDesc;
		GetMessageDescription(s_pxSplitterBlockMessage->m_usMsgID, s_pxSplitterBlockMessage, xMessageDesc);			

		s_pxSplitterOutputDescDialog->SetName(xMessageDesc);
	}
}

void Tools_Message_Editing::RenderMain(float& x, float& y)
{
	Asura_Assert(m_xMessageBlock.IsValid(),"");


	if (!Tools_Object_System::IsChoosingObject())
	{
		int numMessages = m_xMessageBlock.GetNumberOfMessagesInBlock();

		//display heading
		if(numMessages > iMAX_MESSAGES_TO_SHOW || m_iTopMessageIndex > 0)
		{
			int iLastMessage = Asura_Maths::Min(m_iTopMessageIndex + iMAX_MESSAGES_TO_SHOW,numMessages);

			Asura_Text_Output::Print(x,y++,0xff00ff00,0,"Editing Messages %d-%d of %d:",m_iTopMessageIndex+1,iLastMessage,numMessages);
        }
		else
		{
			Asura_Text_Output::Print(x,y++,0xff00ff00,0,"Editing Messages :");
		}
		Asura_Text_Output::Print(x,y++,0xff00ff00,0,"%s",m_xPrompt.GetString());

		m_fFirstMessagePrintX = x;
		m_fFirstMessagePrintY = y;

		u_int uPageUpColour = (m_iTopMessageIndex>0) ? 0xffffff00 : 0x7f7f7f00;
		Asura_Text_Output::Print(x,y++,uPageUpColour,0,"(Page Up)");

		//display information for each message
		for(int i=0;i<iMAX_MESSAGES_TO_SHOW;i++)
		{
			int iMessageIndex = i+m_iTopMessageIndex;
			if(iMessageIndex>=numMessages) break;
			
			Asura_StaticMessage* message = m_xMessageBlock.GetMessagePointer(iMessageIndex);
			int colour;
			if(message->IsDisabled())
			{
				//use dimmed colour for disabled messages
				colour = (iMessageIndex==m_iCurrentMessageIndex) ? 0xff7f0000 : 0xff7f7f7f;
			}
			else
			{
				colour = (iMessageIndex==m_iCurrentMessageIndex) ? 0xffff0000 : 0xffffffff;
			}

			Asura_String xMessageDesc;
			GetMessageDescription(message->m_usMsgID,message,xMessageDesc);			

			Asura_Text_Output::Print(x,y++,colour,0,"%5.3f %s - %s",message->m_fDelay,xMessageDesc.GetString(),GetNameFromGuid(message->m_uSendTo));
		}

		u_int uPageDownColour = (m_iTopMessageIndex+iMAX_MESSAGES_TO_SHOW<numMessages) ? 0xffffff00 : 0x7f7f7f00;
		Asura_Text_Output::Print(x,y++,uPageDownColour,0,"(Page Down)");

	}
}

void Tools_Message_Editing::HandleKeyboardMain()
{
	Asura_Assert(m_xMessageBlock.IsValid(),"");
	int numMessages = m_xMessageBlock.GetNumberOfMessagesInBlock();
	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ALT)) return;


	if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
	{
		if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_UP))
		{
			if(m_iCurrentMessageIndex>0) SelectMessage(m_iCurrentMessageIndex-1);
		}
		if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_DOWN))
		{
			if(m_iCurrentMessageIndex<numMessages-1) SelectMessage(m_iCurrentMessageIndex+1);
		}

		
		if(Asura_UserInput::DebouncedKeyPressed('c'))
		{
			CopyMessages();
		}
		if(Asura_UserInput::DebouncedKeyPressed('v'))
		{
			PasteMessages();
		}
	}
	else if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
	{
		if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_UP))
		{
			//move message up list
			if(SwapMessages(m_iCurrentMessageIndex,m_iCurrentMessageIndex-1))
			{
				SelectMessage(m_iCurrentMessageIndex-1);
			}
		}
		if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_DOWN))
		{
			//move message down list
			if(SwapMessages(m_iCurrentMessageIndex,m_iCurrentMessageIndex+1))
			{
				SelectMessage(m_iCurrentMessageIndex+1);
			}
		}
		if (Asura_UserInput::DebouncedKeyPressed('m'))
		{
			AddMessage(true);
		}
	}
	else
	{
		if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE))
		{
			if(!Tools_Help::IsDisplayingHelp())//(avoid clashing with the help systems use of escape
			{
				Tools_Object_System::EndMessageBlockEditing();
				return;
			}
		}
		if (Asura_UserInput::DebouncedKeyPressed('m'))
		{
			AddMessage(false);
		}
		//		if (Asura_UserInput::DebouncedKeyPressed('x'))
		//		{
		//			EditMessage();
		//		}
		if (Asura_UserInput::DebouncedKeyPressed('g'))
		{
			GotoTarget();
		}
		if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_DEL))
		{
			DeleteMessage();
		}
		if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_PAGEUP))
		{
			if(m_iCurrentMessageIndex>0) SelectMessage(m_iCurrentMessageIndex-iMAX_MESSAGES_TO_SHOW);
		}
		if (Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_PAGEDOWN))
		{
			if(m_iCurrentMessageIndex<numMessages-1) SelectMessage(m_iCurrentMessageIndex+iMAX_MESSAGES_TO_SHOW);
		}
	}
}

bool Tools_Message_Editing::HandleMouseSelectionMain()
{
	Asura_Assert(m_xMessageBlock.IsValid(),"");

	if (Asura_Interface::UsingMenusOrOverlays()) return false;
	if (Tools_Object_System::IsChoosingObject()) return false;
	if (!Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_MOUSEBUTTON1)) return false;


	//get the mouse position in print coords
	int mx,my;
	Asura_Cursor::GetScreenSpaceCoords(mx,my);
	float fPrintMX = (float)mx;
	float fPrintMY = (float)my;
	Asura_Text_Output::ConvertPixelsToPrintCoords(fPrintMX,fPrintMY);
	//make relative to the first message

	fPrintMX -= m_fFirstMessagePrintX;
	fPrintMY -= m_fFirstMessagePrintY;

	if(fPrintMY<0 || fPrintMX<0) return false;

	//determine which message the mouse is over
	int iLine = Asura_Maths::FloatToInt_Chop(fPrintMY);
	if(iLine < 0 || iLine >= iMAX_MESSAGES_TO_SHOW+2) return false;
	
	int iNumMessages = m_xMessageBlock.GetNumberOfMessagesInBlock();

	if(iLine == 0)
	{
		//over the Page Up line
		if(fPrintMX >= 9.0f) return false;

		if(m_iTopMessageIndex > 0)
		{
			SelectMessage(m_iCurrentMessageIndex - iMAX_MESSAGES_TO_SHOW);
		}


		return true;

	}
	else if(iLine == iMAX_MESSAGES_TO_SHOW+1)
	{
		//over the Page Down line
		if(fPrintMX >= 11.0f) return false;

		if(m_iTopMessageIndex+iMAX_MESSAGES_TO_SHOW < iNumMessages)
		{
			SelectMessage(m_iCurrentMessageIndex + iMAX_MESSAGES_TO_SHOW);
		}

		return true;

	}
	else
	{
		int iMessage = iLine + m_iTopMessageIndex - 1;

		if(iMessage >= iNumMessages) return false;

		//get the text that would be displayed for this message , to determine if mouse far enough left
		Asura_StaticMessage* pxMessage = m_xMessageBlock.GetMessagePointer(iMessage);

		Asura_String xMessageDesc;
		GetMessageDescription(pxMessage->m_usMsgID,pxMessage,xMessageDesc);			

		char szTimeText[10];
		Asura_PrintfToBfr(szTimeText,"%5.3f ",10,true,pxMessage->m_fDelay);

		int iLength = xMessageDesc.GetLength() + strlen(GetNameFromGuid(pxMessage->m_uSendTo)) + strlen(szTimeText) + 3;
		if(fPrintMX >= (float)iLength) return false;

		//select the message then
		SelectMessage(iMessage);
		return true;
	}

}


bool Tools_Message_Editing::GetMessageDescription(int iMessageID,const Asura_StaticMessage* pxMessage,Asura_String& xOutputDesc)
{
	if(s_pxThis) return s_pxThis->GetMessageDescriptionMain(iMessageID,pxMessage,xOutputDesc);

	MessageIDAndDescription* message = s_axMessageList;
	for(;message->iID;message++)
	{
		if(iMessageID == message->iID)
		{
			xOutputDesc = message->pszDescription;
			return true;
		}
	}
	xOutputDesc = "<Unknown Message>";
	return true;
}


bool Tools_Message_Editing::GetMessageDescriptionMain(int messageID,const Asura_StaticMessage* pxMessage,Asura_String& xOutputDesc)
{

	char szMessageBuffer[100];
	if(pxMessage)
	{
		switch(pxMessage->m_usMsgID)
		{
			case ASURA_MSG_TEMPLATE_TRIGGERINPUT :
			case ASURA_MSG_TEMPLATE_TRIGGEROUTPUT :
			{
				//try to get a more informative message name
				const char* pszDesc = Asura_ServerEntity_Template::GetMessageDescription(pxMessage);
				if(pszDesc)
				{
					xOutputDesc = pszDesc;
					return true;
				}
				break;
			}

			case ASURA_MSG_LOGICTRIGGER_INPUTON :
			case ASURA_MSG_LOGICTRIGGER_INPUTOFF :
			{
				//try to get a more informative message name
				const char* pszDesc = Asura_ServerEntity_LogicTrigger::GetMessageDescription(pxMessage);
				if(pszDesc)
				{
					xOutputDesc = pszDesc;
					return true;
				}
				break;
			}

			case ASURA_MSG_SPLITTER_ACTIVATEOUTPUT :
			case ASURA_MSG_SPLITTER_DEACTIVATEOUTPUT :
			case ASURA_MSG_SPLITTER_TRIGGEROUTPUT :
			{
				if(Asura_ServerEntity_SplitterBlock::GetMessageDescription(pxMessage,xOutputDesc))
				{
					return true;
				}

				Asura_Message_SplitterBlock_Output* pxData = (Asura_Message_SplitterBlock_Output*) pxMessage->m_pData;

				const char* szMessageName = 0;
				switch(pxMessage->m_usMsgID)
				{
					case ASURA_MSG_SPLITTER_ACTIVATEOUTPUT :
						szMessageName = "Activate Output";
						break;

					case ASURA_MSG_SPLITTER_DEACTIVATEOUTPUT :
						szMessageName = "Deactivate Output";
						break;

					case ASURA_MSG_SPLITTER_TRIGGEROUTPUT :
						szMessageName = "Trigger Output";
						break;
					default	:
						Asura_Assert(0,"Unexpected message type");
						break;

				}
				
				Asura_PrintfToBfr(szMessageBuffer,"%s %d",100,true,szMessageName,pxData->m_uOutputIndex);

				xOutputDesc = szMessageBuffer;
				return true;

			}

			case ASURA_MSG_TIMELINETRIGGER_ENABLEMESSAGE :
			case ASURA_MSG_TIMELINETRIGGER_DISABLEMESSAGE :
			case ASURA_MSG_TIMELINETRIGGER_SETTRIGGERTIME :
			{
				const char* pszDesc = Asura_ServerEntity_TimelineTrigger::GetMessageDescription(pxMessage);
				if(pszDesc)
				{
					xOutputDesc = pszDesc;
					return true;
				}
				break;
			}

			case ASURA_MSG_COUNTEDTRIGGER_ENABLEMESSAGE :
			case ASURA_MSG_COUNTEDTRIGGER_DISABLEMESSAGE :
			case ASURA_MSG_COUNTEDTRIGGER_SETTRIGGERCOUNT :
			case ASURA_MSG_COUNTEDTRIGGER_SETCOUNTER :
			case ASURA_MSG_COUNTEDTRIGGER_ADDCOUNTER :
			{
				const char* pszDesc = Asura_ServerEntity_CountedTrigger::GetMessageDescription(pxMessage);
				if(pszDesc)
				{
					xOutputDesc = pszDesc;
					return true;
				}
				break;
			}

			case ASURA_MSG_LIFT_GOTO_NODE:
			{
				Asura_MsgData_Lift_Goto_Node* pxLiftMessage = static_cast< Asura_MsgData_Lift_Goto_Node* >( pxMessage->m_pData );
				Asura_PrintfToBfr(szMessageBuffer, "Goto '%s'", sizeof(szMessageBuffer), true, GetNameFromGuid(pxLiftMessage->GetNodeGuid()));
				xOutputDesc = szMessageBuffer;
				return true;
			}

			case ASURA_MSG_PROXY_ADDTARGET :
			case ASURA_MSG_PROXY_REMOVETARGET :
			case ASURA_MSG_PROXY_SETTARGET :
			{
				Asura_Message_ProxyTrigger_Target* pxData = (Asura_Message_ProxyTrigger_Target*) pxMessage->m_pData;

				const char* szBaseMessageDesc = 0;
				if(ASURA_MSG_PROXY_ADDTARGET) szBaseMessageDesc = "Proxy Add Target";
				else if(ASURA_MSG_PROXY_REMOVETARGET) szBaseMessageDesc = "Proxy Remove Target";
				else if(ASURA_MSG_PROXY_SETTARGET) szBaseMessageDesc = "Proxy Set Target";

				char* szTargetDesc = 0;
				if(pxData->m_uTargetGuid == ASURA_GUID_UNREGISTERED)
				{
					szTargetDesc = "None";
				}
				else
				{
					//get the name from the guid in the message. This needs to be useable in-game , so can't search the tools objects
					const Asura_Entity_Server* pxEntity = Asura_Entity_Server::FindEntity(pxData->m_uTargetGuid);
					if(pxEntity && pxEntity->ToolsData && !pxEntity->ToolsData->Name.IsBlank())
					{
						szTargetDesc = pxEntity->ToolsData->Name.GetString();
					}
				}

				if(szTargetDesc)
					Asura_PrintfToBfr(szMessageBuffer,"%s '%s'",sizeof(szMessageBuffer),true,szBaseMessageDesc,szTargetDesc);
				else
					Asura_PrintfToBfr(szMessageBuffer,"%s '%d'",sizeof(szMessageBuffer),true,szBaseMessageDesc,pxData->m_uTargetGuid);

				xOutputDesc = szMessageBuffer;


				return true;
			}

#ifdef ASURA_USE_DIALOGUE_SYSTEM
			case ASURA_MSG_DIALOGUE_FROMINSTANCE:
			{
				const u_int* puData = (const u_int*) pxMessage->m_pData;
				const char* szBaseMessageDesc = "Msg from dialogue instance";

				Asura_PrintfToBfr(szMessageBuffer,"%s : %d",sizeof(szMessageBuffer),true,szBaseMessageDesc,*puData);
				xOutputDesc = szMessageBuffer;


				return true;
			}

			case ASURA_MSG_DIALOGUE_PLAYLINE:
			case ASURA_MSG_DIALOGUE_PLAYLINEONENTITY:
			{
				const Asura_Message_Dialogue_PlayLine* pxData = (const Asura_Message_Dialogue_PlayLine*) pxMessage->m_pData;
				const char* szBaseMessageDesc = "Play dialogue line";

				const char* szLine;
				if(pxData->m_uLineID)
				{
					const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::FindLine(pxData->m_uLineID);
					if(pxLine)
					{
						szLine = pxLine->GetName();
					}
					else
					{
						szLine = "*Unknown*";
					}
				}
				else
				{
					szLine = "*Not Set*";
				}

				Asura_PrintfToBfr(szMessageBuffer,"%s : %s",sizeof(szMessageBuffer),true,szBaseMessageDesc,szLine);
				xOutputDesc = szMessageBuffer;


				return true;
			}

			case ASURA_MSG_DIALOGUE_CONTROLLER_INTERRUPT:
			{
				Asura_Msg_Dialogue_Interrupt* pxData = static_cast<Asura_Msg_Dialogue_Interrupt*>( pxMessage->m_pData );
				
				Asura_PrintfToBfr( szMessageBuffer,"%s %s", sizeof(szMessageBuffer), true, "Interrupt dialogue playback", pxData->m_bImmediate ? "(Immediately)" : "(Finishing current line)" );
				
				xOutputDesc = szMessageBuffer;

				return true;
			}

			case ASURA_MSG_DIALOGUE_CONTROLLER_TOGGLE_LINE:
			{
				Asura_Msg_Dialogue_ToggleLine* pxData = static_cast<Asura_Msg_Dialogue_ToggleLine*>( pxMessage->m_pData );

				const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::FindLine(pxData->m_uLineID);

				char szLine[128];
				char szChange[128];

				Asura_PrintfToBfr( szLine,"%s", 128, true, pxLine ? pxLine->GetName() : "all lines" );
								
				Asura_PrintfToBfr( szChange,"%s", 128, true, pxData->m_bEnable ? "Enable" : "Disable" );
				
				Asura_PrintfToBfr( szMessageBuffer,"%s %s", sizeof(szMessageBuffer), true, szChange, szLine );

				xOutputDesc = szMessageBuffer;

				return true;
			}

			case ASURA_MSG_DIALOGUE_CONTROLLER_TRIGGER_LINE:
			{
				Asura_Msg_Dialogue_TriggerLine* pxData = static_cast<Asura_Msg_Dialogue_TriggerLine*>( pxMessage->m_pData );

				const Asura_Dialogue_Line* pxLine = Asura_Dialogue_Storage::FindLine(pxData->m_uLineID);

				xOutputDesc = pxLine ? pxLine->GetName() : "*Not Set*" ;

				return true;
			}


#endif // ASURA_USE_DIALOGUE_SYSTEM


			case ASURA_MSG_APPLY_LINEAR_FORCE:
			{
				Asura_PrintfToBfr( szMessageBuffer,"%s", sizeof(szMessageBuffer), true, "Apply Linear Force");
				xOutputDesc = szMessageBuffer;
				return true;
			}

			case ASURA_MSG_APPLY_RADIAL_FORCE:
			{
				Asura_PrintfToBfr( szMessageBuffer,"%s", sizeof(szMessageBuffer), true, "Apply Radial Force");
				xOutputDesc = szMessageBuffer;
				return true;
			}

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
			case ASURA_MSG_FMV_CONTROLLER_PLAY:
			{
//				Asura_Msg_FMV_Controller_Play* pxData = static_cast<Asura_Msg_FMV_Controller_Play*>( pxMessage->m_pData );

				Asura_PrintfToBfr( szMessageBuffer,"%s", sizeof(szMessageBuffer), true, "FMV Play");

				xOutputDesc = szMessageBuffer;

				return true;
			}

			case ASURA_MSG_FMV_CONTROLLER_PAUSE:
			{
//				Asura_Msg_FMV_Controller_Pause* pxData = static_cast<Asura_Msg_FMV_Controller_Pause*>( pxMessage->m_pData );

				Asura_PrintfToBfr( szMessageBuffer,"%s", sizeof(szMessageBuffer), true, "FMV Pause");

				xOutputDesc = szMessageBuffer;

				return true;
			}
#endif

#if !defined(ASURA_DONT_USE_TELEPORTERS)
			case ASURA_MSGID_TELEPORT_ENTITY:
			{
				Tools_Entity_Teleporter::UpdateTeleportEntityText(*pxMessage, xOutputDesc);
				break;
			}
#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)


#ifdef ASURA_USE_CONTAINER_SYSTEM
			case ASURA_MSG_CONTAINER_CHANGE_STATE:
			{
				const Asura_Message_Container_ChangeState* pxData = static_cast<const Asura_Message_Container_ChangeState*>(pxMessage->m_pData);
				const Asura_Hash_ID uNewStateHash = (pxData ? pxData->m_uStateHash : ASURA_HASH_ID_UNSET);
				const char* szStateName = Tools_Entity_Container::GetStateName(pxMessage->m_uSendTo, uNewStateHash);

				Asura_PrintfToBfr(szMessageBuffer,"Container - Change State to '%s'", 100, true, szStateName);
				xOutputDesc = szMessageBuffer;
				return true;
			}

			case ASURA_MSG_CONTAINER_CHANGE_STATE_EXTENDED:
			{
				const Asura_Message_Container_ChangeStateExtended* pxData = static_cast<const Asura_Message_Container_ChangeStateExtended*>(pxMessage->m_pData);
				const u_int uInputIndex = pxData->m_uInputIndex;
				const Asura_Hash_ID uNewStateHash = (pxData ? pxData->m_uStateHash : ASURA_HASH_ID_UNSET);
				const char* szStateName = Tools_Entity_Container::GetStateName(pxMessage->m_uSendTo, uNewStateHash);

				Asura_PrintfToBfr(szMessageBuffer,"Container - Change State to '%s' for input %u", 100, true, szStateName, uInputIndex);
				xOutputDesc = szMessageBuffer;
				return true;
			}
#endif //ASURA_USE_CONTAINER_SYSTEM

			default :
				//do nothing
				break;
		}
	}

	if(m_pxAsuraMessageList)
	{
		MessageIDAndDescription* message = m_pxAsuraMessageList;
		for(;message->iID;message++)
		{
			if(messageID == message->iID)
			{
				xOutputDesc = message->pszDescription;
				return true;
			}
		}
	}
	if(m_pxProjectMessageList)
	{
		MessageIDAndDescription* message = m_pxProjectMessageList;
		for(;message->iID;message++)
		{
			if(messageID == message->iID)
			{
				xOutputDesc = message->pszDescription;
				return true;
			}
		}
	}

	xOutputDesc = "<Unknown Message>";
	return false;
}

const char* Tools_Message_Editing::GetNameFromGuidMain(Asura_Guid guid)
{
	if(s_pfnProjectGetNameFromGuid)
	{
		const char* pszName = s_pfnProjectGetNameFromGuid(guid);
		if(pszName)
		{
			return pszName;
		}
	}

	/// Richard May - 24/02/2003 16:13:27 - guids from 1000 to 99999 are potentially players 
	if( (guid >= ASURA_GUID_CLIENT_START) && (guid <= ASURA_GUID_CLIENT_END) )
	{
		static char szBuffer[100];
		Asura_PrintfToBfr(szBuffer,"%s %d",100,true,"Player",(guid-(ASURA_GUID_CLIENT_START-1)));

		return szBuffer;
	}
	else if( guid == ASURA_GUID_CLIENT)
	{
		return "Client";
	}

	//check against other special guids	
	for(GuidAndDescription* pxGuidDesc = s_axGuidList;(pxGuidDesc->uGuid || pxGuidDesc->pszDescription);pxGuidDesc++)
	{
		if(guid == pxGuidDesc->uGuid) return pxGuidDesc->pszDescription;
	}

	Tools_Object* pxObject = Tools_Object_System::GetObjectFromGuid(guid,true);
	if(!pxObject) return "<Unknown Object>";

	//does the guid belong to a sub-entity?
	bool bSubEntity = pxObject->IsSubEntity(guid);
	if(!bSubEntity) return pxObject->GetName();

	static char szBuffer[100];
	Asura_PrintfToBfr(szBuffer,"%s: %s",100,true,pxObject->GetName(),pxObject->GetSubEntityName(guid));

	return szBuffer;


}


void Tools_Message_Editing::AddMessage(bool bSubEntity, Tools_BaseEntity_Object* pxEntityObjectOverride /*= NULL*/)
{
	if (Tools_Object_System::SelectedObjectList.size() <= 1 ||
		bSubEntity || m_bClientMessages)
	{
		Asura_Assert(m_xMessageBlock.IsValid(),"");

		Tools_Object* pxObject = pxEntityObjectOverride ? pxEntityObjectOverride : Tools_Object_System::SelectedObject;

		Asura_Guid uGuid = ASURA_GUID_UNREGISTERED;

		if(m_bClientMessages)
		{
			//The client is the only acceptable target for client side messages at the moment
			uGuid = ASURA_GUID_CLIENT;
		}
		else
		{
			//make sure an object is selected
			if(!pxObject)
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"No object selected - please select special GUID");

				// modal overlay here for GUID selection
				if(!EditDestinationGuid(uGuid,"Select Destination Guid")) return;
			}	
			else	
			{
				if(bSubEntity)
				{
					//get the guid for the sub-entity taking the message
					uGuid = pxObject->SelectSubEntity();
				}
				else
				{
					if(!Tools_SecondaryEntityFile_System::IsAllowableTarget(m_xMessageBlockEntityFile.GetString(),(m_xMessageBlock.GetMessageSet()==ASURA_STATICMESSAGE_CUTSCENESET),pxObject))
					{
						return;
					}

					//is this a valid target for messages?
					if(pxObject->GetGuid()==ASURA_GUID_UNREGISTERED)
					{
						// (MH - 15.11.2002) some Tools_Objects only get a guid when they need one. Like now.
						if (!pxObject->AcquireGuid())
						{
							Tools_Timed_Message_System::AddMessage(0xff0000,"That object isn't a valid target for messages.");
							return;
						}
					}
					uGuid = pxObject->GetGuid();
				}
			}
		}

		if(uGuid == ASURA_GUID_UNREGISTERED)
		{
			return;
		}

		Asura_StaticMessage message;
		message.Initialise();
		message.m_uSendTo = uGuid;

		//ask for the message to be sent
		if(!EditMessageID(&message,"Select message to send")) return;
		InitialiseMessageData(&message,true);



		//add the message to the block
		m_xMessageBlock.AddMessage(message);

		//move the current message index to the newly added message
		m_iCurrentMessageIndex = m_xMessageBlock.GetNumberOfMessagesInBlock()-1;
		SelectMessage(m_iCurrentMessageIndex);
	}
	else
	{
		// AW - 20/03/03 - Added support for multiple select in message editing.

		List<Tools_Object*> xSelectedList = Tools_Object_System::SelectedObjectList;
		//remove objects that aren't valid targets
		for(LIF<Tools_Object*> xObjectLIF(&xSelectedList);!xObjectLIF.done();)
		{
			Tools_Object* pxCurrent = xObjectLIF();
			if(Tools_SecondaryEntityFile_System::IsAllowableTarget(m_xMessageBlockEntityFile.GetString(), (m_xMessageBlock.GetMessageSet()==ASURA_STATICMESSAGE_CUTSCENESET), pxCurrent))
			{
				// If we're multiple-selecting things like env hooks, we need to make sure a guid is set up.
				if ((pxCurrent->GetGuid() != ASURA_GUID_UNREGISTERED) || (pxCurrent->AcquireGuid()))
				{
					xObjectLIF.next();
				}
				else
				{
					xObjectLIF.delete_current();
				}
			}
			else
			{
				xObjectLIF.delete_current();
			}
		}


		// AW - 20/03/03 - First iterate through and add all the valid messages.
		ListSelection_Overlay xOverlay("Select message to send");
		for (int i=0;i<2;i++)
		{
			MessageIDAndDescription *pxMessageList = (i == 0) ? m_pxAsuraMessageList : m_pxProjectMessageList;
			if(pxMessageList)
			{
				MessageIDAndDescription* pxMessageDesc = pxMessageList;

				for(;pxMessageDesc->iID;pxMessageDesc++)
				{
					for(LIF<Tools_Object*> olif(&xSelectedList);!olif.done();olif.next())
					{
						//get the target object
						Tools_Object* pxObject = olif();
						if(pxObject && 
							pxObject->HandlesThisMessage(pxMessageDesc->iID))
						{
							xOverlay.AddItem(pxMessageDesc->pszDescription,pxMessageDesc->iID);
							break;
						}
					}
				}
			}
		}

		if(!xOverlay.GetNumItems())
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Targets don't accept any messages.");
			return;
		}
		if(!xOverlay.DoModal()) return;
		unsigned short usMessageID = static_cast<u_short>(xOverlay.GetSelectionData());

		// Now count how many entities actually accept this message.
		unsigned uObjectCount = 0;
		unsigned uTakesMessageCount = 0;
		for(LIF<Tools_Object*> olif(&xSelectedList);!olif.done();olif.next())
		{
			uObjectCount++;
			Tools_Object* pxObject = olif();
			if(pxObject && 
				pxObject->HandlesThisMessage(usMessageID))
			{
				uTakesMessageCount++;
			}
		}
		// Check to see if they want to continue.
		Asura_Assert(uObjectCount >= uTakesMessageCount, "Sanity check");
		if (uObjectCount != uTakesMessageCount)
		{
			char szText[256];
			Asura_PrintfToBfr(szText, "Warning : You have %d objects selected but only %d will accept this message, continue?", 
				256, true, uObjectCount, uTakesMessageCount);
			YesNo_Overlay xDlg(szText);
			if(!xDlg.DoModal()) return;
		}

		//check there aren't more objects selected than will fit in a message block
		if (uTakesMessageCount > 255)
		{
			char szText[256];
			Asura_PrintfToBfr(szText, "Error : you are trying to send a message to %d objects. The maximum is 255. Try using a proxy trigger.", 
				256, true, uTakesMessageCount);
			Message_Overlay xDlg(szText);
			xDlg.DoModal();
			return;
		}

		// Now add the messages.
		unsigned uNeedWarning = 0;
		for(LIF<Tools_Object*> olif(&xSelectedList);!olif.done();olif.next())
		{
			Tools_Object* pxObject = olif();
			if(pxObject && 
				pxObject->HandlesThisMessage(usMessageID))
			{
				Asura_StaticMessage message;
				message.Initialise();
				message.m_uSendTo = pxObject->GetGuid();
				message.m_usMsgID = usMessageID;
				InitialiseMessageData(&message,false);
				if (message.m_usDataSize)
				{
					uNeedWarning++;
				}
				//add the message to the block
				m_xMessageBlock.AddMessage(message);
				//move the current message index to the newly added message
				m_iCurrentMessageIndex = m_xMessageBlock.GetNumberOfMessagesInBlock()-1;
				SelectMessage(m_iCurrentMessageIndex);
			}
		}
		// Now warn them if the message has data to be edited.
		if (uNeedWarning > 1)
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"WARNING : The data you are editing is just for ONE of the messages you added (not multiple).");
		}
	}

	SetModified();
}

bool Tools_Message_Editing::EditTestMessage(Asura_StaticMessage* pxMessage)
{

	//get the selected object
	Tools_Object* pxObject = Tools_Object_System::SelectedObject;
	if(!pxObject) return false;

	Asura_Guid uGuid = pxObject->GetGuid();
	if(uGuid == ASURA_GUID_UNREGISTERED) return false;

	pxMessage->m_uSendTo = uGuid;

	//ask for the message to be sent
	if(!s_pxThis->EditMessageID(pxMessage,"Select message to send")) return false;
	s_pxThis->InitialiseMessageData(pxMessage,true);

	//create modal overlay for editing
	s_pxTestMessageOverlay = new Tools_MessageEdit_Overlay(pxMessage);
	bool bRetVal = s_pxTestMessageOverlay->DoModal();
	delete s_pxTestMessageOverlay;

	return bRetVal;
}



bool Tools_Message_Editing::EditDestinationGuid(Asura_Guid& uDest,const char* prompt)
{
	ListSelection_Overlay overlay(prompt);

	//get the target object

	//add all the message ids to the overlay
	GuidAndDescription* pxGuidDesc = s_axGuidList;

	for(;(pxGuidDesc->uGuid || pxGuidDesc->pszDescription);pxGuidDesc++)
	{
		overlay.AddItem(pxGuidDesc->pszDescription,pxGuidDesc->uGuid);
	}

	if(s_pfnProjectEditDestinationGuid)
	{
		s_pfnProjectEditDestinationGuid(overlay);
	}

	if(!overlay.GetNumItems())
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"No Guids available!");
		return false;
	}

	if(!overlay.DoModal()) return false;

	//get the selected message
	uDest = static_cast<Asura_Guid>(overlay.GetSelectionData());

	return true;
}

// For special cases like player or game server.
bool Tools_Message_Editing::GuidHandlesThisMessage(Asura_Guid uGuid, int iMessageID) const
{
	if(uGuid == ASURA_GUID_CLIENT || uGuid == ASURA_GUID_ALL_CLIENTMACHINES)
	{
#ifdef ASURA_USE_FSFX
		if ( ( iMessageID == ASURA_MSG_SET_FSFX ) || ( iMessageID == ASURA_MSG_SET_FSFX_NEW ) )
		{
			return true;
		}
#endif
		if ( iMessageID == ASURA_MSG_SOUND_MIXER_PRESET || iMessageID == ASURA_MSG_SOUND_MIXER_PRESET_QUEUE || iMessageID == ASURA_MSG_SOUND_MIXER_PRESET_UNQUEUE )
		{
			return true;
		}

		if ( iMessageID == ASURA_MSG_SOUND_SET_REVERB )
		{
			return true;
		}

#ifdef ASURA_USE_DIALOGUE_SYSTEM
		if( iMessageID == ASURA_MSG_DIALOGUE_PLAYLINE || iMessageID == ASURA_MSG_DIALOGUE_CANCELSUBTITLE)
		{
			return true;
		}

		if( iMessageID == ASURA_MSG_DIALOGUE_PLAYLINE_CONTROLLER || iMessageID == ASURA_MSG_DIALOGUE_STOPLINE_CONTROLLER )
		{
			return true;
		}
#endif

#ifdef ASURA_USE_GUI_MENU_SYSTEM
		if(iMessageID == ASURA_MSG_GUIMENU_ACTIVATE)
		{
			return true;
		}
#endif

		return false;
	}
	else if (uGuid == ASURA_GUID_SERVER)
	{
#ifdef ASURA_USE_METAMUSICZONES
		if(iMessageID == ASURA_MSG_METAMUSIC_ACTIVATEZONE || iMessageID == ASURA_MSG_METAMUSIC_DEACTIVATEZONE)
		{
			return true;
		}
#endif

		if( iMessageID == ASURA_MSG_APPLY_RADIAL_FORCE )
		{
			return true;
		}
	}

	return false;
}


void Tools_Message_Editing::GotoTarget()
{
	Asura_Assert(m_xMessageBlock.IsValid(),"");
	int numMessages = m_xMessageBlock.GetNumberOfMessagesInBlock();
	if(!numMessages) return;

	Asura_StaticMessage* message = m_xMessageBlock.GetMessagePointer(m_iCurrentMessageIndex);
	Tools_Object* object = Tools_Object_System::GetObjectFromGuid(message->m_uSendTo,true);
	if(object)
	{
		Tools_Object_System::GotoAndSelect(object);
	}
}

void Tools_Message_Editing::DeleteMessage()
{
	Asura_Assert(m_xMessageBlock.IsValid(),"");
	if(m_iCurrentMessageIndex<0) return;

	if(m_xMessageBlock.DeleteMessage(m_iCurrentMessageIndex))
	{
		int iNumMessages = m_xMessageBlock.GetNumberOfMessagesInBlock();
		if(m_iCurrentMessageIndex>=iNumMessages && m_iCurrentMessageIndex>0)
		{
			m_iCurrentMessageIndex--;
		}
		SelectMessage(m_iCurrentMessageIndex);
		SetModified();
	}
}

void Tools_Message_Editing::EnableAllMessagesInBlock()
{
	Asura_Assert(m_xMessageBlock.IsValid(),"");
	int iNumMessages = m_xMessageBlock.GetNumberOfMessagesInBlock();

	for(int i=0;i<iNumMessages;i++)
	{
		Asura_StaticMessage* pxMessage = m_xMessageBlock.GetMessagePointer(i);
		pxMessage->m_uToolsFlags &=~ uTOOLS_STATICMESSAGE_FLAG_DISABLED;
	}
	SetModified();
}

void Tools_Message_Editing::DisableAllMessagesInBlock()
{
	Asura_Assert(m_xMessageBlock.IsValid(),"");
	int iNumMessages = m_xMessageBlock.GetNumberOfMessagesInBlock();

	for(int i=0;i<iNumMessages;i++)
	{
		Asura_StaticMessage* pxMessage = m_xMessageBlock.GetMessagePointer(i);
		pxMessage->m_uToolsFlags |= uTOOLS_STATICMESSAGE_FLAG_DISABLED;
	}
	SetModified();
}


void Tools_Message_Editing::SelectMessage(int iMessageIndex)
{
	int iNumMessages = m_xMessageBlock.GetNumberOfMessagesInBlock();

	//make sure the index is in range
	m_iCurrentMessageIndex = iMessageIndex;
	if(m_iCurrentMessageIndex<0) m_iCurrentMessageIndex=0;
	if(m_iCurrentMessageIndex>=iNumMessages) m_iCurrentMessageIndex = iNumMessages-1;

	//make sure the selection will be visible
	m_iTopMessageIndex = (m_iCurrentMessageIndex/iMAX_MESSAGES_TO_SHOW)*iMAX_MESSAGES_TO_SHOW;

	//kill the old message editing overlay
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_MESSAGEEDIT);

	if(!iNumMessages) return;

	//and create a new one
	Asura_StaticMessage* pxMessage = m_xMessageBlock.GetMessagePointer(m_iCurrentMessageIndex);

	s_xMessageBackup.Copy(pxMessage);

	Asura_Overlay* pxOverlay;
	Asura_String xMessageDesc;
	GetMessageDescription(pxMessage->m_usMsgID,0,xMessageDesc);
	pxOverlay = new Asura_Overlay(xMessageDesc.GetString(),false,&EditMessageOverlayPosition,&g_uEditMessageOverlayMinimised);


	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	//Disable message option
	Asura_Dialog* pxDialog = new Asura_FlagTickbox_Dialog("Disabled",xOffset,&pxMessage->m_uToolsFlags,uTOOLS_STATICMESSAGE_FLAG_DISABLED);
	pxDialog->SetHelpText("Disables this messages for testing/debugging purposes.\nMessage flagged in this way won't be included in level exports.");
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxOverlay->AddDialog(pxDialog);


	//time delay is available for all messages
	pxDialog = new Asura_Numerical_Dialog("Delay",xOffset,&pxMessage->m_fDelay,0,9999.0f,0.1f,"%.3f");
	pxDialog->SetHelpText("Time delay for message in seconds");
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxOverlay->AddDialog(pxDialog);

	pxDialog = new Asura_ScaledNumerical_Dialog("Probability",xOffset,100.0f,&pxMessage->m_fProbability,0,100.0f,1.0f," %.2f ");
	pxDialog->SetHelpText("Percentage chance of the message being sent.");
	xOffset.y += Asura_Dialog::LargeSpacing();
	pxOverlay->AddDialog(pxDialog);



	AddOverlayControls(pxMessage,pxOverlay,xOffset);

	pxOverlay->m_uID = TOOLS_OVERLAY_MESSAGEEDIT;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);

}

Asura_StaticMessage* Tools_Message_Editing::GetCurrentMessage()
{
	return m_xMessageBlock.GetMessagePointer( m_iCurrentMessageIndex );
}

bool Tools_Message_Editing::SwapMessages(int iIndex1,int iIndex2)
{
	int iNumMessages = m_xMessageBlock.GetNumberOfMessagesInBlock();
	if(iIndex1 >= iNumMessages || iIndex2 >= iNumMessages) return false;
	if(iIndex1 < 0 || iIndex2 < 0) return false;


	Asura_StaticMessage* pxMessage1 = m_xMessageBlock.GetMessagePointer(iIndex1);
	Asura_StaticMessage* pxMessage2 = m_xMessageBlock.GetMessagePointer(iIndex2);

	if(!pxMessage1 || !pxMessage2) return false;

	Asura_StaticMessage xTemp = *pxMessage1;
	*pxMessage1 = *pxMessage2;
	*pxMessage2 = xTemp;

	return true;
}

struct Tools_StaticMessageSortData
{
	int m_iIndex;
	float m_fDelay;
};

static int SortStaticMessages(const void* pData1, const void* pData2)
{
	const Tools_StaticMessageSortData* pxData1 = (const Tools_StaticMessageSortData*) pData1;
	const Tools_StaticMessageSortData* pxData2 = (const Tools_StaticMessageSortData*) pData2;

	//sort by delay, using the original index to resolve tie breaks
	if(pxData1->m_fDelay < pxData2->m_fDelay) return -1;
	if(pxData1->m_fDelay > pxData2->m_fDelay) return 1;
	if(pxData1->m_iIndex < pxData2->m_iIndex) return -1;
	if(pxData1->m_iIndex > pxData2->m_iIndex) return 1;
	return 0;
}

void Tools_Message_Editing::SortMessages()
{
	int iNumMessages = m_xMessageBlock.GetNumberOfMessagesInBlock();
	if(iNumMessages < 2) return;

	//does any sorting need to be done?
	bool bNeedSort = false;
	for(int i=0;i<iNumMessages-1;i++)
	{
		const Asura_StaticMessage* pxMessage1 = m_xMessageBlock.GetMessagePointer(i);
		const Asura_StaticMessage* pxMessage2 = m_xMessageBlock.GetMessagePointer(i+1);
		if(pxMessage1->m_fDelay > pxMessage2->m_fDelay)
		{
			bNeedSort = true;
			break;
		}
	}
	if(!bNeedSort) return;
	
	//get all the delays for sorting
	Tools_StaticMessageSortData* pxSortData = new Tools_StaticMessageSortData[iNumMessages];
	for(int i=0; i<iNumMessages; i++)
	{
		pxSortData[i].m_iIndex = i;
		pxSortData[i].m_fDelay = m_xMessageBlock.GetMessagePointer(i)->m_fDelay;
	}

	Asura_SortAndSearch::Quicksort(pxSortData, iNumMessages, SortStaticMessages);

	//copy the messages to a temporary array
	Asura_StaticMessage* pxMessages = new Asura_StaticMessage[iNumMessages];
	for(int i=0; i<iNumMessages; i++)
	{
		pxMessages[i] = *m_xMessageBlock.GetMessagePointer(i);
	}
	//and copy back in the new order
	int iNewSelection = 0;
	for(int i=0; i<iNumMessages; i++)
	{
		int iSourceIndex = pxSortData[i].m_iIndex;

		*m_xMessageBlock.GetMessagePointer(i) = pxMessages[iSourceIndex];

		//If moving the current selected message , take note of its new location
		if(iSourceIndex == m_iCurrentMessageIndex)
		{
			iNewSelection = i;
		}
	}

	delete [] pxSortData;
	delete [] pxMessages;

	if(iNewSelection != m_iCurrentMessageIndex)
	{
		SelectMessage(iNewSelection);
	}
}

bool Tools_Message_Editing::EditMessageID(Asura_StaticMessage* message,const char* prompt)
{
	ListSelection_Overlay overlay(prompt);

	//get the target object
	Tools_Object* pxObject = Tools_Object_System::GetObjectFromGuid(message->m_uSendTo,true);

	bool bSubEntity = false;
	if(pxObject)
	{
		bSubEntity = pxObject->GetGuid()!=message->m_uSendTo;
	}

	//add all the message ids to the overlay
	for (int i=0;i<2;i++)
	{
		MessageIDAndDescription *pxMessageList = (i == 0) ? m_pxAsuraMessageList : m_pxProjectMessageList;
		if(pxMessageList)
		{
			MessageIDAndDescription* pxMessageDesc = pxMessageList;

			for(;pxMessageDesc->iID;pxMessageDesc++)
			{
				if (pxObject)
				{
					if (!pxObject->HandlesThisMessage(message->m_uSendTo,pxMessageDesc->iID)) continue;
				}
				else
				{
					if (!GuidHandlesThisMessage(message->m_uSendTo, pxMessageDesc->iID)) continue;
				}
				overlay.AddItem(pxMessageDesc->pszDescription,pxMessageDesc->iID);
			}
		}
	}

	if(!overlay.GetNumItems())
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Target doesn't accept any messages.");
		return false;
	}

	if(!overlay.DoModal()) return false;

	//get the selected message
	message->m_usMsgID = static_cast<u_short>(overlay.GetSelectionData());

	return true;
}

bool Tools_Message_Editing::EditMessageID(Asura_StaticMessage* message, const char* prompt, Asura_List<Asura_Guid>& xTargetItemList)
{
	ListSelection_Overlay overlay(prompt);

	//add all the message ids to the overlay
	for (int i=0;i<2;i++)
	{
		MessageIDAndDescription *pxMessageList = (i == 0) ? m_pxAsuraMessageList : m_pxProjectMessageList;
		if(pxMessageList)
		{
			MessageIDAndDescription* pxMessageDesc = pxMessageList;

			for(;pxMessageDesc->iID;pxMessageDesc++)
			{
				Asura_List_It<Asura_Guid> xIter (xTargetItemList);

				bool bIsHandled = true;

				for(; !xIter.Done(); xIter.Next())
				{
					Asura_Guid uThisGuid = *xIter.GetCurrent();

					Tools_Object* pxObject = Tools_Object_System::GetObjectFromGuid(uThisGuid, true);

					if (pxObject)
					{
						if (!pxObject->HandlesThisMessage(uThisGuid,pxMessageDesc->iID))
						{
							bIsHandled = false;
							break;
						}
					}
					else
					{
						if (!GuidHandlesThisMessage(uThisGuid, pxMessageDesc->iID))
						{
							bIsHandled = false;
							break;
						}
					}
				}

				if (bIsHandled)
				{
					overlay.AddItem(pxMessageDesc->pszDescription, pxMessageDesc->iID);
				}
			}
		}
	}

	if(!overlay.GetNumItems())
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Target doesn't accept any messages.");
		return false;
	}

	if(!overlay.DoModal()) return false;

	//get the selected message
	message->m_usMsgID = static_cast<u_short>(overlay.GetSelectionData());

	return true;
}

void Tools_Message_Editing::InitialiseMessageData(struct Asura_StaticMessage* pxMessage,bool bAutoEdit)
{
	InitialiseMessageData(pxMessage);
	if(bAutoEdit)
	{
		//auto-edit some messages as soon as they get created
		switch(pxMessage->m_usMsgID)
		{
			case ASURA_MSG_TEMPLATE_TRIGGERINPUT :
				Tools_Entity_Template_Object::EditTriggerInputMessage(pxMessage);
				break;

			case ASURA_MSG_TEMPLATE_TRIGGEROUTPUT :
				Tools_Entity_Template_Object::EditTriggerOutputMessage(pxMessage);
				break;

			case ASURA_MSG_LOGICTRIGGER_INPUTON :
			case ASURA_MSG_LOGICTRIGGER_INPUTOFF :
				Tools_Entity_LogicTrigger_Object::EditInputMessage(pxMessage);
				break;

			case ASURA_MSG_TIMELINETRIGGER_ENABLEMESSAGE :
			case ASURA_MSG_TIMELINETRIGGER_DISABLEMESSAGE :
			case ASURA_MSG_TIMELINETRIGGER_SETTRIGGERTIME :
				Tools_Entity_TimelineTrigger_Object::EditEnableMessage(pxMessage);
				break;

			case ASURA_MSG_COUNTEDTRIGGER_ENABLEMESSAGE :
			case ASURA_MSG_COUNTEDTRIGGER_DISABLEMESSAGE :
			case ASURA_MSG_COUNTEDTRIGGER_SETTRIGGERCOUNT :
				Tools_Entity_CountedTrigger_Object::EditMessageBlockHash(pxMessage);
				break;

			default :
				break;
		}
	}

}

void Tools_Message_Editing::InitialiseMessageData(struct Asura_StaticMessage* pxMessage)
{
	Asura_Assert(pxMessage,"");
	pxMessage->DeallocateMessageData();

	switch(pxMessage->m_usMsgID)
	{
		case ASURA_MSG_SPLITTER_ACTIVATEOUTPUT :
		case ASURA_MSG_SPLITTER_DEACTIVATEOUTPUT :
		case ASURA_MSG_SPLITTER_TRIGGEROUTPUT :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_SplitterBlock_Output), true);
			break;
		}

		case ASURA_MSG_LOGICTRIGGER_INPUTON :
		case ASURA_MSG_LOGICTRIGGER_INPUTOFF :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_LogicTrigger_Input), true);
			break;
		}

		case ASURA_MSG_TIMELINETRIGGER_ENABLEMESSAGE :
		case ASURA_MSG_TIMELINETRIGGER_DISABLEMESSAGE :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_TimelineTrigger), true);
			break;
		}

		case ASURA_MSG_SETTIMER :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_SetTimer), true);
			break;
		}

		case ASURA_MSG_TIMETRIGGER_SETTRIGGERTIME :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_TimeTrigger_SetTriggerTime), true);
			break;
		}

		case ASURA_MSG_TIMETRIGGER_ADDTOTRIGGERTIME :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_TimeTrigger_AddToTriggerTime), true);
			break;
		}


		case ASURA_MSG_TIMELINETRIGGER_SETTRIGGERTIME :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_TimelineTrigger_SetTriggerTime), true);
			break;
		}

		case ASURA_MSG_COUNTEDTRIGGER_ENABLEMESSAGE :
		case ASURA_MSG_COUNTEDTRIGGER_DISABLEMESSAGE :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_CountedTrigger_MessageHash), true);
			break;
		}

		case ASURA_MSG_COUNTEDTRIGGER_SETCOUNTER :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_CountedTrigger_SetCounter), true);
			break;
		}

		case ASURA_MSG_COUNTEDTRIGGER_ADDCOUNTER :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_CountedTrigger_AddCounter), true);
			Asura_Message_CountedTrigger_AddCounter* pxData = static_cast< Asura_Message_CountedTrigger_AddCounter* >( pxMessage->m_pData );
			pxData->m_iChange = 1;
			break;
		}

		case ASURA_MSG_COUNTEDTRIGGER_SETTRIGGERCOUNT :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_CountedTrigger_SetTriggerCount), true);
			Asura_Message_CountedTrigger_SetTriggerCount *pxData = reinterpret_cast<Asura_Message_CountedTrigger_SetTriggerCount*>(pxMessage->m_pData);
			pxData->m_iTriggerCount = 1;
			break;
		}


		case ASURA_MSG_CHANGE_ANIMATION:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_PhysicalObject_ChangeAnimation), true);
			Asura_Message_PhysicalObject_ChangeAnimation* pxData = reinterpret_cast<Asura_Message_PhysicalObject_ChangeAnimation*>(pxMessage->m_pData);
			pxData->Initialise();
			break;
		}

		case ASURA_MSG_GLASS_SMASH:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_Glass_Smash), true);
			Asura_Message_Glass_Smash *pxData = reinterpret_cast<Asura_Message_Glass_Smash*>(pxMessage->m_pData);
			pxData->Initialise();
			break;
		}

		case ASURA_MSG_LIFTCALL:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_MsgData_LiftCall), true);
			Asura_MsgData_LiftCall *pxData = reinterpret_cast<Asura_MsgData_LiftCall*>(pxMessage->m_pData);
			pxData->Initialise();
			break;
		}

		case ASURA_MSG_TEMPLATE_TRIGGERINPUT :
		case ASURA_MSG_TEMPLATE_TRIGGEROUTPUT :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_Template_Trigger), true);
			break;
		}

		case ASURA_MSG_ACTIVATECUTSCENEWITHTIME:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_ActivateCutsceneWithTime), true);
			Asura_Message_ActivateCutsceneWithTime *pxData = reinterpret_cast<Asura_Message_ActivateCutsceneWithTime*>(pxMessage->m_pData);
			pxData->Initialise();
			break;
		}

		case ASURA_MSG_CUTSCENE_ENABLEEVENT :
		case ASURA_MSG_CUTSCENE_DISABLEEVENT :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_CutsceneEventID), true);
			Asura_Message_CutsceneEventID *pxData = reinterpret_cast<Asura_Message_CutsceneEventID*>(pxMessage->m_pData);
			pxData->m_iEventID = 1;

			break;
		}

		case ASURA_MSG_CUTSCENE_SETCAMERA :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_Cutscene_SetCamera), true);
			break;
		}


		case ASURA_MSG_TEXTUREANIMATION :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_TextureAnimation), true);

			Asura_Message_TextureAnimation *pxData = reinterpret_cast<Asura_Message_TextureAnimation*>(pxMessage->m_pData);
			pxData->m_uFlags = 0;
			pxData->m_uSequence = 0;
			pxData->m_fSpeed = 1.0f;

			break;
		}

		case ASURA_MSG_SET_DESTRUCTIBLE:
		{
			pxMessage->AllocateMessageData( sizeof( bool ), true );
			break;
		}

#ifdef ASURA_USE_METAMUSICZONES
		case ASURA_MSG_METAMUSIC_ACTIVATEZONE :
		case ASURA_MSG_METAMUSIC_DEACTIVATEZONE :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_MsgData_MetaMusic_Zone), true);

		} break;
#endif

#ifdef ASURA_USE_FSFX
		case ASURA_MSG_SET_FSFX:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Hash_ID), true);
			break;
		}
		case ASURA_MSG_SET_FSFX_NEW:
		{
			pxMessage->AllocateMessageData(sizeof( Asura_FSFX_Message ), true);
			Asura_FSFX_Message* const pxFSFXMessage = static_cast<Asura_FSFX_Message*>( pxMessage->m_pData );
			pxFSFXMessage->m_fValue = 1.0f;
			break;
		}
#endif

		case ASURA_MSG_SOUND_MIXER_PRESET:
		{
			pxMessage->AllocateMessageData(sizeof( Asura_Message_SoundMixerPreset ), true);
			Asura_Message_SoundMixerPreset* const pxMessageData = static_cast<Asura_Message_SoundMixerPreset*>( pxMessage->m_pData );
			pxMessageData->m_uHashID = ASURA_HASH_ID_UNSET;
			pxMessageData->m_fFadeTime = 0.0f;
			break;
		}
		case ASURA_MSG_SOUND_MIXER_PRESET_QUEUE:
		case ASURA_MSG_SOUND_MIXER_PRESET_UNQUEUE:
		{
			pxMessage->AllocateMessageData(sizeof( Asura_Message_SoundMixerPreset ), true);
			Asura_Message_SoundMixerPreset* const pxMessageData = static_cast<Asura_Message_SoundMixerPreset*>( pxMessage->m_pData );
			pxMessageData->m_uHashID = ASURA_HASH_ID_UNSET;
			pxMessageData->m_fFadeTime = -1.0f;
			break;
		}

#ifndef ASURA_USE_AUDIO_SYSTEM
		case ASURA_MSG_SOUND_SET_REVERB:
		{
			pxMessage->AllocateMessageData(sizeof( Asura_Message_SoundReverb ), true);
			Asura_Message_SoundReverb* const pxMessageData = static_cast<Asura_Message_SoundReverb*>( pxMessage->m_pData );
			pxMessageData->m_iReverbID = 0;	// TODO pick a default - perhaps Asura_Sound_System::GetDefaultReverb() ?
			break;
		}
#endif

		case ASURA_MSG_PROXY_ADDTARGET :
		case ASURA_MSG_PROXY_REMOVETARGET :
		case ASURA_MSG_PROXY_SETTARGET :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_ProxyTrigger_Target), true);
			Asura_Message_ProxyTrigger_Target* pxMessageData = (Asura_Message_ProxyTrigger_Target*) pxMessage->m_pData;
			pxMessageData->m_uTargetGuid = ASURA_GUID_UNREGISTERED;
			break;
		}
		case ASURA_MSG_LIFT_SETFLAGS:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_Lift_SetFlags), true);
			Asura_Message_Lift_SetFlags* pxLiftMessage = static_cast< Asura_Message_Lift_SetFlags* >( pxMessage->m_pData );
			pxLiftMessage->Reset();
			break;
		}
		case ASURA_MSG_LIFT_NODE_SETFLAGS:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_Lift_Node_SetFlags), true);
			Asura_Message_Lift_Node_SetFlags* pxLiftMessage = static_cast< Asura_Message_Lift_Node_SetFlags* >( pxMessage->m_pData );
			pxLiftMessage->Reset();
			break;
		}
		case ASURA_MSG_LIFT_GOTO_NODE:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_MsgData_Lift_Goto_Node), true);
			Asura_MsgData_Lift_Goto_Node* pxLiftMessage = static_cast< Asura_MsgData_Lift_Goto_Node* >( pxMessage->m_pData );
			*pxLiftMessage->GetFlagsPtr() = 0;
			*pxLiftMessage->GetNodeGuidPtr() = ASURA_GUID_UNREGISTERED;
			break;
		}

#ifdef ASURA_USE_DIALOGUE_SYSTEM
		case ASURA_MSG_DIALOGUE_PLAYLINE :
		case ASURA_MSG_DIALOGUE_PLAYLINEONENTITY :
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_Dialogue_PlayLine),true);
			break;
		}

		case ASURA_MSG_DIALOGUE_CONTROLLER_INTERRUPT:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Msg_Dialogue_Interrupt),true);
			Asura_Msg_Dialogue_Interrupt* pxData = static_cast<Asura_Msg_Dialogue_Interrupt*>( pxMessage->m_pData );
			pxData->SetDefaults();

			break;
		}

		case ASURA_MSG_DIALOGUE_CONTROLLER_TOGGLE_LINE:
		{		 
			pxMessage->AllocateMessageData(sizeof(Asura_Msg_Dialogue_ToggleLine),true);
			Asura_Msg_Dialogue_ToggleLine* pxData = static_cast<Asura_Msg_Dialogue_ToggleLine*>( pxMessage->m_pData );
			pxData->SetDefaults();
			break;
		}

		case ASURA_MSG_DIALOGUE_CONTROLLER_TRIGGER_LINE:
		{		 
			pxMessage->AllocateMessageData(sizeof(Asura_Msg_Dialogue_TriggerLine),true);
			Asura_Msg_Dialogue_TriggerLine* pxData = static_cast<Asura_Msg_Dialogue_TriggerLine*>( pxMessage->m_pData );
			pxData->SetDefaults();
			break;
		}

#endif

		case ASURA_MSG_APPLY_LINEAR_FORCE:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Msg_ApplyForce_Linear),true);
			Asura_Msg_ApplyForce_Linear* pxData = static_cast<Asura_Msg_ApplyForce_Linear*>( pxMessage->m_pData );
			pxData->SetDefaults();
			break;
		}

		case ASURA_MSG_APPLY_RADIAL_FORCE:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Msg_ApplyForce_Radial),true);
			Asura_Msg_ApplyForce_Radial* pxData = static_cast<Asura_Msg_ApplyForce_Radial*>( pxMessage->m_pData );
			pxData->SetDefaults();
			break;
		}

#ifdef ASURA_USE_VIDEO_TO_TEXTURE_SYSTEM
		case ASURA_MSG_FMV_CONTROLLER_PLAY:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Msg_FMV_Controller_Play),true);
			Asura_Msg_FMV_Controller_Play* pxData = static_cast<Asura_Msg_FMV_Controller_Play*>( pxMessage->m_pData );
			pxData->Initialise();
			break;
		}
		case ASURA_MSG_FMV_CONTROLLER_PAUSE:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Msg_FMV_Controller_Pause),true);
			Asura_Msg_FMV_Controller_Pause* pxData = static_cast<Asura_Msg_FMV_Controller_Pause*>( pxMessage->m_pData );
			pxData->Initialise();
			break;
		}
#endif

#ifdef ASURA_USE_AXON
		case ASURA_MSG_AXON_GS2_DONT_STOP_AT_NODE:
		{
			pxMessage->AllocateMessageData( sizeof( Axon_Msg_GS2_Dont_Stop_At_Node ), true );
			Axon_Msg_GS2_Dont_Stop_At_Node* pxData = static_cast< Axon_Msg_GS2_Dont_Stop_At_Node* >( pxMessage->m_pData );
			pxData->Initialise();
			break;
		}

		case ASURA_MSG_AXON_GS2_SET_NODE_PARAMS:
		{
			pxMessage->AllocateMessageData( sizeof( Axon_Msg_GS2_Set_Node_Params ), true );
			Axon_Msg_GS2_Set_Node_Params* pxData = static_cast< Axon_Msg_GS2_Set_Node_Params* >( pxMessage->m_pData );
			pxData->Initialise();
			break;
		}

		case ASURA_MSG_AXON_GS2_TAKE_LINK:
		{
			pxMessage->AllocateMessageData( sizeof( Axon_Msg_GS2_Take_Link ), true );
			Axon_Msg_GS2_Take_Link* pxData = static_cast< Axon_Msg_GS2_Take_Link* >( pxMessage->m_pData );
			pxData->Initialise();
			break;
		}

		case ASURA_MSG_AXON_GS2_SET_LINK_WEIGHT:
		{
			pxMessage->AllocateMessageData( sizeof( Axon_Msg_GS2_Set_Link_Weight ), true );
			Axon_Msg_GS2_Set_Link_Weight* pxData = static_cast< Axon_Msg_GS2_Set_Link_Weight* >( pxMessage->m_pData );
			pxData->Initialise();
			break;
		}

		case ASURA_MSG_AXON_GS2_ANIM_CHANGE:
		{
			pxMessage->AllocateMessageData( sizeof( Axon_Msg_GS2_Anim_Change ), true );
			Axon_Msg_GS2_Anim_Change* pxData = static_cast< Axon_Msg_GS2_Anim_Change* >( pxMessage->m_pData );
			pxData->Initialise();
			break;
		}

		case ASURA_MSG_AXON_GS2_TELEPORT_TO_NODE:
		{
			pxMessage->AllocateMessageData( sizeof( Axon_Msg_GS2_Teleport_To_Node ), true );
			Axon_Msg_GS2_Teleport_To_Node* pxData = static_cast< Axon_Msg_GS2_Teleport_To_Node* >( pxMessage->m_pData );
			pxData->Initialise();
			break;
		}

		case ASURA_MSG_AXON_GS2_PATHFIND_TO_NODE:
		{
			pxMessage->AllocateMessageData( sizeof( Axon_Msg_GS2_Pathfind_To_Node ), true );
			Axon_Msg_GS2_Pathfind_To_Node* pxData = static_cast< Axon_Msg_GS2_Pathfind_To_Node* >( pxMessage->m_pData );
			pxData->Initialise();
			break;
		}
#endif // ASURA_USE_AXON

#ifdef ASURA_USE_GUI_MENU_SYSTEM
		case ASURA_MSG_GUIMENU_ACTIVATE:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_GUIMenu_Activate), true);
			Asura_Message_GUIMenu_Activate* pxGUIData = static_cast< Asura_Message_GUIMenu_Activate* >( pxMessage->m_pData );
			pxGUIData->uHashMenuPageID = ASURA_HASH_ID_UNSET;	// this specifies that it's not a valid id
			break;
		}
#endif

#if !defined(ASURA_DONT_USE_TELEPORTERS)
		case ASURA_MSGID_TELEPORT_ENTITY:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_TeleportEntity), true);
			Asura_Message_TeleportEntity*	pxData	= static_cast<Asura_Message_TeleportEntity*>(pxMessage->m_pData);
			pxData->SetGuidToTeleport(ASURA_GUID_UNREGISTERED);
			break;
		}
#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)


#ifdef ASURA_TARGET_SM3
		case ASURA_MSG_DYNAMICLIGHT_FADE:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_DynamicLight_Fade), true);
			Asura_Message_DynamicLight_Fade*	pxData	= static_cast<Asura_Message_DynamicLight_Fade*>(pxMessage->m_pData);
			pxData->m_fTargetR = 0.0f;
			pxData->m_fTargetG = 0.0f;
			pxData->m_fTargetB = 0.0f;
			pxData->m_fTargetRange = 1.0f;
			pxData->m_fTargetIntensity = 1.0f;
			pxData->m_fDuration = 5.0f;
			pxData->m_uFlags = ASURA_DYNAMICLIGHT_FADE_FLAG_USECOLOUR;
			break;
		}		
#endif

		case ASURA_MSG_TIMETRIGGER_ADDTIME:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_TimeTrigger_AddTime), true);
			break;
		}

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
		case ASURA_MSG_CHANGE_CONSOLEVAR:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_Change_ConsoleVar), true);
			Asura_Message_Change_ConsoleVar* pxData = static_cast< Asura_Message_Change_ConsoleVar* >( pxMessage->m_pData );
			pxData->m_iFlags = Asura_Message_Change_ConsoleVar::FLAG_OPERATION_SET;
			pxData->m_fData  = 0.0f;
			break;
		}
#endif //ASURA_SUPPORT_COMMAND_CONSOLE

		case ASURA_MSG_PLAY_PFX_ON_ENTITY:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Msg_PlayPFXOnEntity), true);
			Asura_Msg_PlayPFXOnEntity* pxData = static_cast< Asura_Msg_PlayPFXOnEntity* >( pxMessage->m_pData );
			pxData->Reset();
			break;
		}

#ifdef ASURA_USE_CONTAINER_SYSTEM
		case ASURA_MSG_CONTAINER_CHANGE_STATE:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_Container_ChangeState), true);
			Asura_Message_Container_ChangeState* pxData = static_cast<Asura_Message_Container_ChangeState*>(pxMessage->m_pData);
			pxData->m_uStateHash = ASURA_HASH_ID_UNSET;
			break;
		}

		case ASURA_MSG_CONTAINER_CHANGE_STATE_EXTENDED:
		{
			pxMessage->AllocateMessageData(sizeof(Asura_Message_Container_ChangeStateExtended), true);
			Asura_Message_Container_ChangeStateExtended* pxData = static_cast<Asura_Message_Container_ChangeStateExtended*>(pxMessage->m_pData);
			pxData->m_uInputIndex = 0;
			pxData->m_uStateHash  = ASURA_HASH_ID_UNSET;
			break;
		}
#endif //ASURA_USE_CONTAINER_SYSTEM

		default:
			//no message data by default ,so do nowt
			break;
	}
}

void Tools_Message_Editing::AddOverlayControls(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	switch(pxMessage->m_usMsgID)
	{
		case ASURA_MSG_SPLITTER_ACTIVATEOUTPUT :
		case ASURA_MSG_SPLITTER_DEACTIVATEOUTPUT :
		case ASURA_MSG_SPLITTER_TRIGGEROUTPUT :
			AddOverlayControls_SplitterOutput(pxMessage,pxOverlay,xOffset);
			break;

		case ASURA_MSG_LOGICTRIGGER_INPUTON :
		case ASURA_MSG_LOGICTRIGGER_INPUTOFF :
			Tools_Entity_LogicTrigger_Object::AddOverlayControls_Input(pxMessage,pxOverlay,xOffset);
			break;

		case ASURA_MSG_TIMELINETRIGGER_ENABLEMESSAGE :
		case ASURA_MSG_TIMELINETRIGGER_DISABLEMESSAGE :
			Tools_Entity_TimelineTrigger_Object::AddOverlayControls_EnableMessage(pxMessage,pxOverlay,xOffset);
			break;

		case ASURA_MSG_SETTIMER :
			Tools_Entity_TimeTrigger_Object::AddOverlayControls_SetTimer(pxMessage,pxOverlay,xOffset);
			break;

		case ASURA_MSG_TIMETRIGGER_SETTRIGGERTIME :
			Tools_Entity_TimeTrigger_Object::AddOverlayControls_SetTriggerTime(pxMessage,pxOverlay,xOffset);
			break;

		case ASURA_MSG_TIMETRIGGER_ADDTOTRIGGERTIME :
			Tools_Entity_TimeTrigger_Object::AddOverlayControls_AddToTriggerTime(pxMessage,pxOverlay,xOffset);
			break;


		case ASURA_MSG_TIMELINETRIGGER_SETTRIGGERTIME :
			Tools_Entity_TimelineTrigger_Object::AddOverlayControls_SetTriggerTime(pxMessage,pxOverlay,xOffset);
			break;

		
		case ASURA_MSG_COUNTEDTRIGGER_ENABLEMESSAGE :
		case ASURA_MSG_COUNTEDTRIGGER_DISABLEMESSAGE :
			Tools_Entity_CountedTrigger_Object::AddOverlayControls_EnableMessage(pxMessage,pxOverlay,xOffset);
			break;

		case ASURA_MSG_COUNTEDTRIGGER_SETCOUNTER :
			Tools_Entity_CountedTrigger_Object::AddOverlayControls_SetCounter(pxMessage,pxOverlay,xOffset);
			break;

		case ASURA_MSG_COUNTEDTRIGGER_ADDCOUNTER :
			Tools_Entity_CountedTrigger_Object::AddOverlayControls_AddCounter(pxMessage,pxOverlay,xOffset);
			break;

		case ASURA_MSG_COUNTEDTRIGGER_SETTRIGGERCOUNT :
			Tools_Entity_CountedTrigger_Object::AddOverlayControls_SetTriggerCount(pxMessage,pxOverlay,xOffset);
			break;

        case ASURA_MSG_CHANGE_ANIMATION:
            AddOverlayControls_ChangeAnim(pxMessage, pxOverlay, xOffset);
            break;

		case ASURA_MSG_GLASS_SMASH:
			AddOverlayControls_GlassSmash(pxMessage, pxOverlay, xOffset);
			break;

		case ASURA_MSG_LIFTCALL:
			AddOverlayControls_LiftCall(pxMessage, pxOverlay, xOffset);
			break;

		case ASURA_MSG_TEMPLATE_TRIGGERINPUT :
		case ASURA_MSG_TEMPLATE_TRIGGEROUTPUT :
			Tools_Entity_Template_Object::AddOverlayControls_TemplateTrigger(pxMessage,pxOverlay,xOffset);
			break;

		case ASURA_MSG_ACTIVATECUTSCENEWITHTIME:
			AddOverlayControls_CutsceneTime(pxMessage, pxOverlay, xOffset);
			break;

		case ASURA_MSG_CUTSCENE_ENABLEEVENT :
		case ASURA_MSG_CUTSCENE_DISABLEEVENT :
			Tools_Entity_CutsceneController_Object::AddOverlayControls_EventID(pxMessage,pxOverlay,xOffset);
			break;

		case ASURA_MSG_CUTSCENE_SETCAMERA :
			Tools_Entity_CutsceneController_Object::AddOverlayControls_SetCamera(pxMessage,pxOverlay,xOffset);
			break;

		case ASURA_MSG_TEXTUREANIMATION :
			Tools_TextureAnimation::AddOverlayControls_TextureAnimationMessage(pxMessage,pxOverlay,xOffset);
			break;

		#ifdef ASURA_USE_METAMUSICZONES
		case ASURA_MSG_METAMUSIC_ACTIVATEZONE :
		case ASURA_MSG_METAMUSIC_DEACTIVATEZONE :
			Tools_MetaMusicZone_Editor::ZoneMessage_AddOverlayControls(pxMessage, pxOverlay, xOffset);
			break;
		#endif

#ifdef ASURA_USE_FSFX
		case ASURA_MSG_SET_FSFX:
			Tools_Mode_FSFX::AddMessageOverlayControls( pxMessage, pxOverlay, xOffset );
			break;
		case ASURA_MSG_SET_FSFX_NEW:
			Tools_Mode_FSFX::AddMessageOverlayControlsNew( pxMessage, pxOverlay, xOffset );
			break;
#endif

        case ASURA_MSG_SET_DESTRUCTIBLE:
            AddOverlayControls_SetDestructible( pxMessage, pxOverlay, xOffset );
            break;

		case ASURA_MSG_SOUND_MIXER_PRESET:
		case ASURA_MSG_SOUND_MIXER_PRESET_QUEUE:
		case ASURA_MSG_SOUND_MIXER_PRESET_UNQUEUE:
			Tools_SoundMixer_PresetMessage::AddOverlayControls( pxMessage, pxOverlay, xOffset );
            break;

#ifndef ASURA_USE_AUDIO_SYSTEM
		case ASURA_MSG_SOUND_SET_REVERB:
			AddOverlayControls_SetReverb( pxMessage, pxOverlay, xOffset );
			break;
#endif

		case ASURA_MSG_PROXY_ADDTARGET :
		case ASURA_MSG_PROXY_REMOVETARGET :
		case ASURA_MSG_PROXY_SETTARGET :
			Tools_Entity_ProxyTrigger_Object::AddMessageOverlayControls(pxMessage, pxOverlay, xOffset);
			break;
		case ASURA_MSG_LIFT_SETFLAGS:
		{
			Tools_Entity_Lift::AddFlagsMessageOverlayControls(pxMessage, pxOverlay, xOffset);
			break;
		}
		case ASURA_MSG_LIFT_NODE_SETFLAGS:
		{
			Tools_EntityNode_LiftNode::AddMessageOverlayControls(pxMessage, pxOverlay, xOffset);
			break;
		}
		case ASURA_MSG_LIFT_GOTO_NODE:
		{
			AddOverlayControls_LiftGotoNode(pxMessage, pxOverlay, xOffset);
			break;
		}

#ifdef ASURA_USE_DIALOGUE_SYSTEM
		case ASURA_MSG_DIALOGUE_FROMINSTANCE:
        {
			Tools_DialogueSpeaker_Object::AddMessageOverlayControls(pxMessage, pxOverlay, xOffset);
			break;
        }

		case ASURA_MSG_DIALOGUE_PLAYLINE:
		case ASURA_MSG_DIALOGUE_PLAYLINEONENTITY:
		{
			Tools_Dialogue_System::AddMessageOverlayControls_PlayLine(pxMessage, pxOverlay, xOffset);
			break;
		}

		case ASURA_MSG_DIALOGUE_CONTROLLER_INTERRUPT:
		{
			Tools_Entity_DialogueController::AddOverlayControls_Interrupt( pxMessage, pxOverlay, xOffset );
			break;
		}
		
		case ASURA_MSG_DIALOGUE_CONTROLLER_TOGGLE_LINE:
		{
			Tools_Entity_DialogueController::AddOverlayControls_ToggleLine( pxMessage, pxOverlay, xOffset );
			break;
		}

		case ASURA_MSG_DIALOGUE_CONTROLLER_TRIGGER_LINE:
		{
			Tools_Entity_DialogueController::AddOverlayControls_TriggerLine( pxMessage, pxOverlay, xOffset );
			break;
		}


#endif //ASURA_USE_DIALOGUE_SYSTEM

		case ASURA_MSG_APPLY_LINEAR_FORCE:
		{
			Tools_ApplyForce_Messages::AddOverlayControls_LinearForce(pxMessage, pxOverlay, xOffset);
			break;
		}

		case ASURA_MSG_APPLY_RADIAL_FORCE:
		{
			Tools_ApplyForce_Messages::AddOverlayControls_RadialForce(pxMessage, pxOverlay, xOffset);
			break;
		}


#ifdef ASURA_USE_AXON
		case ASURA_MSG_AXON_GS2_DONT_STOP_AT_NODE:
		{
			AddOverlayControls_GS2DontStopAtNode( pxMessage, pxOverlay, xOffset );
			break;
		}

		case ASURA_MSG_AXON_GS2_SET_NODE_PARAMS:
		{
			AddOverlayControls_GS2SetNodeParams( pxMessage, pxOverlay, xOffset );
			break;
		}

		case ASURA_MSG_AXON_GS2_TAKE_LINK:
		{
			AddOverlayControls_GS2TakeLink( pxMessage, pxOverlay, xOffset );
			break;
		}

		case ASURA_MSG_AXON_GS2_SET_LINK_WEIGHT:
		{
			AddOverlayControls_GS2SetLinkWeight( pxMessage,  pxOverlay, xOffset );
			break;
		}

		case ASURA_MSG_AXON_GS2_ANIM_CHANGE:
		{
			AddOverlayControls_GS2AnimChange( pxMessage, pxOverlay, xOffset );
			break;
		}

		case ASURA_MSG_AXON_GS2_TELEPORT_TO_NODE:
		{
			AddOverlayControls_GS2TeleportToNode( pxMessage, pxOverlay, xOffset );
			break;
		}

		case ASURA_MSG_AXON_GS2_PATHFIND_TO_NODE:
		{
			AddOverlayControls_GS2PathfindToNode( pxMessage, pxOverlay, xOffset );
			break;
		}
#endif // ASURA_USE_AXON

#ifdef ASURA_USE_GUI_MENU_SYSTEM
		case ASURA_MSG_GUIMENU_ACTIVATE:
		{
			Tools_GUIMenu_System::AddOverlayControls_ActivateGUIMenuPage( pxMessage, pxOverlay, xOffset);
			break;
		}
#endif

#if !defined(ASURA_DONT_USE_TELEPORTERS)
		case ASURA_MSGID_TELEPORT_ENTITY:
		{
			Tools_Entity_Teleporter::AddMessageOverlayControls_TeleportEntity(pxMessage, pxOverlay, xOffset);
			break;
		}
#endif	// !defined(ASURA_DONT_USE_TELEPORTERS)

#ifdef ASURA_TARGET_SM3
		case ASURA_MSG_DYNAMICLIGHT_FADE:
		{
			Tools_Entity_DynamicLight::AddMessageOverlayControls_Fade(pxMessage, pxOverlay, xOffset);
			break;
		}
#endif

		case ASURA_MSG_TIMETRIGGER_ADDTIME:
			Tools_Entity_TimeTrigger_Object::AddOverlayControls_AddTime(pxMessage,pxOverlay,xOffset);
			break;

#ifdef ASURA_SUPPORT_COMMAND_CONSOLE
		case ASURA_MSG_CHANGE_CONSOLEVAR:
		{
			Tools_Entity_ConsoleVar::AddMessageOverlayControls_Change(pxMessage, pxOverlay, xOffset);
			break;
		}
#endif

		case ASURA_MSG_PLAY_PFX_ON_ENTITY:
			AddOverlayControls_PlayPFXOnEntity(pxMessage, pxOverlay, xOffset);
			break;

#ifdef ASURA_USE_CONTAINER_SYSTEM
		case ASURA_MSG_CONTAINER_CHANGE_STATE:
			AddOverlayControls_Container_ChangeState(pxMessage, pxOverlay, xOffset);
			break;

		case ASURA_MSG_CONTAINER_CHANGE_STATE_EXTENDED:
			AddOverlayControls_Container_ChangeStateExtended(pxMessage, pxOverlay, xOffset);
			break;
#endif //ASURA_USE_CONTAINER_SYSTEM

		default:
			break;
	}
}

void Tools_Message_Editing::SelectLiftGotoNodeGuid( int uGuid )
{
	Tools_Lift_SelectGotoNode* pxChooseObject = new Tools_Lift_SelectGotoNode( s_pxLiftGotoNodeMessage->GetNodeGuidPtr() );
	Tools_Object_System::ChooseObject( "Pick the lift node to go to.", pxChooseObject );
}

void Tools_Message_Editing::AddOverlayControls_LiftGotoNode(Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxNewDialogue;

	s_pxLiftGotoNodeMessage = static_cast< Asura_MsgData_Lift_Goto_Node* >( pxMessage->m_pData );

	Asura_String xNodeString( "Node: " );
	if (s_pxLiftGotoNodeMessage->GetNodeGuid() == ASURA_GUID_UNREGISTERED)
	{
		xNodeString += "*Not Set*";
	}
	else
	{
		xNodeString += GetNameFromGuid( s_pxLiftGotoNodeMessage->GetNodeGuid() );
	}

	pxNewDialogue = new Asura_Function_Dialog( xNodeString.GetString(), xOffset, SelectLiftGotoNodeGuid, static_cast<int>(s_pxLiftGotoNodeMessage->GetNodeGuid()) );
	pxNewDialogue->SetHelpText( "Pick the node the lift should go to." );
	pxOverlay->AddDialog( pxNewDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialogue = new Asura_FlagTickbox_Dialog("Teleport To Node", xOffset, s_pxLiftGotoNodeMessage->GetFlagsPtr(), 1<<0);
	pxNewDialogue->SetHelpText("lift will teleport directly to node; the rest of it's state is unchanged. New node does not need to be linked to pervious");
	pxOverlay->AddDialog(pxNewDialogue);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

#ifdef ASURA_USE_AXON
void Tools_Message_Editing::AddOverlayControls_GS2SetNodeParams( Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	Axon_Msg_GS2_Set_Node_Params* pxSNPMsg = static_cast< Axon_Msg_GS2_Set_Node_Params* >( pxMessage->m_pData );

	Asura_FlagTickbox_Dialog* pxFTDialogue = new Asura_FlagTickbox_Dialog( "", xOffset, &pxSNPMsg->m_iDataValidFlags, Axon_Msg_GS2_Set_Node_Params::STOP_AT_NODE, &m_bRequestRefreshMessageOverlay );
	pxOverlay->AddDialog( pxFTDialogue );
	xOffset.x += 5.0f;
	Asura_Dialog* pxDialogue = new Asura_Tickbox_Dialog( "Stop At Node", xOffset, &pxSNPMsg->m_bStopAtNode, &m_bRequestRefreshMessageOverlay );
	pxDialogue->SetEnabled( pxSNPMsg->m_iDataValidFlags & Axon_Msg_GS2_Set_Node_Params::STOP_AT_NODE );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x -= 5.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxFTDialogue = new Asura_FlagTickbox_Dialog( "", xOffset, &pxSNPMsg->m_iDataValidFlags, Axon_Msg_GS2_Set_Node_Params::ANIM_SET, &m_bRequestRefreshMessageOverlay );
	pxOverlay->AddDialog( pxFTDialogue );
	xOffset.x += 5.0f;

	Asura_String xAnimSetString( "Anim Set: " );
	if ( pxSNPMsg->m_uAnimSet == uAXON_ANIMATION_SEQUENCE_ID_NONE )
	{
		xAnimSetString += "*none*";
	}
	else
	{
		const Axon_Gamescene_AnimTemplate* pxTemplate = Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( pxSNPMsg->m_uAnimSet );
		if ( pxTemplate )
		{
			xAnimSetString += pxTemplate->m_xTemplateName;
		}
		else
		{
			xAnimSetString += "NOT FOUND";
		}
	}
	pxDialogue = new Asura_Function_Dialog( xAnimSetString.GetString(), xOffset, PickGS2NodeParamsAnimSet, reinterpret_cast< int >( &pxSNPMsg->m_uAnimSet ), &m_bRequestRefreshMessageOverlay );
	pxDialogue->SetEnabled( ( pxSNPMsg->m_iDataValidFlags & Axon_Msg_GS2_Set_Node_Params::ANIM_SET ) != 0 );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x -= 5.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	Asura_String xExitConditionString( "Exit Condition: " );
	xExitConditionString += Axon_BehaviourParams_GS2Node::s_pszExitConditionStrings[ pxSNPMsg->m_iExitCondition ];
	pxFTDialogue = new Asura_FlagTickbox_Dialog( "", xOffset, &pxSNPMsg->m_iDataValidFlags, Axon_Msg_GS2_Set_Node_Params::EXIT_CONDITION, &m_bRequestRefreshMessageOverlay );
	pxOverlay->AddDialog( pxFTDialogue );
	xOffset.x += 5.0f;
	pxDialogue = new Asura_Function_Dialog( xExitConditionString.GetString(), xOffset, PickGS2NodeParamsExitCondition, reinterpret_cast< int >( &pxSNPMsg->m_iExitCondition ), &m_bRequestRefreshMessageOverlay );
	pxDialogue->SetEnabled( ( pxSNPMsg->m_iDataValidFlags & Axon_Msg_GS2_Set_Node_Params::EXIT_CONDITION ) != 0 );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x -= 5.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxFTDialogue = new Asura_FlagTickbox_Dialog( "", xOffset, &pxSNPMsg->m_iDataValidFlags, Axon_Msg_GS2_Set_Node_Params::MIN_EXIT_WAIT_TIME, &m_bRequestRefreshMessageOverlay );
	pxOverlay->AddDialog( pxFTDialogue );
	xOffset.x += 5.0f;
	pxDialogue = new Asura_Numerical_Dialog( "Min Wait Time", xOffset, &pxSNPMsg->m_fMinExitWaitTime, 0.0f, 999.9f, 0.1f, "%.2f" );
	pxDialogue->SetEnabled( ( pxSNPMsg->m_iDataValidFlags & Axon_Msg_GS2_Set_Node_Params::MIN_EXIT_WAIT_TIME ) != 0 );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x -= 5.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	

	pxFTDialogue = new Asura_FlagTickbox_Dialog( "", xOffset, &pxSNPMsg->m_iDataValidFlags, Axon_Msg_GS2_Set_Node_Params::MAX_EXIT_WAIT_TIME, &m_bRequestRefreshMessageOverlay );
	pxOverlay->AddDialog( pxFTDialogue );
	xOffset.x += 5.0f;
	pxDialogue = new Asura_Numerical_Dialog( "Max Wait Time", xOffset, &pxSNPMsg->m_fMaxExitWaitTime, 0.0f, 999.9f, 0.1f, "%.2f" );
	pxDialogue->SetEnabled( ( pxSNPMsg->m_iDataValidFlags & Axon_Msg_GS2_Set_Node_Params::MAX_EXIT_WAIT_TIME ) != 0 );
	pxOverlay->AddDialog( pxDialogue );
	xOffset.x -= 5.0f;
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Message_Editing::PickGS2NodeParamsAnimSet( int iUserData )
{
	u_int* puAnimSet = reinterpret_cast< u_int* >( iUserData );

	const u_int uTypeBitfield = (1 << AXON_ATT_STATIC);
	u_int uSequenceID;
	if ( Tools_AnimTemplates::ChooseAnimTemplate( uSequenceID, uTypeBitfield, uAXON_ANIMATION_SEQUENCE_ID_NONE, *puAnimSet, true ) )
	{
		*puAnimSet = uSequenceID;
	}
}

void Tools_Message_Editing::PickGS2NodeParamsExitCondition( int iUserData )
{
	int* piExitCondition = reinterpret_cast< int* >( iUserData );

	ListSelection_Overlay xBehaviourList( "Pick The Exit Condition:" );
	for ( int i = 0; i < AXON_GS2_NEC_NUM_CORE_CONDITIONS; ++i )
	{
		xBehaviourList.AddItem( Axon_BehaviourParams_GS2Node::s_pszExitConditionStrings[ i ], i );
	}
	xBehaviourList.SetInitialSelection( *piExitCondition );
	if ( xBehaviourList.DoModal() )
	{
		*piExitCondition = xBehaviourList.GetSelectionData();
	}
}

void Tools_Message_Editing::AddOverlayControls_GS2DontStopAtNode( Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	Axon_Msg_GS2_Dont_Stop_At_Node* pxDSANMsg = static_cast< Axon_Msg_GS2_Dont_Stop_At_Node* >( pxMessage->m_pData );

	Asura_StringValuePair_DropDownList_Dialog* pxDDLD = new Asura_StringValuePair_DropDownList_Dialog( xOffset, "", &pxDSANMsg->m_iNodeType, &m_bRequestRefreshMessageOverlay );
	pxDDLD->AddItem( "Next Node", Axon_Msg_GS2_Dont_Stop_At_Node::NEXT_NODE, ( pxDSANMsg->m_iNodeType == Axon_Msg_GS2_Dont_Stop_At_Node::NEXT_NODE ) );
	pxDDLD->AddItem( "All Nodes", Axon_Msg_GS2_Dont_Stop_At_Node::ALL_NODES, ( pxDSANMsg->m_iNodeType == Axon_Msg_GS2_Dont_Stop_At_Node::ALL_NODES ) );
	pxDDLD->AddItem( "Specific Node", Axon_Msg_GS2_Dont_Stop_At_Node::SPECIFIC_NODE, ( pxDSANMsg->m_iNodeType == Axon_Msg_GS2_Dont_Stop_At_Node::SPECIFIC_NODE ) );
	pxDDLD->AddItem( "Clear Settings", Axon_Msg_GS2_Dont_Stop_At_Node::CLEAR_SETTINGS,
		( pxDSANMsg->m_iNodeType == Axon_Msg_GS2_Dont_Stop_At_Node::CLEAR_SETTINGS ), "Resume stopping at nodes." );
	pxDDLD->InitialiseData();
	pxDDLD->CreateGadgets( xOffset );
	pxOverlay->AddDialog( pxDDLD );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	if ( pxDSANMsg->m_iNodeType == Axon_Msg_GS2_Dont_Stop_At_Node::SPECIFIC_NODE )
	{
		Asura_String xNodeString( "Node: " );
		if ( pxDSANMsg->m_uNodeGuid == ASURA_GUID_UNREGISTERED )
		{
			xNodeString += "*none*";
		}
		else
		{
			xNodeString += GetNameFromGuid( pxDSANMsg->m_uNodeGuid );
		}

		Asura_Dialog* pxDialogue = new Asura_Function_Dialog( xNodeString.GetString(), xOffset, SelectGS2Node, reinterpret_cast< int >( &pxDSANMsg->m_uNodeGuid ) );
		pxOverlay->AddDialog( pxDialogue );
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}
}

void Tools_Message_Editing::SelectGS2Node( int iUserData )
{
	Tools_ChooseObject_GS2Node* pxChooseObject = new Tools_ChooseObject_GS2Node( reinterpret_cast< Asura_Guid* >( iUserData ), &s_pxThis->m_bRequestRefreshMessageOverlay );
	Tools_Object_System::ChooseObject( "Pick the node to skip over.", pxChooseObject );
}

void Tools_Message_Editing::AddOverlayControls_GS2TakeLink( Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	s_pxTakeLinkMessage = static_cast< Axon_Msg_GS2_Take_Link* >( pxMessage->m_pData );

	Asura_String xLinkString( "Link: " );
	if ( s_pxTakeLinkMessage->m_uLinkGuid == ASURA_GUID_UNREGISTERED )
	{
		xLinkString += "*none*";
	}
	else
	{
		xLinkString += GetNameFromGuid( s_pxTakeLinkMessage->m_uLinkGuid );
	}

	Asura_Dialog* pxNewDialogue;
	pxNewDialogue = new Asura_Function_Dialog( xLinkString.GetString(), xOffset, SelectGS2TakeLinkGuid );
	pxNewDialogue->SetHelpText( "The link the actor on gamescene should take if they come across it." );
	pxOverlay->AddDialog(pxNewDialogue);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialogue = new Asura_Function_Dialog( "Reset Link", xOffset, ResetGS2TakeLinkGuid );
	pxNewDialogue->SetHelpText( "Resets/zeroes the link the actor on gamescene should take." );
	pxOverlay->AddDialog(pxNewDialogue);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Message_Editing::SelectGS2TakeLinkGuid()
{
	Tools_ChooseObject_GS2Spline* pxChooseObject = new Tools_ChooseObject_GS2Spline( SetGS2TakeLinkGuid );
	Tools_Object_System::ChooseObject( "Pick the link the actor on gamescene should take if they come across it.", pxChooseObject );
}

void Tools_Message_Editing::SetGS2TakeLinkGuid( Asura_Guid uGuid )
{
	s_pxTakeLinkMessage->m_uLinkGuid = uGuid;
	UpdateMessageOverlay();
}

void Tools_Message_Editing::ResetGS2TakeLinkGuid()
{
	s_pxTakeLinkMessage->m_uLinkGuid = ASURA_GUID_UNREGISTERED;
	UpdateMessageOverlay();
}

void Tools_Message_Editing::AddOverlayControls_GS2SetLinkWeight( Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	const Asura_ServerNode_GS2Node* pxGameGS2Node = Asura_ServerNode_GS2Node::FindEntity( pxMessage->m_uSendTo );
	if ( !pxGameGS2Node )
	{
		Asura_Dialog* pxNewDialogue = new Asura_Dialog( "ERROR - could not find destination node.", xOffset );
		pxOverlay->AddDialog( pxNewDialogue );
		xOffset.y += Asura_Dialog::DefaultSpacing();
		return;
	}

	Axon_Msg_GS2_Set_Link_Weight* pxOldSLWMsg = reinterpret_cast< Axon_Msg_GS2_Set_Link_Weight* >( new u_char[ pxMessage->m_usDataSize ] );
	Asura_Mem::Copy( pxOldSLWMsg, pxMessage->m_pData, pxMessage->m_usDataSize );
	pxMessage->DeallocateMessageData();
	pxMessage->AllocateMessageData( sizeof( Axon_Msg_GS2_Set_Link_Weight ) + pxGameGS2Node->GetNumberLinks() * sizeof( Axon_Msg_GS2_Set_Link_Weight::BoolGuidWeight ), true );
	s_pxGS2SetLinkWeightMsg = static_cast< Axon_Msg_GS2_Set_Link_Weight* >( pxMessage->m_pData );
	s_pxGS2SetLinkWeightMsg->m_uVersion = Axon_Msg_GS2_Set_Link_Weight::CURRENT_VERSION;
	s_pxGS2SetLinkWeightMsg->m_uNumEntries = pxGameGS2Node->GetNumberLinks();

	// Copy over the old data, removing any links that no longer exist and adding any that are new.
	Axon_Msg_GS2_Set_Link_Weight::BoolGuidWeight* pxOldBGWArray = pxOldSLWMsg->GetBGWArray();
	Axon_Msg_GS2_Set_Link_Weight::BoolGuidWeight* pxNewBGWArray = s_pxGS2SetLinkWeightMsg->GetBGWArray();
	for ( u_int u = 0; u < pxGameGS2Node->GetNumberLinks(); ++u )
	{
		pxNewBGWArray[ u ].m_uLinkGuid = pxGameGS2Node->GetLink( u );

		u_int uOldIndex = UINT_MAX;
		for ( u_int v = 0; v < pxOldSLWMsg->m_uNumEntries && uOldIndex == UINT_MAX; ++v )
		{
			if ( pxOldBGWArray[ v ].m_uLinkGuid == pxNewBGWArray[ u ].m_uLinkGuid )
			{
				uOldIndex = v;
			}
		}
		if ( uOldIndex == UINT_MAX )
		{
			pxNewBGWArray[ u ].m_bInUse  = false;
			pxNewBGWArray[ u ].m_fWeight = 1.0f;
		}
		else
		{
			pxNewBGWArray[ u ].m_bInUse  = pxOldBGWArray[ uOldIndex ].m_bInUse;
			pxNewBGWArray[ u ].m_fWeight = pxOldBGWArray[ uOldIndex ].m_fWeight;
		}
	}

	delete [] pxOldSLWMsg;

	// OK, now we can get on with adding the dialogues.
	CreateGS2LinkWeightStrings( pxGameGS2Node );
	if ( s_uNumGS2LinkWeightStrings )
	{
		s_pxGS2LinkWeightTable = new Asura_Table_Dialog( xOffset, 40.0f, 7.5f, true );
		Asura_Vector_2 xCellSpacing = s_pxGS2LinkWeightTable->GetCellSpacing();
		xCellSpacing.y = 0.25f;
		s_pxGS2LinkWeightTable->SetCellSpacing( xCellSpacing );

		const Asura_Vector_2 xCellSize( 5.0f, 1.0f );
		s_pxGS2LinkWeightTable->CreateTable( xCellSize, pxGameGS2Node->GetNumberLinks(), 2, CreateWeightTableCell, this, s_ppszGS2LinkWeightStrings );
		pxOverlay->AddDialog( s_pxGS2LinkWeightTable );
		xOffset.y += s_pxGS2LinkWeightTable->GetTableSize().y + 0.5f;
	}
	else
	{
		Asura_Dialog* pxNewDialogue = new Asura_Dialog( "No outgoing links.", xOffset );
		pxOverlay->AddDialog( pxNewDialogue );
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}
}

class Tools_TableCell_GS2LinkInUse : public Asura_TableCell_Dialog
{
	typedef Asura_TableCell_Dialog PARENT;

public:
	Tools_TableCell_GS2LinkInUse( Asura_Table_Dialog& xTableDialogue, const Asura_Vector_2& xOffset,
								  const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn,
								  bool* pbDialogVar )
	: PARENT( xTableDialogue, xOffset, xMinCellSize, uRow, uColumn )
	{
		Asura_Vector_2 xLocalOffset( 0.0f, 0.0f );
		Asura_Dialog* pxDialog = new Asura_Tickbox_Dialog( "", xLocalOffset, pbDialogVar );
		AddDialogToPanel( pxDialog );
		xLocalOffset.x += pxDialog->m_xMaxCoord.x;
	}

	virtual int	Compare( const Asura_TableCell_Dialog& xOther ) const { return 0; }
};

class Tools_TableCell_GS2LinkWeight : public Asura_TableCell_Dialog
{
	typedef Asura_TableCell_Dialog PARENT;

public:
	Tools_TableCell_GS2LinkWeight( Asura_Table_Dialog& xTableDialogue, const Asura_Vector_2& xOffset,
								   const Asura_Vector_2& xMinCellSize, const u_int uRow, const u_int uColumn,
								   float* pfDialogVar, Axon_Msg_GS2_Set_Link_Weight* pxSLWMessage )
	: PARENT( xTableDialogue, xOffset, xMinCellSize, uRow, uColumn )
	{
		Asura_Vector_2 xLocalOffset( 0.0f, 0.0f );
		Asura_Dialog* pxDialog = new Asura_Numerical_Dialog( "", xLocalOffset, pfDialogVar, 0.0f, 10.0f, 0.05f, "%0.2f" );
		pxDialog->SetIsEnabledFunction( IsEnabled, uRow );
		AddDialogToPanel( pxDialog );
		xLocalOffset.x += pxDialog->m_xMaxCoord.x;

		s_pxSLWMessage = pxSLWMessage;
	}

	virtual int	Compare( const Asura_TableCell_Dialog& xOther ) const { return 0; }

private:
	static bool IsEnabled( int iRow )
	{
		return s_pxSLWMessage->GetBGWArray()[ iRow ].m_bInUse;
	}

private:
	static Axon_Msg_GS2_Set_Link_Weight* s_pxSLWMessage;
};

Axon_Msg_GS2_Set_Link_Weight* Tools_TableCell_GS2LinkWeight::s_pxSLWMessage = NULL;

Asura_TableCell_Dialog* Tools_Message_Editing::CreateWeightTableCell( Asura_Table_Dialog& xTableDialog,
																	  const Asura_Vector_2& xOffset, const Asura_Vector_2& xMinCellSize,
																	  const u_int uRow, const u_int uColumn, void* const pxUserData )
{
	if ( uColumn == 0 )
	{
		return new Tools_TableCell_GS2LinkInUse( xTableDialog, xOffset, xMinCellSize, uRow, uColumn,
												 &s_pxGS2SetLinkWeightMsg->GetBGWArray()[ uRow ].m_bInUse );
	}
	else //if ( uColumn == 1 )
	{
		return new Tools_TableCell_GS2LinkWeight( xTableDialog, xOffset, xMinCellSize, uRow, uColumn,
												  &s_pxGS2SetLinkWeightMsg->GetBGWArray()[ uRow ].m_fWeight, s_pxGS2SetLinkWeightMsg );
	}
}

void Tools_Message_Editing::CreateGS2LinkWeightStrings( const Asura_ServerNode_GS2Node* pxGameGS2Node )
{
	DestroyGS2LinkWeightStrings();

	s_uNumGS2LinkWeightStrings = pxGameGS2Node->GetNumberLinks();
	if ( s_uNumGS2LinkWeightStrings == 0 ) { return; }
	s_ppszGS2LinkWeightStrings = new char*[ s_uNumGS2LinkWeightStrings ];

	for ( u_int u = 0; u < s_uNumGS2LinkWeightStrings; ++u )
	{
		Tools_EntityNode_GS2Spline* pxToolsGS2Spline = Tools_EntityNode_GS2Spline::FindEntity( pxGameGS2Node->GetLink( u ) );
		if ( pxToolsGS2Spline )
		{
			const u_int uNameLengthIncNull = strlen( pxToolsGS2Spline->GetName() ) + 1;
			s_ppszGS2LinkWeightStrings[ u ] = new char[ uNameLengthIncNull ];
			Asura_PrintfToBfr( s_ppszGS2LinkWeightStrings[ u ], "%s", uNameLengthIncNull, true, pxToolsGS2Spline->GetName() );
		}
		else
		{
			static const char* s_pszErrorString = "ERROR - CANNOT FIND GS2 SPLINE";
			const u_int uNameLengthIncNull = strlen( s_pszErrorString ) + 1;
			s_ppszGS2LinkWeightStrings[ u ] = new char[ uNameLengthIncNull ];
			Asura_PrintfToBfr( s_ppszGS2LinkWeightStrings[ u ], "%s", uNameLengthIncNull, true, s_pszErrorString );
		}
	}
}

void Tools_Message_Editing::DestroyGS2LinkWeightStrings()
{
	if ( s_ppszGS2LinkWeightStrings )
	{
		for ( u_int u = 0; u < s_uNumGS2LinkWeightStrings; ++u )
		{
			delete [] s_ppszGS2LinkWeightStrings[ u ];
			s_ppszGS2LinkWeightStrings[ u ] = NULL;
		}
		delete [] s_ppszGS2LinkWeightStrings;
		s_ppszGS2LinkWeightStrings  = NULL;
		s_uNumGS2LinkWeightStrings = 0;
	}
}

void Tools_Message_Editing::AddOverlayControls_GS2AnimChange( Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	s_pxGS2AnimChangeMessage = static_cast< Axon_Msg_GS2_Anim_Change* >( pxMessage->m_pData );

	Asura_String xASTitle( "Anim Set: " );
	const Axon_Gamescene_AnimTemplate* pxTemplate = Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( s_pxGS2AnimChangeMessage->m_uAnimSet );
	if ( pxTemplate )
	{
		xASTitle += pxTemplate->m_xTemplateName;
	}
	else
	{
		xASTitle += "*none*";
	}
	Asura_Dialog* pxNewDialog = new Asura_Function_Dialog( xASTitle.GetString(), xOffset, PickAnimChangeAnimSet );
	pxOverlay->AddDialog( pxNewDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Message_Editing::PickAnimChangeAnimSet()
{
	const u_int uTypeBitfield = (1 << AXON_ATT_STATIC) | (1 << AXON_ATT_LOOPINGMOVEMENT) | (1 << AXON_ATT_RIGIDMOVEMENT);
	u_int uSequenceID;
	if ( Tools_AnimTemplates::ChooseAnimTemplate( uSequenceID, uTypeBitfield, uAXON_ANIMATION_SEQUENCE_ID_NONE, s_pxGS2AnimChangeMessage->m_uAnimSet, true ) )
	{
		s_pxGS2AnimChangeMessage->m_uAnimSet = uSequenceID;
		s_pxThis->UpdateMessageOverlay();
	}
}

void Tools_Message_Editing::AddOverlayControls_GS2TeleportToNode( Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	Asura_Dialog* pxNewDialogue;

	s_pxGS2TeleportToNodeMessage = static_cast< Axon_Msg_GS2_Teleport_To_Node* >( pxMessage->m_pData );

	pxNewDialogue = new Asura_Numerical_Dialog( "Node", xOffset, &s_pxGS2TeleportToNodeMessage->m_uNodeGuid, 0, 10000000, 1, "%d" );
	pxOverlay->AddDialog( pxNewDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialogue = new Asura_Function_Dialog( "Select Node", xOffset, SelectGS2TeleportToNodeGuid );
	pxNewDialogue->SetHelpText( "The node the actor should teleport to." );
	pxOverlay->AddDialog(pxNewDialogue);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Message_Editing::SelectGS2TeleportToNodeGuid()
{
	Tools_ChooseObject_GS2Node* pxChooseObject = new Tools_ChooseObject_GS2Node( &s_pxGS2TeleportToNodeMessage->m_uNodeGuid );
	Tools_Object_System::ChooseObject( "Pick the node the actor should teleport to.", pxChooseObject );
}

void Tools_Message_Editing::AddOverlayControls_GS2PathfindToNode( Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset )
{
	s_pxGS2PathfindToNodeMessage = static_cast< Axon_Msg_GS2_Pathfind_To_Node* >( pxMessage->m_pData );

	Asura_String xNodeString( "Node: " );
	if ( s_pxGS2PathfindToNodeMessage->m_uDestNodeGuid == ASURA_GUID_UNREGISTERED )
	{
		xNodeString += "*unset*";
	}
	else
	{
		xNodeString += GetNameFromGuid( s_pxGS2PathfindToNodeMessage->m_uDestNodeGuid );
	}

	Asura_Dialog* pxNewDialogue;
	pxNewDialogue = new Asura_Function_Dialog( xNodeString.GetString(), xOffset, SelectGS2PathfindToNodeGuid );
	pxNewDialogue->SetHelpText( "Set the node the actor should route find to." );
	pxOverlay->AddDialog( pxNewDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxNewDialogue = new Asura_Function_Dialog( "Reset Node Guid", xOffset, ResetGS2PathfindToNodeGuid );
	pxOverlay->AddDialog( pxNewDialogue );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Message_Editing::SelectGS2PathfindToNodeGuid()
{
	Tools_ChooseObject_GS2Node* pxChooseObject = new Tools_ChooseObject_GS2Node( SetGS2PathfindToNodeGuid );
	Tools_Object_System::ChooseObject( "Pick the node the actor on gamescene should pathfind to.", pxChooseObject );
}

void Tools_Message_Editing::SetGS2PathfindToNodeGuid( Asura_Guid uGuid )
{
	s_pxGS2PathfindToNodeMessage->m_uDestNodeGuid = uGuid;
	UpdateMessageOverlay();
}

void Tools_Message_Editing::ResetGS2PathfindToNodeGuid()
{
	s_pxGS2PathfindToNodeMessage->m_uDestNodeGuid = ASURA_GUID_UNREGISTERED;
	UpdateMessageOverlay();
}
#endif // #ifdef ASURA_USE_AXON

void Tools_Message_Editing::AddOverlayControls_SplitterOutput(Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	Asura_Assert(pxMessage->m_pData,"");
	Asura_Message_SplitterBlock_Output* pxData = (Asura_Message_SplitterBlock_Output*) pxMessage->m_pData;

	//see if we can find the splitter entity for finding the number of outputs
	int iMax = 100;
	Asura_Entity_Server* pxEntity = Asura_Entity_Server::FindEntity(pxMessage->m_uSendTo);
	if(pxEntity && pxEntity->GetClassification()==AsuraEntityClass_SplitterBlock)
	{
		iMax = ((Asura_ServerEntity_SplitterBlock*)pxEntity)->GetNumberOfOutputs();
		if(iMax) iMax--;
	}

	Asura_Dialog* pxDialog = 0;
	pxDialog = new Asura_Numerical_Dialog("Index",xOffset,(int*)&pxData->m_uOutputIndex,0,iMax,1,"%d", &s_bOnSplitterOutputIndexChanged);
	pxDialog->SetHelpText("Set the output to be activated/deactivated by this message.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	Asura_String xMessageDesc;
	GetMessageDescription(pxMessage->m_usMsgID, pxMessage, xMessageDesc);			

	s_pxSplitterBlockMessage = pxMessage;

	s_pxSplitterOutputDescDialog = new Asura_StringDisplay_Dialog(xMessageDesc.GetString(), xOffset);
	pxOverlay->AddDialog(s_pxSplitterOutputDescDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}


//////////////////////////////////////////////////////////////////////////////
//
// FUNCTION     Tools_Message_Editing::AddOverlayControls_ChangeAnim().
// PARAMETERS   pxMessage - message being edited.
//              pxOverlay - Asura_Overlay item to add to
//              xOffset   - 2d offset for overlay item
// RETURNS      None.
// DESCRIPTION  sets up function pointer and ptr to message (via a 32-bit int)
// AUTHOR       Mike Healey - 17.10.2002.
//
//////////////////////////////////////////////////////////////////////////////

static bool IsEnabled_ChangeAnim_Looping(int iFlagPtr)
{
	int iFlags = *((int*)iFlagPtr);
	return !(iFlags & Asura_PhysicalObject_AnimationFlag_UseEndTime)!=0;
}

static bool IsEnabled_ChangeAnim_EndTime(int iFlagPtr)
{
	int iFlags = *((int*)iFlagPtr);
	return (iFlags & Asura_PhysicalObject_AnimationFlag_UseEndTime)!=0;
}

static bool IsEnabled_ChangeAnim_StartTime(int iFlagPtr)
{
	int iFlags = *((int*)iFlagPtr);
	return (iFlags & Asura_PhysicalObject_AnimationFlag_UseStartTime)!=0;
}
                                      
static bool IsEnabled_ChangeAnim_TweenDuration(int iFlagPtr)
{
	int iFlags = *((int*)iFlagPtr);
	return (iFlags & (AsuraAnimationFlag_Tween|AsuraAnimationFlag_AnimatedTween|AsuraAnimationFlag_AnimatedTween_SyncTimers))!=0;
}

void Tools_Message_Editing::AddOverlayControls_ChangeAnim(struct Asura_StaticMessage* pxMessage,class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
    Asura_Message_PhysicalObject_ChangeAnimation* pxData = reinterpret_cast
        <Asura_Message_PhysicalObject_ChangeAnimation*>(pxMessage->m_pData);

    Asura_Dialog* pxDialog;

	//determine the text for the 'select anim' control
	char szText[200];
	if(pxData->m_uAnimFileID == ASURA_HASH_ID_UNSET)
	{
		strcpy(szText,"Anim : <Not Set>");
	}
	else
	{
		Asura_String xAnimFilename = Asura_HashedFilename_System::GetFilename(pxData->m_uAnimFileID,false);
		if(xAnimFilename.IsNull())
		{
			strcpy(szText,"Anim : <Unknown>");
		}
		else
		{
			Asura_String xAnimName;
			xAnimFilename.SplitFilename(0,&xAnimName,0);
			Asura_PrintfToBfr(szText,"Anim : %s",200,true,xAnimName.GetString());
		}
	}

    int iFunctionData = reinterpret_cast<int>(pxMessage);
    pxDialog = new Asura_Function_Dialog(szText, xOffset, ChooseAnimation_Dialog, iFunctionData);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	//start time
	const char* szStartTimeHelpText = "If enabled then the animation will start from the specified time.\n"
										"The times are scaled so that 0 is the start of the animation and 1 is the end.";
	
	pxDialog = new Asura_FlagTickbox_Dialog("", xOffset, &pxData->m_iFlags, Asura_PhysicalObject_AnimationFlag_UseStartTime);
	pxDialog->SetHelpText(szStartTimeHelpText);
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x;
    pxDialog = new Asura_Numerical_Dialog("Start Time", xOffset, &pxData->m_fStartTime, 0.0f, 100.0f, 0.1f, "%.3f");
	pxDialog->SetIsEnabledFunction(IsEnabled_ChangeAnim_StartTime,(int)&pxData->m_iFlags);
	pxDialog->SetHelpText(szStartTimeHelpText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = 1.0f;
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//end time
	const char* szEndTimeHelpText = "If enabled then the animation will play until it reaches the specified time\n."
									"If this animation is already playing , and no start time is specified , then the animation will continue from the current timer.\n"
									"The times are scaled so that 0 is the start of the animation and 1 is the end.";


	pxDialog = new Asura_FlagTickbox_Dialog("", xOffset, &pxData->m_iFlags, Asura_PhysicalObject_AnimationFlag_UseEndTime);
	pxDialog->SetHelpText(szEndTimeHelpText);
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x;
    pxDialog = new Asura_Numerical_Dialog("End Time", xOffset, &pxData->m_fEndTime, 0.0f, 100.0f, 0.1f, "%.3f");
	pxDialog->SetIsEnabledFunction(IsEnabled_ChangeAnim_EndTime,(int)&pxData->m_iFlags);
	pxDialog->SetHelpText(szEndTimeHelpText);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = 1.0f;
	xOffset.y+=Asura_Dialog::LargeSpacing();

    pxDialog = new Asura_FlagTickbox_Dialog("Tween", xOffset, &pxData->m_iFlags, AsuraAnimationFlag_Tween);
	pxDialog->SetHelpText("Previous animation is stopped , and the new animation is tweened in over the duration");
	pxDialog->SetRadioButtonGroup(1,false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

    pxDialog = new Asura_FlagTickbox_Dialog("Animated Tween", xOffset, &pxData->m_iFlags, AsuraAnimationFlag_AnimatedTween);
	pxDialog->SetHelpText("The previous animation continues to play as the new animation is tweened in. The old animation is removed after the tween is over.");
	pxDialog->SetRadioButtonGroup(1,false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

    pxDialog = new Asura_FlagTickbox_Dialog("Animated Tween (Sync Timer)", xOffset, &pxData->m_iFlags, AsuraAnimationFlag_AnimatedTween_SyncTimers);
	pxDialog->SetHelpText("As above , but the timer on the previous animation is kept in sync with the timer on this animation.");
	pxDialog->SetRadioButtonGroup(1,false);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

    pxDialog = new Asura_Numerical_Dialog("Tween Duration", xOffset, &pxData->m_fTweenDuration, 0.0f, 100.0f, 0.1f, "%.3f");
	pxDialog->SetIsEnabledFunction(IsEnabled_ChangeAnim_TweenDuration,(int)&pxData->m_iFlags);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();


    pxDialog = new Asura_FlagTickbox_Dialog("Looping", xOffset, &pxData->m_iFlags, AsuraAnimationFlag_Loop);
	pxDialog->SetIsEnabledFunction(IsEnabled_ChangeAnim_Looping,(int)&pxData->m_iFlags);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

    pxDialog = new Asura_FlagTickbox_Dialog("Reversed", xOffset, &pxData->m_iFlags, AsuraAnimationFlag_Reversed);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Mirror", xOffset, &pxData->m_iFlags, AsuraAnimationFlag_Mirror);
	pxDialog->SetHelpText("Mirrors the animation. This requires project-specific implementation to work correctly.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


}
// End function Tools_Message_Editing::AddOverlayControls_ChangeAnim().

void Tools_Message_Editing::AddOverlayControls_GlassSmash(Asura_StaticMessage *pxMessage, Asura_Overlay *pxOverlay, Asura_Vector_2& xOffset)
{
	Asura_Message_Glass_Smash *pxData = reinterpret_cast<Asura_Message_Glass_Smash*>(pxMessage->m_pData);
    Asura_Dialog* pxDialog;
	pxDialog = new Asura_Numerical_Dialog("Smash power", xOffset, &pxData->m_fPower, 0.0f, 1.0f, 0.01f, "%.3f");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_FlagTickbox_Dialog("Smash in +ve z-dir", xOffset, &pxData->m_uFlags,0x00000001);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
}

void Tools_Message_Editing::AddOverlayControls_LiftCall(struct Asura_StaticMessage* pxMessage, class Asura_Overlay* pxOverlay, class Asura_Vector_2& xOffset)
{
	Asura_MsgData_LiftCall *pxData = reinterpret_cast<Asura_MsgData_LiftCall*>(pxMessage->m_pData);

    Asura_Dialog* pxDialog;
	pxDialog = new Asura_Numerical_Dialog("Stop number", xOffset, (int*)&pxData->uStopNumber,0,999,1,"%d");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
}

void Tools_Message_Editing::AddOverlayControls_CutsceneTime(struct Asura_StaticMessage* pxMessage, class Asura_Overlay* pxOverlay, class Asura_Vector_2& xOffset)
{
	Asura_Message_ActivateCutsceneWithTime *pxData = reinterpret_cast<Asura_Message_ActivateCutsceneWithTime*>(pxMessage->m_pData);

    Asura_Dialog* pxDialog;
	pxDialog = new Asura_Numerical_Dialog("Time to start", xOffset, &pxData->m_fStartTime,0.0f,999.9f,0.1f,"%.2f");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
}

void Tools_Message_Editing::ChooseAnimation_Dialog(int iData)
{
    Asura_StaticMessage* pxMessage = reinterpret_cast<Asura_StaticMessage*>(iData);
    Asura_Guid uGuid = Tools_Message_Editing::GetFinalDestinationGuid(pxMessage);
    const Asura_ServerEntity_PhysicalObject* pxEntity;
    pxEntity = Asura_ServerEntity_PhysicalObject::FindEntity(uGuid);

    bool bOK = true;
	const Asura_Hierarchy_Skin* pxSkin = 0;

    if (pxEntity)
    {
        const Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkinByHash(pxEntity->GetSkinID());
		if(!pxSkin) bOK = false;
	}

	if(bOK)
	{
		//const Asura_Hierarchy_Anim* pxAnim = NULL;
		Asura_String xAnimName;
		Asura_String xAnimFilename;
		const char* szSkinName = pxSkin ? pxSkin->Name.GetString() : 0;
		if (Tools_Animation_Files::ChooseAnimation(szSkinName,0,&xAnimName,&xAnimFilename))
		{
			Asura_Message_PhysicalObject_ChangeAnimation* pxData = reinterpret_cast
	            <Asura_Message_PhysicalObject_ChangeAnimation*>(pxMessage->m_pData);
	        pxData->m_uAnimID = Asura_GetHashID(xAnimName.GetString());
	        pxData->m_uAnimFileID = Asura_GetHashID(xAnimFilename.GetString());
		}
	}

    if (!bOK)
    {
        Tools_Timed_Message_System::AddMessage(0xff0000,"Target not valid for this message.");
    }

	UpdateMessageOverlay();
}

void Tools_Message_Editing::AddOverlayControls_SetDestructible(Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	bool* pbDestructible = reinterpret_cast< bool* >( pxMessage->m_pData );
    Asura_Dialog* pxNewDialogue = new Asura_Tickbox_Dialog( "Set Destructible", xOffset, pbDestructible );
    pxNewDialogue->SetHelpText( "Whether the object should be able to take damage or not." );
    pxOverlay->AddDialog( pxNewDialogue );
    xOffset.y += Asura_Dialog::DefaultSpacing();
}

#ifndef ASURA_USE_AUDIO_SYSTEM
void Tools_Message_Editing::AddOverlayControls_SetReverb(struct Asura_StaticMessage* pxMessage, class Asura_Overlay* pxOverlay, class Asura_Vector_2& xOffset)
{
	Asura_Message_SoundReverb* pxReverb = reinterpret_cast<Asura_Message_SoundReverb*>( pxMessage->m_pData );

	Asura_Dialog* pxDialog = new Asura_DropDown_List_Dialog( xOffset, Asura_Sound_System::s_aszReverbNames, &pxReverb->m_iReverbID,
		10, Asura_Sound_System::ASURA_REVERB_ID_MAX, NULL, NULL, "Reverb ID" );
		
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::DefaultSpacing();
}
#endif

void Tools_Message_Editing::GetMarkerNameStringAndHash(int idata)
{
	Asura_StaticMessage* pxMessage = (Asura_StaticMessage*)(idata);
	Asura_Msg_PlayPFXOnEntity* pxData = (Asura_Msg_PlayPFXOnEntity*)(pxMessage->m_pData);

	if (pxData)
	{
		GetString_Overlay xOverlay("Enter new name for input", (pxData->m_xMarkerString)?pxData->m_xMarkerString:"Empty", ASURA_MSG_PLAY_PFX_MAX_CHARS);
		if (xOverlay.DoModal())
		{
			Asura_Char* pxTempChars = xOverlay.GetString();

			u_int uStringSize = strlen(pxTempChars);

			for ( u_int u = 0; u < uStringSize ; ++u )
			{
				pxData->m_xMarkerString[u] = pxTempChars[u];
			}
			pxData->m_xMarkerString[ uStringSize ] = 0;

			pxData->m_uMarkerHash = Asura_GetHashID( pxData->m_xMarkerString );
		}
	}
}

void Tools_Message_Editing::AddOverlayControls_PlayPFXOnEntity(Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	Asura_Msg_PlayPFXOnEntity* pxData = static_cast<Asura_Msg_PlayPFXOnEntity*>(pxMessage->m_pData);
	
	Asura_Dialog* pxDialog = 0;

	pxDialog = new Tools_PFXSelect_Dialog(Tools_PFXSelect_Dialog::MODE_EFFECTS, xOffset, &pxData->m_uPFXEffectHash);
	pxDialog->SetHelpText("Select PFX to play.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Marker point hash for PFX.
	Asura_ServerEntity_PhysicalObject* pxObject = Asura_ServerEntity_PhysicalObject::FindEntity(pxMessage->m_uSendTo);
	if(pxObject && pxObject->GetSkinID() != ASURA_HASH_ID_UNSET)
	{
		pxDialog = CreateSkinMarkerPointDialog_PlayPFXOnEntity(xOffset, pxMessage->m_uSendTo, &pxData->m_uMarkerHash, "Marker point for PFX");
		pxDialog->SetHelpText("Select marker point to play the PFX on.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}
	else
	{
		pxDialog = new Asura_Dialog("No skin defined, cannot get list of markers, enter marker name instead.", xOffset);
		pxOverlay->AddDialog(pxDialog);
		xOffset.y += Asura_Dialog::DefaultSpacing();
	}
	
	pxDialog = new Asura_Function_Dialog("Marker Name :", xOffset, &GetMarkerNameStringAndHash, (int)(pxMessage), &m_bRequestRefreshMessageOverlay);
	pxDialog->SetHelpText("Name for marker point to attach to. Will override any other form of selection.");
	pxOverlay->AddDialog(pxDialog);	
	xOffset.y += Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_Dialog((pxData->m_xMarkerString[0])?pxData->m_xMarkerString:"*Not Set*", xOffset);
	pxOverlay->AddDialog(pxDialog);	
	xOffset.y += Asura_Dialog::DefaultSpacing();

	// Flags
	pxDialog = new Asura_FlagTickbox_Dialog("Attach PFX", xOffset, &pxData->m_uPlayPFXFlags, static_cast<int>(uASURA_PLAYPFXONENTITY_FLAG_ATTACH));
	pxDialog->SetHelpText("Attach the PFX to the entity, so that it moves with it, and is destroyed with it.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

// Projects will need to override this function if they wish the "Play PFX on Entity" message to play
// PFX on marker points on attachments. The project dialog needs to add any extra markers to the list.
Asura_Dialog* Tools_Message_Editing::CreateSkinMarkerPointDialog_PlayPFXOnEntity(Asura_Vector_2& xOffset, Asura_Guid uEntity, Asura_Hash_ID* puMarkerHash, const char* szName)
{
	return new Tools_SkinMarkerPoint_HashDropDownList_Dialog(xOffset, uEntity, puMarkerHash, szName);
}

#ifdef ASURA_USE_CONTAINER_SYSTEM
void Tools_Message_Editing::AddOverlayControls_Container_ChangeState(Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	Asura_Message_Container_ChangeState* pxData = static_cast<Asura_Message_Container_ChangeState*>(pxMessage->m_pData);
	
	Asura_Dialog* pxDialog = 0;

	// State hash for container.
	pxDialog = new Tools_ContainerState_MessageEditing_Dialog("New state for container", xOffset, pxMessage->m_uSendTo, &pxData->m_uStateHash);
	pxDialog->SetHelpText("Select new state for the container for input slot 0.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}

void Tools_Message_Editing::AddOverlayControls_Container_ChangeStateExtended(Asura_StaticMessage* pxMessage, Asura_Overlay* pxOverlay, Asura_Vector_2& xOffset)
{
	Asura_Message_Container_ChangeStateExtended* pxData = static_cast<Asura_Message_Container_ChangeStateExtended*>(pxMessage->m_pData);
	
	Asura_Dialog* pxDialog = 0;

	// State hash for container.
	pxDialog = new Tools_ContainerState_MessageEditing_Dialog("New state for container", xOffset, pxMessage->m_uSendTo, &pxData->m_uStateHash, &pxData->m_uInputIndex);
	pxDialog->SetHelpText("Select new state for the container for the chosen input slot.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();
	
	const u_int uMax = uASURA_MAX_NUM_CONTAINER_INPUTS-1;
	pxDialog = new Asura_Numerical_Dialog("Input slot", xOffset, &pxData->m_uInputIndex, 0, uMax, 1, "%u");
	pxDialog->SetHelpText("Select the input slot that the message will change.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
}
#endif //ASURA_USE_CONTAINER_SYSTEM

void Tools_Message_Editing::AddFilesToExportList(List<Asura_String>& xNameList)
{
    for(int iSet=0;iSet<ASURA_STATICMESSAGE_NUMSETS;iSet++)
	{
		Asura_StaticMessageSetIndex eSet = (Asura_StaticMessageSetIndex)iSet;

    	int iTotalBlocks = Asura_StaticMessage_System::GetNumberOfMessageBlocks(eSet);
    	for (int iBlockNumber = 0; iBlockNumber < iTotalBlocks; iBlockNumber++)
		{
    	    int iMessagesInBlock = Asura_StaticMessage_System::GetNumberOfMessagesInBlock(eSet,iBlockNumber);
    	    for (int iMessageIndex = 0; iMessageIndex < iMessagesInBlock; iMessageIndex++)
			{
    			const Asura_StaticMessage* pxStaticMessage = Asura_StaticMessage_System::GetMessagePointer(eSet,iBlockNumber, iMessageIndex);
				if(pxStaticMessage->IsDisabled()) continue;

				const Tools_Entity_Object* pxToolsTarget = Tools_Entity_Object::FindEntity(pxStaticMessage->m_uSendTo);

				/// Richard May - 24/02/2003 16:45:51 - Only assert if we can't find the target if the target's guid
				/// is actually a real tools type entity and not a special player/client guid
				if( (pxStaticMessage->m_uSendTo < ASURA_GUID_CLIENT_START) && (pxStaticMessage->m_uSendTo > ASURA_GUID_CLIENT_END) )
				{
		            Asura_Assert(pxToolsTarget, "failed to find target of message");
				}

    	        if (pxToolsTarget)
    	        {
    	            // if we have some data, check the data size hasn't changed
    	            if (pxStaticMessage->m_usMsgID == ASURA_MSG_CHANGE_ANIMATION)
    	            {
    	                Asura_Assert(pxStaticMessage->m_usDataSize == sizeof(Asura_Message_PhysicalObject_ChangeAnimation), "MH: serious error. data size has changed");
    	            }

					if( pxToolsTarget->ShouldAddFilesToExportList() )
					{
    					pxToolsTarget->AddFilesToExportListFromMessage(pxStaticMessage->m_usMsgID, pxStaticMessage->m_pData, xNameList);
					}
    	        }

				if ( ( pxStaticMessage->m_uSendTo < ASURA_GUID_TOOLSCREATEDENTITY_START && ASURA_GUID_TOOLSCREATEDENTITY_END < pxStaticMessage->m_uSendTo ) ||
					( pxToolsTarget && pxToolsTarget->ShouldAddFilesToExportList() ) )
				{
					_AddFilesToExportListFromMessage( pxStaticMessage, xNameList );
				}
			}
		}
	}
}

void Tools_Message_Editing::AddFilesToExportListFromMessage( const Asura_StaticMessage* pxStaticMessage, List<Asura_String>& xNameList ) const
{
	switch ( pxStaticMessage->m_usMsgID )
	{
		default:
			break;

		#ifdef ASURA_USE_AXON
		case ASURA_MSG_AXON_GS2_ANIM_CHANGE:
		{
			// Add the animation files from the anim template to the export list.
			const Axon_Msg_GS2_Anim_Change* pxAnimChangeMsg = static_cast< const Axon_Msg_GS2_Anim_Change* >( pxStaticMessage->m_pData );
			const Axon_Gamescene_AnimTemplate* pxTemplate = Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( pxAnimChangeMsg->m_uAnimSet );
			if ( pxTemplate )
			{
				pxTemplate->AddFilesToExportList( xNameList );
			}
			break;
		}

		case ASURA_MSG_AXON_GS2_SET_NODE_PARAMS:
		{
			const Axon_Msg_GS2_Set_Node_Params* pxSNPMsg = static_cast< const Axon_Msg_GS2_Set_Node_Params* >( pxStaticMessage->m_pData );
			if ( pxSNPMsg->m_iDataValidFlags & Axon_Msg_GS2_Set_Node_Params::ANIM_SET )
			{
				// Add the animation files from the anim template to the export list.
				const Axon_Gamescene_AnimTemplate* pxTemplate = Axon_Gamescene_AnimTemplate_System::GetAnimTemplateByID( pxSNPMsg->m_uAnimSet );
				if ( pxTemplate )
				{
					pxTemplate->AddFilesToExportList( xNameList );
				}
			}
			break;
		}
		#endif

		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		case ASURA_MSG_DIALOGUE_PLAYLINE:
		case ASURA_MSG_DIALOGUE_PLAYLINEONENTITY:
			Tools_Dialogue_System::AddFilesToExportListFromMessage( pxStaticMessage );
			break;
		#endif // ASURA_USE_DIALOGUE_SYSTEM
	}
}

void Tools_Message_Editing::SetModified()
{
	//set entity or cutscene file as modified according to message set
	m_xMessageBlock.SetModified();

	//flag the entity that owns the message block as modified
	Tools_Object* pxEditedObject = Tools_Object_System::GetMessageEditObject();
	if(pxEditedObject && pxEditedObject->IsEntityObject())
	{
		Tools_Entity_Object* pxToolsEntityObject = static_cast<Tools_Entity_Object*>(pxEditedObject);
		pxToolsEntityObject->Changed = true;

		// m_bEntityModified is obsolete, this flag is never tested for
		Asura_Entity* pxEntity = pxToolsEntityObject->Entity;
		if(pxEntity && pxEntity->ToolsData)
		{
			pxEntity->ToolsData->m_bEntityModified = true;
		}
	}
}


const EventIDAndDescription* Tools_Message_Editing::GetEventList()
{
	if(!s_pxThis) return 0;

	if(!s_pxThis->m_pxEventList)
	{
		s_pxThis->BuildEventList();
	}
	return s_pxThis->m_pxEventList;
}


const char* Tools_Message_Editing::GetEventDescriptionMain(int iEventID)
{
	// Special handling for 'On X% Health' events
	if( (ASURA_ENTITY_EVENT_ONHEALTHPERCENT_0 <= iEventID) && (iEventID <= ASURA_ENTITY_EVENT_ONHEALTHPERCENT_99) )
	{
		static char szDescription[32];
		
		Asura_PrintfToBfr( szDescription, "On %i%% Health", sizeof(szDescription), true, iEventID - ASURA_ENTITY_EVENT_ONHEALTHPERCENT_0 );
		return szDescription;
	}
	else
	{
		const EventIDAndDescription* pxEventList = GetEventList();
		if( pxEventList )
		{		
			for(const EventIDAndDescription* pxEvent = pxEventList;pxEvent->m_pszDescription;pxEvent++)
			{
				if(pxEvent->m_iEventID == iEventID)
				{
					return pxEvent->m_pszDescription;
				}
			}
		}
	}

	return "*Unknown Event*";
}

const char* Tools_Message_Editing::GetEventHelpTextMain(int iEventID)
{
	
	const EventIDAndDescription* pxEventList = GetEventList();
	if( pxEventList )
	{		
		for(const EventIDAndDescription* pxEvent = pxEventList;pxEvent->m_pszDescription;pxEvent++)
		{
			if(pxEvent->m_iEventID == iEventID)
			{
				return pxEvent->m_szHelpText;
			}
		}
	}

	return "";
}



void Tools_Message_Editing::BuildEventList()
{
	//count the events
	int iNumEvents = 0;

	for(int iProject=0;iProject<2;iProject++)
	{
		const EventIDAndDescription* pxEventList = iProject ? g_axEventList : GetProjectEventList();
		if(!pxEventList) continue;

		for(const EventIDAndDescription* pxEvent = pxEventList;pxEvent->m_pszDescription;pxEvent++)
		{
			iNumEvents++;
		}
	}

	//allocate storage for combined asura and project event list
	m_pxEventList = new EventIDAndDescription[iNumEvents+1];

	//and copy into this array
	int iEvent = 0;
	for(int iProject=0;iProject<2;iProject++)
	{
		const EventIDAndDescription* pxEventList = iProject ? GetProjectEventList() : g_axEventList;
		if(!pxEventList) continue;

		for(const EventIDAndDescription* pxEvent = pxEventList;pxEvent->m_pszDescription;pxEvent++)
		{
			//see if this is the same as a previous event
			int iMatchingEvent = -1;
			for(int i=0;i<iEvent;i++)
			{
				if(pxEvent->m_iEventID == m_pxEventList[i].m_iEventID)
				{
					iMatchingEvent = i;
					break;
				}
			}
			if(iMatchingEvent >=0)
			{
				//Replace the existing event , allowing the description/help to be replaced
				m_pxEventList[iMatchingEvent] = *pxEvent;
				iNumEvents--;
			}
			else
			{			
				m_pxEventList[iEvent++] = *pxEvent;
			}
		}
	}

	//add terminator
	m_pxEventList[iNumEvents].m_iEventID = 0;
	m_pxEventList[iNumEvents].m_pszDescription = 0;
	m_pxEventList[iNumEvents].m_szHelpText = 0;

}


void Tools_Message_Editing::UpdateMessageOverlay()
{
	Asura_Assert(s_pxThis,"");

	if(s_pxTestMessageOverlay)
	{
		s_pxTestMessageOverlay->RebuildOverlay();
	}
	else
	{
		//reselect the message , to update the dialog
		s_pxThis->SelectMessage(s_pxThis->m_iCurrentMessageIndex);
	}

}

bool Tools_Message_Editing::CanCopyMessages()
{
	if(!m_xMessageBlock.IsValid()) return false;
	return m_xMessageBlock.GetNumberOfMessagesInBlock() != 0;
}

void Tools_Message_Editing::CopyMessages()
{
	Asura_Assert(m_xMessageBlock.IsValid(),"");

	int iNumMessages = m_xMessageBlock.GetNumberOfMessagesInBlock();
	if(!iNumMessages)
	{
		//nothing to copy
        Tools_Timed_Message_System::AddMessage(0xffff00,"Can't copy - No messages in message block.");
		return;
	}
	
	//ask for messages to copy
	Tools_MessageCopy_Overlay xOverlay(&m_xMessageBlock,m_iCurrentMessageIndex);
	if(!xOverlay.DoModal()) return;
	if(!xOverlay.GetNumSelected()) return;

	ClearPasteBuffer();

	m_iNumPasteBufferMessages = xOverlay.GetNumSelected();
	m_pxPasteBufferMessages = new Asura_StaticMessage[m_iNumPasteBufferMessages];

	int iIndex=0;
	for(int i=0;i<iNumMessages;i++)
	{
		if(!xOverlay.IsSelected(i)) continue;		
		
		Asura_StaticMessage* pxMessage = m_xMessageBlock.GetMessagePointer(i);
		m_pxPasteBufferMessages[iIndex++].Copy(pxMessage);
	}

	Tools_Timed_Message_System::AddMessage(0xffffff,"%d message(s) copied",m_iNumPasteBufferMessages);
}

bool Tools_Message_Editing::CanPasteMessages()
{
	if(!m_xMessageBlock.IsValid()) return false;
	return m_iNumPasteBufferMessages!=0;
}

void Tools_Message_Editing::PasteMessages()
{
	Asura_Assert(m_xMessageBlock.IsValid(),"");

	if(!m_iNumPasteBufferMessages)
	{
		//nothing to paste
        Tools_Timed_Message_System::AddMessage(0xffff00,"Can't paste - No messages in paste buffer.");
		return;
	}

	for(int i=0;i<m_iNumPasteBufferMessages;i++)
	{
		Asura_StaticMessage xMessage;
		xMessage.Copy(&m_pxPasteBufferMessages[i]);

		m_xMessageBlock.AddMessage(xMessage);
	}

	//Need to re-select the current message , since the message array will have been reallocated ,and so
	//the message editing overlay will be looking at the wrong memory. RAR 09/05/2006
	int iMessageIndex = m_iCurrentMessageIndex;
	if(iMessageIndex == -1) iMessageIndex = 0;
	SelectMessage(iMessageIndex);

	SetModified();
	
	Tools_Timed_Message_System::AddMessage(0xffffff,"%d message(s) pasted",m_iNumPasteBufferMessages);
}

void Tools_Message_Editing::ClearPasteBuffer()
{
	for(int i=0;i<m_iNumPasteBufferMessages;i++)
	{
		m_pxPasteBufferMessages[i].DeallocateMessageData();
	}
	delete [] m_pxPasteBufferMessages;
	m_pxPasteBufferMessages = 0;
	m_iNumPasteBufferMessages = 0;
}

bool Tools_Message_Editing::IsEditedMessage(u_short usBlockID,int iIndex)
{
	if(!s_pxThis) return false;
	if(!s_pxThis->m_xMessageBlock.IsEditedMessageBlock(usBlockID)) return false;
	if(s_pxThis->m_iCurrentMessageIndex != iIndex) return false;

	return true;

}


Asura_Guid Tools_Message_Editing::GetFinalDestinationGuid(const Asura_StaticMessage* pxMessage)
{
	
	//Is the target a proxy trigger
	Asura_ServerEntity_ProxyTrigger* pxProxy = Asura_ServerEntity_ProxyTrigger::FindEntity(pxMessage->m_uSendTo);
	if(pxProxy)
	{
		//return a targeted guid
		return pxProxy->GetGuidOfTargetClass();
	}
	else
	{
		//Not a proxy ,so return the guid unaltered.
		return pxMessage->m_uSendTo;
	}
}

bool Tools_Message_Editing::CheckOkayToDeleteIncomingMessages(const List<Tools_Object*>& xObjectList)
{
	//get a list of the guids of the objects being destroyed
	List<Asura_Guid> xGuidList;
	for(CLIF<Tools_Object*> xObjectLIF(&xObjectList); !xObjectLIF.done(); xObjectLIF.next())
	{
		Asura_Guid uGuid = xObjectLIF()->GetGuid();
		if(uGuid != ASURA_GUID_UNREGISTERED)
		{
			xGuidList.add_entry(uGuid);
		}
	}

	//check for any messages that target these , and do a count on the ones that can't be saved
	u_int uUnsaveable = 0;

	for(Tools_StaticMessageIterator xIt(true); !xIt.Done(); xIt.Next())
	{
		Asura_Guid uTarget = xIt.GetCurrent()->m_uSendTo;
		if(xGuidList.contains(uTarget))
		{
			if(!xIt.CanSaveFile())
			{
				uUnsaveable++;
			}
		}
	}	

	if(!uUnsaveable) return true;
	
	return ShowYesNoOverlay("The objects being deleted have %d incoming message(s) in unwriteable files.\nDelete them anyway?",uUnsaveable);
}

u_int Tools_Message_Editing::DeleteIncomingMessages(Asura_Guid uTargetGuid)
{
	if(uTargetGuid == ASURA_GUID_UNREGISTERED) return 0;

	u_int uDeleted = 0;
	for(Tools_StaticMessageIterator xIt(true); !xIt.Done(); xIt.Next())
	{
		Asura_Guid uTarget = xIt.GetCurrent()->m_uSendTo;
		if(uTarget == uTargetGuid)
		{
			if(xIt.DeleteCurrent())
			{
				uDeleted++;
			}
		}
	}	
	return uDeleted;
}

////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------**
** Tools_MessageEdit_Overlay **
**---------------------------*/

bool Tools_MessageEdit_Overlay::s_bReturnValue;


Tools_MessageEdit_Overlay::Tools_MessageEdit_Overlay(Asura_StaticMessage* pxMessage)
:Tools_Modal_Overlay(" ")
{
	m_pxMessage = pxMessage;
}

bool Tools_MessageEdit_Overlay::DoModal()
{
	ShowOverlay();
	return s_bReturnValue;
}

void Tools_MessageEdit_Overlay::InitialiseOverlay(class Asura_Overlay* pxOverlay)
{
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	//set the overlay title
	Tools_Message_Editing::GetMessageDescription(m_pxMessage->m_usMsgID,0,pxOverlay->m_xName);

	//add message editing overlay
    const Asura_Vector_2 xOldOffset = xOffset;
	Tools_Message_Editing::s_pxThis->AddOverlayControls(m_pxMessage,pxOverlay,xOffset);

    // If the offset hasn't changed then assume nothing's been added. In that
    // case, add our own bit of text to highlight that nothing can be edited
    // for this message.
    if( xOffset == xOldOffset )
    {
		pxDialog = new Asura_Dialog( "<nothing to edit>", xOffset );
		pxOverlay->AddDialog( pxDialog );
    }

	xOffset.y += Asura_Dialog::LargeSpacing();
	
	//add okay/cancel

	pxDialog = new Asura_Function_Dialog("Okay",xOffset,YesSelected,&pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x+=6.0f;

	pxDialog = new Asura_Function_Dialog("Cancel",xOffset,NoSelected,&pxOverlay->m_bShouldClose);
	pxOverlay->AddDialog(pxDialog);
}



////////////////////////////////////////////////////////////////////////////////////////////
/*---------------------------**
** Tools_MessageCopy_Overlay **
**---------------------------*/

Tools_MessageCopy_Overlay::Tools_MessageCopy_Overlay(Tools_StaticMessageBlock* pxMessageBlock,int iSelectedMessage)
:Tools_Modal_Overlay("Copy Which Messages?")
{
	m_pxMessageBlock = pxMessageBlock;
	m_iNumMessages = pxMessageBlock->GetNumberOfMessagesInBlock();	
	Asura_Assert(m_iNumMessages,"");

	m_pbToBeCopied = new bool[m_iNumMessages];
	m_ppszMessageStrings = new char*[m_iNumMessages];
	

	for(int i=0;i<m_iNumMessages;i++)
	{
		
		//get string to display
		Asura_StaticMessage* pxMessage = m_pxMessageBlock->GetMessagePointer(i);

		char szMessageDesc[50];
		Asura_String xMessageDesc;
		Tools_Message_Editing::GetMessageDescription(pxMessage->m_usMsgID,pxMessage,xMessageDesc);

		Asura_PrintfToBfr(szMessageDesc,"%s - %s",50,true,xMessageDesc.GetString(),Tools_Message_Editing::GetNameFromGuid(pxMessage->m_uSendTo));
		
		m_ppszMessageStrings[i] = new char[strlen(szMessageDesc)+1];
		strcpy(m_ppszMessageStrings[i],szMessageDesc);

		//default to just copying the selected message
		m_pbToBeCopied[i] = (i==iSelectedMessage);
	}
	m_iSelectionIndex = iSelectedMessage;

}

Tools_MessageCopy_Overlay::~Tools_MessageCopy_Overlay()
{
	delete [] m_pbToBeCopied;

	for(int i=0;i<m_iNumMessages;i++)
	{
		delete [] m_ppszMessageStrings[i];
	}
	delete [] m_ppszMessageStrings;
}

bool Tools_MessageCopy_Overlay::DoModal()
{
	m_bReturnValue = false;

	ShowOverlay();
	return m_bReturnValue;
}


void Tools_MessageCopy_Overlay::InitialiseOverlay(class Asura_Overlay* pxOverlay)
{
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	int iNumLines = min(m_iNumMessages,10);

	pxDialog = new Asura_List_Dialog(xOffset,m_ppszMessageStrings,&m_iSelectionIndex,iNumLines,m_iNumMessages,_IsSelected,_ToggleSelected,(int)this);
	pxDialog->SetHelpText("Select the messages that should be copied.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y=pxDialog->m_xMaxCoord.y;
	xOffset.y+=1.0f;

	pxDialog = new Asura_Function_Dialog("Select All",xOffset,_SelectAll,(int)this);
	pxDialog->SetHelpText("Selects all the messages for copying.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Okay",xOffset,OkayPressed,(int)this,&pxOverlay->m_bShouldClose);
	pxDialog->SetHelpText("Copy current selection.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x+=7.0f;

	pxDialog = new Asura_Function_Dialog("Cancel",xOffset,CancelPressed,(int)this,&pxOverlay->m_bShouldClose);
	pxDialog->SetHelpText("Cancel copying.");
	pxOverlay->AddDialog(pxDialog);
	

}

void Tools_MessageCopy_Overlay::Update()
{
	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ENTER))
	{
		GetOverlay()->m_bShouldClose = true;			
		m_bReturnValue = true;
		return;
	}
	if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE))
	{
		GetOverlay()->m_bShouldClose = true;
		m_bReturnValue = false;
		return;
	}
}

bool Tools_MessageCopy_Overlay::IsSelected(int iIndex)
{
	return m_pbToBeCopied[iIndex];
}

void Tools_MessageCopy_Overlay::ToggleSelected(int iIndex)
{
	m_pbToBeCopied[iIndex] = !m_pbToBeCopied[iIndex];
}

void Tools_MessageCopy_Overlay::SelectAll()
{
	for(int i=0;i<m_iNumMessages;i++)
	{
		m_pbToBeCopied[i] = true;
	}
}

int Tools_MessageCopy_Overlay::GetNumSelected()
{
	int iCount = 0;
	for(int i=0;i<m_iNumMessages;i++)
	{
		if(m_pbToBeCopied[i]) iCount++;
	}
	return iCount;
}


//////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------**
** Tools_StaticMessageID **
**-----------------------*/

void Tools_StaticMessageID::SetMessage_StaticMessageSystem(u_short usBlockID, int iMessageIndex)
{
	m_iLocation = TOOLS_STATICMESSAGELOCATION_STATICMESSAGE;
	m_uStaticMessageBlockID = usBlockID;
	m_iMessageIndex = iMessageIndex;
}

void Tools_StaticMessageID::SetMessage_Dialogue(Asura_Hash_ID uDialogue, Asura_Hash_ID uBlockHash, int iMessageIndex)
{
	m_iLocation = TOOLS_STATICMESSAGELOCATION_DIALOGUE;
	m_uDialogueEvent = uDialogue;
	m_uBlockHash = uBlockHash;
	m_iMessageIndex = iMessageIndex;
}

void Tools_StaticMessageID::GetSenderInfo(Asura_Guid* puGuid, Asura_String* pxName, Asura_String* pxEventDesc) const
{
	if(puGuid) *puGuid = ASURA_GUID_UNREGISTERED;
	if(pxName) pxName->MakeNull();
	if(pxEventDesc) pxEventDesc->MakeNull();

	switch(m_iLocation)
	{
		case TOOLS_STATICMESSAGELOCATION_STATICMESSAGE :
		{
			Tools_Object* pxSender = Tools_Object_System::GetOwnerOfMessageBlock((u_short)m_uStaticMessageBlockID);
			if(pxSender)
			{
				if(puGuid) *puGuid = pxSender->GetGuid();
				if(pxName) *pxName = pxSender->GetName();
				if(pxEventDesc)  pxSender->GetEventDescForMessageBlock((u_short)m_uStaticMessageBlockID, *pxEventDesc);

			}
			break;
		}
		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		case TOOLS_STATICMESSAGELOCATION_DIALOGUE :
		{
			const Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByHash(m_uDialogueEvent);
			if(pxEvent)
			{
				if(pxName)
				{
					char szText[100];
					Asura_PrintfToBfr(szText, "DlgEv %s", sizeof(szText), true, pxEvent->GetName());
					*pxName = szText;
				}
				if(pxEventDesc)
				{
					const Asura_Dialogue_Message* pxDialogueMessage = pxEvent->GetMessageFromID(m_uBlockHash);
					if(pxDialogueMessage) *pxEventDesc = pxDialogueMessage->m_xName;
				}
			}
			break;
		}
		#endif
	}
}

Asura_StaticMessage* Tools_StaticMessageID::GetMessagePointer()
{
	switch(m_iLocation)
	{
		case TOOLS_STATICMESSAGELOCATION_STATICMESSAGE :
		{
			if(!Asura_StaticMessage_System::IsMessageBlockValid(m_uStaticMessageBlockID)) return 0;
			if(m_iMessageIndex<0 || m_iMessageIndex >= Asura_StaticMessage_System::GetNumberOfMessagesInBlock(m_uStaticMessageBlockID)) return 0;
			return Asura_StaticMessage_System::GetMessagePointer(m_uStaticMessageBlockID,m_iMessageIndex);
		}
		
		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		case TOOLS_STATICMESSAGELOCATION_DIALOGUE :
		{
			Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByHash(m_uDialogueEvent);
			if(!pxEvent) return 0;

			Asura_Dialogue_Message* pxDialogueMessage = pxEvent->GetMessageFromID(m_uBlockHash);
			if(!pxDialogueMessage) return 0;

			if(m_iMessageIndex<0 || m_iMessageIndex >= (int)pxDialogueMessage->m_xMessageBlock.GetNumberOfMessagesInBlock()) return 0;

 			return pxDialogueMessage->m_xMessageBlock.GetMessagePointer(m_iMessageIndex);
		}
		#endif

		default :
			return 0;
	}
}

u_short Tools_StaticMessageID::GetStaticMessageBlockID() const
{
	switch(m_iLocation)
	{
		case TOOLS_STATICMESSAGELOCATION_STATICMESSAGE :
			return (u_short)m_uStaticMessageBlockID;

		default :
			return ASURA_STATICMESSAGE_EMPTYBLOCK;
		
	}
}

Asura_StaticMessageBlock* Tools_StaticMessageID::GetStaticMessageBlockPointer() const
{
	switch(m_iLocation)
	{
		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		case TOOLS_STATICMESSAGELOCATION_DIALOGUE :
		{
			Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByHash(m_uDialogueEvent);
			if(!pxEvent) return 0;

			Asura_Dialogue_Message* pxDialogueMessage = pxEvent->GetMessageFromID(m_uBlockHash);
			if(!pxDialogueMessage) return 0;

 			return &pxDialogueMessage->m_xMessageBlock;
		}
		#endif

		default :
			return 0;
		
	}
	
}

void Tools_StaticMessageID::SetModified() const
{
	switch(m_iLocation)
	{
		case TOOLS_STATICMESSAGELOCATION_STATICMESSAGE :
		{
			int iBlockIndex;
			Asura_StaticMessageSetIndex eSet;
			if(Asura_StaticMessage_System::ConvertIDToIndex(m_uStaticMessageBlockID, &iBlockIndex, &eSet))
			{
				switch(eSet)
				{
					case ASURA_STATICMESSAGE_ENTITYSET :
					{
						Tools_Object* pxOwner = Tools_Object_System::GetOwnerOfMessageBlock((u_short)m_uStaticMessageBlockID);
						if(pxOwner && pxOwner->IsEntityObject())
						{
							((Tools_Entity_Object*)pxOwner)->SetEntityModified();
						}
						else
						{
							Tools_SaveAsura::SetEntityModified();
						}
						break;
					}
					case ASURA_STATICMESSAGE_CUTSCENESET :
						Tools_SaveAsura::SetCutscenesModified();
						break;

					default :
						Asura_Assert(0,"Unknown message set");
						break;
	
				}
			}
			break;
		}

		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		case TOOLS_STATICMESSAGELOCATION_DIALOGUE :
		{
			Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByHash(m_uDialogueEvent);
			if(pxEvent)
			{
				Tools_Dialogue_System::SetModified(pxEvent);
			}
			break;
		}
		#endif

		default :
			Asura_Assert(0,"Unhandled static message location");
			break;
	}
}

bool Tools_StaticMessageID::CanSaveFile() const
{
	switch(m_iLocation)
	{
		case TOOLS_STATICMESSAGELOCATION_STATICMESSAGE :
		{
			int iBlockIndex;
			Asura_StaticMessageSetIndex eSet;
			if(Asura_StaticMessage_System::ConvertIDToIndex(m_uStaticMessageBlockID, &iBlockIndex, &eSet))
			{
				switch(eSet)
				{
					case ASURA_STATICMESSAGE_ENTITYSET :
					{
						Tools_Object* pxOwner = Tools_Object_System::GetOwnerOfMessageBlock((u_short)m_uStaticMessageBlockID);
						if(pxOwner && pxOwner->IsEntityObject())
						{
							return ((Tools_Entity_Object*)pxOwner)->CanSaveFile();
						}
						else
						{
							return Tools_SaveAsura::CanSaveEntity();
						}
						break;
					}
					case ASURA_STATICMESSAGE_CUTSCENESET :
						return Tools_SaveAsura::CanSaveCutscenes();
						break;

					default :
						Asura_Assert(0,"Unknown message set");
						break;
	
				}
			}
			break;
		}

		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		case TOOLS_STATICMESSAGELOCATION_DIALOGUE :
		{
			Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByHash(m_uDialogueEvent);
			if(pxEvent)
			{
				return Tools_Dialogue_System::CanSaveFile(pxEvent);
			}
			break;
		}
		#endif

		default :
			Asura_Assert(0,"Unhandled static message location");
			break;
	}
	return false;
}



//////////////////////////////////////////////////////////////////////////////////////////////
/*-----------------------------**
** Tools_StaticMessageIterator **
**-----------------------------*/

Tools_StaticMessageIterator::Tools_StaticMessageIterator(bool bAllSystems)
{
	m_bAllSystems = bAllSystems;
	Restart();
}

Asura_StaticMessage* Tools_StaticMessageIterator::GetCurrent() const
{
	switch(m_iLocation)
	{
		case TOOLS_STATICMESSAGELOCATION_STATICMESSAGE :
			return Asura_StaticMessage_System::GetMessagePointer((Asura_StaticMessageSetIndex)m_iMessageSet,m_iBlockIndex,m_iMessageIndex);
		
		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		case TOOLS_STATICMESSAGELOCATION_DIALOGUE :
		{
			Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByIndex(m_iDialogueIndex);
			Asura_StaticMessageBlock& xBlock = pxEvent->GetMessageFromStore(m_iBlockIndex)->m_xMessageBlock;

			return xBlock.GetMessagePointer(m_iMessageIndex);
		}
		#endif
			
		
		default :
			Asura_Assert(0,"Unhandled static message location");
			return 0;
	}
}

u_short Tools_StaticMessageIterator::GetCurrentMessageBlockID() const
{
	switch(m_iLocation)
	{
		case TOOLS_STATICMESSAGELOCATION_STATICMESSAGE :
			return Asura_StaticMessage_System::ConvertIndexToID((Asura_StaticMessageSetIndex)m_iMessageSet,m_iBlockIndex);
			
		default :
			Asura_Assert(0,"GetCurrentMessageBlockID() only useable for messages within the static message system");
			return ASURA_STATICMESSAGE_EMPTYBLOCK;

	}
}

void Tools_StaticMessageIterator::GetCurrentMessageID(Tools_StaticMessageID& xOutputID) const
{
	xOutputID.Invalidate();

	switch(m_iLocation)
	{
		case TOOLS_STATICMESSAGELOCATION_STATICMESSAGE :
			xOutputID.SetMessage_StaticMessageSystem(GetCurrentMessageBlockID(), m_iMessageIndex);
			break;

		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		case TOOLS_STATICMESSAGELOCATION_DIALOGUE :
		{
			Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByIndex(m_iDialogueIndex);
			if(pxEvent && m_iBlockIndex < (int) pxEvent->GetMessageStoreSize())
			{
				Asura_Hash_ID uBlockHash = pxEvent->GetMessageFromStore(m_iBlockIndex)->m_uHashID;
				xOutputID.SetMessage_Dialogue(pxEvent->GetHashID(), uBlockHash, m_iMessageIndex);

			}
			break;
		}
		#endif
			
		default :
			Asura_Assert(0,"Unhandled static message location");
	}
}


void Tools_StaticMessageIterator::Next()
{
	while(m_iLocation < TOOLS_STATICMESSAGELOCATION_END)
	{
		
		bool bFoundMessage = false;
		switch(m_iLocation)
		{
			case TOOLS_STATICMESSAGELOCATION_STATICMESSAGE :
				bFoundMessage = NextMessage_StaticMessageSystem();
				break;
			
			case TOOLS_STATICMESSAGELOCATION_DIALOGUE :
				if(m_bAllSystems)
				{
					bFoundMessage = NextMessage_Dialogue();
				}
				break;

			default :
				Asura_Assert(0,"Unhandled static message location");
				break;
	
		}
		if(bFoundMessage) return;
		m_iLocation++;
	}
}

bool Tools_StaticMessageIterator::NextMessage_StaticMessageSystem()
{
	if(m_iMessageSet >= 0)
	{
		//Not the first time , so return to previous position in loop	
		goto ReEnterLoop;
	}

	for(m_iMessageSet = 0;m_iMessageSet<ASURA_STATICMESSAGE_NUMSETS;m_iMessageSet++)
	{
		
		for(m_iBlockIndex=0;m_iBlockIndex<Asura_StaticMessage_System::GetNumberOfMessageBlocks((Asura_StaticMessageSetIndex)m_iMessageSet);m_iBlockIndex++)
		{
			m_iMessageIndex = -1;
		ReEnterLoop :
			m_iMessageIndex++;
			if(m_iMessageIndex<Asura_StaticMessage_System::GetNumberOfMessagesInBlock((Asura_StaticMessageSetIndex)m_iMessageSet,m_iBlockIndex))
			{
				//have reached next valid message
				return true;
			}
		}
	}
	return false;
}

bool Tools_StaticMessageIterator::NextMessage_Dialogue()
{
	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	Asura_Dialogue_Event* pxEvent = 0;

	if(m_iDialogueIndex >= 0)
	{
		//Not the first time , so return to previous position in loop	
		pxEvent = Asura_Dialogue_Storage::GetEventByIndex(m_iDialogueIndex);
		goto ReEnterLoop;
	}

	for(m_iDialogueIndex=0; m_iDialogueIndex<(int)Asura_Dialogue_Storage::GetNumberOfEvents(); m_iDialogueIndex++)
	{
		pxEvent = Asura_Dialogue_Storage::GetEventByIndex(m_iDialogueIndex);

		for(m_iBlockIndex=0; m_iBlockIndex<(int)pxEvent->GetMessageStoreSize(); m_iBlockIndex++)
		{
			m_iMessageIndex = -1;
		ReEnterLoop :
			m_iMessageIndex++;

			const Asura_StaticMessageBlock& xBlock = pxEvent->GetMessageFromStore(m_iBlockIndex)->m_xMessageBlock;

			if(m_iMessageIndex < (int)xBlock.GetNumberOfMessagesInBlock())
			{
				//have reached next valid message
				return true;
			} 
		}
	}
	
	#endif
	return false;
}


bool Tools_StaticMessageIterator::Done() const
{
	return m_iLocation == TOOLS_STATICMESSAGELOCATION_END;	
}

void Tools_StaticMessageIterator::Restart()
{
	m_iLocation = 0;
	m_iMessageSet = -1;
	m_iBlockIndex = 0;
	m_iMessageIndex = -1;
	m_iDialogueIndex = -1;

	Next();
}

void Tools_StaticMessageIterator::SetModified() const
{
	Tools_StaticMessageID xMessageID;
	GetCurrentMessageID(xMessageID);

	xMessageID.SetModified();
}

bool Tools_StaticMessageIterator::CanSaveFile() const
{
	Tools_StaticMessageID xMessageID;
	GetCurrentMessageID(xMessageID);

	return xMessageID.CanSaveFile();
}

bool Tools_StaticMessageIterator::DeleteCurrent()
{
	switch(m_iLocation)
	{
		case TOOLS_STATICMESSAGELOCATION_STATICMESSAGE :
		{
			if(m_iMessageSet<0 || m_iMessageSet >= ASURA_STATICMESSAGE_NUMSETS) return false;

			int iNumMessages = 	Asura_StaticMessage_System::GetNumberOfMessagesInBlock((Asura_StaticMessageSetIndex)m_iMessageSet,m_iBlockIndex);
			if(m_iMessageIndex < 0 || m_iMessageIndex >= iNumMessages) return false;

			//delete this message
			u_short usBlockID = Asura_StaticMessage_System::ConvertIndexToID((Asura_StaticMessageSetIndex)m_iMessageSet,m_iBlockIndex);
			Asura_StaticMessage_System::DeleteMessage(usBlockID,m_iMessageIndex);

			//move the current index back , so using Next() will get the message after the deleted one
			m_iMessageIndex--;
			SetModified();

			return true;
		}

		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		case TOOLS_STATICMESSAGELOCATION_DIALOGUE :
		{
			Asura_Dialogue_Event* pxEvent = Asura_Dialogue_Storage::GetEventByIndex(m_iDialogueIndex);
			Asura_StaticMessageBlock& xBlock = pxEvent->GetMessageFromStore(m_iBlockIndex)->m_xMessageBlock;

			int iNumMessages = 	xBlock.GetNumberOfMessagesInBlock();
			if(m_iMessageIndex < 0 || m_iMessageIndex >= iNumMessages) return false;

			//delete this message
			xBlock.DeleteMessage(m_iMessageIndex);

			//move the current index back , so using Next() will get the message after the deleted one
			m_iMessageIndex--;
			SetModified();

		}
		#endif

		default :
			Asura_Assert(0,"Unhandled static message location");
			break;

	}
	return false;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////
/*--------------------------**
** Tools_StaticMessageBlock **
**--------------------------*/

Tools_StaticMessageBlock::Tools_StaticMessageBlock()
{
	Reset();
}

void Tools_StaticMessageBlock::Reset()
{
	m_bCanDeleteMessageBlock = false;
	m_usMessageBlock = ASURA_STATICMESSAGE_EMPTYBLOCK;
	m_pusMessageBlock = 0;
	m_pxMessageBlock = 0;
	m_eMessageSet = ASURA_STATICMESSAGE_ENTITYSET;
	m_pbModifiedFlag = 0;
}

bool Tools_StaticMessageBlock::IsValid() const
{
	return m_pusMessageBlock || m_pxMessageBlock || (m_usMessageBlock != ASURA_STATICMESSAGE_EMPTYBLOCK);
}

void Tools_StaticMessageBlock::SetMessageBlock(u_short* pusMessageBlock,enum Asura_StaticMessageSetIndex eSet)
{
	Reset();
	m_pusMessageBlock = pusMessageBlock;
	m_eMessageSet = eSet;
	m_bCanDeleteMessageBlock = true;
}

void Tools_StaticMessageBlock::SetMessageBlock(u_short usMessageBlock,enum Asura_StaticMessageSetIndex eSet)
{
	Reset();

	Asura_Assert(usMessageBlock != ASURA_STATICMESSAGE_EMPTYBLOCK,"Shouldn't be passing in an empty message block to this function");
	if(usMessageBlock == ASURA_STATICMESSAGE_EMPTYBLOCK) return;

	m_usMessageBlock = usMessageBlock;
	m_pusMessageBlock = &m_usMessageBlock;
	m_eMessageSet = eSet;
	m_bCanDeleteMessageBlock = false;
}

void Tools_StaticMessageBlock::SetMessageBlock(Asura_StaticMessageBlock* pxMessageBlock,bool* pbModifiedFlag)
{
	Reset();
	m_pxMessageBlock = pxMessageBlock;
	m_bCanDeleteMessageBlock = true;
	m_pbModifiedFlag = pbModifiedFlag;
}


int Tools_StaticMessageBlock::GetNumberOfMessagesInBlock() const
{
	if(m_pusMessageBlock)
	{
		return Asura_StaticMessage_System::GetNumberOfMessagesInBlock(*m_pusMessageBlock);
	}
	else if(m_pxMessageBlock)
	{
		return m_pxMessageBlock->GetNumberOfMessagesInBlock();
	}
	return 0;
}

Asura_StaticMessage* Tools_StaticMessageBlock::GetMessagePointer(int iIndex)
{
	if(m_pusMessageBlock)
	{
		return Asura_StaticMessage_System::GetMessagePointer(*m_pusMessageBlock,iIndex);
	}
	else if(m_pxMessageBlock)
	{
		return m_pxMessageBlock->GetMessagePointer(iIndex);
	}
	return 0;
}

void Tools_StaticMessageBlock::AddMessage(const Asura_StaticMessage& xMessage)
{
	if(m_pusMessageBlock)
	{
		Asura_StaticMessage_System::AddMessage(m_eMessageSet,xMessage,*m_pusMessageBlock);
	}
	else if(m_pxMessageBlock)
	{
		m_pxMessageBlock->AddMessage(xMessage);
	}
}

bool Tools_StaticMessageBlock::DeleteMessage(int iIndex)
{
	int iNumMessages = GetNumberOfMessagesInBlock();
	if(!iNumMessages) return false;
	
	//if down to the last message ,check to make sure it is allowable to delete the block entirely. RAR 25/05/2060
	if(iNumMessages==1 && !m_bCanDeleteMessageBlock)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Can't remove all messages from message block when editing it in this way.");
		return false;
	}

	if(m_pusMessageBlock)
	{
		Asura_StaticMessage_System::DeleteMessage(m_pusMessageBlock,iIndex);
	}
	else if(m_pxMessageBlock)
	{
		m_pxMessageBlock->DeleteMessage(iIndex);
	}

	return true;	
}

bool Tools_StaticMessageBlock::IsEditedMessageBlock(u_short usBlockID)
{
	if(usBlockID == ASURA_STATICMESSAGE_EMPTYBLOCK) return false;
	if(!m_pusMessageBlock) return false;
	if(*m_pusMessageBlock != usBlockID) return false;

	return true;
}

void Tools_StaticMessageBlock::SetModified()
{
	if(m_pbModifiedFlag) *m_pbModifiedFlag = true;

	if(m_pusMessageBlock)
	{
		//set the file containing this block as modified as well
		switch(m_eMessageSet)
		{
			case ASURA_STATICMESSAGE_ENTITYSET :
				Tools_SaveAsura::SetEntityModified();
				break;
			case ASURA_STATICMESSAGE_CUTSCENESET :
				Tools_SaveAsura::SetCutscenesModified();
				break;

			default :
				Asura_Assert(0,"Unknown message set");
		
		}
	}
}

///////////////////////////////////////////////////
// Tools_SkinMarkerPoint_HashDropDownList_Dialog //
///////////////////////////////////////////////////
// Select a marker point based on those for the specified skin.
Tools_SkinMarkerPoint_HashDropDownList_Dialog::Tools_SkinMarkerPoint_HashDropDownList_Dialog( Asura_Vector_2& xOffset,
																							  Asura_Guid uEntity,
																							  Asura_Hash_ID* puMarkerHash,
																							  const char* szName )
: Asura_Hash_DropDownList_Dialog(xOffset)
, m_pxSkin(0)
{
	// Find skin in order to generate a list of marker points.
	const Asura_ServerEntity_PhysicalObject* pxEntity = Asura_ServerEntity_PhysicalObject::FindEntity(uEntity);
	Asura_Assert(pxEntity, "Couldn't find server entity");
	if (pxEntity)
	{
		const Asura_Hash_ID uSkin = pxEntity->GetSkinID();
		if (uSkin != ASURA_HASH_ID_UNSET)
		{
			m_pxSkin = Asura_Animation_System::GetSkinByHash(uSkin);
		}
		Asura_Assert(m_pxSkin, "Couldn't find skin for marker points");
	}

	SetOnlyUpdateSelectionIndexOnClick(true);
	Initialise(xOffset, 10, puMarkerHash, 0, szName);
}

// Get the number of hash values (not including ASURA_HASH_ID_UNSET)
u_int Tools_SkinMarkerPoint_HashDropDownList_Dialog::GetNumberOfHashValues() const
{
	return (m_pxSkin ? m_pxSkin->NumberOfMarkerPoints : 0);
}
	
// Get the name for the iIndex'th hash value
const char* Tools_SkinMarkerPoint_HashDropDownList_Dialog::GetNameByIndex(int iIndex) const
{
	if (m_pxSkin && m_pxSkin->MarkerPoint && iIndex < m_pxSkin->NumberOfMarkerPoints)
	{
		return m_pxSkin->MarkerPoint[iIndex].GetName().GetString();
	}
	return GetUnknownHashText();
}

/////////////////////////////////////////////////////////////////////////////////
/*--------------------------------------**
** Tools_MessageLogAction_StaticMessage **
**--------------------------------------*/
void Tools_MessageLogAction_StaticMessage::Execute()
{
	switch(m_xMessageID.GetLocation())
	{
		case TOOLS_STATICMESSAGELOCATION_STATICMESSAGE :
			Execute_StaticMessageSystem();
			break;
		
		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		case TOOLS_STATICMESSAGELOCATION_DIALOGUE :
			Execute_Dialogue();
			break;
		#endif

		default :
			Asura_Assert(0,"Unhandled static message location");
			break;
	}
}

void Tools_MessageLogAction_StaticMessage::Execute_StaticMessageSystem()
{
	u_short usMessageBlockID = m_xMessageID.GetStaticMessageBlockID();
	#ifdef ASURA_TOOLS_USE_NEW_MESSAGE_EDITOR
	Tools_MessageEditor::OpenMessageEditorAtMessage(usMessageBlockID, m_xMessageID.GetMessageIndex());
	#else
	Tools_Object* pxSender = Tools_Object_System::GetOwnerOfMessageBlock(usMessageBlockID);
	if(pxSender)
	{
		Tools_Object_System::GotoAndSelect(pxSender);
	}
	else
	{
		Tools_Timed_Message_System::AddUnloggedMessage(0xff0000,"Failed to find owner of static message %d",usMessageBlockID);
	}
	#endif
}

#ifdef ASURA_USE_DIALOGUE_SYSTEM
void Tools_MessageLogAction_StaticMessage::Execute_Dialogue()
{
	if(!Tools_DialogueEvent_Overlay::EditEvent(m_xMessageID.GetDialogueEvent()))
	{
		Tools_Timed_Message_System::AddUnloggedMessage(0xff0000,"Failed to find the dialogue event that this message belonged to");
	}
}
#endif ASURA_USE_DIALOGUE_SYSTEM

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Tools_ChooseMoveObject::CanBeSelected(Tools_Object* pxObject)
{
	Asura_Assert(pxObject, "Null tools object");
	if(!pxObject) return false;

	switch(pxObject->Type)
	{
	case TOT_Phonon:
	case TOT_StreamingBackgroundSound:
	case TOT_AttractorNode:
	case TOT_PositionEdit:
		{
			return true;
		}
	}

	//allow object if its derived physical object , or if it's the player
	const Asura_Guid uGuid = pxObject->GetGuid();
	if(uGuid == ASURA_GUID_UNREGISTERED) return false;

	return true;
}

void Tools_ChooseMoveObject::OnStart()
{ 
}

Tools_ChooseMoveObject::Tools_ChooseMoveObject( Tools_PositionEdit_Object* pxObject, Tools_Object* pxPreviousObject ) 
: m_pxPositionEditObject(pxObject) 
, m_pxPreviousObject(pxPreviousObject)
, m_bBlockFunctionCall_OnFinish(false)
{
}

Tools_ChooseMoveObject::~Tools_ChooseMoveObject()
{
}

void Tools_ChooseMoveObject::OnFinish()
{
	// Function will be called once more recursively. However, we don't want this, so lets block it if necessary.
	if( m_bBlockFunctionCall_OnFinish )
	{
		return;
	}

	//Turn off Flying mode
	Tools_Movement::SetFlyingObject(NULL);

	if( m_pxPositionEditObject )
	{
		m_bBlockFunctionCall_OnFinish = true;		// Make sure to prevent recursive calls of this same function

		Tools_Object_System::DeleteObject(m_pxPositionEditObject);	// Calling this function, OnFinish will be called second time!

		m_bBlockFunctionCall_OnFinish = false;
		m_pxPositionEditObject = NULL;

		// After calling Delete object the s_pxChooseObject should be empty, and therefore we can select previous object.
		Tools_Object_System::SelectObject( m_pxPreviousObject );
	}

	delete this;
}

void Tools_ChooseMoveObject::HandleKeyboard()
{
	if (Asura_UserInput::DebouncedKeyPressed('f'))
	{
		Tools_Object_System::ToggleFlying();
	}

	if (Asura_UserInput::DebouncedKeyPressed('v'))
	{
		Tools_Object_System::StartSnapToVertex();
	}
}

