//
// Tools_Mode_Hierarchy.cpp
//
// Created by Richard Rice - 02/04/2002
// 
// Copyright (C) 2002 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"

#include "Hierarchy/Tools_Mode_Hierarchy.h"

#include "Animation/Asura_Animation.h"
#include "Animation/Asura_Animation_Posable.h"
#include "Animation/Asura_AnimationAttachment.h"
#include "Animation/Asura_Hierarchy.h"
#include "Camera/Asura_Camera.h"
#include "ChunkSystem/Asura_Chunk_Loaders.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Core/Asura_Los.h"
#include "Core/Asura_MemPool.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Timers.h"
#include "Core/Asura_UserInput.h"
#include "Debugging/Asura_Assert.h"
#include "Debugging/Asura_Logfile.h"
#include "Interface/Asura_Cursor.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Gadget.h"
#include "Interface/Asura_Menubar.h"
#include "Interface/Asura_Menubar_System.h"
#include "Interface/Asura_Overlay.h"
#include "Interface/Asura_Overlay_System.h"
#include "Light/Asura_GI_Solution.h"
#include "Light/Asura_Photon.h"
#include "Mesh/Asura_Mesh.h"
#include "Mesh/Asura_Mesh_System.h"
#include "Rendering/Asura_Render.h"
#include "Rendering/Asura_Text_Output.h"
#include "Rendering/Asura_Viewport.h"
#include "Sound/Asura_Sound.h"
#ifdef ASURA_USE_DIALOGUE_SYSTEM
#include "Sound/Tools_Dialogue_System.h"
#endif
#include "Sound/Asura_Sound_Event.h"
#include "Sound/Asura_Sound_Event_System.h"
#include "Sound/Asura_Sound_Event_Storage.h"

// Asura PC
#include "Rendering/Asura_Platform_RenderObjectHierarchy.h"
#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
	#include "Rendering/Asura_PlatformRenderCharacter.h"
#endif
#if defined(ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER) && ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER
	#include "Rendering/Asura_Direct3D_SmoothSkinRenderer.h"
#endif
#include "Rendering/Asura_Direct3D_StoreRenderTargets.h"

// Tools
#include "Tools_Overlays.h"
#include "Tools_System.h"
#include "Hierarchy/Tools_Animation_Files.h"
#include "Hierarchy/Tools_Dialog_PickSkin.h"
#include "Hierarchy/Tools_SkinLOD.h"
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "Hierarchy/Overlays/Tools_Overlay_Hierarchy_ViewFacialAnimation.h"
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
#include "Lighting/Tools_Photon_Object.h"
#include "Load&Save/Tools_AsuraFileContents.h"
#include "Load&Save/LevelExport/Tools_Level_Export.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save/Tools_Sound_Export.h"
#include "Misc/Tools_AVI_Writer.h"
#include "Misc/Tools_Dialog.h"
#include "Misc/Tools_Flag_IniFile.h"
#include "Misc/Tools_IniFile.h"
#include "Misc/Tools_Modal_Overlay.h"
#include "Misc/Tools_Help.h"
#include "Misc/Tools_Render.h"
#include "Misc/Tools_Render_Options.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Movement/Tools_Movement.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_VolumeEdit_Object.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Sound/Tools_PickSound.h"
#include "Sound/Tools_Sound_PS2Settings.h"
#include "Sound/Tools_PickSoundEvent_Dialog.h"
#include "Sound/Tools_SoundEvent_System.h"
#ifdef ASURA_USE_DIALOGUE_SYSTEM
#include "Sound/Tools_DialogueEvent_Dialogs.h"
#endif

static const char* szHIERARCHY_LOGFILE = "logfiles/hierarchy.txt";
static bool s_bOutputToLogfile = false;



///////////////////////////////////////////////////////////////////////////////////////
/*--------------**
** Help Strings **
**--------------*/
static const char* HierarchyModeHelpStrings [] =
{
	"< :                     Previous Anim",
	"> :                     Next Anim",
	"Shift + < :             Previous Secondary Anim",
	"Shift + > :             Next Secondary Anim",
	"Ctrl + < :              Previous Skin",
	"Ctrl + > :              Next Skin",
	"; :                     Animation Timer Step Backwards",
	"' :                     Animation Timer Step Forwards",
	"Enter :                 Play/Resume Animation",
	"Space :                 Pause/Reset to Start",
	"1,2,...,0 :             Restore hotkeyed animation",
	"Shift + 1,2,...,0 :     Save animation to hotkey",
	NULL
};
///////////////////////////////////////////////////////////////////////////////////////
/*-------------------------------**
** Tools_Object_HierarchyCompass **
**-------------------------------*/

//simple object for showing the compass point... doesn't need to do much really
class Tools_Object_HierarchyCompass : public Tools_Object
{
public :
	Tools_Object_HierarchyCompass()
	{
		SetMesh("ToolsShape_Particle");
	}

	virtual const char* GetName(){return "Compass";}
	virtual bool CanBeMoved(){return false;}
	virtual bool CanBeDestroyed(){return false;}
	virtual bool IsHidden()
	{
		return !s_bShow || Tools_Object::IsHidden();
	}

	static bool s_bShow;
	
};
bool Tools_Object_HierarchyCompass::s_bShow = false;


///////////////////////////////////////////////////////////////////////////////////////

// G L O B A L S   A N D   D E B U G V A R S /////////////////////////////////
DEBUGVAR u_int dbg_auBBColours[4] = { 0xffffff00, 0xff00ffff, 0xffff00ff, 0xffff0000 };
/*DEBUGVAR*/ bool dbg_bRecalcBoundingBoxData = false;

//offset for positions in hierarchy mode , so that none of the environmental stuff
//gets in the way.
// (AW - 20.11.2004) - Reduced from 10,000 to 500 to eliminate fp-inaccuracy problems
// when rendering the hierarchy (characters were a bit jelly-like)
// RAR 13.12.2005 - Removed offset entirely now , since I think it is no longer needed.
static Asura_Vector_3 g_xHierarchyModeOrigin(0.0f, 0.0f, 0.0f);

static int g_iPreviousScreenWidth = 0;//used for detecting changes in screen size

static const float fASURA_HR_SOUND_INNER_RADIUS = 1.0f;
static const float fASURA_HR_SOUND_OUTER_RADIUS = 20.0f;

//static member variables
Tools_Mode_Hierarchy*			Tools_Mode_Hierarchy::s_pxThis = NULL;
const Asura_Hierarchy_Skin*		Tools_Mode_Hierarchy::s_pxDefaultSkin = NULL;



// These need to match enum AnimationTimerMode
static const char* const s_aszTimerModeStrings[] = { "Proportion", "Seconds", "Frames", "25 FPS", "30 FPS" };

static char s_aszUnset[] = "  ( None )";

Tools_Mode_Hierarchy::Tools_Mode_Hierarchy()
{
	s_pxThis = this;

	HierarchyOverlayPosition = Asura_Vector_2(3.0f,28.0f);
	HierarchyOverlayOpen = true;

	m_xAnimPropertiesOverlayPosition = Asura_Vector_2(23.0f,28.0f);
	m_bAnimPropertiesOverlayOpen = true;

	m_xSkinPropertiesOverlayPosition = Asura_Vector_2(43.0f,3.0f);
	m_bSkinPropertiesOverlayOpen = true;
	AnimationControlOverlayOpen = true;
	AnimationFlagOverlayPosition = Asura_Vector_2(3.0f,3.0f);

	HierarchyLODOverlayPosition = Asura_Vector_2(23.0f,3.0f);
	HierarchyLODOverlayOpen = false;

	FrameReductionOverlayPosition = Asura_Vector_2(20.0f,3.0f);
	FrameReductionOverlayOpen = false;

	AnimationSoundOverlayPosition = Asura_Vector_2(3.0f,3.0f);
	m_xSkinSoundDefnOverlayPosition = Asura_Vector_2(23.0f,3.0f);
	m_xAnimSoundDefnOverlayPosition = Asura_Vector_2(33.0f,3.0f);

	m_xPosableAnimOverlayPosition = Asura_Vector_2(43.0f,28.0f);
	m_bPosableAnimOverlayOpen = false;

	Hierarchy = 0;
	Animation = 0;
	m_pxSkin = 0;
	m_puSkinID = new Asura_Hash_ID[ASURA_ANIMATION_SKIN_COUNT];
	for(int i=0; i<ASURA_ANIMATION_SKIN_COUNT; i++)
	{
		m_puSkinID[i] = 0;
	}

	m_pxSkinCollisionVolume = 0;
	AnimationFlagList = 0;
	m_pxAnimationSoundList = 0;
	m_pxMostRecentSecondary = 0;

	AnimDuration = 0;
	m_iAnimFrames = 1;
	TreadmillSpeed = 0;
	TreadmillDirection = 0;
	TreadmillMesh = 0;
	m_bAnimHasBB = false;
	m_fLoopPoint = 0;
	m_bWarnedAboutCompressedFrames = false;

	m_fAnimationControl_Speed = 1.0f;
	m_fAnimationControl_DeltaTime = 0;
	m_fAnimationControl_Timer = 0;
	m_fAnimationControl_SlideTimer = 0;
	m_bAnimationControl_TimerChanged = false;
	m_bAnimationControl_SlideTimerChanged = false;
	m_eAnimationControl_TimerMode = TIMER_PROP;

	m_fTimeSliderMin = 0.0f;
	m_fTimeSliderMax = 1.0f;


	SelectedIntervalIndex = 0;
	SelectedIntervalIndexChanged = false;
	AnimationFlagOverlayNeedsUpdate = false;
	FlagCoversInterval = false;
	FlagStartChanged = false;
	FlagEndChanged = false;

	m_iSelectedSoundIndex = 0;
	m_bSelectedSoundIndexChanged = false;
	m_bSoundStartChanged = false;
	m_bSoundEndChanged = false;
	m_bSoundLooped = false;
	m_bSoundLoopedChanged = false;

	ModifiedFlag = false;
	LODModified = false;
	m_bSkinModified = false;
	m_bSkinSelectionChanged = false;

#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
	m_pxSkinLOD = 0;
#endif
	for(int i=0;i<TOOLS_MAX_LOD_MODELS;i++)
	{
		m_afLODDistances[i] = 10.0f*(i+1);
		m_afLODPercent[i] = 0.0f;
	}

	m_bShowLights = true;
	m_bShowMarkerPoints = false;
    m_bShowSectionBoundingBoxes = false;
	m_bShowTreadmill = true;
	m_bUseMovementOffsets = true;
	m_iMaxBonesPerVertex = 4;

	for(int i=0;i<TOOLS_MAX_HOTKEY_ANIM;i++)
	{
		m_auHotkeyedAnimations[i] = ASURA_HASH_ID_UNSET;
	}

	m_iBackupAnimationArraySize = 0;
	m_pxBackupAnimation = 0;

	m_bDoFrameRemoval = false;
	m_bUpdateFrameRemoval = false;
	m_fPositionTolerance = 0;
	m_fAngleTolerance = 0;
	m_iFramesBefore = 0;
	m_iFramesAfter = 0;


	m_pxAnimSoundEdit = 0;
	m_pxSkinSoundEdit = 0;

	m_bRecordingAVI = false;
	m_iAVIFrame = 0;

	m_pxCurrentAnimBackup = 0;
	m_xCurrentAnimFilenameBackup.MakeNull();

	m_uPrimaryAnimFlags = AsuraAnimationFlag_Loop|AsuraAnimationFlag_UseMovementOffset;
	m_bAnimFlagsChanged = false;

	m_pxCompassObject = 0;

	m_bShadowed = false;
	m_bWireframeMesh = false;
	m_bDrawMesh = true;
	m_bDrawSkeleton = false;
	m_bDrawMarkerPointAxes = false;
	m_bDrawCollisionMesh = false;
	m_bShowSkinAndAnimProperties = false;

	Asura_Mem::Set( m_aiTempLightToolsIDs, 0, sizeof( m_aiTempLightToolsIDs ) );

	m_iNumTestSounds = 0; 
	m_pxTestSounds = 0;
	m_iTestSoundIndex = 0;
	m_pszTestSoundNames = 0;
	m_iTestSoundNameArraySize = 0;
	m_bTestSoundIndexChanged = false;
	m_uTestSoundEvent = 0;
	m_bTestSoundEventChanged = false;
	m_xTestSoundOverlayPosition = Asura_Vector_2(33.0f,3.0f);

    Asura_SimpleLogfileClear( szHIERARCHY_LOGFILE );

	m_xPos.Zero();
	m_xOldPos = m_xPos;
	m_xMat.Identity();

}

void Tools_Mode_Hierarchy::InitialiseMode(void)
{

	//create the menubar
	MenuHandle = Asura_Menubar_System::CreateMenuSection();
	Asura_Menubar_Title* title;


	title = Asura_Menubar_System::AddTitle(MenuHandle,"Overlays");
	title->AddItem("Hierarchy Mode",Tools_Mode_Hierarchy::_ToggleHierarchyOverlay,Tools_Mode_Hierarchy::_HierarchyOverlayIsOn);
	title->AddItem("Anim Properties",Tools_Mode_Hierarchy::_ToggleAnimPropertiesOverlay,Tools_Mode_Hierarchy::_AnimPropertiesOverlayIsOn);
	title->AddItem("Skin Properties",Tools_Mode_Hierarchy::_ToggleSkinPropertiesOverlay,Tools_Mode_Hierarchy::_SkinPropertiesOverlayIsOn);
	title->AddItem("Animation Control",Tools_Mode_Hierarchy::_ToggleAnimationControlOverlay,Tools_Mode_Hierarchy::_AnimationControlOverlayIsOn);
	title->AddItem("Level of Detail",Tools_Mode_Hierarchy::_ToggleHierarchyLODOverlay,Tools_Mode_Hierarchy::_HierarchyLODOverlayIsOn);
	title->AddItem("Frame Removal Preview",Tools_Mode_Hierarchy::_ToggleFrameReductionOverlay,Tools_Mode_Hierarchy::_FrameReductionOverlayIsOn);
	title->AddItem("Posable Anim Preview",Tools_Mode_Hierarchy::_TogglePosableAnimOverlay,Tools_Mode_Hierarchy::_PosableAnimOverlayIsOn);
	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	title->AddItem("Dialogue Overlay",Tools_Dialogue_System::ToggleOverlay,Tools_Dialogue_System::OverlayIsOn);
	#endif
#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	title->AddItem("View Facial-Animation", Tools_Overlay_Hierarchy_ViewFacialAnimation::ToggleOverlay, Tools_Overlay_Hierarchy_ViewFacialAnimation::OverlayExists);
#endif	//defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)


	//load values from .dat file
	TransferModeData(TOOLSMODEDATAFILE_LOAD);

	//create overlays
	CreateHierarchyOverlay();
	CreateAnimPropertiesOverlay();
	CreateSkinPropertiesOverlay();
	CreateAnimationControlOverlay();
	CreateHierarchyLODOverlay();
	//CreateRagdollEditorOverlay();
	CreateFrameReductionOverlay();
	CreatePosableAnimOverlay();

	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	if(Tools_Dialogue_System::OverlayShouldBeOpen()) Tools_Dialogue_System::CreateOverlay();
	#endif

	//enable object system
	Tools_Object_System::EnableAllControls(true);
	//but not undo , since the keys clash
	Tools_Object_System::DisableUndo();

	//add help
	Tools_Help::AddHelpTopic("Hierarchy Mode",HierarchyModeHelpStrings);

	//find the shape for the treadmill
	Asura_String mesh_name = "ToolsShape_Treadmill";
	TreadmillMesh = Asura_Mesh_System::GetMesh(mesh_name);
	Asura_Assert(TreadmillMesh,"Failed to find ToolsShape_Treadmill");

	//we want the animation to be drawn even though the environment isn't visible
    Asura_Hierarchy_System::SetNoCullingFlag( true );
	Tools_Render_Options::HideEnvironment = true;

	//initiase secondary anim data
	for(int i=0;i<MAX_SECONDARY_ANIM;i++)
	{
		SecondaryAnims[i].Initialise(i);
	}

	//store current camera position and orientation
	m_xPreModeCameraPosition = Tools_Movement::GetCameraObject()->Position;
	m_xPreModeCameraOrientation = Tools_Movement::GetCameraObject()->Orientation;
	
 	//jump to near the origin
	
 	Asura_Vector_3 position(0,-1,-2);
	position += g_xHierarchyModeOrigin;
 	Asura_Matrix_3x3 orientation;
 	orientation.Identity();


	Tools_Movement::TeleportCameraToPosition(&position,&orientation);
	
	//create some default lights
	Asura_Vector_3 axLightPos[ NUM_TEMP_LIGHTS ] =
	{
		Asura_Vector_3(0.0f,-1.0f,-3.0f),
		Asura_Vector_3(2.6f,-1.0f,1.5f),
		Asura_Vector_3(-2.6f,-1.0f,1.5f)
	};

	{
		// DLH - 15 March 2007 12:52:30 - Find all the Active Photon, turen them off and note them so they can be turned back on later.
		Asura_TrackingHandlePool<Asura_Photon>* pxPhotonList = Asura_Photon_System::GetDynamicPhotonList();
		Asura_TrackingHandlePool<Asura_Photon>::Iterator xIterator = pxPhotonList->GetIterator();
		
		this->m_xPhotonsActiveAtStart.empty();

		for (xIterator.Begin(); !xIterator.End(); xIterator.Next())
		{
			Asura_Photon* pxPhoton = xIterator.GetCurrentPointer();
			
			if( false == pxPhoton->IsActive())
			{
				continue;
			}
			
			pxPhoton->SetAsActive( false);
			this->m_xPhotonsActiveAtStart.add_entry( pxPhoton);
		}
	}

	for( u_int u = 0; u < NUM_TEMP_LIGHTS; ++u )
	{
		Tools_Object* const pxNewLight = new Tools_Photon_Object( &axLightPos[u], &Asura_Maths::IdentityMatrix3x3 );
		if( pxNewLight )
		{
			m_aiTempLightToolsIDs[u] = pxNewLight->GetToolsID();
		}
	}

	m_pxSoundIniFile = new Tools_Flag_IniFile("ToolsIniFiles\\KeyframeSounds.ini");
	m_pxAnimationSoundList = m_pxSoundIniFile->GetFlagList("Keyframe Sounds");

	m_xPosableHipOrient.Zero();
	m_xPosableHeadOrient.Zero();

	if(s_pxDefaultSkin)
	{
		const char* szFilename = Asura_HashedFilename_System::GetFullFilename(Asura_GetHashID(s_pxDefaultSkin->Name.GetString()));
		if(szFilename && Tools_AsuraFileContents::GetAsuraFileType(szFilename) == AsuraFile_Skin)
		{
			SetSkin(s_pxDefaultSkin->Name.GetString(),szFilename);
		}
		
	}
	else if(Tools_SaveAsura::GetFileType() == AsuraFile_Skin || Tools_SaveAsura::GetFileType() == AsuraFile_AnimFragment)
	{
		//loaded file is a skin file ,so select it by default
		Asura_String xSkinName;
		Asura_String xFileName = Tools_SaveAsura::GetMainFilename();
		xFileName.SplitFilename(0,&xSkinName,0);

		SetSkin(xSkinName.GetString(),xFileName.GetString());

		if(Tools_SaveAsura::GetFileType() == AsuraFile_AnimFragment)
		{
			//Is also an anim file , so select the animation
			//(anim name and skin name will be the same)
			Asura_Hierarchy_Anim* pxAnim = Asura_Animation_System::GetAnim(xSkinName.GetString());
			if(pxAnim) SelectAnimation(pxAnim,xFileName.GetString());
		}
	}

	Tools_MarkerPoint_Object::LoadMarkerPointConfigFile();

	Asura_Hierarchy_Anim::s_bPreviewCompressedKeyframes = true;

	//Prevent the environment lighting from affecting th models in this mode
	Asura_GI_Solution::DisableStaticLighting(true);


	// (MH - 25.05.2006) force regeneration of skin strips when entering this mode, if we might
	// have modified the mesh/texturing/materials in a different mode (i.e. if we're editing a skin)
	if(Tools_SaveAsura::GetFileType() == AsuraFile_Skin)
	{
#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
		Asura_Direct3D_CharacterRenderer::GetCurrentHandler().Reset();
#endif
#if defined(ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER) && ASURA_PC_USE_SHADER_MODEL_3_SMOOTH_SKIN_RENDERER
		Asura_Direct3D_SmoothSkinRenderer::Reset();
#endif

		Asura_Direct3D_ObjectHierarchyRenderer::GetCurrentHandler().Reset();
	}

	m_pxCompassObject = new Tools_Object_HierarchyCompass;

}


void Tools_Mode_Hierarchy::ShutdownMode(void)
{
	Asura_Hierarchy_Anim::s_bPreviewCompressedKeyframes = false;

	//restore keyframes to current anim
	RestoreAnimationKeyFrames();


	Tools_Render_Options::HideEnvironment = false;
	Asura_Hierarchy_System::SetNoCullingFlag( false );
	
	Tools_Help::RemoveHelpTopic("Hierarchy Mode");

	//save values to .dat file
	TransferModeData(TOOLSMODEDATAFILE_SAVE);

	//shutdown the menu
	if(MenuHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Menubar_System::DestroyMenuSection(MenuHandle);
		MenuHandle = ASURA_INVALID_HANDLE;
	}

	if(m_pxSkinCollisionVolume) m_pxSkinCollisionVolume->Destroy();

	//close overlays
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_MODE);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ANIMPROPERTIES);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SKINPROPERTIES);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ANIMATIONPLAY);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ANIMATIONFLAG);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ANIMATIONSOUND);
	//Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SKINSOUNDS);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_HIERARCHYLOD);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ANIMATIONFRAMEREDUCTION);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_POSABLEANIM);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_HIERARCHY_TESTSOUNDS);
	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	Tools_Dialogue_System::CloseOverlays();
	#endif
	#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	Tools_Overlay_Hierarchy_ViewFacialAnimation::CloseOverlay();
	#endif

	
	//remove secondary anims
	for(int i=0;i<MAX_SECONDARY_ANIM;i++)
	{
		RemoveSecondaryAnim(&SecondaryAnims[i],true);
		Asura_Overlay_System::CloseOverlays(SecondaryAnims[i].OverlayID);
	}

	delete m_pxSkinSoundEdit;
	m_pxSkinSoundEdit = 0;

	delete m_pxAnimSoundEdit;
	m_pxAnimSoundEdit = 0;
	
#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
	delete m_pxSkinLOD;
	m_pxSkinLOD = 0;
#endif
	DeleteMarkerPointObjects();
	DeleteBoundingBoxObjects();

	Tools_Object_System::DeleteAllModeSpecific();
	m_pxCompassObject = 0;

	Tools_MarkerPoint_Object::DeleteMarkerPointConfigs();

	for( u_int u = 0; u < NUM_TEMP_LIGHTS; ++u )
	{
		Tools_Object* const pxCurrLight = Tools_Object_System::GetObjectFromToolsID( m_aiTempLightToolsIDs[u] );
		if( pxCurrLight )
		{
			pxCurrLight->Destroy();
		}
	}

	{
		// DLH - 15 March 2007 12:52:05 - Reactivate Dynamic Photons
		while( 0 != this->m_xPhotonsActiveAtStart.size())
		{
			Asura_Photon* pxPhoton = this->m_xPhotonsActiveAtStart.first_entry();
			pxPhoton->SetAsActive( true);
			
			this->m_xPhotonsActiveAtStart.delete_first_entry();
		}
	}
	
	//Store the current skin as the default to use when next entering the mode
	if(Hierarchy)
	{
		s_pxDefaultSkin = Hierarchy->GetSkin();
	}


	delete Hierarchy;
	Hierarchy = 0;

	delete [] m_pxBackupAnimation;
	m_pxBackupAnimation = 0;
	m_iBackupAnimationArraySize = 0;

	
	Animation = 0;
		
	delete m_pxSoundIniFile;
	m_pxSoundIniFile = 0;


    SkinFilename.MakeNull();
	SkinName.MakeNull();
	AnimName.MakeNull();

	Tools_Object_System::EnableAllControls(false);
	Tools_Object_System::EnableUndo();

	while(m_xLODBatchList.size())
	{
		delete m_xLODBatchList.first_entry(); 
		m_xLODBatchList.delete_first_entry();
	}

	//restore the camera position
	Tools_Movement::TeleportCameraToPosition(&m_xPreModeCameraPosition,&m_xPreModeCameraOrientation);

	//re-enable lighting
	Asura_GI_Solution::DisableStaticLighting(false);

	delete [] m_pxTestSounds;
	m_pxTestSounds = 0;
	m_iNumTestSounds = 0;
	DeleteTestSoundNameArray();


	delete [] m_puSkinID;
	m_puSkinID = 0;

	ToolsSoundDefnEdit::ClearPasteBuffer();
}

void Tools_Mode_Hierarchy::TransferModeData(int mode)
{
	//open the file
	Tools_ModeDataFile file("HierarchyMode.dat",mode);

	//tranfer the data
	//(none of these should be removed , and new data should go on the end, otherwise
	//the files will get screwed up)
	file.Transfer(HierarchyOverlayPosition);
	file.Transfer(HierarchyOverlayOpen);
	file.Transfer(AnimationControlOverlayOpen);
	file.Transfer(AnimationFlagOverlayPosition);
	file.Transfer(HierarchyLODOverlayPosition);
	file.Transfer(HierarchyLODOverlayOpen);
	for(int i=0;i<10;i++)
	{
		file.Transfer(m_auHotkeyedAnimations[i]);
	}
	file.Transfer(FrameReductionOverlayPosition);
	file.Transfer(FrameReductionOverlayOpen);
	file.Transfer(m_fPositionTolerance);
	file.Transfer(m_fAngleTolerance);
	file.Transfer(m_xAnimPropertiesOverlayPosition);
	file.Transfer(m_bAnimPropertiesOverlayOpen);
	file.Transfer(m_xSkinPropertiesOverlayPosition);
	file.Transfer(m_bSkinPropertiesOverlayOpen);
	file.Transfer(AnimationSoundOverlayPosition);
	file.Transfer(m_xSkinSoundDefnOverlayPosition);
	file.Transfer(m_xPosableAnimOverlayPosition);
	file.Transfer(m_bPosableAnimOverlayOpen);
	file.Transfer(m_xAnimSoundDefnOverlayPosition);
}

void Tools_Mode_Hierarchy::Update(void)
{

	UpdateTreadmill();

	if(m_xLODBatchList.size())
	{
		UpdateLODBatch();
	}

	//act upon keyboard input
	Tools_Mode_Hierarchy::HandleKeyboard();

	if(m_bSkinSelectionChanged)
	{
		m_bSkinSelectionChanged = false;
		SetSkinsInAsuraAnimation();
	}

	if(LODModified)
	{
		if(m_pxSkin)
		{
			GenerateLODModels();
		}
		LODModified = false;
	}

	if(m_bSkinModified)
	{
		if(m_pxSkin)
		{
			if(CanEditCollisionBB())
			{
				m_pxSkin->SetCollisionBB(m_xSkinCollisionBB);
			}
			SetSkinModified();
		}
		m_bSkinModified = false;
	}

	if(m_bAnimFlagsChanged)
	{
		UpdatePrimaryAnimFlags();
		m_bAnimFlagsChanged = false;
	}

	//update frame reduction preview. Deal with the overlay
	UpdateFrameReductionOverlay();
	//and recalculate frames if needed
	if(m_bUpdateFrameRemoval)
	{
		RemoveUnneededKeyFrames();
	}

	UpdateAnimationControlOverlay();

	//update editing of test sounds
	UpdateTestSounds();


	if(ModifiedFlag)
	{
		if(Animation)
		{
			//update the valules in the animation
			Animation->Duration = AnimDuration;
			Animation->Speed = TreadmillSpeed;
			Animation->Direction = Asura_Maths::DegToRad(TreadmillDirection);
			if(m_bAnimHasBB && !Animation->GetBB())
			{
				CalculateAnimationBB(true);	
			}
			else if(!m_bAnimHasBB)
			{
				Animation->SetBB(0);
			}
			Animation->SetLoopPoint(m_fLoopPoint);

			ModifiedFlag = false;
			SetAnimModified();

			if ( Hierarchy )
			{
				Hierarchy->m_xPrimaryWrapper.SetAnimDuration(Animation->Duration);
			}
		}
	}
	//update posable anim
	if ( Hierarchy && Hierarchy->IsPosableAnim() )
	{
		Asura_Animation_Posable* pxPosable = (Asura_Animation_Posable*) Hierarchy;
		pxPosable->SetHipOrient(m_xPosableHipOrient);
		pxPosable->SetHeadOrientation(m_xPosableHeadOrient);
	}


	//update weightings for secondary anims
	UpdateSecondaryAnims();

	//deal with changes to animation flags
	if(Animation && SelectedIntervalIndex<Animation->NumberOfAnimationFlags)
	{
		Asura_Assert(Animation->AnimationFlags,"");

		Asura_Animation_Flag_Interval* flagInterval = &Animation->AnimationFlags[(int)SelectedIntervalIndex];
	
		if(FlagStartChanged)
		{
			//if the start time has changed , then adjust the end time if required
			if(FlagCoversInterval)
			{
				if(flagInterval->EndTime<flagInterval->StartTime) flagInterval->EndTime=flagInterval->StartTime;
			}
			else
			{
				flagInterval->EndTime = flagInterval->StartTime;
			}
			FlagStartChanged = false;
			//move the animation to the flag start time
			m_fAnimationControl_Timer = flagInterval->StartTime;
			m_bAnimationControl_TimerChanged = true;
			m_fAnimationControl_Speed = 0;

			ModifiedFlag = true;

		}
		if(FlagEndChanged)
		{
			if(flagInterval->EndTime<flagInterval->StartTime) flagInterval->StartTime=flagInterval->EndTime;
			FlagEndChanged = false;
			//move the animation to the flag end time
			m_fAnimationControl_Timer = flagInterval->EndTime;
			m_bAnimationControl_TimerChanged = true;
			m_fAnimationControl_Speed = 0;

			ModifiedFlag = true;
		}
	}

	//deal with changes to animation sound 
	bool bNeedToUpdateSoundOverlay = false;
	if ( Animation && m_iSelectedSoundIndex < Animation->m_iNumberOfAnimationSounds )
	{
		Asura_Assert( Animation->m_pxAnimationSounds, "" );
		Asura_Animation_Sound* const pxSound = &Animation->m_pxAnimationSounds[m_iSelectedSoundIndex];

		if(m_bSoundStartChanged)
		{
			//if the start time has changed , then adjust the end time if required
			if(m_bSoundLooped)
			{
				if(pxSound->m_fEndTime<pxSound->m_fStartTime) pxSound->m_fEndTime=pxSound->m_fStartTime;
			}
			else
			{
				pxSound->m_fEndTime = pxSound->m_fStartTime;
			}
			m_bSoundStartChanged = false;
			//move the animation to the sound start time
			m_fAnimationControl_Timer = pxSound->m_fStartTime;
			m_bAnimationControl_TimerChanged = true;
			m_fAnimationControl_Speed = 0;

			ModifiedFlag = true;

		}
		if(m_bSoundEndChanged)
		{
			if(pxSound->m_fEndTime<pxSound->m_fStartTime) pxSound->m_fStartTime=pxSound->m_fEndTime;
			m_bSoundEndChanged = false;
			//move the animation to the flag end time
			m_fAnimationControl_Timer = pxSound->m_fEndTime;
			m_bAnimationControl_TimerChanged = true;
			m_fAnimationControl_Speed = 0;

			ModifiedFlag = true;
		}
		if(m_bSoundLoopedChanged)
		{
			//looped flag has changed so change the sound to cover the whole anim or a single point
			//depending on which way it has gone
			if(m_bSoundLooped)
			{
				pxSound->m_fStartTime = 0.0f;
				pxSound->m_fEndTime = 1.0f;
			}
			else
			{
				pxSound->m_fEndTime = pxSound->m_fStartTime;
			}
			m_bSoundLoopedChanged = false;

			ModifiedFlag = true;
			bNeedToUpdateSoundOverlay = true;
		}

	}

	if ( AnimationFlagOverlayNeedsUpdate || SelectedIntervalIndexChanged )
	{
		UpdateAnimationFlagOverlay();
	}

	if ( m_bSelectedSoundIndexChanged || bNeedToUpdateSoundOverlay)
	{
		UpdateAnimationSoundOverlay();
	}

	if(m_pxAnimSoundEdit) m_pxAnimSoundEdit->Update();
	if(m_pxSkinSoundEdit) m_pxSkinSoundEdit->Update();
	

    // (MH - 01.10.2002) Test code for trying out different ways of creating the bb data
    if (dbg_bRecalcBoundingBoxData)
    {
        dbg_bRecalcBoundingBoxData = false;
        if (m_pxSkin && m_pxSkin->m_pxSectionBoundingBoxData)
        {
			Asura_ResourcePushType(ASURA_RESOURCE_ANIMATION);
			{
				Asura_ResourceDeallocateArray(m_pxSkin->m_pxSectionBoundingBoxData);
			}
			Asura_ResourcePopType();
            m_pxSkin->m_pxSectionBoundingBoxData = NULL;
        }
    }

	int iScreenWidth = Asura_Viewport::GetWidth();
	if(iScreenWidth != g_iPreviousScreenWidth)
	{
		g_iPreviousScreenWidth = iScreenWidth;

		//screen size has changed , may need to reposition some overlays
		if(AnimationControlOverlayIsOn())
		{
			//re-create overlay by toggling off and on
			ToggleAnimationControlOverlay();
			ToggleAnimationControlOverlay();
		}

	}

	//Play any test sounds associated with flags that have occurred this frame
	if(Hierarchy)
	{
		int iFlags = Hierarchy->m_xPrimaryWrapper.GetFlagsForAllAnimsLastFrame();

		if(iFlags)
		{
			for(int i=0;i<m_iNumTestSounds;i++)
			{
				if(m_pxTestSounds[i].m_iFlag & iFlags)
				{
					if(m_pxTestSounds[i].m_uSoundEvent)
					{
						//play this sound
						const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::FindEvent(m_pxTestSounds[i].m_uSoundEvent);
						if(pxEvent)
						{
							Asura_Vector_3 xPosition = GetHierarchyPosition();
							Asura_Sound_Event_System::PlayEvent(pxEvent,0,&xPosition);
						}
					}
				}
			}
		}		
	}

	if(m_bRecordingAVI)
	{
		if(m_iAVIFrame > m_iAnimFrames)
		{
			//finished
			Tools_System::EndAVI();
		}
		m_bRecordingAVI = TheAviWriter.IsWritingFile();
	}
	
	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	Tools_Dialogue_System::Update();
	#endif
}

void Tools_Mode_Hierarchy::EarlyRender()
{
	SCOPEDPIXEVENT((0xff8800, L"Hierarchy Mode"));

	if(m_bShowTreadmill)RenderTreadmill();

	Asura_Render::CurrentStates.RequestWireFrameMode( m_bWireframeMesh );
	if(Hierarchy)
	{
		m_xOldPos = m_xPos;
		m_xPos = g_xHierarchyModeOrigin;
		m_xMat.Identity();

		float fFrameTime = Asura_Timers::GetGameTimeSlice() * m_fAnimationControl_Speed;
		
		if(m_bRecordingAVI)
		{
			//when recording AVIs advance one anim frame at a time
			m_fAnimationControl_Timer = m_iAVIFrame/(float)m_iAnimFrames;
			m_iAVIFrame++;

			AnimationControl_UpdateSlideTimer();

			SetHierarchyTimer(m_fAnimationControl_Timer);
			fFrameTime = 0;
		}
		else if(m_bAnimationControl_SlideTimerChanged)
		{
			AnimationControl_UpdateTimer();

			SetHierarchyTimer(m_fAnimationControl_Timer);
			fFrameTime = 0;
		}
		else if(m_fAnimationControl_DeltaTime || m_bAnimationControl_TimerChanged)
		{
			m_fAnimationControl_Timer+=m_fAnimationControl_DeltaTime;
			while(m_fAnimationControl_Timer<0)m_fAnimationControl_Timer++;
			while(m_fAnimationControl_Timer>1)m_fAnimationControl_Timer--;

			AnimationControl_UpdateSlideTimer();

			SetHierarchyTimer(m_fAnimationControl_Timer);
			fFrameTime = 0;
		}

		if(m_bUseMovementOffsets)
		{
			//add the movement offset onto the position to draw the hierarchy at
			Asura_Vector_3 xOffset;
			Asura_Quat xRelQuat;
			if(Hierarchy->GetMovementOffset(&xOffset,&xRelQuat))
			{
				m_xPos += xOffset;
				xRelQuat.ToMatrix( m_xMat );
			}
		}

#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
		//set max bones to use when rendering
		int iOldMaxBonesPerVertex = Asura_Direct3D_CharacterRenderer::GetMaxBonesPerVertex();
		Asura_Direct3D_CharacterRenderer::SetMaxBonesPerVertex(m_iMaxBonesPerVertex);
#endif

		// JAT 09/02/07 - toggle drawing the mesh on/off 
		u_int uRenderFlags = 0;
		if ( !m_bDrawMesh ) uRenderFlags |= ASURA_HIERARCHY_RENDER_DONTRENDER;

		{
			Hierarchy->Update( m_xPos, m_xMat, fFrameTime, uRenderFlags );
		}

		if ( m_bShadowed )
		{
			Hierarchy->RenderShadow( m_xPos, m_xMat, 0 );
		}

		// JAT 02/03/07 draw the skin collision mesh
		if ( m_bDrawCollisionMesh )
		{
			const Asura_CollisionMesh* const pxColMesh = m_pxSkin->GetCollisionMesh();
			if ( pxColMesh )
			{
				// Red
				const u_int uColMeshColour = 0xFFFF0000;
				Tools_Render::RenderCollisionMesh( pxColMesh, &m_xPos, &m_xMat, uColMeshColour );
			}
		}
		//update position of compass object
		m_pxCompassObject->Position = m_xPos;
		m_pxCompassObject->Orientation = m_xMat;

		//restore the original setting
#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
		Asura_Direct3D_CharacterRenderer::SetMaxBonesPerVertex(iOldMaxBonesPerVertex);
#endif

		if(fFrameTime)
		{
			m_fAnimationControl_Timer = Hierarchy->m_xPrimaryWrapper.GetPrimaryTimer();
			AnimationControl_UpdateSlideTimer();
		}
		
		//update positions of marker points
		UpdateMarkerPointObjects();

        // (MH - 01.10.2002) display the section BBs. The data calculated has already been 
        // done for rendering but by the time we get here the PC renderer has transformed everything
        // into model space so we need to redo it. See comment in Asura_Animation::LOSTest().
		// RAR  16/07/04 Changed to update positions of some bounding box tools objects instead
        if ( m_pxSkin && m_pxSkin->m_pxSectionBoundingBoxData && m_xBoundingBoxList.size())
        {
            Asura_Vector_3 BonePosition[ASURA_MAX_NO_OF_BONES_PER_MODEL];
            Asura_Matrix_3x3 BoneOrientation[ASURA_MAX_NO_OF_BONES_PER_MODEL];

	        // Get the tween storage, so we can get the currrent location for the bones. If the character is not
            // on screen this will probably be set to the previous pose when the character was rendered.
	        Asura_Hierarchy_Tween* pxTweenStorage = Hierarchy->GetTweenStore();
            Asura_Assert( pxTweenStorage, "Need tween storage to get marker point positions" );
	        if ( pxTweenStorage->IsPositionDataValid() )  
            {
	            for(int i = 0; i < m_pxSkin->NumberOfBones; i++)
	            {
					Asura_Matrix_3x3	xBoneOri;
					Asura_Vector_3		xBonePos;

					pxTweenStorage->GetBoneData(i).GetOrientation(xBoneOri);
					pxTweenStorage->GetBoneData(i).GetPosition(xBonePos);

					// If ragdoll is active then all bone positions & orientations are world relative
					if ( pxTweenStorage->IsPositionDataInWorldspace() )
					{
						BonePosition[i] = xBonePos;
						BoneOrientation[i] = xBoneOri;
					}
					else
					{
						// Compute the worldspace positions by hand
						if(i)
						{
							int parentIndex = m_pxSkin->ParentBone[i];
							Asura_Assert(parentIndex>=0 && parentIndex<i,"");

							BonePosition[i] = xBonePos * BoneOrientation[parentIndex]+BonePosition[parentIndex];
							BoneOrientation[i] =  BoneOrientation[parentIndex] * xBoneOri;
						}
						else
						{
							BonePosition[0] = xBonePos * m_xMat + m_xPos;
							BoneOrientation[0] = m_xMat * xBoneOri;
						}
					}
	            }

                for (int iBoneID = 0; iBoneID < m_pxSkin->NumberOfBones; iBoneID++)
                {
					m_xBoundingBoxList[iBoneID]->UpdatePosition(BonePosition[iBoneID],BoneOrientation[iBoneID]);

					// Purple for skeleton
					const u_int uSkeletonColour = 0xFF9030FF;
					const Asura_Vector_3 xJointPos = BonePosition[iBoneID];
					if ( m_bDrawSkeleton )
					{
						if ( iBoneID )
						{
							const int iParentIndex = m_pxSkin->ParentBone[iBoneID];
							Asura_Assert( iParentIndex >= 0 && iParentIndex < iBoneID, "" );
							const Asura_Vector_3 xParentPos = BonePosition[iParentIndex];
							// Draw a line from parent to joint
							Asura_Render::RenderLine( xParentPos, xJointPos, uSkeletonColour );
						}
					}
					if ( s_bOutputToLogfile )
					{
						const Asura_Quat xJointOrient( BoneOrientation[iBoneID] );
						Asura_SimpleLogfileOutput( ( szHIERARCHY_LOGFILE, "Joint[%d] '%s' Pos %f %f %f Orient %f %f %f %f\n", 
													 iBoneID, m_pxSkin->SectionData[iBoneID].SectionName.GetString(),
													 xJointPos.x, xJointPos.y, xJointPos.z,
													 xJointOrient.x, xJointOrient.y, xJointOrient.z, xJointOrient.w ) );
					}
                }
				if ( s_bOutputToLogfile )
				{
					s_bOutputToLogfile = false;
				}
            }
        }
	}

	// Draw the marker point axes
	if ( m_bDrawMarkerPointAxes )
	{
#if defined( ASURA_TARGET_SM3)
		// DLH - Store and Clear all the RenderTargets after the first one
		Asura_Direct3D_StoreRenderTargets xRenderTargetStore;
		
		xRenderTargetStore.StoreRenderTargets();
		Asura_Direct3D::ClearRenderTargets( 1);
#endif

		const float fMarkerPointAxesSize = 0.1f;
		for(LIF<Tools_MarkerPoint_Object*> xMarkerLIF(&m_xMarkerList);!xMarkerLIF.done();xMarkerLIF.next())
		{
			const Asura_Vector_3& xPos = xMarkerLIF()->Position;
			const Asura_Matrix_3x3& xOrient = xMarkerLIF()->Orientation;
			Asura_Render::RenderAxes( xPos, xOrient, fMarkerPointAxesSize );
		}
		
#if defined( ASURA_TARGET_SM3)
		Asura_Render::FlushDisplay();
		
		// DLH - Store and Clear all the RenderTargets after the first one
		xRenderTargetStore.RestoreRenderTargets();
#endif
	}
	Asura_Render::CurrentStates.RequestWireFrameMode(false);

	//update visibility of marker points and lights
	Tools_Object_System::SetTypeHidden(TOT_MarkerPoint,ObjectHideFlag_Mode,!m_bShowMarkerPoints);
	Tools_Object_System::SetTypeHidden(TOT_SectionBoundingBox,ObjectHideFlag_Mode,!m_bShowSectionBoundingBoxes);
	Tools_Object_System::SetTypeHidden(TOT_ToolsPhoton,ObjectHideFlag_Mode,!m_bShowLights);

	m_bAnimationControl_SlideTimerChanged = false;
	m_bAnimationControl_TimerChanged = false;
	m_fAnimationControl_DeltaTime = 0;
}

void Tools_Mode_Hierarchy::RenderUI(void)
{
	float x = 1;
	float y = Tools_Object_System::GetMaxTextY()+1.0f;

	if(Hierarchy && m_bUseMovementOffsets)
	{
		Asura_Vector_3 xOffset(0,0,0);
		Hierarchy->GetMovementOffset(&xOffset);
		//display the offset coordinates
		Asura_Text_Output::Print(x,y++,0xffffffff,0,"Pos  :  %8.3f %8.3f %8.3f",xOffset.x,xOffset.y,xOffset.z);
	}

	if(m_pxSkin)
	{
		//display the number of bones
		char szText[200];
		sprintf(szText,"Bones : %d",m_pxSkin->NumberOfBones);
		Asura_Text_Output::Print(x,y,0xffffffff,0,szText);
		
		//warn if the animation uses a different number
		if(Animation && Animation->NumberOfBones != m_pxSkin->NumberOfBones)
		{
			int iLength = strlen(szText);
			Asura_Text_Output::Print(x+iLength,y,0xffff0000,0," (%d in animation)",Animation->NumberOfBones);
		}
		y++;
	}
	
	if(m_bShowSkinAndAnimProperties)
	{
		if(m_pxSkin)
		{
			Asura_Text_Output::Print(x,y++,0xff00ff00,0,"Skin Properties :");
			Asura_Text_Output::Print(x+1,y++,0xffffffff,0,"Mirroring : %s", m_pxSkin->ExportedForMirroring() ? "Yes" : "No");
		}
		if(Animation)
		{
			Asura_Text_Output::Print(x,y++,0xff00ff00,0,"Anim Properties :");
			Asura_Text_Output::Print(x+1,y++,0xffffffff,0,"Mirroring : %s", Animation->ExportedForMirroring() ? "Yes" : "No");
			Asura_Text_Output::Print(x+1,y++,0xffffffff,0,"Delta : %s", Animation->IsDeltaAnimation ? "Yes" : "No");
			if(Animation->IsSharedAnim())
			{
				if(Animation->IsSharedAnimDeltaOrient())
				{
					Asura_Text_Output::Print(x+1,y++,0xffffffff,0,"Shared : Delta Orient");
				}
				else if(Animation->IsSharedAnimDeltaPos())
				{
					Asura_Text_Output::Print(x+1,y++,0xffffffff,0,"Shared : Delta Pos");
				}
				else
				{
					Asura_Text_Output::Print(x+1,y++,0xffffffff,0,"Shared : Bone Lengths");
				}
			}
			else
			{
				Asura_Text_Output::Print(x+1,y++,0xffffffff,0,"Shared : No");
			}
			Asura_Text_Output::Print(x+1,y++,0xffffffff,0,"Compass Movement : %s", Animation->HasMovementData() ? "Yes" : "No");
			Asura_Text_Output::Print(x+1,y++,0xffffffff,0,"Face Anim : %s", Animation->HasFacialAnimation() ? "Yes" : "No");
		}
	}

	if(m_bDoFrameRemoval && m_iFramesBefore)
	{
		//show results of the frame removal
		float m_fPercent = (m_iFramesAfter/(float)m_iFramesBefore) *100.0f;

		Asura_Text_Output::Print(x,y++,0xffffffff,0,"Frame Reduction : %d / %d  %.2f%%",m_iFramesAfter,m_iFramesBefore,m_fPercent);
		
	}

	if(Animation && Animation->NumberOfAnimationFlags)
	{
		Asura_Text_Output::Print( x, y++, 0xffffff00, 0, "'Keyframe flags' (%s)", 
								  s_aszTimerModeStrings[m_eAnimationControl_TimerMode] );
		
		//list the flags set on this animation
		for(int i=0;i<Animation->NumberOfAnimationFlags;i++)
		{
			Asura_Animation_Flag_Interval* flagInterval = &Animation->AnimationFlags[i];
			const float fDisplayStartTime = ConvertParametricTimer( flagInterval->StartTime );
			const float fDisplayEndTime = ConvertParametricTimer( flagInterval->EndTime );
			if(flagInterval->StartTime == flagInterval->EndTime)
				Asura_Text_Output::Print(x,y++,0xffffffff,0,"At %.3f :", fDisplayStartTime );
			else
				Asura_Text_Output::Print(x,y++,0xffffffff,0,"From %.3f to %.3f:", fDisplayStartTime, fDisplayEndTime );		

			if(!flagInterval->Flag)
			{
				Asura_Text_Output::Print(x+2,y++,0xffff0000,0,"<No flags set>");		
			}
			else if(!AnimationFlagList)
			{
				Asura_Text_Output::Print(x+2,y++,0xff00ff00,0,"0x%x",flagInterval->Flag);		
			}
			else
			{
				for(int bit=0;bit<32;bit++)
				{
					if(flagInterval->Flag & (1<<bit))
					{
						Asura_Text_Output::Print(x+2,y++,0xff00ff00,0,"%s",AnimationFlagList->GetFlagName(1<<bit));		
					}
				}	
			}
		}
	}

	if(Animation && Animation->m_iNumberOfAnimationSounds && m_pxAnimationSoundList)
	{
		y++;
		Asura_Text_Output::Print( x, y++, 0xffffff00, 0, "'Keyframe sounds' (%s)", 
								  s_aszTimerModeStrings[m_eAnimationControl_TimerMode] );
		
		//list the sounds set on this animation
		for(int i=0;i<Animation->m_iNumberOfAnimationSounds;i++)
		{
			Asura_Animation_Sound* pxSound = & Animation->m_pxAnimationSounds[i];
			const float fDisplayStartTime = ConvertParametricTimer( pxSound->m_fStartTime );
			const float fDisplayEndTime = ConvertParametricTimer( pxSound->m_fEndTime );
			
			if(pxSound->m_fStartTime == pxSound->m_fEndTime)
				Asura_Text_Output::Print(x,y++,0xffffffff,0,"At %.3f :", fDisplayStartTime );
			else
				Asura_Text_Output::Print(x,y++,0xffffffff,0,"From %.3f to %.3f:", fDisplayStartTime, fDisplayEndTime );		
			
			const char* pszName = m_pxAnimationSoundList->GetFlagName(pxSound->m_iAnimSoundID);
			if(pszName[0] == '<')
			{
				Asura_Text_Output::Print(x+2,y++,0xffff0000,0,"<Unknown sound>");		
			}
			else
			{
				Asura_Text_Output::Print(x+2,y++,0xff00ff00,0,"%s",pszName);		
			}
		}
	}

	#if 0 //LOS Test
	if(Hierarchy && Animation)
	{
		Asura_Vector_3 xPos(0,0,0);
		Asura_Matrix_3x3 xOrient;
		xOrient.Identity();

		//get the z direction for the mouse
		Asura_Vector_3 zvec;
		Asura_Cursor::GetLOSDirection(zvec);

		//and transform this direction into worldspace
		const Asura_Vector_3* cameraPosition = Asura_Camera::GetPosition();
		const Asura_Matrix_3x3* cameraOrientation = Asura_Camera::GetOrientation();

		zvec = zvec * (*cameraOrientation);

		//test against the model
		Asura_LOS_InputData xInput(*cameraPosition,zvec,ASURA_GUID_UNREGISTERED,ASURA_LOS_MAX_DISTANCE);
		Asura_LOS_OutputData xOutput;
		memset(&xOutput,0,sizeof(xOutput));
		xOutput.m_fDistanceToIntersection = ASURA_LOS_MAX_DISTANCE;

		if(Hierarchy->LOSTest(xPos,xOrient,&xInput,&xOutput))
		{
			const char* pszSectionName = m_pxSkin->SectionData ? m_pxSkin->SectionData[xOutput.m_iHitBoneID].SectionName.GetString() : 0;
			Asura_Text_Output::Print(x,y++,0xffffffff,0,"LOS : Bone %d - %s",xOutput.m_iHitBoneID,pszSectionName ? pszSectionName : "<Unnamed>");

			if( xOutput.m_iHitAttachmentID != Asura_AnimationAttachment::INVALID_ID )
			{
				Asura_Text_Output::Print(x,y++,0xffffffff,0,"Attachment ID : %d",xOutput.m_iHitAttachmentID);
			}
		}
	}
	#endif

	// Debug text for checking description data
	//if (m_pxRagdollDescription) m_pxRagdollDescription->Tools_DebugTextOutput(51, 3);
}

void Tools_Mode_Hierarchy::QuickSave(void)
{
	PreSave();
	Tools_SaveAsura::SaveModifiedAsuraFiles();
	Tools_SoundEvent_System::SaveModified();

	//also save out changes to PS2 sound settings
	if(Tools_Sound_PS2Settings::IsModified())
	{
		if(Tools_Sound_PS2Settings::Save())
		{
			Tools_Timed_Message_System::AddMessage(0x00ff00,"Saved %s.",PS2SettingsFilename);
		}
		else
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to save %s. (Write-protected file?)",PS2SettingsFilename);
		}
		
	}
}

void Tools_Mode_Hierarchy::PreSave()
{
	//make sure the animation has its proper frames before it gets saved
	RestoreAnimationKeyFrames();
	if(m_bDoFrameRemoval) m_bUpdateFrameRemoval = true;
}

bool Tools_Mode_Hierarchy::IsModified()
{
	if(Tools_SoundEvent_System::IsModified()) return true;

	return Tools_SaveAsura::AreAsuraFilesModified() || Tools_Sound_PS2Settings::IsModified();
}

void Tools_Mode_Hierarchy::SetSkinModified()
{
	Asura_Assert(s_pxThis,"");
	if(s_pxThis->m_pxSkin && !s_pxThis->SkinFilename.IsBlank())
	{
		Tools_SaveAsura::SetSkinModified(s_pxThis->m_pxSkin,s_pxThis->SkinFilename.GetString());
	}
}

void Tools_Mode_Hierarchy::SetAnimModified()
{
	Asura_Assert(s_pxThis,"");
	Asura_Hierarchy_Anim* pxAnim = s_pxThis->Animation;

	if(pxAnim && !s_pxThis->m_xAnimFilename.IsBlank())
	{
		if(pxAnim->BoneAnims)
		{
			Tools_SaveAsura::SetAnimationModified(pxAnim,s_pxThis->m_xAnimFilename.GetString());

			//reset the modified time , since the anim will no longer match the original file. RAR 24/05/2006
			pxAnim->m_xModifiedTime.dwLowDateTime = 0;
			pxAnim->m_xModifiedTime.dwHighDateTime = 0;
		}
		else if(!s_pxThis->m_bWarnedAboutCompressedFrames)
		{
			Tools_Timed_Message_System::AddMessage(0xffff00,"This animation has compressed frames. So any changes won't be saved.");
			s_pxThis->m_bWarnedAboutCompressedFrames = true;	
		}
	}
}

void Tools_Mode_Hierarchy::HandleKeyboard()
{
	if(Asura_Overlay_System::OverlaysHaveKeyboardFocus()) return;
 	if(Asura_UserInput::KeyPressed(ASURA_KEYID_ALT)) return;

	//disable use of number keys while volume edit object is selected , to avoid clash of key useage
	bool bDisableNumeric = false;
	if(Tools_Object_System::SelectedObject)
	{
		if(	Tools_Object_System::SelectedObject->Type == TOT_VolumeEdit || 
			Tools_Object_System::SelectedObject->Type == TOT_VolumeGroupEdit || 
			Tools_Object_System::SelectedObject->Type == TOT_SectionBoundingBox)
		{
			bDisableNumeric = true;
		}
	}

	if(Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
 	{
		if ( Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT) )
		{
			if ( Asura_UserInput::DebouncedKeyPressed( 's' ) ) 
			{
				m_bDrawSkeleton ^= true;
			}
		}
		else
		{
			if ( Asura_UserInput::DebouncedKeyPressed( 'x' ) ) 
			{
				m_bShadowed ^= true;
			}
		
			if ( Asura_UserInput::DebouncedKeyPressed( 'w' ) ) 
			{
				m_bWireframeMesh ^= true;
			}
			if ( Asura_UserInput::DebouncedKeyPressed( 'p' ) ) 
			{
				m_bDrawMarkerPointAxes ^= true;
			}
			if (Asura_UserInput::DebouncedKeyPressed(','))
			{
				PreviousSkin();
			}
			if (Asura_UserInput::DebouncedKeyPressed('.'))
			{
				NextSkin();
			}

		}
	}
	else if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
	{
		if ( Asura_UserInput::DebouncedKeyPressed( 'p' ) ) 
		{
			m_bShowMarkerPoints ^= true;
		}
		if ( Asura_UserInput::DebouncedKeyPressed( 'c' ) ) 
		{
			m_bDrawCollisionMesh ^= true;
		}

		if(!bDisableNumeric)
		{
			for(int i=0;i<10;i++)
			{
				if(Asura_UserInput::DebouncedKeyPressed('0'+i))
				{
					StoreAnimation(i);
					return;
				}
			}
		}
		if (Asura_UserInput::DebouncedKeyPressed(','))
		{
			NextSecondaryAnim(-1);
		}
		if (Asura_UserInput::DebouncedKeyPressed('.'))
		{
			NextSecondaryAnim(1);
		}
		
	}
	else
	{
		if(Asura_UserInput::DebouncedKeyPressed(';'))
		{
			AnimationControl_FrameBack();
		}
		if(Asura_UserInput::DebouncedKeyPressed('\''))
		{
			AnimationControl_FrameForward();
		}
		if ( Asura_UserInput::DebouncedKeyPressed( 'p' ) ) 
		{
			m_bDrawMesh ^= true;
		}
		if ( Asura_UserInput::DebouncedKeyPressed( 'l' ) ) 
		{
			m_bShowLights ^= true;
		}
		if ( Asura_UserInput::DebouncedKeyPressed( 'o' ) ) 
		{
			m_bUseMovementOffsets ^= true;
		}
		if ( Asura_UserInput::DebouncedKeyPressed( 'b' ) ) 
		{
			m_bShowSectionBoundingBoxes ^= true;
		}
		if ( Asura_UserInput::DebouncedKeyPressed( 't' ) ) 
		{
			m_bShowTreadmill ^= true;
		}
		if (Asura_UserInput::DebouncedKeyPressed(','))
		{
			PreviousAnimation();
		}
		if (Asura_UserInput::DebouncedKeyPressed('.'))
		{
			NextAnimation();
		}
		if ( Asura_UserInput::DebouncedKeyPressed( 'm' ) ) 
		{
			Tools_Object_HierarchyCompass::s_bShow ^= true;
		}
		
		if(!bDisableNumeric)
		{
			for(int i=0;i<10;i++)
			{
				if(Asura_UserInput::DebouncedKeyPressed('0'+i))
				{
					RestoreAnimation(i);
					return;
				}
			}
		}


		if ( Asura_UserInput::DebouncedKeyPressed( ASURA_KEYID_ENTER ) )
		{
 			// start / pause animation
			if ( m_fAnimationControl_Speed == 0.0f )
			{
				AnimationControl_Play();
			}
			else
			{
				AnimationControl_Pause();
			}
		}
		if ( Asura_UserInput::DebouncedKeyPressed( ' ' ) )
		{
			// pause / reset animation
			if ( m_fAnimationControl_Speed == 0.0f )
			{
				{
					// Reset the animation to beginning
					m_fAnimationControl_Timer = 0.0f;
					m_bAnimationControl_TimerChanged = true;
				}
			}
			else
			{
				AnimationControl_Pause();
			}
		}

		if(Asura_UserInput::DebouncedKeyPressed(ASURA_KEYID_ESCAPE))
		{
			CancelLODBatch();
		}
	}
}


void Tools_Mode_Hierarchy::ToggleHierarchyOverlay(void)
{
	HierarchyOverlayOpen = !HierarchyOverlayOpen;

	if (HierarchyOverlayOpen)
	{
		CreateHierarchyOverlay();
	}
	else
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_MODE);
	}
}
bool Tools_Mode_Hierarchy::HierarchyOverlayIsOn(void)
{
	return HierarchyOverlayOpen;
}

void Tools_Mode_Hierarchy::CreateHierarchyOverlay(void)
{
	if(!HierarchyOverlayOpen) return;

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Hierarchy Mode",false,&HierarchyOverlayPosition);
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	char acText[200];

	if(SkinName.GetLength())
		Asura_PrintfToBfr(acText,"Skin : %s",sizeof(acText),true,SkinName.GetString());
	else
		strcpy(acText,"Skin : *None*");


	pxDialog = new Asura_Function_Dialog(acText,xOffset,_ChooseSkin);
	pxDialog->SetHelpText("Choose skin to edit/view.");
	pxDialog->m_iControlID = DialogID_Skin;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	#if ASURA_ANIMATION_SKIN_COUNT > 1
	for(int i=1; i<ASURA_ANIMATION_SKIN_COUNT; i++)
	{
		Asura_PrintfToBfr(acText, "Skin %d : ", sizeof(acText), true, i);
		pxDialog = new Tools_Dialog_PickSkin(acText, xOffset, &m_puSkinID[i], &m_bSkinSelectionChanged);
		pxDialog->SetHelpText("Set additional skins to use for multi-part skins.\nThis is only used if a primary skin has been selected above.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}
	#endif


	if(AnimName.GetLength())
		Asura_PrintfToBfr(acText,"Anim : %s",sizeof(acText), true, AnimName.GetString());
	else
		strcpy(acText,"Anim : *None*");


	pxDialog = new Asura_Function_Dialog(acText,xOffset,_ChooseAnim);
	pxDialog->SetHelpText("Choose animation to edit/view.");
	pxDialog->m_iControlID = DialogID_Anim;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_Function_Dialog("Add Light",xOffset,_CreateLight);
	pxDialog->SetHelpText("Creates a new temporary light for viewing the hierarchy/skin");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Restore Attachments",xOffset,_RestoreAttachments);
	pxDialog->SetHelpText("Restore a previously saved set of attachments.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Save Attachments",xOffset,_SaveAttachments);
	pxDialog->SetHelpText("Save the current marker point/attachment configuration , for later use in this mode.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show Marker Points (SHIFT+p)",xOffset,&m_bShowMarkerPoints);
	pxDialog->SetHelpText("Create marker point objects for all marker points in the skin.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show Marker Points as Axes (CTRL+p)",xOffset,&m_bDrawMarkerPointAxes);
	pxDialog->SetHelpText("Draw the axes for all marker points in the skin.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show Skin/Anim Properties",xOffset,&m_bShowSkinAndAnimProperties );
	pxDialog->SetHelpText("Displays various bits of read-only information about the skin and animation in the corner of the screen.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show Compass (m)",xOffset,&Tools_Object_HierarchyCompass::s_bShow);
	pxDialog->SetHelpText("Creates a tools object that shows the compass point (the location of the hierarchy/skin).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

#if defined(ASURA_USE_INDEPENDENT_SHADOW_SYSTEM)
	// No real harm having this on if you don't have the shadow system, but you won't see any shadows so best
	// to disable it to avoid bug reports from old-gen Tools builds.
	pxDialog = new Asura_Tickbox_Dialog("Shadowed (CTRL+x)",xOffset,&m_bShadowed);
	pxDialog->SetHelpText("Enable self-shadowing on the hierarchy/skin.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
#endif
//	pxDialog = new Asura_Tickbox_Dialog("Wireframe (CTRL+w)",xOffset,&m_bWireframeMesh);
//	pxDialog->SetHelpText("Draw the mesh in wireframe.");
//	pxOverlay->AddDialog(pxDialog);
//	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show Mesh (p)",xOffset,&m_bDrawMesh );
	pxDialog->SetHelpText("Draw the mesh.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show Skeleton (CTRL+SHIFT+s)",xOffset,&m_bDrawSkeleton);
	pxDialog->SetHelpText("Draw the skeleton as lines between joints.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show Lights (l)",xOffset,&m_bShowLights);
	pxDialog->SetHelpText("Show the temporary light objects.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show Section Bounding Boxes (b)",xOffset,&m_bShowSectionBoundingBoxes);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Show Treadmill (t)",xOffset,&m_bShowTreadmill);
	pxDialog->SetHelpText("Show animating treadmill object , for setting the animation speeds.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Use Movement Offsets (o)",xOffset,&m_bUseMovementOffsets);
	pxDialog->SetHelpText("Makes use of movement offsets store with the animation if they exist (animation of the compass point in Max).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Use Compressed Keyframes",xOffset,&Asura_Hierarchy_Anim::s_bPreviewCompressedKeyframes);
	pxDialog->SetHelpText("Shows the animations using the compression that would be done during a level export.\nAnimations that have been flagged as 'Dont Compress keyframes' will ignore this.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Numerical_Dialog("Bones Per Vertex",xOffset,&m_iMaxBonesPerVertex,1,4,1,"%d");
	pxDialog->SetHelpText("This is for testing rendering a skin using fewer than 4 bones per vertex.\nThis will only affect how skins are rendered in this mode (so changes won't be made to the skin files).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Function_Dialog("Batch LOD Creation",xOffset,_StartLODBatch);
	pxDialog->SetHelpText("Initialise LOD models for multiple skin files , reading in the settings from a text file.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Check Anim BBs",xOffset,_CheckAnimBoundingBoxes);
	pxDialog->SetHelpText("Checks all animations in a chosen directory to see if they go beyond the extents of the current skin's bounding box.\nIf they do , an anim specific bounding box will be added.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Write AVI",xOffset,_WriteAVI);
	pxDialog->SetHelpText("Writes out one loop of the current animation as an AVI.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxOverlay->m_uID = TOOLS_OVERLAY_MODE;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}

void Tools_Mode_Hierarchy::ToggleAnimPropertiesOverlay(void)
{
	m_bAnimPropertiesOverlayOpen = !m_bAnimPropertiesOverlayOpen;

	if (m_bAnimPropertiesOverlayOpen)
	{
		CreateAnimPropertiesOverlay();
	}
	else
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ANIMPROPERTIES);
	}
}
bool Tools_Mode_Hierarchy::AnimPropertiesOverlayIsOn(void)
{
	return m_bAnimPropertiesOverlayOpen;
}

void Tools_Mode_Hierarchy::CreateAnimPropertiesOverlay(void)
{
	if(!m_bAnimPropertiesOverlayOpen) return;
	if(!Animation) return;

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Anim Properties",false,&m_xAnimPropertiesOverlayPosition);
	
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	AddAnimPropertiesOverlayControls(pxOverlay,xOffset);
			
	pxOverlay->m_uID = TOOLS_OVERLAY_ANIMPROPERTIES;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
 
}

void Tools_Mode_Hierarchy::AddAnimPropertiesOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;

	pxDialog = new Asura_Function_Dialog("Output Text File",xOffset,_OutputAnimHierarchyTextFile);
	pxDialog->SetHelpText("Creates a text file showing the ordering and hierarchy of the bones in the animation.");
	pxOverlay->AddDialog(pxDialog);						 
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Add Flag",xOffset,_AddAnimationFlag);
	pxDialog->SetHelpText("Add a new keyframe flag to the animation.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x+=12.5f;

	pxDialog = new Asura_Function_Dialog("Edit Flag",xOffset,_EditAnimationFlag);
	pxDialog->SetHelpText("Edit the keyframe flags.");
	pxOverlay->AddDialog(pxDialog);						 
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	xOffset.x = 1.0f;

	pxDialog = new Asura_Function_Dialog("Add Sound Marker",xOffset,_AddAnimationSound);
	pxDialog->SetHelpText("Add a new keyframe sound to the animation.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x+=12.5f;

	pxDialog = new Asura_Function_Dialog("Edit Sound Marker",xOffset,_EditAnimationSound);
	pxDialog->SetHelpText("Edit the keyframe sounds.");
	pxOverlay->AddDialog(pxDialog);						 
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	xOffset.x = 1.0f;

	pxDialog = new Asura_Function_Dialog("Edit Sound Marker Defns",xOffset,_EditAnimSoundDefns);
	pxDialog->SetHelpText("Edit the sound definitions for this anim.");
	pxOverlay->AddDialog(pxDialog);						 
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Edit Test Sound",xOffset,_EditTestSounds);
	pxDialog->SetHelpText("This allows a temporary sound to be associated with a keyframe flag.\nWhenever the flag is triggered in this mode , the sound will be played.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Duration (s)",xOffset,&AnimDuration,0,1000,0.1f,"%.3f",&ModifiedFlag);
	pxDialog->SetHelpText("Animation duration in seconds.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Speed (m/s)",xOffset,&TreadmillSpeed,0,1000,0.1f,"%.3f",&ModifiedFlag);
	pxDialog->SetHelpText("Animation speed in metres/second. (Default movement speed for anything using this anim).");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Direction (deg)",xOffset,&TreadmillDirection,0,359,1.0f,"%.0f",&ModifiedFlag);
	pxDialog->SetHelpText("Direction of movement. Probably not actually used.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Loop Point",xOffset,&m_fLoopPoint,0,1,0.01f,"%.3f",&ModifiedFlag);
	pxDialog->SetHelpText("Point that animation should loop back to when the end is reached (0 to 1).\nBy default this is 0 , which means the entire anim is looped.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Tickbox_Dialog("Use Anim's BB for Culling",xOffset,&m_bAnimHasBB,&ModifiedFlag);
	pxDialog->SetHelpText("Anything using this anim will be culled for rendering based on the extents of this animation.\nOtherwise the skin's rest pose bounding box is used.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Disable keyframe removal",xOffset,&Animation->m_bDontRemoveKeyframes,&ModifiedFlag);
	pxDialog->SetHelpText("Prevents the automatic removal of redundant keyframes.\nFor fixing animations where the small errors in position are noticable.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Disable keyframe compression",xOffset,&Animation->m_bDontCompressKeyframes,&ModifiedFlag);
	pxDialog->SetHelpText("Prevents compression of keyframes during level exports. This may fix cases where subtle orientation animations are being messed up.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

}

void Tools_Mode_Hierarchy::ToggleSkinPropertiesOverlay(void)
{
	m_bSkinPropertiesOverlayOpen = !m_bSkinPropertiesOverlayOpen;

	if (m_bSkinPropertiesOverlayOpen)
	{
		CreateSkinPropertiesOverlay();
	}
	else
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SKINPROPERTIES);
	}
}
bool Tools_Mode_Hierarchy::SkinPropertiesOverlayIsOn(void)
{
	return m_bSkinPropertiesOverlayOpen;
}

void Tools_Mode_Hierarchy::CreateSkinPropertiesOverlay(void)
{
	if(!m_bSkinPropertiesOverlayOpen) return;
	if(!m_pxSkin) return;

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Skin Properties",false,&m_xSkinPropertiesOverlayPosition);
	
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	AddSkinPropertiesOverlayControls(pxOverlay,xOffset);
		
	pxOverlay->m_uID = TOOLS_OVERLAY_SKINPROPERTIES;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
 
}

void Tools_Mode_Hierarchy::AddSkinPropertiesOverlayControls(class Asura_Overlay* pxOverlay,class Asura_Vector_2& xOffset)
{
	Asura_Dialog* pxDialog;

	pxDialog = new Asura_Function_Dialog("Output Text File",xOffset,OutputSkinHierarchyTextFile);
	pxDialog->SetHelpText("Creates a text file showing the ordering and hierarchy of the bones in the skin.");
	pxOverlay->AddDialog(pxDialog);						 
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Output Basic Text File",xOffset,OutputBasicSkinHierarchyTextFile);
	pxDialog->SetHelpText("Creates a text file showing the ordering and hierarchy of the bones in the skin.\nThis creates a cut down version of the file suitable for use with Anim_Hierarchy_Conversion.exe.");
	pxOverlay->AddDialog(pxDialog);						 
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Draw Collision Mesh (C)",xOffset,&m_bDrawCollisionMesh );
	pxDialog->SetHelpText("Draw the collision mesh if it exists");
	pxDialog->SetIsEnabledFunction( _CanDrawCollisionMesh );
	pxOverlay->AddDialog(pxDialog);						 
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Recalc. Section Coll. Mesh",xOffset,_RecalculateSectionCollisionMesh,NULL,_IsObjectHierarchy);
	pxDialog->SetHelpText("Forces the section collision meshes to be recalculated from their meshes.\nOnly applies to object hierarchies.");
	pxOverlay->AddDialog(pxDialog);						 
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Reset Collision BB",xOffset,_ResetCollisionBB,NULL,_CanEditCollisionBB);
	pxDialog->SetHelpText("Removes the custom collision bounding box.");
	pxOverlay->AddDialog(pxDialog);						 
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Edit Collision BB",xOffset,_EditCollisionBB,NULL,_CanEditCollisionBB);
	pxDialog->SetHelpText("Set a custom collision bounding box.");
	pxOverlay->AddDialog(pxDialog);						 
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Edit Sound Marker Defns",xOffset,_EditSkinSoundDefns);
	pxDialog->SetHelpText("Edit the sounds attached to this skin.");
	pxOverlay->AddDialog(pxDialog);						 
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Create Marker Point",xOffset,_CreateMarkerPoint);
	pxDialog->SetHelpText("Create a new marker point for this skin.");
	pxOverlay->AddDialog(pxDialog);						 
	xOffset.y+=Asura_Dialog::DefaultSpacing();

}




void Tools_Mode_Hierarchy::ToggleAnimationControlOverlay(void)
{
	AnimationControlOverlayOpen = !AnimationControlOverlayOpen;

	if (AnimationControlOverlayOpen)
	{
		CreateAnimationControlOverlay();
	}
	else
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ANIMATIONPLAY);
	}
}
bool Tools_Mode_Hierarchy::AnimationControlOverlayIsOn(void)
{
	return AnimationControlOverlayOpen;
}

void Tools_Mode_Hierarchy::CreateAnimationControlOverlay(void)
{
	if(!AnimationControlOverlayOpen) return;

	Asura_Overlay* pxOverlay;
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset;

	//create an overlay at the bottom of the screen for controlling animation playback
	AnimationControlOverlayPosition.x = 0;
	AnimationControlOverlayPosition.y = (Asura_Viewport::GetHeight()/14.0f)-4.0f;
	pxOverlay = new Asura_Overlay("Animation Control",false,&AnimationControlOverlayPosition);
	pxOverlay->m_bOnPanel = true;

	
	//create the time slider control
	xOffset.x = 0;
	xOffset.y = 0;
	float slideWidth = (Asura_Viewport::GetWidth()/10.0f);
	pxDialog = new Asura_NumberSlider_Dialog(xOffset,&m_fAnimationControl_SlideTimer,&m_fTimeSliderMin,&m_fTimeSliderMax,slideWidth,"%.3f",&m_bAnimationControl_SlideTimerChanged);
	pxDialog->m_iControlID = DialogID_TimeSlider;
	pxOverlay->AddDialog(pxDialog);
	
	xOffset.x = 1.0f;
	xOffset.y = 1.2f;
	
	//values for the buttons
	Asura_Vector_2 ButtonSizes[] =
	{
		//Asura_Vector_2(30.0f,24.0f),		
		Asura_Vector_2(24.0f,24.0f),		
		Asura_Vector_2(24.0f,24.0f),		
		Asura_Vector_2(24.0f,24.0f),		
		//Asura_Vector_2(24.0f,24.0f),		
		Asura_Vector_2(24.0f,24.0f),		
		//Asura_Vector_2(30.0f,24.0f),		
	};
	Asura_Vector_2 ButtonPositions[] =
	{
		//Asura_Vector_2(0.0f,0.0f),		
		Asura_Vector_2(30.0f,0.0f),		
		Asura_Vector_2(54.0f,0.0f),		
		Asura_Vector_2(78.0f,0.0f),		
		//Asura_Vector_2(102.0f,0.0f),		
		Asura_Vector_2(126.0f,0.0f),		
		//Asura_Vector_2(150.0f,0.0f),		
	};

	void (*ButtonFunctions[])(void) =
	{
		//_AnimationControl_Rewind,
		_AnimationControl_FrameBack,
		_AnimationControl_Play,
		_AnimationControl_Pause,
		//_AnimationControl_Stop,
		_AnimationControl_FrameForward,
		//_AnimationControl_FastForward,
	};
	
	static int ButtonsTextureIndex=-1;
	if(ButtonsTextureIndex==-1)
	{
		ButtonsTextureIndex = Asura_Texture_Management::AddTexture("\\tools\\buttons.bmp");
		Asura_Texture_Management::LoadTextures();
	}
	
	
	//create all the buttons

	for(int i=0;i<4;i++)
	{
		pxDialog = new Asura_Button_Dialog(xOffset,ButtonsTextureIndex,256,ButtonPositions[i],ButtonSizes[i],ButtonFunctions[i]);
		pxOverlay->AddDialog(pxDialog);
		xOffset.x += ButtonSizes[i].x/10.0f;
	}

	xOffset.y+=0.5f;
	xOffset.x+=2.0f;

	// Use scaled numeric dialog and fLargestAnimEnd for timer just to make it same size as scaled timer dialog
	float fLargestAnimEnd=1.0f;
	if(AnimDuration>fLargestAnimEnd) fLargestAnimEnd=AnimDuration;
	if(static_cast<float>(m_iAnimFrames)>fLargestAnimEnd) fLargestAnimEnd=static_cast<float>(m_iAnimFrames);
	pxDialog = new Asura_ScaledNumerical_Dialog("Timer (0 to 1)",xOffset,1.0f,&m_fAnimationControl_Timer,0.0f,fLargestAnimEnd,0.01f,"%.3f",&m_bAnimationControl_TimerChanged);
	pxDialog->m_iControlID = DialogID_UnScaledTimer;
	pxOverlay->AddDialog(pxDialog);

	xOffset.y+=Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_ScaledNumerical_Dialog("Should never see this!",xOffset,1.0f,&m_fAnimationControl_Timer, 0.0f, fLargestAnimEnd,0.01f,"%0.3f",&m_bAnimationControl_TimerChanged);
	pxDialog->SetHelpText("Should never see this!");
	pxDialog->m_iControlID = DialogID_ScaledTimer;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y-=Asura_Dialog::DefaultSpacing();

	xOffset.x+=20;
	
	pxDialog = new Asura_Function_Dialog("",xOffset,_AnimationControl_SetSliderMode);
	pxDialog->SetHelpText(	"Change the time bar above to one of these modes :\n"
							"Frames - Displays frame index numbers\n"
							"Seconds - Displays time in seconds\n"
							"Proportion - Goes from 0 to 1\n"
							"25 FPS - In frames, assuming 25 frames per second\n"
							"30 FPS - In frames, assuming 30 frames per second");
	pxDialog->m_iControlID = DialogID_SliderMode;
	pxOverlay->AddDialog(pxDialog);

	xOffset.x+=20;

	float fX1 = xOffset.x;
	pxDialog = new Asura_FlagTickbox_Dialog("Reverse",xOffset,&m_uPrimaryAnimFlags,AsuraAnimationFlag_Reversed,&m_bAnimFlagsChanged);
	pxDialog->SetHelpText("Reverses the primary anim.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;


	pxDialog = new Asura_FlagTickbox_Dialog("Mirrored",xOffset,&m_uPrimaryAnimFlags,AsuraAnimationFlag_Mirror,&m_bAnimFlagsChanged);
	pxDialog->SetHelpText("Mirrors the primary anim.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = pxDialog->m_xMaxCoord.x + 2.0f;
	
	
	pxDialog = new Asura_Function_Dialog("Add Secondary Anim",xOffset,_AddSecondaryAnim);
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x = fX1;
	pxDialog = new Asura_FlagTickbox_Dialog("Loop",xOffset,&m_uPrimaryAnimFlags,AsuraAnimationFlag_Loop,&m_bAnimFlagsChanged);
	pxDialog->SetHelpText("Loop the primary anim.");
	pxOverlay->AddDialog(pxDialog);

	


	//submit the overlay
	pxOverlay->m_uID = TOOLS_OVERLAY_ANIMATIONPLAY;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false, true, false);

	UpdateAnimationControlOverlay();
}

void Tools_Mode_Hierarchy::UpdateAnimationControlOverlay()
{
	static char szPrintFormat[40];

	//find the overlay
	Asura_Overlay* const pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_ANIMATIONPLAY);
	if(!pxOverlay) return;

	//find the time slider control
	Asura_NumberSlider_Dialog* const pxSliderDialog = (Asura_NumberSlider_Dialog*)pxOverlay->GetDialog(DialogID_TimeSlider);
	if(!pxSliderDialog) return;

	switch(m_eAnimationControl_TimerMode)
	{
		case TIMER_PROP:
		{
			strcpy(szPrintFormat,"%.3f");
			m_fTimeSliderMax = 1.0f;
			break;
		}
		case TIMER_SECONDS:
		{
			strcpy(szPrintFormat,"%.3f");
			m_fTimeSliderMax = AnimDuration;
			break;
		}
		case TIMER_FRAMES:
		{
			sprintf(szPrintFormat,"%%.0f\\%d",m_iAnimFrames);
			m_fTimeSliderMax = (float)m_iAnimFrames;
			break;
		}
		case TIMER_25FPS:
		{
			strcpy(szPrintFormat,"%.0f");
			m_fTimeSliderMax = (float)Asura_Maths::FloatToInt_Chop(AnimDuration*25.0f);
			if(m_fTimeSliderMax < AnimDuration*25.0f) m_fTimeSliderMax ++;
			break;
		}
		case TIMER_30FPS:
		{
			strcpy(szPrintFormat,"%.0f");
			m_fTimeSliderMax = (float)Asura_Maths::FloatToInt_Chop(AnimDuration*30.0f);
			if(m_fTimeSliderMax < AnimDuration*30.0f) m_fTimeSliderMax++;
			break;
		}
	}
	pxSliderDialog->SetPrintFormat(szPrintFormat);

	//find the time slider mode dialog
	Asura_Dialog* const pxModeDialog = pxOverlay->GetDialog(DialogID_SliderMode);
	if(!pxModeDialog) return;

	Asura_String xLabel( "Timer Bar : " );
	xLabel += s_aszTimerModeStrings[m_eAnimationControl_TimerMode];
	pxModeDialog->UpdateLabel( xLabel.GetString() );

	Asura_ScaledNumerical_Dialog* const pxUnScaledTimerDialog = reinterpret_cast<Asura_ScaledNumerical_Dialog*>( pxOverlay->GetDialog( DialogID_UnScaledTimer ) );
	if(!pxUnScaledTimerDialog) return;
	pxUnScaledTimerDialog->SetScale(1.0f, 0.0f, 1.0f);
	pxUnScaledTimerDialog->SetDelta( 0.01f );

	Asura_ScaledNumerical_Dialog* const pxScaledTimerDialog = reinterpret_cast<Asura_ScaledNumerical_Dialog*>( pxOverlay->GetDialog( DialogID_ScaledTimer ) );
	if(!pxScaledTimerDialog) return;

	xLabel = "Time (";
	xLabel += s_aszTimerModeStrings[m_eAnimationControl_TimerMode];
	xLabel += ")";
	pxScaledTimerDialog->UpdateLabel( xLabel.GetString() );
	float fDelta = 0.01f;
	float fScale = 0.0f;
	float fMin = 0.0f;
	float fMax = 0.0f;
	const char* szHelpText = "Should never see this";
	switch(m_eAnimationControl_TimerMode)
	{
		case TIMER_PROP:
		{
			szHelpText = "Parametric Time (0 to 1)";
			fScale = 1.0f;
			fMin = 0.0f;
			fMax = 1.0f;
			pxScaledTimerDialog->SetScale(1.0f, 0.0f, 1.0f);
			fDelta = 0.01f;
			strcpy( szPrintFormat,"%.3f" );
			break;
		}
		case TIMER_SECONDS:
		{
			szHelpText = "Time in seconds";
			fScale = AnimDuration;
			fMin = 0.0f;
			fMax = AnimDuration;
			pxScaledTimerDialog->SetScale(AnimDuration, 0.0f, AnimDuration);
			fDelta = 0.01f;
			strcpy( szPrintFormat,"%.3f" );
			break;
		}
		case TIMER_FRAMES:
		{
			szHelpText = "Time in frames";
			fScale = static_cast<float>(m_iAnimFrames); 
			fMin = 0.0f; 
			fMax = static_cast<float>(m_iAnimFrames);
			fDelta = 1.0f;
			strcpy( szPrintFormat,"%.1f" );
			break;
		}

		case TIMER_25FPS:
		{
			szHelpText = "Time in frames at 25 FPS";
			fScale = m_fTimeSliderMax; 
			fMin = 0.0f; 
			fMax = m_fTimeSliderMax;
			fDelta = 1.0f;
			strcpy( szPrintFormat,"%.1f" );
			break;
		}

		case TIMER_30FPS:
		{
			szHelpText = "Time in frames at 30 FPS";
			fScale = m_fTimeSliderMax; 
			fMin = 0.0f; 
			fMax = m_fTimeSliderMax;
			fDelta = 1.0f;
			strcpy( szPrintFormat,"%.1f" );
			break;
		}
	}
	// Don't let the box get smaller
	pxScaledTimerDialog->SetPrintFormat( szPrintFormat );
	pxScaledTimerDialog->SetHelpText( szHelpText );
	pxScaledTimerDialog->SetScale( fScale, fMin, fMax );
	pxScaledTimerDialog->SetDelta( fDelta );
}

void Tools_Mode_Hierarchy::ToggleHierarchyLODOverlay(void)
{
	HierarchyLODOverlayOpen = !HierarchyLODOverlayOpen;

	if (HierarchyLODOverlayOpen)
	{
		CreateHierarchyLODOverlay();
	}
	else
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_HIERARCHYLOD);
	}
}

bool Tools_Mode_Hierarchy::HierarchyLODOverlayIsOn(void)
{
	return HierarchyLODOverlayOpen;
}

void Tools_Mode_Hierarchy::CreateHierarchyLODOverlay(void)
{
	if(!HierarchyLODOverlayOpen) return;

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Level of Detail",false,&HierarchyLODOverlayPosition);
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	//add the headings
	pxDialog = new Asura_Dialog("Detail",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x += 12.0f;

	pxDialog = new Asura_Dialog("Distance",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x -= 12.0f;
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	for(int i = 0; i < TOOLS_MAX_LOD_MODELS; i++)
	{
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_afLODPercent[i],0,1.0f,0.01f,"%.3f",&LODModified);
		pxDialog->SetHelpText("Proportion of original polygons that should be used by this detail level.");
		pxDialog->SetIsEnabledFunction(_CanEditLOD);
		pxOverlay->AddDialog(pxDialog);
		xOffset.x+=12.0f;
		
		pxDialog = new Asura_Numerical_Dialog("",xOffset,&m_afLODDistances[i],0,999.0f,1.0f,"%.1f",&LODModified);
		pxDialog->SetHelpText("Minimum distance at which this detail level should be used.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.x-=12.0f;

		xOffset.y+=Asura_Dialog::DefaultSpacing();

	}


//	pxDialog = new Asura_Function_Dialog("Generate Models",xOffset,_GenerateLODModels);
//	pxOverlay->AddDialog(pxDialog);

	pxOverlay->m_uID = TOOLS_OVERLAY_HIERARCHYLOD;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}


bool Tools_Mode_Hierarchy::CanEditLOD()
{
	return m_pxSkin ? !m_pxSkin->m_bManualLOD : false;
}


void Tools_Mode_Hierarchy::ToggleFrameReductionOverlay(void)
{
	FrameReductionOverlayOpen = !FrameReductionOverlayOpen;

	if (FrameReductionOverlayOpen)
	{
		CreateFrameReductionOverlay();
	}
	else
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ANIMATIONFRAMEREDUCTION);
	}
}
bool Tools_Mode_Hierarchy::FrameReductionOverlayIsOn(void)
{
	return FrameReductionOverlayOpen;
}

void Tools_Mode_Hierarchy::CreateFrameReductionOverlay(void)
{
	if(!FrameReductionOverlayOpen) return;

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Frame Removal Preview",false,&FrameReductionOverlayPosition);
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);


	pxDialog = new Asura_Tickbox_Dialog("Enable",xOffset,&m_bDoFrameRemoval,&m_bUpdateFrameRemoval);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Position Tol. (m)",xOffset,&m_fPositionTolerance,0,1.0f,0.0001f,"%.4f",&m_bUpdateFrameRemoval);
	pxDialog->m_iControlID = DialogID_PositionTolerance;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Angle Tol. (deg)",xOffset,&m_fAngleTolerance,0,180.0f,0.01f,"%.2f",&m_bUpdateFrameRemoval);
	pxDialog->m_iControlID = DialogID_QuatTolerance;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


			
	pxOverlay->m_uID = TOOLS_OVERLAY_ANIMATIONFRAMEREDUCTION;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}

void Tools_Mode_Hierarchy::UpdateFrameReductionOverlay()
{
	if(!FrameReductionOverlayOpen) return;

	//find the overlay
	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_ANIMATIONFRAMEREDUCTION);
	if(!pxOverlay) return;

	//update the enabled status of the controls
	Asura_Dialog* pxDialog = pxOverlay->GetDialog(DialogID_PositionTolerance);
	if(pxDialog) pxDialog->SetEnabled(m_bDoFrameRemoval);

	pxDialog = pxOverlay->GetDialog(DialogID_QuatTolerance);
	if(pxDialog) pxDialog->SetEnabled(m_bDoFrameRemoval);

}



void Tools_Mode_Hierarchy::UpdateAnimationFlagOverlay()
{
	AnimationFlagOverlayNeedsUpdate = false;
	
	//destroy the existing overlay (if any)
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ANIMATIONFLAG);
	
	if(!Animation || !Animation->NumberOfAnimationFlags) return;
	//make sure the index is in range
	int index = (int)SelectedIntervalIndex;
	if(index>=Animation->NumberOfAnimationFlags)
	{
		index = Animation->NumberOfAnimationFlags-1;
		SelectedIntervalIndex = (float)index;
	}

	Asura_Animation_Flag_Interval* flagInterval = &Animation->AnimationFlags[index];

	if(SelectedIntervalIndexChanged)
	{
		//work out whether this flag interval is an interval or instantaneous
		FlagCoversInterval = (flagInterval->StartTime != flagInterval->EndTime);
	}

	//create the overlay
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Animation Flag",false,&AnimationFlagOverlayPosition);

	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	pxDialog = new Asura_Numerical_Dialog("Selected Flag",xOffset,&SelectedIntervalIndex,0,(float)(Animation->NumberOfAnimationFlags-1),1,"%.0f",&SelectedIntervalIndexChanged);
	pxDialog->SetHelpText("The index of selected keyframe flag.\nUsed for changing the current selection.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	

	pxDialog = new Asura_Tickbox_Dialog("Interval",xOffset,&FlagCoversInterval,&AnimationFlagOverlayNeedsUpdate);
	pxDialog->SetHelpText("If set then this flag should cover a period of the animation.\nOtherwise it covers a single point in time.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
																			 
	float fLargestAnimEnd=1.0f;
	if(AnimDuration>fLargestAnimEnd) fLargestAnimEnd=AnimDuration;
	if(static_cast<float>(m_iAnimFrames)>fLargestAnimEnd) fLargestAnimEnd=static_cast<float>(m_iAnimFrames);

	if ( FlagCoversInterval )
	{
		pxDialog = new Asura_ScaledNumerical_Dialog("Should never see this!",xOffset,1.0f,&flagInterval->StartTime, 0.0f, fLargestAnimEnd,0.01f,"%0.3f",&FlagStartChanged);
		pxDialog->SetHelpText("Should never see this!");
		pxDialog->m_iControlID = DialogID_FlagStartTime;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_ScaledNumerical_Dialog("Should never see this!",xOffset,1.0f,&flagInterval->EndTime, 0.0f, fLargestAnimEnd,0.01f,"%0.3f",&FlagEndChanged);
		pxDialog->SetHelpText("Should never see this!");
		pxDialog->m_iControlID = DialogID_FlagEndTime;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

	}
	else
	{
		if(flagInterval->EndTime != flagInterval->StartTime)
		{
			//This should cause the animation to be flagged as modified when the interval tickbox becomes unticked.
			ModifiedFlag = true;
		}
		
		flagInterval->EndTime = flagInterval->StartTime;
		
		pxDialog = new Asura_ScaledNumerical_Dialog("Should never see this!",xOffset,1.0f,&flagInterval->StartTime, 0.0f, fLargestAnimEnd,0.01f,"%0.3f",&FlagStartChanged);
		pxDialog->SetHelpText( "Should never see this!" );
		pxDialog->m_iControlID = DialogID_FlagStartTime;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}

	//add checkboxes for the flags that have been set
	for(int bit=0;bit<32;bit++)
	{
		if(flagInterval->Flag & (1<<bit))
		{
			if(AnimationFlagList)
			{
				pxDialog = new Asura_FlagTickbox_Dialog(AnimationFlagList->GetFlagName(1<<bit),xOffset,&flagInterval->Flag,1<<bit,&ModifiedFlag);
			}
			else
			{
				char flagName[200];
				sprintf(flagName,"Flag %d",bit);
				pxDialog = new Asura_FlagTickbox_Dialog(flagName,xOffset,&flagInterval->Flag,1<<bit,&ModifiedFlag);
				pxDialog->SetHelpText("Flag to be triggered at this time.");
			}
			pxOverlay->AddDialog(pxDialog);
			xOffset.y+=Asura_Dialog::DefaultSpacing();
		}
	}

	xOffset.y+=0.3f;

	pxDialog = new Asura_Function_Dialog("Set Flags",xOffset,_SetAnimationFlag);
	pxDialog->SetHelpText("Add a new flag to the selected keyframe flag.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	// Put a divider in the dialog
	pxDialog = new Asura_Divider_Dialog( xOffset );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_Function_Dialog("Add Flag", xOffset,_AddAnimationFlag);
	pxDialog->SetHelpText("Add a new keyframe flag.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x += pxDialog->GetMaxCoords().x;
	
	pxDialog = new Asura_Function_Dialog("Delete Flag",xOffset,_DeleteAnimationFlag);
	pxDialog->SetHelpText("Delete this keyframe flag.");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxOverlay->m_uID = TOOLS_OVERLAY_ANIMATIONFLAG;
	pxOverlay->m_xOrigin = AnimationFlagOverlayPosition;
 	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);

	//reset the various 'changed' flags
	FlagStartChanged = false;
	FlagEndChanged = false;
	SelectedIntervalIndexChanged = false;

	UpdateAnimationFlagTimeDisplay();
}

void Tools_Mode_Hierarchy::UpdateAnimationFlagTimeDisplay()
{
	Asura_Overlay* const pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_ANIMATIONFLAG);

	if ( pxOverlay )
	{
		// If the endtime dialog exists then this is an interval flag
		Asura_ScaledNumerical_Dialog* const pxEndDialog = ( FlagCoversInterval ) ? reinterpret_cast<Asura_ScaledNumerical_Dialog*>( pxOverlay->GetDialog(DialogID_FlagEndTime) ) : NULL;
		Asura_ScaledNumerical_Dialog* const pxStartDialog = reinterpret_cast<Asura_ScaledNumerical_Dialog*>(pxOverlay->GetDialog(DialogID_FlagStartTime));

		if ( pxStartDialog )
		{
			Asura_String xLabel( "Time (" );
			xLabel += s_aszTimerModeStrings[m_eAnimationControl_TimerMode];
			xLabel += ")";
			if ( pxEndDialog == NULL )
			{
				pxStartDialog->UpdateLabel( xLabel.GetString() );
			}
			else
			{
				Asura_String xText;
				xText = "Start ";
				xText += xLabel;
				pxStartDialog->UpdateLabel( xText.GetString() );
				xText = "End ";
				xText += xLabel;
				pxEndDialog->UpdateLabel( xText.GetString() );
			}
			switch(m_eAnimationControl_TimerMode)
			{
				case TIMER_PROP:
				{
					pxStartDialog->SetScale(1.0f, 0.0f, 1.0f);
					pxStartDialog->SetDelta( 0.01f );
					if ( pxEndDialog == NULL )
					{
						pxStartDialog->SetHelpText("Time (0 to 1) when this flag should be triggered");
					}
					else
					{
						pxStartDialog->SetHelpText("The interval start time (0 to 1) for triggering this flag");
						pxEndDialog->SetHelpText("The interval end time (0 to 1) for triggering this flag");
						pxEndDialog->SetScale(1.0f, 0.0f, 1.0f);
						pxEndDialog->SetDelta( 0.01f );
					}
					break;
				}

				case TIMER_SECONDS:
				{
					pxStartDialog->SetScale(AnimDuration, 0.0f, AnimDuration);
					pxStartDialog->SetDelta( 0.01f );
					if ( pxEndDialog == NULL )
					{
						pxStartDialog->SetHelpText("Time in seconds when this flag should be triggered");
					}
					else
					{
						pxStartDialog->SetHelpText("The interval start time in seconds for triggering this flag");
						pxEndDialog->SetHelpText("The interval end time in seconds for triggering this flag");
						pxEndDialog->SetScale(AnimDuration, 0.0f, AnimDuration);
						pxEndDialog->SetDelta( 0.01f );
					}
					break;
				}

				case TIMER_FRAMES:
				{
					pxStartDialog->SetScale(static_cast<float>(m_iAnimFrames), 0.0f, static_cast<float>(m_iAnimFrames));
					pxStartDialog->SetDelta( 1.00f );
					if ( pxEndDialog == NULL )
					{
						pxStartDialog->SetHelpText("Time in frames when this flag should be triggered");
					}
					else
					{
						pxStartDialog->SetHelpText("The interval start time in frames for triggering this flag");
						pxEndDialog->SetHelpText("The interval end time in frames for triggering this flag");
						pxEndDialog->SetScale(static_cast<float>(m_iAnimFrames), 0.0f, static_cast<float>(m_iAnimFrames));
						pxEndDialog->SetDelta( 1.00f );
					}
					break;
				}

				case TIMER_25FPS:
				case TIMER_30FPS:
				{
					pxStartDialog->SetScale(m_fTimeSliderMax, 0.0f, m_fTimeSliderMax);
					pxStartDialog->SetDelta( 1.00f );
					if ( pxEndDialog == NULL )
					{
						pxStartDialog->SetHelpText("Time in frames when this flag should be triggered");
					}
					else
					{
						pxStartDialog->SetHelpText("The interval start time in frames for triggering this flag");
						pxEndDialog->SetHelpText("The interval end time in frames for triggering this flag");
						pxEndDialog->SetScale(m_fTimeSliderMax, 0.0f, m_fTimeSliderMax);
						pxEndDialog->SetDelta( 1.00f );
					}
					break;
				}

			}
		}
	}
}

void Tools_Mode_Hierarchy::UpdateAnimationSoundOverlay()
{
	
	//destroy the existing overlay (if any)
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ANIMATIONSOUND);
	
	if(!Animation || !Animation->m_iNumberOfAnimationSounds) return;

	//make sure the index is in range
	if(m_iSelectedSoundIndex>=Animation->m_iNumberOfAnimationSounds)
	{
		m_iSelectedSoundIndex = Animation->m_iNumberOfAnimationSounds-1;
	}

	if(!m_pxAnimationSoundList)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Unable to find list of animation sound IDs");
		return;
	}

	Asura_Animation_Sound* pxSound = &Animation->m_pxAnimationSounds[m_iSelectedSoundIndex];

	m_bSoundLooped = pxSound->m_fStartTime != pxSound->m_fEndTime;
	

   	m_bSelectedSoundIndexChanged = false;



	//create the overlay
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Animation Sound Markers",false,&AnimationSoundOverlayPosition);

	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	pxDialog = new Asura_Numerical_Dialog("Selected Sound",xOffset,&m_iSelectedSoundIndex,0,Animation->m_iNumberOfAnimationSounds-1,1,"%d",&m_bSelectedSoundIndexChanged);
	pxDialog->SetHelpText("The index of the selected keyframe sound.\nUsed for changing the current selection.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_FlagTickbox_Dialog("Follow Skin", xOffset, &pxSound->m_uFlags, ASURA_ANIMATIONSOUNDFLAG_FOLLOWSKIN, &ModifiedFlag);
	pxDialog->SetHelpText("Causes the sound position to be updated each frame to follow the skin.\nThis is effectively always on for dialogue and looped sounds.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Looped",xOffset,&m_bSoundLooped,&m_bSoundLoopedChanged);
	pxDialog->SetHelpText("If set then the sound will be looped between the start and end time.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	
	float fLargestAnimEnd=1.0f;
	if(AnimDuration>fLargestAnimEnd) fLargestAnimEnd=AnimDuration;
	if(static_cast<float>(m_iAnimFrames)>fLargestAnimEnd) fLargestAnimEnd=static_cast<float>(m_iAnimFrames);
	
	
	if(m_bSoundLooped)
	{
		pxDialog = new Asura_ScaledNumerical_Dialog("Should never see this!",xOffset,1.0f,&pxSound->m_fStartTime, 0.0f, fLargestAnimEnd,0.01f,"%0.3f",&m_bSoundStartChanged);
		pxDialog->SetHelpText("Should never see this!");
		pxDialog->m_iControlID = DialogID_SoundStartTime;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		pxDialog = new Asura_ScaledNumerical_Dialog("Should never see this!",xOffset,1.0f,&pxSound->m_fEndTime, 0.0f, fLargestAnimEnd,0.01f,"%0.3f",&m_bSoundEndChanged);
		pxDialog->SetHelpText("Should never see this!");
		pxDialog->m_iControlID = DialogID_SoundEndTime;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}
	else
	{
		pxDialog = new Asura_ScaledNumerical_Dialog("Should never see this!",xOffset,1.0f,&pxSound->m_fStartTime, 0.0f,fLargestAnimEnd,0.01f,"%0.3f",&m_bSoundStartChanged);
		pxDialog->SetHelpText( "Should never see this!" );
		pxDialog->m_iControlID = DialogID_SoundStartTime;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();
	}

	pxDialog = new Tools_Enumeration_Dialog("Sound ID",xOffset,&pxSound->m_iAnimSoundID,m_pxAnimationSoundList,&ModifiedFlag);
	pxDialog->SetHelpText("Select the ID of the sound to play.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	// Put a divider in the dialog
	pxDialog = new Asura_Divider_Dialog( xOffset );
	pxOverlay->AddDialog( pxDialog );
	xOffset.y += Asura_Dialog::SmallSpacing();

	pxDialog = new Asura_Function_Dialog("Add Sound Marker",xOffset,_AddAnimationSound);
	pxDialog->SetHelpText("Add a new keyframe sound marker.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x += pxDialog->GetMaxCoords().x;
	
	pxDialog = new Asura_Function_Dialog("Delete Sound Marker",xOffset,_DeleteAnimationSound);
	pxDialog->SetHelpText("Delete this keyframe sound marker.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	

	pxOverlay->m_uID = TOOLS_OVERLAY_ANIMATIONSOUND;

 	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);

	UpdateAnimationSoundTimeDisplay();
}


void Tools_Mode_Hierarchy::UpdateAnimationSoundTimeDisplay()
{
	Asura_Overlay* const pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_ANIMATIONSOUND);

	if(pxOverlay)
	{
		Asura_ScaledNumerical_Dialog* const pxEndDialog = ( m_bSoundLooped ) ? reinterpret_cast<Asura_ScaledNumerical_Dialog*>( pxOverlay->GetDialog(DialogID_SoundEndTime) ) : NULL;
		Asura_ScaledNumerical_Dialog* const pxStartDialog = reinterpret_cast<Asura_ScaledNumerical_Dialog*>(pxOverlay->GetDialog(DialogID_SoundStartTime));

		if ( pxStartDialog )
		{
			Asura_String xLabel( "Time (" );
			xLabel += s_aszTimerModeStrings[m_eAnimationControl_TimerMode];
			xLabel += ")";
			if ( pxEndDialog == NULL )
			{
				pxStartDialog->UpdateLabel( xLabel.GetString() );
			}
			else
			{
				Asura_String xText;
				xText = "Start ";
				xText += xLabel;
				pxStartDialog->UpdateLabel( xText.GetString() );
				xText = "End ";
				xText += xLabel;
				pxEndDialog->UpdateLabel( xText.GetString() );
			}
			switch(m_eAnimationControl_TimerMode)
			{
				case TIMER_PROP:
				{
					pxStartDialog->SetScale(1.0f, 0.0f, 1.0f);
					pxStartDialog->SetDelta( 0.01f );
					if ( pxEndDialog == NULL )
					{
						pxStartDialog->SetHelpText("Time (0 to 1) when this sound should be triggered");
					}
					else
					{
						pxStartDialog->SetHelpText("The start time (0 to 1) for playing this time");
						pxEndDialog->SetHelpText("The  end time (0 to 1) for playing this sound");
						pxEndDialog->SetScale(1.0f, 0.0f, 1.0f);
						pxEndDialog->SetDelta( 0.01f );
					}
					break;
				}

				case TIMER_SECONDS:
				{
					pxStartDialog->SetScale(AnimDuration, 0.0f, AnimDuration);
					pxStartDialog->SetDelta( 0.01f );
					if ( pxEndDialog == NULL )
					{
						pxStartDialog->SetHelpText("Time in seconds when this sound should be triggered");
					}
					else
					{
						pxStartDialog->SetHelpText("The start time in seconds for playing this sound");
						pxEndDialog->SetHelpText("The end time in seconds for playing this sound");
						pxEndDialog->SetScale(AnimDuration, 0.0f, AnimDuration);
						pxEndDialog->SetDelta( 0.01f );
					}
					break;
				}

				case TIMER_FRAMES:
				{
					pxStartDialog->SetScale(static_cast<float>(m_iAnimFrames), 0.0f, static_cast<float>(m_iAnimFrames));
					pxStartDialog->SetDelta( 1.00f );
					if ( pxEndDialog == NULL )
					{
						pxStartDialog->SetHelpText("Time in frames when this sound should be triggered");
					}
					else
					{
						pxStartDialog->SetHelpText("The start time in frames for playing this sound");
						pxEndDialog->SetHelpText("The end time in frames for playing this sound");
						pxEndDialog->SetScale(static_cast<float>(m_iAnimFrames), 0.0f, static_cast<float>(m_iAnimFrames));
						pxEndDialog->SetDelta( 1.00f );
					}
					break;
				}

				case TIMER_25FPS:
				case TIMER_30FPS:
				{
					pxStartDialog->SetScale(m_fTimeSliderMax, 0.0f, m_fTimeSliderMax);
					pxStartDialog->SetDelta( 1.00f );
					if ( pxEndDialog == NULL )
					{
						pxStartDialog->SetHelpText("Time in frames when this sound should be triggered");
					}
					else
					{
						pxStartDialog->SetHelpText("The start time in frames for playing this sound");
						pxEndDialog->SetHelpText("The end time in frames for playing this sound");
						pxEndDialog->SetScale(m_fTimeSliderMax, 0.0f, m_fTimeSliderMax);
						pxEndDialog->SetDelta( 1.00f );
					}
					break;
				}

			}
		}
	}
}


void Tools_Mode_Hierarchy::TogglePosableAnimOverlay(void)
{
	m_bPosableAnimOverlayOpen = !m_bPosableAnimOverlayOpen;

	if (m_bPosableAnimOverlayOpen)
	{
		CreatePosableAnimOverlay();
	}
	else
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_POSABLEANIM);
	}
}
bool Tools_Mode_Hierarchy::PosableAnimOverlayIsOn(void)
{
	return m_bPosableAnimOverlayOpen;
}

void Tools_Mode_Hierarchy::CreatePosableAnimOverlay(void)
{
	if(!m_bPosableAnimOverlayOpen) return;

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Posable Anim",false,&m_xPosableAnimOverlayPosition);
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	const float fCol1 = 1.0f;
	const float fCol2 = 7.0f;
	const float fCol3 = 17.5f;

	//add the headings
	xOffset.x = fCol2+2.0f;
	pxDialog = new Asura_Dialog("Pitch (X)",xOffset);
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = fCol3+2.0f;
	pxDialog = new Asura_Dialog("Yaw (Y)",xOffset);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//Hip
	xOffset.x = fCol1;
	pxDialog = new Asura_Dialog("Hip",xOffset);
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = fCol2;
	pxDialog = new Asura_AngleNumerical_Dialog("",xOffset,&m_xPosableHipOrient.x,-360.0f,360.0f,1.0f,"%.1f");
	pxDialog->SetIsEnabledFunction(_AnimIsPosable);
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = fCol3;
	pxDialog = new Asura_AngleNumerical_Dialog("",xOffset,&m_xPosableHipOrient.y,-360.0f,360.0f,1.0f,"%.1f");
	pxDialog->SetIsEnabledFunction(_AnimIsPosable);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();


	//Head
	xOffset.x = fCol1;
	pxDialog = new Asura_Dialog("Head",xOffset);
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = fCol2;
	pxDialog = new Asura_AngleNumerical_Dialog("",xOffset,&m_xPosableHeadOrient.x,-360.0f,360.0f,1.0f,"%.1f");
	pxDialog->SetIsEnabledFunction(_AnimIsPosable);
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = fCol3;
	pxDialog = new Asura_AngleNumerical_Dialog("",xOffset,&m_xPosableHeadOrient.y,-360.0f,360.0f,1.0f,"%.1f");
	pxDialog->SetIsEnabledFunction(_AnimIsPosable);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	xOffset.x = 1.0f;

	pxDialog = new Asura_Function_Dialog("Reset",xOffset,_ResetPosableAnim);	
	pxDialog->SetIsEnabledFunction(_AnimIsPosable);
	pxDialog->SetHelpText("Resets all angles to zero.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

			
	pxOverlay->m_uID = TOOLS_OVERLAY_POSABLEANIM;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}

void Tools_Mode_Hierarchy::ChooseSkin()
{
	Asura_String xSkinName,xSkinFilename;

	Asura_Hierarchy_Skin* pxSkin = Tools_Animation_Files::ChooseSkin(&xSkinName,&xSkinFilename);
	if(!pxSkin) return;

	Tools_Object_System::DeselectAllObjects();

	SetSkin(xSkinName.GetString(),xSkinFilename.GetString());
}

void Tools_Mode_Hierarchy::AllocateHierarchy( const char* szName )
{
	Hierarchy = new Asura_Animation( szName );
}

void Tools_Mode_Hierarchy::SelectSkin(Asura_Hash_ID uSkinID)
{
	if(!s_pxThis)
	{
		Tools_Timed_Message_System::AddMessage(0xffff00,"Must be in hierarchy mode to select a skin.");
		return;
	}
	const char* szFilename = Asura_HashedFilename_System::GetFullFilename(uSkinID);
	if(!szFilename)
	{
		Tools_Timed_Message_System::AddMessage(0xffff00,"Failed to find skin's filename.");
		return;
	}
	Asura_String xName;
	Asura_String(szFilename).SplitFilename(0,&xName,0);

	s_pxThis->SetSkin(xName.GetString(), szFilename);
}

bool Tools_Mode_Hierarchy::SetSkin(const char* pszSkinName,const char* pszSkinFilename)
{
	//see if skin is already loaded
	Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkin(pszSkinName);
	if(!pxSkin)
	{
		//try loading it then
		LoadAsuraFile(pszSkinFilename);
		pxSkin = Asura_Animation_System::GetSkin(pszSkinName);
	}
	if(!pxSkin) return false;

	SkinName = pszSkinName;
	SkinFilename = pszSkinFilename;

	//make sure we have the Asura_Meshes for the selected skin
	Tools_Animation_Files::LoadMeshForSkin(pxSkin);

	if(m_pxSkin) OnDeselectSkin(m_pxSkin);
	m_pxSkin = pxSkin;
	if(m_pxSkin) OnSelectSkin(m_pxSkin);

	m_puSkinID[0] = m_pxSkin ? m_pxSkin->GetHashID() : 0;

	//update the overlay
	Asura_Overlay* modeOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_MODE);
	if(modeOverlay)
	{
		char text[200];

		sprintf(text,"Skin : %s",SkinName.GetString());
		
		modeOverlay->UpdateDialogLabel(DialogID_Skin,text);
	}

	//remove secondary anims
	for(int i=0;i<MAX_SECONDARY_ANIM;i++)
	{
		RemoveSecondaryAnim(&SecondaryAnims[i],true);
		Asura_Overlay_System::CloseOverlays(SecondaryAnims[i].OverlayID);
	}

	DeleteMarkerPointObjects();
	DeleteBoundingBoxObjects();

	delete Hierarchy;

	AllocateHierarchy( SkinName.GetString() );
	SetSkinsInAsuraAnimation();

	Asura_Assert(Hierarchy->GetSkin() == m_pxSkin,"");
	
	CreateMarkerPointObjects();

#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
	delete m_pxSkinLOD;
	m_pxSkinLOD = 0;
#endif

	//set the default directory name
	DirectoryName = Tools_Animation_Files::GetDefaultAnimDirForSkin(SkinName.GetString());

	//can we continue to use the previous animation?	
	if(Animation && Animation->NumberOfBones == m_pxSkin->NumberOfBones)
	{
		Hierarchy->m_xPrimaryWrapper.StartAnim2(AnimName.GetHash(),m_uPrimaryAnimFlags,0.0f);
	}
	else
	{
		SelectAnimation(NULL,NULL);
	}

	//make sure the skin's bounding box has been calculated
	if( !m_pxSkin->m_pxSectionBoundingBoxData )
    {
        m_pxSkin->InitSectionBoundingBoxes();
    }
	
	CreateBoundingBoxObjects();

 	//end volume editing on previous skin , and get the new skin's BB
	if(m_pxSkinCollisionVolume) m_pxSkinCollisionVolume->Destroy();
	if(m_pxSkin) m_xSkinCollisionBB = m_pxSkin->GetCollisionBB();


	//Get LOD settings from the model
	if(m_pxSkin && m_pxSkin->m_pxNextLowerSkin)
	{
		//count triangles for highest detail model
		int iRootTriCount = 0;
		if(m_pxSkin->IsSkinnedModel())
		{
			Asura_Mesh* pxMesh = Asura_Mesh_System::GetMesh(m_pxSkin->Name);
			Asura_Assert(pxMesh,"");
			iRootTriCount = pxMesh->NumberOfTriangles;
		
		}
		else
		{
			for(int j=0;j<m_pxSkin->NumberOfBones;j++)
			{
				if(m_pxSkin->SectionData[j].Mesh)
				{
					iRootTriCount += m_pxSkin->SectionData[j].Mesh->NumberOfTriangles;
				}
			}
		}

		Asura_Hierarchy_Skin* pxSkin = m_pxSkin->m_pxNextLowerSkin;
		for(int i=0;i<TOOLS_MAX_LOD_MODELS;i++)
		{
			if(pxSkin)
			{
				//count triangles for LOD model
				int iLODTriCount = 0;
				if(pxSkin->IsSkinnedModel())
				{
					Asura_Mesh* pxMesh = Asura_Mesh_System::GetMesh(pxSkin->Name);
					Asura_Assert(pxMesh,"");
					iLODTriCount = pxMesh->NumberOfTriangles;
				
				}
				else
				{
					for(int j=0;j<pxSkin->NumberOfBones;j++)
					{
						if(pxSkin->SectionData[j].Mesh)
						{
							iLODTriCount += pxSkin->SectionData[j].Mesh->NumberOfTriangles;
						}
					}
				}
				
				m_afLODDistances[i] = pxSkin->m_fMinDistance;
				if(iRootTriCount)
				{
					m_afLODPercent[i] = iLODTriCount/(float)iRootTriCount;
				}
				else
				{
					m_afLODPercent[i] = 0.0f;
				}

				pxSkin = pxSkin->m_pxNextLowerSkin;
			}
			else
			{
				m_afLODDistances[i] =  (i+1)*10.0f;
				m_afLODPercent[i] = 0;	
			}
		}
	}
	else
	{
		for(int i=0;i<TOOLS_MAX_LOD_MODELS;i++)
		{
			m_afLODPercent[i] = 0;
		}
	}

	delete m_pxSkinSoundEdit;
	m_pxSkinSoundEdit = 0;

	//make sure the sounds are loaded
	Tools_Animation_Files::LoadSoundsForSkin(m_pxSkin);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_SKINPROPERTIES);
	CreateSkinPropertiesOverlay();

	m_bSkinModified = false;
	return true;
}

void Tools_Mode_Hierarchy::SetSkinsInAsuraAnimation()
{
	if(!Hierarchy) return;


	
	Asura_Hierarchy_Skin* pxPrimarySkin = Asura_Animation_System::GetSkinByHash(m_puSkinID[0]);

	if(pxPrimarySkin)
	{
		for(int i=0; i<ASURA_ANIMATION_SKIN_COUNT; i++)
		{
			Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkinByHash(m_puSkinID[i]);
			if(!pxSkin)continue;

			//make sure each skin has the same number of bones
			if(pxSkin->NumberOfBones == pxPrimarySkin->NumberOfBones)
			{
				Hierarchy->SetSkin(m_puSkinID[i], false, i);
			}
			else
			{
				Hierarchy->SetSkin(0, false, i);
				Tools_Timed_Message_System::AddMessage(0xffffff,"Number of bones in skin '%s' (%d) differs from skin '%s' (%d)", pxSkin->Name.GetString(), pxSkin->NumberOfBones, pxPrimarySkin->Name.GetString(), pxPrimarySkin->NumberOfBones);
			}
		}
	}
}




void Tools_Mode_Hierarchy::ChooseAnim()
{
	if(!SkinName.GetLength())
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Please choose a skin first");
		return;
	}
	
	Asura_String xFilename;
	Asura_String xOldDir = DirectoryName;
	Asura_Hierarchy_Anim* pxAnim = Tools_Animation_Files::ChooseAnimation(SkinName.GetString(),&DirectoryName,0,&xFilename,Animation,true);
	if(pxAnim == Animation) return;


	SelectAnimation(pxAnim,xFilename.GetString());

}

void Tools_Mode_Hierarchy::ChooseDirectory()
{
	const List<ToolsAnimationDirectory*>* pxDirList = Tools_Animation_Files::GetDirectoryList();
	
	ListSelection_Overlay overlay("Select Animation Directory");

	for(CLIF<ToolsAnimationDirectory*> xDirLIF(pxDirList);!xDirLIF.done();xDirLIF.next())
	{
		overlay.AddItem(xDirLIF()->m_xName.GetString(),(int)xDirLIF()->m_xDirectory.GetString());
	}

	//add an entry for the default dir
	const char* pszDefault = Tools_Animation_Files::GetDefaultAnimDirForSkin(SkinName.GetString());
	if(pszDefault)
	{
		overlay.AddItem("*Default*",(int)pszDefault);
	}

	//show the selection overlay
	if(!overlay.DoModal()) return;

	DirectoryName = (const char*)overlay.GetSelectionData();


}


void Tools_Mode_Hierarchy::NextAnimation()
{
	if(!Hierarchy || !Animation) return;

	Asura_String xFilename;
	Asura_Hierarchy_Anim* pxAnim = GetNextAnimation(Animation,&DirectoryName,&xFilename);
	if(pxAnim)
	{
		SelectAnimation(pxAnim,xFilename.GetString());
	}
}

void Tools_Mode_Hierarchy::PreviousAnimation()
{
	if(!Hierarchy || !Animation) return;

	Asura_String xFilename;
	Asura_Hierarchy_Anim* pxAnim = GetPreviousAnimation(Animation,&DirectoryName,&xFilename);
	if(pxAnim)
	{
		SelectAnimation(pxAnim,xFilename.GetString());
	}
}

void Tools_Mode_Hierarchy::SelectAnim(Asura_Hash_ID uAnimID)
{
	if(!s_pxThis)
	{
		Tools_Timed_Message_System::AddMessage(0xffff00,"Must be in hierarchy mode to select an animation.");
		return;
	}
	
	Asura_Hierarchy_Anim* pxAnim = Tools_Animation_Files::GetAnimFromID(uAnimID);
	if(!pxAnim)
	{
		Tools_Timed_Message_System::AddMessage(0xffff00,"Failed to find the animation.");
		return;
	}

	s_pxThis->SelectAnimation(pxAnim, NULL);
}



void Tools_Mode_Hierarchy::SelectAnimation(Asura_Hierarchy_Anim* pxAnim,const char* pszFilename)
{
	if(Animation)
	{
		OnDeselectAnimation(pxAnim);
	}
	
	//restore keyframes for current animation
	RestoreAnimationKeyFrames();
	
	AnimName = pxAnim ? pxAnim->Name : "";
	Animation = pxAnim;
	m_xAnimFilename = pszFilename;

	if(Animation && m_xAnimFilename.IsBlank())
	{
		//no filename provided , try to find the file
		if(!Tools_Animation_Files::FindFile(AnimName.GetString(),&m_xAnimFilename))
		{
			Tools_Timed_Message_System::AddMessage(0xffff00,"Filename for animation not found. Won't be able to save changes.");
		}
	}

	if(Animation)
	{
		//make sure we have thr original uncompressed animation loaded
		Tools_Animation_Files::LoadUncompressedAnimation(pxAnim,m_xAnimFilename.GetString());
	}

	//and take a backup of the new animation
	BackupAnimationKeyFrames();

	if(m_bDoFrameRemoval)
	{
		RemoveUnneededKeyFrames();	
	}

	//update the overlay
	Asura_Overlay* modeOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_MODE);
	if(modeOverlay)
	{
		char text[200];

		if(AnimName.GetLength())
			sprintf(text,"Anim : %s",AnimName.GetString());
		else
			strcpy(text,"Anim : *None*");
		
		modeOverlay->UpdateDialogLabel(DialogID_Anim,text);
	}

	//update treadmill settings
	if(Animation)
	{
		AnimDuration = Animation->Duration;
		TreadmillSpeed = Animation->Speed;
		TreadmillDirection = (float)(int)(Asura_Maths::RadToDeg(Animation->Direction)+0.5f);
		m_fLoopPoint = Animation->GetLoopPoint();
		m_iAnimFrames = GetNumAnimFrames(Animation);

		m_bAnimHasBB = Animation->GetBB()!=0;
		m_bWarnedAboutCompressedFrames = false;
	}

	//play the selected animation
	if(Hierarchy)
	{
		Hierarchy->m_xPrimaryWrapper.StartAnim2(AnimName.GetHash(),m_uPrimaryAnimFlags,0.0f);
		#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
		//Play any associated facial animation as well.
		Tools_Overlay_Hierarchy_ViewFacialAnimation::PlayLinkedFacialAnimation(Animation);
		#endif

	}
	m_fAnimationControl_Speed =1.0f;

	//allow derived classes to do stuff
	if(Animation)
	{
		OnSelectAnimation(Animation);
	}

	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ANIMATIONFLAG);
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ANIMATIONSOUND);

	AnimationFlagList = GetAnimationFlagList();

	delete m_pxAnimSoundEdit;
	m_pxAnimSoundEdit = 0;
	
	//make sure the sounds are loaded
	if(Animation)
	{
		Tools_Animation_Files::LoadSoundsForAnim(Animation);
	}


	//close anim properties overlay , and re-create if needed
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_ANIMPROPERTIES);
	CreateAnimPropertiesOverlay();


}

void Tools_Mode_Hierarchy::UpdatePrimaryAnimFlags()
{
	if(!Hierarchy || AnimName.IsBlank()) return;

	float fTimer = Hierarchy->m_xPrimaryWrapper.GetTimer();
	Hierarchy->m_xPrimaryWrapper.StartAnim2(AnimName.GetHash(),m_uPrimaryAnimFlags,0.0f);
	Hierarchy->m_xPrimaryWrapper.SetTimer(fTimer);

	#if defined(ASURA_USE_FACIAL_ANIMATION_SYSTEM)
	//Play any associated facial animation as well.
	Tools_Overlay_Hierarchy_ViewFacialAnimation::PlayLinkedFacialAnimation(Animation);
	#endif
}



Tools_Flag_List* Tools_Mode_Hierarchy::GetAnimationFlagList()
{
	Asura_Assert(Tools_System::CommonFlagIni,"");
	return Tools_System::CommonFlagIni->GetFlagList("Keyframe Flags");
}



Asura_Hierarchy_Skin* Tools_Mode_Hierarchy::LoadSkin(const Asura_String& xSkinName,const Asura_String& xSkinFileName)
{
	if(!xSkinName.GetLength()) return 0;

	Asura_Hierarchy_Skin* pxSkin = Asura_Animation_System::GetSkin(xSkinName.GetString());
	
	if(!pxSkin)
	{
		//skin not loaded , so attempt to do so
		LoadAsuraFile(xSkinFileName.GetString());
		Asura_Texture_Management::LoadTextures();

		pxSkin = Asura_Animation_System::GetSkin(xSkinName.GetString());
		if(!pxSkin)
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to get skin '%s'",xSkinName.GetString());
			Tools_Timed_Message_System::AddMessage(0xff0000,"Skin name doesn't match filename?");
			return 0;
		}
	}

	return pxSkin;
}

Asura_Hierarchy_Anim* Tools_Mode_Hierarchy::LoadAnim(const Asura_String& xAnimName,const Asura_String& xAnimFileName)
{
	if(!xAnimName.GetLength()) return false;

	Asura_Hierarchy_Anim* pxAnimation = Asura_Animation_System::GetAnim(xAnimName.GetString());
	if(!pxAnimation)
	{
		//attempt to load the animation

		LoadAsuraFile(xAnimFileName.GetString());

		//see if anim was loaded successfully
		pxAnimation = Asura_Animation_System::GetAnim(xAnimName.GetString());
		if(!pxAnimation)
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to get anim '%s'",xAnimName.GetString());
			Tools_Timed_Message_System::AddMessage(0xff0000,"Anim name doesn't match filename?");
		}
	}


	return pxAnimation;

}

void Tools_Mode_Hierarchy::UpdateTreadmill()
{
	if(!TreadmillMesh) return;
	if(!Animation) return;
	if(m_bUseMovementOffsets && Animation->HasMovementData())
	{
		//If the character is being moved forward by the animation , then we don't need to animate the treadmill
		return;
	}
	Asura_Assert(TreadmillMesh->TextureDataValid,"");
	float offset = TreadmillSpeed*Asura_Timers::GetGameTimeSlice()*m_fAnimationControl_Speed;

	//get the texture info for the only polygon
	Asura_Mesh_Polygon* poly = &TreadmillMesh->PolygonDataPtr[0];
	Asura_Mesh_TextureInfo* tinfo = &TreadmillMesh->TextureDataPtr[0];

	//apply the offset to each coordinate
	int num_verts = poly->NumberOfVertices();
	offset -= (float)(int)tinfo->V[0][0];
	for(int i=0;i<num_verts;i++)
	{
		tinfo->V[0][i] += offset;
	}
	
	Asura_Mesh_System::PlatformResetRenderMesh(TreadmillMesh);
}

void Tools_Mode_Hierarchy::RenderTreadmill()
{
	if(!TreadmillMesh) return;

	SCOPEDPIXEVENT((0xff8800, L"Hierarchy Tread Mill"));

	Asura_Vector_3 pos = g_xHierarchyModeOrigin;

	Asura_Matrix_3x3 mat;
	Asura_Euler_3 euler(0,Asura_Maths::DegToRad(TreadmillDirection),0);
	euler.ToMatrix(mat);

	Asura_Mesh_System::PlatformRenderMesh(TreadmillMesh,&pos,&mat);
}

void Tools_Mode_Hierarchy::RenderWorld_PostToneMap()
{

}


void Tools_Mode_Hierarchy::AddSecondaryAnim()
{
	//find an unused slot
	SecondaryAnimationData* secondary = 0;
	for(int i=0;i<MAX_SECONDARY_ANIM;i++)
	{
		if(!SecondaryAnims[i].Used)
		{
			secondary = &SecondaryAnims[i];
			secondary->Reset();
			break;
		}
	}

	if(!secondary)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Can't add any more secondary animations");
		return;
	}

	secondary->Used = true;

	//create an overlay for this animation
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Animation",false);
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);


	pxDialog = new Asura_Function_Dialog("Anim : *None*",xOffset,_ChooseSecondaryAnim,(int) secondary);
	pxDialog->m_iControlID = DialogID_Anim;
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Numerical_Dialog("Weight",xOffset,&secondary->Weight,0,1,0.1f,"%.3f");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	secondary->m_bTimerChanged = false;
	pxDialog = new Asura_Numerical_Dialog("Timer",xOffset,&secondary->m_fTimer,0,1,0.1f,"%.3f",&secondary->m_bTimerChanged);
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Paused",xOffset,&secondary->m_bPaused);
	pxDialog->SetHelpText("Leaves this secondary animation active , but doesn't about the timer each frame.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Mirrored",xOffset,&secondary->m_bMirrored);
	pxDialog->SetHelpText("Toggles the mirrored flag for the secondary anim.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Reversed",xOffset,&secondary->m_bReversed,&secondary->Restart);
	pxDialog->SetHelpText("Toggles the reversed flag for the secondary anim.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	pxDialog = new Asura_Tickbox_Dialog("Sync With Primary",xOffset,&secondary->SyncWithPrimary,&secondary->Restart);
	pxDialog->SetHelpText("The timer on this animation is set to match the timer on the main animation.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Tickbox_Dialog("Enabled",xOffset,&secondary->m_bEnabled,&secondary->Restart);
	pxDialog->SetHelpText("Disabling the animation allows it to be removed temporarily without losing the settings.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Function_Dialog("Remove",xOffset,_RemoveSecondaryAnim,(int) secondary,&pxOverlay->m_bShouldClose);
	pxDialog->SetHelpText("Remove this secondary animation..");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::DefaultSpacing();


 			
	pxOverlay->m_uID = secondary->OverlayID;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}


void Tools_Mode_Hierarchy::ChooseSecondaryAnim(SecondaryAnimationData* secondary)
{
	
	if(!SkinName.GetLength())
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Please choose a skin first");
		return;
	}
	Asura_Assert(Hierarchy,"");

	
	secondary->m_pxAnim = Tools_Animation_Files::ChooseAnimation(SkinName.GetString(),&secondary->m_xDirectory,0,0);

	
	//update the overlay
	Asura_Overlay* modeOverlay = Asura_Overlay_System::GetOverlay(secondary->OverlayID);
	if(modeOverlay)
	{
		char text[200];
	 	if(secondary->m_pxAnim)
			sprintf(text,"Anim : %s",secondary->m_pxAnim->Name.GetString());
		else
			strcpy(text,"Anim : *None*");
		modeOverlay->UpdateDialogLabel(DialogID_Anim,text);
	}

	secondary->Restart = true;

	m_pxMostRecentSecondary = secondary;
}

void Tools_Mode_Hierarchy::NextSecondaryAnim(int iDirection)
{
	if(!SkinName.GetLength())
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Please choose a skin first");
		return;
	}
	Asura_Assert(Hierarchy,"");

	//determine which secondary anim to affect
	if(!m_pxMostRecentSecondary || !m_pxMostRecentSecondary->Used)
	{
		for(int i=0;i<MAX_SECONDARY_ANIM;i++)
		{
			if(SecondaryAnims[i].Used)
			{
				m_pxMostRecentSecondary = &SecondaryAnims[i];
			}
		}
	}
	if(!m_pxMostRecentSecondary) return;
	if(!m_pxMostRecentSecondary->m_pxAnim) return;

	Asura_Hierarchy_Anim* pxAnim = GetNextAnimation(m_pxMostRecentSecondary->m_pxAnim,&m_pxMostRecentSecondary->m_xDirectory,0);
	if(pxAnim)
	{
		m_pxMostRecentSecondary->m_pxAnim = pxAnim;
	//update the overlay
		Asura_Overlay* modeOverlay = Asura_Overlay_System::GetOverlay(m_pxMostRecentSecondary->OverlayID);
		if(modeOverlay)
		{
			char text[200];
		 	if(m_pxMostRecentSecondary->m_pxAnim)
				sprintf(text,"Anim : %s",m_pxMostRecentSecondary->m_pxAnim->Name.GetString());
			else
				strcpy(text,"Anim : *None*");
			modeOverlay->UpdateDialogLabel(DialogID_Anim,text);
		}

		m_pxMostRecentSecondary->Restart = true;
	}
}


void Tools_Mode_Hierarchy::RemoveSecondaryAnim(SecondaryAnimationData* secondary,bool reset)
{
	if(secondary->AnimHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Assert(Hierarchy,"");
		Hierarchy->m_xPrimaryWrapper.RemoveOneShotAnim(secondary->AnimHandle);
		secondary->AnimHandle = ASURA_INVALID_HANDLE;
	}
	if(reset) secondary->Reset();
}

void Tools_Mode_Hierarchy::RestartSecondaryAnim(SecondaryAnimationData* secondary)
{
	RemoveSecondaryAnim(secondary,false);
	if(secondary->m_pxAnim && secondary->m_bEnabled)
	{
		
		//play the animation
		int animFlags = AsuraAnimationFlag_Loop;
		if(secondary->SyncWithPrimary) animFlags |= AsuraAnimationFlag_SyncWithPrimary;
		if(secondary->m_bMirrored) animFlags |= AsuraAnimationFlag_Mirror;
		if(secondary->m_bReversed) animFlags |= AsuraAnimationFlag_Reversed;
		
		secondary->AnimHandle = Hierarchy->m_xPrimaryWrapper.AddOneShotAnim2(secondary->m_pxAnim->Name.GetHash(),secondary->Weight,animFlags,DEFAULT_TWEEN_TIME,0.0f);

		if(!secondary->SyncWithPrimary)
		{
			Hierarchy->m_xPrimaryWrapper.SetTimer(secondary->m_fTimer,secondary->AnimHandle);
		}
		Hierarchy->m_xPrimaryWrapper.SetPaused(secondary->m_bPaused,secondary->AnimHandle);
	}
	secondary->Restart = false;
}

void Tools_Mode_Hierarchy::UpdateSecondaryAnims()
{
	if(!Hierarchy) return;

	bool bNeedRestart = false;
	for(int i=0;i<MAX_SECONDARY_ANIM;i++)
	{
		SecondaryAnimationData* pxSecondary = &SecondaryAnims[i];
		if(pxSecondary->AnimHandle != ASURA_INVALID_HANDLE)
		{
			//timer changed by hand?
			if(pxSecondary->m_bTimerChanged && !pxSecondary->SyncWithPrimary)
			{
				Hierarchy->m_xPrimaryWrapper.SetTimer(pxSecondary->m_fTimer,pxSecondary->AnimHandle);
			}
			else
			{
				pxSecondary->m_fTimer = Hierarchy->m_xPrimaryWrapper.GetTimer(pxSecondary->AnimHandle);
			}
			pxSecondary->m_bTimerChanged = false;

			//update paused flag
			Hierarchy->m_xPrimaryWrapper.SetPaused(pxSecondary->m_bPaused,pxSecondary->AnimHandle);
			
			// update mirrored flag
			Hierarchy->m_xPrimaryWrapper.SetMirrored(pxSecondary->m_bMirrored,pxSecondary->AnimHandle);

			
			//update weightings
			Hierarchy->m_xPrimaryWrapper.ChangeOneShotAnimWeighting(pxSecondary->AnimHandle,pxSecondary->Weight);
		}
		bNeedRestart |= pxSecondary->Restart;
	}

	//if any need restarting , restart all the anims to make sure the ordering stays the same
	if(bNeedRestart)
	{
		for(int i=0;i<MAX_SECONDARY_ANIM;i++)
		{
			SecondaryAnimationData* pxSecondary = &SecondaryAnims[i];
			RestartSecondaryAnim(pxSecondary);
		}
	}
}

#if 0
void Tools_Mode_Hierarchy::AnimationControl_Rewind()
{
	
	//go x1,x2,or x4 speed in reverse
	if(AnimationControl_Speed>-1.0f) AnimationControl_Speed = -1.0f;
	else if(AnimationControl_Speed>-2.0f) AnimationControl_Speed = -2.0f;
	else if(AnimationControl_Speed>-4.0f) AnimationControl_Speed = -4.0f;
	else AnimationControl_Speed = -1.0f;
}
#endif

void Tools_Mode_Hierarchy::AnimationControl_FrameBack()
{
	switch(m_eAnimationControl_TimerMode)
	{
		case TIMER_FRAMES :
			{
				int iFrame = Asura_Maths::FloatToInt_Near(m_fAnimationControl_SlideTimer-1.0f);
				if(iFrame<0) iFrame = m_iAnimFrames;
				m_fAnimationControl_SlideTimer = (float) iFrame;
				m_bAnimationControl_SlideTimerChanged = true;
				break;
			}

		case TIMER_25FPS :
		case TIMER_30FPS :
			{
				int iFrame = Asura_Maths::FloatToInt_Near(m_fAnimationControl_SlideTimer-1.0f);
				if(iFrame<0) iFrame = Asura_Maths::FloatToInt_Near(m_fTimeSliderMax);
				m_fAnimationControl_SlideTimer = (float) iFrame;
				m_bAnimationControl_SlideTimerChanged = true;
				break;
			}

			
		default :
			m_fAnimationControl_Speed = 0;
			m_fAnimationControl_DeltaTime = -.01f;
			break;
	}
}

void Tools_Mode_Hierarchy::AnimationControl_Play()
{
	m_fAnimationControl_Speed = 1.0f;
}

void Tools_Mode_Hierarchy::AnimationControl_Pause()
{
	m_fAnimationControl_Speed = 0.0f;
}

void Tools_Mode_Hierarchy::AnimationControl_FrameForward()
{
	switch(m_eAnimationControl_TimerMode)
	{
		case TIMER_FRAMES :
			{
				int iFrame = Asura_Maths::FloatToInt_Near(m_fAnimationControl_SlideTimer+1.0f);
				if(iFrame>m_iAnimFrames) iFrame = 0;
				m_fAnimationControl_SlideTimer = (float) iFrame;
				m_bAnimationControl_SlideTimerChanged = true;
				break;
			}

		case TIMER_25FPS :
		case TIMER_30FPS :
			{
				int iFrame = Asura_Maths::FloatToInt_Near(m_fAnimationControl_SlideTimer+1.0f);
				if(iFrame>m_fTimeSliderMax) iFrame = 0;
				m_fAnimationControl_SlideTimer = (float) iFrame;
				m_bAnimationControl_SlideTimerChanged = true;
				break;
			}

		default :
			m_fAnimationControl_Speed = 0;
			m_fAnimationControl_DeltaTime = .01f;
			break;
	}
}

#if 0
void Tools_Mode_Hierarchy::AnimationControl_FastForward()
{

	//go x1,x2,or x4 speed
	if(AnimationControl_Speed<1.0f) AnimationControl_Speed = 1.0f;
	else if(AnimationControl_Speed<2.0f) AnimationControl_Speed = 2.0f;
	else if(AnimationControl_Speed<4.0f) AnimationControl_Speed = 4.0f;
	else AnimationControl_Speed = 1.0f;
}
#endif

void Tools_Mode_Hierarchy::AnimationControl_SetSliderMode()
{
	m_eAnimationControl_TimerMode = (AnimationTimerMode)(((int)m_eAnimationControl_TimerMode+1)%5);
	UpdateAnimationControlOverlay();
	AnimationControl_UpdateSlideTimer();
}

void Tools_Mode_Hierarchy::AddAnimationFlag()
{
	if(!Animation)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"An animation must be chosen first");
		return;
	}

	Asura_ResourcePushType(ASURA_RESOURCE_ANIMATION);
	{
		//create an enlarged animation flag array
		Asura_Animation_Flag_Interval* newAnimationFlags;

		Asura_ResourceAllocateArray(newAnimationFlags, Animation->NumberOfAnimationFlags+1);
		//copy the old values
		if(Animation->NumberOfAnimationFlags)
		{
			memcpy(newAnimationFlags,Animation->AnimationFlags,sizeof(Asura_Animation_Flag_Interval)*Animation->NumberOfAnimationFlags);
		}
		//replace the previous array
		Asura_ResourceDeallocateArray(Animation->AnimationFlags);
		Animation->AnimationFlags = newAnimationFlags;
		Animation->NumberOfAnimationFlags++;
	}
	Asura_ResourcePopType();

	int newIndex = Animation->NumberOfAnimationFlags-1;
	Animation->AnimationFlags[newIndex].StartTime = m_fAnimationControl_Timer;
	Animation->AnimationFlags[newIndex].EndTime = m_fAnimationControl_Timer;
	Animation->AnimationFlags[newIndex].Flag = 0;

	//show the flag editing overlay
	SelectedIntervalIndex = (float)newIndex;
	SelectedIntervalIndexChanged = true;
	UpdateAnimationFlagOverlay();

	Tools_Timed_Message_System::AddMessage(0xffffff,"Flag added");

	ModifiedFlag = true;

	SetAnimationFlag();
}

void Tools_Mode_Hierarchy::DeleteAnimationFlag()
{
	Asura_Assert(Animation,"");
	Asura_Assert(Animation->AnimationFlags,"");
	Asura_Assert(SelectedIntervalIndex<Animation->NumberOfAnimationFlags,"");

	//copy the last entry down into the current slot
	int index = (int)SelectedIntervalIndex;	

	Animation->NumberOfAnimationFlags--;
	Animation->AnimationFlags[index] = Animation->AnimationFlags[Animation->NumberOfAnimationFlags];

	//update the overlay	
	SelectedIntervalIndexChanged = true;
	UpdateAnimationFlagOverlay();

	Tools_Timed_Message_System::AddMessage(0xffffff,"Flag deleted");

	ModifiedFlag = true;
}

void Tools_Mode_Hierarchy::EditAnimationFlag()
{
	if(!Animation)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"An animation must be chosen first");
		return;
	}
	if(!Animation->NumberOfAnimationFlags)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"This animation doesn't have any flags yet");
		return;
	}

	SelectedIntervalIndex = 0;
	SelectedIntervalIndexChanged = true;

	UpdateAnimationFlagOverlay();
}

void Tools_Mode_Hierarchy::SetAnimationFlag()
{
	Asura_Assert(Animation,"");
	Asura_Assert(Animation->AnimationFlags,"");
	Asura_Assert(SelectedIntervalIndex<Animation->NumberOfAnimationFlags,"");

	if(!AnimationFlagList)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find list of flag names");
		return;
	}

	//create an overlay for selecting the flags
	ListSelection_Overlay overlay("Select a flag");

	for(LIF<NameAndValue> flif(&AnimationFlagList->ValueList);!flif.done();flif.next())
	{
		overlay.AddItem(flif().Name.GetString(),flif().Value);
	}
	
	if(!overlay.DoModal()) return;

	Animation->AnimationFlags[(int)SelectedIntervalIndex].Flag |= overlay.GetSelectionData();
	
	ModifiedFlag = true;
	UpdateAnimationFlagOverlay();
}

void Tools_Mode_Hierarchy::AddAnimationSound()
{
	if(!Animation)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"An animation must be chosen first");
		return;
	}

	if(!m_pxAnimationSoundList)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Unable to found list of animation sound IDs");
		return;
	}

	//ask for the sound ID to use
	int iSoundID = 0;
	if(!m_pxAnimationSoundList->SelectValue("Pick sound to play",&iSoundID))
	{
		return;
	}

	//Check to see if the sound ID has been associated with a looping sound event
	//If it has then the sound marker will be setup as looping by default , to prevent an assert
	bool bLooping = false;
	const Asura_Hierarchy_Sound* pxSoundProp = m_pxSkin ? m_pxSkin->GetSoundPropertiesFromAnimSoundID(iSoundID) : 0;
	if(!pxSoundProp) pxSoundProp = Animation->GetSoundPropertiesFromAnimSoundID(iSoundID);

	if(pxSoundProp)
	{
		const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::FindEvent(pxSoundProp->m_uSoundEvent);
		if(pxEvent && pxEvent->IsEndlessPlayback())
		{
			bLooping = true;
		}
		
	}

	Asura_ResourcePushType(ASURA_RESOURCE_ANIMATION);
	{
		//create an enlarged animation sound array
		Asura_Animation_Sound* pxSounds;
		Asura_ResourceAllocateArray(pxSounds, Animation->m_iNumberOfAnimationSounds+1);
		//copy the old values
		for(int i=0;i<Animation->m_iNumberOfAnimationSounds;i++)
		{
			pxSounds[i] = Animation->m_pxAnimationSounds[i];	
		}
		//replace the previous array
		Asura_ResourceDeallocateArray(Animation->m_pxAnimationSounds);
		Animation->m_pxAnimationSounds = pxSounds;
		Animation->m_iNumberOfAnimationSounds++;

		int iIndex = Animation->m_iNumberOfAnimationSounds-1;
		if(bLooping)
		{
			pxSounds[iIndex].m_fStartTime = 0.0f;
			pxSounds[iIndex].m_fEndTime = 1.0f;
		}
		else
		{
			pxSounds[iIndex].m_fStartTime = m_fAnimationControl_Timer;
			pxSounds[iIndex].m_fEndTime = m_fAnimationControl_Timer;
		}
		pxSounds[iIndex].m_iAnimSoundID = iSoundID;
		pxSounds[iIndex].m_uFlags = 0;

		//show the Sound editing overlay
		m_iSelectedSoundIndex = iIndex;
		m_bSelectedSoundIndexChanged = true;
		UpdateAnimationSoundOverlay();

		Tools_Timed_Message_System::AddMessage(0xffffff,"Sound added");

		ModifiedFlag = true;
	}
	Asura_ResourcePopType();
}

void Tools_Mode_Hierarchy::DeleteAnimationSound()
{
	Asura_Assert(Animation,"");
	Asura_Assert(Animation->m_pxAnimationSounds,"");
	Asura_Assert(m_iSelectedSoundIndex<Animation->m_iNumberOfAnimationSounds,"");


	//copy down the remaining sounds to fill in the gap
	for(int i=m_iSelectedSoundIndex;i<Animation->m_iNumberOfAnimationSounds-1;i++)
	{
		Animation->m_pxAnimationSounds[i] = Animation->m_pxAnimationSounds[i+1];
	}

	Animation->m_iNumberOfAnimationSounds--;

	//update the overlay	
	m_bSelectedSoundIndexChanged = true;
	UpdateAnimationSoundOverlay();

	Tools_Timed_Message_System::AddMessage(0xffffff,"Sound deleted");

	ModifiedFlag = true;
}

void Tools_Mode_Hierarchy::EditAnimationSound()
{
	if(!Animation)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"An animation must be chosen first");
		return;
	}
	if(!Animation->m_iNumberOfAnimationSounds)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"This animation doesn't have any sounds yet");
		return;
	}

	m_iSelectedSoundIndex = 0;
	m_bSelectedSoundIndexChanged = true;

	UpdateAnimationSoundOverlay();
}


void Tools_Mode_Hierarchy::EditSkinSoundDefns()
{
	if(m_pxSkin)
	{
		if(!m_pxSkinSoundEdit)
		{
			m_pxSkinSoundEdit = new ToolsSkinSoundDefnEdit(m_pxSkin);
		}
		m_pxSkinSoundEdit->UpdateOverlay();
	}
}

void Tools_Mode_Hierarchy::EditAnimSoundDefns()
{
	if(Animation)
	{
		if(!m_pxAnimSoundEdit)
		{
			m_pxAnimSoundEdit = new ToolsAnimSoundDefnEdit(Animation);
		}
		m_pxAnimSoundEdit->UpdateOverlay();
	}
}



void Tools_Mode_Hierarchy::GenerateLODModels()
{
	if(!m_pxSkin)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"A skin must be selected first");
		return;
	}
	

	
	if(m_pxSkin->m_bManualLOD)
	{
		//for manual LOD's just update the distances
		Asura_Hierarchy_Skin* pxLODSkin = m_pxSkin->m_pxNextLowerSkin;
		for(int i=0;i<TOOLS_MAX_LOD_MODELS && pxLODSkin;i++)
		{
			pxLODSkin->m_fMinDistance = m_afLODDistances[i];
			pxLODSkin = pxLODSkin->m_pxNextLowerSkin;
		}
	}
	else
	{
		if(!m_pxSkin->IsSkinnedModel())
		{
			Tools_Timed_Message_System::AddMessage(0xffffff,"This only works for skinned models");
			return;
		}

#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
		//create the LOD generating thing if need be
		if(!m_pxSkinLOD) m_pxSkinLOD = new Tools_SkinLOD(m_pxSkin);
#endif

		//delete old LOD models..
		if(m_pxSkin->m_pxNextLowerSkin)
		{
#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
			Asura_Direct3D_CharacterRender_Handler&	xHandler	= Asura_Direct3D_CharacterRenderer::GetCurrentHandler();
#endif
			Asura_Hierarchy_Skin*					pxSkin		= m_pxSkin->m_pxNextLowerSkin;
			while(pxSkin)
			{
				Asura_Mesh* pxMesh = Asura_Mesh_System::GetMesh(pxSkin->Name);
				Asura_Mesh_System::DeleteMesh(pxMesh);
#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
				xHandler.DeleteCharacterMesh(pxSkin);
#endif
				//Asura_Direct3D_CharacterRenderer::Shutdown();
				//Asura_Direct3D_CharacterRenderer::Initialise();

				pxSkin = pxSkin->m_pxNextLowerSkin;
			}

			delete m_pxSkin->m_pxNextLowerSkin;
			m_pxSkin->m_pxNextLowerSkin = 0;
		}
	
			
	

#if !defined(ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER) || !ASURA_PC_DISABLE_PRESM3_SMOOTH_SKIN_RENDERER
		for(int i=0;i<TOOLS_MAX_LOD_MODELS;i++)
		{
			if(m_afLODPercent[i]==0 || m_afLODPercent[i]==1.0f) continue;

			int iPoly = (int)(0.5f+m_afLODPercent[i]*m_pxSkinLOD->GetMaxPolygons());
			m_pxSkinLOD->GenerateSkin(iPoly,m_afLODDistances[i]);

			//m_afLODPercent[i] = iActualPoly / (float)m_pxSkinLOD->GetMaxPolygons();
		}
#endif
	}
	SetSkinModified();
	
	
//	Tools_Timed_Message_System::AddMessage(0xffffff,"LOD Models Generated");
}

void Tools_Mode_Hierarchy::CreateMarkerPointObjects()
{
	if(!Hierarchy) return;
	if(!m_pxSkin) return;
	
	for(int i=0;i<m_pxSkin->NumberOfMarkerPoints;i++)
	{
		m_xMarkerList.add_entry(new Tools_MarkerPoint_Object(Hierarchy,m_pxSkin->MarkerPoint[i].GetName().GetString(),i,0));
	}
}

void Tools_Mode_Hierarchy::DeleteMarkerPointObjects()
{
	while(m_xMarkerList.size())
	{
		Tools_Object_System::DeleteObject(m_xMarkerList.first_entry());
		m_xMarkerList.delete_first_entry();
	}
}

void Tools_Mode_Hierarchy::UpdateMarkerPointObjects()
{
	for(LIF<Tools_MarkerPoint_Object*> xMarkerLIF(&m_xMarkerList);!xMarkerLIF.done();xMarkerLIF.next())
	{
		xMarkerLIF()->UpdatePositionFromAsuraData();
	}
}

void Tools_Mode_Hierarchy::CreateMarkerPoint()
{
	if(!Hierarchy) return;
	if(!m_pxSkin) return;
	if(!m_pxSkin->SectionData) return;

	//get a name for the marker point
	GetString_Overlay xOverlay("Enter name for marker point","");
	if(!xOverlay.DoModal()) return;

	const char* szName = xOverlay.GetString();
	if(!szName || !*szName) return;

	//make sure it doesn't already exist
	int iMarkerIndex;
	if(m_pxSkin->GetMarkerPointIndex(szName,iMarkerIndex))
	{
		ShowMessageOverlay("The skin already has a marker point with that names");
		return;
	}

	//get the bone to attach it to
	ListSelection_Overlay xListOverlay("Attach the marker point to which bone?");
	for(int i=0; i<m_pxSkin->NumberOfBones; i++)
	{
		xListOverlay.AddItem(m_pxSkin->SectionData[i].SectionName.GetString(),i);
	}
	xListOverlay.AddItem("*Compass*",-1);

	if(!xListOverlay.DoModal()) return;

	int iBoneIndex = xListOverlay.GetSelectionData();

	//create a new array of marker points and add this one to the end
	Asura_Hierarchy_Marker_Point* pxNewMarkerPoints = new Asura_Hierarchy_Marker_Point[m_pxSkin->NumberOfMarkerPoints+1];
	iMarkerIndex = m_pxSkin->NumberOfMarkerPoints;
	for(int i=0; i<m_pxSkin->NumberOfMarkerPoints; i++)
	{
		pxNewMarkerPoints[i] = m_pxSkin->MarkerPoint[i];
	}
	pxNewMarkerPoints[iMarkerIndex].Position = Asura_Maths::ZeroVector3;
	pxNewMarkerPoints[iMarkerIndex].Matrix = Asura_Maths::IdentityMatrix3x3;
	pxNewMarkerPoints[iMarkerIndex].BoneIndex = iBoneIndex;
	pxNewMarkerPoints[iMarkerIndex].SetName(szName);

	
	delete [] m_pxSkin->MarkerPoint;
	m_pxSkin->MarkerPoint = pxNewMarkerPoints;
	m_pxSkin->NumberOfMarkerPoints++;

	SetSkinModified();

	//create a tools object for the new point
	Tools_MarkerPoint_Object* pxObject = new Tools_MarkerPoint_Object(Hierarchy,m_pxSkin->MarkerPoint[iMarkerIndex].GetName().GetString(),iMarkerIndex,0);
	m_xMarkerList.add_entry(pxObject);

	Tools_Object_System::SelectObject(pxObject);
}

void Tools_Mode_Hierarchy::HandleMarkerPointDeletion(Tools_MarkerPoint_Object* pxObject)
{
	Asura_Assert(s_pxThis, "");
	if(s_pxThis->m_xMarkerList.contains(pxObject))
	{
		s_pxThis->m_xMarkerList.delete_entry(pxObject);
	}

}

void Tools_Mode_Hierarchy::DeleteBoundingBoxObjects()
{
	while(m_xBoundingBoxList.size())
	{
		Tools_Object_System::DeleteObject(m_xBoundingBoxList.first_entry());
		m_xBoundingBoxList.delete_first_entry();
	}
}

void Tools_Mode_Hierarchy::CreateBoundingBoxObjects()
{
	if(!Hierarchy) return;
	if(!m_pxSkin) return;
	
	for(int i=0;i<m_pxSkin->NumberOfBones;i++)
	{
		m_xBoundingBoxList.add_entry(new Tools_SectionBoundingBox_Object(m_pxSkin,i));
	}
}

void Tools_Mode_Hierarchy::StoreAnimation(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex<TOOLS_MAX_HOTKEY_ANIM,"");

	if(Animation)
	{
		m_auHotkeyedAnimations[iIndex] = Asura_GetHashID(Animation->Name.GetString());
		Tools_Timed_Message_System::AddMessage(0xffffff,"Animation stored to %d",iIndex);
	}
}

void Tools_Mode_Hierarchy::RestoreAnimation(int iIndex)
{
	Asura_Assert(iIndex>=0 && iIndex<TOOLS_MAX_HOTKEY_ANIM,"");

	Asura_Hash_ID uAnimHash = m_auHotkeyedAnimations[iIndex];
	if(uAnimHash!=ASURA_HASH_ID_UNSET)
	{
		//Is the animation loaded?
		Asura_Hierarchy_Anim* pxAnim = Asura_Animation_System::GetAnimByHash(uAnimHash);
		if(!pxAnim)
		{
			//determine the filename from the hash id
			const char* pxFilename = Asura_HashedFilename_System::GetFullFilename(uAnimHash);
			if(!pxFilename)
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to determine filename for hotkeyed animation",iIndex);
				return;
			}

			//try loading
			if(!LoadAsuraFile(pxFilename))
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to load %s",pxFilename);
				return;
			}

			//Animation there now?
			pxAnim = Asura_Animation_System::GetAnimByHash(uAnimHash);
			if(!pxAnim)
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find animation in %s",pxFilename);
				m_auHotkeyedAnimations[iIndex] = ASURA_HASH_ID_UNSET;
				return;
			}
			

		}
		
		SelectAnimation(pxAnim,NULL);
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"No stored animation for %d",iIndex);
	}
}


Asura_Hierarchy_Anim* Tools_Mode_Hierarchy::GetNextAnimationMain(Asura_Hierarchy_Anim* pxCurrentAnim,Asura_String* pxDirectory,Asura_String* pxFilename,int iDirection)
{
	Asura_Assert(pxCurrentAnim,"");
	Asura_Assert(pxDirectory,"");

	//get the list of animations that we are cycling through
	List<Asura_String> xNameList;
	List<Asura_String> xFileNameList;

	Tools_Animation_Files::GetChooseAnimListFromDirectory(pxDirectory->GetString(),&xNameList,&xFileNameList);
	if(!xNameList.size()) return 0;

	const char* pszCurrent = pxCurrentAnim->Name.GetString();
	for(int iTries = 0;iTries<xNameList.size();iTries++)
	{
		//find the next name alphabetically in the list , looping back to the beginning if needed
		const char* pszNext = xNameList[0].GetString();
		int iNextIndex = 0;

		for(int i=1;i<xNameList.size();i++)
		{
					
			if(stricmp(pszCurrent,xNameList[i].GetString())*iDirection<0)
			{
				
				//This name is higher than the current anims name. It is better if it is lower
				//than the previous one found , or if the previous one found is lower than the current animation
				if((stricmp(xNameList[i].GetString(),pszNext)*iDirection<0) || stricmp(pszNext,pszCurrent)*iDirection<=0)
				{
					pszNext = xNameList[i].GetString();
					iNextIndex = i;
				}
			}
			else
			{
				//This name is lower than the current anims name. Take it if it is lower than the previously found name,
				//and the previously found name is also lower than the current anims name
				if((stricmp(xNameList[i].GetString(),pszNext)*iDirection<0) && stricmp(pszNext,pszCurrent)*iDirection<=0)
				{
					pszNext = xNameList[i].GetString();
					iNextIndex = i;
				}

			}
		}

		//Is the chosen animation already loaded?
		Asura_Hierarchy_Anim* pxAnimation = Asura_Animation_System::GetAnim(pszNext);
		if(!pxAnimation)
		{
			//attempt to load the animation
			LoadAsuraFile(xFileNameList[iNextIndex].GetString());

			//see if anim was loaded successfully
			pxAnimation = Asura_Animation_System::GetAnim(pszNext);
		}
		if(pxAnimation)
		{
			if(pxFilename) *pxFilename = xFileNameList[iNextIndex];
			return pxAnimation;
		}
		
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to get anim '%s'",pszNext);
		Tools_Timed_Message_System::AddMessage(0xff0000,"Anim name doesn't match filename?");
		
		//try the next in the list then
		pszCurrent = pszNext;		

	}
	//all the entries in the list were duds
	return 0;
}

Asura_Hierarchy_Anim* Tools_Mode_Hierarchy::GetNextAnimation(Asura_Hierarchy_Anim* pxCurrentAnim,Asura_String* pxDirectory,Asura_String* pxFilename)
{
	return GetNextAnimationMain(pxCurrentAnim,pxDirectory,pxFilename,1);
}

Asura_Hierarchy_Anim* Tools_Mode_Hierarchy::GetPreviousAnimation(Asura_Hierarchy_Anim* pxCurrentAnim,Asura_String* pxDirectory,Asura_String* pxFilename)
{
	return GetNextAnimationMain(pxCurrentAnim,pxDirectory,pxFilename,-1);
}


void Tools_Mode_Hierarchy::NextSkin()
{
	
	Asura_String xFilename;
	Asura_String xSkinName;
	GetNextSkin(SkinName.GetString(),xSkinName,xFilename);
	if(!xSkinName.IsBlank())
	{
		SetSkin(xSkinName.GetString(),xFilename.GetString());
	}
}

void Tools_Mode_Hierarchy::PreviousSkin()
{
	
	Asura_String xFilename;
	Asura_String xSkinName;
	GetPreviousSkin(SkinName.GetString(),xSkinName,xFilename);
	if(!xSkinName.IsBlank())
	{
		SetSkin(xSkinName.GetString(),xFilename.GetString());
	}
}

void Tools_Mode_Hierarchy::GetNextSkin(const char* szCurrentSkin,Asura_String& xOutputSkinName,Asura_String& xOutputSkinFile)
{
	GetNextSkinMain(szCurrentSkin, xOutputSkinName, xOutputSkinFile,1);
}

void Tools_Mode_Hierarchy::GetPreviousSkin(const char* szCurrentSkin,Asura_String& xOutputSkinName,Asura_String& xOutputSkinFile)
{
	GetNextSkinMain(szCurrentSkin, xOutputSkinName, xOutputSkinFile,-1);
}

void Tools_Mode_Hierarchy::GetNextSkinMain(const char* szCurrentSkin,Asura_String& xOutputSkinName,Asura_String& xOutputSkinFile,int iDirection)
{
	if(!szCurrentSkin) szCurrentSkin = "";

	//get the list of skin files
	const List<ToolsSkinFile*>& xSkinList = *Tools_Animation_Files::GetSkinList();
	if(!xSkinList.size()) return;

	
	//find the next name alphabetically in the list , looping back to the beginning if needed
	const char* szNext = xSkinList[0]->m_xName.GetString();
	int iNextIndex = 0;

	for(int i=1;i<xSkinList.size();i++)
	{
				
		const char* szSkinName = xSkinList[i]->m_xName.GetString();
		if(stricmp(szCurrentSkin,szSkinName)*iDirection<0)
		{
			
			//This name is higher than the current skin's name. It is better if it is lower
			//than the previous one found , or if the previous one found is lower than the current skin
			if((stricmp(szSkinName,szNext)*iDirection<0) || stricmp(szNext,szCurrentSkin)*iDirection<=0)
			{
				szNext = szSkinName;
				iNextIndex = i;
			}
		}
		else
		{
			//This name is lower than the current skin's name. Take it if it is lower than the previously found name,
			//and the previously found name is also lower than the current skins name
			if((stricmp(szSkinName,szNext)*iDirection<0) && stricmp(szNext,szCurrentSkin)*iDirection<=0)
			{
				szNext = szSkinName;
				iNextIndex = i;
			}

		}
	}

	xOutputSkinName = szNext;
	xOutputSkinFile = xSkinList[iNextIndex]->m_xFilename.GetString();
	
}




void Tools_Mode_Hierarchy::BackupAnimationKeyFrames()
{
	if(!Animation) return;
	if(!Animation->BoneAnims) return;

	//resize backup array if needed
	if(Animation->NumberOfBones != m_iBackupAnimationArraySize)
	{
		delete [] m_pxBackupAnimation;
		m_iBackupAnimationArraySize = Animation->NumberOfBones;
		m_pxBackupAnimation = new Asura_Hierarchy_Bone_Anim[m_iBackupAnimationArraySize];
	}

	//copy the frames
	for(int i=0;i<Animation->NumberOfBones;i++)
	{
		m_pxBackupAnimation[i] = Animation->BoneAnims[i];
	}
}

void Tools_Mode_Hierarchy::RestoreAnimationKeyFrames()
{
	if(!Animation) return;
	if(!Animation->BoneAnims) return;

	if(Animation->NumberOfBones != m_iBackupAnimationArraySize)
	{
		Asura_Assert(Animation->NumberOfBones == m_iBackupAnimationArraySize,"Backup keyframe data incorrect");
		return;
	}

	//copy the frames
	for(int i=0;i<Animation->NumberOfBones;i++)
	{
		Animation->BoneAnims[i] = m_pxBackupAnimation[i];
	}
	
}

void Tools_Mode_Hierarchy::RemoveUnneededKeyFrames()
{
	m_bUpdateFrameRemoval = false;
	if(!Animation) return;
	if(!Animation->BoneAnims) return;

	RestoreAnimationKeyFrames();

	if(m_bDoFrameRemoval)
	{
		Animation->RemoveUnneededKeyFrames(m_fPositionTolerance,m_fAngleTolerance,&m_iFramesBefore,&m_iFramesAfter);
	}
}

Asura_Vector_3 Tools_Mode_Hierarchy::GetHierarchyPosition()
{
	Asura_Vector_3 xPos = g_xHierarchyModeOrigin;
	if(s_pxThis && s_pxThis->Hierarchy && s_pxThis->m_bUseMovementOffsets)
	{
		Asura_Vector_3 xOffset;
		if(s_pxThis->Hierarchy->GetMovementOffset(&xOffset))
		{
			xPos += xOffset;
		}

	}
	return xPos;
}


void Tools_Mode_Hierarchy::CreateLight()
{
	//create a light at the current camera position
	Tools_Photon_Object* pxPhoton = new Tools_Photon_Object(Asura_Camera::GetPosition(),Asura_Camera::GetOrientation());

	//and select it
	Tools_Object_System::SelectObject(pxPhoton);
}


const Asura_Vector_3& Tools_Mode_Hierarchy::GetHierarchyModeOrigin()
{
	return g_xHierarchyModeOrigin;
}

void Tools_Mode_Hierarchy::CalculateAnimationBB(bool bAlwaysGenerate)
{
	Asura_Assert(Hierarchy && Animation && m_pxSkin,"");

	Asura_Vector_3 xHierPos(0,0,0);
	Asura_Matrix_3x3 xHierOrient;
	xHierOrient.Identity();

	Asura_Vector_3* pxBonePositions = new Asura_Vector_3[m_pxSkin->NumberOfBones];
	Asura_Matrix_3x3* pxBoneOrients = new Asura_Matrix_3x3[m_pxSkin->NumberOfBones];

	Asura_Bounding_Box xBB;
	xBB.Reset();

	bool bBoundingBoxNeeded = bAlwaysGenerate ? true : false;
	float fSkinRadius = m_pxSkin->GetBoundingRadius();
	
    Asura_Hierarchy_Tween* pxTweenStorage = Hierarchy->GetTweenStore();

	for(float fTime = 0;fTime<=1.0f;fTime += 0.01f)
	{
		//calculate the bone positions at this time
		Hierarchy->m_xPrimaryWrapper.SetTimer(fTime);
		Hierarchy->InvalidateBonesUpdateFrameStamp();
		Hierarchy->Update(xHierPos,xHierOrient,0,ASURA_HIERARCHY_RENDER_UPDATEBONESALWAYS|ASURA_HIERARCHY_RENDER_DONTRENDER);
		
		//calculate the absolute bone positions/orientations
		for(int i=0;i<m_pxSkin->NumberOfBones;i++)
		{
			Asura_Vector_3		xPos;
			Asura_Matrix_3x3	xMat;

			pxTweenStorage->GetBoneData(i).GetOrientation(xMat);
			pxTweenStorage->GetBoneData(i).GetPosition(xPos);

			if(i>0)
			{
				int iParent = m_pxSkin->ParentBone[i];
				xPos = pxBonePositions[iParent] + xPos*pxBoneOrients[iParent];
				xMat = pxBoneOrients[iParent]*xMat;
			}
			pxBonePositions[i] = xPos;
			pxBoneOrients[i] = xMat;
		}

		//get the centre of the skin's bounding box for this frame
		Asura_Vector_3 xSkinBBCentre = m_pxSkin->GetBB().CalculateCentre();
		
		//offset tby movement of the first bone
		Asura_Vector_3	xBonePos;

		pxTweenStorage->GetBoneData(0).GetPosition(xBonePos);
#if defined(ASURA_HIERARCHY_QUAD)
		Asura_Vector_3	xPosition = m_pxSkin->m_pxBoneRestPoseTransforms[0].GetTranslation();
		xSkinBBCentre += xBonePos - xPosition;
#else
		xSkinBBCentre += xBonePos - m_pxSkin->BoneRestPosition[0];
#endif	//defined(ASURA_HIERARCHY_QUAD)

		if(m_pxSkin->IsSkinnedModel())
		{
			//Calculate each vertex's position , and update BB
			for(int v=0;v<m_pxSkin->NumberOfVertices;v++)
			{
				Asura_Vector_3 xPos(0,0,0);
				for (int b=0; b<4; b++)
				{
					float fWeight = m_pxSkin->VertexData[v].Weight[b];
					if (fWeight==-1.0f) break;
					
					int iBone = m_pxSkin->VertexData[v].BoneID[b];

					Asura_Matrix_3x3& m = pxBoneOrients[iBone];
					Asura_Vector_3& p = pxBonePositions[iBone];
					
					xPos += ((m_pxSkin->VertexData[v].Offset[b]*m) + p)*fWeight;

				}
				xBB.Set(xPos);

				if(!bBoundingBoxNeeded)
				{
					//check to see if this point is covered by the skin's bounding sphere
					float fDist = (xPos-xSkinBBCentre).Magnitude();
					if(fDist>fSkinRadius)
					{
						bBoundingBoxNeeded = true;
					}
				}
			}
		}
		else
		{
			Asura_Assert(m_pxSkin->SectionData,"");
			//determine the position of each vertex in each mesh , updating the BB as we go
			for(int i=0;i<m_pxSkin->NumberOfBones;i++)
			{
				Asura_Mesh* pxMesh = m_pxSkin->SectionData[i].Mesh;
				if(!pxMesh) continue;
				
				for(int j=0;j<pxMesh->NumberOfVertices;j++)
				{
					Asura_Vector_3 xPos = pxMesh->VertexDataPtr[j].Position;
					xPos = xPos*pxBoneOrients[i] + pxBonePositions[i];

					xBB.Set(xPos);

					if(!bBoundingBoxNeeded)
					{
						//check to see if this point is covered by the skin's bounding sphere
						float fDist = (xPos-xSkinBBCentre).Magnitude();
						if(fDist>fSkinRadius)
						{
							bBoundingBoxNeeded = true;
						}
					}

				}	
			}
		}

	}
	//update the bounding box if it was determined that it is needed , or if the animation already had one
	if(bBoundingBoxNeeded || Animation->GetBB())
	{
		Animation->SetBB(&xBB);
		SetAnimModified();
	}

	delete [] pxBonePositions;
	delete [] pxBoneOrients;
}

void Tools_Mode_Hierarchy::CheckAnimBoundingBoxes()
{
	if(!m_pxSkin)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"A skin needs to be selected first.");
		return;
	}
	
	char szRootDir[MAX_PATH];
	GetCurrentDirectory(MAX_PATH,szRootDir);
		
	//ask for the directory to do
	Tools_FileSystem_SelectFolderInfo xInfo;
	xInfo.StartDirectory = szRootDir;
	xInfo.DisplayText = "Check bounding boxes for anims in which directory?";
	xInfo.InitialSelection = Tools_Animation_Files::GetDefaultAnimDirForSkin(SkinName.GetString());

	if(!Tools_FileSystem::SelectFolder(xInfo))
	{
		return;
	}

	//get a list of all the asura files in the chosen directory and subdirs
	List<Asura_String> xFileList;
	char szSearch[MAX_PATH];
	Asura_PrintfToBfr( szSearch, "%s\\*.asr", MAX_PATH, true, xInfo.SelectedFolder );      
	Tools_FileSystem::FindFiles( szSearch, xFileList, TOOLS_FILEFIND_RECURSIVE|TOOLS_FILEFIND_IGNOREBACKUPDIR );

	//store the currently selection anim
	Asura_Hierarchy_Anim* pxInitialAnim = Animation;
	Asura_String xInitialAnimFilename = m_xAnimFilename;
	
//	Tools_ProgressIndicator xProgress("Checking bounding boxes");
	int iIndex = 0;
	
	//process all the files
	Tools_AsuraFileContents xFileContents;
	for(LIF<Asura_String> xFileLIF(&xFileList);!xFileLIF.done();xFileLIF.next(),iIndex++)
	{
		//xProgress.Update(iIndex,xFileList.size());

		const char* pszFilename = xFileLIF().GetString();
		//make sure this is an anim file
		xFileContents.LoadAsuraFile(pszFilename);
		if(xFileContents.GetAnimList()->size() != 1) continue;

		const char* pszAnimName = xFileContents.GetAnimList()->first_entry().GetString();

		//anim already loaded?
		Asura_Hierarchy_Anim* pxAnim = Asura_Animation_System::GetAnim(pszAnimName);
		if(!pxAnim)
		{
			if(!LoadAsuraFile(pszFilename))
			{
				//xProgress.AddMessage("Failed to load %s",Tools_FileSystem::StripPath(pszFilename));
				continue;
			}
			pxAnim = Asura_Animation_System::GetAnim(pszAnimName);
			if(!pxAnim)
			{
				//xProgress.AddMessage("Failed to get anim from %s",Tools_FileSystem::StripPath(pszFilename));
				continue;
			}
		}

		if(pxAnim->NumberOfBones != m_pxSkin->NumberOfBones)
		{
			//anim doesn't work with this skin , so skip it
			continue;
		}


		//select this anim
		SelectAnimation(pxAnim,pszFilename);
		//and do the bounding box
		CalculateAnimationBB(false);
	}

	if(pxInitialAnim)
	{
		SelectAnimation(pxInitialAnim,xInitialAnimFilename.GetString());
	}
	

	Tools_Timed_Message_System::AddMessage(0xffffff,"Finished checking anim bounding boxes.");
}

bool Tools_Mode_Hierarchy::CanEditCollisionBB()
{
	if(!m_pxSkin) return false;
	//don't allow BB to be edited , if a collision mesh exists
	return !m_pxSkin->HasCollisionMesh();
}

bool Tools_Mode_Hierarchy::CanDrawCollisionMesh()
{
	if(!m_pxSkin) return false;
	// only draw a collision mesh if it exists
	return m_pxSkin->HasCollisionMesh();
}

bool Tools_Mode_Hierarchy::IsObjectHierarchy()
{
	if(!m_pxSkin) return false;
	return !(m_pxSkin->IsSkinnedModel());
}
void Tools_Mode_Hierarchy::RecalculateSectionCollisionMesh()
{
	if(!m_pxSkin) return;
	if(m_pxSkin->IsSkinnedModel()) return;

	for(int i=0; i<m_pxSkin->NumberOfBones; i++)
	{
		if(m_pxSkin->SectionData[i].Mesh)
		{
			if(!m_pxSkin->SectionData[i].m_pxCollisionMesh)
			{
				m_pxSkin->SectionData[i].m_pxCollisionMesh = new Asura_CollisionMesh;
			}
			m_pxSkin->SectionData[i].m_pxCollisionMesh->UpdateFromMesh(m_pxSkin->SectionData[i].Mesh);
		}				
	}
	Tools_Timed_Message_System::AddMessage(0xffffff,"Updated the section collision meshes.");
	SetSkinModified();
}


void Tools_Mode_Hierarchy::EditCollisionBB()
{
	if(!CanEditCollisionBB()) return;
	
	if(!m_pxSkinCollisionVolume)
	{
		new Tools_VolumeEdit_Object("Skin Collision BB",&m_xSkinCollisionBB,&m_pxSkinCollisionVolume,&m_bSkinModified);
		m_pxSkinCollisionVolume->SetOffset(g_xHierarchyModeOrigin);

		Tools_Object_System::SelectObject(m_pxSkinCollisionVolume);
	}	
}

void Tools_Mode_Hierarchy::ResetCollisionBB()
{
	if(!CanEditCollisionBB()) return;
	if(!m_pxSkin) return;
	
	YesNo_Overlay xOverlay("Reset the bounding box for this skin to match the rest pose?");
	if(!xOverlay.DoModal()) return;
	
	m_pxSkin->ResetCollisionBB();

	if(m_pxSkinCollisionVolume) m_pxSkinCollisionVolume->Destroy();

	SetSkinModified();
}

void Tools_Mode_Hierarchy::StartLODBatch()
{
	m_iBatchStep = 0;

	//ask for text file to load
	Tools_FileSystem_DialogInfo	xFileSelector;

	xFileSelector.DefaultExtension		= ".txt";
	xFileSelector.DefaultFilename		= "";
	xFileSelector.DefaultPath			= ".";
	xFileSelector.FileFilter			= "Text Files\0*.txt;\0";
	xFileSelector.WindowTitle			= "Read LOD batch settings from which file?";
	xFileSelector.AllowMultipleSelect	= false;

	Asura_String xFilename = Tools_FileSystem::GetLoadFilename( xFileSelector );

	Tools_IniFile xIniFile(xFilename.GetString(),false);
	if(!xIniFile.LoadIniFile())
	{
		ShowMessageOverlay("Failed to load %s",xFilename.GetString());
	}


	//read the settings from the file
	for(int i=0;i<xIniFile.GetNumberOfSections();i++)
	{
		xIniFile.GetIniTextSectionByIndex(i);

		//get the LOD settings for this section
		Tools_LODBatchEntry xDefault;
		bool bOkay = true;
		for(int i=0;i<TOOLS_MAX_LOD_MODELS;i++)
		{
			xDefault.m_afDistance[i] = 10.0f * i;
			xDefault.m_afPercent[i] = 0;
						
			char szDistance[20];
			char szDetail[20];
			sprintf(szDistance,"LOD_Distance%d",i);
			sprintf(szDetail,"LOD_Detail%d",i);

			xIniFile.GetFloatInCurrentSection(szDistance,xDefault.m_afDistance[i]);
			xIniFile.GetFloatInCurrentSection(szDetail,xDefault.m_afPercent[i]);

			if(xDefault.m_afDistance[i]<0)
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"Distance %d for %s isn't valid.",i,xIniFile.GetCurrentSectionName());
				bOkay = false;
			}

			if(xDefault.m_afPercent[i]<0 || xDefault.m_afPercent[i] > 1)
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"LOD Detail %d for %s isn't valid. It must be between 0 and 1.",i,xIniFile.GetCurrentSectionName());
				bOkay = false;
			}
		}
		if(!bOkay) continue;

		//now scan for all the filenames in this section
		xIniFile.GetIniTextSectionByIndex(i);
		while(xIniFile.GetNextIniTextLine())
		{
			Asura_String xLine;
			if(!xIniFile.GetString(xLine)) continue;
			if(xLine.IsBlank()) continue;

			//ignore lines if it contains '='
			if(strchr(xLine.GetString(),'=')) continue;

			//add an enrty for this then
			Tools_LODBatchEntry* pxEntry = new Tools_LODBatchEntry;
			*pxEntry = xDefault;
			pxEntry->m_xFilename = xLine;

			m_xLODBatchList.add_entry(pxEntry);

		}
	}
}

void Tools_Mode_Hierarchy::UpdateLODBatch()
{
	if(!m_xLODBatchList.size()) return;
	Tools_LODBatchEntry* pxEntry = m_xLODBatchList.first_entry();

	if(m_iBatchStep == 0)
	{
		//make sure the file exists
		if(!Tools_FileSystem::FileExists(pxEntry->m_xFilename.GetString()))
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find %s",pxEntry->m_xFilename.GetString());
			delete pxEntry;
			m_xLODBatchList.delete_first_entry();
			return;
		}

		//make sure the file isn't read-only
		if(Tools_FileSystem::IsReadOnly(pxEntry->m_xFilename.GetString()))
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"%s isn't writeable",pxEntry->m_xFilename.GetString());
			delete pxEntry;
			m_xLODBatchList.delete_first_entry();
			return;
		}

		//extract the skin name from the filename
		Asura_String xSkinName;
		pxEntry->m_xFilename.SplitFilename(0,&xSkinName,0);
	
		if(SetSkin(xSkinName.GetString(),pxEntry->m_xFilename.GetString()))
		{
			//make sure skin doesn't have manual LODs
			if(m_pxSkin->m_bManualLOD)
			{
				Tools_Timed_Message_System::AddMessage(0xff0000,"%s has manually generated LOD models",xSkinName.GetString());
				delete pxEntry;
				m_xLODBatchList.delete_first_entry();
				return;
			}

			Tools_Timed_Message_System::AddMessage(0xffffff,"%s loaded",pxEntry->m_xFilename.GetString());
			m_iBatchStep++;
		}
		else
		{
			Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to get %s from %s",xSkinName.GetString(),pxEntry->m_xFilename.GetString());
			delete pxEntry;
			m_xLODBatchList.delete_first_entry();
			return;
		}
	}
	else if(m_iBatchStep == 1)
	{
		//initialise the LOD settings
		for(int i=0;i<TOOLS_MAX_LOD_MODELS;i++)
		{
			m_afLODDistances[i] = pxEntry->m_afDistance[i];
			m_afLODPercent[i] = pxEntry->m_afPercent[i];
		}
		//generate the LOD models
		GenerateLODModels();
		Tools_Timed_Message_System::AddMessage(0xffffff,"LOD models generated");
		m_iBatchStep++;
	}
	else if(m_iBatchStep == 2)
	{
		//and save the changes
		Tools_SaveAsura::SaveModifiedAsuraFiles();

		//finished on this file
		delete pxEntry;
		m_xLODBatchList.delete_first_entry();
		m_iBatchStep = 0;
	}


	if(!m_xLODBatchList.size())
	{
		//all done now
		Tools_Timed_Message_System::AddMessage(0xffffff,"Finished");
	}

}

void Tools_Mode_Hierarchy::CancelLODBatch()
{
	if(m_xLODBatchList.size())
	{
		while(m_xLODBatchList.size())
		{
			delete m_xLODBatchList.first_entry();
			m_xLODBatchList.delete_first_entry();
		}
		Tools_Timed_Message_System::AddMessage(0xffffff,"Aborted");

	}
}

bool Tools_Mode_Hierarchy::AnimIsPosable()
{
	return Hierarchy && Hierarchy->IsPosableAnim();
}

void Tools_Mode_Hierarchy::ResetPosableAnim()
{
	m_xPosableHipOrient.Zero();
	m_xPosableHeadOrient.Zero();
}

void Tools_Mode_Hierarchy::EditTestSounds()
{
	//create the test sound overlay if not already in existence
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_HIERARCHY_TESTSOUNDS))
	{
		Asura_Overlay_System::BringOverlayToTop(TOOLS_OVERLAY_HIERARCHY_TESTSOUNDS);
		return;
	}

	if(!AnimationFlagList)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find list of flag names");
		return;
	}

	BuildTestSoundNameArray();


	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("Test Sounds",false,&m_xTestSoundOverlayPosition);

	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	Asura_Dialog* pxDialog = new Asura_Function_Dialog("Add",xOffset,_AddTestSound);
	pxDialog->SetHelpText("Add a new entry to the list of flags with test sounds attached.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x += 12.0f;

	pxDialog = new Asura_Function_Dialog("Delete",xOffset,_DeleteTestSound);
	pxDialog->SetHelpText("Delete the selected entry from the list.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = 1.0f;
	xOffset.y+=Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Load Defaults",xOffset,_LoadDefaultTestSounds);
	pxDialog->SetHelpText("Loads some default test sound settings that have been set up within a text file");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = 1.0f;
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	//heading for list box
	pxDialog = new Asura_Dialog("Flags :",xOffset);	
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::SmallSpacing();

	//and the list box
	m_bTestSoundIndexChanged = false;
	pxDialog = new Asura_List_Dialog(xOffset,m_pszTestSoundNames,&m_iTestSoundIndex,8,m_iTestSoundNameArraySize,NULL,&m_bTestSoundIndexChanged);
	pxDialog->m_iControlID = DialogID_TestSoundList;
	pxDialog->SetHelpText("List of flags to trigger test sounds.\nDetermines the currently selected sound.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y = pxDialog->m_xMaxCoord.y + Asura_Dialog::SmallSpacing();


	//sound event selection controls
	m_uTestSoundEvent = (m_iTestSoundIndex < m_iNumTestSounds) ? m_pxTestSounds[m_iTestSoundIndex].m_uSoundEvent : 0;

	m_bTestSoundEventChanged = false;
	Asura_Dialog* pxSoundEventDialog = pxDialog = new Tools_PickSoundEvent_Dialog("Event : ", xOffset, &m_uTestSoundEvent, &m_bTestSoundEventChanged);
	pxDialog->SetIsEnabledFunction(_IsEnabled_TestSoundEvent);
	pxDialog->SetHelpText("Pick sound event to use with this flag.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//add control for editing the selected sound event
	pxDialog = new Tools_EditSoundEvent_Dialog("Edit",xOffset,&m_uTestSoundEvent);
	pxDialog->SetIsEnabledFunction(_IsEnabled_TestSoundEvent);
	pxOverlay->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxSoundEventDialog,Asura_Vector_2(3.0f,0.0f),false,true);

	pxOverlay->m_uID = TOOLS_OVERLAY_HIERARCHY_TESTSOUNDS;
 	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);
}

void Tools_Mode_Hierarchy::BuildTestSoundNameArray()
{
	Asura_Assert(AnimationFlagList,"");

	DeleteTestSoundNameArray();

	if(m_iNumTestSounds)
	{
		m_iTestSoundNameArraySize = m_iNumTestSounds;
		m_pszTestSoundNames = new char*[m_iNumTestSounds];
		for(int i=0;i<m_iNumTestSounds;i++)
		{
			int iFlag = m_pxTestSounds[i].m_iFlag;
							
			const char* szFlagName = AnimationFlagList->GetFlagName(iFlag);

			if(szFlagName)
			{
				m_pszTestSoundNames[i] = new char[strlen(szFlagName)+1];
				strcpy(m_pszTestSoundNames[i],szFlagName);
			}
			else
			{
				m_pszTestSoundNames[i] = new char[20];
				strcpy(m_pszTestSoundNames[i],"*Unknown*");
			}
		}
	}
	else
	{
		//no sounds , so just create one entry for the list control to show that
		m_iTestSoundNameArraySize = 1;
		m_pszTestSoundNames = new char*[1];
		m_pszTestSoundNames[0] = new char[10];
		strcpy(m_pszTestSoundNames[0],"*None*");
	}

	m_iTestSoundIndex = Asura_Maths::ClampToRange(m_iTestSoundIndex,0,m_iTestSoundNameArraySize-1);

	//update overlay if it exists
	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(TOOLS_OVERLAY_HIERARCHY_TESTSOUNDS);
	if(pxOverlay)
	{
		Asura_Dialog* pxDialog = pxOverlay->GetDialog(DialogID_TestSoundList);
		if(pxDialog)
		{
			Asura_List_Dialog* pxListDialog = (Asura_List_Dialog*) pxDialog;
			pxListDialog->ReplaceStrings(m_iTestSoundNameArraySize,m_pszTestSoundNames);
		}
	}
	
}

void Tools_Mode_Hierarchy::DeleteTestSoundNameArray()
{
	for(int i=0;i<m_iTestSoundNameArraySize;i++)
	{
		delete [] m_pszTestSoundNames[i];
	}
	delete [] m_pszTestSoundNames;
	m_pszTestSoundNames = 0;
	m_iTestSoundNameArraySize = 0;
}

void Tools_Mode_Hierarchy::AddTestSound()
{
	//create an overlay for selecting the flags
	ListSelection_Overlay xOverlay("Select flag for test sound");

	for(LIF<NameAndValue> xFlagLIF(&AnimationFlagList->ValueList);!xFlagLIF.done();xFlagLIF.next())
	{
		xOverlay.AddItem(xFlagLIF().Name.GetString(),xFlagLIF().Value);
	}
	
	if(!xOverlay.DoModal()) return;

	int iFlag = xOverlay.GetSelectionData();

	//enlarge the test sound array
	Tools_FlagSoundPair* pxTestSounds = new Tools_FlagSoundPair[m_iNumTestSounds+1];
	for(int i=0;i<m_iNumTestSounds;i++)
	{
		pxTestSounds[i] = m_pxTestSounds[i];
	}
	pxTestSounds[m_iNumTestSounds].m_iFlag = iFlag;
	pxTestSounds[m_iNumTestSounds].m_uSoundEvent = 0;
	delete [] m_pxTestSounds;
	m_pxTestSounds = pxTestSounds;
	m_iNumTestSounds++;


	//update the list , and select the new sound
	BuildTestSoundNameArray();

	m_iTestSoundIndex = m_iNumTestSounds-1;
	m_uTestSoundEvent = m_pxTestSounds[m_iTestSoundIndex].m_uSoundEvent;
}

void Tools_Mode_Hierarchy::DeleteTestSound()
{
	if(m_iTestSoundIndex >= m_iNumTestSounds) return;

	//update the test sound array
	for(int i=m_iTestSoundIndex;i<m_iNumTestSounds-1;i++)
	{
		m_pxTestSounds[i] = m_pxTestSounds[i+1];
	}
	m_iNumTestSounds--;

	//update the list , and the current selection
	BuildTestSoundNameArray();
	m_uTestSoundEvent = (m_iTestSoundIndex < m_iNumTestSounds) ? m_pxTestSounds[m_iTestSoundIndex].m_uSoundEvent : 0;

}

void Tools_Mode_Hierarchy::LoadDefaultTestSounds()
{
	//attempt to load the ini file
	const char* szFilename = "ToolsIniFiles\\HierarchyTestSounds.ini";
	Tools_IniFile xIni(szFilename);
	if(!xIni.LoadIniFile())
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to load %s",szFilename);
		return;
	}

	//Search for a section named after the current skin , or one named default
	bool bFoundSection = false;
	if(!SkinName.IsBlank())
	{
		if(xIni.GetIniTextSection(SkinName.GetString()))
		{
			bFoundSection = true;
		}
	}
	if(!bFoundSection)
	{
		if(xIni.GetIniTextSection("Default"))
		{
			bFoundSection = true;
		}
	}
	if(!bFoundSection)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find a section heading named Default or named after the skin in %s",szFilename);
		return;
	}


	List<int> xFlagList;
	List<Asura_Hash_ID> xSoundList;

	//get the list of flag names from the ini section
	List<Asura_String> xFlagNameList;
	xIni.ListIdentifiersInCurrentSection(xFlagNameList);

	//parse each one
	for(LIF<Asura_String> xFlagNameLIF(&xFlagNameList); !xFlagNameLIF.done(); xFlagNameLIF.next())
	{
		//find the value of the matching flag
		int iFlag;
		if(!AnimationFlagList->GetValueFromName(xFlagNameLIF().GetString(), iFlag))
		{
			Tools_Timed_Message_System::AddMessage(0xffff00,"Unknown flag name : %s",xFlagNameLIF().GetString());
			continue;
		}

		//get the name of the sound event
		Asura_String xEventName;
		if(!xIni.GetStringInCurrentSection(xFlagNameLIF().GetString(),xEventName))
		{
			Asura_Assert(0,"Unexpected error getting sound event name for flag");
			continue;
		}

		//check to see if the sound event exists
		Asura_Hash_ID uHash = xEventName.GetHash();
		if(!Asura_Sound_Event_Storage::FindEvent(uHash))
		{
			Tools_Timed_Message_System::AddMessage(0xffff00,"Flag '%s' is using sound event '%s' , which doesn't exist",xFlagNameLIF().GetString(),xEventName.GetString());
			continue;
		}
		//all is in order if we've got here , so add to the lists
		xFlagList.add_entry(iFlag);
		xSoundList.add_entry(uHash);
	}

	//replace the test sound list
	m_iNumTestSounds = xFlagList.size();
	delete [] m_pxTestSounds;
	m_pxTestSounds = m_iNumTestSounds ? new Tools_FlagSoundPair[m_iNumTestSounds] : 0;
	for(int i=0; i<m_iNumTestSounds; i++)
	{
		m_pxTestSounds[i].m_iFlag = xFlagList[i];
		m_pxTestSounds[i].m_uSoundEvent = xSoundList[i];
	}

	//update the list for the overlay
	BuildTestSoundNameArray();

	m_iTestSoundIndex = 0;
	m_uTestSoundEvent = m_iNumTestSounds  ? m_pxTestSounds[0].m_uSoundEvent : 0;


}

bool Tools_Mode_Hierarchy::IsEnabled_TestSoundEvent()
{
	return m_iNumTestSounds>0;
}

void Tools_Mode_Hierarchy::UpdateTestSounds()
{
	if(m_bTestSoundIndexChanged)
	{
		m_uTestSoundEvent = (m_iTestSoundIndex < m_iNumTestSounds) ? m_pxTestSounds[m_iTestSoundIndex].m_uSoundEvent : 0;
		m_bTestSoundIndexChanged = false;
	}
	if(m_bTestSoundEventChanged)
	{
		if(m_iTestSoundIndex < m_iNumTestSounds)
		{
			//make sure a looping sound hasn't been chosen
			const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::FindEvent(m_uTestSoundEvent);
			if(pxEvent && pxEvent->IsEndlessPlayback())
			{
				ShowMessageOverlay("Looping sound events can't be used for animation sounds.");
				m_uTestSoundEvent = m_pxTestSounds[m_iTestSoundIndex].m_uSoundEvent;
			}
			else if(!pxEvent)
			{
				m_uTestSoundEvent = m_pxTestSounds[m_iTestSoundIndex].m_uSoundEvent;
			}
			else
			{
				m_pxTestSounds[m_iTestSoundIndex].m_uSoundEvent = m_uTestSoundEvent;
			}
			
		}
		m_bTestSoundEventChanged = false;
	}
}


void Tools_Mode_Hierarchy::AnimationControl_UpdateSlideTimer()
{
	//update the slide timer from the timer scaled 0 to 1
	switch ( m_eAnimationControl_TimerMode )
	{
		case TIMER_PROP:
		{
			m_fAnimationControl_SlideTimer = m_fAnimationControl_Timer;
			break;
		}
		case TIMER_SECONDS:
		{
			m_fAnimationControl_SlideTimer = m_fAnimationControl_Timer*AnimDuration;
			break;
		}

		case TIMER_FRAMES:
		{
			m_fAnimationControl_SlideTimer = m_fAnimationControl_Timer*m_iAnimFrames;
			break;
		}

		case TIMER_25FPS:
		case TIMER_30FPS:
		{
			m_fAnimationControl_SlideTimer = m_fAnimationControl_Timer*m_fTimeSliderMax;
			break;
		}

	}

	UpdateAnimationSoundTimeDisplay();
	UpdateAnimationFlagTimeDisplay();
}

void Tools_Mode_Hierarchy::AnimationControl_UpdateTimer()
{

	switch(m_eAnimationControl_TimerMode)
	{
		case TIMER_PROP :
			m_fAnimationControl_Timer = m_fAnimationControl_SlideTimer;
			break;

		case TIMER_SECONDS :
			m_fAnimationControl_Timer = AnimDuration ? m_fAnimationControl_SlideTimer/AnimDuration : 0;
			break;

		case TIMER_FRAMES :
			//make sure slide timer is an int first
			m_fAnimationControl_SlideTimer = (float)Asura_Maths::FloatToInt_Near(m_fAnimationControl_SlideTimer);

			m_fAnimationControl_Timer = m_fAnimationControl_SlideTimer/((float)m_iAnimFrames);
			break;

		case TIMER_25FPS :
		case TIMER_30FPS :
			//make sure slide timer is an int first
			m_fAnimationControl_SlideTimer = (float)Asura_Maths::FloatToInt_Near(m_fAnimationControl_SlideTimer);

			m_fAnimationControl_Timer = m_fAnimationControl_SlideTimer/(m_fTimeSliderMax);
			break;

	}

	if(m_fAnimationControl_Timer<0.0f) m_fAnimationControl_Timer = 0.0f;
	if(m_fAnimationControl_Timer>1.0f) m_fAnimationControl_Timer = 1.0f;
}


int Tools_Mode_Hierarchy::GetNumAnimFrames(const Asura_Hierarchy_Anim* pxAnim) const
{

	//guess the original number of frames
	if(!pxAnim->BoneAnims) return 1;

	int iBestFrames = 1;
	float fBestError = 10000000.0f;

	for(int iFrames=1;iFrames<500;iFrames++)
	{
		float fError = 0.0f;
		for(int i=0;i<pxAnim->NumberOfBones;i++)
		{
			for(int j=0;j<pxAnim->BoneAnims[i].NumberOfKeyFrames;j++)
			{
				float fFrameIndex = pxAnim->BoneAnims[i].KeyFrames[j].Time * iFrames;
				
				fError += Asura_Maths::Abs(fFrameIndex - Asura_Maths::FloatToInt_Near(fFrameIndex));
			}
		}


		if(fError<=0.001f)
		{
			return iFrames;
		}
		if(fError<fBestError)
		{
			iBestFrames = iFrames;
			fBestError = fError;
		}
	}
	
	return iBestFrames;

}

void Tools_Mode_Hierarchy::WriteAVI()
{
	if(!Animation || !m_pxSkin)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"A skin and animation must be chosen before an AVI can be created.");
		return;
	}
	
	//ensure screenshots directory exists
	CreateDirectory("Screenshots",0);

	//generate name for the AVI
	char szFilename[MAX_PATH];
	Asura_PrintfToBfr(szFilename,"Screenshots\\%s.avi",MAX_PATH,true,AnimName.GetString());

	//determine frames per second
	float fFPS = 25.0f;
	if(AnimDuration)
	{
		fFPS = m_iAnimFrames/AnimDuration;
	}

	//start recording
	if(TheAviWriter.StartAVI(szFilename,fFPS))
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Writing %s",szFilename);
		m_iAVIFrame = 0;
		m_bRecordingAVI = true;
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Error creating %s",szFilename);
		m_bRecordingAVI = false;

	}
}


void Tools_Mode_Hierarchy::SaveAttachments()
{
	Tools_MarkerPoint_Object::StoreMarkerPointSettings(m_xMarkerList);
}

void Tools_Mode_Hierarchy::RestoreAttachments()
{
	Tools_MarkerPoint_Object::RestoreMarkerPointSettings(m_xMarkerList);
}


void Tools_Mode_Hierarchy::OutputSkinHierarchyTextFile(bool bFullFile)
{
	if(!m_pxSkin) return;
	
	//get a filename to save as
	//ask for a filename
	Tools_FileSystem_DialogInfo xInfo;

	xInfo.DefaultExtension = "txt";

	xInfo.DefaultFilename = m_pxSkin->Name.GetString();
	xInfo.DefaultPath = ".";
	xInfo.FileFilter = "Text Files\0*.txt\0";
	xInfo.WindowTitle = "Save hierarchy text file as :";

	Asura_Char* pszName = Tools_FileSystem::GetSaveFilename(xInfo);
	if(!pszName) return;
		
	CreateSkinHierarchyTextFile(m_pxSkin,pszName,bFullFile);
}

void Tools_Mode_Hierarchy::CreateSkinHierarchyTextFile(Asura_Hierarchy_Skin* pxSkin,const char* pszFilename,bool bFullFile)
{
	
	if(!pxSkin->SectionData)
	{
		//oughtn't to be any hierarcies old enough for this to be an issue
		Asura_Assert(pxSkin->SectionData,"Can't get bone names for skin");
		return;
	}
	
	FILE* pFile = fopen(pszFilename,"wt");
	if(!pFile)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to open %s for writing",pszFilename);
		return;
	}

	fprintf(pFile,"*** %s ***\n\n",pxSkin->Name.GetString());

	fprintf(pFile,"*Bone Order*\n\n");
	u_int uRenderBoneCount(0);

	for(int iBone=0; iBone<pxSkin->NumberOfBones; ++iBone)
	{
		// VGP 17Mar2006 Count number of verts attached to this bone
		u_int uVertCount(0);

		for (int iVert = 0; iVert != pxSkin->NumberOfVertices; ++iVert)
		{
			const Asura_Hierarchy_Skin_Vertex& xVert = pxSkin->VertexData[iVert];

			for (int i=0; i!=4; ++i)
			{
				if (xVert.Weight[i]>0.0001f && xVert.BoneID[i] == iBone)
				{
					++uVertCount;
				}
			}
		}

		if (uVertCount)
		{
			++uRenderBoneCount;
		}

		//Print bone index
		if(bFullFile)
			fprintf(pFile,"%-3d: %s (%d verts)\n",iBone,pxSkin->SectionData[iBone].SectionName.GetString(), uVertCount);
		else
			fprintf(pFile,"%-3d: %s\n",iBone,pxSkin->SectionData[iBone].SectionName.GetString());
	}

	if(bFullFile)
	{
		fprintf(pFile,"\n");
		fprintf(pFile,"Render bones : %d\n", uRenderBoneCount);
		fprintf(pFile,"(must be <=27 for low end PC support)\n");
	}

	fprintf(pFile,"\n\n");

	fprintf(pFile,"*Hierarchy*\n\n");

	for(int i=0;i<pxSkin->NumberOfBones;i++)
	{
		if(i==0 || pxSkin->ParentBone[i]<0)
		{
			OutputHierarchy(pFile,pxSkin,0,i,0);
		}
	}
	fprintf(pFile,"\n\n");

	if(pxSkin->NumberOfMarkerPoints && bFullFile)
	{
		fprintf(pFile,"*Marker Points*\n\n");
		fprintf(pFile,"     %-20s %-20s\n","Marker Name","Parent");
		
		for(int i=0;i<pxSkin->NumberOfMarkerPoints;i++)
		{
			Asura_Hierarchy_Marker_Point* pxMarker = &pxSkin->MarkerPoint[i];

			const char* szParentName = (pxMarker->BoneIndex == -1) ? "Compass" : pxSkin->SectionData[pxMarker->BoneIndex].SectionName.GetString();

			fprintf(pFile,"%-3d: %-20s %-20s\n",i,pxMarker->GetName().GetString(),szParentName);
		}
	}


	Tools_Timed_Message_System::AddMessage(0xffffff,"%s written",pszFilename);
	fclose(pFile);
}

void Tools_Mode_Hierarchy::OutputAnimHierarchyTextFile()
{
	if(!Animation) return;

	if(!Animation->m_piParentBones || !Animation->m_pxBoneNames)
	{
		//can't do this for old anims.
		Tools_Timed_Message_System::AddMessage(0xffffff,"Animation is too old to contain the required hierarchy information.");
		return;
	}
	
	//get a filename to save as
	//ask for a filename
	Tools_FileSystem_DialogInfo xInfo;

	xInfo.DefaultExtension = "txt";

	xInfo.DefaultFilename = Animation->Name.GetString();
	xInfo.DefaultPath = ".";
	xInfo.FileFilter = "Text Files\0*.txt\0";
	xInfo.WindowTitle = "Save hierarchy text file as :";

	Asura_Char* pszName = Tools_FileSystem::GetSaveFilename(xInfo);
	if(!pszName) return;
		
	CreateAnimHierarchyTextFile(Animation,pszName);
}

void Tools_Mode_Hierarchy::CreateAnimHierarchyTextFile(Asura_Hierarchy_Anim* pxAnim,const char* pszFilename)
{
	
	if(!pxAnim->m_piParentBones || !pxAnim->m_pxBoneNames)
	{
		return;
	}
	
	FILE* pFile = fopen(pszFilename,"wt");
	if(!pFile)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to open %s for writing",pszFilename);
		return;
	}

	fprintf(pFile,"*** %s ***\n\n",pxAnim->Name.GetString());

	fprintf(pFile,"*Bone Order*\n\n");

	for(int iBone=0; iBone<pxAnim->NumberOfBones; ++iBone)
	{
		//Print bone index
		fprintf(pFile,"%-3d: %s\n",iBone,pxAnim->m_pxBoneNames[iBone].GetString());
	}

	fprintf(pFile,"\n\n");

	fprintf(pFile,"*Hierarchy*\n\n");

	for(int i=0;i<pxAnim->NumberOfBones;i++)
	{
		if(i==0 || pxAnim->m_piParentBones[i]<0)
		{
			OutputHierarchy(pFile,0,pxAnim,i,0);
		}
	}
	fprintf(pFile,"\n\n");


	Tools_Timed_Message_System::AddMessage(0xffffff,"%s written",pszFilename);
	fclose(pFile);
}


void Tools_Mode_Hierarchy::OutputHierarchy(FILE* pFile,Asura_Hierarchy_Skin* pxSkin,Asura_Hierarchy_Anim* pxAnim,int iBoneIndex,int iIndent)
{
	Asura_Assert(pxSkin || pxAnim,"");
	const int* piParentBones = 0;

	if(pxSkin)
	{
		piParentBones = pxSkin->ParentBone;
	}
	else
	{
		if(!pxAnim->m_piParentBones || !pxAnim->m_pxBoneNames) return;
		piParentBones = pxAnim->m_piParentBones;
	}


	int iNumberOfBones = pxSkin ? pxSkin->NumberOfBones : pxAnim->NumberOfBones;

	#if 1
	{
		int aiParentBones[100];
		Asura_Assert(iIndent+1<100,"");

		int iIndex = iBoneIndex;
		for(int i=iIndent;i>=0;i--)
		{
			aiParentBones[i] = iIndex;
			iIndex = piParentBones[iIndex];
		}

		for(int i=0;i<iIndent;i++)
		{
			bool bAnyOtherChild = false;
			for(int j=aiParentBones[i+1]+1;j<iNumberOfBones;j++)
			{
				if(piParentBones[j] == aiParentBones[i])
				{
					bAnyOtherChild = true;
					break;
				}
			}
			if(i==iIndent-1)
			{
				if(bAnyOtherChild)
					fprintf(pFile,"|_");
				else
					fprintf(pFile,"\\_");
			}
			else
			{
				if(bAnyOtherChild)
					fprintf(pFile,"| ");
				else
					fprintf(pFile,"  ");
			}
		}
	}
	#else
	for(int i=0;i<iIndent;i++)
	{
		fprintf(pFile,"  ");
	}
	#endif

	const char* szBoneName = pxSkin ? pxSkin->SectionData[iBoneIndex].SectionName.GetString() : pxAnim->m_pxBoneNames[iBoneIndex].GetString();

	fprintf(pFile,"%s\n",szBoneName);


	//recurse through children
	for(int i=iBoneIndex+1;i<iNumberOfBones;i++)
	{
		if(piParentBones[i] == iBoneIndex)
		{
			OutputHierarchy(pFile,pxSkin,pxAnim,i,iIndent+1);
		}
	}
}

void Tools_Mode_Hierarchy::SetHierarchyTimer(float fTime)
{
	if(!Hierarchy || AnimName.IsBlank()) return;

	//set primary animation timer
	//restarting the anim to do this , in case it is non-looping and been
	//flagged as finished
	Hierarchy->m_xPrimaryWrapper.StartAnim2(AnimName.GetHash(),m_uPrimaryAnimFlags,0.0f);
	Hierarchy->m_xPrimaryWrapper.SetTimer(fTime);

	//secondary anim timers
	for(int i=0;i<MAX_SECONDARY_ANIM;i++)
	{
		SecondaryAnimationData* pxSec = &SecondaryAnims[i];
		
		if(pxSec->Used && 
			pxSec->AnimHandle != ASURA_INVALID_HANDLE &&
			!pxSec->m_bPaused &&
			!pxSec->SyncWithPrimary)
		{
			pxSec->m_fTimer = fTime;
			Hierarchy->m_xPrimaryWrapper.SetTimer(fTime,pxSec->AnimHandle);
		}
	}

	//attachment timers
	for(LIF<Tools_MarkerPoint_Object*> xMarkerLIF(&m_xMarkerList);!xMarkerLIF.done();xMarkerLIF.next())
	{
		xMarkerLIF()->SetTimer(fTime);
	}
}


void Tools_Mode_Hierarchy::PreReloadAnims()
{
	if(!s_pxThis) return;

	
	//Deselect the current animation while reloading  , in case the edited anim gets reloaded
	if(s_pxThis->Animation)
	{
		s_pxThis->m_pxCurrentAnimBackup = s_pxThis->Animation;
		s_pxThis->m_xCurrentAnimFilenameBackup = s_pxThis->m_xAnimFilename;
		s_pxThis->SelectAnimation(0,0);
	}
	else
	{
		s_pxThis->m_pxCurrentAnimBackup = 0;
		s_pxThis->m_xCurrentAnimFilenameBackup.MakeNull();
	}
}

void Tools_Mode_Hierarchy::PostReloadAnims()
{
	if(!s_pxThis) return;

	//restore the originally selected anim
	if(s_pxThis->m_pxCurrentAnimBackup)
	{
		//Check to see if the filename for the anim has changed (which may happen if changing platforms)
		//If the filename isn't available , then keep with the one we know about , since the loading
		//of platform specific files won't have been able to change it anyway.
		//RAR 26/04/2007
		const char* szFilename = Asura_HashedFilename_System::GetFullFilename(s_pxThis->m_pxCurrentAnimBackup->GetHashID(),false);

		s_pxThis->SelectAnimation(s_pxThis->m_pxCurrentAnimBackup,szFilename ? szFilename : s_pxThis->m_xCurrentAnimFilenameBackup.GetString());
		s_pxThis->m_pxCurrentAnimBackup = 0;
		s_pxThis->m_xCurrentAnimFilenameBackup.MakeNull();
	}
}


void Tools_Mode_Hierarchy::PreReloadSkins()
{
	if(!s_pxThis) return;

	
	PreReloadAnims();
	//Deselect the current skin while reloading  , in case the edited skin gets reloaded
	s_pxThis->m_pxSkin = 0;
	
}

void Tools_Mode_Hierarchy::PostReloadSkins()
{
	if(!s_pxThis) return;

	//restore the originally selected skin
	if(!s_pxThis->SkinName.IsBlank() && s_pxThis->m_pxSkin==0)
	{
		//Check to see if the filename for the skin has changed (which may happen if changing platforms)
		//If the filename isn't available , then keep with the one we know about , since the loading
		//of platform specific files won't have been able to change it anyway.
		//RAR 26/04/2007
		Asura_Hash_ID uSkinHash = Asura_GetHashID(s_pxThis->SkinName.GetString());
		const char* szFilename = Asura_HashedFilename_System::GetFullFilename(uSkinHash,false);
		
		s_pxThis->SetSkin(s_pxThis->SkinName.GetString(),szFilename ? szFilename : s_pxThis->SkinFilename.GetString());
	}

	//restore animation as well.
	PostReloadAnims();
}


void Tools_Mode_Hierarchy::StopLoopedSounds()
{
	if(!s_pxThis) return;
	if(!s_pxThis->Hierarchy) return;

	s_pxThis->Hierarchy->StopLoopedSounds();
}

bool Tools_Mode_Hierarchy::IsPrimaryAnimReversed()
{
	if(s_pxThis && (s_pxThis->m_uPrimaryAnimFlags & AsuraAnimationFlag_Reversed))
	{
		return true;
	}
	else
	{
		return false;
	}
}



///////////////////////////////////////////////////////////////////////////////////////
void SecondaryAnimationData::Initialise(int index)
{
	OverlayID = TOOLS_OVERLAY_SECONDARYANIM+index;
	Asura_Assert(OverlayID<=TOOLS_OVERLAY_SECONDARYANIM_MAX,"");

	Reset();
}

void SecondaryAnimationData::Reset()
{
	Used = false;
	Restart = false;
	SyncWithPrimary = false;
	m_bEnabled = true;
	AnimHandle = ASURA_INVALID_HANDLE;
	Weight = 1.0f;
	m_pxAnim = 0;
	m_fTimer = 0.0f;
	m_bPaused = false;
	m_bMirrored = false;
	m_bReversed = false;
	m_xDirectory = Tools_Mode_Hierarchy::s_pxThis->DirectoryName;
}


/////////////////////////////////////////////////////////////////////////////////////
/*--------------------**
** ToolsSoundDefnEdit **
**--------------------*/

int ToolsSoundDefnEdit::s_iPasteBufferSize = 0;
Asura_Hierarchy_Sound* ToolsSoundDefnEdit::s_pxPasteBuffer = 0;

ToolsSoundDefnEdit::ToolsSoundDefnEdit(int iOverlayID)
{
	m_iOverlayID = iOverlayID;

	m_iSoundSelection = 0;
	m_iSoundOffset = 0;
	m_iSoundIndex = 0;
	m_iSoundNameArraySize = 0;
	m_ppszSoundNames = 0;

	m_uSoundEvent = 0;
	m_bSoundEventChanged = false;

	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	m_uDialogueEvent = 0;
	m_bDialogueEventChanged  = false;
	#endif


}

ToolsSoundDefnEdit::~ToolsSoundDefnEdit()
{
	//destroy the overlay (if any)
	Asura_Overlay_System::CloseOverlays(m_iOverlayID);

	//delete the sound name array
	DeleteSoundNameArray();
}


void ToolsSoundDefnEdit::UpdateOverlay()
{
	//destroy the existing overlay (if any)
	Asura_Overlay_System::CloseOverlays(m_iOverlayID);

	//if(!m_pxSkin && !m_pxAnim) return;

	BuildSoundNameArray();

	//create the overlay
	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay(GetOverlayTitle(),false,GetOverlayPosition());

	
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,3.0f);

	Asura_Dialog* pxDialog = new Asura_Function_Dialog("Add",xOffset,AddSound,(int) this);
	pxDialog->SetHelpText("Add a new sound definition to this skin or anim.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x += 12.0f;

	pxDialog = new Asura_Function_Dialog("Delete",xOffset,DeleteSound,(int) this);
	pxDialog->SetHelpText("Delete the selected sound.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = 1.0f;
	xOffset.y+=Asura_Dialog::DefaultSpacing();

	
	pxDialog = new Asura_Function_Dialog("Copy",xOffset,Copy,(int) this);
	pxDialog->SetHelpText("Copies all the sound definitions to a paste buffer.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.x += 12.0f;

	pxDialog = new Asura_Function_Dialog("Paste",xOffset,Paste,(int) this);
	pxDialog->SetHelpText("Pastes the most recently copied sound definitions to this skin/anim.");
	pxDialog->SetIsEnabledFunction(CanPaste);
	pxOverlay->AddDialog(pxDialog);
	xOffset.x = 1.0f;
	xOffset.y+=Asura_Dialog::DefaultSpacing();


	//heading for list box
	pxDialog = new Asura_Dialog("Selected Sound :",xOffset);	
	pxOverlay->AddDialog(pxDialog);
	xOffset.y+=Asura_Dialog::SmallSpacing();

	//and the list box
	int iNumberOfLines = Asura_Maths::Min(10,m_iSoundNameArraySize);
	pxDialog = new Asura_List_Dialog(xOffset,m_ppszSoundNames,&m_iSoundOffset,&m_iSoundSelection,iNumberOfLines,m_iSoundNameArraySize,NULL);
	pxDialog->SetHelpText("List of sounds attached to this skin or anim.\nDetermines the currently selected sound.");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y = pxDialog->m_xMaxCoord.y + Asura_Dialog::SmallSpacing();

	//get the selection indeces in range
	m_iSoundSelection = m_iSoundIndex;
	m_iSoundOffset = 0;
	Asura_List_Dialog::AdjustListSelectionAndOffset(&m_iSoundOffset,&m_iSoundSelection,iNumberOfLines,m_iSoundNameArraySize);
	m_iSoundIndex = m_iSoundSelection + m_iSoundOffset;


	pxOverlay->m_uID = m_iOverlayID;
 	Asura_Overlay_System::SubmitNewOverlay(pxOverlay);

	HandleSelectionChange();
}


void ToolsSoundDefnEdit::HandleSelectionChange()
{
	m_iSoundIndex = m_iSoundSelection + m_iSoundOffset;

	//find the overlay
	Asura_Overlay* pxOverlay = Asura_Overlay_System::GetOverlay(m_iOverlayID);
	if(!pxOverlay) return;

	//delete the old skin properties dialog controls
	Asura_Dialog* pxDialog = 0;
	do
	{
		pxDialog = pxOverlay->GetDialog(DialogID_SoundProperties);
		if(pxDialog) pxOverlay->DeleteSingleDialog(pxDialog);
	}while(pxDialog);

	
	//add new properties controls
	if(NumberOfSoundDefns())
	{
		Asura_Vector_2 xOffset = pxOverlay->m_pxDialogList->m_xOffset;
		xOffset.y = pxOverlay->m_pxDialogList->m_pxNextDialog->m_xMaxCoord.y + Asura_Dialog::SmallSpacing();
		
		//sound properties heading
		pxDialog = new Asura_Dialog("Sound Properties :",xOffset);
		pxDialog->m_iControlID = DialogID_SoundProperties;
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		Asura_Hierarchy_Sound* pxSound = &SoundDefns()[m_iSoundIndex];

		m_bSoundEventChanged = false;
		m_uSoundEvent = pxSound->m_uSoundEvent;

		Asura_Dialog* pxSoundEventDialog = pxDialog = new Tools_PickSoundEvent_Dialog("Sound : ",xOffset,&m_uSoundEvent,&m_bSoundEventChanged);
		pxDialog->m_iControlID = DialogID_SoundProperties;
		pxDialog->SetHelpText("The sound event to play.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		//add control for editing the selected sound event
		pxDialog = new Tools_EditSoundEvent_Dialog("Edit",xOffset,&m_uSoundEvent);
		pxDialog->m_iControlID = DialogID_SoundProperties;
		pxOverlay->AddDialog(pxDialog);
		pxDialog->SetRelativePosition(pxSoundEventDialog,Asura_Vector_2(3.0f,0.0f),false,true);

		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		m_bDialogueEventChanged = false;
		m_uDialogueEvent = pxSound->m_uDialogueEvent;

		Asura_Dialog* pxDialogueEventDialog = pxDialog = new Tools_PickDialogueEvent_Dialog("Dialogue : ",xOffset,&m_uDialogueEvent,&m_bDialogueEventChanged);
		pxDialog->m_iControlID = DialogID_SoundProperties;
		pxDialog->SetHelpText("The dialogue event to play.");
		pxOverlay->AddDialog(pxDialog);
		xOffset.y+=Asura_Dialog::DefaultSpacing();

		//add control for editing the selected dialogue event
		pxDialog = new Tools_EditDialogueEvent_Dialog("Edit",xOffset,&m_uDialogueEvent);
		pxDialog->m_iControlID = DialogID_SoundProperties;
		pxOverlay->AddDialog(pxDialog);
		pxDialog->SetRelativePosition(pxDialogueEventDialog,Asura_Vector_2(3.0f,0.0f),false,true);
		#endif

	}

	pxOverlay->CalculateOverlaySize();	

}

void ToolsSoundDefnEdit::BuildSoundNameArray()
{
	DeleteSoundNameArray();

	int iNumberOfSoundDefns = NumberOfSoundDefns();
	Asura_Hierarchy_Sound* pxSoundDefns = SoundDefns();

	Tools_Flag_List* pxAnimationSoundList = Tools_Mode_Hierarchy::GetAnimationSoundList();

	if(iNumberOfSoundDefns)
	{
		m_iSoundNameArraySize = iNumberOfSoundDefns;
		m_ppszSoundNames = new char*[iNumberOfSoundDefns];
		for(int i=0;i<iNumberOfSoundDefns;i++)
		{
			Asura_Hierarchy_Sound* pxSound = &pxSoundDefns[i];
			const char* pszName = 0;

			//get the name for this sound
			if(pxAnimationSoundList)
			{
				pszName = pxAnimationSoundList->GetFlagName(pxSound->m_iAnimSoundID);
				if(pszName && pszName[0] =='<') pszName = 0;//deal with '<unknown flag>'
			}
			if(pszName)
			{
				m_ppszSoundNames[i] = new char[strlen(pszName)+1];
				strcpy(m_ppszSoundNames[i],pszName);
			}
			else
			{
				m_ppszSoundNames[i] = new char[20];
				Asura_PrintfToBfr(m_ppszSoundNames[i],"Unknown (%d)",20,true,pxSound->m_iAnimSoundID);
			}
		}
	}
	else
	{
		//no sounds , so just create one entry for the list control to show that
		m_iSoundNameArraySize = 1;
		m_ppszSoundNames = new char*[1];
		m_ppszSoundNames[0] = new char[10];
		strcpy(m_ppszSoundNames[0],"*None*");
	}
}

void ToolsSoundDefnEdit::DeleteSoundNameArray()
{
	for(int i=0;i<m_iSoundNameArraySize;i++)
	{
		delete [] m_ppszSoundNames[i];
	}
	delete [] m_ppszSoundNames;
	m_ppszSoundNames = 0;
	m_iSoundNameArraySize = 0;
}

void ToolsSoundDefnEdit::AddSound()
{
	Tools_Flag_List* pxAnimationSoundList = Tools_Mode_Hierarchy::GetAnimationSoundList();
		
	if(!pxAnimationSoundList)
	{
		Tools_Timed_Message_System::AddMessage(0xffffff,"Unable to find list of animation sound IDs");
		return;
	}

	//ask for the sound ID to use
	int iSoundID = 0;
	if(!pxAnimationSoundList->SelectValue("Pick sound to add to this skin/anim",&iSoundID))
	{
		return;
	}

	//allocate new improved array of sounds
	Asura_Hierarchy_Sound* pxSounds = new Asura_Hierarchy_Sound[NumberOfSoundDefns()+1];

	//copy accross the old sounds , leaving a space for the new one
	int iInsertIndex = -1;
	int iToIndex = 0;
	for(int i=0;i<NumberOfSoundDefns();i++,iToIndex++)
	{
		if(iInsertIndex==-1 && iSoundID<SoundDefns()[i].m_iAnimSoundID)
		{
			iInsertIndex = i;
			iToIndex++;
		}
		
		pxSounds[iToIndex] = SoundDefns()[i];
	}

	if(iInsertIndex == -1) iInsertIndex = NumberOfSoundDefns();

	//initialise the new sound
	pxSounds[iInsertIndex].m_iAnimSoundID = iSoundID;
	pxSounds[iInsertIndex].m_fVolume = 1.0f;
	pxSounds[iInsertIndex].m_fPitch = 1.0f;
	pxSounds[iInsertIndex].m_fInnerRadius = fASURA_HR_SOUND_INNER_RADIUS;
	pxSounds[iInsertIndex].m_fOuterRadius = fASURA_HR_SOUND_OUTER_RADIUS;
	pxSounds[iInsertIndex].m_uSoundHash = ASURA_HASH_ID_UNSET;
	pxSounds[iInsertIndex].m_xSoundName.MakeNull();
	pxSounds[iInsertIndex].m_uSoundEvent = 0;
	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	pxSounds[iInsertIndex].m_uDialogueEvent = 0;
	#endif

	//replace the old array
	delete [] SoundDefns();
	SoundDefns() = pxSounds;
	NumberOfSoundDefns()++;

	//change the selection , and update the overlay
	m_iSoundIndex = iInsertIndex;

	UpdateOverlay();

	SetModified();
}

void ToolsSoundDefnEdit::DeleteSound()
{
	if(m_iSoundIndex<0 || m_iSoundIndex>= NumberOfSoundDefns())
	{
		return;
	}

	//delete this entry , and shuffle the rest down
	for(int i=m_iSoundIndex;i<NumberOfSoundDefns()-1;i++)
	{
		SoundDefns()[i] = SoundDefns()[i+1];
	}
	NumberOfSoundDefns()--;

	//and update the overlay
	UpdateOverlay();

	SetModified();
}

void ToolsSoundDefnEdit::Update()
{
	bool bModified = false;

	if(m_bSoundEventChanged)
	{
		//Stop any looping sounds , to make sure they get restarted with the new sound if needed
		Tools_Mode_Hierarchy::StopLoopedSounds();

		m_bSoundEventChanged = false;
		bModified = true;
	}
	#ifdef ASURA_USE_DIALOGUE_SYSTEM
	if(m_bDialogueEventChanged)
	{

		m_bDialogueEventChanged = false;
		bModified = true;
	}
	#endif


	if(bModified)
	{
		//update the sound properties
		Asura_Hierarchy_Sound* pxSound = &SoundDefns()[m_iSoundIndex];
		bool bClearOldSound = false;

		pxSound->m_uSoundEvent = m_uSoundEvent;
		if(m_uSoundEvent)
		{
			bClearOldSound = true;
		}

		#ifdef ASURA_USE_DIALOGUE_SYSTEM
		pxSound->m_uDialogueEvent = m_uDialogueEvent;
		if(m_uDialogueEvent)
		{
			bClearOldSound = true;
		}
		#endif

		if(bClearOldSound)
		{
			pxSound->m_xSoundName.MakeNull();
			pxSound->m_uSoundHash = ASURA_HASH_ID_UNSET;
		}


		SetModified();
		
	}

	

	if(m_iSoundIndex != (m_iSoundSelection+m_iSoundOffset))
	{
		HandleSelectionChange();
	}

}

void ToolsSoundDefnEdit::ClearPasteBuffer()
{
	delete [] s_pxPasteBuffer;
	s_pxPasteBuffer = 0;
	s_iPasteBufferSize = 0;
}

void ToolsSoundDefnEdit::Copy()
{
	ClearPasteBuffer();

	s_iPasteBufferSize = NumberOfSoundDefns();
	if(!s_iPasteBufferSize) return;

	s_pxPasteBuffer = new Asura_Hierarchy_Sound[s_iPasteBufferSize];
	for(int i=0; i<s_iPasteBufferSize; i++)
	{
		s_pxPasteBuffer[i] = SoundDefns()[i];
	}
}

void ToolsSoundDefnEdit::Paste()
{
	if(!s_iPasteBufferSize) return;

	//allocate array large enough for old sounds and the sounds being pasted
	Asura_Hierarchy_Sound* pxSounds = new Asura_Hierarchy_Sound[NumberOfSoundDefns()+s_iPasteBufferSize];


	const Asura_Hierarchy_Sound* pxSource1 = SoundDefns();
	int iNumSource1 = NumberOfSoundDefns();

	const Asura_Hierarchy_Sound* pxSource2 = s_pxPasteBuffer;
	const int iNumSource2 = s_iPasteBufferSize;

	
	//merge together the contents of the two arrays
	int iSourceIndex1 = 0;
	int iSourceIndex2 = 0;
	int iDestIndex = 0;

	while(iSourceIndex1<iNumSource1 || iSourceIndex2<iNumSource2)
	{
		bool bTakeFromSource1 = true;
		if(iSourceIndex1 < iNumSource1)
		{
			if(iSourceIndex2 < iNumSource2)
			{
				//take from the appropriate array to keep the final array sorted by sound ID
				bTakeFromSource1 = pxSource1[iSourceIndex1].m_iAnimSoundID <= pxSource2[iSourceIndex2].m_iAnimSoundID;
			}
		}
		else
		{
			bTakeFromSource1 = false;
		}

		if(bTakeFromSource1)
		{
			pxSounds[iDestIndex++] = pxSource1[iSourceIndex1++];
		}
		else
		{
			//taking from the paste buffer
			//make sure an identical entry doesn't already exist
			bool bSkipThis = false;
			for(int i=0; i<iDestIndex; i++)
			{
				if(pxSounds[i].m_iAnimSoundID == pxSource2[iSourceIndex2].m_iAnimSoundID &&
					pxSounds[i].m_uSoundEvent == pxSource2[iSourceIndex2].m_uSoundEvent &&
					pxSounds[i].m_uDialogueEvent == pxSource2[iSourceIndex2].m_uDialogueEvent)
				{
					bSkipThis = true;
					break;
				}
			}
			if(bSkipThis)
			{
				iSourceIndex2++;
			}
			else
			{
				pxSounds[iDestIndex++] = pxSource2[iSourceIndex2++];
			}
		}
	}

	//replace the old array
	delete [] SoundDefns();
	SoundDefns() = pxSounds;
	NumberOfSoundDefns() = iDestIndex;

	UpdateOverlay();

	SetModified();
}



/////////////////////////////////////////////////////////////////////////////////////
/*------------------------**
** ToolsAnimSoundDefnEdit **
**------------------------*/
ToolsAnimSoundDefnEdit::ToolsAnimSoundDefnEdit(Asura_Hierarchy_Anim* pxAnim)
:ToolsSoundDefnEdit(TOOLS_OVERLAY_ANIMSOUNDDEFNS)
{
	m_pxAnim = pxAnim;
}

int& ToolsAnimSoundDefnEdit::NumberOfSoundDefns() 
{
	return m_pxAnim->m_iNumberOfSoundDefns;
}

Asura_Hierarchy_Sound*& ToolsAnimSoundDefnEdit::SoundDefns() 
{
	return m_pxAnim->m_pxSoundDefns;
}

/////////////////////////////////////////////////////////////////////////////////////
/*------------------------**
** ToolsSkinSoundDefnEdit **
**------------------------*/

ToolsSkinSoundDefnEdit::ToolsSkinSoundDefnEdit(Asura_Hierarchy_Skin* pxSkin)
:ToolsSoundDefnEdit(TOOLS_OVERLAY_SKINSOUNDDEFNS)
{
	m_pxSkin = pxSkin;
}

int& ToolsSkinSoundDefnEdit::NumberOfSoundDefns() 
{
	return m_pxSkin->m_iNumberOfSoundDefns;
}

Asura_Hierarchy_Sound*& ToolsSkinSoundDefnEdit::SoundDefns() 
{
	return m_pxSkin->m_pxSoundDefns;
}


// Helper function to convert parametric time (0 -> 1) depending on the timer mode setting
// e.g. parametric/proportional, seconds, frames
float Tools_Mode_Hierarchy::ConvertParametricTimer( const float fTime )
{
	float fReturnTime = fTime;
	switch(m_eAnimationControl_TimerMode)
	{
		case TIMER_PROP:
		{
			fReturnTime = fTime;
			break;
		}
		case TIMER_SECONDS:
		{
			fReturnTime = fTime * AnimDuration;
			break;
		}
		case TIMER_FRAMES:
		{
			fReturnTime = fTime * m_iAnimFrames;
			break;
		}
		
		case TIMER_25FPS:
		case TIMER_30FPS:
			fReturnTime = fTime * m_fTimeSliderMax;
			break;

	}
	return fReturnTime;
}

Tools_MessageLogAction_SelectSkin::Tools_MessageLogAction_SelectSkin(const Asura_Hierarchy_Skin* pxSkin)
{
	m_uHash = pxSkin  ? pxSkin->GetHashID() : 0;
}

void Tools_MessageLogAction_SelectSkin::Execute()
{
	Tools_Mode_Hierarchy::SelectSkin(m_uHash);
}

Tools_MessageLogAction_SelectAnim::Tools_MessageLogAction_SelectAnim(const Asura_Hierarchy_Anim* pxAnim)
{
	m_uHash = pxAnim  ? pxAnim->GetHashID() : 0;
}

void Tools_MessageLogAction_SelectAnim::Execute()
{
	Tools_Mode_Hierarchy::SelectAnim(m_uHash);
}
