//
// Tools_Mode_PFX.cpp
//
// Created by KOR - 01/03/2004 15:48:42
//
// Copyright (C) 2004 Rebellion, All Rights Reserved.
//

#include "Core/Asura.h"


// This file's header.
#include "Particles/Tools_Mode_PFX.h"

// Asura includes
#include "Camera/Asura_Camera.h"
#include "ChunkSystem/Asura_Chunk_PFX.h"
#include "ChunkSystem/Asura_Chunk_Stream.h"
#include "ChunkSystem/Asura_SaveBlock.h"
#include "Core/Asura_TextureManagement.h"
#include "Core/Asura_Material_System.h"
#include "Debugging/Asura_Debug_Timers.h"
#include "Entity/ParticleGenerator/Asura_ServerEntity_PFX_Effect.h"
#include "Entity/ParticleGenerator/Asura_ClientEntity_PFX_Effect.h"
#include "Interface/Asura_Dialog.h"
#include "Interface/Asura_Graph_Dialog.h"
#include "Interface/Asura_Menubar_System.h"
#include "Interface/Asura_Menubar.h"
#include "Interface/Asura_Overlay_System.h"
#include "Interface/Asura_Overlay.h"
#ifndef ASURA_USE_AUDIO_SYSTEM
#include "Sound/Asura_ProjectSoundIDs.h"
#endif
#include "Sound/Asura_Sound.h"
#include "Sound/Asura_Sound_Event_Storage.h"
#include "SpecialFX/PFX/Asura_PFX_Classes.h"
#include "SpecialFX/PFX/Asura_PFX_Templates.h"

// Tools includes
#include "Tools_Overlays.h"
#include "Tools_System.h"
#include "Entity/Tools_Entity_PFX_Effect.h"
#include "Entity/Tools_SecondaryEntityFile_System.h"
#include "Load&Save/Tools_SaveAsura.h"
#include "Load&Save/Tools_ModeDataFile.h"
#include "Load&Save/Tools_LoadedFileList.h"
#include "Load&Save/Tools_ResourceExportLog.h"
#include "Misc/Tools_IniFile.h"
#include "Misc/Tools_EnumerationReader.h"
#include "Misc/Tools_ProgressIndicator.h"
#include "Misc/Tools_Render_Options.h"
#include "Misc/Tools_Timed_Messages.h"
#include "Objects/Tools_Object_System.h"
#include "Objects/Tools_PFX_Vector_Object.h"
#include "Particles/Tools_PFX_SourceVolume_Object.h"
#include "Particles/Tools_PFX_Dialog.h"
#include "Particles/Tools_PFX_Import.h"
#include "Particles/Tools_PFX_Overlay.h"
#include "PC/FileSystem/Tools_FileSystem.h"
#include "Sound/Tools_PickSound.h"
#include "Sound/Tools_SoundEvent_System.h"
#include "Sound/Tools_PickSoundEvent_Dialog.h"
#include "Texturing/Tools_PickTexture.h"
#include "Texturing/Tools_Texturing_Texture.h"
#include "Texturing/Tools_Texture_Management.h"


#include "Debugging/Asura_Logfile.h"

// Static members /////////////////////////////////////////////////////////////////////////////////

Asura_Handle		Tools_Mode_PFX::s_iMenuHandle;
bool				Tools_Mode_PFX::s_bPFXOverlayOpen = true;
Asura_Vector_2		Tools_Mode_PFX::s_xPFXOverlayPosition = Asura_Vector_2(0.0f,10.0f);
Asura_Vector_2		Tools_Mode_PFX::s_xEditParticleTempPos = Asura_Vector_2(95.0f,0.0f);
Asura_Vector_2		Tools_Mode_PFX::s_xEditSourceTempPos = Asura_Vector_2(55.f,0.0f);
Asura_Vector_2		Tools_Mode_PFX::s_xEditEffectTempPos = Asura_Vector_2(17.5f,0.0f);
Asura_Vector_2		Tools_Mode_PFX::s_xEditTextureAnimPos = Asura_Vector_2(40.0f,20.0f);
Tools_Mode_PFX*		Tools_Mode_PFX::s_pxThis = NULL;
u_int				Tools_Mode_PFX::s_uTotalParticlesRendered = 0;
u_int				Tools_Mode_PFX::s_uEffectParticlesRendered = 0;
bool				Tools_Mode_PFX::s_bParticleEffectBeingRenderedIsSelected;
bool				Tools_Mode_PFX::s_bShowParticleCount = true;
bool				Tools_Mode_PFX::s_bHideIcons = false;
bool				Tools_Mode_PFX::s_bHideNonPFX = true;
float				Tools_Mode_PFX::s_fTimeSinceAutosave = 0.f;

// Edit Particle Template members
Asura_Hash_ID		Tools_Mode_PFX::s_uEditingParticleTempHash;
Asura_Graph_Dialog<Asura_RGBA>*	Tools_Mode_PFX::s_pxParticleColourGraph = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxParticleSizeGraph = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxParticleRotationGraph = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxParticleSizeVariationGraph = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxBlurLengthDialog = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxFixedRotationDialog = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxNumFixedRotationsDialog = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxLightingScaleDialog = NULL;
Asura_Tabs_Dialog*	Tools_Mode_PFX::s_pxParticleTabs = NULL;
int					Tools_Mode_PFX::s_iParticleTempAttrib = 0;
u_int				Tools_Mode_PFX::s_uParticleCurrentTextureFrame;
bool				Tools_Mode_PFX::s_bParticleCurrentTextureFrameChanged;
float				Tools_Mode_PFX::s_afParticleTextureUVs[4];
bool				Tools_Mode_PFX::s_bUseUVAsAspect;
float				Tools_Mode_PFX::s_fParticleTextureFrameTime;
bool				Tools_Mode_PFX::s_bParticleTextureFrameTimeChanged;
float				Tools_Mode_PFX::s_fParticleMaxLifetime;
bool				Tools_Mode_PFX::s_bParticleUseLighting;
bool				Tools_Mode_PFX::s_bParticlePreLighting;
bool				Tools_Mode_PFX::s_bParticleUseFogging;
bool				Tools_Mode_PFX::s_bParticleUseMotionBlur;
bool				Tools_Mode_PFX::s_bParticleUseRandRot;
bool				Tools_Mode_PFX::s_bParticleSpriteRotation;
bool				Tools_Mode_PFX::s_bParticleRandomTextureOrient;
bool				Tools_Mode_PFX::s_bParticle3DRotation;
bool				Tools_Mode_PFX::s_bParticleMirrorRotation;
bool				Tools_Mode_PFX::s_bParticleAlignEffectX;
bool				Tools_Mode_PFX::s_bParticleAlignEffectY;
bool				Tools_Mode_PFX::s_bParticleAlignEffectZ;
bool				Tools_Mode_PFX::s_bParticleBillboardJustX;
bool				Tools_Mode_PFX::s_bParticleBillboardJustY;
bool				Tools_Mode_PFX::s_bParticleFixedRotation;
bool				Tools_Mode_PFX::s_bParticleNearCameraFade;
bool				Tools_Mode_PFX::s_bParticleFarCameraFade;
bool				Tools_Mode_PFX::s_bParticleRandomLifetime;
bool				Tools_Mode_PFX::s_bParticleUseSizeScale;
bool				Tools_Mode_PFX::s_bParticleCanUseExpensiveClipping;
bool				Tools_Mode_PFX::s_bParticleUseFPS;
bool				Tools_Mode_PFX::s_bParticleCrossfadeTextureFrames;
bool				Tools_Mode_PFX::s_bParticleLoopAnimation;
bool				Tools_Mode_PFX::s_bParticleLooped;
bool				Tools_Mode_PFX::s_bParticleRenderEarly;
int					Tools_Mode_PFX::s_iParticleTranslucency;
Tools_UV_Editor*	Tools_Mode_PFX::s_pxParticleTextureEdit = NULL;

float				Tools_Mode_PFX::s_fTextureAnim_U;
float				Tools_Mode_PFX::s_fTextureAnim_V;
float				Tools_Mode_PFX::s_fTextureAnim_Width;
float				Tools_Mode_PFX::s_fTextureAnim_Height;
u_int				Tools_Mode_PFX::s_uTextureAnim_Frames;
int					Tools_Mode_PFX::s_iTextureAnim_TextureWidth;
int					Tools_Mode_PFX::s_iTextureAnim_TextureHeight;


// Edit Source Template members
Asura_Hash_ID		Tools_Mode_PFX::s_uEditingSourceTempHash;
Asura_Dialog*		Tools_Mode_PFX::s_pxSpeedGraph = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxAngleGraph = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxSpeedSpreadGraph = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxAngleSpreadGraph = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxReleaseRateGraph = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxReleaseRandGraph = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxGravityGraph = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxTurbulenceGraph = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxTurbulenceDialog = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxTimeScaleDialog = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxInheritVelocityDialog = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxParticleBounceDialog = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxSourceBounceDialog = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxParticleFrictionDialog = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxEditGravityDialog = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxSourceEditPcleTempDialog = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxSourceTextureNameDialog = NULL;
Asura_Tabs_Dialog*	Tools_Mode_PFX::s_pxSourceTabs = NULL;
int					Tools_Mode_PFX::s_iSrcTempAttrib = 0;
int					Tools_Mode_PFX::s_iSrcSpawnIndex = 1;
bool				Tools_Mode_PFX::s_bSrcUseWind;
bool				Tools_Mode_PFX::s_bSrcUseParticleDynamics;
bool				Tools_Mode_PFX::s_bSrcUseSourceDynamics; 
bool				Tools_Mode_PFX::s_bSrcUseTimeScale;
bool				Tools_Mode_PFX::s_bSrcMovePclesWithSource;
bool				Tools_Mode_PFX::s_bSrcRotatePclesWithSource;
bool				Tools_Mode_PFX::s_bSrcBurst;
bool				Tools_Mode_PFX::s_bSrcUseTurbulence;
bool				Tools_Mode_PFX::s_bSrcARPTS;
bool				Tools_Mode_PFX::s_bSrcFixedAirResistance;
bool				Tools_Mode_PFX::s_bSrcDirectionalGravity;
bool				Tools_Mode_PFX::s_bSrcPointGravity;
bool				Tools_Mode_PFX::s_bSrcInheritVelocity;
bool				Tools_Mode_PFX::s_bSrcUseRange;
bool				Tools_Mode_PFX::s_bSrcUseSpeedScale;
bool				Tools_Mode_PFX::s_bSrcStopInstantly;
bool				Tools_Mode_PFX::s_bSrcSpawnFromParticles;
bool				Tools_Mode_PFX::s_bSrcScaleReleaseRateWithVolume;
bool				Tools_Mode_PFX::s_bSrcUseSourceBBForVisibility;
Asura_DropDown_List_Dialog* Tools_Mode_PFX::s_pxSourcePcleTempDialog = NULL;
int					Tools_Mode_PFX::s_iSourcePcleTemp;
const char**		Tools_Mode_PFX::s_pszPcleTempList = NULL;
Tools_PFX_Vector_Object* Tools_Mode_PFX::s_pxSourceGravityObject = NULL;
int					Tools_Mode_PFX::s_iMaximumParticles;
bool				Tools_Mode_PFX::s_bMaxParticlesChanged;

// Edit Effect Template members
Asura_Hash_ID		Tools_Mode_PFX::s_uEditingEffectTempHash;
Asura_Dialog*		Tools_Mode_PFX::s_pxPhotonColourGraph = NULL;
Asura_Dialog*		Tools_Mode_PFX::s_pxPhotonRadiusGraph = NULL;
int					Tools_Mode_PFX::s_iEffectPhotonPerParticleSource = -1;
Asura_Tabs_Dialog*	Tools_Mode_PFX::s_pxEffectTabs = NULL;
Asura_DropDown_List_Dialog* Tools_Mode_PFX::s_pxEffectAttribDialog = NULL;
int					Tools_Mode_PFX::s_iEffectAttrib;
Asura_DropDown_List_Dialog* Tools_Mode_PFX::s_pxEffectSourceNumDialog = NULL;
int					Tools_Mode_PFX::s_iEffectSourceNum;
int					Tools_Mode_PFX::s_iEffectCurrentSourceNum;
Asura_DropDown_List_Dialog* Tools_Mode_PFX::s_pxEffectSourceTempDialog = NULL;
int					Tools_Mode_PFX::s_iEffectSourceTemp;
bool				Tools_Mode_PFX::s_bEffectSourceChanged;
char*				Tools_Mode_PFX::s_aszSourceList[TOOLS_PFX_MAX_SOURCES_PER_EFFECT];
const char**		Tools_Mode_PFX::s_pszSourceTempList = NULL;
int					Tools_Mode_PFX::s_iSourceTempListSize = 0;
//Asura_Hash_ID		Tools_Mode_PFX::s_uEffectSrcTempHash;
Asura_Dialog*		Tools_Mode_PFX::s_pxEffectEditSourceTempDialog = NULL;
float				Tools_Mode_PFX::s_fScalingFactor = 1.f;
int					Tools_Mode_PFX::s_iNumberOfSources;
bool				Tools_Mode_PFX::s_bNumSourcesChanged;
Asura_Dialog*		Tools_Mode_PFX::s_pxEffectEditSourceMovement = NULL;
float				Tools_Mode_PFX::s_fEffectSrcSpeed;
bool				Tools_Mode_PFX::s_bEffectSrcSpeedChanged;
float				Tools_Mode_PFX::s_fEffectSrcTimeOffset;
bool				Tools_Mode_PFX::s_bEffectSrcTimeOffsetChanged;
bool				Tools_Mode_PFX::s_bEffectSrcDisabled;
bool				Tools_Mode_PFX::s_bIsolateSource;
bool				Tools_Mode_PFX::s_bSourceRemoved = false;
Asura_Hash_ID		Tools_Mode_PFX::s_uEffectSoundEvent = 0;
Asura_Hash_ID		Tools_Mode_PFX::s_uParticleSoundEvent = 0;

bool				Tools_Mode_PFX::s_bEffectUseRange = false;
float				Tools_Mode_PFX::s_fEffectRangeDelayTime = 5.f;
float				Tools_Mode_PFX::s_fEffectRangeMin = 0.f;
float				Tools_Mode_PFX::s_fEffectRangeMax = 100.f;

// Flags
bool				Tools_Mode_PFX::s_bEffectHasPhoton;
bool				Tools_Mode_PFX::s_bEffectLooping;
bool				Tools_Mode_PFX::s_bEffectOrderedRender;
bool				Tools_Mode_PFX::s_bEffectMoveWithEffect;
bool				Tools_Mode_PFX::s_bEffectAlwaysOnTop;
bool				Tools_Mode_PFX::s_bEffectHudEffect;
bool				Tools_Mode_PFX::s_bEffectPregenerate;
bool				Tools_Mode_PFX::s_bEffectParticlePhotons;
bool				Tools_Mode_PFX::s_bForceRenderOnMainThread;

// Code orientation
bool				Tools_Mode_PFX::s_bEffectUseCodeOrient;
int					Tools_Mode_PFX::s_iEffectSrcOrientID;
bool				Tools_Mode_PFX::s_bEffectSrcOrientIDChanged;
// Edit vector objects
Tools_PFX_SourceVolume_Object* Tools_Mode_PFX::s_pxEffectSrcOffsetObject;
Tools_PFX_Vector_Object* Tools_Mode_PFX::s_pxEffectSrcMovementObject;

bool				Tools_Mode_PFX::s_bParticleTempChanged;
bool				Tools_Mode_PFX::s_bSourceTempChanged;
bool				Tools_Mode_PFX::s_bEffectTempChanged;
bool				Tools_Mode_PFX::s_bEffectPhotonPropertiesChanged;
bool				Tools_Mode_PFX::s_bEffectMinChanged;
bool				Tools_Mode_PFX::s_bEffectMaxChanged;
u_int				Tools_Mode_PFX::s_uLastParticleEditUpdate = 0;
u_int				Tools_Mode_PFX::s_uLastSourceEditUpdate = 0;
u_int				Tools_Mode_PFX::s_uLastEffectEditUpdate = 0;
Asura_Vector_2		Tools_Mode_PFX::s_xChooseTemplatePosition = Asura_Vector_2(0.0f,10.0f);
const char**		Tools_Mode_PFX::s_pszChooseTemplateNameList = NULL;
Asura_Hash_ID		Tools_Mode_PFX::s_uChooseTemplateHash;
bool				Tools_Mode_PFX::s_bAutoCreateTemplates = true;
bool				Tools_Mode_PFX::s_bAutoCopyTemplates = true;
bool				Tools_Mode_PFX::s_bAutoDeleteTemplates = false;
Tools_Mode_PFX_Template_Type Tools_Mode_PFX::s_eChooseTemplateType = Tools_Mode_PFX_Template_None;

Asura_PFX_Particle_Template*	Tools_Mode_PFX::s_pxCurrentParticleTemp = NULL;
Asura_PFX_Particle_Template*	Tools_Mode_PFX::s_pxSavedParticleTemp = NULL;
Asura_PFX_Source_Template*		Tools_Mode_PFX::s_pxCurrentSourceTemp = NULL;
Asura_PFX_Source_Template*		Tools_Mode_PFX::s_pxSavedSourceTemp = NULL;
Asura_PFX_Effect_Template*		Tools_Mode_PFX::s_pxCurrentEffectTemp = NULL;
Asura_PFX_Effect_Template*		Tools_Mode_PFX::s_pxSavedEffectTemp = NULL;

int Tools_Mode_PFX::s_iPreviewPlatform = Tools_Level_Export::Export_PC;
bool Tools_Mode_PFX::s_bPreviewPlatformChanged = false;

u_int Tools_Mode_PFX::s_uSelectedTab_Effects = 0;
u_int Tools_Mode_PFX::s_uSelectedTab_Particles = 0;
u_int Tools_Mode_PFX::s_uSelectedTab_Source = 0;


#ifdef ASURA_USING_ALL_LEVEL_EXPORT
Asura_HashTree* Tools_Mode_PFX::m_pxPFXToIgnoreList = NULL;
#endif 

bool Tools_Mode_PFX::s_bDontIncludePFXInExport = false;

int Tools_Mode_PFX::s_iProjectTextureFlags = 0;


// Methods ////////////////////////////////////////////////////////////////////////////////////////



void Tools_Mode_PFX::Initialise(void)
{
	//create the menubar
	s_iMenuHandle = Asura_Menubar_System::CreateMenuSection();
	Asura_Menubar_Title* title;

	title = Asura_Menubar_System::AddTitle(s_iMenuHandle ,"Overlays");
	title->AddItem("PFX",Tools_Mode_PFX::TogglePFXOverlay,Tools_Mode_PFX::PFXOverlayIsOn);
	title->AddItem("Entity Manager [Ctrl+E]",Tools_SecondaryEntityFile_System::ToggleOverlay,Tools_SecondaryEntityFile_System::OverlayIsOn);

	//load values from .dat file
	TransferModeData(TOOLSMODEDATAFILE_LOAD);


	//create overlays
	CreatePFXOverlay();
	
	//Allow flying around of objects etc
	Tools_Object_System::EnableAllControls(true);
	Tools_Entity_Object::CreateAllObjects();
	
	Tools_SecondaryEntityFile_System::CreateOverlay();
	Tools_SecondaryEntityFile_System::SelectInitialEntityFile();

	Tools_Render_Options::SetPFXControlledByMode(true);

#ifdef ASURA_USING_ALL_LEVEL_EXPORT
	m_pxPFXToIgnoreList = 0;
#endif

	Tools_Object_System::SetTextRenderPos(1.0f,7.0f);	// Move object system text down a bit to leave room for this mode's text
}


void Tools_Mode_PFX::Shutdown()
{

	Tools_Render_Options::SetPFXControlledByMode(false);

	Tools_Object_System::EnableAllControls(false);
	
	//delete all the objects
	Tools_Object_System::DeleteAllModeSpecific();

	//save values to .dat file
	TransferModeData(TOOLSMODEDATAFILE_SAVE);

	//shutdown the menu
	if(s_iMenuHandle != ASURA_INVALID_HANDLE)
	{
		Asura_Menubar_System::DestroyMenuSection(s_iMenuHandle);
		s_iMenuHandle = ASURA_INVALID_HANDLE;
	}

	Tools_Object_System::SetTypeHidden(TOT_Entity_PFX_Effect,ObjectHideFlag_Mode,false);

	//close overlays
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_PFX);
	Tools_SecondaryEntityFile_System::CloseOverlay();
	CloseEditParticleTemplate();
	CloseEditSourceTemplate();
	CloseEditEffectTemplate();

	Tools_Object_System::ResetTextRenderPos();

	FinishUVEdit();
	// Save all our hard work
//	SaveAllTemplates();
}

void Tools_Mode_PFX::TransferModeData(int mode)
{
	//open the file
	Tools_ModeDataFile file("PFXMode.dat",mode);

	//transfer the data
	//(none of these should be removed , and new data should go on the end, otherwise
	//the files will get screwed up)
	file.Transfer(s_xPFXOverlayPosition);
	file.Transfer(s_bPFXOverlayOpen);
	file.Transfer(s_xEditParticleTempPos);
	file.Transfer(s_xEditSourceTempPos);
	file.Transfer(s_xEditEffectTempPos);
	file.Transfer(s_xEditTextureAnimPos);
}

void Tools_Mode_PFX::Update()
{
	s_fTimeSinceAutosave += Asura_Timers::GetFrameTime();
	if(s_fTimeSinceAutosave > 60.0f)
	{
		AutoSave();
	}

	HandleKeyboard();

	// TODO:
	// Handle the selection of different tools objects, 
	// e.g. for attaching an effect to another entity

	//update hidden state of the PFX entities
	Tools_Object_System::SetTypeHidden(TOT_Entity_PFX_Effect,ObjectHideFlag_Mode,s_bHideIcons);
	Tools_Object_System::SetTypeHidden(TOT_Spline_Point,ObjectHideFlag_Mode,s_bHideIcons);
	//and update hidden state of more or less everything else
	for(LIF<Tools_Object*> xObjectLIF(&Tools_Object_System::ObjectList);!xObjectLIF.done();xObjectLIF.next())
	{
		//ignore PFX entities , and the position editing type things
		if(xObjectLIF()->Type == TOT_Entity_PFX_Effect) continue;
		if(xObjectLIF()->Type == TOT_Spline_Point) continue;
		if(xObjectLIF()->IsTemporaryEditObject()) continue;
		if(xObjectLIF()->ShouldPersistBetweenModes()) continue;
		
		xObjectLIF()->SetHiddenFlag(ObjectHideFlag_Mode,s_bHideNonPFX);	
	}

	if(s_bPreviewPlatformChanged)
	{
		s_bPreviewPlatformChanged = false;
		//need to restart effects to make use of different templates
		Tools_Entity_PFX_Effect::RestartAllEffects();
	}
	
	Tools_SecondaryEntityFile_System::Update();
}

void Tools_Mode_PFX::RenderWorld_PostToneMap(void)
{
	if(s_pxCurrentSourceTemp)
	{
		if(s_iSrcTempAttrib == 1)
		{
			Asura_Graph_Dialog<Asura_Vector_3>* pxAngleGraph = reinterpret_cast<Asura_Graph_Dialog<Asura_Vector_3>*>(s_pxAngleGraph);
			const Asura_Graph_Point<Asura_Vector_3>* pxAngleNode = pxAngleGraph->m_pxGraphGadget->GetCurrentPoint();
			if(pxAngleNode)
			{
				Tools_Entity_PFX_Effect::RenderAngle(&(pxAngleNode->m_xValue));
			}
			else
			{
				Tools_Entity_PFX_Effect::RenderAngle(NULL);
			}
		}
	}
}

void Tools_Mode_PFX::RenderUI(void)
{
	if(s_bShowParticleCount)
	{
		Asura_Text_Output::Print (1.0f, 4.0f, 0xffffff00, 0, "Total particles rendered: %u", s_uTotalParticlesRendered);
		Asura_Text_Output::Print (1.0f, 5.0f, 0xffffff00, 0, "Particles rendered for selected effects: %u", s_uEffectParticlesRendered);
	}
	s_uTotalParticlesRendered = s_uEffectParticlesRendered = 0;

}

void Tools_Mode_PFX::RenderUIExMode()
{
	if(s_pxParticleTextureEdit) RenderUVEdit();
}


void Tools_Mode_PFX::HandleKeyboard()
{
	if (Asura_Overlay_System::OverlaysHaveKeyboardFocus()) return;
	if (Asura_UserInput::KeyPressed(ASURA_KEYID_ALT)) return;
 
	if (Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL))
 	{
		if (Asura_UserInput::DebouncedKeyPressed('e'))
		{
			Tools_SecondaryEntityFile_System::ToggleOverlay();
		}
	}
	else if(Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT))
	{
	}
	else
	{
		if (Asura_UserInput::DebouncedKeyPressed('h'))
		{
			s_bHideIcons = !s_bHideIcons;
		}
	}
}

bool Tools_Mode_PFX::IsPFXModified()
{
	Asura_ResourceSetWithOverride_ConstResourceIterator<Asura_PFX_Effect_Template>	xIterator(Asura_PFX_System::GetEffectTempResourceSet());

	//Not bothering to check modified flag for particles and sources since
	//those will be dealt with by the effect IsModified() function.
	//This will also prevent orphaned sources/particles from claiming that the mode is modified , despite
	//the fact that they have nowhere to live.
	//RAR 11/01/2007
	while(!xIterator.Done())
	{
		const Asura_PFX_Effect_Template*	pxTemplate	= xIterator.GetCurrent();

		if(pxTemplate->IsModified())
		{
			return true;
		}
		else
		{
			xIterator.Next();
		}
	}
	if(Tools_SoundEvent_System::IsModified()) return true;

	return false;
}

bool Tools_Mode_PFX::IsModified()
{
	if(IsPFXModified()) return true;
	return Tools_SaveAsura::IsEntityModified();
}

void Tools_Mode_PFX::CreatePFXOverlay(void)
{
	if(!s_bPFXOverlayOpen) return;

	Asura_Overlay* pxOverlay;
	pxOverlay = new Asura_Overlay("PFX Mode",false,&s_xPFXOverlayPosition);
	
	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset = Asura_Vector_2(1.0f,2.0f);

	pxDialog = new Asura_Function_Dialog( "Create PFX Effect",xOffset, NewPFXEffect );
	pxOverlay->AddDialog(pxDialog);

	//add copy/paste controls
	xOffset.y+=Asura_Dialog::DefaultSpacing();
	Tools_Object_System::AddCopyOverlayControls(pxOverlay,xOffset);

	//platform selection
	xOffset.y+=Asura_Dialog::LargeSpacing()-Asura_Dialog::DefaultSpacing();
	static char* pszPlatforms[] = {"PC","PS2","XBox","GC","PSP"};
	pxDialog = new Asura_DropDown_List_Dialog(xOffset,pszPlatforms,&s_iPreviewPlatform,5,5,NULL,&s_bPreviewPlatformChanged,"Platform");
	pxDialog->SetHelpText("Sets the platform whose PFX effects should be shown.\nThis is used for seeing the effects that have been set for the platform overrides.");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y+=Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Function_Dialog( "Verify all templates",xOffset, Verify_Manual );
	pxOverlay->AddDialog(pxDialog);

	xOffset.y+=Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Function_Dialog( "Reload all templates",xOffset, ReloadAllTemplates );
	pxOverlay->AddDialog(pxDialog);

	// Import templates
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Function_Dialog( "Import PFX Effect", xOffset, ImportPFXEffect );
	pxOverlay->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Function_Dialog( "Convert templates from old format", xOffset, ConvertToNewFormat );
	pxOverlay->AddDialog(pxDialog);

	xOffset.y+=Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Show particle count", xOffset, &s_bShowParticleCount);
	pxOverlay->AddDialog(pxDialog);

	xOffset.y+=Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Hide icons [H]", xOffset, &s_bHideIcons);
	pxDialog->SetHelpText("Hides the icons used to represent the PFX entities");
	pxOverlay->AddDialog(pxDialog);

	xOffset.y+=Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Hide Non-PFX", xOffset, &s_bHideNonPFX);
	pxDialog->SetHelpText("Hides everything except PFX entities , and some position editing objects");
	pxOverlay->AddDialog(pxDialog);


	pxOverlay->m_uID = TOOLS_OVERLAY_PFX;

	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);
}

void Tools_Mode_PFX::TogglePFXOverlay(void)
{
	s_bPFXOverlayOpen = !s_bPFXOverlayOpen;

	if(s_bPFXOverlayOpen)
	{
		CreatePFXOverlay();
	}
	else
	{
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_PFX);
	}
}

bool Tools_Mode_PFX::PFXOverlayIsOn(void)
{
	return s_bPFXOverlayOpen;
}

void Tools_Mode_PFX::NewPFXEffect()
{
	// Create tools object (which creates server and client entities too)
	Tools_Entity_PFX_Effect* pxObject = Tools_Entity_PFX_Effect::Create(false);

	//assign to secondary entity file if needed
	if(pxObject)
	{
		Tools_Object_System::SelectObject(pxObject);	
		Tools_SecondaryEntityFile_System::AddToCurrentFile(pxObject);
	}
}

void Tools_Mode_PFX::ImportPFXEffect()
{
	Tools_PFX_Import::ImportPFXEffect();
}


/// Template editing //////////////////////////////////////////////////////////////////////////////


void Tools_Mode_PFX::Static_AddItemsToEffectOtherTab(Asura_Tabs_Dialog* pxTab, Asura_Vector_2& xOffset, Asura_PFX_Effect_Template* pxCurrentEffectTemp)
{
	Asura_Dialog *pxDialog = 0;

	// Lifetime
	pxDialog = new Asura_Numerical_Dialog("Lifetime", xOffset, &pxCurrentEffectTemp->m_fLifeTime, 0.f, 100.f, .1f, "%.4f", &s_bEffectTempChanged);
	pxTab->AddDialog(pxDialog);

	// Flags
	s_bEffectLooping = pxCurrentEffectTemp->m_bLooping;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Looping", xOffset, &s_bEffectLooping, &s_bEffectTempChanged);
	pxTab->AddDialog(pxDialog);

	s_bEffectOrderedRender = pxCurrentEffectTemp->m_bOrderedRender;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Ordered render", xOffset, &s_bEffectOrderedRender, &s_bEffectTempChanged);
	pxTab->AddDialog(pxDialog);

	s_bEffectMoveWithEffect = pxCurrentEffectTemp->m_bMoveWithEffect;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Move particles with effect", xOffset, &s_bEffectMoveWithEffect, &s_bEffectTempChanged);
	pxTab->AddDialog(pxDialog);

	s_bEffectAlwaysOnTop = pxCurrentEffectTemp->m_bAlwaysOnTop;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Always on top", xOffset, &s_bEffectAlwaysOnTop, &s_bEffectTempChanged);
	pxTab->AddDialog(pxDialog);

	s_bEffectHudEffect = pxCurrentEffectTemp->m_bHudEffect;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("HUD Effect", xOffset, &s_bEffectHudEffect, &s_bEffectTempChanged);
	pxTab->AddDialog(pxDialog);

	s_bForceRenderOnMainThread = pxCurrentEffectTemp->m_bForceRenderOnMainThread;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Use slower, lag free rendering", xOffset, &s_bForceRenderOnMainThread, &s_bEffectTempChanged);
	pxDialog->SetHelpText("Forces PFX to render on the main thread avoiding lagging behind camera movement, is slower tho so use sparingly (primarily for effects attached to a first person rig)");
	pxTab->AddDialog(pxDialog);
}

void Tools_Mode_PFX::AddItemsToEffectOtherTab(Asura_Tabs_Dialog* pxTab, Asura_Vector_2& xOffset, Asura_PFX_Effect_Template* pxCurrentEffectTemp)
{
	Static_AddItemsToEffectOtherTab(pxTab, xOffset, pxCurrentEffectTemp);
}




void Tools_Mode_PFX::AddParticleTemplateOverlayControls(class Asura_Tabs_Dialog* pxTabs, class Asura_Vector_2& xOffset, Asura_Hash_ID uTemplateNameHash)
{
	// Make sure there isn't one open already.
	CloseEditParticleTemplate();

	Asura_Dialog* pxDialog = 0;

	// Get template
	s_pxCurrentParticleTemp = Asura_PFX_System::GetParticleTempByHash(uTemplateNameHash);
	// Check we have it
	if(!s_pxCurrentParticleTemp) return;
	// Save it for undo changes
	s_pxSavedParticleTemp = new Asura_PFX_Particle_Template(*s_pxCurrentParticleTemp);

	pxDialog = new Asura_Function_Dialog("Save", xOffset, SaveCurrentTemplate );
	pxDialog->SetHelpText("Save current effect template and all source and particle templates used by it.");
	pxTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 2.0f;
	pxDialog = new Asura_Function_Dialog("Reload", xOffset, ReloadCurrentEffectTempFull );
	pxDialog->SetHelpText("Reload current effect template and all source and particle templates used by it.");
	pxTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 2.f;
	pxDialog = new Asura_Function_Dialog("Cancel", xOffset, CancelParticleTemplateChanges );
	pxDialog->SetHelpText("Undo changes to this particle template");
	pxTabs->AddDialog(pxDialog);

	// Quick Source Selector. It is using same memory as Effect/Source tab selector
	xOffset.x = pxDialog->m_xMaxCoord.x + 2.f;
	pxDialog = new Asura_DropDown_List_Dialog(xOffset, s_aszSourceList, &s_iEffectSourceNum, 20, s_pxCurrentEffectTemp->m_uNumSources, NULL);
	pxTabs->AddDialog(pxDialog);

	xOffset.x = 1.0f;

	// Template name
	xOffset.y += Asura_Dialog::LargeSpacing();
	const Asura_String* pxName = Asura_PFX_System::GetParticleTempNameByHash(uTemplateNameHash);
	s_uEditingParticleTempHash = uTemplateNameHash;
	pxDialog = new Asura_Dialog(pxName ? pxName->GetString() : "", xOffset);
	pxTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::LargeSpacing();
	const char* aszTabNames[] = { "Graphs", "Texturing", "Flags", "Rotation", "Sound" };
	pxDialog = s_pxParticleTabs = new Asura_Tabs_Dialog(xOffset, aszTabNames, 5, 40);
	pxTabs->AddDialog(pxDialog);
	s_pxParticleTabs->SelectTab(0);
	xOffset.y = pxDialog->m_xMaxCoord.y + 0.5f;

	// Save y position
	const float fTabsPosition = xOffset.y;

	// Choose attribute to edit
	static char* pszList[] = {"Colour", "Size", "Rotation speed", "Size variation"};
	pxDialog = new Asura_DropDown_List_Dialog(xOffset, pszList, &s_iParticleTempAttrib, 4, 4, NULL);
	s_pxParticleTabs->AddDialog(pxDialog);

	// All of these graphs occupy the same space, and the active one is chosen by the above drop-down dialog

	// Asura_Graph<Asura_RGBA> m_xColour;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = s_pxParticleColourGraph = new Asura_Graph_Dialog<Asura_RGBA>("Colour", xOffset, &s_pxCurrentParticleTemp->m_xColour, 0.0f, fASURA_PFX_MAX_PARTICLE_COLOUR, &s_bParticleTempChanged);
	s_pxParticleColourGraph->SetForceNormalisedAlpha (true);
	s_pxParticleTabs->AddDialog(pxDialog);

	Asura_Graph_Dialog<float>* pxGraphDialog;
	// Asura_Graph<float> m_xSize;
	s_pxParticleSizeGraph = pxDialog = pxGraphDialog = new Asura_Graph_Dialog<float>("Size", xOffset, &s_pxCurrentParticleTemp->m_xSize, 0.0f, 1.0f, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);
	pxGraphDialog->m_pxGraphGadget->AdjustToFit(false, true);

	// Asura_Graph<float> m_xRotation;
	s_pxParticleRotationGraph = pxDialog = pxGraphDialog = new Asura_Graph_Dialog<float>("Rotation", xOffset, &s_pxCurrentParticleTemp->m_xRotation, -1.0f, 1.0f, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);
	pxGraphDialog->m_pxGraphGadget->AdjustToFit(false, true);

	// Asura_Graph<float> m_pxSizeVariationGraph;
	s_pxParticleSizeVariationGraph = pxDialog = new Asura_Graph_Dialog<float>("Size variation", xOffset, &s_pxCurrentParticleTemp->m_xSizeVariation, 0.f, 1.0f, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);

	xOffset.y = pxDialog->m_xMaxCoord.y;

	// float m_fMinLifetime;
	xOffset.y += 0.5f;
	pxDialog = new Asura_Dialog("Lifetime", xOffset);
	s_pxParticleTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x;
	pxDialog = new Asura_Numerical_Dialog("Min", xOffset, &s_pxCurrentParticleTemp->m_fMinLifetime, 0.f, 60.f, .1f, "%.3f", &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);

	s_bParticleRandomLifetime = s_pxCurrentParticleTemp->m_bRandomLifetime;
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Tickbox_Dialog("", xOffset, &s_bParticleRandomLifetime, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);

	s_fParticleMaxLifetime = s_pxCurrentParticleTemp->m_fMinLifetime + s_pxCurrentParticleTemp->m_fMaxLifetimeDiff;
	xOffset.x = pxDialog->m_xMaxCoord.x - 1.f;
	pxDialog = new Asura_Numerical_Dialog("Max", xOffset, &s_fParticleMaxLifetime, 0.f, 60.f, .1f, "%.3f", &s_bParticleTempChanged);
	pxDialog->SetIsEnabledFunction(ParticleIsUsingRandomLifetime);
	s_pxParticleTabs->AddDialog(pxDialog);
	xOffset.x = 1.f;

	// Texturing
	s_pxParticleTabs->SelectTab(1);
	xOffset.y = fTabsPosition;

	// float m_afTextureUVs[4];
	pxDialog = new Asura_Dialog("Texture coordinates: (Tab)", xOffset);
	s_pxParticleTabs->AddDialog(pxDialog);

	
	//texture coordinates are copied from the template into values for use by the overlay
	s_uParticleCurrentTextureFrame = 0;
	s_bParticleCurrentTextureFrameChanged = false;
	Asura_PFX_TextureFrame* pxTextureFrame = &s_pxCurrentParticleTemp->m_pxTextureFrames[s_uParticleCurrentTextureFrame];

	for(int i=0;i<4;i++)
	{
		s_afParticleTextureUVs[i] = pxTextureFrame->m_afTextureUVs[i];
	}
	s_fParticleTextureFrameTime = pxTextureFrame->m_fTime;

	float fTextureDialogY = xOffset.y;
	
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog(" Top left U", xOffset, &s_afParticleTextureUVs[0], 0.f, 1.f, .001f, "%.3f", &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);
	
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog(" Top left V", xOffset, &s_afParticleTextureUVs[1], 0.f, 1.f, .001f, "%.3f", &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);
	
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog(" Bottom right U", xOffset, &s_afParticleTextureUVs[2], 0.f, 1.f, .001f, "%.3f", &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);
	
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog(" Bottom right V", xOffset, &s_afParticleTextureUVs[3], 0.f, 1.f, .001f, "%.3f", &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Function_Dialog("Reload texture", xOffset, ReloadParticleTexture );
	s_pxParticleTabs->AddDialog(pxDialog);

	// ASURA_TRANSLUCENCY_MODE m_eTranslucency;
	switch(s_pxCurrentParticleTemp->m_eTranslucency)
	{
		case ASURA_TRANSLUCENCY_NORMAL:				s_iParticleTranslucency = 0;	break;
		case ASURA_TRANSLUCENCY_COLOUR:				s_iParticleTranslucency = 1;	break;
		case ASURA_TRANSLUCENCY_INVCOLOUR:			s_iParticleTranslucency = 2;	break;
		case ASURA_TRANSLUCENCY_GLOWING:			s_iParticleTranslucency = 3;	break;
		case ASURA_TRANSLUCENCY_SPECIAL_HEATHAZE:	s_iParticleTranslucency = 4;	break;
		case ASURA_TRANSLUCENCY_OFF:				s_iParticleTranslucency = 5;	break;
		default: Asura_Assert(0, "Invalid translucency type");
	}
	
	static char* pszTranslucencyList[] = {"Normal translucency", "Colour translucency", "Inverse colour translucency", "Glowing translucency", "Heat Haze translucency", "Solid"};
	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_DropDown_List_Dialog(xOffset, pszTranslucencyList, &s_iParticleTranslucency, 6, 6, NULL);
	s_pxParticleTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Function_Dialog("Choose texture", xOffset, ChooseParticleTexture );
	s_pxParticleTabs->AddDialog(pxDialog);

	if( s_pxCurrentParticleTemp )	// Show also current name of the texture.
	{
		xOffset.y += Asura_Dialog::DefaultSpacing();
		s_pxSourceTextureNameDialog = new Asura_Dialog(s_pxCurrentParticleTemp->m_xTextureName.GetString(), xOffset);
		s_pxParticleTabs->AddDialog(s_pxSourceTextureNameDialog);
	}
	else
	{
		s_pxSourceTextureNameDialog = NULL;
	}

	s_bParticleSpriteRotation = s_pxCurrentParticleTemp->m_bSpriteRotation;
	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Sprite rotation", xOffset, &s_bParticleSpriteRotation, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);

	//put some frame controls to the side
	xOffset.y = fTextureDialogY;
	xOffset.x += 20.0f;

	pxDialog = new Asura_Function_Dialog("Setup Anim Frames", xOffset, InitialiseParticleTextureAnim );
	pxDialog->SetHelpText("Sets up all the animation frames in one go , by assuming that the frames run from left to right on the texture.");
	s_pxParticleTabs->AddDialog(pxDialog);


	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Function_Dialog("Edit Num Frames",xOffset,EditNumParticleTextureFrames);
	pxDialog->SetHelpText("Sets the number of frames of animation for the texture coordinates");
	s_pxParticleTabs->AddDialog(pxDialog);

	s_bParticleLoopAnimation = s_pxCurrentParticleTemp->m_bLoopAnimation;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Loop animation", xOffset, &s_bParticleLoopAnimation, &s_bParticleTempChanged);
	pxDialog->SetIsEnabledFunction(CanEditParticleTextureFrame);
	s_pxParticleTabs->AddDialog(pxDialog);

	s_bParticleUseFPS = s_pxCurrentParticleTemp->m_bUseFPS;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Use FPS", xOffset, &s_bParticleUseFPS, &s_bParticleTempChanged);
	pxDialog->SetIsEnabledFunction(ParticleIsNotLooping);
	s_pxParticleTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("FPS",xOffset,&s_pxCurrentParticleTemp->m_fFPS,0.01f,60.0f,0.01f,"%.2f",&s_bParticleTempChanged);
	pxDialog->SetHelpText("Frames per second.");
	pxDialog->SetIsEnabledFunction(ParticleIsUsingFPS);
	s_pxParticleTabs->AddDialog(pxDialog);

	s_bParticleCrossfadeTextureFrames = s_pxCurrentParticleTemp->m_bCrossfadeTextureFrames;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Crossfade frames", xOffset, &s_bParticleCrossfadeTextureFrames, &s_bParticleTempChanged);
	pxDialog->SetIsEnabledFunction(CanEditParticleTextureFrame);
	pxDialog->SetEnabled(s_pxCurrentParticleTemp->m_uNumTextureFrames>1);
	s_pxParticleTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Frame",xOffset,(int*)&s_uParticleCurrentTextureFrame,0,99,1,"%d",&s_bParticleCurrentTextureFrameChanged);
	pxDialog->SetHelpText("Select the frame of the texture animation to edit");
	pxDialog->SetIsEnabledFunction(CanEditParticleTextureFrame);
	pxDialog->SetEnabled(s_pxCurrentParticleTemp->m_uNumTextureFrames>1);
	s_pxParticleTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Time",xOffset,&s_fParticleTextureFrameTime,0.0f,1.0f,0.1f,"%.3f",&s_bParticleTextureFrameTimeChanged);
	pxDialog->SetHelpText("The time between 0 and 1 at which this frame should be used.");
	pxDialog->SetIsEnabledFunction(ParticleIsNotUsingFPS);
	s_pxParticleTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing()*2;
	xOffset.x -= 20.0f;

	// Flags
	s_pxParticleTabs->SelectTab(2);
	xOffset.y = fTabsPosition;

	s_bParticleLooped = s_pxCurrentParticleTemp->m_bLooped;
	pxDialog = new Asura_Tickbox_Dialog("Looped", xOffset, &s_bParticleLooped, &s_bParticleTempChanged);
	pxDialog->SetHelpText("Loops the particle until the effect is stopped. Looped particles are not allowed to move. Rotating particles will reset to their initial rotation when looping.");
	s_pxParticleTabs->AddDialog(pxDialog);

	// unsigned char m_bUseLighting		:1;
	s_bParticleUseLighting = s_pxCurrentParticleTemp->m_bUseLighting;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Use lighting", xOffset, &s_bParticleUseLighting, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);
	
	// float m_fLightingScale
	xOffset.x += 14.f;
	s_pxLightingScaleDialog = pxDialog = new Asura_Numerical_Dialog("Scale", xOffset, &s_pxCurrentParticleTemp->m_fLightingScale, 0.01f, 1.0f, 0.01f, "%.3f", &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);
	s_pxLightingScaleDialog->SetEnabled(s_bParticleUseLighting);
	xOffset.x -= 14.f;

	// unsigned char m_bPreLighting		:1;
	s_bParticlePreLighting = s_pxCurrentParticleTemp->m_bPreLighting;
	xOffset.y += Asura_Dialog::DefaultSpacing();		
	pxDialog = new Asura_Tickbox_Dialog("Pre-lighting", xOffset, &s_bParticlePreLighting, &s_bParticleTempChanged);
	pxDialog->SetHelpText("Forces particle sources to only evaluate lighting when created");
	pxDialog->SetIsEnabledFunction(ParticleIsUsingLighting);
	s_pxParticleTabs->AddDialog(pxDialog);

	// unsigned char m_bUseFogging			:1;
	s_bParticleUseFogging = s_pxCurrentParticleTemp->m_bUseFogging;
	xOffset.y += Asura_Dialog::DefaultSpacing();		
	pxDialog = new Asura_Tickbox_Dialog("Fogging (expensive-ish)", xOffset, &s_bParticleUseFogging, &s_bParticleTempChanged);
	pxDialog->SetHelpText("Enables fogging. Using a distance fade is preferable, but this may be required for some effects such as dark smoke in the distance");	
	s_pxParticleTabs->AddDialog(pxDialog);


	// unsigned char m_bUseMotionBlur	:1;
	s_bParticleUseMotionBlur = s_pxCurrentParticleTemp->m_bUseMotionBlur;
	xOffset.y += Asura_Dialog::DefaultSpacing();		
	pxDialog = new Asura_Tickbox_Dialog("Use motion blur", xOffset, &s_bParticleUseMotionBlur, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);

	// float m_fBlurLength
	xOffset.x += 14.f;
	s_pxBlurLengthDialog = pxDialog = new Asura_Numerical_Dialog("Length", xOffset, &s_pxCurrentParticleTemp->m_fBlurLength, 0.f, 1.f, .01f, "%.3f", &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);
	s_pxBlurLengthDialog->SetEnabled(s_bParticleUseMotionBlur);
	xOffset.x -= 14.f;

	// unsigned int	m_bNearCameraFade				:1;
	s_bParticleNearCameraFade = s_pxCurrentParticleTemp->m_bNearCameraFade;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Fade near camera", xOffset, &s_bParticleNearCameraFade, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);

	// float m_fNearFadeDistance
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Far", xOffset, &s_pxCurrentParticleTemp->m_fNearFadeDistance, 0.f, 1000.f, .1f, "%.2f", &s_bParticleTempChanged);
	pxDialog->SetHelpText("Distance at which particle starts to fade");
	s_pxParticleTabs->AddDialog(pxDialog);
	pxDialog->SetIsEnabledFunction(ParticleIsUsingNearCameraFade);

	// float m_fNearZeroDistance
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Numerical_Dialog("Near", xOffset, &s_pxCurrentParticleTemp->m_fNearZeroDistance, 0.f, 1000.f, .1f, "%.2f", &s_bParticleTempChanged);
	pxDialog->SetHelpText("Distance at which particle is no longer visible");
	s_pxParticleTabs->AddDialog(pxDialog);
	pxDialog->SetIsEnabledFunction(ParticleIsUsingNearCameraFade);
	xOffset.x = 1.f;

	// unsigned int	m_bFarCameraFade				:1;
	s_bParticleFarCameraFade = s_pxCurrentParticleTemp->m_bFarCameraFade;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Fade far from camera", xOffset, &s_bParticleFarCameraFade, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);

	// float m_fFarFadeDistance
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Near", xOffset, &s_pxCurrentParticleTemp->m_fFarFadeDistance, 0.f, 1000.f, .1f, "%.2f", &s_bParticleTempChanged);
	pxDialog->SetHelpText("Distance at which particle starts to fade");
	s_pxParticleTabs->AddDialog(pxDialog);
	pxDialog->SetIsEnabledFunction(ParticleIsUsingFarCameraFade);

	// float m_fFarZeroDistance
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Numerical_Dialog("Far", xOffset, &s_pxCurrentParticleTemp->m_fFarZeroDistance, 0.f, 1000.f, .1f, "%.2f", &s_bParticleTempChanged);
	pxDialog->SetHelpText("Distance at which particle is no longer visible");
	s_pxParticleTabs->AddDialog(pxDialog);
	pxDialog->SetIsEnabledFunction(ParticleIsUsingFarCameraFade);
	xOffset.x = 1.f;


	// unsigned int m_bUseSizeScale :1;
	s_bParticleUseSizeScale = s_pxCurrentParticleTemp->m_bUseSizeScale;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Use code size scale", xOffset, &s_bParticleUseSizeScale, &s_bParticleTempChanged);
	pxDialog->SetHelpText("Particle can have a size scale factor modified in code");
	s_pxParticleTabs->AddDialog(pxDialog);
	
	// unsigned int m_bCanUseExpensiveClipping :1;
	s_bParticleCanUseExpensiveClipping = s_pxCurrentParticleTemp->m_bCanUseExpensiveClipping;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Can use expensive clipping", xOffset, &s_bParticleCanUseExpensiveClipping, &s_bParticleTempChanged);
	pxDialog->SetHelpText("On PSP (and maybe other platforms which clip the entire particle when any vertex is outside the guardband) some code will try to subdivide or scissor the particle so it still appears correctly. This is intended to be used on a small number of particles which might get very big.");
	s_pxParticleTabs->AddDialog(pxDialog);
	
	// JGP - modify aspect by UV
	s_bUseUVAsAspect = s_pxCurrentParticleTemp->m_bUseUVAsAspect;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Aspect U:V", xOffset, &s_bUseUVAsAspect, &s_bParticleTempChanged );
	s_pxParticleTabs->AddDialog(pxDialog);


	// DMR 17-11-06 - Depth scale for soft/volumetric particles
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Volumetric Depth Scale", xOffset, &s_pxCurrentParticleTemp->m_fVolumetricDepthScale,0.0f,10.0f,0.01f,"%.2f", &s_bParticleTempChanged );
	pxDialog->SetHelpText("Enables/controls soft particles: 0.0 = Off, 1.0 = Recommended");
	s_pxParticleTabs->AddDialog(pxDialog);

	s_bParticleRenderEarly = s_pxCurrentParticleTemp->m_bRenderEarly;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Render early", xOffset, &s_bParticleRenderEarly, &s_bParticleTempChanged );
	s_pxParticleTabs->AddDialog(pxDialog);	

	// Rotation
	s_pxParticleTabs->SelectTab(3);
	xOffset.y = fTabsPosition;

	// unsigned char m_bUseRandomRotation:1;
	s_bParticleUseRandRot = s_pxCurrentParticleTemp->m_bUseRandomRotation;
	pxDialog = new Asura_Tickbox_Dialog("Use random rotation", xOffset, &s_bParticleUseRandRot, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);

	// unsigned char m_bRandomTextureOrient	:1;
	s_bParticleRandomTextureOrient = s_pxCurrentParticleTemp->m_bRandomTextureOrient;
	xOffset.y += Asura_Dialog::DefaultSpacing();		
	pxDialog = new Asura_Tickbox_Dialog("Random texture orientation", xOffset, &s_bParticleRandomTextureOrient, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);
	
	// unsigned char m_b3DRotation	:1;
	s_bParticle3DRotation = s_pxCurrentParticleTemp->m_b3DRotation;
	xOffset.y += Asura_Dialog::DefaultSpacing();		
	pxDialog = new Asura_Tickbox_Dialog("3D Rotation", xOffset, &s_bParticle3DRotation, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);

	// unsigned char m_bMirrorRotation	:1;
	s_bParticleMirrorRotation = s_pxCurrentParticleTemp->m_bMirrorRotation;
	xOffset.y += Asura_Dialog::DefaultSpacing();		
	pxDialog = new Asura_Tickbox_Dialog("Mirror rotation", xOffset, &s_bParticleMirrorRotation, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);

	// Align with effect axes
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Dialog("Align with effect axis:", xOffset);
	pxDialog->SetHelpText("Aligns with camera direction if nothing is ticked");
	s_pxParticleTabs->AddDialog(pxDialog);

	// unsigned char m_bAlignSourceX	:1;
	s_bParticleAlignEffectX = s_pxCurrentParticleTemp->m_bAlignEffectX;
	xOffset.x += 12.f;
	pxDialog = new Asura_Tickbox_Dialog("X", xOffset, &s_bParticleAlignEffectX, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);

	// unsigned char m_bAlignSourceY	:1;
	s_bParticleAlignEffectY = s_pxCurrentParticleTemp->m_bAlignEffectY;
	xOffset.x += 7.f;
	pxDialog = new Asura_Tickbox_Dialog("Y", xOffset, &s_bParticleAlignEffectY, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);

	// unsigned char m_bAlignSourceZ	:1;
	s_bParticleAlignEffectZ = s_pxCurrentParticleTemp->m_bAlignEffectZ;
	xOffset.x += 7.f;
	pxDialog = new Asura_Tickbox_Dialog("Z", xOffset, &s_bParticleAlignEffectZ, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);
	xOffset.x -= 26.f;

	// Billboard single axis
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Dialog("Billboard Single Axis:", xOffset);
	pxDialog->SetHelpText("Billboard only along the given axis");
	s_pxParticleTabs->AddDialog(pxDialog);

	// unsigned char m_bBillboardJustX	:1;
	s_bParticleBillboardJustX = s_pxCurrentParticleTemp->m_bBillboardJustX;
	xOffset.x += 12.f;
	pxDialog = new Asura_Tickbox_Dialog("X", xOffset, &s_bParticleBillboardJustX, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);

	// unsigned char m_bBillboardJustY	:1;
	s_bParticleBillboardJustY = s_pxCurrentParticleTemp->m_bBillboardJustY;
	xOffset.x += 7.f;
	pxDialog = new Asura_Tickbox_Dialog("Y", xOffset, &s_bParticleBillboardJustY, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);
	xOffset.x -= 19.f;

	// unsigned char m_bFixedRotation	:1;
	s_bParticleFixedRotation = s_pxCurrentParticleTemp->m_bFixedRotation;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Fixed rotation", xOffset, &s_bParticleFixedRotation, &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);

	// float m_fFixedRotation
	xOffset.y += Asura_Dialog::DefaultSpacing();		
	s_pxFixedRotationDialog = pxDialog = new Asura_Numerical_Dialog("Angle", xOffset, &s_pxCurrentParticleTemp->m_fFixedRotation, 0.f, 360.f, 1.f, "%.1f", &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);
	s_pxFixedRotationDialog->SetEnabled(s_bParticleFixedRotation);

	// u_int m_uNumFixedRotations
	xOffset.x += 13.f;
	s_pxNumFixedRotationsDialog = pxDialog = new Asura_Numerical_Dialog("Number", xOffset, &s_pxCurrentParticleTemp->m_uNumFixedRotations, 1, 360, 1, "%d", &s_bParticleTempChanged);
	s_pxParticleTabs->AddDialog(pxDialog);
	s_pxNumFixedRotationsDialog->SetEnabled(s_bParticleFixedRotation);
	xOffset.x -= 13.f;

	// Sound
	s_pxParticleTabs->SelectTab(4);
	xOffset.y = fTabsPosition;

	s_uParticleSoundEvent = s_pxCurrentParticleTemp->m_uSoundEvent;
	Asura_Dialog* pxSoundEventDialog = pxDialog = new Tools_PickSoundEvent_Dialog("Event : ", xOffset, &s_uParticleSoundEvent, &s_bParticleTempChanged);
	pxDialog->SetHelpText("Pick sound event to play whenever a particle is created.\nObviously not a good idea to use this if the effect is generating a large number of this particle type.");
	s_pxParticleTabs->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	//add control for editing the selected sound event
	pxDialog = new Tools_EditSoundEvent_Dialog("Edit",xOffset,&s_uParticleSoundEvent);
	s_pxParticleTabs->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxSoundEventDialog,Asura_Vector_2(3.0f,0.0f),false,true);



	xOffset.y = s_pxParticleTabs->GetMaxCoords().y;

	// Make sure we remember previously used Tabs, so we can restore them once the window is recreated.
	s_pxParticleTabs->SetSelectedTabPointer(&s_uSelectedTab_Particles);
}

void Tools_Mode_PFX::AddSourceTemplateOverlayControls(class Asura_Tabs_Dialog* pxTabs, class Asura_Vector_2& xOffset, Asura_Hash_ID uTemplateNameHash)
{
	// Make sure there isn't one open already.
	CloseEditSourceTemplate();

	Asura_Dialog* pxDialog = 0;

	// Get template
	s_pxCurrentSourceTemp = Asura_PFX_System::GetSourceTempByHash(uTemplateNameHash);
	// Check we have it
	if(!s_pxCurrentSourceTemp) return;
	// Save it for undo changes
	s_pxSavedSourceTemp = new Asura_PFX_Source_Template(*s_pxCurrentSourceTemp);

	pxDialog = new Asura_Function_Dialog("Save", xOffset, SaveCurrentTemplate );
	pxDialog->SetHelpText("Save current effect template and all source and particle templates used by it.");
	pxTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 2.0f;
	pxDialog = new Asura_Function_Dialog("Reload", xOffset, ReloadCurrentEffectTempFull );
	pxDialog->SetHelpText("Reload current effect template and all source and particle templates used by it.");
	pxTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 2.f;
	pxDialog = new Asura_Function_Dialog("Cancel", xOffset, CancelSourceTemplateChanges );
	pxDialog->SetHelpText("Undo changes to this source template");
	pxTabs->AddDialog(pxDialog);

	// Quick Source Selector. It is using same memory as Effect/Source tab Selector
	xOffset.x = pxDialog->m_xMaxCoord.x + 2.f;
	pxDialog = new Asura_DropDown_List_Dialog(xOffset, s_aszSourceList, &s_iEffectSourceNum, 20, s_pxCurrentEffectTemp->m_uNumSources, NULL);
	pxTabs->AddDialog(pxDialog);

	xOffset.x = 1.0f;

	// Create heading
	xOffset.y += Asura_Dialog::LargeSpacing();
	const Asura_String* pxString = Asura_PFX_System::GetSourceTempNameByHash(uTemplateNameHash);
	s_uEditingSourceTempHash = uTemplateNameHash;
	pxDialog = new Asura_Dialog(pxString ? pxString->GetString() : "", xOffset);
	pxTabs->AddDialog(pxDialog);	
	
	// Asura_Hash_ID m_uParticleTemplateHash
	const float fBackupX = xOffset.x;
	xOffset.y += Asura_Dialog::LargeSpacing();
	const char* aszTabNames[] = { "Graphs", "Gravity", "Turbulence", "Scaling", "Generation", "Other" };
	pxDialog = s_pxSourceTabs = new Asura_Tabs_Dialog(xOffset, aszTabNames, 6, 30);
	pxTabs->AddDialog(pxDialog);
	s_pxSourceTabs->SelectTab(0);
	xOffset.y = pxDialog->m_xMaxCoord.y + 0.5f;

	// Save y position
	const float fTabsPosition = xOffset.y;

	// Choose attribute to edit
	static char* pszList[] = {"Speed", "Angle", "Speed spread", "Angle spread", "Release rate", "Release probability" };
	pxDialog = new Asura_DropDown_List_Dialog(xOffset, pszList, &s_iSrcTempAttrib, 6, 6, NULL);
	s_pxSourceTabs->AddDialog(pxDialog);

	// Asura_Graph<float> m_xSpeed;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	Asura_Graph_Dialog<float>* pxGraphDialog;
	s_pxSpeedGraph = pxDialog = pxGraphDialog = new Asura_Graph_Dialog<float>("Speed", xOffset, &s_pxCurrentSourceTemp->m_xSpeed, 0.0f, 1.0f, &s_bSourceTempChanged);
	pxDialog->SetHelpText("Speed in metres per second");
	s_pxSourceTabs->AddDialog(pxDialog);
	pxGraphDialog->m_pxGraphGadget->AdjustToFit(false, true);

	// Asura_Graph<Asura_Vector_3> m_xAngle;
	s_pxAngleGraph = pxDialog = new Asura_Graph_Dialog<Asura_Vector_3>("Angle", xOffset, &s_pxCurrentSourceTemp->m_xAngle, -1.0f, 1.0f, &s_bSourceTempChanged);
	pxDialog->SetHelpText("Angle offset from source direction; Red = Pitch, Green = Yaw, Blue = Roll");
	s_pxSourceTabs->AddDialog(pxDialog);

	// Asura_Graph<float> m_xSpeedSpread;
	s_pxSpeedSpreadGraph = pxDialog = new Asura_Graph_Dialog<float>("Speed spread", xOffset, &s_pxCurrentSourceTemp->m_xSpeedSpread, 0.0f, 1.0f, &s_bSourceTempChanged);
	pxDialog->SetHelpText("Randomness in speed. 0 = uniform speed, 1 = random speed from 0 to value given by speed graph");
	s_pxSourceTabs->AddDialog(pxDialog);

	// Asura_Graph<Asura_Vector_3> m_xAngleSpread;
	s_pxAngleSpreadGraph = pxDialog = new Asura_Graph_Dialog<Asura_Vector_3>("Angle spread", xOffset, &s_pxCurrentSourceTemp->m_xAngleSpread, 0.0f, 1.0f, &s_bSourceTempChanged);
	pxDialog->SetHelpText("Angle spread; Red = Pitch, Green = Yaw, Blue = Roll");
	s_pxSourceTabs->AddDialog(pxDialog);

	// Asura_Graph<float> m_xReleaseRate;
	s_pxReleaseRateGraph = pxDialog = pxGraphDialog = new Asura_Graph_Dialog<float>("Release rate", xOffset, &s_pxCurrentSourceTemp->m_xReleaseRate, 0.0f, 10.0f, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);
	pxDialog->SetEnabled(!s_pxCurrentSourceTemp->m_bBurst);
	pxGraphDialog->m_pxGraphGadget->AdjustToFit(false, true);

	// Asura_Graph<float> m_xReleaseRand;
	s_pxReleaseRandGraph = pxDialog = new Asura_Graph_Dialog<float>("Release probability", xOffset, &s_pxCurrentSourceTemp->m_xReleaseRand, 0.0f, 1.0f, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);

	xOffset.y = pxDialog->m_xMaxCoord.y;

	s_pxSourceTabs->SelectTab(1);
	xOffset.y = fTabsPosition;

	// Gravity
	pxDialog = new Asura_Dialog("Gravity", xOffset);
	s_pxSourceTabs->AddDialog(pxDialog);

	// unsigned int	m_bDirectionalGravity	:1;
	s_bSrcDirectionalGravity = s_pxCurrentSourceTemp->m_bDirectionalGravity;
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Tickbox_Dialog("Directional", xOffset, &s_bSrcDirectionalGravity, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);

	// unsigned int	m_bPointGravity			:1;
	s_bSrcPointGravity = s_pxCurrentSourceTemp->m_bPointGravity;
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Tickbox_Dialog("Point", xOffset, &s_bSrcPointGravity, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);
	xOffset.x = 1.f;

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = s_pxEditGravityDialog = new Asura_Function_Dialog("Edit", xOffset, EditGravity );
	pxDialog->SetEnabled(s_bSrcDirectionalGravity);// || s_bSrcPointGravity); - KOR 07/06/2005 - point gravity is always at effect position for now
	s_pxSourceTabs->AddDialog(pxDialog);

	xOffset.x = fBackupX;

	// Asura_Graph<float> m_xGravityStrength;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	s_pxGravityGraph = pxDialog = pxGraphDialog = new Asura_Graph_Dialog<float>("", xOffset, &s_pxCurrentSourceTemp->m_xGravityStrength, 0.0f, 10.0f, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);
	pxDialog->SetEnabled(s_pxCurrentSourceTemp->m_bDirectionalGravity || s_pxCurrentSourceTemp->m_bPointGravity);
	pxGraphDialog->m_pxGraphGadget->AdjustToFit(false, true);

	s_pxSourceTabs->SelectTab(2);
	xOffset.y = fTabsPosition;

	// unsigned int	m_bUseTurbulence		:1;
	s_bSrcUseTurbulence = s_pxCurrentSourceTemp->m_bUseTurbulence;
	pxDialog = new Asura_Tickbox_Dialog("Use turbulence", xOffset, &s_bSrcUseTurbulence, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);

	// float m_fTurbulenceFrequency;
	xOffset.x += pxDialog->m_xMaxCoord.x;
	pxDialog = s_pxTurbulenceDialog = new Asura_Numerical_Dialog("Frequency", xOffset, &s_pxCurrentSourceTemp->m_fTurbulenceFrequency, 0.f, 100.f, .1f, "%.3f", &s_bSourceTempChanged);
	pxDialog->SetEnabled(s_bSrcUseTurbulence);
	s_pxSourceTabs->AddDialog(pxDialog);
	xOffset.x = 1.f;

	// Asura_Graph<float> m_xTurbulenceStrength;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	s_pxTurbulenceGraph = pxDialog = pxGraphDialog = new Asura_Graph_Dialog<float>("", xOffset, &s_pxCurrentSourceTemp->m_xTurbulenceStrength, 0.0f, 10.0f, &s_bSourceTempChanged);
	pxDialog->SetEnabled(s_pxCurrentSourceTemp->m_bUseTurbulence);
	s_pxSourceTabs->AddDialog(pxDialog);
	pxGraphDialog->m_pxGraphGadget->AdjustToFit(false, true);

	// Scaling
	s_pxSourceTabs->SelectTab(3);
	xOffset.y = fTabsPosition;

	// unsigned int	m_bUseTimeScale			:1;
	s_bSrcUseTimeScale = s_pxCurrentSourceTemp->m_bUseTimeScale;
	pxDialog = new Asura_Tickbox_Dialog("Use timescale", xOffset, &s_bSrcUseTimeScale, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);

	// float m_fTimeScale;
	xOffset.x += pxDialog->m_xMaxCoord.x;
	pxDialog = s_pxTimeScaleDialog = new Asura_Numerical_Dialog("Scale", xOffset, &s_pxCurrentSourceTemp->m_fTimeScale, 0.f, 100.f, .1f, "%.3f", &s_bSourceTempChanged);
	pxDialog->SetEnabled(s_bSrcUseTimeScale);
	s_pxSourceTabs->AddDialog(pxDialog);
	xOffset.x = 1.f;

	// Scaling factor
	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Numerical_Dialog("Size scaling factor", xOffset, &s_fScalingFactor, 0.f, 100.f, .1f, "%.4f");
	s_pxSourceTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Function_Dialog("Scale source preserving particle count", xOffset, ScaleSourcePreserve);
	pxDialog->SetHelpText("This will scale particle size, speed, and gravity");
	s_pxSourceTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Function_Dialog("Scale source changing particle count", xOffset, ScaleSourceChange);
	pxDialog->SetHelpText("This will scale particle size and lifetime");
	s_pxSourceTabs->AddDialog(pxDialog);

	// unsigned int	m_bUseSpeedScale		:1;
	s_bSrcUseSpeedScale = s_pxCurrentSourceTemp->m_bUseSpeedScale;
	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Use code speed scale", xOffset, &s_bSrcUseSpeedScale, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);

	// Generation
	s_pxSourceTabs->SelectTab(4);
	xOffset.y = fTabsPosition;

	// u_int m_uMaxParticles;
	s_bMaxParticlesChanged = false;
	s_iMaximumParticles = s_pxCurrentSourceTemp->m_uMaxParticles;
	pxDialog = new Asura_Numerical_Dialog("Maximum particles", xOffset, &s_iMaximumParticles, 1, TOOLS_PFX_MAX_BURST_PARTICLES, 1, "%d", &s_bMaxParticlesChanged);
	pxDialog->SetHelpText("Maximum number of particles that source can have at one time.\nCan only be set manually for burst effects.");
	pxDialog->SetIsEnabledFunction(SourceIsBurst);
	s_pxSourceTabs->AddDialog(pxDialog);

	// unsigned int m_bBurst				:1;
	s_bSrcBurst = s_pxCurrentSourceTemp->m_bBurst;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Burst", xOffset, &s_bSrcBurst, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);

	// unsigned int	m_bUseRange				:1;
	s_bSrcUseRange = s_pxCurrentSourceTemp->m_bUseRange;
	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Generate in range", xOffset, &s_bSrcUseRange, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);

	// float m_fRangeDelayTime;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Delay Time", xOffset, &s_pxCurrentSourceTemp->m_fRangeDelayTime, 0.f, 60.f, 1.f, "%.1f", &s_bSourceTempChanged);
	pxDialog->SetIsEnabledFunction(SourceIsUsingRange);
	s_pxSourceTabs->AddDialog(pxDialog);

	// float m_fRangeMin;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Range min (m)", xOffset, &s_pxCurrentSourceTemp->m_fRangeMin, 0.f, 2000.f, 1.f, "%.1f", &s_bSourceTempChanged);
	pxDialog->SetIsEnabledFunction(SourceIsUsingRange);
	s_pxSourceTabs->AddDialog(pxDialog);

	// float m_fRangeMax;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Range max (m)", xOffset, &s_pxCurrentSourceTemp->m_fRangeMax, 0.f, 2000.f, 1.f, "%.1f", &s_bSourceTempChanged);
	pxDialog->SetIsEnabledFunction(SourceIsUsingRange);
	s_pxSourceTabs->AddDialog(pxDialog);

	s_bSrcScaleReleaseRateWithVolume = s_pxCurrentSourceTemp->m_bScaleReleaseRateWithVolume;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Scale release rate with volume", xOffset, &s_bSrcScaleReleaseRateWithVolume, &s_bSourceTempChanged);
	pxDialog->SetHelpText("Scales the release rate by the source volume size (in m^3), so that sources with different volumes maintain the same density.");
	s_pxSourceTabs->AddDialog(pxDialog);

	s_bSrcSpawnFromParticles = (s_pxCurrentSourceTemp->m_iSpawnSourceIndex != -1);
	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Spawn from particles", xOffset, &s_bSrcSpawnFromParticles, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);


	s_iSrcSpawnIndex = s_bSrcSpawnFromParticles ? s_pxCurrentSourceTemp->m_iSpawnSourceIndex + 1 : 1;
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Numerical_Dialog("Source index", xOffset, &s_iSrcSpawnIndex, 1, 128, 1, "%d", &s_bSourceTempChanged);
	pxDialog->SetIsEnabledFunction(SourceIsSpawningFromParticles);
	s_pxSourceTabs->AddDialog(pxDialog);
	xOffset.x = 1.f;	

	// unsigned int	m_bStopInstantly		:1;
	s_bSrcStopInstantly = s_pxCurrentSourceTemp->m_bStopInstantly;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Stop Instantly", xOffset, &s_bSrcStopInstantly, &s_bSourceTempChanged);
	pxDialog->SetHelpText("Causes the particles from this source to be destroyed instantly when the effect is stopped");
	s_pxSourceTabs->AddDialog(pxDialog);

	// Other
	s_pxSourceTabs->SelectTab(5);
	xOffset.y = fTabsPosition;

	// float m_fDuration;
	pxDialog = new Asura_Numerical_Dialog("Duration", xOffset, &s_pxCurrentSourceTemp->m_fDuration, 0.f, 60.f, .1f, "%.4f", &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);

	// unsigned int m_bUseParticleDynamics	:1;
	s_bSrcUseParticleDynamics = s_pxCurrentSourceTemp->m_bUseParticleDynamics;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Use particle dynamics", xOffset, &s_bSrcUseParticleDynamics, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);

	// float m_fParticleBounce;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = s_pxParticleBounceDialog = new Asura_Numerical_Dialog("Bounciness", xOffset, &s_pxCurrentSourceTemp->m_fParticleBounce, 0.f, 1.f, .01f, "%.2f", &s_bSourceTempChanged);
	pxDialog->SetEnabled(s_bSrcUseParticleDynamics);
	s_pxSourceTabs->AddDialog(pxDialog);

	// float m_fParticleFriction;
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = s_pxParticleFrictionDialog = new Asura_Numerical_Dialog("Friction", xOffset, &s_pxCurrentSourceTemp->m_fParticleFriction, 0.f, 10.f, .1f, "%.2f", &s_bSourceTempChanged);
	pxDialog->SetEnabled(s_bSrcUseParticleDynamics);
	s_pxSourceTabs->AddDialog(pxDialog);
	xOffset.x = 1.f;

	// unsigned int m_bUseSourceDynamics	:1; 
	s_bSrcUseSourceDynamics = s_pxCurrentSourceTemp->m_bUseSourceDynamics;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Use source dynamics", xOffset, &s_bSrcUseSourceDynamics, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);

	// float m_fSourceBounce;
	xOffset.x += pxDialog->m_xMaxCoord.x;
	pxDialog = s_pxSourceBounceDialog = new Asura_Numerical_Dialog("Bounciness", xOffset, &s_pxCurrentSourceTemp->m_fSourceBounce, 0.f, 1.f, .1f, "%.2f", &s_bSourceTempChanged);
	pxDialog->SetEnabled(s_bSrcUseSourceDynamics);
	s_pxSourceTabs->AddDialog(pxDialog);
	xOffset.x = 1.f;

	// unsigned int	m_bMovePclesWithSource	:1;
	s_bSrcMovePclesWithSource = s_pxCurrentSourceTemp->m_bMovePclesWithSource;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Move particles with effect", xOffset, &s_bSrcMovePclesWithSource, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);

	// unsigned int	s_bRotatePclesWithSource	:1;
	s_bSrcRotatePclesWithSource = s_pxCurrentSourceTemp->m_bRotatePclesWithSource;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	xOffset.x+=2;
	pxDialog = new Asura_Tickbox_Dialog("Also rotate particles with effect", xOffset, &s_bSrcRotatePclesWithSource, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);
	xOffset.x-=2;

	// unsigned int	m_bInheritVelocity		:1;
	s_bSrcInheritVelocity = s_pxCurrentSourceTemp->m_bInheritVelocity;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Inherit velocity from effect", xOffset, &s_bSrcInheritVelocity, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);

	// float m_fVelocityInheritance;
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = s_pxInheritVelocityDialog = new Asura_Numerical_Dialog("Proportion", xOffset, &s_pxCurrentSourceTemp->m_fVelocityInheritance, 0.f, 1.f, .1f, "%.3f", &s_bSourceTempChanged);
	pxDialog->SetEnabled(s_bSrcInheritVelocity);
	s_pxSourceTabs->AddDialog(pxDialog);
	xOffset.x = 1.f;

	s_bSrcUseSourceBBForVisibility = s_pxCurrentSourceTemp->m_bUseSourceBBForVisibility;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Use source BB for visibility", xOffset, &s_bSrcUseSourceBBForVisibility, &s_bSourceTempChanged);
	pxDialog->SetHelpText("Uses the source bounding box for the visibility bounding box. Only use this if you know the particles will never leave their spawn volume, for instance if point gravity is used.");
	s_pxSourceTabs->AddDialog(pxDialog);

	// Air resistance
	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Dialog("Air resistance", xOffset);
	s_pxSourceTabs->AddDialog(pxDialog);

	// float m_fAirResistance;
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Numerical_Dialog("", xOffset, &s_pxCurrentSourceTemp->m_fAirResistance, 0.f, 100.f, .1f, "%.2f", &s_bSourceTempChanged);
	pxDialog->SetIsEnabledFunction(SourceIsUsingAirResistance);
	s_pxSourceTabs->AddDialog(pxDialog);
	xOffset.x = 1.f;

	// unsigned int	m_bARPTS				:1;
	s_bSrcARPTS = s_pxCurrentSourceTemp->m_bARPTS;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Proportional to size", xOffset, &s_bSrcARPTS, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);

	// unsigned int	m_bFixedAirResistance	:1;
	s_bSrcFixedAirResistance = s_pxCurrentSourceTemp->m_bFixedAirResistance;
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Tickbox_Dialog("Fixed", xOffset, &s_bSrcFixedAirResistance, &s_bSourceTempChanged);
	s_pxSourceTabs->AddDialog(pxDialog);
	xOffset.x = 1.f;

	xOffset.y = s_pxSourceTabs->GetMaxCoords().y;

//	s_pxSourceTabs->SelectTab(-1);
//	pxOverlay->ID = TOOLS_OVERLAY_PFX_EDIT_SOURCE_TEMPLATE;
//	Asura_Overlay_System::SubmitNewOverlay(pxOverlay, false, true, true);

	// Make sure we remember previously used Tabs, so we can restore them once the window is recreated.
	s_pxSourceTabs->SetSelectedTabPointer(&s_uSelectedTab_Source);
}

void Tools_Mode_PFX::AddEffectTemplateOverlayControls(Asura_Tabs_Dialog* pxTabs, Asura_Vector_2& xOffset, Asura_Hash_ID uTemplateNameHash)
{
	// Make sure there isn't one open already.
	CloseEditEffectTemplate();

	Asura_Dialog* pxDialog = 0;
	
	// Get template
	s_pxCurrentEffectTemp = Asura_PFX_System::GetEffectTempByHash(uTemplateNameHash);
	// Check we have it
	if(!s_pxCurrentEffectTemp) return;
	// Save it for undo changes
	s_pxSavedEffectTemp = new Asura_PFX_Effect_Template(*s_pxCurrentEffectTemp);

	pxDialog = new Asura_Function_Dialog("Save", xOffset, SaveCurrentTemplate );
	pxDialog->SetHelpText("Save current effect template and all source and particle templates used by it.");
	pxTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 2.0f;
	pxDialog = new Asura_Function_Dialog("Reload", xOffset, ReloadCurrentEffectTempFull );
	pxDialog->SetHelpText("Reload current effect template and all source and particle templates used by it.");
	pxTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 2.f;
	pxDialog = new Asura_Function_Dialog("Cancel", xOffset, CancelEffectTemplateChanges );
	pxDialog->SetHelpText("Undo changes to this effect template");
	pxTabs->AddDialog(pxDialog);

	xOffset.x = 1.0f;

	// Create heading
	xOffset.y += Asura_Dialog::LargeSpacing();
	const Asura_String* pxName = Asura_PFX_System::GetEffectTempNameByHash(uTemplateNameHash);
	s_uEditingEffectTempHash = uTemplateNameHash;
	pxDialog = new Asura_Dialog(pxName ? pxName->GetString() : "", xOffset);
	pxTabs->AddDialog(pxDialog);	

	xOffset.y += Asura_Dialog::LargeSpacing();
	const char* aszTabNames[] = { "Sources", "Photon", "Sound", "Scaling", "Other", "Platform", "Generation" };
	pxDialog = s_pxEffectTabs = new Asura_Tabs_Dialog(xOffset, aszTabNames, 7, 30);
	pxTabs->AddDialog(pxDialog);
	s_pxEffectTabs->SelectTab(0);
	xOffset.y = pxDialog->m_xMaxCoord.y + 0.5f;

	// Save y position
	const float fTabsPosition = xOffset.y;

	// unsigned int	m_uNumSources;
	s_bNumSourcesChanged = false;
	s_iNumberOfSources = s_pxCurrentEffectTemp->m_uNumSources;
	pxDialog = new Asura_Numerical_Dialog("Number of particle sources", xOffset, &s_iNumberOfSources, 1, TOOLS_PFX_MAX_SOURCES_PER_EFFECT, 1, "%d", &s_bNumSourcesChanged);
	s_pxEffectTabs->AddDialog(pxDialog);

	// Choose source to edit
	s_iEffectSourceNum = 0;
	UpdateSourceChooser();
	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = s_pxEffectSourceNumDialog = new Asura_DropDown_List_Dialog(xOffset, s_aszSourceList, &s_iEffectSourceNum, 8, s_pxCurrentEffectTemp->m_uNumSources, NULL);
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Dialog("Reorder current source: ", xOffset);
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Function_Dialog("Move up", xOffset, MoveSourceUp, NULL, CanMoveSourceUp);
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Function_Dialog("Move down", xOffset, MoveSourceDown, NULL, CanMoveSourceDown);
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.x = 1.f;

	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Function_Dialog("Remove Current", xOffset, RemoveCurrentSource, NULL, CanRemoveCurrentSource);
	s_pxEffectTabs->AddDialog(pxDialog);

	// Isolate source
	s_bIsolateSource = false;
	xOffset.x += pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Tickbox_Dialog("Isolate current", xOffset, &s_bIsolateSource, NULL);
	s_pxEffectTabs->AddDialog(pxDialog);
	xOffset.x = 1.f;

	// Separator
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Dialog("-------------------------------", xOffset);
	s_pxEffectTabs->AddDialog(pxDialog);

	// Per source attributes
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Dialog("Source Attributes", xOffset);
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Dialog("Template: ", xOffset);
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Function_Dialog("New", xOffset, CreateNewSourceTemplate);
	pxDialog->SetHelpText("Create a new source template and use it for this source");
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Function_Dialog("Copy", xOffset, CopySourceTemplate);
	pxDialog->SetHelpText("Copy a source template and use it for this source.\nThe template is selected from a list of all templates from all effects");
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Function_Dialog("Rename", xOffset, RenameSourceTemplate);
	pxDialog->SetHelpText("Rename the currently selected source template");
	s_pxEffectTabs->AddDialog(pxDialog);
	xOffset.x = 1.f;

	// Asura_Hash_ID m_auSourceTemplates[ASURA_PFX_MAX_SOURCES_PER_EFFECT];
	s_iEffectSourceTemp = 0;
	s_bEffectSourceChanged = false;
	s_iEffectCurrentSourceNum = -1;
	UpdateSourceTempList();
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = s_pxEffectSourceTempDialog = new Asura_DropDown_List_Dialog(xOffset, s_pszSourceTempList, &s_iEffectSourceTemp, 8, s_iSourceTempListSize, NULL, NULL, "");
	s_pxEffectTabs->AddDialog(pxDialog);

/*	xOffset.x = pxDialog->MaxCoord.x + 2.f;
	pxDialog = s_pxEffectEditSourceTempDialog = new Asura_Function_Dialog("Edit", xOffset, EditEffectSourceTemplate);
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.x = 1.f;
*/
	// Asura_Vector_3 m_axSrcPosOffsets[ASURA_PFX_MAX_SOURCES_PER_EFFECT];
	// Asura_Vector_3 m_axSrcOrientations[ASURA_PFX_MAX_SOURCES_PER_EFFECT];
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Function_Dialog("Edit volume", xOffset, EditSourceVector);
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.x += pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Function_Dialog("Reset volume", xOffset, ResetSourceVector);
	s_pxEffectTabs->AddDialog(pxDialog);
	xOffset.x = 1.f;

	// Asura_Vector_3 m_axSrcVelocities[ASURA_PFX_MAX_SOURCES_PER_EFFECT];
	xOffset.y += Asura_Dialog::DefaultSpacing();
	s_fEffectSrcSpeed = s_pxCurrentEffectTemp->m_pxSources[0].m_xVelocity.Magnitude();
	s_bEffectSrcSpeedChanged = false;
	pxDialog = new Asura_Numerical_Dialog("Movement speed", xOffset, &s_fEffectSrcSpeed, 0.f, 100.f, .1f, "%.4f", &s_bEffectSrcSpeedChanged);
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.x += pxDialog->m_xMaxCoord.x;
	pxDialog = s_pxEffectEditSourceMovement = new Asura_Function_Dialog("Edit direction", xOffset, EditSourceMovementVector);
	s_pxEffectTabs->AddDialog(pxDialog);
	s_pxEffectEditSourceMovement->SetEnabled(s_fEffectSrcSpeed > 0.f);
	xOffset.x = 1.f;

	s_fEffectSrcTimeOffset = s_pxCurrentEffectTemp->m_pxSources[0].m_fTimeOffset;
	s_bEffectSrcTimeOffsetChanged = false;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Time offset", xOffset, &s_fEffectSrcTimeOffset, 0.f, 100.f, .1f, "%.4f", &s_bEffectSrcTimeOffsetChanged);
	s_pxEffectTabs->AddDialog(pxDialog);

	s_bEffectUseCodeOrient = (s_pxCurrentEffectTemp->m_pxSources[0].m_iOrientationID != 0);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Use code orientation", xOffset, &s_bEffectUseCodeOrient, NULL);
	s_pxEffectTabs->AddDialog(pxDialog);

	s_iEffectSrcOrientID = (s_pxCurrentEffectTemp->m_pxSources[0].m_iOrientationID == 0) ? 1 : s_pxCurrentEffectTemp->m_pxSources[0].m_iOrientationID;
	s_bEffectSrcOrientIDChanged = false;
	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Numerical_Dialog("Orientation ID", xOffset, &s_iEffectSrcOrientID, 1, 8, 1, "%u", &s_bEffectSrcOrientIDChanged);
	pxDialog->SetIsEnabledFunction(EffectIsUsingCodeOrient);
	s_pxEffectTabs->AddDialog(pxDialog);
	xOffset.x = 1.f;

	xOffset.y += Asura_Dialog::LargeSpacing();
	s_bEffectSrcDisabled = s_pxCurrentEffectTemp->m_pxSources[0].m_bDisabled;
	pxDialog = new Asura_Tickbox_Dialog("Disable", xOffset, &s_bEffectSrcDisabled, NULL);
	pxDialog->SetHelpText("Temporarily disables rendering of this source for testing purposes while editing this effect.");
	s_pxEffectTabs->AddDialog(pxDialog);


	s_pxEffectTabs->SelectTab(1);
	xOffset.y = fTabsPosition;

	// bool m_bHasPhoton;
	s_bEffectHasPhoton = s_pxCurrentEffectTemp->m_bHasPhoton;
	pxDialog = new Asura_Tickbox_Dialog("Use photon", xOffset, &s_bEffectHasPhoton, &s_bEffectTempChanged);
	s_pxEffectTabs->AddDialog(pxDialog);

	s_bEffectParticlePhotons = s_pxCurrentEffectTemp->m_bParticlePhotons;
	xOffset.x += 12.f;
	pxDialog = new Asura_Tickbox_Dialog("Photon per particle (Expensive!)", xOffset, &s_bEffectParticlePhotons, &s_bEffectTempChanged);
	pxDialog->SetIsEnabledFunction(EffectHasPhoton);
	s_pxEffectTabs->AddDialog(pxDialog);
	xOffset.x -= 12.f;

	s_iEffectPhotonPerParticleSource = s_pxCurrentEffectTemp->m_iPhotonPerParticleSource+1;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("From Source",xOffset,&s_iEffectPhotonPerParticleSource,0,TOOLS_PFX_MAX_SOURCES_PER_EFFECT,1,"%d",  &s_bEffectTempChanged);
	pxDialog->SetHelpText("If this is set to a non-zero value then this will limit the photon per particle option to a single source.");
	pxDialog->SetIsEnabledFunction(EffectHasPhotonPerParticle);
	s_pxEffectTabs->AddDialog(pxDialog);

	// Choose attribute to edit
	static char* pszList[] = {"Photon Colour", "Photon Radius"};
	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = s_pxEffectAttribDialog = new Asura_DropDown_List_Dialog(xOffset, pszList, &s_iEffectAttrib, 2, 2, NULL, NULL, "Photon attribute");
	s_pxEffectTabs->AddDialog(pxDialog);

	s_bEffectPhotonPropertiesChanged = false;
	// Asura_Graph<Asura_RGBA> m_xPhotonColour;
	xOffset.y += Asura_Dialog::DefaultSpacing();
	s_pxPhotonColourGraph = pxDialog = new Asura_Graph_Dialog<Asura_RGBA>("Photon colour", xOffset, &s_pxCurrentEffectTemp->m_xPhotonColour, 0.f, fASURA_PFX_MAX_PHOTON_COLOUR, &s_bEffectPhotonPropertiesChanged);
	s_pxEffectTabs->AddDialog(pxDialog);

	Asura_Graph_Dialog<float>* pxGraphDialog;
	// Asura_Graph<float> m_xPhotonRadius;
	s_pxPhotonRadiusGraph = pxDialog = pxGraphDialog = new Asura_Graph_Dialog<float>("Photon radius", xOffset, &s_pxCurrentEffectTemp->m_xPhotonRadius, 0.f, 10.f, &s_bEffectPhotonPropertiesChanged);
	pxGraphDialog->m_pxGraphGadget->AdjustToFit(false, true);
	s_pxEffectTabs->AddDialog(pxDialog);

	s_pxEffectTabs->SelectTab(2);
	xOffset.y = fTabsPosition;

	s_uEffectSoundEvent = s_pxCurrentEffectTemp->m_uSoundEvent;
	Asura_Dialog* pxSoundEventDialog = pxDialog = new Tools_PickSoundEvent_Dialog("Event : ", xOffset, &s_uEffectSoundEvent, &s_bEffectTempChanged);
	pxDialog->SetHelpText("Pick sound event to use.");
	s_pxEffectTabs->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	//add control for editing the selected sound event
	pxDialog = new Tools_EditSoundEvent_Dialog("Edit",xOffset,&s_uEffectSoundEvent);
	s_pxEffectTabs->AddDialog(pxDialog);
	pxDialog->SetRelativePosition(pxSoundEventDialog,Asura_Vector_2(3.0f,0.0f),false,true);


	s_pxEffectTabs->SelectTab(3);
	xOffset.y = fTabsPosition;

	// Scaling factor
	pxDialog = new Asura_Numerical_Dialog("Scaling factor", xOffset, &s_fScalingFactor, 0.f, 100.f, .1f, "%.4f");
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Function_Dialog("Scale effect size preserving particle count", xOffset, ScaleEffectPreserve);
	pxDialog->SetHelpText("This will scale particle size, speed, and gravity");
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Function_Dialog("Scale effect size changing particle count", xOffset, ScaleEffectChange);
	pxDialog->SetHelpText("This will scale particle size and lifetime");
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Function_Dialog("Scale effect timescale", xOffset, ScaleEffectTime);
	pxDialog->SetHelpText("This will change the timescales of the sources and the lifetime of the effect");
	s_pxEffectTabs->AddDialog(pxDialog);

	// Other
	s_pxEffectTabs->SelectTab(4);
	xOffset.y = fTabsPosition;

	if(s_pxThis)
	{
		s_pxThis->AddItemsToEffectOtherTab(s_pxEffectTabs, xOffset, s_pxCurrentEffectTemp);
	}
	else
	{
		Static_AddItemsToEffectOtherTab(s_pxEffectTabs, xOffset, s_pxCurrentEffectTemp);
	}

	//platform overrides tab
	s_pxEffectTabs->SelectTab(5);
	xOffset.y = fTabsPosition;

	pxDialog = new Asura_Function_Dialog("Add Platform Override",xOffset,AddEffectPlatformOverride);
	pxDialog->SetHelpText("Adds a platform override for this effect. This allows a different effect to be substituted in for the specified platform.");
	s_pxEffectTabs->AddDialog(pxDialog);

	for(u_int u=0;u<s_pxCurrentEffectTemp->m_uNumPlatformOverrides;u++)
	{
		xOffset.y += Asura_Dialog::LargeSpacing();

		//platform name
		pxDialog = new Asura_Dialog(Tools_Level_Export::GetPlatformName(s_pxCurrentEffectTemp->m_pxPlatformOverrides[u].m_iPlatform),xOffset);
		s_pxEffectTabs->AddDialog(pxDialog);

		//effect list
		xOffset.x += 8.0f;
		pxDialog = new Tools_PFXSelect_Dialog(Tools_PFXSelect_Dialog::MODE_EFFECTS, xOffset,&s_pxCurrentEffectTemp->m_pxPlatformOverrides[u].m_uEffect,&s_bEffectTempChanged);
		pxDialog->SetHelpText("This effect will be used instead when doing level exports for this platform.");
		s_pxEffectTabs->AddDialog(pxDialog);
		xOffset.x -= 8.0f;

	}

	// Generation tab
	s_pxEffectTabs->SelectTab(6);
	xOffset.y = fTabsPosition;

	// unsigned int	m_bPregenerate				:1;
	s_bEffectPregenerate = s_pxCurrentEffectTemp->m_bPregenerate;
	pxDialog = new Asura_Tickbox_Dialog("Pregenerate", xOffset, &s_bEffectPregenerate, &s_bEffectTempChanged);
	pxDialog->SetHelpText("Run all sources on first update, looping if necessary");
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Dialog("Generate in range", xOffset);
	pxDialog->SetHelpText("Set up values then click \"Apply to sources\" to use them for all member sources.");
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Tickbox_Dialog("Use range", xOffset, &s_bEffectUseRange);
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Delay Time", xOffset, &s_fEffectRangeDelayTime, 0.f, 60.f, 1.f, "%.1f");
	pxDialog->SetIsEnabledFunction(EffectIsUsingRange);
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Numerical_Dialog("Min", xOffset, &s_fEffectRangeMin, 0.f, 2000.f, 1.f, "%.1f");
	pxDialog->SetIsEnabledFunction(EffectIsUsingRange);
	s_pxEffectTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.f;
	pxDialog = new Asura_Numerical_Dialog("Max", xOffset, &s_fEffectRangeMax, 0.f, 2000.f, 1.f, "%.1f");
	pxDialog->SetIsEnabledFunction(EffectIsUsingRange);
	s_pxEffectTabs->AddDialog(pxDialog);
	xOffset.x = 1.f;

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Function_Dialog("Apply to sources", xOffset, SetEffectRange);
	s_pxEffectTabs->AddDialog(pxDialog);

	// End of tabs
	xOffset.y = s_pxEffectTabs->GetMaxCoords().y;

	// Save options
/*	xOffset.y += Asura_Dialog::LargeSpacing();
	pxDialog = new Asura_Function_Dialog("Reload", xOffset, ReloadCurrentEffectTemp );
	pxDialog->SetHelpText("Reload this template");
	pxTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->MaxCoord.x + 2.0f;
	pxDialog = new Asura_Function_Dialog("Reload + Children", xOffset, ReloadCurrentEffectTempFull );
	pxDialog->SetHelpText("Reload this template and all source and particle templates used by it.");
	pxTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->MaxCoord.x + 2.0f;
	pxDialog = new Asura_Function_Dialog("Cancel", xOffset, CancelEffectTemplateChanges );
	pxDialog->SetHelpText("Undo changes to this effect template");
	pxTabs->AddDialog(pxDialog);

	xOffset.x = 1.0f;

	xOffset.y += Asura_Dialog::DefaultSpacing();
	pxDialog = new Asura_Function_Dialog("Save", xOffset, SaveCurrentEffectTemplate );
	pxDialog->SetHelpText("Save this template");
	pxTabs->AddDialog(pxDialog);

	xOffset.x = pxDialog->MaxCoord.x + 4.0f;
	pxDialog = new Asura_Function_Dialog("Save + Children", xOffset, SaveCurrentEffectTemplateWithChildren );
	pxDialog->SetHelpText("Save this template and all source and particle templates used by it.");
	pxTabs->AddDialog(pxDialog);
	xOffset.x = 1.0f;
*/
	if(!s_bEffectHasPhoton)
	{
		s_pxEffectAttribDialog->SetEnabled(false);
		s_pxPhotonColourGraph->SetEnabled(false);
		s_pxPhotonRadiusGraph->SetEnabled(false);
	}

	// Make sure we remember previously used Tabs, so we can restore them once the window is recreated.
	s_pxEffectTabs->SetSelectedTabPointer(&s_uSelectedTab_Effects);
}

/// Close overlays ////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_PFX::CloseEditParticleTemplate()
{
	if(s_pxCurrentParticleTemp)
	{
		s_pxCurrentParticleTemp = 0;
	}
	s_pxParticleColourGraph = NULL;
	s_pxParticleSizeGraph = NULL;
	s_pxParticleRotationGraph = NULL;
	s_pxParticleSizeVariationGraph = NULL;

	s_uEditingParticleTempHash = 0;

	if(s_pxSavedParticleTemp)
	{
		delete s_pxSavedParticleTemp;
		s_pxSavedParticleTemp = 0;
	}
	FinishUVEdit();
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_PFX_EDIT_PARTICLE_TEMPLATE);
    	
}

void Tools_Mode_PFX::CloseEditSourceTemplate()
{
	if(s_pxCurrentSourceTemp)
	{
		s_pxCurrentSourceTemp = 0;
	}
	// Clear out dialog pointers
	s_pxSpeedGraph = NULL;
	s_pxAngleGraph = NULL;
	s_pxSpeedSpreadGraph = NULL;
	s_pxAngleSpreadGraph = NULL;
	s_pxReleaseRateGraph = NULL;
	s_pxReleaseRandGraph = NULL;
	s_pxSourcePcleTempDialog = NULL;
	s_pxSourceEditPcleTempDialog = NULL;
	s_pxEditGravityDialog = NULL;
	s_pxSourceTextureNameDialog = NULL;

	s_uEditingSourceTempHash = 0;

	if(s_pxSavedSourceTemp)
	{
		delete s_pxSavedSourceTemp;
		s_pxSavedSourceTemp = 0;
	}
	if(s_pxSourceGravityObject)
	{
		if(s_pxSourceGravityObject->IsSelected())
		{
			Tools_Object_System::SelectObject(NULL);
		}
		delete s_pxSourceGravityObject;
		s_pxSourceGravityObject = NULL;
		Tools_Entity_PFX_Effect::FinishTempEntityEdit();
	}
	if(s_pszPcleTempList)
	{
		delete [] s_pszPcleTempList;
		s_pszPcleTempList = 0;
	}
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_PFX_EDIT_SOURCE_TEMPLATE);
}

void Tools_Mode_PFX::CloseEditEffectTemplate()
{
	if(s_pxCurrentEffectTemp)
	{
		s_pxCurrentEffectTemp->m_iIsolatedSource = -1;
		for(u_int i=0;i<s_pxCurrentEffectTemp->m_uNumSources;i++)
		{
			s_pxCurrentEffectTemp->m_pxSources[i].m_bDisabled = false;
		}

		s_pxCurrentEffectTemp = 0;
	}

	// Clear out dialog pointers
	s_pxPhotonColourGraph = NULL;
	s_pxPhotonRadiusGraph = NULL;
	s_pxEffectAttribDialog = NULL;
	s_pxEffectSourceNumDialog = NULL;
	s_pxEffectSourceTempDialog = NULL;
	s_pxEffectEditSourceTempDialog = NULL;
	s_pxEffectEditSourceMovement = NULL;

	for(u_int u = 0; u < TOOLS_PFX_MAX_SOURCES_PER_EFFECT; ++u)
	{
		delete [] s_aszSourceList[u];
		s_aszSourceList[u] = 0;
	}

	s_uEditingEffectTempHash = 0;

	if(s_pxSavedEffectTemp)
	{
		delete s_pxSavedEffectTemp;
		s_pxSavedEffectTemp = 0;
	}
	DeleteSourceTempList();
	
	Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_PFX_EDIT_EFFECT_TEMPLATE);
}

/// Update overlays ///////////////////////////////////////////////////////////////////////////////

void Tools_Mode_PFX::UpdateEditParticleTemplate()
{
	if(!s_pxParticleColourGraph || !s_pxParticleSizeGraph || !s_pxParticleRotationGraph || !s_pxParticleSizeVariationGraph) return;

	if(s_uLastParticleEditUpdate == Asura_Timers::GetFrameNumber()) return;
	s_uLastParticleEditUpdate = Asura_Timers::GetFrameNumber();
	
	if(!Asura_Overlay_System::OverlaysHaveKeyboardFocus())
	{
		if(!Asura_UserInput::KeyPressed(ASURA_KEYID_CTRL) && 
		   !Asura_UserInput::KeyPressed(ASURA_KEYID_SHIFT) &&
		   !Asura_UserInput::KeyPressed(ASURA_KEYID_ALT))
		{
			//Don't make use of tab key when using the texture anim initalise overlay , since it is more convenient
			//to use it for switching between controls.
			//RAR 13/06/2006
			if(!Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_PFX_TEXTUREANIM))
			{
				if(Asura_UserInput::DebouncedKeyPressed('\t') && s_pxParticleTabs->GetSelected() == 1)
				{
					if(s_pxParticleTextureEdit)
					{
						FinishUVEdit();
					}
					else
					{
						StartUVEdit();
					}
				}
			}
		}
	}

	if(s_bParticleCurrentTextureFrameChanged)
	{
		s_bParticleCurrentTextureFrameChanged = false;
		//make sure frame number hasn't gone out of range
		if(s_uParticleCurrentTextureFrame >= s_pxCurrentParticleTemp->m_uNumTextureFrames)
		{
			s_uParticleCurrentTextureFrame = s_pxCurrentParticleTemp->m_uNumTextureFrames-1;
		}

		//get the values for the selected frame
		Asura_PFX_TextureFrame* pxTextureFrame = &s_pxCurrentParticleTemp->m_pxTextureFrames[s_uParticleCurrentTextureFrame];

		for(int i=0;i<4;i++)
		{
			s_afParticleTextureUVs[i] = pxTextureFrame->m_afTextureUVs[i];
		}
		s_fParticleTextureFrameTime = pxTextureFrame->m_fTime;
		SetModified(s_pxCurrentParticleTemp);
	}

	if(s_pxParticleTextureEdit)
	{
		if(s_pxCurrentParticleTemp->m_eTranslucency == ASURA_TRANSLUCENCY_SPECIAL_HEATHAZE)
		{
			s_pxParticleTextureEdit->SetTranslucencyMode(ASURA_TRANSLUCENCY_OFF);
		}
		else
		{
			s_pxParticleTextureEdit->SetTranslucencyMode(s_pxCurrentParticleTemp->m_eTranslucency);
		}
		UpdateUVEdit();
	}

	if(s_bParticleTextureFrameTimeChanged)
	{
		s_bParticleTextureFrameTimeChanged = false;

		Asura_PFX_TextureFrame* pxTextureFrame = &s_pxCurrentParticleTemp->m_pxTextureFrames[s_uParticleCurrentTextureFrame];
		pxTextureFrame->m_fTime = s_fParticleTextureFrameTime;


		//sort frames if needed
		while(s_uParticleCurrentTextureFrame>0 && pxTextureFrame->m_fTime<s_pxCurrentParticleTemp->m_pxTextureFrames[s_uParticleCurrentTextureFrame-1].m_fTime)
		{
			Asura_PFX_TextureFrame xTemp = s_pxCurrentParticleTemp->m_pxTextureFrames[s_uParticleCurrentTextureFrame-1];
			s_pxCurrentParticleTemp->m_pxTextureFrames[s_uParticleCurrentTextureFrame-1] = s_pxCurrentParticleTemp->m_pxTextureFrames[s_uParticleCurrentTextureFrame];
			s_pxCurrentParticleTemp->m_pxTextureFrames[s_uParticleCurrentTextureFrame] = xTemp;
			s_uParticleCurrentTextureFrame--;
		}
		while(s_uParticleCurrentTextureFrame<s_pxCurrentParticleTemp->m_uNumTextureFrames-1 && pxTextureFrame->m_fTime>s_pxCurrentParticleTemp->m_pxTextureFrames[s_uParticleCurrentTextureFrame+1].m_fTime)
		{
			Asura_PFX_TextureFrame xTemp = s_pxCurrentParticleTemp->m_pxTextureFrames[s_uParticleCurrentTextureFrame+1];
			s_pxCurrentParticleTemp->m_pxTextureFrames[s_uParticleCurrentTextureFrame+1] = s_pxCurrentParticleTemp->m_pxTextureFrames[s_uParticleCurrentTextureFrame];
			s_pxCurrentParticleTemp->m_pxTextureFrames[s_uParticleCurrentTextureFrame] = xTemp;
			s_uParticleCurrentTextureFrame++;
		}
		s_bParticleTempChanged = true;
	}


	if(s_bParticleTempChanged)
	{
		// All rotation flags are mutually exclusive
		if(s_bParticleUseMotionBlur && !s_pxCurrentParticleTemp->m_bUseMotionBlur)
		{
			s_bParticleFixedRotation = false;
			s_bParticleUseRandRot = false;
		}
		if(s_bParticleUseRandRot && !s_pxCurrentParticleTemp->m_bUseRandomRotation)
		{
			s_bParticleFixedRotation = false;
			s_bParticleUseMotionBlur = false;
		}
		if(s_bParticleFixedRotation && !s_pxCurrentParticleTemp->m_bFixedRotation)
		{
			s_bParticleUseRandRot = false;
			s_bParticleUseMotionBlur = false;
		}
		// 3D rotation disables align with effect
		if(s_bParticle3DRotation && !s_pxCurrentParticleTemp->m_b3DRotation)
		{
			s_bParticleAlignEffectX = false;
			s_bParticleAlignEffectY = false;
			s_bParticleAlignEffectZ = false;
			s_bParticleBillboardJustX = false;
			s_bParticleBillboardJustY = false;
		}
		// Align effect X, Y, Z disable each other
		if(s_bParticleAlignEffectX && !s_pxCurrentParticleTemp->m_bAlignEffectX)
		{
			s_bParticleAlignEffectY = false;
			s_bParticleAlignEffectZ = false;
			s_bParticle3DRotation = false;
			s_bParticleBillboardJustX = false;
			s_bParticleBillboardJustY = false;
		}
		if(s_bParticleAlignEffectY && !s_pxCurrentParticleTemp->m_bAlignEffectY)
		{
			s_bParticleAlignEffectX = false;
			s_bParticleAlignEffectZ = false;
			s_bParticle3DRotation = false;
			s_bParticleBillboardJustX = false;
			s_bParticleBillboardJustY = false;
		}
		if(s_bParticleAlignEffectZ && !s_pxCurrentParticleTemp->m_bAlignEffectZ)
		{
			s_bParticleAlignEffectY = false;
			s_bParticleAlignEffectX = false;
			s_bParticle3DRotation = false;
			s_bParticleBillboardJustX = false;
			s_bParticleBillboardJustY = false;
		}
		if( s_bParticleBillboardJustX && !s_pxCurrentParticleTemp->m_bBillboardJustX )
		{
			s_bParticleAlignEffectX = false;
			s_bParticleAlignEffectY = false;
			s_bParticleAlignEffectZ = false;
			s_bParticle3DRotation = false;
			s_bParticleBillboardJustY = false;
		}
		if( s_bParticleBillboardJustY && !s_pxCurrentParticleTemp->m_bBillboardJustY )
		{
			s_bParticleAlignEffectX = false;
			s_bParticleAlignEffectY = false;
			s_bParticleAlignEffectZ = false;
			s_bParticle3DRotation = false;
			s_bParticleBillboardJustX = false;
		}
		if(s_bParticleLoopAnimation)
		{
			s_bParticleUseFPS = true;
		}
		if(s_pxCurrentParticleTemp->m_uNumTextureFrames<=1)
		{
			s_bParticleCrossfadeTextureFrames = false;
		}

		// Odd style forced by signed/unsigned mismatch
		if((s_pxCurrentParticleTemp->m_bRenderEarly != 0) != s_bParticleRenderEarly)
		{
			s_pxCurrentParticleTemp->m_bRenderEarly	= s_bParticleRenderEarly;
		}

		s_pxCurrentParticleTemp->m_bUseLighting						= s_bParticleUseLighting;
		s_pxCurrentParticleTemp->m_bPreLighting						= s_bParticlePreLighting;
		s_pxCurrentParticleTemp->m_bUseFogging						= s_bParticleUseFogging;
		s_pxCurrentParticleTemp->m_bUseMotionBlur					= s_bParticleUseMotionBlur;
		s_pxCurrentParticleTemp->m_bUseRandomRotation				= s_bParticleUseRandRot;
		s_pxCurrentParticleTemp->m_bRandomTextureOrient				= s_bParticleRandomTextureOrient;
		s_pxCurrentParticleTemp->m_b3DRotation						= s_bParticle3DRotation;
		s_pxCurrentParticleTemp->m_bMirrorRotation					= s_bParticleMirrorRotation;
		s_pxCurrentParticleTemp->m_bAlignEffectX					= s_bParticleAlignEffectX;
		s_pxCurrentParticleTemp->m_bAlignEffectY					= s_bParticleAlignEffectY;
		s_pxCurrentParticleTemp->m_bAlignEffectZ					= s_bParticleAlignEffectZ;
		s_pxCurrentParticleTemp->m_bBillboardJustX					= s_bParticleBillboardJustX;
		s_pxCurrentParticleTemp->m_bBillboardJustY					= s_bParticleBillboardJustY;
		s_pxCurrentParticleTemp->m_bFixedRotation					= s_bParticleFixedRotation;
		s_pxCurrentParticleTemp->m_bNearCameraFade					= s_bParticleNearCameraFade;
		s_pxCurrentParticleTemp->m_bFarCameraFade					= s_bParticleFarCameraFade;
		s_pxCurrentParticleTemp->m_bUseFPS							= s_bParticleUseFPS;
		s_pxCurrentParticleTemp->m_bSpriteRotation					= s_bParticleSpriteRotation;
		s_pxCurrentParticleTemp->m_bCrossfadeTextureFrames			= s_bParticleCrossfadeTextureFrames;
		s_pxCurrentParticleTemp->m_bLoopAnimation					= s_bParticleLoopAnimation;
		s_pxCurrentParticleTemp->m_bCanUseExpensiveClipping			= s_bParticleCanUseExpensiveClipping;
		s_pxCurrentParticleTemp->m_bUseUVAsAspect					= s_bUseUVAsAspect;		

		s_pxBlurLengthDialog->SetEnabled(s_bParticleUseMotionBlur);
		s_pxParticleRotationGraph->SetEnabled(!(s_bParticleUseMotionBlur || s_bParticleUseRandRot || s_bParticleFixedRotation));
		s_pxFixedRotationDialog->SetEnabled(s_bParticleFixedRotation);
		s_pxNumFixedRotationsDialog->SetEnabled(s_bParticleFixedRotation);
		s_pxLightingScaleDialog->SetEnabled(s_bParticleUseLighting);

		if(s_bParticleRandomLifetime)
		{
			// Clamp max using min
			if(s_fParticleMaxLifetime < s_pxCurrentParticleTemp->m_fMinLifetime)
			{
				s_fParticleMaxLifetime = s_pxCurrentParticleTemp->m_fMinLifetime;
				s_pxCurrentParticleTemp->m_fMaxLifetimeDiff = 0.f;
			}
			else
			{
				s_pxCurrentParticleTemp->m_fMaxLifetimeDiff = s_fParticleMaxLifetime - s_pxCurrentParticleTemp->m_fMinLifetime;
			}			
		}
		else
		{
			s_fParticleMaxLifetime = s_pxCurrentParticleTemp->m_fMinLifetime;
			s_pxCurrentParticleTemp->m_fMaxLifetimeDiff = 0.f;
		}
		s_pxCurrentParticleTemp->m_bRandomLifetime = s_bParticleRandomLifetime;
		s_pxCurrentParticleTemp->m_bUseSizeScale = s_bParticleUseSizeScale;
		s_pxCurrentParticleTemp->m_bLooped = s_bParticleLooped;

		Asura_PFX_TextureFrame* pxTextureFrame = &s_pxCurrentParticleTemp->m_pxTextureFrames[s_uParticleCurrentTextureFrame];

		//copy overlay values to current texture frame
		for(int i=0;i<4;i++)
		{
			pxTextureFrame->m_afTextureUVs[i] = s_afParticleTextureUVs[i];
		}
		
		if(s_pxCurrentParticleTemp->m_uSoundEvent != s_uParticleSoundEvent)
		{
			//complain if the new sound event loops
			const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::FindEvent(s_uParticleSoundEvent);
			if(pxEvent && pxEvent->IsEndlessPlayback())
			{
				s_uParticleSoundEvent = s_pxCurrentParticleTemp->m_uSoundEvent;
				ShowMessageOverlay("The sound event selected for the particle template loops.\nParticle templates can only have non-looping sound events assigned to them");
			}
		}
		s_pxCurrentParticleTemp->m_uSoundEvent = s_uParticleSoundEvent;
		

		s_bParticleTempChanged = 0;
		SetModified(s_pxCurrentParticleTemp);
	}

	s_pxParticleColourGraph->SetHidden(true);
	s_pxParticleSizeGraph->SetHidden(true);
	s_pxParticleRotationGraph->SetHidden(true);
	s_pxParticleSizeVariationGraph->SetHidden(true);

	ASURA_TRANSLUCENCY_MODE eOldTranslucency = s_pxCurrentParticleTemp->m_eTranslucency;
	switch(s_iParticleTranslucency)
	{
		case 0: s_pxCurrentParticleTemp->m_eTranslucency = ASURA_TRANSLUCENCY_NORMAL;			break;
		case 1: s_pxCurrentParticleTemp->m_eTranslucency = ASURA_TRANSLUCENCY_COLOUR;			break;
		case 2: s_pxCurrentParticleTemp->m_eTranslucency = ASURA_TRANSLUCENCY_INVCOLOUR;		break;
		case 3: s_pxCurrentParticleTemp->m_eTranslucency = ASURA_TRANSLUCENCY_GLOWING;			break;
		case 4: s_pxCurrentParticleTemp->m_eTranslucency = ASURA_TRANSLUCENCY_SPECIAL_HEATHAZE;	break;
		case 5: s_pxCurrentParticleTemp->m_eTranslucency = ASURA_TRANSLUCENCY_OFF;				break;
		default: Asura_Assert(0, "Invalid translucency type");
	}
	if(eOldTranslucency != s_pxCurrentParticleTemp->m_eTranslucency)
	{
		SetModified(s_pxCurrentParticleTemp);
	}
	if(s_pxParticleTabs->GetSelected() == 0 && !s_pxParticleTabs->IsHidden())
	{
		switch(s_iParticleTempAttrib)
		{
			case 0:	s_pxParticleColourGraph->SetHidden(false);	break;
			case 1:	s_pxParticleSizeGraph->SetHidden(false);	break;
			case 2:	s_pxParticleRotationGraph->SetHidden(false);break;
			case 3: s_pxParticleSizeVariationGraph->SetHidden(false);break;
			default: Asura_Assert(0, "Invalid particle attribute selection");
		}
	}
}

void Tools_Mode_PFX::UpdateEditSourceTemplate()
{
	if(!s_pxCurrentSourceTemp) return;
	if(!s_pxSpeedGraph) return;

	if(s_uLastSourceEditUpdate == Asura_Timers::GetFrameNumber()) return;
	s_uLastSourceEditUpdate = Asura_Timers::GetFrameNumber();
	
	if(s_bSourceTempChanged)
	{
		// Different air resistance types cancel each other
		if(s_bSrcARPTS && !s_pxCurrentSourceTemp->m_bARPTS)
		{
			s_bSrcFixedAirResistance = false;
		}
		if(s_bSrcFixedAirResistance && !s_pxCurrentSourceTemp->m_bFixedAirResistance)
		{
			s_bSrcARPTS = false;
		}
		// Different gravity types cancel each other
		if(s_bSrcDirectionalGravity && !s_pxCurrentSourceTemp->m_bDirectionalGravity)
		{
			s_bSrcPointGravity = false;
			s_pxCurrentSourceTemp->m_xGravityVector = Asura_Vector_3(0.f, 1.f, 0.);
		}
		if(s_bSrcPointGravity && !s_pxCurrentSourceTemp->m_bPointGravity)
		{
			s_bSrcDirectionalGravity = false;
			s_pxCurrentSourceTemp->m_xGravityVector = Asura_Vector_3(0.f, 0.f, 0.);
		}
		if(s_bSrcSpawnFromParticles)
		{
			s_pxCurrentSourceTemp->m_iSpawnSourceIndex = s_iSrcSpawnIndex - 1;
		}
		else
		{
			s_pxCurrentSourceTemp->m_iSpawnSourceIndex = -1;
		}
		
		if((s_pxCurrentSourceTemp->m_bScaleReleaseRateWithVolume ? true : false) != s_bSrcScaleReleaseRateWithVolume)
		{
			// Need to restart to regenerate particle array with correct size.
			Tools_Entity_PFX_Effect::ClearAndFlagForRestart_SourceTemplate(s_pxCurrentSourceTemp->GetTemplateHash());
			s_pxCurrentSourceTemp->m_bScaleReleaseRateWithVolume = s_bSrcScaleReleaseRateWithVolume;
		}

		// Update all source flags
		s_pxCurrentSourceTemp->m_bUseWind = s_bSrcUseWind;
		s_pxCurrentSourceTemp->m_bUseParticleDynamics = s_bSrcUseParticleDynamics;
		s_pxCurrentSourceTemp->m_bUseSourceDynamics = s_bSrcUseSourceDynamics;
		s_pxCurrentSourceTemp->m_bUseTimeScale = s_bSrcUseTimeScale;
		s_pxCurrentSourceTemp->m_bMovePclesWithSource = s_bSrcMovePclesWithSource;
		s_pxCurrentSourceTemp->m_bRotatePclesWithSource = s_bSrcRotatePclesWithSource;
		s_pxCurrentSourceTemp->m_bBurst = s_bSrcBurst;
		s_pxCurrentSourceTemp->m_bUseTurbulence = s_bSrcUseTurbulence;
		s_pxCurrentSourceTemp->m_bARPTS = s_bSrcARPTS;
		s_pxCurrentSourceTemp->m_bFixedAirResistance = s_bSrcFixedAirResistance;
		s_pxCurrentSourceTemp->m_bDirectionalGravity = s_bSrcDirectionalGravity;
		s_pxCurrentSourceTemp->m_bPointGravity = s_bSrcPointGravity;
		s_pxCurrentSourceTemp->m_bInheritVelocity = s_bSrcInheritVelocity;
		s_pxCurrentSourceTemp->m_bUseRange = s_bSrcUseRange;
		s_pxCurrentSourceTemp->m_bUseSpeedScale = s_bSrcUseSpeedScale;
		s_pxCurrentSourceTemp->m_bStopInstantly = s_bSrcStopInstantly;
		s_pxCurrentSourceTemp->m_bUseSourceBBForVisibility = s_bSrcUseSourceBBForVisibility;

		s_pxReleaseRateGraph->SetEnabled(!s_bSrcBurst);
		//s_pxReleaseRandGraph->SetEnabled(!s_bSrcBurst);
		s_pxGravityGraph->SetEnabled(s_bSrcPointGravity || s_bSrcDirectionalGravity);
		s_pxEditGravityDialog->SetEnabled(s_bSrcDirectionalGravity);// || s_bSrcPointGravity); - KOR 07/06/2005 - point gravity is always at effect position for now
		s_pxTurbulenceGraph->SetEnabled(s_bSrcUseTurbulence);
		s_pxTurbulenceDialog->SetEnabled(s_bSrcUseTurbulence);
		s_pxTimeScaleDialog->SetEnabled(s_bSrcUseTimeScale);
		s_pxInheritVelocityDialog->SetEnabled(s_bSrcInheritVelocity);
		s_pxParticleBounceDialog->SetEnabled(s_bSrcUseParticleDynamics);
		s_pxParticleFrictionDialog->SetEnabled(s_bSrcUseParticleDynamics);
		s_pxSourceBounceDialog->SetEnabled(s_bSrcUseSourceDynamics);
		

		s_bSourceTempChanged = 0;
		SetModified(s_pxCurrentSourceTemp);
	}

	s_pxSpeedGraph->SetHidden(true);
	s_pxAngleGraph->SetHidden(true);
	s_pxSpeedSpreadGraph->SetHidden(true);
	s_pxAngleSpreadGraph->SetHidden(true);
	s_pxReleaseRateGraph->SetHidden(true);
	s_pxReleaseRandGraph->SetHidden(true);

	if(s_pxSourceTabs->GetSelected() == 0 && !s_pxSourceTabs->IsHidden())
	{
		switch(s_iSrcTempAttrib)
		{
			case 0:	s_pxSpeedGraph->SetHidden(false);		break;
			case 1:	s_pxAngleGraph->SetHidden(false);		break;
			case 2:	s_pxSpeedSpreadGraph->SetHidden(false);	break;
			case 3:	s_pxAngleSpreadGraph->SetHidden(false);	break;
			case 4:	s_pxReleaseRateGraph->SetHidden(false);	break;
			case 5:	s_pxReleaseRandGraph->SetHidden(false);	break;
			default: Asura_Assert(0, "Invalid source attribute selection");
		}
	}
	// Choose particle template
	if(s_pxSourceGravityObject)
	{
		if(s_pxSourceGravityObject->m_bGameEntitiesNeedUpdating)
		{
			// Vector has moved, so update gravity
			s_pxCurrentSourceTemp->m_xGravityVector.x = s_pxSourceGravityObject->Orientation.e13;
			s_pxCurrentSourceTemp->m_xGravityVector.y = s_pxSourceGravityObject->Orientation.e23;
			s_pxCurrentSourceTemp->m_xGravityVector.z = s_pxSourceGravityObject->Orientation.e33;
			s_pxCurrentSourceTemp->m_xGravityVector.Normalise();
			// Reset flag
			s_pxSourceGravityObject->m_bGameEntitiesNeedUpdating = false;
			SetModified(s_pxCurrentSourceTemp);
		}

		// Remove object if necessary...
		if(s_pxSourceGravityObject->m_bReadyToDelete)
		{
			delete s_pxSourceGravityObject;
			s_pxSourceGravityObject = NULL;
			Tools_Entity_PFX_Effect::FinishTempEntityEdit();
			//The overlays may become closed as a result. If so then don't try going any further
			if(!s_pxCurrentEffectTemp) return;
		}
	}
	
	if(s_bSrcBurst)
	{
		if(s_bMaxParticlesChanged)
		{
			Tools_Entity_PFX_Effect::ClearAndFlagForRestart_SourceTemplate(s_pxCurrentSourceTemp->GetTemplateHash());
			s_pxCurrentSourceTemp->m_uMaxParticles = s_iMaximumParticles;
			SetModified(s_pxCurrentSourceTemp);
		}
	}
	else
	{
		//Update estimate of maximum particles needed
		CalculateMaxParticles(s_pxCurrentSourceTemp,false);
		s_iMaximumParticles = s_pxCurrentSourceTemp->m_uMaxParticles;

		
	}
	s_bMaxParticlesChanged = false;

}

void Tools_Mode_PFX::UpdateEditEffectTemplate()
{
	if(!s_pxPhotonColourGraph || !s_pxPhotonRadiusGraph) return;

	if(s_uLastEffectEditUpdate == Asura_Timers::GetFrameNumber()) return;
	s_uLastEffectEditUpdate = Asura_Timers::GetFrameNumber();
	
	s_pxPhotonColourGraph->SetHidden(true);
	s_pxPhotonRadiusGraph->SetHidden(true);

	// Choose photon attribute graph
	if(s_pxEffectTabs->GetSelected() == 1 && !s_pxEffectTabs->IsHidden())
	{
		switch(s_iEffectAttrib)
		{
			case 0:	s_pxPhotonColourGraph->SetHidden(false);break;
			case 1:	s_pxPhotonRadiusGraph->SetHidden(false);break;
			default: Asura_Assert(0, "Invalid effect attribute selection");
		}
	}
	// Change number of sources?
	if(s_bNumSourcesChanged)
	{
		bool bConfirmed = true;
		Asura_Assert(s_iNumberOfSources > 0, "Invalid number of sources");
		if(s_iNumberOfSources > (int)s_pxCurrentEffectTemp->m_uNumSources)
		{
			Asura_ResourcePushType(ASURA_RESOURCE_PFX);
			{
				//allocate extra sources
				Asura_PFX_Effect_SourceProperties* pxSources;
				Asura_ResourceAllocateArray(pxSources, s_iNumberOfSources);
				//copy old sources

				for(u_int uSource = 0;uSource<s_pxCurrentEffectTemp->m_uNumSources;uSource++)
				{
					pxSources[uSource] = s_pxCurrentEffectTemp->m_pxSources[uSource];	
				}
				//initialise new ones
				for(u_int uSource = s_pxCurrentEffectTemp->m_uNumSources;uSource<(u_int)s_iNumberOfSources;uSource++)
				{
					pxSources[uSource].SetToDefault();
				}
				Asura_ResourceDeallocateArray(s_pxCurrentEffectTemp->m_pxSources);
				s_pxCurrentEffectTemp->m_pxSources = pxSources;

				//select the last source
				s_iEffectSourceNum = s_iNumberOfSources-1;
			}
			Asura_ResourcePopType();
		}
		else
		{
			if(!s_bSourceRemoved)
			{
				// Decreasing sources - this will lose data.
				bConfirmed = ShowYesNoOverlay("Are you sure you want to decrease the number of sources?");
			}
			s_bSourceRemoved = false;
		}
		if(bConfirmed)
		{
			Tools_Entity_PFX_Effect::ClearAndFlagForRestart_EffectTemplate(s_pxCurrentEffectTemp->GetTemplateHash());
			s_pxCurrentEffectTemp->m_uNumSources = s_iNumberOfSources;
			UpdateSourceChooser();
		}
		else
		{
			s_iNumberOfSources = s_pxCurrentEffectTemp->m_uNumSources;
		}
		s_bNumSourcesChanged = false;
		SetModified(s_pxCurrentEffectTemp);
	}

	const int iSourceNumber = s_iEffectSourceNum;
	Asura_Assert(iSourceNumber>=0 && iSourceNumber<(int)s_pxCurrentEffectTemp->m_uNumSources,"Invalid source index");
	
	if(s_iEffectSourceNum != s_iEffectCurrentSourceNum)
	{
		s_bEffectSourceChanged = true;
	}
	if (s_bEffectSourceChanged)
	{
		s_bEffectSourceChanged = false;
		// Source number changed; get corresponding template and show in template chooser
		s_iEffectCurrentSourceNum = iSourceNumber;
		UpdateSourceProperties();
	}
	else
	{
		// Choose template
		Asura_Hash_ID uSourceHash = 0;
		if(s_iEffectSourceTemp)
		{
			uSourceHash = Asura_GetHashID(s_pszSourceTempList[s_iEffectSourceTemp]);
		}

		if(uSourceHash != s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_uTemplate)
		{
			//if the source type has changed , then force immediate restart on particle
			//effects to avoid crashes.
			Tools_Entity_PFX_Effect::ClearAndFlagForRestart_EffectTemplate(s_pxCurrentEffectTemp->GetTemplateHash());
			s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_uTemplate = uSourceHash;

			UpdateSourceChooser();
			SetModified(s_pxCurrentEffectTemp);
		}
		if(s_bEffectSrcTimeOffsetChanged)
		{
			// Set time offset
			s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_fTimeOffset = s_fEffectSrcTimeOffset;
			s_bEffectSrcTimeOffsetChanged = false;
			SetModified(s_pxCurrentEffectTemp);
		}
		if(s_bEffectUseCodeOrient)
		{
			if(!s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_iOrientationID)
			{
				s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_iOrientationID = 1;
				SetModified(s_pxCurrentEffectTemp);
			}
		}
		else
		{
			if(s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_iOrientationID)
			{
				s_iEffectSrcOrientID = 1;
				s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_iOrientationID = 0;
				SetModified(s_pxCurrentEffectTemp);
			}
		}
		if(s_bEffectSrcOrientIDChanged)
		{
			s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_iOrientationID = s_iEffectSrcOrientID;
			s_bEffectSrcOrientIDChanged = false;
			SetModified(s_pxCurrentEffectTemp);
		}
	}

	if(s_bEffectPhotonPropertiesChanged)
	{
		s_bEffectPhotonPropertiesChanged = false;
		SetModified(s_pxCurrentEffectTemp);
	}
	if(s_bEffectTempChanged)
	{
		// Photon flag - only enable photon attributes if checked
		s_pxCurrentEffectTemp->m_bHasPhoton = s_bEffectHasPhoton;
		s_pxEffectAttribDialog->SetEnabled(s_bEffectHasPhoton);
		s_pxPhotonColourGraph->SetEnabled(s_bEffectHasPhoton);
		s_pxPhotonRadiusGraph->SetEnabled(s_bEffectHasPhoton);
		s_pxCurrentEffectTemp->m_iPhotonPerParticleSource = s_iEffectPhotonPerParticleSource-1;

		if(s_pxCurrentEffectTemp->m_uSoundEvent != s_uEffectSoundEvent)
		{
			//restart effects to force new sound to be used
			Tools_Entity_PFX_Effect::ClearAndFlagForRestart_EffectTemplate(s_pxCurrentEffectTemp->GetTemplateHash());
		}
		s_pxCurrentEffectTemp->m_uSoundEvent = s_uEffectSoundEvent;

		// Other flags
		s_pxCurrentEffectTemp->m_bLooping = s_bEffectLooping;
		s_pxCurrentEffectTemp->m_bOrderedRender = s_bEffectOrderedRender;
		s_pxCurrentEffectTemp->m_bMoveWithEffect = s_bEffectMoveWithEffect;
		s_pxCurrentEffectTemp->m_bAlwaysOnTop = s_bEffectAlwaysOnTop;
		s_pxCurrentEffectTemp->m_bPregenerate = s_bEffectPregenerate;
		s_pxCurrentEffectTemp->m_bParticlePhotons = s_bEffectParticlePhotons;
		s_pxCurrentEffectTemp->m_bForceRenderOnMainThread = s_bForceRenderOnMainThread;
		s_pxCurrentEffectTemp->m_bHudEffect = s_bEffectHudEffect;

		s_bEffectTempChanged = false;
		SetModified(s_pxCurrentEffectTemp);
	}

	if(s_pxThis)
	{
		s_pxThis->ProjectUpdateEditEffectTemplate(s_pxCurrentEffectTemp);
	}

	// Edit offset and orientation
	if(s_pxEffectSrcOffsetObject)
	{
		if(s_pxEffectSrcOffsetObject->m_bGameEntitiesNeedUpdating)
		{
			// Vector has moved, so update offset + orientation
			Asura_Matrix_3x3 xMat = Tools_Entity_PFX_Effect::s_pxCurrentObject->Orientation;
			xMat.Transpose();
			s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_xOrient = xMat * s_pxEffectSrcOffsetObject->Orientation;
			// Set position and
			s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_xOffset = (s_pxEffectSrcOffsetObject->Position - Tools_Entity_PFX_Effect::s_pxCurrentObject->Position) * xMat;
			//Set volume
			s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_xVolume = s_pxEffectSrcOffsetObject->m_xVolume;

			// Reset flag
			s_pxEffectSrcOffsetObject->m_bGameEntitiesNeedUpdating = false;
			// Set flag on entity
			Tools_Entity_PFX_Effect::s_pxCurrentObject->m_bGameEntitiesNeedUpdating = true;

			Tools_Entity_PFX_Effect::ClearAndFlagForRestart_EffectTemplate(s_pxCurrentEffectTemp->GetTemplateHash());

			SetModified(s_pxCurrentEffectTemp);
		}

		//Remove object if neccessary...
		if(s_pxEffectSrcOffsetObject->m_bReadyToDelete)
		{
			delete s_pxEffectSrcOffsetObject;
			s_pxEffectSrcOffsetObject = NULL;
			Tools_Entity_PFX_Effect::FinishTempEntityEdit();
			//The overlays may become closed as a result. If so then don't try going any further
			if(!s_pxCurrentEffectTemp) return;
		}
	}
	// Edit movement vector
	if(s_pxEffectSrcMovementObject)
	{
		if(s_pxEffectSrcMovementObject->m_bGameEntitiesNeedUpdating)
		{
			// Vector has moved, so update offset + orientation
			Asura_Matrix_3x3 xMat = Tools_Entity_PFX_Effect::s_pxCurrentObject->Orientation;
			xMat.Transpose();
			xMat = xMat * s_pxEffectSrcMovementObject->Orientation;			
			s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_xVelocity.x = xMat.e13;
			s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_xVelocity.y = xMat.e23;
			s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_xVelocity.z = xMat.e33;
			s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_xVelocity.Normalise();
			s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_xVelocity *= s_fEffectSrcSpeed;
			// Reset flag
			s_pxEffectSrcMovementObject->m_bGameEntitiesNeedUpdating = false;
			SetModified(s_pxCurrentEffectTemp);
		}

		//Remove object if neccessary...
		if(s_pxEffectSrcMovementObject->m_bReadyToDelete)
		{
			delete s_pxEffectSrcMovementObject;
			s_pxEffectSrcMovementObject = NULL;
			Tools_Entity_PFX_Effect::FinishTempEntityEdit();
			//The overlays may become closed as a result. If so then don't try going any further
			if(!s_pxCurrentEffectTemp) return;
		}
	}
	//update source disabled flag
	s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_bDisabled = s_bEffectSrcDisabled;

	if(s_bEffectSrcSpeedChanged)
	{
		s_pxEffectEditSourceMovement->SetEnabled(s_fEffectSrcSpeed > 0.f);

		if(s_fEffectSrcSpeed > 0 && s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_xVelocity.Magnitude()==0.0f)
		{
			//Speed was zero , and is now non-zero , so we need to set a default direction
			const Asura_Matrix_3x3& xMat = s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_xOrient;
			s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_xVelocity.x = xMat.e13;
			s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_xVelocity.y = xMat.e23;
			s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_xVelocity.z = xMat.e33;
		}

		s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_xVelocity.Normalise();
		s_pxCurrentEffectTemp->m_pxSources[iSourceNumber].m_xVelocity *= s_fEffectSrcSpeed;
		s_bEffectSrcSpeedChanged = 0;
		SetModified(s_pxCurrentEffectTemp);
	}
	if(s_bIsolateSource)
	{
		s_pxCurrentEffectTemp->m_iIsolatedSource = iSourceNumber;
	}
	else
	{
		s_pxCurrentEffectTemp->m_iIsolatedSource = -1;
	}

	//update max particles on all sources used
	for(u_int uSource = 0;uSource<s_pxCurrentEffectTemp->m_uNumSources;uSource++)
	{
		if(s_pxCurrentEffectTemp->m_pxSources[uSource].m_uTemplate)
		{
			Asura_PFX_Source_Template* pxSourceTemp = Asura_PFX_System::GetSourceTempByHash(s_pxCurrentEffectTemp->m_pxSources[uSource].m_uTemplate);
			if(pxSourceTemp) CalculateMaxParticles(pxSourceTemp,false);
		}
	}

}

/// Save templates ////////////////////////////////////////////////////////////////////////////////

void Tools_Mode_PFX::QuickSave(void)
{

	Tools_SaveAsura::SaveEntities();
	SaveAllTemplates();

	Tools_SoundEvent_System::SaveModified();

	//Some of the other files may have been modified by creation/updates of template
	//instance objects. So save them as well if required. RAR 19/08/04
	if(Tools_SaveAsura::IsMainFileModified()) Tools_SaveAsura::QuickSaveFile();
	if(Tools_SaveAsura::IsSoundsModified()) Tools_SaveAsura::SaveSounds();
	if(Tools_SaveAsura::IsGlobalIlluminationModified()) Tools_SaveAsura::SaveGlobalIllumination();
	if(Tools_SaveAsura::IsLightingModified()) Tools_SaveAsura::SaveLights();
	if(Tools_SaveAsura::IsEnvironmentalLightTransportModified()) Tools_SaveAsura::SaveEnvironmentalLightTransport();

}

void Tools_Mode_PFX::AutoSave()
{
	u_int uSaved = SaveAllTemplates(TOOLS_MODE_PFX_SAVE_AUTO);
	if(uSaved)
	{
		Tools_Timed_Message_System::AddMessage(0x00ff00,"Autosaved %d PFX template(s)", uSaved);
	}
	s_fTimeSinceAutosave = 0.f;
}

u_int Tools_Mode_PFX::SaveAllTemplates(TOOLS_MODE_PFX_SAVE_TYPE eSaveType /* = TOOLS_MODE_PFX_SAVE_NORMAL */, bool bForceSaveAll /* = false */)
{
	Asura_PFX_EffectTemplateResourceSet_ResourceIterator xIterator(Asura_PFX_System::GetEffectTempResourceSet());

	Asura_Assert(eSaveType == TOOLS_MODE_PFX_SAVE_NORMAL || eSaveType == TOOLS_MODE_PFX_SAVE_AUTO, "Invalid save type");
	u_int uNumSaved = 0;
	while(!xIterator.Done())
	{
		const Asura_Hash_ID	uHash	= xIterator.GetCurrentHash();

		if(!Asura_PFX_System::IsEffectTempLoadedByHash(uHash))
		{
			if(bForceSaveAll)
			{
				Asura_PFX_System::LoadEffectTemplateByHash(uHash);
			}
			else
			{
				continue;
			}
		}

		if(bForceSaveAll)
		{
			Asura_PFX_Effect_Template* pxTemplate = xIterator.GetCurrent();
			if(pxTemplate) pxTemplate->m_bModified = true;
		}
		if(SaveTemplateByHash(uHash, eSaveType))
		{
			uNumSaved++;
		}

		xIterator.Next();
	}
	return uNumSaved;
}

void Tools_Mode_PFX::ConvertToNewFormat()
{
	// Make sure there's no pointers hanging around.
	Asura_PFX_System::ClearAllEffects();
	Tools_Entity_PFX_Effect::RestartAllEffects();

	Tools_Entity_PFX_Effect::CloseOverlays();

	//enable reloading of templates while doing the upgrade.RAR 19/01/2007
	Asura_PFX_System::SetAllowTemplateReload(true);

	// Load up all the old templates
	LoadFromOldFormat();

	// Save them all out. This might change some template names'n'stuff...
	SaveAllTemplates(TOOLS_MODE_PFX_SAVE_NORMAL, true);

	// So we should clear them all out
	Asura_PFX_System::ClearAllTemplates();

	// And load them back up using the new files.
	Asura_PFX_System::LoadAllTemplateFiles();

	Asura_PFX_System::SetAllowTemplateReload(false);

}

void Tools_Mode_PFX::LoadFromOldFormat()
{
	for(int iType = 0;iType<3;iType++)
	{
		Asura_PFX_System::LoadTemplate(iType);
	}
}

bool Tools_Mode_PFX::CanSaveParticleTempByHash(Asura_Hash_ID uHash,bool bDisplayMessage,bool bAsError /* = false */)
{
	const Asura_String* pxTempName = Asura_PFX_System::GetParticleTempNameByHash(uHash);
	if(!pxTempName) return false;
  	Asura_String xFilename;
  	GetParticleTempFileName(pxTempName, &xFilename, TOOLS_MODE_PFX_SAVE_NORMAL);

	if(bDisplayMessage && !bAsError)
	{
		return Tools_SaveAsura::WarnIfReadOnly(xFilename.GetString());
	}
	return Tools_SaveAsura::CanSaveFile(xFilename.GetString(),bDisplayMessage);
}

bool Tools_Mode_PFX::CanSaveSourceTempByHash(Asura_Hash_ID uHash,bool bDisplayMessage,bool bAsError /* = false */)
{
	const Asura_String* pxTempName = Asura_PFX_System::GetSourceTempNameByHash(uHash);
	if(!pxTempName) return false;
  	Asura_String xFilename;
  	GetSourceTempFileName(pxTempName, &xFilename, TOOLS_MODE_PFX_SAVE_NORMAL);

	if(bDisplayMessage && !bAsError)
	{
		return Tools_SaveAsura::WarnIfReadOnly(xFilename.GetString());
	}
	return Tools_SaveAsura::CanSaveFile(xFilename.GetString(),bDisplayMessage);
}

bool Tools_Mode_PFX::CanSaveEffectTempByHash(Asura_Hash_ID uHash,bool bDisplayMessage,bool bAsError /* = false */)
{
	const Asura_String* pxTempName = Asura_PFX_System::GetEffectTempNameByHash(uHash);
	if(!pxTempName) return false;
  	Asura_String xFilename;
  	GetEffectTempFileName(pxTempName, &xFilename, TOOLS_MODE_PFX_SAVE_NORMAL);

	if(bDisplayMessage && !bAsError)
	{
		return Tools_SaveAsura::WarnIfReadOnly(xFilename.GetString());
	}
	return Tools_SaveAsura::CanSaveFile(xFilename.GetString(),bDisplayMessage);
}

bool Tools_Mode_PFX::CanSaveTemplateByHash(Asura_Hash_ID uEffectHash,bool bDisplayMessage,bool bAsError /* = false */)
{
	const Asura_String* pxTempName = Asura_PFX_System::GetEffectTempNameByHash(uEffectHash);
	if(!pxTempName) return false;
	Asura_String xFilename;
	GetTemplateFileName(pxTempName, &xFilename, TOOLS_MODE_PFX_SAVE_NORMAL);

	if(bDisplayMessage && !bAsError)
	{
		return Tools_SaveAsura::WarnIfReadOnly(xFilename.GetString());
	}
	return Tools_SaveAsura::CanSaveFile(xFilename.GetString(),bDisplayMessage);
}

void Tools_Mode_PFX::SaveParticleTempByHash(Asura_Hash_ID uHash, TOOLS_MODE_PFX_SAVE_TYPE eSaveType /* = TOOLS_MODE_PFX_SAVE_NORMAL */)
{
	Asura_PFX_Particle_Template* pxTemplate = Asura_PFX_System::GetParticleTempByHash(uHash);
	Asura_Assert(pxTemplate, "No particle template to save");
	if(pxTemplate)
	{
		switch(eSaveType)
		{
			case TOOLS_MODE_PFX_SAVE_NORMAL:	
				// Only bother saving if modified
				if(!pxTemplate->m_bModified) return;
				// Switch off modified flag
				pxTemplate->m_bModified = false;
				break;
		}
		if(eSaveType != TOOLS_MODE_PFX_SAVE_NORMAL || CanSaveParticleTempByHash(uHash,true,true))
		{
			Asura_SaveBlock* pxSaveBlock = new Asura_SaveBlock();
			Asura_Assert(pxSaveBlock, "Unable to create saveblock");
			if(pxSaveBlock)
			{
				const Asura_String* pxTempName = Asura_PFX_System::GetParticleTempNameByHash(uHash);
				Asura_String xFilename;
				GetParticleTempFileName(pxTempName, &xFilename, eSaveType);
				Tools_FileSystem::EnsureDirectoryExists(pxTempName->GetString());
				if  (eSaveType == TOOLS_MODE_PFX_SAVE_EXPORT && Tools_FileSystem::FileExists(xFilename.GetString()))
				{
					delete pxSaveBlock;
					return;
				}
				Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
				Asura_Chunk_PFX_ParticleTemp::Create(pxTemplate, pxTempName, xStream, eSaveType == TOOLS_MODE_PFX_SAVE_EXPORT);
				//Terminate and save chunk
				pxSaveBlock->AddAsuraTerminator();				
				bool bOK = pxSaveBlock->SaveFile(xFilename.GetString(), false );
				if(bOK)
				{
					if(eSaveType == TOOLS_MODE_PFX_SAVE_NORMAL) Tools_Timed_Message_System::AddMessage(0x00ff00,"Success: saved \"%s\" particle template.", pxTempName->GetString());
					Tools_LoadedFileList::UpdateModifiedTime(xFilename.GetString());

				}
				else
				{
					Tools_Timed_Message_System::AddMessage(0xff0000,"Error: failed to save \"%s\" particle template. Is the file writable?", pxTempName->GetString());
				}
				
				//Asura_Assert(bOK, "Failed to save particle template");
				delete pxSaveBlock;
				
			}
		}
	}
}



void Tools_Mode_PFX::SaveSourceTempByHash(Asura_Hash_ID uHash, TOOLS_MODE_PFX_SAVE_TYPE eSaveType /* = TOOLS_MODE_PFX_SAVE_NORMAL */)
{
	Asura_PFX_Source_Template* pxTemplate = Asura_PFX_System::GetSourceTempByHash(uHash);
	if(pxTemplate)
	{
		//update max particles before saving
		CalculateMaxParticles(pxTemplate,true);

		#if 1
		switch(eSaveType)
		{
			case TOOLS_MODE_PFX_SAVE_NORMAL:	
				// Only bother saving if modified
				if(!pxTemplate->m_bModified) return;
				// Switch off modified flag
				pxTemplate->m_bModified = false;
				break;
		}
		#endif
		if(eSaveType != TOOLS_MODE_PFX_SAVE_NORMAL || CanSaveSourceTempByHash(uHash,true,true))
		{
			Asura_SaveBlock* pxSaveBlock = new Asura_SaveBlock();
			Asura_Assert(pxSaveBlock, "Unable to create saveblock");
			if(pxSaveBlock)
			{
				const Asura_String* pxTempName = Asura_PFX_System::GetSourceTempNameByHash(uHash);
				Asura_String xFilename;
				GetSourceTempFileName(pxTempName, &xFilename, eSaveType);
				Tools_FileSystem::EnsureDirectoryExists(pxTempName->GetString());
				if  (eSaveType == TOOLS_MODE_PFX_SAVE_EXPORT && Tools_FileSystem::FileExists(xFilename.GetString()))
				{
					delete pxSaveBlock;
					return;
				}
				Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
				Asura_Chunk_PFX_SourceTemp::Create(pxTemplate, pxTempName, xStream, eSaveType == TOOLS_MODE_PFX_SAVE_EXPORT);
				//Terminate and save chunk
				pxSaveBlock->AddAsuraTerminator();
				bool bOK = pxSaveBlock->SaveFile(xFilename.GetString(), false );
				if(bOK)
				{
					if(eSaveType == TOOLS_MODE_PFX_SAVE_NORMAL) Tools_Timed_Message_System::AddMessage(0x00ff00,"Success: saved \"%s\" source template.", pxTempName->GetString());
					Tools_LoadedFileList::UpdateModifiedTime(xFilename.GetString());
				}   
				else
				{
					Tools_Timed_Message_System::AddMessage(0xff0000,"Error: failed to save \"%s\" source template. Is the file writable?", pxTempName->GetString());
				}

				if(Tools_Level_Export::IsLevelExport() && pxTemplate->m_uMaxParticles > TOOLS_PFX_MAX_PARTICLE_WARNING_LEVEL)
				{
					Tools_Level_Export::AddMessage(Tools_Level_Export::MT_WARNING,"PFX Source %s uses %d particles",pxTemplate->m_xName.GetString(),pxTemplate->m_uMaxParticles);
				}

				//test
				//Asura_SimpleLogfileOutput(("MaxParticles.txt","%-6d %c   %s\r\n",pxTemplate->m_uMaxParticles,pxTemplate->m_bBurst ? 'Y' : 'N',pxTempName->GetString()));

				
				//Asura_Assert(bOK, "Failed to save source template");
				delete pxSaveBlock;
			}
		}
	}
}

void Tools_Mode_PFX::SaveEffectTempByHash(Asura_Hash_ID uHash, TOOLS_MODE_PFX_SAVE_TYPE eSaveType /* = TOOLS_MODE_PFX_SAVE_NORMAL */)
{
	Asura_PFX_Effect_Template* pxTemplate = Asura_PFX_System::GetEffectTempByHash(uHash);
	if(pxTemplate)
	{
		switch(eSaveType)
		{
			case TOOLS_MODE_PFX_SAVE_NORMAL:	
				// Only bother saving if modified
				if(!pxTemplate->m_bModified) return;
				// Switch off modified flag
				pxTemplate->m_bModified = false;
				break;
		}
		if(eSaveType != TOOLS_MODE_PFX_SAVE_NORMAL || CanSaveEffectTempByHash(uHash,true,true))
		{
			Asura_SaveBlock* pxSaveBlock = new Asura_SaveBlock();
			Asura_Assert(pxSaveBlock, "Unable to create saveblock");
			if(pxSaveBlock)
			{
				const Asura_String* pxTempName = Asura_PFX_System::GetEffectTempNameByHash(uHash);
				Asura_String xFilename;
				GetEffectTempFileName(pxTempName, &xFilename, eSaveType);
				Tools_FileSystem::EnsureDirectoryExists(pxTempName->GetString());
				if  (eSaveType == TOOLS_MODE_PFX_SAVE_EXPORT && Tools_FileSystem::FileExists(xFilename.GetString()))
				{
					delete pxSaveBlock;
					return;
				}
				const Asura_PFX_Effect_Template* pxTemplateToSave = pxTemplate;
				if(eSaveType == TOOLS_MODE_PFX_SAVE_EXPORT)
				{
					//For level exports we may need to substitute in a different template according to platform. RAR 01/04/2005
					pxTemplateToSave = pxTemplate->GetTemplateForPlatform(Tools_Level_Export::s_eExportType);


				}

				Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
				Asura_Chunk_PFX_EffectTemp::Create(pxTemplateToSave, pxTempName, xStream, eSaveType == TOOLS_MODE_PFX_SAVE_EXPORT);
				//Terminate and save chunk
				pxSaveBlock->AddAsuraTerminator();
				
				bool bOK = pxSaveBlock->SaveFile(xFilename.GetString(), false );
				if(bOK)
				{
					if(eSaveType == TOOLS_MODE_PFX_SAVE_NORMAL) Tools_Timed_Message_System::AddMessage(0x00ff00,"Success: saved \"%s\" effect template.", pxTempName->GetString());
					Tools_LoadedFileList::UpdateModifiedTime(xFilename.GetString());
				}
				else
				{
					Tools_Timed_Message_System::AddMessage(0xff0000,"Error: failed to save \"%s\" effect template. Is the file writable?", pxTempName->GetString());
				}
				
				//Asura_Assert(bOK, "Failed to save effect template");
				delete pxSaveBlock;
			}
		}
	}
}

static void CheckForFixingUpSourceName( const Asura_String* pxTempName, Asura_Hash_ID &uSourceHash)
{
	Asura_PFX_Source_Template* pxSourceTemp = Asura_PFX_System::GetSourceTempByHash(uSourceHash, true);
	if(!pxSourceTemp) { return; }

	const Asura_String* pxSourceName = Asura_PFX_System::GetSourceTempNameByHash(uSourceHash);
	if(!pxSourceName) { return; }

	Asura_String xNewSourceName;
	xNewSourceName.MakeBlank();
	// Check whether the source name is in our new format (Effect:Source)
	if(!pxSourceName->Contains(":"))
	{
		// Firstly, see if we have the format:
		// Effect: "effect_E"
		// Source: "effect_source_S"
		// In which case, we want something like:
		// effect:source_S
		if(pxTempName->EndsWith("_E"))
		{
			Asura_String xTruncatedEffectName(*pxTempName);
			// Knock off the E.
			xTruncatedEffectName.Truncate(pxTempName->GetLength() - 1);
			if(pxSourceName->StartsWith(xTruncatedEffectName.GetString()))
			{
				xNewSourceName = *pxTempName;
				xNewSourceName += ":";
				const char* szJustSourceName = &pxSourceName->GetString()[xTruncatedEffectName.GetLength()];
				xNewSourceName += szJustSourceName;
			}
		}

		// Okay, that failed. Default to:
		if(xNewSourceName.IsBlank())
		{
			// Append the effect name to the source name to ensure uniqueness.
			xNewSourceName = *pxTempName;
			xNewSourceName += ":";
			xNewSourceName += *pxSourceName;
		}

		pxSourceName = &xNewSourceName;
		// Patch up the effect template.
		uSourceHash = Asura_GetHashID(pxSourceName->GetString());
	}

}

bool Tools_Mode_PFX::SaveTemplateByHash(Asura_Hash_ID uEffectHash, TOOLS_MODE_PFX_SAVE_TYPE eSaveType /* = TOOLS_MODE_PFX_SAVE_NORMAL */)
{
	Asura_PFX_Effect_Template* pxTemplate = Asura_PFX_System::GetEffectTempByHash(uEffectHash);
	bool bSaved = false;
	if(pxTemplate)
	{
		switch(eSaveType)
		{
			case TOOLS_MODE_PFX_SAVE_NORMAL:	
			{
				// Only bother saving if modified
				if(!pxTemplate->IsModified()) { return false; }
				break;
			}
			case TOOLS_MODE_PFX_SAVE_AUTO:	
			{
				// Save if modified and an autosave has yet to be done since the last modification
				if(!pxTemplate->IsModified() || pxTemplate->m_bDoneAutosave) { return false; }
				break;
			}
		}
		if(eSaveType != TOOLS_MODE_PFX_SAVE_NORMAL || CanSaveTemplateByHash(uEffectHash,true,true))
		{
			Asura_SaveBlock* pxSaveBlock = new Asura_SaveBlock();
			Asura_Assert(pxSaveBlock, "Unable to create saveblock");
			if(pxSaveBlock)
			{
				if(eSaveType == TOOLS_MODE_PFX_SAVE_NORMAL)
				{
					BackupTemplateFileIfRequired( pxTemplate);
				}
				else if(eSaveType == TOOLS_MODE_PFX_SAVE_AUTO)
				{
					RenameTemplateAutosaveFiles(pxTemplate);
				}
				
				const Asura_String* pxTempName = Asura_PFX_System::GetEffectTempNameByHash(uEffectHash);
				Asura_String xFilename;
				GetTemplateFileName(pxTempName, &xFilename, eSaveType);
				Tools_FileSystem::EnsureDirectoryExists(xFilename.GetString());
				if  (eSaveType == TOOLS_MODE_PFX_SAVE_EXPORT && Tools_FileSystem::FileExists(xFilename.GetString()))
				{
					delete pxSaveBlock;
					return false;
				}

				const Asura_PFX_Effect_Template* pxTemplateToSave = pxTemplate;
				if(eSaveType == TOOLS_MODE_PFX_SAVE_EXPORT)
				{
					//For level exports we may need to substitute in a different template according to platform. RAR 01/04/2005
					pxTemplateToSave = pxTemplate->GetTemplateForPlatform(Tools_Level_Export::s_eExportType);
				}

				Asura_Chunk_SaveBlock_Stream xStream(pxSaveBlock);
				
				// Add all associated source and particle temps
				if(pxTemplateToSave)
				{
					Asura_TemplateHashTree<u_int> xSourceTemplatesToSave;
					Asura_TemplateHashTree<u_int> xParticleTemplatesToSave;

					for(u_int uSource = 0; uSource < pxTemplateToSave->m_uNumSources; ++uSource)
					{
						Asura_Hash_ID uSourceHash = pxTemplateToSave->m_pxSources[uSource].m_uTemplate;

						CheckForFixingUpSourceName(pxTempName, pxTemplateToSave->m_pxSources[uSource].m_uTemplate);

						// Mark the un-overridden one as used
						*xSourceTemplatesToSave.AddOrFindNode(uSourceHash) += 1;

						// and each overridden entry
						for(Asura_PFX_SourceTemplateResourceSet_ConstMapIterator xMapIter(Asura_PFX_System::GetSourceTempResourceSet()); !xMapIter.Done(); xMapIter.Next())
						{
							*xSourceTemplatesToSave.AddOrFindNode(xMapIter.GetCurrent()->GetOverride(uSourceHash)) += 1;
						}


						for (Asura_TemplateHashTree_ConstIterator<u_int> xSource(xSourceTemplatesToSave); !xSource.Done(); xSource.Next())
						{
							Asura_PFX_Source_Template* pxSourceTemp = Asura_PFX_System::GetSourceTempByHash(xSource.GetCurrentHash(), true);

							if (pxSourceTemp)
							{
								Asura_Hash_ID uParticleHash = pxSourceTemp->m_uParticleTemplateHash;

								*xParticleTemplatesToSave.AddOrFindNode(uParticleHash) += 1;

								for(Asura_PFX_ParticleTemplateResourceSet_ConstMapIterator xMapIter(Asura_PFX_System::GetParticleTempResourceSet()); !xMapIter.Done(); xMapIter.Next())
								{
									*xParticleTemplatesToSave.AddOrFindNode(xMapIter.GetCurrent()->GetOverride(uParticleHash)) += 1;
								}
							}
						}
					}
					for (Asura_TemplateHashTree_ConstIterator<u_int> xParticle(xParticleTemplatesToSave); !xParticle.Done(); xParticle.Next())
					{
						const Asura_PFX_Particle_Template* pxParticleTemp = Asura_PFX_System::GetParticleTempByHash(xParticle.GetCurrentHash(), true);
						if (pxParticleTemp)
						{
							Asura_Chunk_PFX_ParticleTemp::Create(pxParticleTemp, Asura_PFX_System::GetParticleTempNameByHash(xParticle.GetCurrentHash()), xStream, eSaveType == TOOLS_MODE_PFX_SAVE_EXPORT);
						}

					}
					for (Asura_TemplateHashTree_ConstIterator<u_int> xSource(xSourceTemplatesToSave); !xSource.Done(); xSource.Next())
					{
						Asura_PFX_Source_Template* pxSourceTemp = Asura_PFX_System::GetSourceTempByHash(xSource.GetCurrentHash(), true);
						if (pxSourceTemp)
						{
							Asura_Chunk_PFX_SourceTemp::Create(pxSourceTemp,  Asura_PFX_System::GetSourceTempNameByHash(xSource.GetCurrentHash()), xStream, eSaveType == TOOLS_MODE_PFX_SAVE_EXPORT);
						}

					}
				}

				// Save the effect template.
				// Must be done after source templates to ensure source templates are loaded first.
				Asura_Chunk_PFX_EffectTemp::Create(pxTemplateToSave, pxTempName, xStream, eSaveType == TOOLS_MODE_PFX_SAVE_EXPORT);

				//Terminate and save chunk
				pxSaveBlock->AddAsuraTerminator();

				bool bOK = pxSaveBlock->SaveFile(xFilename.GetString(), false );
				if(bOK)
				{
					if(eSaveType == TOOLS_MODE_PFX_SAVE_NORMAL) Tools_Timed_Message_System::AddMessage(0x00ff00,"Success: saved \"%s\" template.", pxTempName->GetString());
					Tools_LoadedFileList::UpdateModifiedTime(xFilename.GetString());

					if(eSaveType == TOOLS_MODE_PFX_SAVE_NORMAL)
					{
						// Switch off modified flag
						pxTemplate->SetModified(false);
						if(!pxTemplate->m_xOriginalNameForDeletion.IsBlank())
						{
							//Effect has recently been renamed , so get rid of the old file , unless an effect
							//has since been created using the old name
							//RAR 21/05/2008
							if(!Asura_PFX_System::DoesEffectHashExist(pxTemplate->m_xOriginalNameForDeletion.GetHash()))
							{
								Asura_String xOldFilename;
								GetTemplateFileName(&pxTemplate->m_xOriginalNameForDeletion, &xOldFilename);
								if(!DeleteFile(xOldFilename.GetString()))
								{
									Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to delete %s", xOldFilename.GetString());
								}
							}
							pxTemplate->m_xOriginalNameForDeletion.MakeNull();
						}
					}
					else if(eSaveType == TOOLS_MODE_PFX_SAVE_AUTO)
					{
						pxTemplate->m_bDoneAutosave = true;

					}
					bSaved = true;
				}
				else
				{
					Tools_Timed_Message_System::AddMessage(0xff0000,"Error: failed to save \"%s\" effect template. Is the file writable?", pxTempName->GetString());
				}

				//Asura_Assert(bOK, "Failed to save effect template");
				delete pxSaveBlock;
			}
		}
	}
	return bSaved;
}

void Tools_Mode_PFX::BackupTemplateFileIfRequired(Asura_PFX_Effect_Template* pxTemplate)
{
	if(Tools_Level_Export::IsExport())
	{
		Asura_Assert(0, "Shouldn't be calling BackupTemplateIfRequired() during an export");
		return;
	}
	
	//Done a backup yet this session?
	if(pxTemplate->m_bDoneBackup)
	{
		//If so was it long enough ago for another one to be needed?
		if((GetTickCount() - pxTemplate->m_uLastBackupTime) < (u_int)(60*1000*Tools_SaveAsura::GetBackupInterval())) return;
	}

	//get the non-backup filename
	char acMainFilename[MAX_PATH];
	Asura_PrintfToBfr(acMainFilename, "PFXTemps\\%s.pfx", sizeof(acMainFilename), true, pxTemplate->m_xName.GetString());
	
	char acFromName[MAX_PATH];
	char acToName[MAX_PATH];

	//rename the previous backup files
	for(int i=Tools_SaveAsura::GetNumBackupsToMake();i;i--)
	{
		Asura_PrintfToBfr(acToName,"PFXTemps\\Backup\\%s_B%d.pfx",sizeof(acToName), true, pxTemplate->m_xName.GetString(), i);
		if(i-1)Asura_PrintfToBfr(acFromName,"PFXTemps\\Backup\\%s_B%d.pfx",sizeof(acFromName), true, pxTemplate->m_xName.GetString(), i-1);
		

		DWORD uAttr = GetFileAttributes(acToName);
		if(uAttr != 0xffffffff)
		{
			uAttr &=~ FILE_ATTRIBUTE_READONLY;
			SetFileAttributes(acToName,uAttr);
		}
		DeleteFile(acToName);
		if(i-1)
		{
			//renaming files in the backup directory
			MoveFile(acFromName,acToName);
		}
		else
		{
			//copy the original non-backup file into the first slot in the backup directory
			Tools_FileSystem::EnsureDirectoryExists(acToName);
			CopyFile(acMainFilename, acToName,FALSE);
		}
	}
	//reset the time of the last backup
	pxTemplate->m_bDoneBackup = true;
	pxTemplate->m_uLastBackupTime = GetTickCount();
	
}

void Tools_Mode_PFX::RenameTemplateAutosaveFiles(Asura_PFX_Effect_Template* pxTemplate)
{
	//minimum differences in seconds of the  modified times for one file to overwrite the next
	u_int auMinTimeDiff[6] ={0,0,0,5*60,10*60,20*60};
	
	char acFromName[MAX_PATH];
	char acToName[MAX_PATH];

	//rename the previous backup files
	for(int i=5;i;i--)
	{
		Asura_PrintfToBfr(acToName,"PFXTemps\\Autosave\\%s_A%d.pfx",sizeof(acToName), true, pxTemplate->m_xName.GetString(), i+1);
		Asura_PrintfToBfr(acFromName,"PFXTemps\\Autosave\\%s_A%d.pfx",sizeof(acFromName), true, pxTemplate->m_xName.GetString(), i);

		//The lower numbered autosaves only get overwritten if enough time has passed since the previous.
		//So check that now
		u_int uMinTimeDiff = auMinTimeDiff[i];
		if(uMinTimeDiff)
		{
			FILETIME xToTime;
			FILETIME xFromTime;

			if(!Tools_FileSystem::GetModifiedTime(acFromName, &xFromTime)) continue;
			if(Tools_FileSystem::GetModifiedTime(acToName, &xToTime))
			{
				LONGLONG xTimeDiff = *(LONGLONG*)&xFromTime - *(LONGLONG*)&xToTime;

				if(xTimeDiff<(LONGLONG)uMinTimeDiff*10000000)
				{
					continue;
				}
			}
		}
		
		//delete the destination file if it exists
		DWORD uAttr = GetFileAttributes(acToName);
		if(uAttr != 0xffffffff)
		{
			uAttr &=~ FILE_ATTRIBUTE_READONLY;
			SetFileAttributes(acToName,uAttr);
		}
		DeleteFile(acToName);
		//and rename the lower numbered autosave file
		if(uMinTimeDiff)
		{
			CopyFile(acFromName,acToName, FALSE);
		}
		else
		{
			MoveFile(acFromName,acToName);
		}
	}
	
}

// void Tools_Mode_PFX::SaveCurrentParticleTemplate()
// {
// 	// Save it
// 	SaveParticleTempByHash(s_uEditingParticleTempHash);
// 	// Reset the 'undo' point
// 	*s_pxSavedParticleTemp = *s_pxCurrentParticleTemp;
// }
 
// void Tools_Mode_PFX::SaveCurrentSourceTemplate()
// {
// 	// Save it
// 	SaveSourceTempByHash(s_uEditingSourceTempHash);
// 	// Reset the 'undo' point
// 	*s_pxSavedSourceTemp = *s_pxCurrentSourceTemp;
// }

// void Tools_Mode_PFX::SaveCurrentSourceTemplateWithChildren()
// {
// 	// Save it
// 	SaveSourceTempByHash(s_uEditingSourceTempHash,TOOLS_MODE_PFX_SAVE_NORMAL);
// 	// Reset the 'undo' point
// 	*s_pxSavedSourceTemp = *s_pxCurrentSourceTemp;
// 
// 	//save particle
// 	const Asura_PFX_Source_Template* pxSourceTemp = Asura_PFX_System::GetSourceTempByHash(s_uEditingSourceTempHash);
// 	if(!pxSourceTemp) return;
// 
// 	//save particle
// 	Asura_Hash_ID uPartHash = pxSourceTemp->m_uParticleTemplateHash;
// 	if(!uPartHash) return;
// 
// 	SaveParticleTempByHash(uPartHash);
// 	if(s_uEditingParticleTempHash == uPartHash && s_pxSavedParticleTemp && s_pxCurrentParticleTemp)
// 	{
// 		*s_pxSavedParticleTemp = *s_pxCurrentParticleTemp;
// 	}
// }


// void Tools_Mode_PFX::SaveCurrentEffectTemplate()
// {
// 	// Save it
// 	SaveEffectTempByHash(s_uEditingEffectTempHash);
// 	// Reset the 'undo' point
// 	*s_pxSavedEffectTemp = *s_pxCurrentEffectTemp;
// }

// void Tools_Mode_PFX::SaveCurrentEffectTemplateWithChildren()
// {
// 	// Always save when save + children is selected
// 	SetModified(s_pxCurrentEffectTemp);
// 	// Save it
// 	SaveEffectTempByHash(s_uEditingEffectTempHash);
// 	// Reset the 'undo' point
// 	*s_pxSavedEffectTemp = *s_pxCurrentEffectTemp;
// 
// 	//save children
// 	const Asura_PFX_Effect_Template* pxTemplate = Asura_PFX_System::GetEffectTempByHash(s_uEditingEffectTempHash);
// 	if(pxTemplate)
// 	{
// 		for(u_int uSource = 0; uSource < pxTemplate->m_uNumSources; ++uSource)
// 		{
// 			//save source
// 			Asura_Hash_ID uSourceHash = pxTemplate->m_pxSources[uSource].m_uTemplate;
// 			if(!uSourceHash) continue;
// 
// 			Asura_PFX_Source_Template* pxSourceTemp = Asura_PFX_System::GetSourceTempByHash(uSourceHash);
// 			if(!pxSourceTemp) continue;
// 
// 			// Always save when save + children is selected
// 			pxSourceTemp->m_bModified = true;
// 
// 			SaveSourceTempByHash(uSourceHash);
// 			if(s_uEditingSourceTempHash == uSourceHash && s_pxSavedSourceTemp && s_pxCurrentSourceTemp )
// 			{
// 				*s_pxSavedSourceTemp = *s_pxCurrentSourceTemp;
// 			}
// 
// 			//save particle
// 			Asura_Hash_ID uPartHash = pxSourceTemp->m_uParticleTemplateHash;
// 			if(!uPartHash) continue;
// 
// 			// Always save when save + children is selected
// 			Asura_PFX_Particle_Template* pxParticleTemp = Asura_PFX_System::GetParticleTempByHash(uPartHash);
// 			if(!pxParticleTemp) continue;
// 			pxParticleTemp->m_bModified = true;
// 
// 			SaveParticleTempByHash(uPartHash);
// 			if(s_uEditingParticleTempHash == uPartHash && s_pxSavedParticleTemp && s_pxCurrentParticleTemp)
// 			{
// 				*s_pxSavedParticleTemp = *s_pxCurrentParticleTemp;
// 			}
// 		}
// 	}
// }

void Tools_Mode_PFX::SaveCurrentTemplate()
{
	// Always save when Save is clicked directly
	s_pxCurrentEffectTemp->m_bModified = true;
	// Save it
	SaveTemplateByHash(s_uEditingEffectTempHash);
}

/// Template creation //////////////////////////////////////////////////////////////////////////////


void Tools_Mode_PFX::CreateNewSourceTemplate()
{
	Asura_String xBaseName("New source");
	Asura_String xSourceName;
	GetNewSourceTemplateName(xBaseName);
	const Asura_String* pxEffectName = Asura_PFX_System::GetEffectTempNameByHash(s_uEditingEffectTempHash);
	Asura_String xFullName;
	Asura_Assert(pxEffectName, "Should have an effect name");
	if(pxEffectName)
	{
		xFullName = *pxEffectName;
		xFullName += ":";
		xFullName += xBaseName;
	}
	else
	{
		xFullName = xBaseName;
	}

	GetUniqueSourceTempName(xFullName, xSourceName);
	// Add a new template
	Asura_PFX_Source_Template* pxSourceTemp = new Asura_PFX_Source_Template();
	Asura_PFX_System::AddSourceTemplate(xSourceName.GetString(), pxSourceTemp);

	// Use the same name for the particle template
	Asura_PFX_Particle_Template* pxParticleTemp = new Asura_PFX_Particle_Template();
	Asura_PFX_System::AddParticleTemplate(xSourceName.GetString(), pxParticleTemp);

	const Asura_Hash_ID uNameHash = Asura_GetHashID(xSourceName.GetString());
	pxSourceTemp->m_uParticleTemplateHash = uNameHash;
	s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum].m_uTemplate = uNameHash;

	SaveTemplateByHash(s_uEditingEffectTempHash);

	RefreshSourceTemplateList();
	UpdateSourceChooser();
}


Asura_PFX_Effect_Template* Tools_Mode_PFX::CreateNewEffectTemplate()
{
	Asura_String xBaseName("New effect template");
	Asura_String xEffectName;

	//Ask for an effect name
	if(!GetNewEffectTemplateName(xBaseName)) return NULL;
	GetUniqueEffectTempName(xBaseName, xEffectName);

	// Ask for a source name to append to the effect name
	Asura_String xBaseSourceName("source");
	if(!GetNewSourceTemplateName(xBaseSourceName)) return NULL;

	// Add a new template
	Asura_PFX_Effect_Template* pxEffectTemp = new Asura_PFX_Effect_Template();
	Asura_PFX_System::AddEffectTemplate(xEffectName.GetString(), pxEffectTemp);


	// Construct source template name as "effect:source"
	xBaseName += Asura_String(":");
	xBaseName += xBaseSourceName;

	Asura_String xSourceName;
	GetUniqueSourceTempName(xBaseName, xSourceName);
	Asura_PFX_Source_Template* pxSourceTemp = new Asura_PFX_Source_Template();
	Asura_PFX_System::AddSourceTemplate(xSourceName.GetString(), pxSourceTemp);

	// Use same name for source and particle templates
	Asura_PFX_Particle_Template* pxParticleTemp = new Asura_PFX_Particle_Template();
	Asura_PFX_System::AddParticleTemplate(xSourceName.GetString(), pxParticleTemp);

	pxSourceTemp->m_uParticleTemplateHash = Asura_GetHashID(xSourceName.GetString());
	pxEffectTemp->m_pxSources[0].m_uTemplate =  Asura_GetHashID(xSourceName.GetString());

	// Select the new template
	s_uChooseTemplateHash = xEffectName.GetHash();
	SaveTemplateByHash(s_uChooseTemplateHash);

	// Update list
	Tools_Entity_PFX_Effect::UpdateEffectList();

	return pxEffectTemp;
}

/// Template copying //////////////////////////////////////////////////////////////////////////////


void Tools_Mode_PFX::CopySourceTemplate()
{
	Asura_Assert(s_pszSourceTempList,"");
	//ask which template should be copied
	Tools_PFXSelect_Overlay xOverlay("Copy which source template?", Tools_PFXSelect_Overlay::MODE_SOURCES, "");
	xOverlay.SetSelected(s_pszSourceTempList[s_iEffectSourceTemp]);
	if(!xOverlay.DoModal()) return;

	
	const char* szSelectedSource = xOverlay.GetTemplateName();

	// Get template ID
	Asura_PFX_Source_Template* pxOriginalSourceTemp = Asura_PFX_System::GetSourceTempByHash(xOverlay.GetTemplateHash());
	if(!pxOriginalSourceTemp)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find the template being copied");
		return;
	}
	// Create copy of template
	Asura_String xOriginalSourceName = szSelectedSource;
	StripTemplateNameSuffix(xOriginalSourceName);
	Asura_String xBaseSourceName(xOriginalSourceName);

	Asura_String xJustSourceName;
	int iColonPosition = 0;
	if(xBaseSourceName.Contains(":", &iColonPosition))
	{
		xJustSourceName = &xBaseSourceName.GetString()[iColonPosition + 1];
	}
	else
	{
		xJustSourceName = xBaseSourceName;
	}

	if(!GetCopySourceTemplateName(xJustSourceName)) return;
	Asura_PFX_Source_Template* pxSourceTemp = new Asura_PFX_Source_Template(*pxOriginalSourceTemp);

	xBaseSourceName = s_pxCurrentEffectTemp->m_xName.GetString();
//	Asura_String xFullName = xBaseSourceName;
	xBaseSourceName += ":";
	xBaseSourceName += xJustSourceName;
	Asura_String xCopyName;
	GetUniqueSourceTempName(xBaseSourceName, xCopyName);
	const Asura_Hash_ID uNameHash = Asura_GetHashID(xCopyName.GetString());
	// Add the copy
	Asura_PFX_System::AddSourceTemplate(xCopyName.GetString(), pxSourceTemp);

	int iParticleTemplateHash = pxSourceTemp->m_uParticleTemplateHash;
	const Asura_PFX_Particle_Template* pxSourceParticleTemp = Asura_PFX_System::GetParticleTempByHash(iParticleTemplateHash);
	if(pxSourceParticleTemp)
	{
		Asura_PFX_Particle_Template* pxParticleTemp = new Asura_PFX_Particle_Template(*pxSourceParticleTemp);
		Asura_PFX_System::AddParticleTemplate(xCopyName.GetString(), pxParticleTemp);
		pxSourceTemp->m_uParticleTemplateHash = uNameHash;
	}
	else
	{
		pxSourceTemp->m_uParticleTemplateHash = ASURA_HASH_ID_UNSET;
	}
	s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum].m_uTemplate = uNameHash;

	SaveTemplateByHash(s_uEditingEffectTempHash);

	RefreshSourceTemplateList();
	UpdateSourceChooser();
}

Asura_PFX_Effect_Template* Tools_Mode_PFX::CopyEffectTemplate()
{
	// Get template ID
	Asura_Hash_ID	uTemplateHash	= s_uChooseTemplateHash;
	// Create copy of template
	if(uTemplateHash == ASURA_HASH_ID_UNSET)
	{
		// cant copy nothing!
		return NULL;
	}
	const Asura_PFX_Effect_Template* pxOriginalEffectTemp = Asura_PFX_System::GetEffectTempByHash(uTemplateHash);
	if(!pxOriginalEffectTemp)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find the template being copied");
		return NULL;
	}

	Asura_String xOriginalEffectName = *Asura_PFX_System::GetEffectTempNameByHash(uTemplateHash);
	StripTemplateNameSuffix(xOriginalEffectName);
	Asura_String xBaseEffectName(xOriginalEffectName);
	xBaseEffectName += "Copy";
	if(!GetCopyEffectTemplateName(xBaseEffectName)) return NULL;
	Asura_String xCopyName;
	GetUniqueEffectTempName(xBaseEffectName, xCopyName);
	// Add the copy
	Asura_PFX_Effect_Template* pxEffectTemp = new Asura_PFX_Effect_Template(*pxOriginalEffectTemp);
	Asura_PFX_System::AddEffectTemplate(xCopyName.GetString(), pxEffectTemp);

	for(u_int uSource = 0; uSource < pxEffectTemp->m_uNumSources; ++uSource)
	{			
		u_int uSourceTemplateHash = pxEffectTemp->m_pxSources[uSource].m_uTemplate;
		if(!uSourceTemplateHash) continue;
		// Check to see if we've already copied this source template
		bool bUnique = true;
		for(u_int uSourceCheck = 0; uSourceCheck < uSource; ++uSourceCheck)
		{
			if(uSourceTemplateHash == pxOriginalEffectTemp->m_pxSources[uSourceCheck].m_uTemplate)
			{
				bUnique = false;
				// Use the existing copy
				pxEffectTemp->m_pxSources[uSource].m_uTemplate = pxEffectTemp->m_pxSources[uSourceCheck].m_uTemplate;
			}
		}
		if(!bUnique) continue;
		// Have a go at generating a name for this source
		const Asura_String* pxOriginalSourceName = Asura_PFX_System::GetSourceTempNameByHash(uSourceTemplateHash);
		if(!pxOriginalSourceName)
		{
			//can't copy source if it doesn't exist...
			pxEffectTemp->m_pxSources[uSource].m_uTemplate = ASURA_HASH_ID_UNSET;
			continue;
		}

		Asura_String xBaseSourceName = xBaseEffectName;
		int iColonPosition = 0;
		if(pxOriginalSourceName->Contains(":", &iColonPosition))
		{
			const char* szJustSourceName = &pxOriginalSourceName->GetString()[iColonPosition];
			xBaseSourceName += szJustSourceName;
		}
		else
		{
			xBaseSourceName += Asura_String(":");
			xBaseSourceName += *pxOriginalSourceName;
		}

		// Create source template
		Asura_PFX_Source_Template* pxSourceTemp = new Asura_PFX_Source_Template(*Asura_PFX_System::GetSourceTempByHash(uSourceTemplateHash));
		Asura_PFX_System::AddSourceTemplate(xBaseSourceName.GetString(), pxSourceTemp);
		// Point copied effect template to it
		pxEffectTemp->m_pxSources[uSource].m_uTemplate = Asura_GetHashID(xBaseSourceName.GetString());

		// Create particle template
		int iParticleTemplateHash = pxSourceTemp->m_uParticleTemplateHash;
		if(!iParticleTemplateHash || !Asura_PFX_System::GetParticleTempByHash(iParticleTemplateHash))
		{
			pxSourceTemp->m_uParticleTemplateHash = ASURA_HASH_ID_UNSET;
		}
		else
		{
			Asura_PFX_Particle_Template* pxParticleTemp = new Asura_PFX_Particle_Template(*Asura_PFX_System::GetParticleTempByHash(iParticleTemplateHash));
			Asura_PFX_System::AddParticleTemplate(xBaseSourceName.GetString(), pxParticleTemp);
			// Point copied source template to it
			pxSourceTemp->m_uParticleTemplateHash = Asura_GetHashID(xBaseSourceName.GetString());
		}
	}
	// Select the new copy
	s_uChooseTemplateHash = xCopyName.GetHash();
	SaveTemplateByHash(s_uChooseTemplateHash);
	// Update list
	Tools_Entity_PFX_Effect::UpdateEffectList();

	return pxEffectTemp;
}

/// Helper functions for template creation ////////////////////////////////////////////////////////

static bool PromptForNewPFXEffectFileName(const Asura_String &xTitle, Asura_String &xFileName)
{
	// Prompt the user with the standard Windows save dialogue, so they can create folders as required
	Tools_FileSystem_DialogInfo xSaveDialogInfo;

	// Start with the existing name, if any
	Asura_String xDefaultFilename;
	if( !xFileName.IsBlank() )
	{
		xDefaultFilename = "PFXTemps\\";
		xDefaultFilename += xFileName;
		if (!xDefaultFilename.EndsWith(".pfx"))
		{
			xDefaultFilename += ".pfx";
		}

		xSaveDialogInfo.DefaultFilename = xDefaultFilename.GetString();
	}
	else
	{
		xSaveDialogInfo.DefaultPath = "PFXTemps\\";
	}

	xSaveDialogInfo.FileFilter = "PFX Effect Templates\0*.pfx\0\0";
	xSaveDialogInfo.DefaultExtension = "pfx"; // Strip off the .
	xSaveDialogInfo.WindowTitle = xTitle.GetString();

	const Asura_Char *pRet = Tools_FileSystem::GetSaveFilename( xSaveDialogInfo );
	if( !pRet) return false;

	// Check for correct extension
	if( !Asura_StringUtil::EndsWith( pRet, ".pfx", true ) )
	{
		Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_ERROR, "The filename must have a .pfx extension." );
		return false;
	}

	// Strip off Asura_Files\Materials from the start of the path
	{
		pRet = Asura_FileAccess::PlatformRemoveNetworkOrExecutableDir( pRet );

		if( !Asura_StringUtil::StartsWith( pRet, "PFXTemps\\", true ) )
		{
			Tools_Timed_Message_System::AddMessage( Tools_Timed_Message_System::COLOUR_ERROR, "The filename is not within the PFXTemps folder." );
			return false;
		}

		pRet += strlen( "PFXTemps\\" );
	}

	// Remove extension from the path
	Asura_String xTempName;
	xFileName.MakeNull();
	Asura_String( pRet ).SplitFilename( &xFileName, &xTempName, NULL );

	if( !xFileName.IsBlank() )
	{
		xFileName += "\\";
	}

	xFileName += xTempName;

	return true;
}


bool Tools_Mode_PFX::GetNewEffectTemplateName(Asura_String& xName)
{
	do
	{
		// Get new name
		if(!PromptForNewPFXEffectFileName("Enter name for new effect template", xName))	return false;
		// Strip _[E, S, P]
		StripTemplateNameSuffix(xName);
	}
	while(xName.IsBlank());

	return true;
}

bool Tools_Mode_PFX::GetNewSourceTemplateName(Asura_String& xName)
{
	do
	{
		// Get new name
		if(!Tools_Object_System::AskForName(xName, "Enter name for new source template"))	return false;
		// Strip _[E, S, P]
		StripTemplateNameSuffix(xName);
	}
	while(xName.IsBlank());

	return true;
}
bool Tools_Mode_PFX::GetNewParticleTemplateName(Asura_String& xName)
{
	do
	{
		// Get new name
		if(!Tools_Object_System::AskForName(xName, "Enter name for new particle template"))	return false;
		// Strip _[E, S, P]
		StripTemplateNameSuffix(xName);
	}
	while(xName.IsBlank());

	return true;
}


void Tools_Mode_PFX::GetUniqueParticleTempName(const Asura_String& xBaseName, Asura_String& xUniqueName)
{
	xUniqueName = xBaseName;
	xUniqueName += Asura_String("_P");
	int uNum = 1;
	while(Asura_PFX_System::DoesParticleHashExist( Asura_GetHashID( xUniqueName.GetString() ) ))
	{
		char szNum[8];
		Asura_PrintfToBfr(szNum, "_P%u", 8, true, uNum);
		xUniqueName = xBaseName;
		xUniqueName += Asura_String(szNum);
		++uNum;
	}
}

void Tools_Mode_PFX::GetUniqueSourceTempName(const Asura_String& xBaseName, Asura_String& xUniqueName)
{
	xUniqueName = xBaseName;
	xUniqueName += Asura_String("_S");
	int uNum = 1;
	while(Asura_PFX_System::DoesSourceHashExist( Asura_GetHashID( xUniqueName.GetString() ) ))
	{
		char szNum[8];
		Asura_PrintfToBfr(szNum, "_S%u", 8, true, uNum);
		xUniqueName = xBaseName;
		xUniqueName += Asura_String(szNum);
		++uNum;
	}
}

void Tools_Mode_PFX::GetUniqueEffectTempName(const Asura_String& xBaseName, Asura_String& xUniqueName)
{
	xUniqueName = xBaseName;
	xUniqueName += Asura_String("_E");
	int uNum = 1;
	while(Asura_PFX_System::DoesEffectHashExist( Asura_GetHashID( xUniqueName.GetString() ) ))
	{
		char szNum[8];
		Asura_PrintfToBfr(szNum, "_E%u", 8, true, uNum);
		xUniqueName = xBaseName;
		xUniqueName += Asura_String(szNum);
		++uNum;
	}
}

bool Tools_Mode_PFX::GetCopyEffectTemplateName(Asura_String& xName)
{
	do
	{
		// Get new name
		if(!PromptForNewPFXEffectFileName("Enter name for copy of effect template", xName))	return false;
		// Strip _[E, S, P]
		StripTemplateNameSuffix(xName);
	}
	while(xName.IsBlank());

	return true;
}


bool Tools_Mode_PFX::GetCopySourceTemplateName(Asura_String& xName)
{
	
	GetString_Overlay xOverlay("Enter name for copy of source template",xName.GetString());

	do
	{
		if(!xOverlay.DoModal()) return false;
		
		xName = xOverlay.GetString();
		// Strip _[E, S, P]
		StripTemplateNameSuffix(xName);
	}
	while(xName.IsBlank());

	return true;
}

bool Tools_Mode_PFX::GetCopyParticleTemplateName(Asura_String& xName)
{
	GetString_Overlay xOverlay("Enter name for copy of source template",xName.GetString());

	do
	{
		if(!xOverlay.DoModal()) return false;

		xName = xOverlay.GetString();
		// Strip _[E, S, P]
		StripTemplateNameSuffix(xName);
	}
	while(xName.IsBlank());

	return true;
}


void Tools_Mode_PFX::StripTemplateNameSuffix(Asura_String& xName)
{
	int iLength = xName.GetLength();
	const char* szName = xName.GetString();
	
	//ignore any numbers at the end of the name
	int iPos = iLength-1;
	while(iPos >=0 && szName[iPos] >= '0' && szName[iPos] <= '9')
	{
		iPos--;
	}
	if(iPos<0) return;

	//at a _E , _S or _P?
	if(iPos>=1 && szName[iPos-1] == '_')
	{
		char cLastChar = xName.GetString()[iPos];
		if(cLastChar == 'E' || cLastChar== 'S' || cLastChar == 'P')
		{
			// Terminate string here instead.
			xName.Truncate(iPos-1);
		}
	
	}

}

/// Template renaming //////////////////////////////////////////////////////////////////////////////



void Tools_Mode_PFX::RenameSourceTemplate()
{
	Asura_Assert(s_pszSourceTempList,"");
	
	Asura_String xOriginalSourceName = s_pszSourceTempList[s_iEffectSourceTemp];
	Asura_Hash_ID uOldHash = xOriginalSourceName.GetHash();

	// Get template ID
	Asura_PFX_Source_Template* pxSourceTemp = Asura_PFX_System::GetSourceTempByHash(uOldHash);
	if(!pxSourceTemp)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find the template being renamed");
		return;
	}
	// Ask for a new name for the source template
	Asura_String xBaseSourceName = xOriginalSourceName;
	StripTemplateNameSuffix(xBaseSourceName);

	Asura_String xJustSourceName;
	int iColonPosition = 0;
	if(xBaseSourceName.Contains(":", &iColonPosition))
	{
		xJustSourceName = &xBaseSourceName.GetString()[iColonPosition + 1];
	}
	else
	{
		xJustSourceName = xBaseSourceName;
	}

	if(!GetCopySourceTemplateName(xJustSourceName)) return;

	xBaseSourceName = s_pxCurrentEffectTemp->m_xName.GetString();
//	Asura_String xFullName = xBaseSourceName;
	xBaseSourceName += ":";
	xBaseSourceName += xJustSourceName;

	Asura_String xCopyName;
	GetUniqueSourceTempName(xBaseSourceName, xCopyName);
	
	const Asura_Hash_ID uNameHash = Asura_GetHashID(xCopyName.GetString());
	
	
	// Rename the source
	bool bSourceOkay = Asura_PFX_System::RenameSourceTemplate(xOriginalSourceName.GetString(), xCopyName.GetString());
	if(!bSourceOkay)
	{
		//I think the unique name picking above should prevent this sort of thing happening
		Asura_Assert(0,"Failed to rename source template");
		return;
	}

	// Rename the particle template to match
	int iParticleTemplateHash = pxSourceTemp->m_uParticleTemplateHash;
	const Asura_PFX_Particle_Template* pxSourceParticleTemp = Asura_PFX_System::GetParticleTempByHash(iParticleTemplateHash);
	if(pxSourceParticleTemp)
	{
		bool bParticleOkay = Asura_PFX_System::RenameParticleTemplate(xOriginalSourceName.GetString(), xCopyName.GetString());
		Asura_Assert(bParticleOkay,"Error renaming particle template");
		pxSourceTemp->m_uParticleTemplateHash = uNameHash;
	}
	else
	{
		pxSourceTemp->m_uParticleTemplateHash = ASURA_HASH_ID_UNSET;
	}
	//update the references to the source in the effect template
	for(u_int i=0; i<s_pxCurrentEffectTemp->m_uNumSources; i++)
	{
		if(s_pxCurrentEffectTemp->m_pxSources[i].m_uTemplate == uOldHash)
		{
			s_pxCurrentEffectTemp->m_pxSources[i].m_uTemplate = uNameHash;
		}

	}

	SaveTemplateByHash(s_uEditingEffectTempHash);

	RefreshSourceTemplateList();
	UpdateSourceChooser();
}

Asura_PFX_Effect_Template* Tools_Mode_PFX::RenameEffectTemplate()
{
	// Get template ID and the template
	Asura_Hash_ID	uTemplateHash	= s_uChooseTemplateHash;
	if (uTemplateHash == ASURA_HASH_ID_UNSET)
	{
		return NULL;
	}
	Asura_PFX_Effect_Template* pxEffectTemp = Asura_PFX_System::GetEffectTempByHash(uTemplateHash);

	if(!pxEffectTemp)
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Failed to find the template being renamed");
		return NULL;
	}

	Asura_String xOriginalEffectName = *Asura_PFX_System::GetEffectTempNameByHash(uTemplateHash);
	Asura_Hash_ID uOldEffectHash = Asura_GetHashID(xOriginalEffectName.GetString());

	//remove _E suffix while asking for new name
	Asura_String xBaseEffectName = xOriginalEffectName;
	StripTemplateNameSuffix(xBaseEffectName);

	//ask for new name for template
	if(!GetNewEffectTemplateName(xBaseEffectName)) return NULL;

	Asura_String xNewEffectName = xBaseEffectName;
	if(xNewEffectName.IsBlank()) return NULL;

	//Add _E suffix back on
	xNewEffectName += Asura_String("_E");

	//make sure the name is okay
	if(Asura_PFX_System::DoesEffectHashExist( Asura_GetHashID(xNewEffectName.GetString())))
	{
		ShowMessageOverlay("That name is already in use");
		return NULL;
	}

	//update the effect template
	bool bOkay = Asura_PFX_System::RenameEffectTemplate(xOriginalEffectName.GetString(), xNewEffectName.GetString());
	if(!bOkay) return NULL;

	Asura_Hash_ID uNewEffectHash = Asura_GetHashID(xNewEffectName.GetString());

	Asura_Hash_ID * puOriginalSourceIDs = pxEffectTemp->m_uNumSources ? new Asura_Hash_ID[pxEffectTemp->m_uNumSources] : 0;

	//rename the source and particle template
	for(u_int uSource = 0; uSource < pxEffectTemp->m_uNumSources; ++uSource)
	{			
		u_int uSourceTemplateHash = pxEffectTemp->m_pxSources[uSource].m_uTemplate;
		puOriginalSourceIDs[uSource] = uSourceTemplateHash;
		if(!uSourceTemplateHash) continue;
		// Check to see if we've already renamed this source template
		bool bUnique = true;
		for(u_int uSourceCheck = 0; uSourceCheck < uSource; ++uSourceCheck)
		{
			if(uSourceTemplateHash == puOriginalSourceIDs[uSourceCheck])
			{
				bUnique = false;
				// Use the existing copy
				pxEffectTemp->m_pxSources[uSource].m_uTemplate = pxEffectTemp->m_pxSources[uSourceCheck].m_uTemplate;
			}
		}
		if(!bUnique) continue;
		// Have a go at generating a name for this source
		const Asura_String* pxOriginalSourceName = Asura_PFX_System::GetSourceTempNameByHash(uSourceTemplateHash);
		Asura_PFX_Source_Template* pxSourceTemp = Asura_PFX_System::GetSourceTempByHash(uSourceTemplateHash);
		if(!pxOriginalSourceName || !pxSourceTemp)
		{
			//can't rename source if it doesn't exist...
			pxEffectTemp->m_pxSources[uSource].m_uTemplate = ASURA_HASH_ID_UNSET;
			continue;
		}



		Asura_String xBaseSourceName = /*BaseEffectName*/ xNewEffectName;
		int iColonPosition = 0;
		if(pxOriginalSourceName->Contains(":", &iColonPosition))
		{
			const char* szJustSourceName = &pxOriginalSourceName->GetString()[iColonPosition];
			xBaseSourceName += szJustSourceName;
		}
		else
		{
			xBaseSourceName += Asura_String(":");
			xBaseSourceName += *pxOriginalSourceName;
		}

		//search for a name that is available as both a source and a particle template
		Asura_String xSourceName = xBaseSourceName;
		int uNum = 1;
		while(Asura_PFX_System::DoesSourceHashExist( Asura_GetHashID( xSourceName.GetString())) ||
				Asura_PFX_System::DoesParticleHashExist( Asura_GetHashID( xSourceName.GetString())))
		{
			char szNum[8];
			Asura_PrintfToBfr(szNum, "%u", 8, true, uNum);
			xSourceName = xBaseSourceName;
			xSourceName += Asura_String(szNum);
			++uNum;
		}
		

			

		//rename source template and update the effect template to look at it
		bool bSourceOkay = Asura_PFX_System::RenameSourceTemplate(pxOriginalSourceName->GetString(), xSourceName.GetString());
		Asura_Assert(bSourceOkay,"Error renaming source template");
		pxEffectTemp->m_pxSources[uSource].m_uTemplate = Asura_GetHashID(xSourceName.GetString());

		//get the particle template
		Asura_Hash_ID uParticleTemplateHash = pxSourceTemp->m_uParticleTemplateHash;
		const Asura_String* pxOriginalParticleName = Asura_PFX_System::GetParticleTempNameByHash(uParticleTemplateHash);
		Asura_PFX_Particle_Template* pxParticleTemp = Asura_PFX_System::GetParticleTempByHash(uParticleTemplateHash);
		if(!pxOriginalParticleName || !pxParticleTemp)
		{
			//can't rename particle if it doesn't exist...
			pxSourceTemp->m_uParticleTemplateHash = ASURA_HASH_ID_UNSET;
			continue;
		}

		//rename particle template and update the source template to look at it
		bool bParticleOkay = Asura_PFX_System::RenameParticleTemplate(pxOriginalParticleName->GetString(), xSourceName.GetString());
		Asura_Assert(bParticleOkay,"Error renaming particle template");
		pxSourceTemp->m_uParticleTemplateHash = Asura_GetHashID(xSourceName.GetString());
	}
	delete [] puOriginalSourceIDs;

	Tools_Entity_PFX_Effect::UpdateEffectList();
	
	// Store the original name, so the old file can be deleted once the save succeeds
	// If there is already an original name (effect renamed twice in a row maybe) then leave it alone
	// RAR 21/05/2008
	if(pxEffectTemp->m_xOriginalNameForDeletion.IsBlank())
	{
		pxEffectTemp->m_xOriginalNameForDeletion = xOriginalEffectName;
	}

	// Save renamed template
	SetModified(pxEffectTemp);
	SaveTemplateByHash(uTemplateHash);

	//update and PFX entities that were using this template
	Tools_Entity_PFX_Effect::ReplaceEffectTemplate(uOldEffectHash, uNewEffectHash);

	return pxEffectTemp;
}

/// Template deleting //////////////////////////////////////////////////////////////////////////////



/// Cancel changes to templates //////////////////////////////////////////////////////////////////
void Tools_Mode_PFX::CancelParticleTemplateChanges()
{
	if(!ShowYesNoOverlay("Undo changes to this particle template?")) return;
	
	if(s_pxCurrentSourceTemp)
	{
		Tools_Entity_PFX_Effect::ClearAndFlagForRestart_ParticleTemplate(s_pxCurrentSourceTemp->m_uParticleTemplateHash);
	}

	//undo the changes
	*s_pxCurrentParticleTemp = *s_pxSavedParticleTemp;

}

void Tools_Mode_PFX::CancelSourceTemplateChanges()
{
	if(!ShowYesNoOverlay("Undo changes to this source template?")) return;

	Tools_Entity_PFX_Effect::ClearAndFlagForRestart_SourceTemplate(s_pxCurrentSourceTemp->GetTemplateHash());
	
	//undo the changes
	*s_pxCurrentSourceTemp = *s_pxSavedSourceTemp;
}

void Tools_Mode_PFX::CancelEffectTemplateChanges()
{
	if(!ShowYesNoOverlay("Undo changes to this effect template?")) return;

	Tools_Entity_PFX_Effect::ClearAndFlagForRestart_EffectTemplate(s_pxCurrentEffectTemp->GetTemplateHash());

	//undo the changes
	*s_pxCurrentEffectTemp = *s_pxSavedEffectTemp;
}

void Tools_Mode_PFX::ReloadCurrentParticleTemp()
{
	if(!ShowYesNoOverlay("Revert to saved version of this particle template?")) return;
	ReloadParticleTemplate(s_uEditingParticleTempHash);
}

void Tools_Mode_PFX::ReloadCurrentSourceTemp()
{
	if(!ShowYesNoOverlay("Revert to saved version of this source template?")) return;
	ReloadSourceTemplate(s_uEditingSourceTempHash);
}

void Tools_Mode_PFX::ReloadCurrentEffectTemp()
{
	if(!ShowYesNoOverlay("Revert to saved version of this effect template?")) return;
	ReloadEffectTemplate(s_uEditingEffectTempHash);
}

void Tools_Mode_PFX::ReloadSourceTempFull(Asura_Hash_ID uHashID)
{
	ReloadSourceTemplate(uHashID);
	Asura_PFX_Source_Template* pxTemplate = Asura_PFX_System::GetSourceTempByHash(uHashID);
	if(pxTemplate)
	{
		ReloadParticleTemplate(pxTemplate->m_uParticleTemplateHash);
	}
}

void Tools_Mode_PFX::ReloadCurrentSourceTempFull()
{
	if(!ShowYesNoOverlay("Revert to saved version of this source + particle template?")) return;
	ReloadSourceTempFull(s_uEditingSourceTempHash);
}

void Tools_Mode_PFX::ReloadCurrentEffectTempFull()
{
	if(!ShowYesNoOverlay("Revert to saved version of this effect and all its source and particle templates?")) return;
	ReloadEffectTemplate(s_uEditingEffectTempHash);

	//close and open the entity's overlay
	if(Tools_Entity_PFX_Effect::s_pxCurrentObject)
	{
		Tools_Entity_PFX_Effect::CloseOverlays();
		Tools_Entity_PFX_Effect::s_pxCurrentObject->CreateOverlay();
	}
}

void Tools_Mode_PFX::ReloadParticleTemplate(Asura_Hash_ID uHash)
{
	bool bEditing = (uHash == s_uEditingParticleTempHash);
	if(bEditing) CloseEditParticleTemplate();
	Tools_Entity_PFX_Effect::ClearAndFlagForRestart_ParticleTemplate(uHash);
	Asura_PFX_System::SetAllowTemplateReload(true);
	Asura_PFX_System::LoadTemplate(0, uHash);
	Asura_PFX_System::SetAllowTemplateReload(false);
}

void Tools_Mode_PFX::ReloadSourceTemplate(Asura_Hash_ID uHash)
{
	bool bEditing = (uHash == s_uEditingSourceTempHash);
	if(bEditing) CloseEditSourceTemplate();
	Tools_Entity_PFX_Effect::ClearAndFlagForRestart_SourceTemplate(uHash);
	Asura_PFX_System::SetAllowTemplateReload(true);
	Asura_PFX_System::LoadTemplate(1, uHash);
	Asura_PFX_System::SetAllowTemplateReload(false);
}

void Tools_Mode_PFX::ReloadEffectTemplate(Asura_Hash_ID uHash)
{
	bool bEditing = (uHash == s_uEditingEffectTempHash);
	if(bEditing) CloseEditEffectTemplate();
	Tools_Entity_PFX_Effect::ClearAndFlagForRestart_EffectTemplate(uHash);
	Asura_PFX_System::SetAllowTemplateReload(true);
	Asura_PFX_System::LoadTemplate(2, uHash);
	Asura_PFX_System::SetAllowTemplateReload(false);
}

void Tools_Mode_PFX::ReloadAllTemplates()
{
	if(!ShowYesNoOverlay("Are you sure you want to reload all templates? You will lose any unsaved work.")) return;

	CloseEditParticleTemplate();
	CloseEditSourceTemplate();
	CloseEditEffectTemplate();
	Tools_Entity_PFX_Effect::ClearAllEffectsAndFlagForRestart();

	Asura_PFX_System::ReloadAllTemplates();

	//close and open the entity's overlay
	if(Tools_Entity_PFX_Effect::s_pxCurrentObject)
	{
		Tools_Entity_PFX_Effect::CloseOverlays();
		Tools_Entity_PFX_Effect::s_pxCurrentObject->CreateOverlay();
	}
}

void Tools_Mode_PFX::GetTemplateFileName(const Asura_String* pxTempName, Asura_String* pxFileName, TOOLS_MODE_PFX_SAVE_TYPE eSaveType /* = TOOLS_MODE_PFX_SAVE_NORMAL */)
{
	switch(eSaveType)
	{
		case TOOLS_MODE_PFX_SAVE_NORMAL:	*pxFileName = "PFXTemps\\";				break;
		case TOOLS_MODE_PFX_SAVE_EXPORT:	*pxFileName = Tools_Level_Export::GetTempDirectory(); *pxFileName += "\\";	break;
		case TOOLS_MODE_PFX_SAVE_AUTO:	*pxFileName = "PFXTemps\\Autosave\\";				break;
	}
	*pxFileName += Asura_String(pxTempName->GetString());
	if(eSaveType == TOOLS_MODE_PFX_SAVE_AUTO) *pxFileName += Asura_String("_A1");
	*pxFileName += Asura_String(".pfx");
	if(eSaveType == TOOLS_MODE_PFX_SAVE_EXPORT && Tools_Level_Export::ShouldExportBigEndian()) *pxFileName += Asura_String(ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX);
}

void Tools_Mode_PFX::GetParticleTempFileName(const Asura_String* pxTempName, Asura_String* pxFileName, TOOLS_MODE_PFX_SAVE_TYPE eSaveType /* = TOOLS_MODE_PFX_SAVE_NORMAL */)
{
	switch(eSaveType)
	{
		case TOOLS_MODE_PFX_SAVE_NORMAL:	*pxFileName = "PFXTemps\\Particle\\"; break;
		case TOOLS_MODE_PFX_SAVE_EXPORT:	*pxFileName = Tools_Level_Export::GetTempDirectory(); *pxFileName += "\\";	break;
	}
	*pxFileName += Asura_String(pxTempName->GetString());
	*pxFileName += Asura_String(".pcl");
	if(eSaveType == TOOLS_MODE_PFX_SAVE_EXPORT && Tools_Level_Export::ShouldExportBigEndian()) *pxFileName += Asura_String(ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX);
}

void Tools_Mode_PFX::GetSourceTempFileName(const Asura_String* pxTempName, Asura_String* pxFileName, TOOLS_MODE_PFX_SAVE_TYPE eSaveType /* = TOOLS_MODE_PFX_SAVE_NORMAL */)
{
	switch(eSaveType)
	{
		case TOOLS_MODE_PFX_SAVE_NORMAL:	*pxFileName = "PFXTemps\\Source\\"; break;
		case TOOLS_MODE_PFX_SAVE_EXPORT:	*pxFileName = Tools_Level_Export::GetTempDirectory(); *pxFileName += "\\";	break;
	}
	*pxFileName += Asura_String(pxTempName->GetString());
	*pxFileName += Asura_String(".src");
	if(eSaveType == TOOLS_MODE_PFX_SAVE_EXPORT && Tools_Level_Export::ShouldExportBigEndian()) *pxFileName += Asura_String(ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX);
}

void Tools_Mode_PFX::GetEffectTempFileName(const Asura_String* pxTempName, Asura_String* pxFileName, TOOLS_MODE_PFX_SAVE_TYPE eSaveType /* = TOOLS_MODE_PFX_SAVE_NORMAL */)
{
	switch(eSaveType)
	{
		case TOOLS_MODE_PFX_SAVE_NORMAL:	*pxFileName = "PFXTemps\\Effect\\"; break;
		case TOOLS_MODE_PFX_SAVE_EXPORT:	*pxFileName = Tools_Level_Export::GetTempDirectory(); *pxFileName += "\\";	break;
	}
	*pxFileName += Asura_String(pxTempName->GetString());
	*pxFileName += Asura_String(".fct");
	if(eSaveType == TOOLS_MODE_PFX_SAVE_EXPORT && Tools_Level_Export::ShouldExportBigEndian()) *pxFileName += Asura_String(ASURA_STANDARD_FILE_EXTENSION_BE_SUFFIX);
}

/// Miscellaneous helpers /////////////////////////////////////////////////////////////////////////

void Tools_Mode_PFX::EditGravity()
{
	Asura_Assert(s_pxCurrentSourceTemp, "No current source template to edit gravity for.");

	if(!s_pxSourceGravityObject)
	{
		Tools_Entity_PFX_Effect::StartTempEntityEdit();
		s_pxSourceGravityObject = new Tools_PFX_Vector_Object();
		s_pxSourceGravityObject->m_xName = "PFX Source Gravity Direction";
		s_pxSourceGravityObject->m_xTypeDescription = "Gravity direction for current PFX source template";
	}
		
	Asura_Assert(s_pxSourceGravityObject, "Null gravity vector");
	if(s_pxSourceGravityObject)
	{
		Asura_Vector_3 xDirection = s_pxCurrentSourceTemp->m_xGravityVector;
		if(!xDirection.Normalise())
		{
			// No vector. Set it to a default
			xDirection.y = 1.f;
		}
		// Set orientation by current source gravity
		Asura_Maths::OrientationFromDirection(s_pxSourceGravityObject->Orientation, xDirection);
		// Set position in front of camera
		Asura_Camera::GetForwardVector(s_pxSourceGravityObject->Position);
		s_pxSourceGravityObject->Position += *Asura_Camera::GetPosition();
	}
}

void Tools_Mode_PFX::EditSourceVector()
{
	Asura_Assert(s_pxCurrentEffectTemp, "No current effect template to edit offset for.");
	bool bHaveValidObject = false;
	if(Tools_Entity_PFX_Effect::s_pxCurrentObject)
	{
		if(Tools_Entity_PFX_Effect::s_pxCurrentObject->m_pxServerEntity->GetTemplateHashID() == s_uEditingEffectTempHash)
		{
			bHaveValidObject = true;
		}
	}
	Asura_Assert(bHaveValidObject, "Sorry - can only edit offset using a selected PFX effect using this effect template at the moment");
	if(!bHaveValidObject) return;

	u_int uSourceNumber = s_iEffectSourceNum;
	if(!s_pxEffectSrcOffsetObject)
	{
		Tools_Entity_PFX_Effect::StartTempEntityEdit();
		s_pxEffectSrcOffsetObject = new Tools_PFX_SourceVolume_Object(s_pxCurrentEffectTemp->m_pxSources[uSourceNumber].m_xVolume);
		//s_pxEffectSrcOffsetObject->m_xName = "PFX Effect Source Offset and Orientation";
		//s_pxEffectSrcOffsetObject->m_xTypeDescription = "Source Offset and Orientation for current PFX effect template";
	}
	Asura_Assert(s_pxEffectSrcOffsetObject, "Null offset vector");
	if(s_pxEffectSrcOffsetObject)
	{
		Asura_Vector_3 xPos = s_pxCurrentEffectTemp->m_pxSources[uSourceNumber].m_xOffset * Tools_Entity_PFX_Effect::s_pxCurrentObject->Orientation + Tools_Entity_PFX_Effect::s_pxCurrentObject->Position;
		Asura_Matrix_3x3 xOrient = Tools_Entity_PFX_Effect::s_pxCurrentObject->Orientation * s_pxCurrentEffectTemp->m_pxSources[uSourceNumber].m_xOrient;
		s_pxEffectSrcOffsetObject->SetPositionAndOrient(xPos,xOrient);
	}
}

void Tools_Mode_PFX::EditSourceMovementVector()
{
	Asura_Assert(s_pxCurrentEffectTemp, "No current effect template to edit offset for.");
	bool bHaveValidObject = false;
	if(Tools_Entity_PFX_Effect::s_pxCurrentObject)
	{
		if(Tools_Entity_PFX_Effect::s_pxCurrentObject->m_pxServerEntity->GetTemplateHashID() == s_uEditingEffectTempHash)
		{
			bHaveValidObject = true;
		}
	}
	Asura_Assert(bHaveValidObject, "Sorry - can only edit offset using a selected PFX effect using this effect template at the moment");
	if(!bHaveValidObject) return;

	if(!s_pxEffectSrcMovementObject)
	{
		Tools_Entity_PFX_Effect::StartTempEntityEdit();
		s_pxEffectSrcMovementObject = new Tools_PFX_Vector_Object();
		s_pxEffectSrcMovementObject->m_xName = "PFX Effect Source Movement Vector";
		s_pxEffectSrcMovementObject->m_xTypeDescription = "Source movement vector for current PFX effect template";
	}
	Asura_Assert(s_pxEffectSrcMovementObject, "Null offset vector");
	if(s_pxEffectSrcMovementObject)
	{
		u_int uSourceNumber = s_iEffectSourceNum;
		// Set orientation 
		Asura_Vector_3 xDirection = s_pxCurrentEffectTemp->m_pxSources[uSourceNumber].m_xVelocity;
		if(!xDirection.Normalise())
		{
			// No vector. Set it to a default
			xDirection.z = 1.f;
		}
		Asura_Maths::OrientationFromDirection(s_pxEffectSrcMovementObject->Orientation, xDirection * Tools_Entity_PFX_Effect::s_pxCurrentObject->Orientation);
		// Set position 1m above current entity
		s_pxEffectSrcMovementObject->Position = Tools_Entity_PFX_Effect::s_pxCurrentObject->Position;
		s_pxEffectSrcMovementObject->Position.y -= 1.f;
	}
}

void Tools_Mode_PFX::ResetSourceVector()
{
	const u_int uSourceNumber = s_iEffectSourceNum;
	// Set orientation
	s_pxCurrentEffectTemp->m_pxSources[uSourceNumber].m_xOrient.Identity();
	// Set position
	s_pxCurrentEffectTemp->m_pxSources[uSourceNumber].m_xOffset = Asura_Vector_3(0.f, 0.f, 0.f);
	SetModified(s_pxCurrentEffectTemp);
	if(s_pxEffectSrcOffsetObject)
	{
		Asura_Vector_3 xPos = s_pxCurrentEffectTemp->m_pxSources[uSourceNumber].m_xOffset + Tools_Entity_PFX_Effect::s_pxCurrentObject->Position;
		Asura_Matrix_3x3 xOrient = Tools_Entity_PFX_Effect::s_pxCurrentObject->Orientation * s_pxCurrentEffectTemp->m_pxSources[uSourceNumber].m_xOrient;
		s_pxEffectSrcOffsetObject->SetPositionAndOrient(xPos,xOrient);
	}
	if(Tools_Entity_PFX_Effect::s_pxCurrentObject)
	{
		Tools_Entity_PFX_Effect::s_pxCurrentObject->m_bGameEntitiesNeedUpdating = true;
	}
}

void Tools_Mode_PFX::RemoveCurrentSource()
{
	Asura_Assert(s_pxCurrentEffectTemp, "Sanity check");
	if (!s_pxCurrentEffectTemp)
	{
		return;
	}

	Tools_Entity_PFX_Effect::ClearAndFlagForRestart_EffectTemplate(s_pxCurrentEffectTemp->GetTemplateHash());


	s_bNumSourcesChanged = true;
	s_iNumberOfSources--;
	Asura_Assert(s_iNumberOfSources, "Sanity check");
	s_pxCurrentEffectTemp->CopySourceData(s_iEffectCurrentSourceNum, s_pxCurrentEffectTemp, s_iNumberOfSources);
	s_bEffectSourceChanged = true;
	s_bSourceRemoved = true;

}

bool Tools_Mode_PFX::CanRemoveCurrentSource()
{
	return (s_pxCurrentEffectTemp && s_pxCurrentEffectTemp->m_uNumSources > 1);
}


void Tools_Mode_PFX::EditEffectSourceTemplate()
{
	if(s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum].m_uTemplate)
	{
		Asura_Hash_ID	uHash	= s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum].m_uTemplate;
		CanSaveSourceTempByHash(uHash,true,false);//warn about readonly files
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Error: no source template selected");
	}
}

void Tools_Mode_PFX::EditSourceParticleTemplate()
{
	if(s_pxCurrentSourceTemp->m_uParticleTemplateHash)
	{
		Asura_Hash_ID	uHash	= s_pxCurrentSourceTemp->m_uParticleTemplateHash;
		CanSaveParticleTempByHash(uHash,true,false);//warn about readonly files
	}
	else
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"Error: no particle template selected");
	}
}

void Tools_Mode_PFX::AddEffectPlatformOverride()
{
	//get a list of platforms to add (that aren't already there)
	ListSelection_Overlay xOverlay("Add override for which platform?");

	for(int i=0;i<Tools_Level_Export::ExportType_NumPlatforms;i++)
	{
		//this platform used yet?
		bool bAdd = true;
		for(u_int j=0;j<s_pxCurrentEffectTemp->m_uNumPlatformOverrides;j++)
		{
			if(i==s_pxCurrentEffectTemp->m_pxPlatformOverrides[j].m_iPlatform)
			{
				bAdd = false;
				break;

			}
		}
		if(bAdd)
		{
			xOverlay.AddItem(Tools_Level_Export::GetPlatformName(i),i);
		}
	}

	if(!xOverlay.GetNumItems())
	{
		ShowMessageOverlay("Overrides have already been added for all platforms");
		return;
	}

	//ask for platform
	if(!xOverlay.DoModal()) return;

	Asura_ResourcePushType(ASURA_RESOURCE_PFX);
	{
		int iPlatform = xOverlay.GetSelectionData();

		//allocate new array , and copy old values
		Asura_PFX_Effect_PlatformOverride* pxPlatformOverrides;
		Asura_ResourceAllocateArray(pxPlatformOverrides, s_pxCurrentEffectTemp->m_uNumPlatformOverrides+1);
		for(u_int i=0;i<s_pxCurrentEffectTemp->m_uNumPlatformOverrides;i++)
		{
			pxPlatformOverrides[i] = s_pxCurrentEffectTemp->m_pxPlatformOverrides[i];
		}

		//initialise new entry
		int iIndex = s_pxCurrentEffectTemp->m_uNumPlatformOverrides;
		pxPlatformOverrides[iIndex].m_iPlatform = iPlatform; 
		pxPlatformOverrides[iIndex].m_uEffect = ASURA_HASH_ID_UNSET; 

		//and replace
		s_pxCurrentEffectTemp->m_uNumPlatformOverrides++;
		Asura_ResourceDeallocateArray(s_pxCurrentEffectTemp->m_pxPlatformOverrides);
		s_pxCurrentEffectTemp->m_pxPlatformOverrides = pxPlatformOverrides;

		Tools_Entity_PFX_Effect::s_bNeedToRefreshOverlay = true;

		//flag as modified
		SetModified(s_pxCurrentEffectTemp);
	}
	Asura_ResourcePopType();
}

void Tools_Mode_PFX::RefreshParticleTemplateList()
{
	// Check if there's anything to refresh
	if(!s_pxSourcePcleTempDialog) return;

	if(s_pszPcleTempList) delete [] s_pszPcleTempList;

	const Asura_ResourceSetWithOverride<Asura_PFX_Particle_Template>&			xResourceSet	= Asura_PFX_System::GetParticleTempResourceSet();
	const u_int														uNumEntries		= xResourceSet.GetNumResources() + 1;

	Asura_ResourceSetWithOverride_ConstResourceIterator<Asura_PFX_Particle_Template>	xIterator(xResourceSet);

	s_pszPcleTempList = new const char*[uNumEntries];
	s_pszPcleTempList[0] = "None";

	s_pxSourcePcleTempDialog->SelectEntry(0);
	for(u_int u = 1; !xIterator.Done(); xIterator.Next(), u++)
	{
		const Asura_Hash_ID	uHash	= xIterator.GetCurrentHash();
		const Asura_String*	pxName	= Asura_PFX_System::GetParticleTempNameByHash(uHash);
		const char*			szName	= (pxName == NULL) ? NULL : pxName->GetString();

		s_pszPcleTempList[u] = szName;

		if(uHash == s_pxCurrentSourceTemp->m_uParticleTemplateHash)
		{
			s_pxSourcePcleTempDialog->SelectEntry(u);
		}
	}
	s_pxSourcePcleTempDialog->ReplaceStrings(uNumEntries, s_pszPcleTempList);

	const float fNewEditPos = s_pxSourcePcleTempDialog->m_xOffset.x + s_pxSourcePcleTempDialog->m_xMaxCoord.x + 2.f;
	s_pxSourceEditPcleTempDialog->m_pxGadgetList->m_xOffset.x = fNewEditPos;
	s_pxSourceEditPcleTempDialog->m_xOffset.x = fNewEditPos + 2.f;
	s_pxSourceEditPcleTempDialog->m_xMaxCoord.x = fNewEditPos + 6.f;
	s_pxSourceEditPcleTempDialog->m_pxOverlay->ResizeOverlay();
}

void Tools_Mode_PFX::RefreshSourceTemplateList()
{
	// Check if there's anything to refresh
	if(!s_pxEffectSourceTempDialog) return;

	UpdateSourceTempList();
	s_pxEffectSourceTempDialog->ReplaceStrings(s_iSourceTempListSize, s_pszSourceTempList);

	UpdateSourceTempListSelection(s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum].m_uTemplate);
}

void Tools_Mode_PFX::ScaleEffectTime()
{
	if(s_fScalingFactor > 0.f)
	{
		s_pxCurrentEffectTemp->m_fLifeTime /= s_fScalingFactor;
		SetModified(s_pxCurrentEffectTemp);
	}
	for(u_int uSource = 0; uSource < s_pxCurrentEffectTemp->m_uNumSources; uSource++)
	{
		// Scale source properties
		Asura_PFX_Source_Template* pxSourceTemplate = Asura_PFX_System::GetSourceTempByHash(s_pxCurrentEffectTemp->m_pxSources[uSource].m_uTemplate);
		// Check that we haven't done this one already
		bool bDuplicateSource = false;
		for(u_int uPrevSource = 0; uPrevSource < uSource; ++uPrevSource)
		{
			if(s_pxCurrentEffectTemp->m_pxSources[uPrevSource].m_uTemplate == s_pxCurrentEffectTemp->m_pxSources[uSource].m_uTemplate)
			{
				bDuplicateSource = true;
			}
		}
		if(bDuplicateSource)
		{
			// If the source template is the same, the particle template is the same, so scale neither
			continue;
		}
		if(pxSourceTemplate)
		{
			if(!pxSourceTemplate->m_bUseTimeScale)
			{
				pxSourceTemplate->m_bUseTimeScale = true;
				pxSourceTemplate->m_fTimeScale = 1.f;
			}
			pxSourceTemplate->m_fTimeScale *= s_fScalingFactor;
			pxSourceTemplate->m_bModified = true;
		}
	}
}

void Tools_Mode_PFX::ScaleEffect(bool bPreserve)
{
	// Scale effect properties
	s_pxCurrentEffectTemp->m_xPhotonRadius *= s_fScalingFactor;

	for(u_int uSource = 0; uSource < s_pxCurrentEffectTemp->m_uNumSources; uSource++)
	{
		s_pxCurrentEffectTemp->m_pxSources[uSource].m_xOffset *= s_fScalingFactor;
		s_pxCurrentEffectTemp->m_pxSources[uSource].m_xVelocity *= s_fScalingFactor;
		s_pxCurrentEffectTemp->m_pxSources[uSource].m_xVolume *= s_fScalingFactor;
		SetModified(s_pxCurrentEffectTemp);

		// Scale source properties
		Asura_PFX_Source_Template* pxSourceTemplate = Asura_PFX_System::GetSourceTempByHash(s_pxCurrentEffectTemp->m_pxSources[uSource].m_uTemplate);
		// Check that we haven't done this one already
		bool bDuplicateSource = false;
		bool bDuplicateParticle = false;
		for(u_int uPrevSource = 0; uPrevSource < uSource; ++uPrevSource)
		{
			if(s_pxCurrentEffectTemp->m_pxSources[uPrevSource].m_uTemplate == s_pxCurrentEffectTemp->m_pxSources[uSource].m_uTemplate)
			{
				bDuplicateSource = true;
			}
			Asura_PFX_Source_Template* pxPrevSourceTemplate = Asura_PFX_System::GetSourceTempByHash(s_pxCurrentEffectTemp->m_pxSources[uPrevSource].m_uTemplate);
			if(pxPrevSourceTemplate)
			{
				if(pxPrevSourceTemplate->m_uParticleTemplateHash == pxSourceTemplate->m_uParticleTemplateHash)
				{
					bDuplicateParticle = true;
				}
			}
		}
		if(bDuplicateSource)
		{
			// If the source template is the same, the particle template is the same, so scale neither
			continue;
		}
		if(pxSourceTemplate)
		{
			ScaleSource(bPreserve, pxSourceTemplate, s_pxCurrentEffectTemp->m_pxSources[uSource].m_uTemplate, !bDuplicateParticle);
		}
	}
}

void Tools_Mode_PFX::ScaleSource(bool bPreserve, Asura_PFX_Source_Template* pxTemplate, Asura_Hash_ID uHash, bool bScaleParticle /* = true */)
{
	if(bPreserve)
	{
		pxTemplate->m_xGravityVector *= s_fScalingFactor;
		pxTemplate->m_xSpeed *= s_fScalingFactor;
	}
	else
	{
		pxTemplate->m_fAirResistance /= s_fScalingFactor;
		int iMaximumParticles;
		iMaximumParticles = static_cast<int>(pxTemplate->m_uMaxParticles * s_fScalingFactor);
		if(iMaximumParticles > 512)
		{
			const Asura_String* pxName = Asura_PFX_System::GetSourceTempNameByHash(uHash);
			Tools_Timed_Message_System::AddMessage(0xffff00,"Warning: Scaling has increased max particles for %s to %u; clamping to 512", pxName->GetString(), iMaximumParticles);
			iMaximumParticles = 512;
		}
		Tools_Entity_PFX_Effect::ClearAndFlagForRestart_SourceTemplate(pxTemplate->GetTemplateHash());
		pxTemplate->m_uMaxParticles = iMaximumParticles;
	}
	pxTemplate->m_bModified = true;
	if(bScaleParticle)
	{
		// Scale particle properties
		Asura_PFX_Particle_Template* pxParticleTemplate = Asura_PFX_System::GetParticleTempByHash(pxTemplate->m_uParticleTemplateHash);
		if(pxParticleTemplate)
		{
			if(!bPreserve)
			{
				pxParticleTemplate->m_fMinLifetime *= s_fScalingFactor;
				pxParticleTemplate->m_fMaxLifetimeDiff *= s_fScalingFactor;
			}
			pxParticleTemplate->m_fBlurLength *= s_fScalingFactor;
			pxParticleTemplate->m_xSize *= s_fScalingFactor;
			pxParticleTemplate->m_bModified = true;
		}
	}
}

void Tools_Mode_PFX::StartUVEdit()
{
	Asura_Assert(s_pxCurrentParticleTemp, "No particle template");
	if(!s_pxCurrentParticleTemp) return;

	//stop object system mouse contorls intefering
	Tools_Object_System::EnableAllControls(false);
		
	//allocate the uv editing thing
	s_pxParticleTextureEdit = new Tools_UV_Editor;
	const int iTextureID = s_pxCurrentParticleTemp->m_iTextureHandle;
	s_pxParticleTextureEdit->SetTextureID(iTextureID);

	//initalise the uv coordinates
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_PFX_TEXTUREANIM))
	{
		s_pxParticleTextureEdit->Nodes[0].x = s_fTextureAnim_U / s_iTextureAnim_TextureWidth;
		s_pxParticleTextureEdit->Nodes[0].y = s_fTextureAnim_V / s_iTextureAnim_TextureHeight;
		s_pxParticleTextureEdit->Nodes[1].x = (s_fTextureAnim_U +s_fTextureAnim_Width)/ s_iTextureAnim_TextureWidth;
		s_pxParticleTextureEdit->Nodes[1].y = s_fTextureAnim_V / s_iTextureAnim_TextureHeight;
		s_pxParticleTextureEdit->Nodes[2].x = (s_fTextureAnim_U +s_fTextureAnim_Width)/ s_iTextureAnim_TextureWidth;
		s_pxParticleTextureEdit->Nodes[2].y = (s_fTextureAnim_V +s_fTextureAnim_Height)/ s_iTextureAnim_TextureHeight;
		s_pxParticleTextureEdit->Nodes[3].x = s_fTextureAnim_U / s_iTextureAnim_TextureWidth; 
		s_pxParticleTextureEdit->Nodes[3].y = (s_fTextureAnim_V +s_fTextureAnim_Height)/ s_iTextureAnim_TextureHeight; 
	}
	else
	{
		s_pxParticleTextureEdit->Nodes[0].x = s_afParticleTextureUVs[0];
		s_pxParticleTextureEdit->Nodes[0].y = s_afParticleTextureUVs[1];
		s_pxParticleTextureEdit->Nodes[1].x = s_afParticleTextureUVs[2];
		s_pxParticleTextureEdit->Nodes[1].y = s_afParticleTextureUVs[1];
		s_pxParticleTextureEdit->Nodes[2].x = s_afParticleTextureUVs[2];
		s_pxParticleTextureEdit->Nodes[2].y = s_afParticleTextureUVs[3];
		s_pxParticleTextureEdit->Nodes[3].x = s_afParticleTextureUVs[0]; 
		s_pxParticleTextureEdit->Nodes[3].y = s_afParticleTextureUVs[3]; 
	}

	s_pxParticleTextureEdit->CreateMouseModeOverlay();

}							  

void Tools_Mode_PFX::FinishUVEdit()
{
	if(s_pxParticleTextureEdit)
	{
		s_pxParticleTextureEdit->CloseMouseModeOverlay();
		delete s_pxParticleTextureEdit;
		s_pxParticleTextureEdit = 0;
		Tools_Object_System::EnableAllControls(true);
		Asura_Overlay_System::CloseOverlays(TOOLS_OVERLAY_PFX_TEXTUREANIM);
	}
}

void Tools_Mode_PFX::UpdateUVEdit()
{
	if(!s_pxParticleTextureEdit) return;

	s_pxParticleTextureEdit->HandleMouse();

	bool bDoingTextureAnim = Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_PFX_TEXTUREANIM);

	if(s_pxParticleTextureEdit->NodesMoved)
	{
		//make sure the nodes stay in rectangular formation
		s_pxParticleTextureEdit->EnsureNodesAreStillOkayForTextureWrap();
		
		s_pxParticleTextureEdit->Nodes[1].x = s_pxParticleTextureEdit->Nodes[2].x;
		s_pxParticleTextureEdit->Nodes[1].y = s_pxParticleTextureEdit->Nodes[0].y;

		s_pxParticleTextureEdit->Nodes[3].x = s_pxParticleTextureEdit->Nodes[0].x;
		s_pxParticleTextureEdit->Nodes[3].y = s_pxParticleTextureEdit->Nodes[2].y;


		s_pxParticleTextureEdit->NodesMoved = false;


		if(bDoingTextureAnim)
		{
			s_fTextureAnim_U = s_pxParticleTextureEdit->Nodes[0].x*s_iTextureAnim_TextureWidth;
			s_fTextureAnim_V = s_pxParticleTextureEdit->Nodes[0].y*s_iTextureAnim_TextureHeight;
			s_fTextureAnim_Width = (s_pxParticleTextureEdit->Nodes[2].x - s_pxParticleTextureEdit->Nodes[0].x) * s_iTextureAnim_TextureWidth; 
			s_fTextureAnim_Height = (s_pxParticleTextureEdit->Nodes[2].y - s_pxParticleTextureEdit->Nodes[0].y) * s_iTextureAnim_TextureHeight; 
		}
		else
		{
			s_afParticleTextureUVs[0] = s_pxParticleTextureEdit->Nodes[0].x; 
			s_afParticleTextureUVs[1] = s_pxParticleTextureEdit->Nodes[0].y; 
			s_afParticleTextureUVs[2] = s_pxParticleTextureEdit->Nodes[2].x; 
			s_afParticleTextureUVs[3] = s_pxParticleTextureEdit->Nodes[2].y; 


			s_bParticleTempChanged = true;
		}
	}
	else
	{
		//make sure the uv coordinates being edited match those on the overlay
		if(bDoingTextureAnim)
		{
			s_pxParticleTextureEdit->Nodes[0].x = s_fTextureAnim_U / s_iTextureAnim_TextureWidth;
			s_pxParticleTextureEdit->Nodes[0].y = s_fTextureAnim_V / s_iTextureAnim_TextureHeight;
			s_pxParticleTextureEdit->Nodes[1].x = (s_fTextureAnim_U +s_fTextureAnim_Width)/ s_iTextureAnim_TextureWidth;
			s_pxParticleTextureEdit->Nodes[1].y = s_fTextureAnim_V / s_iTextureAnim_TextureHeight;
			s_pxParticleTextureEdit->Nodes[2].x = (s_fTextureAnim_U +s_fTextureAnim_Width)/ s_iTextureAnim_TextureWidth;
			s_pxParticleTextureEdit->Nodes[2].y = (s_fTextureAnim_V +s_fTextureAnim_Height)/ s_iTextureAnim_TextureHeight;
			s_pxParticleTextureEdit->Nodes[3].x = s_fTextureAnim_U / s_iTextureAnim_TextureWidth; 
			s_pxParticleTextureEdit->Nodes[3].y = (s_fTextureAnim_V +s_fTextureAnim_Height)/ s_iTextureAnim_TextureHeight; 
		}
		else
		{
			s_pxParticleTextureEdit->Nodes[0].x = s_afParticleTextureUVs[0]; 
			s_pxParticleTextureEdit->Nodes[0].y = s_afParticleTextureUVs[1]; 
			s_pxParticleTextureEdit->Nodes[1].x = s_afParticleTextureUVs[2]; 
			s_pxParticleTextureEdit->Nodes[1].y = s_afParticleTextureUVs[1]; 
			s_pxParticleTextureEdit->Nodes[2].x = s_afParticleTextureUVs[2]; 
			s_pxParticleTextureEdit->Nodes[2].y = s_afParticleTextureUVs[3]; 
			s_pxParticleTextureEdit->Nodes[3].x = s_afParticleTextureUVs[0]; 
			s_pxParticleTextureEdit->Nodes[3].y = s_afParticleTextureUVs[3]; 
		}
	}
}

void Tools_Mode_PFX::RenderUVEdit()
{
	if(!s_pxParticleTextureEdit) return;

	bool bDoingTextureAnim = Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_PFX_TEXTUREANIM);

	if(bDoingTextureAnim && s_uTextureAnim_Frames>1)
	{
		//split up the rendering so we can add in lines for the extra frames as well. RAR 13/6/2006
		s_pxParticleTextureEdit->RenderTexture();
		s_pxParticleTextureEdit->RenderLines();
	
		//determine the UVs of each frame
		float fU = s_fTextureAnim_U;
		float fV = s_fTextureAnim_V;

		for(u_int i=0;i<s_uTextureAnim_Frames;i++)
		{
			u_int uColour = 0xffafafaf;
			if(i)
			{
				float fLeft = fU/s_iTextureAnim_TextureWidth;		
				float fTop = fV/s_iTextureAnim_TextureHeight;
				float fRight = (fU+s_fTextureAnim_Width) / s_iTextureAnim_TextureWidth;		
				float fBottom = (fV+s_fTextureAnim_Height) / s_iTextureAnim_TextureHeight;

				Asura_Vector_2 xCorner[4];
				xCorner[0].x = fLeft;
				xCorner[0].y = fTop;
				xCorner[1].x = fRight;
				xCorner[1].y = fTop;
				xCorner[2].x = fRight;
				xCorner[2].y = fBottom;
				xCorner[3].x = fLeft;
				xCorner[3].y = fBottom;
				
				for(int j=0;j<4;j++)
				{
					s_pxParticleTextureEdit->RenderLine(xCorner[j],xCorner[(j+1)%4],uColour);
				}

			}

			//find the start of the next frame
			fU += s_fTextureAnim_Width;
			if((fU + s_fTextureAnim_Width) > (float)s_iTextureAnim_TextureWidth)
			{
				//start on next row
				fU = 0.0f;
				fV += s_fTextureAnim_Height;
			}
		}
		
	
		s_pxParticleTextureEdit->RenderNodes();
		s_pxParticleTextureEdit->RenderSelectionBox();


	}
	else
	{
		s_pxParticleTextureEdit->Render();
	}
}

void Tools_Mode_PFX::ReloadParticleTexture()
{
	Asura_Assert(s_pxCurrentParticleTemp, "No particle template");
	if(!s_pxCurrentParticleTemp) { return; }

	Asura_Texture_Management::ReloadTexture(s_pxCurrentParticleTemp->m_iTextureHandle);
}

void Tools_Mode_PFX::ChooseParticleTexture()
{
	Asura_Assert(s_pxCurrentParticleTemp, "No particle template");
	if(!s_pxCurrentParticleTemp) { return; }

	// UV edit texture will be incorrect after this, so close it.
	FinishUVEdit();

	// Use Tools_PickTexture to get our new texture.
	const int iOldTextureID = s_pxCurrentParticleTemp->m_iTextureHandle;
	const int iNewTextureID = Tools_PickTexture::PickTexture("Choose texture for particle", iOldTextureID, PICKTEXTURE_ALLTEXTURES);
	const Asura_String* pxNewTextureName = Asura_Texture_Management::GetTextureName(iNewTextureID);
	Asura_Assert(pxNewTextureName, "No texture name");
	if(!pxNewTextureName) { return; }

	// Update particle template properties
	s_pxCurrentParticleTemp->m_xTextureName = *pxNewTextureName;
	s_pxCurrentParticleTemp->m_iTextureHandle = iNewTextureID;

	if( s_pxSourceTextureNameDialog )	// Refresh also texture name.
	{
		s_pxSourceTextureNameDialog->SetName( s_pxCurrentParticleTemp->m_xTextureName.GetString() );
	}

	SetModified(s_pxCurrentParticleTemp);
}

void Tools_Mode_PFX::InitialiseParticleTextureAnim()
{
	//Overlay already exists?
	if(Asura_Overlay_System::DoesOverlayExist(TOOLS_OVERLAY_PFX_TEXTUREANIM)) return;

	//make sure other UV editing isn't already in progress
	FinishUVEdit();

	//Initialise values from the first frame
	Asura_Assert(s_pxCurrentParticleTemp,"");

	s_iTextureAnim_TextureWidth = 256;
	s_iTextureAnim_TextureHeight = 256;
	Tools_Texture_Management::GetTextureSize(s_pxCurrentParticleTemp->m_iTextureHandle,s_iTextureAnim_TextureWidth,s_iTextureAnim_TextureHeight);

	Asura_PFX_TextureFrame* pxTextureFrame = &s_pxCurrentParticleTemp->m_pxTextureFrames[0];
	s_fTextureAnim_U = pxTextureFrame->m_afTextureUVs[0]*s_iTextureAnim_TextureWidth;
	s_fTextureAnim_V = pxTextureFrame->m_afTextureUVs[1]*s_iTextureAnim_TextureHeight;
	s_fTextureAnim_Width = (pxTextureFrame->m_afTextureUVs[2]-pxTextureFrame->m_afTextureUVs[0])*s_iTextureAnim_TextureWidth;
	s_fTextureAnim_Height = (pxTextureFrame->m_afTextureUVs[3]-pxTextureFrame->m_afTextureUVs[1])*s_iTextureAnim_TextureHeight;
	s_uTextureAnim_Frames = s_pxCurrentParticleTemp->m_uNumTextureFrames;
	
	//Create the overlay
	Asura_Overlay* pxOverlay = new Asura_Overlay("Initialise Anim",false,&s_xEditTextureAnimPos);

	Asura_Dialog* pxDialog;
	Asura_Vector_2 xOffset(1.0f,3.0f);

	pxDialog = new Asura_Numerical_Dialog("Left",xOffset,&s_fTextureAnim_U,0.0f,(float)s_iTextureAnim_TextureWidth,1.0f,"%.1f");
	pxDialog->SetHelpText("Left of first frame in pixels");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Top",xOffset,&s_fTextureAnim_V,0.0f,(float)s_iTextureAnim_TextureHeight,1.0f,"%.1f");
	pxDialog->SetHelpText("Top of first frame in pixels");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();
	
	pxDialog = new Asura_Numerical_Dialog("Width",xOffset,&s_fTextureAnim_Width,0.0f,(float)s_iTextureAnim_TextureWidth,1.0f,"%.1f");
	pxDialog->SetHelpText("Width of each frame in pixels");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Height",xOffset,&s_fTextureAnim_Height,0.0f,(float)s_iTextureAnim_TextureHeight,1.0f,"%.1f");
	pxDialog->SetHelpText("Height of each frame in pixels");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::DefaultSpacing();

	pxDialog = new Asura_Numerical_Dialog("Num Frames",xOffset,&s_uTextureAnim_Frames,1,99,1,"%d");
	pxDialog->SetHelpText("Number of frames in animation");
	pxOverlay->AddDialog(pxDialog);
	xOffset.y += Asura_Dialog::LargeSpacing();

	pxDialog = new Asura_Function_Dialog("Okay",xOffset,TextureAnimOverlay_Okay);
	pxDialog->SetHelpText("Accepts these changes and closes the overlay");
	pxOverlay->AddDialog(pxDialog);

	xOffset.x = pxDialog->m_xMaxCoord.x + 1.0f;
	pxDialog = new Asura_Function_Dialog("Cancel",xOffset,TextureAnimOverlay_Cancel);
	pxDialog->SetHelpText("Cancels these changes and closes the overlay");
	pxOverlay->AddDialog(pxDialog);

	pxOverlay->m_uID = TOOLS_OVERLAY_PFX_TEXTUREANIM;
	Asura_Overlay_System::SubmitNewOverlay(pxOverlay,false);


	//Start UV editing as well
	StartUVEdit();

}

void Tools_Mode_PFX::TextureAnimOverlay_Okay()
{
	Asura_Assert(s_pxCurrentParticleTemp,"");
	FinishUVEdit();

	//make sure there is room for the frames
	if(s_uTextureAnim_Frames > s_pxCurrentParticleTemp->m_uNumTextureFrames)
	{
		Asura_ResourcePushType(ASURA_RESOURCE_PFX);
		{
			//need to allocate some extra storage space
			Asura_PFX_TextureFrame* pxTextureFrames;
			Asura_ResourceAllocateArray(pxTextureFrames, s_uTextureAnim_Frames);

			//replace old array
			Asura_ResourceDeallocateArray(s_pxCurrentParticleTemp->m_pxTextureFrames);
			s_pxCurrentParticleTemp->m_pxTextureFrames = pxTextureFrames;
		}
		Asura_ResourcePopType();
	}
	s_pxCurrentParticleTemp->m_uNumTextureFrames = s_uTextureAnim_Frames;

	//Fill in the frames
	float fU = s_fTextureAnim_U;
	float fV = s_fTextureAnim_V;

	Asura_PFX_TextureFrame* pxTextureFrame = s_pxCurrentParticleTemp->m_pxTextureFrames;
	for(u_int i=0;i<s_uTextureAnim_Frames;i++,pxTextureFrame++)
	{
		pxTextureFrame->m_afTextureUVs[0] = fU/s_iTextureAnim_TextureWidth;		
		pxTextureFrame->m_afTextureUVs[1] = fV/s_iTextureAnim_TextureHeight;
		pxTextureFrame->m_afTextureUVs[2] = (fU+s_fTextureAnim_Width) / s_iTextureAnim_TextureWidth;		
		pxTextureFrame->m_afTextureUVs[3] = (fV+s_fTextureAnim_Height) / s_iTextureAnim_TextureHeight;
		pxTextureFrame->m_fTime = (float)i/(float) s_uTextureAnim_Frames;

		//find the start of the next frame
		fU += s_fTextureAnim_Width;
		if((fU + s_fTextureAnim_Width) > (float)s_iTextureAnim_TextureWidth)
		{
			//start on next row
			fU = 0.0f;
			fV += s_fTextureAnim_Height;
		}
	}

	//Flag the frame number as being changed so that the main overlay gets updated
	s_uParticleCurrentTextureFrame = 0;
	s_bParticleCurrentTextureFrameChanged = true;
	s_bParticleTempChanged = true;
		
}

void Tools_Mode_PFX::TextureAnimOverlay_Cancel()
{
	FinishUVEdit();
}


bool Tools_Mode_PFX::CloseEditParticleTempIfNecessary(Asura_Hash_ID uHash)
{
	if(s_pxCurrentParticleTemp == Asura_PFX_System::GetParticleTempByHash(uHash))
	{
		// We are currently editing this template; close it and return true
		CloseEditParticleTemplate();
		return true;
	}
	return false;
}

bool Tools_Mode_PFX::CloseEditSourceTempIfNecessary(Asura_Hash_ID uHash)
{
	if(s_pxCurrentSourceTemp == Asura_PFX_System::GetSourceTempByHash(uHash))
	{
		// We are currently editing this template; close it and return true
		CloseEditSourceTemplate();
		return true;
	}
	return false;
}

bool Tools_Mode_PFX::CloseEditEffectTempIfNecessary(Asura_Hash_ID uHash)
{
	if(s_pxCurrentEffectTemp == Asura_PFX_System::GetEffectTempByHash(uHash))
	{
		// We are currently editing this template; close it and return true
		CloseEditEffectTemplate();
		return true;
	}
	return false;
}

void Tools_Mode_PFX::SetParticleEffectBeingRenderedIsSelected(bool bSet)
{
	s_bParticleEffectBeingRenderedIsSelected = bSet;
}

void Tools_Mode_PFX::AddToParticlesRendered(u_int uNumParticles)
{
	s_uTotalParticlesRendered += uNumParticles;
	if(s_bParticleEffectBeingRenderedIsSelected)
	{
		s_uEffectParticlesRendered += uNumParticles;
	}
}

bool Tools_Mode_PFX::AddParticleTemplateToExportListByHash(List<Asura_String>* pxList, Asura_Hash_ID uHash, const bool bTexturesOnly)
{
	if(uHash == ASURA_HASH_ID_UNSET) return false;

	//skip this for fake level exports
	if(Tools_Level_Export::IsDoingFakeLevelExport())
	{
		//But get the template to force it to be loaded
		Asura_PFX_System::GetParticleTempByHash(uHash);	
		return(true);
	}

	Asura_PFX_Particle_Template* pxParticleTemplate = Asura_PFX_System::GetParticleTempByHash(uHash);
	if(!bTexturesOnly)
	{
		//generate template in LevelExportTemp
		const Asura_String* pxTempName = Asura_PFX_System::GetParticleTempNameByHash(uHash);
		Asura_Assert(pxTempName,"Failed to get name for particle template");

		Asura_String xExportFileName;
		GetParticleTempFileName(pxTempName, &xExportFileName, TOOLS_MODE_PFX_SAVE_EXPORT);
		SaveParticleTempByHash(uHash, TOOLS_MODE_PFX_SAVE_EXPORT);

		//and add to list
		Tools_Level_Export::AddFileToExportList(*pxList, xExportFileName, false);
   
	}

	//Add texture used by the particle
	if(pxParticleTemplate)
	{
		const int iTextureID = Asura_Texture_Management::GetTextureIndex(pxParticleTemplate->m_xTextureName);
		if(iTextureID != ASURA_TEXTURE_NONE)
		{
			Asura_Texture_Management::AddTextureFlags(iTextureID,AsuraTextureFlag_8BitAlpha|s_iProjectTextureFlags);
			Tools_Level_Export::AddTextureToExportList(iTextureID);
		}
	}


	return(true);
}

bool Tools_Mode_PFX::AddTemplateToExportListByHash(List<Asura_String>* pxList, Asura_Hash_ID uHash, Asura_Guid uGuid)
{
	if(uHash == ASURA_HASH_ID_UNSET) return false;

#ifdef ASURA_USING_ALL_LEVEL_EXPORT
	// DF 19/04/07 -
	// If PFX has been added to the ignore list from the common_export file, don't export it.
	int iJunk;
	if ( m_pxPFXToIgnoreList->FindNodeIndexByHash(uHash, false, iJunk) )
	{
		return false;
	}
#endif //ASURA_USING_ALL_LEVEL_EXPORT

	if (s_bDontIncludePFXInExport)
	{
		// Skipping PFX files.
		return(false);
	}

	// Get effect template name
	const Asura_String* pxEffectTempName = Asura_PFX_System::GetEffectTempNameByHash(uHash);
	if (pxEffectTempName)
	{
		if(!Asura_PFX_System::IsEffectTempLoadedByHash(uHash))
		{
			Asura_PFX_System::LoadTemplateByHash(uHash);
		}
		// Get effect template
		const Asura_PFX_Effect_Template* pxEffectTemp = Asura_PFX_System::GetEffectTempByHash(uHash);
		Asura_Assert(pxEffectTemp, "Got effect template name but not template itself");
		if (pxEffectTemp)
		{
			//get the effect temp that will be used for this export type
			const Asura_PFX_Effect_Template* pxPlatformEffectTemp = pxEffectTemp->GetTemplateForPlatform(Tools_Level_Export::s_eExportType);

			bool bAdded = false;

			//No need to bother with generating the files in LevelExportTemp when quitting to game
			if(!Tools_Level_Export::IsDoingFakeLevelExport())
			{
				// Iterate through source templates
				for(u_int uSource = 0; uSource < pxPlatformEffectTemp->m_uNumSources; ++uSource)
				{
					if( !AddSourceTemplateToExportListByHash(pxList, pxPlatformEffectTemp->m_pxSources[uSource].m_uTemplate, uGuid, true) )
					{
						Tools_Level_Export::AddMessage( Tools_Level_Export::MT_WARNING,
							"Warning: PFX effect with GUID %u is using invalid effect template '%s'.",
							uGuid, pxEffectTempName->GetString() );
					}
				}

				// Add effect template
				//(passing the non-platform-overridden version since it needs to be dealt
				//with in SaveEffectTemp so it gets saved with the subsituted hash id)
				Asura_String xEffectFilename;
				GetTemplateFileName(pxEffectTempName, &xEffectFilename, TOOLS_MODE_PFX_SAVE_EXPORT);
				SaveTemplateByHash(uHash, TOOLS_MODE_PFX_SAVE_EXPORT);
				bAdded = Tools_Level_Export::AddFileToExportList(*pxList, xEffectFilename, false);
			}
			else
			{
				// Ensure all overrides are loaded.

				// Iterate through sources checking for overrides and building a list of potential sources.
				Asura_TemplateHashTree<u_int> xSourceTemplatesRequired;
				for(u_int uSource = 0; uSource < pxPlatformEffectTemp->m_uNumSources; ++uSource)
				{
					const Asura_Hash_ID uSourceHash = pxPlatformEffectTemp->m_pxSources[uSource].m_uTemplate;
					// Add the base source hash to the list.
					*xSourceTemplatesRequired.AddOrFindNode(uSourceHash) += 1;
					for(Asura_PFX_SourceTemplateResourceSet_ConstMapIterator xMapIter(Asura_PFX_System::GetSourceTempResourceSet()); !xMapIter.Done(); xMapIter.Next())
					{
						const Asura_PFX_SourceTemplateOverrideMap& xMap = *xMapIter.GetCurrent();
						const Asura_PFX_SourceTemplateOverrideEntry* pxEntry = xMap.GetOverrideEntry(uSourceHash);
						if(pxEntry)
						{
							// If there's an override, add it to the list, and load it if necessary.
							const u_int uNewSourceHash = pxEntry->GetHash();
							*xSourceTemplatesRequired.AddOrFindNode(uNewSourceHash) += 1;
							if(!Asura_PFX_System::GetSourceTempByHash(uNewSourceHash))
							{
								Asura_PFX_System::LoadEffectTemplateByOverrideName(pxEntry->GetNewName());
							}
						}
					}
				}
				// Iterate through list of potential sources checking for particle overrides.
				for (Asura_TemplateHashTree_ConstIterator<u_int> xSource(xSourceTemplatesRequired); !xSource.Done(); xSource.Next())
				{
					const Asura_PFX_Source_Template* pxSourceTemp = Asura_PFX_System::GetSourceTempByHash(xSource.GetCurrentHash(), true);
					if (!pxSourceTemp) { continue; }

					const Asura_Hash_ID uParticleHash = pxSourceTemp->m_uParticleTemplateHash;
					for(Asura_PFX_ParticleTemplateResourceSet_ConstMapIterator xMapIter(Asura_PFX_System::GetParticleTempResourceSet()); !xMapIter.Done(); xMapIter.Next())
					{
						const Asura_PFX_ParticleTemplateOverrideMap& xMap = *xMapIter.GetCurrent();
						const Asura_PFX_ParticleTemplateOverrideEntry* pxEntry = xMap.GetOverrideEntry(uParticleHash);
						if(pxEntry)
						{
							// If there's an override, load it if necessary.
							if(!Asura_PFX_System::GetParticleTempByHash(pxEntry->GetHash()))
							{
								Asura_PFX_System::LoadEffectTemplateByOverrideName(pxEntry->GetNewName());
							}
						}
					}
				}
			}

			//add any sounds used by the effect
			if(bAdded || Tools_Level_Export::IsDoingFakeLevelExport())
			{
				TOOLS_RESOURCEEXPORTLOG("PFX", pxPlatformEffectTemp->m_xName.GetString(), bAdded);

				if(pxPlatformEffectTemp->m_uSoundEvent)
				{
					Tools_Level_Export::AddSoundEventToExportList(pxPlatformEffectTemp->m_uSoundEvent, pxPlatformEffectTemp->m_xName.GetString());
				}

				for(u_int uSource = 0; uSource < pxPlatformEffectTemp->m_uNumSources; ++uSource)
				{
					const Asura_PFX_Source_Template* pxSourceTemp = Asura_PFX_System::GetSourceTempByHash(pxPlatformEffectTemp->m_pxSources[uSource].m_uTemplate);
					if(!pxSourceTemp) continue;

					const Asura_PFX_Particle_Template* pxParticleTemp = Asura_PFX_System::GetParticleTempByHash(pxSourceTemp->m_uParticleTemplateHash);
					if(!pxParticleTemp) continue;
					
					if(pxParticleTemp->m_uSoundEvent)
					{
						const Asura_String* pxParticleName = Asura_PFX_System::GetParticleTempNameByHash(pxSourceTemp->m_uParticleTemplateHash);
						Asura_Assert(pxParticleName, "Failed to get particle template name");

						Tools_Level_Export::AddSoundEventToExportList(pxParticleTemp->m_uSoundEvent, pxParticleName->GetString());
					}
				}
			}
		}
	}
	return(true);
}

bool Tools_Mode_PFX::AddSourceTemplateToExportListByHash( List<Asura_String>* pxList, Asura_Hash_ID uSourceTemplateHashID, Asura_Guid uGUID, const bool bTexturesOnly )
{
    bool bOK = true;

	// Get source template name
	const Asura_String* pxSourceTempName = Asura_PFX_System::GetSourceTempNameByHash( uSourceTemplateHashID );
	if( pxSourceTempName )
	{
	    // Get source template
	    const Asura_PFX_Source_Template* pxSourceTemp = Asura_PFX_System::GetSourceTempByHash( uSourceTemplateHashID );
	    Asura_Assert(pxSourceTemp, "Got source template name but not template itself");
	    if(pxSourceTemp)
	    {
            // Add particle template.
            if( !AddParticleTemplateToExportListByHash(pxList, pxSourceTemp->m_uParticleTemplateHash, bTexturesOnly) )
            {
                Tools_Level_Export::AddMessage( Tools_Level_Export::MT_WARNING,
                                                "Warning: PFX source used by GUID %u is using invalid particle template with hash ID 0x%08X.",
                                                uGUID, pxSourceTemp->m_uParticleTemplateHash );				
                bOK = false;
            }
	    }

		if(!bTexturesOnly)
		{
			// Add source template
			Asura_String xSourceFilename;
			GetSourceTempFileName(pxSourceTempName, &xSourceFilename, TOOLS_MODE_PFX_SAVE_EXPORT);
			SaveSourceTempByHash(uSourceTemplateHashID, TOOLS_MODE_PFX_SAVE_EXPORT );
			Tools_Level_Export::AddFileToExportList(*pxList, xSourceFilename, false);
		}
    }
    else
    {
		Tools_Level_Export::AddMessage( Tools_Level_Export::MT_WARNING,
                                        "Warning: PFX source used by GUID %u is using invalid source template.",
                                        uGUID );
        bOK = false;
    }

    return bOK;
}

void Tools_Mode_PFX::AddAlwaysExportParticleTemplates(List<Asura_String>* pxList)
{
	//should only be done during level exports (fake or otherwise)
	Asura_Assert(Tools_Level_Export::IsLevelExport(),"");

	//Load ini file , and add all templates listed
	const char* pszIniName = "ToolsIniFiles\\AlwaysExportPFX.ini";
	Tools_IniFile xIni(pszIniName);
	if(!xIni.LoadIniFile())
	{
		Tools_Level_Export::AddMessage(Tools_Level_Export::MT_WARNING,"Failed to load %s",pszIniName);
		return;
	}
	Tools_Level_Export::AddMessage(Tools_Level_Export::MT_LOGONLY,"Loaded %s",pszIniName);

	while(xIni.GetNextIniTextLine())
	{
		Asura_String xFilename;
		if(!xIni.GetString(xFilename)) continue;

		//split filename
		Asura_String xTemplateName,xExtension;
		xFilename.SplitFilename(0,&xTemplateName,&xExtension);

		Asura_Hash_ID uHash = Asura_GetHashID(xTemplateName.GetString());

		if(!xExtension.Compare("pfx"))
		{
			if(!AddTemplateToExportListByHash(pxList, uHash, ASURA_GUID_UNREGISTERED))
			{
				Tools_Level_Export::AddMessage(Tools_Level_Export::MT_ERROR,"Failed to find template '%s'",xFilename.GetString());
			}
		}
		else
		{
			Tools_Level_Export::AddMessage(Tools_Level_Export::MT_ERROR,"Unexpected file type %s in %s",xFilename.GetString(),pszIniName);
		}
				
	}	
}

void Tools_Mode_PFX::EditNumParticleTextureFrames()
{
	if(!s_pxCurrentParticleTemp) return;

	//ask for the new number of frames
	GetNumber_Overlay xOverlay("Set new number of texture frames",(float)s_pxCurrentParticleTemp->m_uNumTextureFrames,1.0f,99.0f,"%.0f");
	if(!xOverlay.DoModal()) return;

	u_int uNumFrames = (u_int) xOverlay.GetNumber();

	if(uNumFrames > s_pxCurrentParticleTemp->m_uNumTextureFrames)
	{
		Asura_ResourcePushType(ASURA_RESOURCE_PFX);
		{
			//need to allocate some extra storage space
			Asura_PFX_TextureFrame* pxTextureFrames;
			Asura_ResourceAllocateArray(pxTextureFrames, uNumFrames);
			//copy old values
			for(u_int u =0;u<s_pxCurrentParticleTemp->m_uNumTextureFrames;u++)
			{
				pxTextureFrames[u] = s_pxCurrentParticleTemp->m_pxTextureFrames[u];
			}
			//initialise new frames by copying the previous ones
			for(u_int u = s_pxCurrentParticleTemp->m_uNumTextureFrames;u<uNumFrames;u++)
			{
				pxTextureFrames[u] = pxTextureFrames[u-1];
				pxTextureFrames[u].m_fTime = 1.0f;
			}

			//replace old array
			Asura_ResourceDeallocateArray(s_pxCurrentParticleTemp->m_pxTextureFrames);
			s_pxCurrentParticleTemp->m_pxTextureFrames = pxTextureFrames;
		}
		Asura_ResourcePopType();
	}
	s_pxCurrentParticleTemp->m_uNumTextureFrames = uNumFrames;

	//make sure current frame is in range
	if(s_uParticleCurrentTextureFrame >= uNumFrames)
	{
		s_uParticleCurrentTextureFrame = 0;
		s_bParticleCurrentTextureFrameChanged = true;
	}


	s_bParticleTempChanged = true;
}

bool Tools_Mode_PFX::CanEditParticleTextureFrameTime()
{
	return s_uParticleCurrentTextureFrame!=0;
}

bool Tools_Mode_PFX::CanEditParticleTextureFrame()
{
    // (MH - 12.01.2006) note this is also used by tickbox for crossfading frames.
	if(!s_pxCurrentParticleTemp) return false;
	return s_pxCurrentParticleTemp->m_uNumTextureFrames>1;
}


void Tools_Mode_PFX::CalculateMaxParticles(Asura_PFX_Source_Template* pxSourceTemp,bool bQuiet)
{
	//This doesn't apply to sources with burst generation
	if(pxSourceTemp->m_bBurst)
	{
		return;
	}

	u_int uMaxParticles = 0;
	//must have a particle template assigned
	Asura_PFX_Particle_Template* pxPartTemplate = Asura_PFX_System::GetParticleTempByHash(pxSourceTemp->m_uParticleTemplateHash);
	if(pxPartTemplate)
	{
		//get the particle duration , taking the average if a variable duration is used
		float fParticleDuration = (pxPartTemplate->m_fMinLifetime + pxPartTemplate->GetMaxLifetime())/2.0f;

		//Get a list of all effects using this source.
		//We will need to calculatie the max particles for each one , and take the
		//highest overall.

		List<const Asura_PFX_Effect_Template*> xEffectList;
		bool bUsedByLoopingEffect = false;
		Asura_Hash_ID uSourceHash = pxSourceTemp->GetTemplateHash();

		const Asura_ResourceSetWithOverride<Asura_PFX_Effect_Template>&	xResourceSet	= Asura_PFX_System::GetEffectTempResourceSet();
		Asura_ResourceSetWithOverride_ConstResourceIterator<Asura_PFX_Effect_Template>		xIterator(xResourceSet);

		while(!xIterator.Done())
		{
			const Asura_Hash_ID	uHash	= xIterator.GetCurrentHash();

			//Only check effects that have been loaded , since I don't really want to
			//force all of them to be loaded at this point.
			//Hopefully in practice , the source will have been loaded as a result of needing
			//(and loading) the effect. So the appropriate effect template will be loaded already
			//RAR 25/10/2005
			if(Asura_PFX_System::IsEffectTempLoadedByHash(uHash))
			{
				const Asura_PFX_Effect_Template*	pxEffect	= xIterator.GetCurrent();

				if(pxEffect != NULL)
				{
					//check sources for this effect
					for(u_int j=0;j<pxEffect->m_uNumSources;j++)
					{
						if(pxEffect->m_pxSources[j].m_uTemplate == uSourceHash)
						{
							xEffectList.add_entry(pxEffect);
							if(pxEffect->m_bLooping)
							{
								bUsedByLoopingEffect = true;
							}
							break;
						}
						const Asura_ResourceSetWithOverride<Asura_PFX_Source_Template>&	xSourceResourceSet	= Asura_PFX_System::GetSourceTempResourceSet();
						Asura_ResourceSetWithOverride_ConstOverrideMapIterator<Asura_PFX_Source_Template> xMapIter(xSourceResourceSet);
						for(;!xMapIter.Done();xMapIter.Next())
						{
							Asura_Hash_ID uNewHash = xMapIter.GetCurrent()->GetOverride(pxEffect->m_pxSources[j].m_uTemplate);
							if (uNewHash != pxEffect->m_pxSources[j].m_uTemplate)
							{
								if(uSourceHash == uNewHash)
								{
									xEffectList.add_entry(pxEffect);
									if(pxEffect->m_bLooping)
									{
										bUsedByLoopingEffect = true;
									}
									break;
								}
							}
						}

					}
				}
			}
			xIterator.Next();
		}

		//Determine how long we need to simulate effect
		//If looping , need to loop enough for a particle created at the end of one loop to expire
		float fSimDuration = pxSourceTemp->m_fDuration;
		if(bUsedByLoopingEffect) fSimDuration += fParticleDuration;


		const float fSimInterval = 0.04f;
		int iSimSteps = Asura_Maths::FloatToInt_Near(fSimDuration/fSimInterval);


		//try each effect and get the maximum particles needed for each
		float fMaxParticles = 0.0f;
		float* pfParticleCounts = new float[iSimSteps];
		for(LIF<const Asura_PFX_Effect_Template*> xEffectLIF(&xEffectList);!xEffectLIF.done();xEffectLIF.next())
		{	
			//get history of particle counts
			DetermineParticleCountsByTime(xEffectLIF(),pxSourceTemp,fSimInterval,iSimSteps,pfParticleCounts);
		
			//get max particle count
			for(int i=0;i<iSimSteps;i++)
			{
				fMaxParticles = Asura_Maths::Max(fMaxParticles,pfParticleCounts[i]);
			}
		}
		delete [] pfParticleCounts;
		
	
		uMaxParticles = Asura_Maths::FloatToInt_Near(fMaxParticles);
		if(uMaxParticles<1) uMaxParticles = 1;

		if(!bQuiet)
		{
			//show overlay warning if first time over limit
			if(uMaxParticles > TOOLS_PFX_MAX_PARTICLE_WARNING_LEVEL && !pxSourceTemp->m_bMaxParticleWarning)
			{
				ShowMessageOverlay("Warning.\nSource %s uses %d particles.\n(%d K per instance)",pxSourceTemp->m_xName.GetString(),uMaxParticles,uMaxParticles*sizeof(Asura_PFX_Particle)/1024);
				pxSourceTemp->m_bMaxParticleWarning = true;
			}
			//also show warning at bottom of screen if particles have increased
			if(uMaxParticles > TOOLS_PFX_MAX_PARTICLE_WARNING_LEVEL && uMaxParticles > pxSourceTemp->m_uMaxParticles)
			{
				Tools_Timed_Message_System::AddMessage(0xffff00,"Source %s uses %d particles.", pxSourceTemp->m_xName.GetString(),uMaxParticles);
			}
		}
	}

	if(uMaxParticles != pxSourceTemp->m_uMaxParticles)
	{
		//Force a reset on all playing particle effects to avoid potential crashes
		Tools_Entity_PFX_Effect::ClearAndFlagForRestart_SourceTemplate(pxSourceTemp->GetTemplateHash());
		
		//update maximum particles from our calculated values
		pxSourceTemp->m_uMaxParticles = uMaxParticles;
	}
}

void Tools_Mode_PFX::DetermineParticleCountsByTime(const Asura_PFX_Effect_Template* pxEffectTemp,const Asura_PFX_Source_Template* pxSourceTemp,const float fSimInterval,const int iSimSteps,float* pfParticleCounts,int iRecursionCount)
{
	//initialise to zeroes
	memset(pfParticleCounts,0,iSimSteps*sizeof(float));

	if(iRecursionCount>10)
	{
		Asura_Assert(0,"Infinite recursion (probably) while calcuating max particles... not good.");
		return;
	}


	//must have a particle template assigned
	Asura_PFX_Particle_Template* pxPartTemplate = Asura_PFX_System::GetParticleTempByHash(pxSourceTemp->m_uParticleTemplateHash);
	if(pxPartTemplate)
	{
		//get the particle duration , taking the average if a variable duration is used
		float fParticleDuration = (pxPartTemplate->m_fMinLifetime + pxPartTemplate->GetMaxLifetime())/2.0f;

		bool bLoopingEffect = pxEffectTemp->m_bLooping;
		float fEffectDuration = pxEffectTemp->m_fLifeTime;

		//if spawning from another source's particles , then we need to track the history of those
		float* pfSpawnParticleCounts = 0;
		if(pxSourceTemp->m_iSpawnSourceIndex >=0 && pxEffectTemp)
		{
			
			if((int)pxEffectTemp->m_uNumSources<=pxSourceTemp->m_iSpawnSourceIndex)
			{
				return;//source doesn't exist , so none will spawn from it
			}
			 Asura_Hash_ID uSpawnHash = pxEffectTemp->m_pxSources[pxSourceTemp->m_iSpawnSourceIndex].m_uTemplate;
			 const Asura_PFX_Source_Template* pxSpawnSourceTemp = Asura_PFX_System::GetSourceTempByHash(uSpawnHash);

			if(!pxSpawnSourceTemp)
			{
				return;
			}

			pfSpawnParticleCounts = new float[iSimSteps];
			DetermineParticleCountsByTime(pxEffectTemp,pxSpawnSourceTemp,fSimInterval,iSimSteps,pfSpawnParticleCounts,iRecursionCount+1);
		}




		//Create a buffer containing the average number of particles generated at each time interval.
		//This buffer is sized so that we cycle back to the beginning as the first particles expire.
		int iMaxParticleSlots = Asura_Maths::FloatToInt_Chop(fParticleDuration/fSimInterval) + 1;
		float* pfParticles = new float[iMaxParticleSlots];
		for(int i=0;i<iMaxParticleSlots;i++)
		{
			pfParticles[i] = 0.0f;
		}

		float fCurrentParticles = 0.0f;
		int iSlot = 0;
		int iStep = 0;
		bool bSourceActive = true;
		float fEffectTimer = 0.0f;
		
		for(float fSourceTimer = 0;iStep<iSimSteps;fSourceTimer += fSimInterval,fEffectTimer += fSimInterval,iSlot++,iStep++)
		{
			if(iSlot>=iMaxParticleSlots) iSlot = 0;
			
			//remove 'expired' particles
			fCurrentParticles -= pfParticles[iSlot];

			//check for reaching the end of a loop of the effect
			if(fEffectTimer > fEffectDuration)
			{
				if(bLoopingEffect)
				{
					//start new effect/source loop
					fEffectTimer = 0.0f;
					fSourceTimer = 0.0f;
					bSourceActive = true;
				}
				else
				{
					//not looping , so source shuts down now
					bSourceActive = false;
				}
			}
			
			
			//calculate source age at this time
			float fAge = (pxSourceTemp->m_fDuration) ? fSourceTimer/pxSourceTemp->m_fDuration : 1.0f;
			if(fAge > 1.0f)
			{
				//source will stop until effect loops
				bSourceActive = false;
			}

			//get release rate and probablity for the time
			if(pfSpawnParticleCounts && !pfSpawnParticleCounts[iStep])
			{
				//spawn source has no particles at this stage , so can't generate any
				pfParticles[iSlot] = 0.0f;

			}
			else if(bSourceActive)
			{
				if(pxSourceTemp->m_bBurst)
				{
					pfParticles[iSlot] = (float)pxSourceTemp->m_uMaxParticles;
					bSourceActive = false;
				}
				else
				{
					float fReleaseRate = pxSourceTemp->m_xReleaseRate.GetValueAtTime_Discrete(fAge);
					float fReleaseProb = pxSourceTemp->m_xReleaseRand.GetValueAtTime_Discrete(fAge);

					//if a probability has been set , overestimate the number to be generated
					fReleaseProb *= 1.1f;
					if(fReleaseProb > 1.0f) fReleaseProb = 1.0f;
			
					//determine average particles generated in this time interval
					pfParticles[iSlot] = fReleaseRate*fSimInterval*fReleaseProb;
				}
			}
			else
			{
				pfParticles[iSlot] = 0.0f;
			}

			fCurrentParticles += pfParticles[iSlot];

			//update history of total particles
			pfParticleCounts[iStep] = fCurrentParticles;
		}
		delete [] pfParticles;
		delete [] pfSpawnParticleCounts;
	}

}

void Tools_Mode_PFX::UpdateSourceChooser()
{
	for(u_int u = 0; u < TOOLS_PFX_MAX_SOURCES_PER_EFFECT; ++u)
	{
		if(s_aszSourceList[u]) delete [] s_aszSourceList[u];
		s_aszSourceList[u] = new char[128];
		const Asura_String* pxName = NULL;
		int iColonPosition = 0;
		const Asura_Char* szName = NULL;
		if(u < s_pxCurrentEffectTemp->m_uNumSources)
		{
			pxName = Asura_PFX_System::GetSourceTempNameByHash(s_pxCurrentEffectTemp->m_pxSources[u].m_uTemplate);
			if(pxName && pxName->Contains(":", &iColonPosition))
			{
				szName = &pxName->GetString()[iColonPosition + 1];
			}
		}
		Asura_PrintfToBfr(s_aszSourceList[u], "%u - %s", 128, true, u + 1, szName ? szName : "No template");
	}
	if(s_pxEffectSourceNumDialog)
	{
		s_pxEffectSourceNumDialog->ReplaceStrings(s_pxCurrentEffectTemp->m_uNumSources, s_aszSourceList);
	}
}

// Reorder sources
void Tools_Mode_PFX::MoveSourceUp()
{
	Asura_Assert(CanMoveSourceUp(), "Already at top");
	Tools_Entity_PFX_Effect::ClearAndFlagForRestart_EffectTemplate(s_pxCurrentEffectTemp->GetTemplateHash());


	Asura_PFX_Effect_SourceProperties xTempSource = s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum];
	s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum] = s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum - 1];
	s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum - 1] = xTempSource;
	s_iEffectCurrentSourceNum--;
	s_pxEffectSourceNumDialog->SelectEntry(s_iEffectCurrentSourceNum);
	SetModified(s_pxCurrentEffectTemp);

	UpdateSourceChooser();
	UpdateSourceProperties();
}

void Tools_Mode_PFX::MoveSourceDown()
{
	Asura_Assert(CanMoveSourceDown(), "Already at bottom");
	Tools_Entity_PFX_Effect::ClearAndFlagForRestart_EffectTemplate(s_pxCurrentEffectTemp->GetTemplateHash());

	Asura_PFX_Effect_SourceProperties xTempSource = s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum];
	s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum] = s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum + 1];
	s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum + 1] = xTempSource;
	s_iEffectCurrentSourceNum++;
	s_pxEffectSourceNumDialog->SelectEntry(s_iEffectCurrentSourceNum);
	SetModified(s_pxCurrentEffectTemp);

	UpdateSourceChooser();
	UpdateSourceProperties();
}

bool Tools_Mode_PFX::CanMoveSourceUp()
{
	if(!s_pxCurrentEffectTemp) { return false; }
	return (s_pxCurrentEffectTemp->m_uNumSources > 1) && (s_iEffectCurrentSourceNum > 0);
}

bool Tools_Mode_PFX::CanMoveSourceDown()
{
	if(!s_pxCurrentEffectTemp) { return false; }
	return (s_pxCurrentEffectTemp->m_uNumSources > 1) && (s_iEffectCurrentSourceNum < (int)s_pxCurrentEffectTemp->m_uNumSources - 1);
}

void Tools_Mode_PFX::UpdateSourceProperties()
{
	UpdateSourceTempListSelection(s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum].m_uTemplate);

	// Set speed
	s_fEffectSrcSpeed = s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum].m_xVelocity.Magnitude();
	// Set time offset
	s_fEffectSrcTimeOffset = s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum].m_fTimeOffset;
	// Set orientation ID
	s_bEffectUseCodeOrient = (s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum].m_iOrientationID != 0);
	s_iEffectSrcOrientID = s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum].m_iOrientationID;
	//(tools) disabled flag
	s_bEffectSrcDisabled = s_pxCurrentEffectTemp->m_pxSources[s_iEffectCurrentSourceNum].m_bDisabled;

}

void Tools_Mode_PFX::SetEffectRange()
{
	if(!s_pxCurrentEffectTemp) return;

	for(u_int uSource = 0; uSource < s_pxCurrentEffectTemp->m_uNumSources; uSource++)
	{
		// Scale source properties
		Asura_PFX_Source_Template* pxSourceTemplate = Asura_PFX_System::GetSourceTempByHash(s_pxCurrentEffectTemp->m_pxSources[uSource].m_uTemplate);
		// Check that we haven't done this one already
		bool bDuplicateSource = false;
		for(u_int uPrevSource = 0; uPrevSource < uSource; ++uPrevSource)
		{
			if(s_pxCurrentEffectTemp->m_pxSources[uPrevSource].m_uTemplate == s_pxCurrentEffectTemp->m_pxSources[uSource].m_uTemplate)
			{
				bDuplicateSource = true;
			}
		}
		if(bDuplicateSource)
		{
			continue;
		}
		if(pxSourceTemplate)
		{
			pxSourceTemplate->m_bUseRange = s_bEffectUseRange;
			if(s_bEffectUseRange)
			{
				pxSourceTemplate->m_fRangeDelayTime = s_fEffectRangeDelayTime;
				pxSourceTemplate->m_fRangeMin = s_fEffectRangeMin;
				pxSourceTemplate->m_fRangeMax = s_fEffectRangeMax;
			}
			if(pxSourceTemplate == s_pxCurrentSourceTemp)
			{
				s_bSrcUseRange = s_bEffectUseRange;
			}
		}
	}
}

void Tools_Mode_PFX::Verify(bool bAutomatic)
{
	ASURA_CPU_PROFILE_SCOPED_TIMER("Tools_Mode_PFX::Verify");

	Tools_ProgressIndicator xProgress( "PFX Verification", true );

	//Clear out existing messages
	Tools_MessageLog::DeleteMessages(Tools_MessageLog::PFX,0);
	//Get current position to see if any errors were logged
	u_int uInitialPos = Tools_MessageLog::GetNextID();

	//check each effect
	const Asura_TemplateHashTree<Asura_String>&	xTemplateNameTree = Asura_PFX_System::GetEffectTempNames();

	if(xTemplateNameTree.GetNumberOfNodes())
	{
		u_int uCurrentNode=0;
		u_int uNumberOfNodes = xTemplateNameTree.GetNumberOfNodes();
		//using the 'save' iterator to avoid loading all the templates in ascending hash order.RAR 27/06/2008
		for(Asura_TemplateHashTree_SaveIterator xIt(xTemplateNameTree.GetNumberOfNodes());!xIt.Done();xIt.Next(), uCurrentNode++)
		{
			xProgress.Update( uCurrentNode, uNumberOfNodes );
			const Asura_String*	pxName = xTemplateNameTree.GetDataByIndex(xIt.GetCurrent());
			const Asura_Hash_ID	uEffectHash	= pxName->GetHash();;
					
			const Asura_PFX_Effect_Template* pxEffectTemp = Asura_PFX_System::GetEffectTempByHash(uEffectHash);

			if(pxEffectTemp != NULL)
			{	
				//check for missing sources
				for(u_int u=0;u<pxEffectTemp->m_uNumSources;u++)
				{
					if(!Asura_PFX_System::DoesSourceHashExist(pxEffectTemp->m_pxSources[u].m_uTemplate))
					{

						Tools_MessageLog::AddMessage(Tools_MessageLog::PFX,"Missing Template",new Tools_MessageLogAction_PFXEffect(uEffectHash),
							"Source %d of effect '%s' doesn't exist",
							u+1,pxName->GetString());
					}
				}

				//check for particles using looping sound events
				for(u_int u=0;u<pxEffectTemp->m_uNumSources;u++)
				{
					const Asura_PFX_Source_Template* pxSourceTemp = Asura_PFX_System::GetSourceTempByHash(pxEffectTemp->m_pxSources[u].m_uTemplate);
					if(!pxSourceTemp) continue;

					const Asura_PFX_Particle_Template* pxParticleTemp = Asura_PFX_System::GetParticleTempByHash(pxSourceTemp->m_uParticleTemplateHash);
					if(!pxParticleTemp) continue;

					if(pxParticleTemp->m_uSoundEvent)
					{
						const Asura_Sound_Event* pxEvent = Asura_Sound_Event_Storage::FindEvent(pxParticleTemp->m_uSoundEvent);
						if(pxEvent && pxEvent->IsEndlessPlayback())
						{
							const Asura_String* pxParticleName = Asura_PFX_System::GetParticleTempNameByHash(pxSourceTemp->m_uParticleTemplateHash);
							Asura_Assert(pxParticleName, "Failed to get particle template name");
							
							Tools_MessageLog::AddMessage(Tools_MessageLog::PFX,"Sound",new Tools_MessageLogAction_PFXEffect(uEffectHash),
								"Particle '%s' in effect '%s' uses a looping sound event",
								pxParticleName->GetString(), pxName->GetString());
							
						}
					}	// pxParticleTemp->m_uSoundEvent != 0

					// Check if texture exists or not.
					if( pxParticleTemp->m_xTextureName.IsBlank()==false )
					{
						bool bIsTextureLoaded = Asura_Texture_Management::IsTextureLoaded(pxParticleTemp->m_iTextureHandle); 
						if( bIsTextureLoaded==false )
						{
							const Asura_String* pxParticleName = Asura_PFX_System::GetParticleTempNameByHash(pxSourceTemp->m_uParticleTemplateHash);
							Tools_MessageLog::AddMessage(Tools_MessageLog::PFX,"Texture",new Tools_MessageLogAction_PFXEffect(uEffectHash),
								"Particle '%s'     is accessing the missing texture: '%s'",
								pxParticleName->GetString(), pxParticleTemp->m_xTextureName );
						}
					}	// pxParticleTemp->m_xTextureName.IsBlank()
				}	// for...

			}	// pxEffectTemp != NULL
		}
	}

	xProgress.Finish();

	if(Tools_MessageLog::AreThereAnyMessages(Tools_MessageLog::PFX, 0, uInitialPos))
	{
		//Found some errors ,so show message log
		Tools_MessageLog::ShowOverlay(Tools_MessageLog::PFX,0);
	}
	else if(!bAutomatic)
	{
		ShowMessageOverlay("No errors found.");
	}
}


void Tools_Mode_PFX::UpdateSourceTempList()
{
	DeleteSourceTempList();

	//get prefix used by all sources belonging to this effect
	Asura_Assert(s_pxCurrentEffectTemp,"");

	char szSourceNamePrefix[100];
	Asura_PrintfToBfr(szSourceNamePrefix,"%s:",sizeof(szSourceNamePrefix),true,s_pxCurrentEffectTemp->m_xName.GetString());
	int iPrefixLength = strlen(szSourceNamePrefix);

	const Asura_ResourceSetWithOverride<Asura_PFX_Source_Template>&			xResourceSet	= Asura_PFX_System::GetSourceTempResourceSet();
	Asura_ResourceSetWithOverride_ConstResourceIterator<Asura_PFX_Source_Template>	xIterator(xResourceSet);

	s_iSourceTempListSize	= xResourceSet.GetNumResources() + 1;
	s_pszSourceTempList		= new const char*[s_iSourceTempListSize];
	s_pszSourceTempList[0]	= "None";

	//get the names of the templates to use in the list
	int iDestIndex = 1;
	while(!xIterator.Done())
	{
		const Asura_Hash_ID	uSourceHash		= xIterator.GetCurrentHash();
		const Asura_String*	pxSourceName	= Asura_PFX_System::GetSourceTempNameByHash(uSourceHash);
		const char*			szSourceName	= (pxSourceName == NULL) ? NULL : pxSourceName->GetString();

		//When using the new PFX UI , we only want the source templates that belong with the effect being edited
		bool bOkay = false;

		if(!strnicmp(szSourceNamePrefix,szSourceName,iPrefixLength))
		{
			//starts with correct prefix
			bOkay = true;
		}
		else
		{
			//see if this template is being used by the effect
			for(u_int j=0;j < s_pxCurrentEffectTemp->m_uNumSources; j++)
			{
				if(s_pxCurrentEffectTemp->m_pxSources[j].m_uTemplate == uSourceHash)
				{
					bOkay = true;
					break;
				}
			}
		}

		if(bOkay)
		{
			s_pszSourceTempList[iDestIndex++] = szSourceName;
		}

		xIterator.Next();
	}

	s_iSourceTempListSize = iDestIndex;
}

void Tools_Mode_PFX::DeleteSourceTempList()
{
	delete [] s_pszSourceTempList;
	s_pszSourceTempList = 0;
	s_iSourceTempListSize = 0;

}

void Tools_Mode_PFX::UpdateSourceTempListSelection(Asura_Hash_ID uSourceHash)
{
	Asura_Assert(s_pxEffectSourceTempDialog,"");
	Asura_Assert(s_pszSourceTempList,"");

	//find the index in the source list
	int iIndex = 0;
	if(uSourceHash)
	{
		for(int i=0;i<s_iSourceTempListSize;i++)
		{
			if(uSourceHash == Asura_GetHashID(s_pszSourceTempList[i]))
			{
				iIndex = i;
				break;
			}
		}
	}

	//update the dialog
	s_pxEffectSourceTempDialog->SelectEntry(iIndex);
}

void Tools_Mode_PFX::SetModified(Asura_PFX_Effect_Template* pxEffectTemplate)
{
	Asura_Assert(pxEffectTemplate,"");
	if(!pxEffectTemplate->m_bModified)
	{
	  	Asura_String xFilename;
	  	GetTemplateFileName(&pxEffectTemplate->m_xName, &xFilename, TOOLS_MODE_PFX_SAVE_NORMAL);

		Tools_SaveAsura::WarnIfReadOnly(xFilename.GetString());
		
	}
	pxEffectTemplate->m_bModified = true;
	pxEffectTemplate->m_bDoneAutosave = false;
}

void Tools_Mode_PFX::SetModified(Asura_PFX_Source_Template* pxSourceTemplate)
{
	Asura_Assert(pxSourceTemplate,"");
	//Want to flag the effect template as modified.
	//There isn't a good way to get back to the effect from the source , so rely
	//on just the current template being set as modified
	//RAR 22/01/2008
	if(pxSourceTemplate == s_pxCurrentSourceTemp && s_pxCurrentEffectTemp)
	{
		SetModified(s_pxCurrentEffectTemp);

	}
	else
	{
		Asura_Assert(0,"Only expecting the current source to be flagged as modified");
		pxSourceTemplate->m_bModified = true;
	}
}

void Tools_Mode_PFX::SetModified(Asura_PFX_Particle_Template* pxParticleTemplate)
{
	Asura_Assert(pxParticleTemplate,"");
	//Want to flag the effect template as modified.
	//There isn't a good way to get back to the effect from the particle , so rely
	//on just the current template being set as modified
	//RAR 22/01/2008
	if(pxParticleTemplate == s_pxCurrentParticleTemp && s_pxCurrentEffectTemp)
	{
		//flag the effect template as modified instead
		SetModified(s_pxCurrentEffectTemp);

	}
	else
	{
		Asura_Assert(0,"Only expecting the current particle to be flagged as modified");
		pxParticleTemplate->m_bModified = true;
	}
}




////////////////////////////////////////////////////////////////////////////////////////////////////
Asura_Guid Tools_MessageLogAction_PFXEffect::s_uLastTimeUsedPFXGuid = ASURA_GUID_UNREGISTERED;

void Tools_MessageLogAction_PFXEffect::Execute()
{
	// Just safe side check. Probably it don't need to be tested here. But anyway. Point is, that we can modify PFX only when PFX overlay is on.
	if( Tools_Mode_PFX::PFXOverlayIsOn()==false )
	{
		Tools_Timed_Message_System::AddMessage(0xff0000,"You must turn on PFX OVERLAY to work with this.");
		return;
	}
	
	// Ok, now we need to find temporary object. (In which we can modify the PFX template.) If this object
	// does not exist, lets create it and ask user for name. However, if the user already created such object 
	// a few minutes ago, don't create new one, but instead use last one. (We remember last used one in s_uLastTimeUsedPFXGuid.)
	Tools_Object* pxToolsObject = Tools_Object_System::GetObjectFromGuid(s_uLastTimeUsedPFXGuid);
	bool bObjectIsCorrect = pxToolsObject ? pxToolsObject->IsBaseOrDerivativeOf( TOT_Entity_PFX_Effect ) : false;

	if( bObjectIsCorrect )
	{
		// We found one. Huraaaaah. So now, just make sure it is selected.

		Tools_Object* pxPreviousToolsObject = Tools_Object_System::GetMainSelectionForCurrentSelectionMode();
		if( pxToolsObject!=pxPreviousToolsObject )
		{
			if( pxPreviousToolsObject )
			{
				// Just tell the user that we will Deselect previously selected object. (To decrease user confusion)
				Tools_Timed_Message_System::AddMessage(0xff00ff,"Unselecting object: '%s'", pxPreviousToolsObject->GetName() );
			}

			Tools_Object_System::SelectObject(pxToolsObject);	
			Tools_Timed_Message_System::AddMessage(0xffffff,"Selecting temporary object with name: %s. In this object you can modify PFX template.", pxToolsObject->GetName() );
		}
	}
	else
	{
		// Well, we didn't find any PFX object. So lets create one.

		Tools_Timed_Message_System::AddMessage(0xffffff,"Create dummy object, where you can do all changes to PFX template. The dummy is excluded from export on all platforms." );

		// Create new PFX object and select it.
		Tools_Mode_PFX::NewPFXEffect();

		// Get that selection and remember it. So next time, we can use it as our starting point
		pxToolsObject = Tools_Object_System::GetMainSelectionForCurrentSelectionMode();
		if(pxToolsObject)
		{
			// Safe side check.
			if( pxToolsObject->IsBaseOrDerivativeOf(TOT_Entity_PFX_Effect)==false )
			{
				// Fatal error. Don't continue to do anything. Because we created PFX effect object, but we get some other object.
				Asura_Assert( false, "Tools_MessageLogAction_PFXEffect::Execute: Bad error when creating new PFX effect. We tried to create PFX effect, but it failed for some reason. And now we can't select this 'maybe' created PFX effect. I am suggesting to save work on PFXs, and restart the tool." );
				return;
			}
	
			// Make sure that dummy is clearly differentiated from other pfx objects. So we add suffix "_DUMMY_DELETEME" and
			// we will turn off all flags for exporting, so dummy will be excluded from export on all platforms.
			Tools_Entity_PFX_Effect* pxPFXToolsEntityObject = static_cast <Tools_Entity_PFX_Effect*>(pxToolsObject);
			Asura_String xName = pxToolsObject->GetName(); 
			xName += "_DUMMY_DELETEME";
			pxPFXToolsEntityObject->Rename( xName.GetString() );
			pxPFXToolsEntityObject->DisableExportForAllPlatforms();

			s_uLastTimeUsedPFXGuid = pxToolsObject->GetGuid();	// Make sure that next time, we are not going to create new object, but instead we will select this one.
		}
	}

	// Ok set new template to PFX effect.
	// Note:The value of Tools_Entity_PFX_Effect::s_pxEffectTempDialog is changing in object selection code. So in this point, it should be valid. However, I am sure that it wasn't valid at the beginning of this function.
	if( Tools_Entity_PFX_Effect::s_pxEffectTempDialog )
	{
		Tools_Entity_PFX_Effect::s_pxEffectTempDialog->SetTemplateHash( m_uHashID );
	}
	else
	{
		// Error correction. Make 100% sure that if things don't go ok, we can start all over again.
		s_uLastTimeUsedPFXGuid = ASURA_GUID_UNREGISTERED;
	}
}


#ifdef ASURA_USING_ALL_LEVEL_EXPORT
bool Tools_Mode_PFX::AddPFXToIgnoreList(const Asura_Hash_ID uPFXToIgnore)
{
	int iJunk;
	return m_pxPFXToIgnoreList->FindNodeIndexByHash(uPFXToIgnore, true, iJunk);
}
#endif

/// End of file. ///
